/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_app.c
*
* @brief This file provides BLDC control parameters initialization, state machine and millisecond time base functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "string.h"
#include "gpio_drv.h"
#include "bldc_app.h"
#include "bldc_startup.h"
#include "bldc_pid.h"
#include "bldc_pwm.h"
#include "bldc_keys.h"
#include "bldc_ui_uart.h"
#include "bldc_control_api.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
status_t g_flashOperationStatus = STATUS_UNSUPPORTED;                   /*!< Return value of eflash operation status */
uint16_t g_flashTaskFinish = 0;                                         /*!< Finish tag of flash task, used to uart communication */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief BLDC speed control task, 1ms execution cycle in RUN State Machine.
*
* @param[in] none
* @return none
*/
void BLDC_SpeedTask(void)
{
#if (defined BLDC_HALL)
    BLDC_SpeedLoop_Calculate(&g_bldc_speedCmd, &g_bldc_asrPid, &g_bldcVarsCtrl, pBldcVarsCfg);
#elif (defined BLDC_SENSORLESS)
    if (g_bldcVarsCtrl.pBemf->aveChangePhaseTime > 0)
    {
        BLDC_SpeedLoop_Calculate(&g_bldc_speedCmd, &g_bldc_asrPid, &g_bldcVarsCtrl, pBldcVarsCfg);
    }
#else
#endif
}

/*!
* @brief Motor bldc control common variable parameters initialization.
*
* @param[in] none
* @return none
*/
void BLDC_Common_ParamInit(void)
{
    pBldcMotorParam = &defBldcMotorParam;
    p_bldc_PidCfg = &defBldcPid;
    pBldcVarsCfg = &defBldcVars;

    pBldcBemfCfg = &defBemfVars;
    /* PID structure */
    g_bldc_acrPid.pPidInput = &g_bldc_acrPIIn;
    g_bldc_acrPid.pPidCoef = &g_bldc_acrCof;
    g_bldc_acrPid.pPidOutput = &g_bldc_acrPIOut;

    g_bldc_asrPid.pPidInput = &g_bldc_asrPIIn;
    g_bldc_asrPid.pPidCoef = &g_bldc_asrCof;
    g_bldc_asrPid.pPidOutput = &g_bldc_asrPIOut;

    /* BLDC common initialize */
    Set_BLDC_InitDuty(&g_bldcVarsCtrl, pBldcVarsCfg);
    g_bldcVarsCtrl.pBemf = &g_bemfControl;
    g_bldcVarsCtrl.pHall = &g_hallControl;
    g_bldcVars.pCtrl = &g_bldcVarsCtrl;
    g_bldcVars.pCfg = pBldcVarsCfg;

    pBldcHWParam = &defBldcHwBoard;
    pEflashCfg = &defEflashAccessCfg;
}

/*!
* @brief Bldc control variables initialization, clear all the variables of BLDC_VARS_CTRL to zero.
*
* @param[in] none
* @return none
*/
void BLDC_CtrlVarsInit(void)
{
    memset(&g_bldcVarsCtrl, 0U , sizeof(g_bldcVarsCtrl));
    g_bldcVarsCtrl.pBemf = &g_bemfControl;
    g_bldcVarsCtrl.pHall = &g_hallControl;
#if (defined BLDC_HALL)
    memset(&g_hallControl, 0U , sizeof(g_hallControl));
#else
    memset(&g_bemfControl, 0U , sizeof(g_bemfControl));
#endif
    g_flashTaskFinish = 0;
}

/*!
* @brief Bldc control parameters initialization, executed in the Stop state and the first Power on.
*
* @param[in] none
* @return none
*/
void BLDC_Parameters_Initialize(void)
{
    BLDC_CtrlVarsInit();
    Clear_CurrentSample(&g_bldc_adSample);
    BLDC_ASR_CommandInit(pBldcVarsCfg, &g_bldc_speedCmd);

    BLDC_Acr_PIInit(&g_bldc_acrPid, p_bldc_PidCfg);
    BLDC_Asr_PIInit(&g_bldc_asrPid, p_bldc_PidCfg);
}

/*!
* @brief Bldc control initialization, include control parameters and peripheral initialization.
*
* @param[in] none
* @return none
*/
void MC_Initialize(void)
{
    BLDC_Parameters_Initialize();
    BLDC_Peripheral_Initialize();
}

/*!
* @brief LED1 light turn on or off.
*
* @param[in] ledStatus: turn on or off LED, 1: off; 0: on
* @return none
*/
void LED1Display(uint8_t ledStatus)
{
    GPIO_DRV_WritePin(GPIOD, 15U, ledStatus);
}

/*!
* @brief LED2 light turn on or off.
*
* @param[in] ledStatus: turn on or off LED, 1: off; 0: on
* @return none
*/
void LED2Display(uint8_t ledStatus)
{
    GPIO_DRV_WritePin(GPIOE, 3U, ledStatus);
}

/*!
* @brief The Main state machine, used to set the running state of the bldc control system.
*
* @param[in] none
* @return none
*/
void BldcStateMachine(void)
{
    PullOut_McBufStatus();
    switch (Get_McStatus())
    {
    case POWER_ON:
    BLDC_Parameters_Initialize();
//    BLDC_ASR_CommandInit(pBldcVarsCfg, &g_bldc_speedCmd);
    BLDC_ASR_CommandSet(pBldcVarsCfg, &g_bldc_speedCmd);
    #if (defined DATA_ACCESS_FLASH_ENABLE)
    DisableInterrupts;
    g_flashOperationStatus = Eflash_DataAccess_DefaultSave(&g_dataAccessFlash, g_VarAccessArray);
    if (g_flashOperationStatus == STATUS_SUCCESS)
    {
        Eflash_DataAccess_PowerOnRecovery(&g_dataAccessFlash, g_VarAccessArray);
        Eflash_DataAccessEflashInit(&g_dataAccessFlash);
        Set_McStatus(IDLE);
        EnableInterrupts;
    }
    #else
    Set_McStatus(IDLE);
    #endif
    break;

    case IDLE:
    BLDC_Parameters_Initialize();
    #if (defined DATA_ACCESS_FLASH_ENABLE)
    if (g_dataAccessFlash.operateEnable == 1)
    {
        Set_McStatus(FLASH_TASK);
    }
    #endif

    if ((g_bldc_speedCmd.speedTarget * pBldcVarsCfg->motorDir) < 0)
    {
        g_bldc_speedCmd.speedTarget = 0 - g_bldc_speedCmd.speedTarget;
        g_bldc_speedCmd.speedTargetRamp = 0 - g_bldc_speedCmd.speedTargetRamp;
    }
    break;

    case START:
    BLDC_Startup(&g_bldcVarsCtrl, pBldcVarsCfg);
    if (BLDC_Get_Startup_Status(&g_bldcVarsCtrl) == STARTUP_READY)
    {
        Set_McStatus(RUN);
    }
    break;

    case RUN:
    BLDC_SpeedTask();
    break;

    case STOP:
    BLDC_Stop();
    BLDC_Parameters_Initialize();
//    BLDC_ASR_CommandInit(pBldcVarsCfg, &g_bldc_speedCmd);
    BLDC_ASR_CommandSet(pBldcVarsCfg, &g_bldc_speedCmd);
    Set_McStatus(IDLE);
    break;

    case FAULT:
    BLDC_Stop();
    if (g_bldc_protector.faultFlag.all == 0)
    {
        Set_McStatus(STOP);
    }
    break;

    #if (defined DATA_ACCESS_FLASH_ENABLE)
    case FLASH_TASK:
    DisableInterrupts;
    g_flashOperationStatus = Eflash_DataAccessEflashProcess(&g_dataAccessFlash, g_VarAccessArray);
    if (g_flashOperationStatus == STATUS_SUCCESS)
    {
        Set_McStatus(IDLE);

        g_flashOperationStatus = STATUS_UNSUPPORTED;
        Eflash_DataAccessEflashInit(&g_dataAccessFlash);
        g_flashTaskFinish = 1;
        EnableInterrupts;
    }
    break;
    #endif

    default:
    break;
    }
}

/*!
* @brief Bldc task in 1ms time base, do speed loop, speed ramp, etc. BLDC 1ms task , callback in task_scheduler.
*
* @param[in] none
* @return none
*/
void Motor_1ms_Task(void)
{
    BldcStateMachine();
    BLDC_Motor_Protection();
    BLDC_ASR_FbkUpdate(&g_bldcVarsCtrl, pBldcVarsCfg, &g_bldc_speedCmd);
    if (Get_McStatus() == RUN)
    {
        BLDC_ASR_RampCalc(pBldcVarsCfg, &g_bldc_speedCmd);
    }
    ADC_DRV_SoftwareStartRegularConvert(0U);
    ADC_DRV_SoftwareStartRegularConvert(1U);
}

/*!
* @brief Bldc task in 2 ms time base. If TOOL_UART_FUNCTION macro is defined, the UART function with host PC tool is executed in 2ms time base task.
*
* @param[in] none
* @return none
*/
void Motor_2ms_Task(void)
{
    #if (defined TOOL_UART_FUNCTION)
    UI_UART_ToolCmd_Proc(&g_uiRxBuf);
    UI_UART_TimeOut_Cnt();
    UI_MasterCmd_ProcCnt();
    #endif
}

/* =============================================  EOF  ============================================== */
