/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_isr.c
*
* @brief This file provides BLDC control interruptcallback functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "adc_hw.h"
#include "pwm_hw.h"
#include "bldc_keys.h"
#include "bldc_isr.h"
#include "bldc_pwm.h"
#include "bldc_adc.h"
#include "bldc_bemf.h"
#include "task_scheduler.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Callback routine of TIMER0, 1ms. The Task scheduler update in Timer0 interrupt,use for bldc control time base.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void BLDC_Base_TimerCallback(void *device, uint32_t wpara, uint32_t lpara)
{
    /* same as FOC_TIMER callback, to update the time base tasks in TIMER_ISR */
    if (wpara & TIMER_CTRL_SR_TIF0_Msk)
    {
        SCH_Update();
    }
}

/*!
* @brief Callback routine of TIMER1, use for back EMF zero crossing detection in bldc sensorless control.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void Timer1_Callback(void *device, uint32_t wpara, uint32_t lpara)
{
    if (wpara & TIMER_CTRL_SR_TIF1_Msk)
    {
        if (Get_McStatus() == RUN)
        {
            /* 60000 * 6 * TIMER_XUS = 8.64s, constrained the minmum operating elec frequency 0.12Hz */
            if (g_bemfControl.changePhaseTime < 60000)
            {
                g_bemfControl.changePhaseTime++;
            }
            BemfCheckFun(&g_bldcVarsCtrl, pBldcVarsCfg, &g_bldc_protector);
        }
    }
}

/*!
* @brief Callback routine of ADC0, use for ADC sample data and current loop in bldc control.
*
* @param[in] info: device info
* @param[in] parameter: callback parameter
* @return none
*/
void BLDC_ADC_Callback(adc_interrupt_info_t *info, void *parameter)
{
    if (info->event & ADC_EVENT_EOC)
    {
    #if (defined DAC_FUNCTIONALITY)
        PWM_SetChannelCountValue(PWM2, PWM_CH_0, ((g_bldc_speedCmd.speedTargetRamp + 32768) >> 4));
        PWM_SetChannelCountValue(PWM2, PWM_CH_1, ((g_bldc_speedCmd.speedFbk + 32768) >> 4));
    #endif
        Get_VoltageValue(&g_bldc_adSample);
        if ((Get_McStatus() == FAULT) || (Get_McStatus() == STOP) || (Get_McStatus() == IDLE) || (Get_McStatus() == POWER_ON))
        {
            BLDC_Stop();
            return;
        }
        else if (Get_McStatus() == RUN)
        {
            g_bldc_adSample.busCurAD = ADC_GetInjectData(ADC0, ADC_ISEQ_0);
            g_bldcVarsCtrl.pwmOutDuty = Get_EffectiveDuty(&g_bldcVarsCtrl, PWM3);
            BusCurrent_Control(&g_bldcVarsCtrl, &g_bldc_adSample);
        }
        else
        {

        }
    }
}

/*!
* @brief PWMx Fault CallBack Function. When hardware over-current fault occurs, PWM fault interrupt is triggered and this function is called.
*
* @param[in] instance: device instance
* @param[in] status: fault status
* @param[in] userData: user data parameter
* @return none
*/
void BLDC_PWM_FaultCallback(uint8_t instance, uint32_t status, void *userData)
{
    if (status & PWM_FDSR_FAULTDF_Msk)
    {
        PWM_DRV_ClearFaultChannelFlag(instance, PWM_FAULT_CH_0);
        Set_McStatus(FAULT);
        g_bldc_protector.faultFlag.bits.hwFault = 1;
    }
}

/*!
* @brief Callback routine of PWDT, occur while hall signal update, use for back EMF zero crossing detection hall signal captrue in bldc sensorless control.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void BLDC_Bemf_PWDTCallback(void *device, uint32_t wpara, uint32_t lpara)
{
//    if (wpara & PWDT_INIT0_OVF_Msk)
    {

    }

//    if (wpara & PWDT_INIT0_RDYF_Msk)
    {

    }
}

/*!
* @brief Call back function of PWM_Hall modular used by hall sensor.
*        When PWM_Hall modular input signal level change will trriger Hall interrupt.
*
* @param[in] instance: device instance
* @param[in] status: callback status
* @param[in] userData: callback userData
* @return none
*/
void BLDC_Hall_ChannelCallback(uint8_t instance, uint32_t status, void *userData)
{
    uint32_t posWidth = 0U;
    uint32_t negWidth = 0U;

    if ((status & PWM_STR_CH0SF_Msk) || (status & PWM_STR_CH1SF_Msk))
    {
//        if (g_hallControl.pwdtValid < 2U)
//        {
//            g_hallControl.pwdtValid++;
//            return;
//        }
        g_hallControl.currentPhase = PWM_DRV_GetHallStatus(0U);
        posWidth = PWM_DRV_GetChannelCountValue(0U, PWM_CH_0);
        negWidth = PWM_DRV_GetChannelCountValue(0U, PWM_CH_1);
        StorageDataCyclic(&g_hallControl.totalPulseWidth, (posWidth + negWidth));
        if (Get_McStatus() == RUN)
        {
            BLDC_Commutate_BaseSector(pBldcVarsCfg->motorDir, g_hallControl.currentPhase);
        }
    }
}

/*!
* @brief Call back function of PWM_Hall modular used by hall sensor.
*        When PWM_Hall modular input signal captrue count overflow will trriger Hall interrupt.
*
* @param[in] instance: device instance
* @param[in] status: callback status
* @param[in] userData: callback userData
* @return none
*/
void BLDC_Hall_OFCallback(uint8_t instance, uint32_t status, void *userData)
{
    if (status & PWM_INIT_CNTOF_Msk)
    {
//        memset(&g_hallControl.totalPulseWidth, 0U , sizeof(g_hallControl.totalPulseWidth));
        if (0U == g_bldc_protector.stallFlag)
        {
            g_hallControl.pwdtValid = 0U;
        }
        else
        {
            g_hallControl.currentPhase = PWM_DRV_GetHallStatus(0U);
            if (Get_McStatus() == RUN)
            {
                BLDC_Commutate_BaseSector(pBldcVarsCfg->motorDir, g_hallControl.currentPhase);
            }
        }
        return;
    }
}

/* =============================================  EOF  ============================================== */
