/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_pid.c
*
* @brief This file provides BLDC control PID calculate functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "bldc_pid.h"
#include "bldc_parameters_define.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Set input reference value of PID regulator.
*
* @param[in] PID: pointer to PID_TYPE structure
* @param[in] ref: Given reference value of PID reguletor input
* @return none
*/
void Set_BLDC_PidRef(PID_TYPE *PID, int16_t ref)
{
    PID->pPidInput->refPu = ref;
}

/*!
* @brief Set input feedback value of PID regulator.
*
* @param[in] PID: pointer to PID_TYPE structure
* @param[in] fbk: Given feedback value of PID reguletor input
* @return none
*/
void Set_BLDC_PidFbk(PID_TYPE *PID, int16_t fbk)
{
    PID->pPidInput->fbkPu = fbk;
}

/*!
* @brief Pid calculate of motor control.
*
* @param[in] pid: pointer to PID_TYPE structure
* @return none
*/
int16_t PID_Calculate(PID_TYPE *pid)
{
    int32_t min;
    int32_t max;

    pid->pPidOutput->errPu = pid->pPidInput->refPu - pid->pPidInput->fbkPu;
    /* proportional term */
    pid->pPidOutput->upPu = MATH_Mpy(pid->pPidOutput->errPu, pid->pPidCoef->kpPu);
    /* integral term */
    pid->pPidOutput->uiPu += MATH_Mpy(pid->pPidOutput->errPu, pid->pPidCoef->kiPu);

    /* saturate integral output with a dynamic limit */
    max = pid->pPidInput->maxPu - pid->pPidOutput->upPu;
    max = Q15_Sat(max, max, 0);

    min = pid->pPidInput->minPu - pid->pPidOutput->upPu;
    min = Q15_Sat(min, 0, min);

    pid->pPidOutput->uiPu = Q15_Sat(pid->pPidOutput->uiPu, max, min);

    /* calculate total output */
    pid->pPidOutput->outPu = pid->pPidOutput->upPu + pid->pPidOutput->uiPu;

    /* saturate total output */
    pid->pPidOutput->outPu = Q15_Sat(pid->pPidOutput->outPu, pid->pPidInput->maxPu, pid->pPidInput->minPu);

    return (int16_t)pid->pPidOutput->outPu;
}

/*!
* @brief Speed loop processing of motor control.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @param[in] pid: pointer to PID_TYPE structure
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @return none
*/
void BLDC_SpeedLoop_Calculate(SPEED_RAMP_TYPE *speed, PID_TYPE *pid, BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg)
{
    Set_BLDC_PidRef(pid, (bldcCfg->motorDir * speed->speedTargetRamp));
    Set_BLDC_PidFbk(pid, (bldcCfg->motorDir * speed->speedFbk));
#if (defined CURRENT_LOOP_CTRL)
    bldcCtrl->ibusRefPu = PID_Calculate(pid);
#else
    bldcCtrl->pwmOutPu = PID_Calculate(pid);
#endif
}

/*!
* @brief Current loop processing of motor control.
*
* @param[in] pid: pointer to PID_TYPE structure
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @return none
*/
void BLDC_CurrentLoop_Calculate(PID_TYPE *pid, BLDC_VARS_CTRL *bldcCtrl)
{
    Set_BLDC_PidRef(pid, bldcCtrl->ibusRefPu);
    Set_BLDC_PidFbk(pid, bldcCtrl->ibusFdkPu);
    bldcCtrl->pwmOutPu = PID_Calculate(pid);
}

/* =============================================  EOF  ============================================== */
