/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_speed.c
*
* @brief This file provides speed command ramp and speed loop processing functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "bldc_bemf.h"
#include "bldc_common.h"
#include "bldc_parameters_define.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief The per-unit value of speed targe value is calculated according to the actual value of the speed command unit rpm.
*
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @param[in] speedCmdRpm: Given speed command unit rpm
* @return none
*/
void BLDC_ASR_SetTargetRpm(SPEED_RAMP_TYPE *command, int16_t speedCmdRpm)
{
    command->speedTarget = (int32_t)(speedCmdRpm) * 32768 / MOTOR_MAX_SPEED_RPM;
}

/*!
* @brief The per-unit value of speed targe value is calculated according to the speed command unit pu.
*
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @param[in] speedCmdPu: Given speed command unit pu
* @return none
*/
void BLDC_ASR_SetTargetPu(SPEED_RAMP_TYPE *command, int16_t speedCmdPu)
{
    command->speedTarget = speedCmdPu;
}

/*!
* @brief The per-unit value of speed ramp value is calculated according to the speed command unit pu.
*
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @param[in] speedCmdPu: Given speed command unit pu
* @return none
*/
void BLDC_ASR_SetRampPu(SPEED_RAMP_TYPE *command, int16_t speedCmdPu)
{
    command->speedTargetRamp = speedCmdPu;
}

/*!
* @brief Get the speed target value.
*
* @param[in] speedRamp: pointer to SPEED_RAMP_TYPE structure
* @return speedTarget in Q15 unit
*/
int16_t BLDC_ASR_GetTargetPu(SPEED_RAMP_TYPE *speedRamp)
{
    return speedRamp->speedTarget;
}

/*!
* @brief Get the feedback speed unit in Rpm.
*
* @param[in] speed: pointer to SPEED_RAMP_TYPE structure
* @return speedFbk in Rpm unit
*/
int16_t BLDC_ASR_GetFbkRpm(SPEED_RAMP_TYPE *speed)
{
    return speed->speedFbkRpm;
}

/*!
* @brief Get the feedback speed unit in Pu.
*
* @param[in] speed: pointer to SPEED_RAMP_TYPE structure
* @return speedFbk in Q15 unit
*/
int16_t BLDC_ASR_GetFbkPu(SPEED_RAMP_TYPE *speed)
{
    return speed->speedFbk;
}

/*!
* @brief Set the per-unit value of feedback speed.
*
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @param[in] speedFbk: speed feedback unit pu
* @return none
*/
void BLDC_ASR_SetFbkPu(SPEED_RAMP_TYPE *command, int16_t speedFbk)
{
    command->speedFbk = speedFbk;
}

/*!
* @brief Speed ramp calculation initialization.
*
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_TYPE structure
* @return none
*/
void BLDC_ASR_CommandInit(BLDC_VARS_CFG *bldcCfg, SPEED_RAMP_TYPE *speed)
{
//    BLDC_ASR_SetTargetPu(speed, bldcCfg->motorDir * DEFAULT_TARGET_FREQ_START);
//    BLDC_ASR_SetRampPu(speed, bldcCfg->motorDir * bldcCfg->speedMinLimit);

    speed->speedAcceleration = (bldcCfg->motorAcceleration / 10);
    speed->speedDeceleration = (bldcCfg->motorDeceleration / 10);
    if (speed->speedAcceleration < 1)
    {
        speed->speedAcceleration = 1;
    }
    if (speed->speedDeceleration < 1)
    {
        speed->speedDeceleration = 1;
    }

    speed->speedMax = bldcCfg->speedMaxLimit;
    speed->speedMin = bldcCfg->speedMinLimit;

    speed->speedFactor = MOTOR_MAX_SPEED_RPM;
}

/*!
* @brief Speed ramp configuration.
*
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_TYPE structure
* @return none
*/
void BLDC_ASR_CommandSet(BLDC_VARS_CFG *bldcCfg, SPEED_RAMP_TYPE *speed)
{
    BLDC_ASR_SetTargetPu(speed, bldcCfg->motorDir * DEFAULT_TARGET_FREQ_START);
    BLDC_ASR_SetRampPu(speed, bldcCfg->motorDir * bldcCfg->speedMinLimit);
}

/*!
* @brief Limit the max and min value of speed target.
*
* @param[in] motorDir: motor rotation direction
* @param[in] speedRamp: pointer to SPEED_RAMP_TYPE structure
* @return none
*/
void BLDC_ASR_TargetLimit(int16_t motorDir, SPEED_RAMP_TYPE *speedRamp)
{
    if (motorDir == FORWARD_ROTATE)
    {
        if (speedRamp->speedTarget < speedRamp->speedMin)
        {
            speedRamp->speedTarget = speedRamp->speedMin;
        }
        else if (speedRamp->speedTarget > speedRamp->speedMax)
        {
            speedRamp->speedTarget = speedRamp->speedMax;
        }
        else
        {
        }
    }
    else if (motorDir == REVERSE_ROTATE)
    {
        if (speedRamp->speedTarget > -(speedRamp->speedMin))
        {
            speedRamp->speedTarget = -(speedRamp->speedMin);
        }
        else if (speedRamp->speedTarget < -(speedRamp->speedMax))
        {
            speedRamp->speedTarget = -(speedRamp->speedMax);
        }
        else
        {
        }
    }
    else
    {
    }
}

/*!
* @brief Speed ramp calculation. calculate the ramp output according to the speed target value.
*
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] speedRamp: pointer to SPEED_RAMP_TYPE structure
* @return none
*/
void BLDC_ASR_RampCalc(BLDC_VARS_CFG *bldcCfg, SPEED_RAMP_TYPE *speedRamp)
{
    BLDC_ASR_TargetLimit(bldcCfg->motorDir, speedRamp);

    speedRamp->rampCount++;
    /* calculate speed ramp every 10ms, 9 = 10 - 1, s_countSpdCmd range is 1~10, then back to 0, and becomes to 1 after '++' */
    if (speedRamp->rampCount > 9)
    {
        speedRamp->rampCount = 0;
        if (bldcCfg->motorDir == FORWARD_ROTATE)
        {
            if ((speedRamp->speedTarget - speedRamp->speedTargetRamp) > speedRamp->speedAcceleration)
            {
                speedRamp->speedTargetRamp += speedRamp->speedAcceleration;
            }
            else if ((speedRamp->speedTargetRamp - speedRamp->speedTarget) > speedRamp->speedDeceleration)
            {
                speedRamp->speedTargetRamp -= speedRamp->speedDeceleration;
            }
            else
            {
                speedRamp->speedTargetRamp = speedRamp->speedTarget;
            }
        }
        else if (bldcCfg->motorDir == REVERSE_ROTATE)
        {
            if ((speedRamp->speedTargetRamp - speedRamp->speedTarget) > speedRamp->speedAcceleration)
            {
                speedRamp->speedTargetRamp -= speedRamp->speedAcceleration;
            }
            else if ((speedRamp->speedTarget - speedRamp->speedTargetRamp) > speedRamp->speedDeceleration)
            {
                speedRamp->speedTargetRamp += speedRamp->speedDeceleration;
            }
            else
            {
                speedRamp->speedTargetRamp = speedRamp->speedTarget;
            }
        }
        else
        {
        }
    }
}

/*!
* @brief Get the speed ramp value. Assign to speed loop reference value.
*
* @param[in] speedRamp: pointer to SPEED_RAMP_TYPE structure
* @return speedTargetRamp: speed targe ramp value
*/
int16_t BLDC_ASR_GetRampPu(SPEED_RAMP_TYPE *speedRamp)
{
    return speedRamp->speedTargetRamp;
}

/*!
* @brief get motor elec freq in bldc control with hall sensor.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @return elec freq in Q15
*/
int16_t BLDC_Hall_SpeedGet(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg)
{
    int16_t s_elecFreq = 0;
    int16_t s_elecFreqPu = 0;

    bldcCtrl->pHall->averagePulseWidth = Filter_AverageCalc(&bldcCtrl->pHall->totalPulseWidth, BUFFER_SIZE);
    if (bldcCtrl->pHall->averagePulseWidth)
    {
        s_elecFreq = ((MOTOR_BUS_CLK / (PWDT_CLK_PSC + 1))/(3 * bldcCtrl->pHall->averagePulseWidth));     /* elec cycle = 3 PWDT cycles */
    }
    s_elecFreqPu = bldcCfg->motorDir * MATH_IQ(s_elecFreq * 1.0 / BASE_FREQ);

    return s_elecFreqPu;
}

/*!
* @brief get motor elec freq in bldc control with hall sensor.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @return elec freq in Q15
*/
int16_t BLDC_Bemf_SpeedGet(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg)
{
    uint32_t s_elecFreq = 0;
    uint32_t s_elecFreqPu = 0;

    if (bldcCtrl->pBemf->aveChangePhaseTime)
    {
        s_elecFreq = 166667 / (int32_t)(bldcCtrl->pBemf->aveChangePhaseTime * TIMER_XUS);   /* 166667 refers 6 sector time in us, 166667 = (6 / 10^-6) */
    }

    s_elecFreqPu = bldcCfg->motorDir * MATH_IQ(s_elecFreq * 1.0 / BASE_FREQ);

    return s_elecFreqPu;
}

/*!
* @brief Update the feedback speed of motor in different modes(Hall/sensorless).
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @return none
*/
void BLDC_ASR_FbkUpdate(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg, SPEED_RAMP_TYPE *command)
{
#if (defined BLDC_HALL)
    command->speedFbk = BLDC_Hall_SpeedGet(bldcCtrl, bldcCfg);
#elif (defined BLDC_SENSORLESS)
    command->speedFbk = BLDC_Bemf_SpeedGet(bldcCtrl, bldcCfg);
#else

#endif
    command->speedFbkRpm = MATH_Mpy(command->speedFactor, command->speedFbk);
}

/* =============================================  EOF  ============================================== */
