/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __ENCODER_H__
#define __ENCODER_H__
/*!
* @file encoder.h
*
* @brief  This file provides calculate and update encoder elec angle functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "hall_sensor.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Encoder sensor mode control and data structure.
*/
typedef struct
{
    uint8_t  encoderDir;               /*!< encoder turn direction */
    uint8_t  encoderPolarity;          /*!< encoder position direction polarity */
    uint16_t encoderCnt;               /*!< encoder paulse Cnt */
    uint16_t encoderElecAngle;         /*!< encoder elec angle in foc control */
    uint16_t corssZFlag;               /*!< whether of the rotor cross Z signal point */
    int16_t  encoderCntOffSet;         /*!< encoder paulse Cntoffset Initial */
    int16_t  alignOffSet;              /*!< encoder paulse Cntoffset after align */
    int16_t  zCntOffSet;               /*!< encoder paulse Cntoffset while Z signal */
    int16_t  zCntOffSetBase;           /*!< encoder paulse Cntoffset while first Z signal */
    int16_t  encoderAngleCnt;          /*!< encoder paulse total Cnt to calculate in mechanical angle */
    int16_t  encoderFocCnt;            /*!< encoder paulse Cnt Used in Foc */
    int16_t  encoderCircleCnt;         /*!< encoder circle count */
} ENCODER_TYPE;

/*!
* @brief Encoder speed measure data structure.
*/
typedef struct
{
    uint8_t  speedUpdateFlag;          /*!< encoder M_T measure speed update status flag */
    int16_t  measureFreq;              /*!< encoder M_T measure elec frequency */
    int16_t  mtSpeed;                  /*!< encoder M_T measure speed in Q15 */
    int16_t  mtSpeedFilter;            /*!< encoder M_T measure speed filter value */
    int16_t  mtSpeedCoeff;             /*!< encoder M_T measure speed calculate coeff */
    int16_t  measurePulse1;            /*!< encoder M_T measure last paulse Cnt */
    int16_t  measurePulse2;            /*!< encoder M_T measure current paulse Cnt */
    int16_t  deltaPulse;               /*!< encoder M_T measure paulse Cnt change value in update cycle */
    int16_t  deltaPulseLast;           /*!< encoder M_T measure last paulse Cnt change value in update cycle */
    uint16_t measureTime1;             /*!< encoder M_T measure last time Cnt */
    uint16_t measureTime2;             /*!< encoder M_T measure current time Cnt */
    uint16_t deltaTime;                /*!< encoder M_T measure time Cnt change value in update cycle */
    uint16_t deltaTimeLast;            /*!< encoder M_T measure last time Cnt change value in update cycle */
    uint32_t pulseTime;                /*!< encoder M_T measure time Cnt change value in update cycle */
} ENCODER_SPEED_MEASURE;

/*!
* @brief Encoder position data structure.
*/
typedef struct
{
    int16_t positionTar;               /*!< encoder position target in */
    int16_t positionTarMax;            /*!< encoder position max target in */
    int16_t positionTarMin;            /*!< encoder position min target in */
    int16_t positionFdk;               /*!< encoder position feedback in */
    int16_t positionFdkMax;            /*!< encoder position max feedback in */
    int16_t positionFdkMin;            /*!< encoder position min feedback in */
} PID_POSITION_IN;

/*!
* @brief Hall AB sensor mode control and data structure.
*/
typedef struct
{
    uint8_t  run0HzFlag;               /*!< 0Hz run flag */
    uint16_t estTheta;                 /*!< encoder calculate rotor position */
    uint16_t encoderCntVal;            /*!< pwm1 cnt value */
    uint16_t encoderCntValBuf;         /*!< pwm1 cnt value buf */
    uint16_t firstHallStatus;          /*!< power up first hall status */
    uint16_t adIsrCnt;                 /*!< ad isr counter */
    uint16_t adIsrCntBuf;              /*!< ad isr counter buf */
    uint16_t calCoff;                  /*!< fre calculate coff */
    int16_t  estThetaOffset;           /*!< theta offset at pwdt isr */
    int16_t  encoderCntOffset;         /*!< pwm1 cnt offset at pwdt isr */
} ENCODER_HALL_SENSOR;

/* ==========================================  Variables  =========================================== */
//extern PID_POSITION_IN  g_positionParam;        /*!< reserver */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Get pulse count captured by PWM module.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @return none
*/
void Encoder_GetPulseCount(ENCODER_TYPE *encoderSensor);

/*!
* @brief Obtain pulse count offset after encoder ALIGN.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @return none
*/
void Encoder_GetAlignOffset(ENCODER_TYPE *encoderSensor);

/*!
* @brief Statistic motor turn circles according to PWM module overflow direction.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
*
* @return none
*/
void Encoder_MotorCircleCalc(ENCODER_TYPE *encoderSensor);

/*!
* @brief Calculate encoder elec angle and update in foc control current loop.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @param[in] motor: pointer to MOTOR_PARAM structure
* @return encoder elec angle
*/
uint16_t Encoder_ElecAngleCalc(ENCODER_TYPE *encoderSensor, MOTOR_PARAM *motor);

/*!
* @brief Initialize g_encoderHallSensor struct. The initial rotor position get from hall sensor.
*        The coefficient is relevant to np (pole pairs number) and AB_encoder sensor revolution.
*
* @param[in] encoderHallSensor: hall and AB_encoder sensor data variable, pointer to ENCODER_HALL_SENSOR struct
* @param[in] hallSensor: hallSensor data, pointer to HALL_SENSOR_DATA structure
* @param[in] motor: pointer to MOTOR_PARAM structure
* @return none
*/
void EncoderHall_Init(ENCODER_HALL_SENSOR *encoderHallSensor, HALL_SENSOR_DATA *hallSensor, MOTOR_PARAM *motor);

/*!
* @brief Calculate rotor position. This function is called in erery ADC isr function.
*        Base on PWM modular, get counter per Ts, then calculate rotor position.
*
* @param[in] encoderHallSensor: hall and AB_encoder sensor data variable, pointer to ENCODER_HALL_SENSOR struct
* @return none
*/
void EncoderHall_AngleCal(ENCODER_HALL_SENSOR *encoderHallSensor);

/*!
* @brief If the AB_encoder sensor PULSE_PER_REVOLUTION is not accurate, maybe need compensate rotor position ervery Ts.
*        This function can make rotor position angle accumulate more precise.
*
* @param[in] encoderHallSensor: hall and AB_encoder sensor data variable, pointer to ENCODER_HALL_SENSOR struct
* @return none
*/
void EncoderHall_AngleStep(ENCODER_HALL_SENSOR *encoderHallSensor);

/*!
* @brief If only use AB_encoder sensor calculate rotor position, will generate cumulative error. In order to ekiminate cumulative error
*        need use hall adjust rotor position every electrical cycle. This function is called in PWDT interrupt.
*
* @param[in] encoderHallSensor: hall and AB_encoder sensor data variable, pointer to ENCODER_HALL_SENSOR struct
* @param[in] hallSensor: hallSensor data, pointer to HALL_SENSOR_DATA structure
* @return none
*/
void EncoderHall_AngleAdjust(ENCODER_HALL_SENSOR *encoderHallSensor, HALL_SENSOR_DATA *hallSensor);

/*!
* @brief Used to obtain electrical angle in encoder sensor FOC.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @return elec angle
*/
uint16_t Encoder_GetElecAngle(ENCODER_TYPE *encoderSensor);

/*!
* @brief Output encoder_hall mode rotor position estimated.
*
* @param[in] encoderHallSensor: hall and AB_encoder sensor data variable, pointer to ENCODER_HALL_SENSOR struct
* @return rotor postion angle estimation
*/
uint16_t EncoderHall_GetElecAngle(ENCODER_HALL_SENSOR *encoderHallSensor);

/*!
* @brief Encoder M_T measure method parameters initialization.
*
* @param[in] encoderMeasure: encoder speed measure variable, pointer to ENCODER_SPEED_MEASURE struct
* @return none
*/
void Encoder_MT_Init(ENCODER_SPEED_MEASURE *encoderMeasure);

/*!
* @brief Measure encoder pulse and time.
*
* @param[in] encoderMeasure: encoder speed measure variable, pointer to ENCODER_SPEED_MEASURE struct
* @return none
*/
void Encoder_MT_Measure(ENCODER_SPEED_MEASURE *encoderMeasure);

/*!
* @brief Calculate encoder pulse elec frequency by Measured encoder pulse and time.
*
* @param[in] encoderMeasure: encoder speed measure variable, pointer to ENCODER_SPEED_MEASURE struct
* @return none
*/
void Encoder_MT_SpeedCalc(ENCODER_SPEED_MEASURE *encoderMeasure);

/* ======================================  Functions define  ======================================== */

#endif /* __ENCODOR_H__ */

/* =============================================  EOF  ============================================== */
