/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __MOTOR_CTRL_H__
#define __MOTOR_CTRL_H__
/*!
* @file motorctrl.h
*
* @brief This file provides the FOC current loop control functions interface.
*
*/

/* ===========================================  Includes  =========================================== */
#include "coordnt.h"
#include "param_init.h"
#include "pid_regulator.h"
#include "flux_observer.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Define the conversion factor of motor control speed, current and voltage parameters from real value to pu value types structure.
*/
typedef struct
{
    int32_t speedFactor;                        /*!< Speed conversion factor */
    int32_t curFactor;                          /*!< Current conversion factor */
    int32_t voltageFactor;                      /*!< Voltage conversion factor */
} TRANSFORM_CFG;

/*!
* @brief Define the overvoltage suppression structure.
*/
typedef struct
{
    uint16_t overVoltPoint;                     /*!< Over voltage suppression action voltage */
    int32_t outLimit;                           /*!< Over voltage suppression output limit */
} OVER_VOLTAGE_SUPPRESS;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Calculation and modulation of SVPWM.
*
* @param[in] none
* @return none
*/
void Svpwm_Process(void);

/*!
* @brief Load the duty cycle calculated by SVPWM algorithm into the PWM module.
*
* @param[in] none
* @return none
*/
void LoadToPwmModule(void);

/*!
* @brief FOC current loop processing. According to the Vector Control principle,
*        the conduction time of each switch tube in the inverter is calculated
*        to drive the motor.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void CurrentLoop_Process(FOC_VARS_CTRL *focVars);

/*!
* @brief Motor control loop: Speed loop and Current loop parameters initialization.
*
* @param[in] none
* @return none
*/
void MC_SpeedTrqInit(void);

/*!
* @brief Current loop PI regulator input and output parameters initialization.
*
* @param[in] pidAcrD: pointer to PID_REGULATOR structure
* @param[in] pidAcrQ: pointer to PID_REGULATOR structure
* @param[in] pidCfg: pointer to PID_CFG structure
* @return none
*/
void Acr_CurPIInit(PID_REGULATOR *pidAcrD, PID_REGULATOR *pidAcrQ, PID_CFG *pidCfg);

/*!
* @brief Motor control loop: Position loop parameters initialization.
*
* @param[in] none
* @return none
*/
void MC_PositionInit(void);

/*!
* @brief Calculate the average value of motor stator current vector amplitude, expressed as 'ialphabeta'.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Acr_GetAvgIs(FOC_VARS_CTRL *focVars);

/*!
* @brief Calculate the average value of input variable.
*
* @param[in] avgVar: input variable before average
* @param[in] avgOrder: power exponent, used to calculated the average times, e.g. avgOrder = 5, calculate the average of (2^5 = 32) values
* @return avgValue: output variable after average
*/
int32_t GetAvgCalc(int32_t avgVar, uint16_t avgOrder);

/*!
* @brief Parameter identify run in isr. This function provides Rs Ld Lq Flux identify isr interface.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return temp, indicate whether to end the foc isr
*/
uint16_t ParamIdentify_AdcSubCallBack(FOC_VARS *focVars);

/*!
* @brief Over voltage suppression initialization.
*
* @param[in] overVoltSupp: pointer to OVER_VOLTAGE_SUPPRESS structure
* @param[in] pid: pointer to PID_REGULATOR structure
* @return none
*/
void OverVoltageSuppressInit(OVER_VOLTAGE_SUPPRESS *overVoltSupp, PID_REGULATOR *pid);

/*!
* @brief Over voltage suppression to prevent bus voltage overvoltage when Motor motion status is slow down.
*
* @param[in] udcPu: DC Bus voltage
* @param[in] overVoltSupp: pointer to OVER_VOLTAGE_SUPPRESS structure
* @param[in] pid: pointer to PID_REGULATOR structure
* @param[in] asrIn: pointer to PID_REGULATOR_IN structure
* @return none
*/
void OverVoltageSuppressProcess(uint16_t udcPu, OVER_VOLTAGE_SUPPRESS *overVoltSupp, PID_REGULATOR *pid, PID_REGULATOR_IN *asrIn);

/*!
* @brief Speed loop PI regulator input and output parameters initialization.
*
* @param[in] pidAsr: pointer to PID_REGULATOR structure
* @param[in] pidCfg: pointer to PID_CFG structure
* @return none
*/
void Asr_SpeedPIInit(PID_REGULATOR *pidAsr, PID_CFG *pidCfg);

/*!
* @brief Pre-exciting startup process of induction motor.
*
* @param[in] acimFoc: pointer to ACIM_FOC_CTRL structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] time: pre-exciting duration time, unit: ms
* @return PreExcit Startup status
*/
uint16_t Acim_FocPreExcitStartup(ACIM_FOC_CTRL *acimFoc, FOC_VARS *focVars, uint16_t time);

/* ======================================  Functions define  ======================================== */

#endif /* __MOTOR_CTRL_H__ */

/* =============================================  EOF  ============================================== */
