/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file hfi_api.c
*
* @brief This file provides the state machine and application layer functions of the high frequency injection algorithm.
*
*/

/* ===========================================  Includes  =========================================== */
#include "hfi_api.h"
#include "globalvar.h"
#include "motorctrl_api.h"
#include "foc_app.h"
#include "foc_keys.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief The main process of high frequency injection algorithm, including high frequency injection
*        state machine and application layer functions.
*
* @param[in] hfiCtrl: pointer to HFI_CTRL structure
* @param[in] hfiCfg: pointer to HFI_CFG structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] pidHfi: pointer to hfi pll PID_REGULATOR structure
* @param[in] pidAsr: pointer to speed loop PID_REGULATOR structure
* @return 0 or -1
*         0: success
*        -1: failure
*/
int16_t HighFreqInj_Proc(HFI_CTRL *hfiCtrl, HFI_CFG *hfiCfg, FOC_VARS *focVars, PID_REGULATOR *pidHfi, PID_REGULATOR *pidAsr)
{
    switch (hfiCtrl->hfi->state)
    {
    case HF_STATE0:
    hfiCtrl->hfi->vsqRef = 0;
    hfiCtrl->hfi->hfiVoltTheta += hfiCtrl->hfi->voltThetaInc;
    hfiCtrl->hfi->vsdRef = Math_Mpy(hfiCtrl->hfi->detectVpk, Math_Cos(hfiCtrl->hfi->hfiVoltTheta));
    focVars->pCtrl->elecAngle = Hfi_AngleObtain(hfiCtrl, pidHfi, focVars, hfiCfg);
    if (++hfiCtrl->hfi->delaycnt > (HFI_JUDGE_TIME1 << 1))
    {
        hfiCtrl->hfi->delaycnt = 0;
        hfiCtrl->initPos->index = 0;
        if (hfiCfg->hfiDebugEnable == (uint16_t)TRUE)
        {
            hfiCtrl->hfi->state = HF_STATE0;
        }
        else
        {
            hfiCtrl->hfi->state = HF_STATE1;
        }

        hfiCtrl->hfi->hfiEstTheta1 = hfiCtrl->hfi->hfiEstTheta;

    }
    break;

    case HF_STATE1:
    if (hfiCtrl->hfi->delaycnt < hfiCfg->clearTime)
    {
        hfiCtrl->hfi->delaycnt ++;
        hfiCtrl->hfi->hfiVoltTheta = 0;
        hfiCtrl->hfi->vsdRef = 0;
    }
    else
    {
        hfiCtrl->hfi->delaycnt = 0;
        hfiCtrl->hfi->state = HF_STATE2;
    }
    break;

    case HF_STATE2:
    Hfi_InitPolarCheck(hfiCtrl, focVars, hfiCfg);
    Acr_CurPIInit(&g_pidAcrD, &g_pidAcrQ, pPidCfg);
    hfiCtrl->hfi->hfiEstThetaErr = 0;
    hfiCtrl->hfi->hfiEstFreq = 0;
    focVars->pCtrl->idBeforeLpf = 0;
    focVars->pCtrl->iqBeforeLpf = 0;
    focVars->pCtrl->vd = 0;
    focVars->pCtrl->vq = 0;
    break;

    case HF_STATE3:
    hfiCtrl->hfi->hfiVoltTheta += hfiCtrl->hfi->voltThetaInc;
    hfiCtrl->hfi->vsdRef = Math_Mpy(hfiCtrl->hfi->detectVpk, Math_Cos(hfiCtrl->hfi->hfiVoltTheta));
    focVars->pCtrl->elecAngle = Hfi_AngleObtain(hfiCtrl, pidHfi, focVars, hfiCfg);
    hfiCtrl->hfi->weightFreq = pidHfi->pPidOutput->outPuFliter;
    #if (defined SMC_OBSERVE)
    g_startUpCtrl.elecThetaEst = Smc_PositionEstimator(&g_STC, &g_smcEstimator, &g_rotationalSpeed, &g_pidPll);
    #elif (defined FLUX_OBSERVE)
    g_startUpCtrl.elecThetaEst = FluxObserver_PosEst(&g_STC, &g_fluxObserver, &g_rotationalSpeed, &g_pidPll);
    #elif (defined MRAS_OBSERVE)
    g_startUpCtrl.elecThetaEst = Mras_PosEst(&g_STC, &g_mrasObserver, &g_pidPll, g_startUpCtrl.startUpFlag);
    #else
    #endif

    if (hfiCtrl->hfi->hfiToStoCount >= HFI_JUDGE_TIME2)
    {
        hfiCtrl->hfi->hfiToStoCount = HFI_JUDGE_TIME2;
        Hfi_InitPolarRecheck(hfiCtrl, focVars);
        if(hfiCfg->hfiToStoEnable == (uint16_t)ENABLE)
        {
            if (Math_Qabs(hfiCtrl->hfi->weightFreq) > HFI_FREQ_HIGH)
            {
                hfiCtrl->hfi->state = HF_STATE4;
                hfiCtrl->hfi->hfiToStoFlag = 1;
                g_pidPll.pPidOutput->uiPu = hfiCtrl->hfi->weightFreq;
                #if (defined FLUX_OBSERVE)
                g_fluxObserverCtrl.estTheta = hfiCtrl->hfi->hfiEstTheta;
                #elif (defined SMC_OBSERVE)
                g_smcEstimator.estTheta = hfiCtrl->hfi->hfiEstTheta;
                #elif (defined MRAS_OBSERVE)
                g_mrasObserver.estFreqBuf = hfiCtrl->hfi->weightFreq;
                g_mrasObserver.estTheta = hfiCtrl->hfi->hfiEstTheta;
                #endif
            }
        }
        else
        {
            hfiCtrl->hfi->hfiToStoFlag = 0;
        }
    }
    else
    {
        hfiCtrl->hfi->hfiToStoCount ++;
    }

    break;

    case HF_STATE4:
    #if (defined SMC_OBSERVE)
    g_startUpCtrl.elecThetaEst = Smc_PositionEstimator(&g_STC, &g_smcEstimator, &g_rotationalSpeed, &g_pidPll);
    #elif (defined FLUX_OBSERVE)
    g_startUpCtrl.elecThetaEst = FluxObserver_PosEst(&g_STC, &g_fluxObserver, &g_rotationalSpeed, &g_pidPll);
    #elif (defined MRAS_OBSERVE)
    g_startUpCtrl.elecThetaEst = Mras_PosEst(&g_STC, &g_mrasObserver, &g_pidPll, g_startUpCtrl.startUpFlag);
    #else
    #endif

    focVars->pCtrl->elecAngle = g_startUpCtrl.elecThetaEst;
    hfiCtrl->hfi->weightFreq = g_pidPll.pPidOutput->outPuFliter;

    if (Math_Qabs(hfiCtrl->hfi->weightFreq) < HFI_FREQ_LOW)
    {
        hfiCtrl->hfi->state = HF_STATE3;
        hfiCtrl->hfi->hfiToStoFlag = 0;
        hfiCtrl->hfi->hfiEstFreq = hfiCtrl->hfi->weightFreq;
        pidHfi->pPidOutput->uiPu = hfiCtrl->hfi->weightFreq;
        hfiCtrl->hfi->hfiThetaTemp = g_startUpCtrl.elecThetaEst;
        hfiCtrl->hfi->hfiEstTheta = g_startUpCtrl.elecThetaEst;
    }

    Hfi_VdAmp_Calc(hfiCtrl, hfiCfg);
    hfiCtrl->hfi->hfiVoltTheta += hfiCtrl->hfi->voltThetaInc;
    hfiCtrl->hfi->vsdRef = Math_Mpy(hfiCtrl->hfi->detectVpk, Math_Cos(hfiCtrl->hfi->hfiVoltTheta));
    Hfi_AngleObtain(hfiCtrl, pidHfi, focVars, hfiCfg);
    break;

    case HF_ERROR:
    Stop_Motor();
    FOC_Parameters_Initialize();
    ASR_SpeedCommandSet(pFocVarsCfg, &g_speedCommand);
    ASR_SetSpeedTargetRamp_Pu(&g_speedCommand, (HFI_INIT_FREQ * pFocVarsCfg->motorDir));
    PWM_Update(PWM_PERIOD_VALUE - 1, PWM_PERIOD_VALUE - 1, PWM_PERIOD_VALUE - 1);
    Set_McStatus(START);
    break;

    default:
    break;
    }

    /* Speed PID Control */
    if ((hfiCtrl->hfi->state == HF_STATE3) || (hfiCtrl->hfi->state == HF_STATE4))
    {
        if (hfiCtrl->hfi->hfiToStoCount == HFI_JUDGE_TIME2)
        {
            if (++hfiCtrl->hfi->spdCtrlloopCnt > hfiCtrl->hfi->msUnit)
            {
                hfiCtrl->hfi->spdCtrlloopCnt = 0;
                ASR_SpeedTargetRampCalc(pFocVarsCfg, &g_speedCommand);
                pidAsr->pPidInput->refPu = g_speedCommand.speedTargetRamp;
                pidAsr->pPidInput->fbkPu = hfiCtrl->hfi->weightFreq;
                focVars->pCtrl->iqRef = PID_ParallelRegulator(pidAsr);
                focVars->pCtrl->idRef = 0;
            }
            if ((hfiCtrl->hfi->weightFreq * focVars->pCfg->motorDir) < (-HFI_DIR_ERR_FREQ))
            {
                if (hfiCtrl->hfi->dirErrCnt < HFI_DIR_ERR_TIME)
                {
                    hfiCtrl->hfi->dirErrCnt ++;
                }
                else
                {
                    hfiCtrl->hfi->dirErrCnt = 0;
                    hfiCtrl->hfi->state = HF_ERROR;
                }
            }
            else
            {
                hfiCtrl->hfi->dirErrCnt = 0;
            }
        }
    }

    return 0;
}

/* =============================================  EOF  ============================================== */
