/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file protector.c
*
* @brief This file provides motor protection functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "adc_hw.h"
#include "foc_app.h"
#include "globalvar.h"
#include "motorctrl_api.h"
#include "foc_keys.h"
#include "pulse_inject_api.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
STALL_PROTECT     g_stallProtect;     /*!< Stall protection structure */
POWER_DECREASE    g_powerDecrease;    /*!< Power reduced protection structure */
DCCUR_LIMIT       g_dcCurLimit;       /*!< DC Bus current limit structure */
POWER_LIMIT       g_powerLimit;       /*!< Input power limit structure */
FOC_RESTARTUP     g_restart;          /*!< startup error restart structure */
OVERLOAD_PRO      g_overLoadPro;      /*!< over load protection structure */

int32_t g_estBemfDebug = 0;           /*!< Back-EMF estimated value */
int32_t g_bemfReliableThreshold = 0;  /*!< Back-EMF reliable threshold value */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Clear Fault identification.
*
* @param[in] faultid: Fault identification
* @return none
*/
void Fault_Clear(uint16_t faultid)
{
    g_protector.ErrFlag.all &= ~(1U << faultid);
}

/*!
* @brief Report Fault identification.
*
* @param[in] faultid: Fault identification
* @return none
*/
void Fault_Report(uint16_t faultid)
{
    g_protector.ErrFlag.all |= (1U << faultid);
}

/*!
* @brief System protection initialization.
*
* @param[in] none
* @return none
*/
void Init_Protection(void)
{
    /* Clear all Error Flag */
    memset(&g_protector, 0 , sizeof(g_protector));
    g_protector.losePhaseDelay = 100;
}

/*!
* @brief Set motor fault code and recovery time.
*
* @param[in] error: Fault identification
* @return recoveryDelay: recovery delay time
*/
uint16_t SetMotorErrCode(uint16_t error)
{
    uint16_t recoveryDelay = 0;

    Fault_Report(error);
    if (g_protector.ErrFlag.all)
    {
        if (error == LOSE_PHASEA_FAILURE)
        {
            recoveryDelay = 1 * P_SEC_UINT;
        }
        else if (error == LOSE_PHASEB_FAILURE)
        {
            recoveryDelay = 1 * P_SEC_UINT;
        }
        else if (error == LOSE_PHASEC_FAILURE)
        {
            recoveryDelay = 1 * P_SEC_UINT;
        }
        else if (error == LOSE_SPEED_FAILURE)
        {
            recoveryDelay = 1 * P_SEC_UINT;
        }
        else
        {
            recoveryDelay = 1 * P_SEC_UINT;
        }

    }

    return recoveryDelay;
}

/*!
* @brief Current sencor sampling zero drift check. The current sampling
*        ADC zero drift value exceeds the limit, it is a fault.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @return none
*/
void Current_Sensor_Check(ADC_SAMPLE *adcSampleReal)
{
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    /* 2048 *5% = 102 */
    if (Math_Qabs(adcSampleReal->ibusOffset - 2048) > 102)
    {
        Fault_Report(PHASEA_CURRENT_MIDPOINT_FAILURE);
    }
    else
    {
        Fault_Clear(PHASEA_CURRENT_MIDPOINT_FAILURE);
    }

#else
    /* 2048 *5% = 102 */
    if (Math_Qabs(adcSampleReal->iaOffset - 2048) > 102)
    {
        Fault_Report(PHASEA_CURRENT_MIDPOINT_FAILURE);
    }
    else
    {
        Fault_Clear(PHASEA_CURRENT_MIDPOINT_FAILURE);
    }

    if (Math_Qabs(adcSampleReal->ibOffset - 2048) > 102)
    {
        Fault_Report(PHASEB_CURRENT_MIDPOINT_FAILURE);
    }
    else
    {
        Fault_Clear(PHASEB_CURRENT_MIDPOINT_FAILURE);
    }

#if (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    /* 2048 *5% = 102 */
    if (Math_Qabs(adcSampleReal->icOffset - 2048) > 102)
    {
        Fault_Report(PHASEC_CURRENT_MIDPOINT_FAILURE);
    }
    else
    {
        Fault_Clear(PHASEC_CURRENT_MIDPOINT_FAILURE);
    }
#else
#endif
#endif
}

/*!
* @brief DC Bus over voltage protection. When the bus voltage exceeds the
*        threshold value, Overvoltage fault is reported, and it will return
*        to normal operation after the bus voltage drops to the recovery threshold.
*
* @param[in] volt: DC Bus voltage value, 10 times the real value
* @return none
*/
void Vbus_OverVoltage_Check(uint16_t volt)
{
    static int16_t s_vbusStatus = VBUS_NORMAL;
    static int16_t s_prevVbusStatus = VBUS_NORMAL;
    static int16_t s_debounce = 0;

    if (volt > VBUS_OVERVOLTAGE_THRESHOLD)
    {
        s_vbusStatus = VBUS_OVERVOLTAGE;
    }
    else if (volt < VBUS_OVERVOLTAGE_RECOVERY_THRESHOLD)
    {
        if (s_vbusStatus == VBUS_OVERVOLTAGE)
        {
            s_vbusStatus = VBUS_RECOVER;
        }
    }
    else
    {
    }

    if (s_vbusStatus != s_prevVbusStatus)
    {
        s_prevVbusStatus = s_vbusStatus;
        s_debounce = 0;
    }
    else
    {
        if (s_vbusStatus == VBUS_RECOVER)
        {
            if (s_debounce < 10)
            {
                s_debounce++;
            }
            else
            {
//                PushIn_McBufStatus(STOP);
                s_vbusStatus = VBUS_NORMAL;
                Fault_Clear(VBUS_OVER_VOLTAGE_FAILURE);
            }
        }
        else if (s_vbusStatus == VBUS_NORMAL)
        {
            if (s_debounce < 10)
            {
                s_debounce++;
            }
            else
            {
                Fault_Clear(VBUS_OVER_VOLTAGE_FAILURE);
            }
        }
        else
        {
            if (s_debounce < 10)
            {
                s_debounce++;
            }
            else
            {
                Fault_Report(VBUS_OVER_VOLTAGE_FAILURE);
            }
        }
    }
}

/*!
* @brief DC Bus under voltage protection. When the bus voltage is lower than
*        threshold value, Undervoltage fault is reported, and it will return
*        to normal operation after the bus voltage rises to the recovery threshold.
*
* @param[in] volt: DC Bus voltage value, 10 times the real value
* @return none
*/
void Vbus_UnderVoltage_Check(uint16_t volt)
{
    static int16_t s_vbusStatus = VBUS_NORMAL;
    static int16_t s_prevVbusStatus = VBUS_NORMAL;
    static int16_t s_debounce = 0;

    if (volt < VBUS_UNDERVOLTAGE_THRESHOLD)
    {
        s_vbusStatus = VBUS_UNDERVOLTAGE;
    }
    else if (volt > VBUS_UNDERVOLTAGE_RECOVERY_THRESHOLD)
    {
        if (s_vbusStatus == VBUS_UNDERVOLTAGE)
        {
            s_vbusStatus = VBUS_RECOVER;
        }
    }
    else
    {
    }

    if (s_vbusStatus != s_prevVbusStatus)
    {
        s_prevVbusStatus = s_vbusStatus;
        s_debounce = 0;
    }
    else
    {
        if (s_vbusStatus == VBUS_RECOVER)
        {
            if (s_debounce < 10)
            {
                s_debounce++;
            }
            else
            {
//                PushIn_McBufStatus(STOP);
                s_vbusStatus = VBUS_NORMAL;
                Fault_Clear(VBUS_UNDER_VOLTAGE_FAILURE);

            }
        }
        else if (s_vbusStatus == VBUS_NORMAL)
        {
            if (s_debounce < 10)
            {
                s_debounce++;
            }
            else
            {
                Fault_Clear(VBUS_UNDER_VOLTAGE_FAILURE);
            }
        }
        else
        {
            if (s_debounce < 10)
            {
                s_debounce++;
            }
            else
            {
                Fault_Report(VBUS_UNDER_VOLTAGE_FAILURE);
            }
        }
    }
}

/*!
* @brief Phase over current protection. When the amplitude of the phase current
*        synthesis vector exceeds the threshold value, phase over current fault is reported.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
int16_t Phase_Over_Current_Check(FOC_VARS_CTRL *focVars)
{
    static int16_t cnta = 0;
    int16_t status = DIAGNOSTIC_OK;
    uint16_t statusTemp = 0;

    if (focVars->idq > MAX_IPHASE_THRESHOLD)
    {
        statusTemp |= 0x1;
        if (cnta < IPHASE_DBC)
        {
            cnta++;
        }
        else
        {
            cnta = 0;
            Fault_Report(OVER_CURRENT_PHASE_FAILURE);
        }
    }
    else
    {
        cnta = 0;
    }

    if (statusTemp != 0)
    {
        status = DIAGNOSTIC_FAIL;
    }

    return status;
}

/*!
* @brief Motor over load decrease frequency protection parameters initilization.
*
* @param[in] overLoad: pointer to OVERLOAD_PRO structure
* @return none
*/
void Is_OverLoad_Init(OVERLOAD_PRO *overLoad)
{
    memset(overLoad, 0, sizeof(*overLoad));
    overLoad->fallFreqCof = Math_IQ(1.0);
}

/*!
* @brief Motor over load decrease frequency protection. When the amplitude of the phase current synthesis vector exceeds
*        the threshold value and for a certain amount of time, the frequency command will fall.
*
* @param[in] overLoad: pointer to OVERLOAD_PRO structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void Is_OverLoad_FallFreq(OVERLOAD_PRO *overLoad, FOC_VARS_CTRL *focVars, SPEED_RAMP_HANDLE *speed)
{
    overLoad->inputIs = Math_Q15Mag(focVars->idFdb, focVars->iqFdb);
    overLoad->filterIs = (overLoad->inputIs + overLoad->filterIs * 127) >> 7;

    switch (overLoad->state)
    {
        case OL_STATE_RUN:

            /* OL_JUDGE_CUR1: Math_IQ(0.85) */
            if (overLoad->filterIs > OL_JUDGE_CUR1)
            {
                overLoad->fallTime1++;
            }
            else
            {
                overLoad->fallTime1 = 0;
            }

            /* OL_JUDGE_CUR2: Math_IQ(0.9) */
            if (overLoad->filterIs > OL_JUDGE_CUR2)
            {
                overLoad->fallTime2++;
            }
            else
            {
                overLoad->fallTime2 = 0;
            }

            if ((overLoad->fallTime1 > OL_JUDGE_TIME1) || (overLoad->fallTime2 > OL_JUDGE_TIME2))
            {
//                if (Math_Qabs(speed->speedTarget) > MIN_SPEED_VALUE)
                if (Math_Qabs(speed->speedTarget) > speed->speedMin)
                {
                    overLoad->state = OL_STATE_FALL;
                    overLoad->speedBuff = speed->speedTarget;
                }
                else
                {
                    overLoad->state = OL_STATE_FAULT;
                }
                overLoad->fallTime1 = 0;
                overLoad->fallTime2 = 0;
            }
            /* OL_JUDGE_CUR3: Math_IQ(0.95) */
            if (overLoad->filterIs > OL_JUDGE_CUR3)
            {
                overLoad->state = OL_STATE_FALL;
                overLoad->speedBuff = speed->speedTarget;
            }
        break;

        case OL_STATE_HOLD:
            if (overLoad->filterIs <= OL_RECOVER_CUR)
            {
                overLoad->recoverTime++;
            }
            else
            {
                overLoad->recoverTime = 0;
            }

            if (overLoad->recoverTime > HOLD_TO_RUN_TIME)
            {
                overLoad->state = OL_STATE_RUN;
                overLoad->recoverTime = 0;

                speed->speedTarget = overLoad->speedBuff;
                overLoad->fallFreqCof = Math_IQ(1.0);
                overLoad->fallFreqFlag = 0;
            }

            if (overLoad->filterIs > OL_JUDGE_CUR1)
            {
                overLoad->holdToFallTime++;
            }
            else
            {
                overLoad->holdToFallTime = 0;
            }
            if (overLoad->holdToFallTime > HOLD_TO_FALL_TIME)
            {
//                if (Math_Qabs(speed->speedTarget) > MIN_SPEED_VALUE)
                if (Math_Qabs(speed->speedTarget) > speed->speedMin)
                {
                    overLoad->state = OL_STATE_FALL;
                }
                else
                {
                    overLoad->state = OL_STATE_FAULT;
                }
                overLoad->holdToFallTime = 0;
            }
        break;

        case OL_STATE_FALL:
            overLoad->fallFreqCof = OL_FALL_FREQ_COF;
            overLoad->fallFreqFlag = 1;
            speed->speedTarget = Math_Mpy(speed->speedFbk, overLoad->fallFreqCof);
            if (pFocVarsCfg->motorDir == 1)
            {
//                speed->speedTarget = Math_Sat(speed->speedTarget, MAX_SPEED_VALUE, MIN_SPEED_VALUE);
                speed->speedTarget = Math_Sat(speed->speedTarget, speed->speedMax, speed->speedMin);
            }
            else
            {
//                speed->speedTarget = Math_Sat(speed->speedTarget, -MIN_SPEED_VALUE, -MAX_SPEED_VALUE);
                speed->speedTarget = Math_Sat(speed->speedTarget, -(speed->speedMin), -(speed->speedMax));
            }
            overLoad->state = OL_STATE_HOLD;
        break;

        case OL_STATE_FAULT:
            Fault_Report(OVER_CURRENT_IS_FAILURE);
        break;

        default:
        break;
    }
}

/*!
* @brief DC Bus over current protection. When the DC Bus current exceeds
*        the threshold value, DC Bus over current fault is reported.
*
* @param[in] ibus: DC Bus current real vaule
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
int16_t Bus_Over_Current_Check(int16_t ibus)
{
    static int16_t cnt = 0;
    int16_t status = DIAGNOSTIC_OK;
    uint16_t statusTemp = 0;

    if (ibus > BUS_OVERCURRENT_THRESHOLD)
    {
        statusTemp = 1;
        if (cnt < BUS_OVERCURRENT_DBC)
        {
            cnt++;
        }
        else
        {
            cnt = 0;
            Fault_Report(OVER_CURRENT_IBUS_FAILURE);
        }
    }
    else
    {
        cnt = 0;
    }

    if (statusTemp != 0)
    {
        status = DIAGNOSTIC_FAIL;
    }

    return status;
}

/*!
* @brief Motor stall protection detection for position sensor application.
*
* @param[in] spdRpm: Motor rotation speed real value unit RPM
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
int16_t Stall_Check_Sensor(int16_t spdRpm, FOC_VARS_CTRL *focVars)
{
    int16_t status = DIAGNOSTIC_OK;
    static int16_t cnt = 0;
    uint16_t isAmp = 0;

    isAmp = Math_Q15Mag(focVars->idRef, focVars->iqRef);
    if ((Math_Qabs(spdRpm) < STALL_SPD_THRESHOLD) && (isAmp > STALL_CUR_THRESHOLD))
    {
        if (cnt < STALL_DBC)
        {
            cnt ++;
        }
        else
        {
            cnt = 0;
            status = DIAGNOSTIC_FAIL;
            Fault_Report(MOTOR_STALL_FAILURE);
        }
    }
    else
    {
        cnt = 0;
//        Fault_Clear(MOTOR_STALL_FAILURE);
    }
    return status;
}

/*!
* @brief Motor stall protection detection for position sensorless application.
*
* @param[in] pHandle: pointer to SPEED_RAMP_HANDLE structure
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
int16_t Stall_Check_Sensorless(SPEED_RAMP_HANDLE *pHandle)
{
    int16_t status = DIAGNOSTIC_OK;

    static int16_t bemfReliableCnt = 0;

    if (g_startUpCtrl.startUpFlag == 6)
    {
        g_bemfReliableThreshold = (int32_t)(pHandle->avgSpeedFbk);

        #if (defined SMC_OBSERVE)
        g_estBemfDebug = g_smcEstimator.bemfAmp * BEMF_CONSISTENT_THRESHOLD;
        #elif (defined FLUX_OBSERVE)
        g_estBemfDebug = g_fluxObserverCtrl.emfAmp * BEMF_CONSISTENT_THRESHOLD;
        #endif
        if (g_estBemfDebug < g_bemfReliableThreshold)
        {
            if (++bemfReliableCnt >= BEMF_DBC)
            {
                status = DIAGNOSTIC_FAIL;
                bemfReliableCnt = 0;
                g_protector.stallProFlag = 1;
                #if (defined STARTUP_CHECK_RESTART_ENABLE)
                #else
                Fault_Report(MOTOR_STALL_FAILURE);
                #endif
            }
        }
        else
        {
            bemfReliableCnt=0;
        }
    }
    else
    {
//        g_estBemfDebug = 0;
//        g_bemfReliableThreshold = 0;
    }

    return status;
}

/*!
* @brief Motor Speed protection detection.
*
* @param[in] spdfbk: Speed feedback pu value
* @return none
*/
void Speed_Protect(int16_t spdfbk)
{
    if (g_startUpCtrl.startUpFlag == 0)
    {
        if (g_startUpCtrl.startUpTimes == 0)
        {
            g_protector.startUpDelayCounter = 0;
        }
        else if (g_startUpCtrl.startUpTimes < 2)
        {
            g_protector.startUpDelayCounter++;
            /* 10 s */
            if (g_protector.startUpDelayCounter > 10000)
            {
                g_protector.startUpDelayCounter = 0;
                ClearStartUpParam(&g_startUp);
                //Smc_ParamInit();
                g_startUpCtrl.startUpFlag = 1;
                PWM_Enable();
            }
        }
        else
        {
            //g_startUpCtrl.startUp_Times = 0;
            g_protector.startUpErrFlag = 1;
            //Fault_Report(STARTUP_FAILURE);
        }
    }
    else if (g_startUpCtrl.startUpFlag < 3)
    {
        g_protector.countSpeedProA1 = 0;
        g_protector.countSpeedProA2 = 0;
        g_protector.countSpeedProA3 = 0;
        g_protector.countSpeedProB1 = 0;
        g_protector.countSpeedProB2 = 0;
        g_protector.countSpeedProB3 = 0;
    }
    else if (g_startUpCtrl.startUpFlag < 6)
    {
        if ((Math_Qabs(g_adcTransform.iaPu) > Math_IQ(0.95)) || (Math_Qabs(g_adcTransform.ibPu) > Math_IQ(0.95))
        || (Math_Qabs(g_adcTransform.icPu) > Math_IQ(0.95)))
        {
            g_startUpCtrl.startUpFlag = 0;
            PWM_Disable();
            g_startUpCtrl.startUpTimes++;
        }

        if (g_startUpCtrl.startUpFlag > 3)
        {
            /* 0 Hz */
            if (spdfbk < Math_IQ(0))
            {
                g_protector.countSpeedProA1++;
                /* 100 ms */
                if (g_protector.countSpeedProA1 > 100)
                {
                    g_protector.countSpeedProA1 = 0;
                    g_startUpCtrl.startUpFlag = 0;
                    PWM_Disable();
                    g_startUpCtrl.startUpTimes++;
                }
            }
            /* 0.025 * BASE_FREQ Hz */
            else if (spdfbk < Math_IQ(0.025))
            {
                g_protector.countSpeedProA2++;
                /* 1 s */
                if (g_protector.countSpeedProA2 > 1000)
                {
                    g_protector.countSpeedProA2 = 0;
                    g_startUpCtrl.startUpFlag = 0;
                    PWM_Disable();
                    g_startUpCtrl.startUpTimes++;
                }
            }
            /* 0.4 * BASE_FREQ Hz */
            else if (spdfbk > Math_IQ(0.4))
            {
                g_protector.countSpeedProA3++;
                /* 0.5 s */
                if (g_protector.countSpeedProA3 > 500)
                {
                    g_protector.countSpeedProA3 = 0;
                    g_startUpCtrl.startUpFlag = 0;
                    PWM_Disable();
                    g_startUpCtrl.startUpTimes++;
                }
            }
            else
            {
                g_protector.countSpeedProA1 = 0;
                g_protector.countSpeedProA2 = 0;
                g_protector.countSpeedProA3 = 0;
            }
        }
    }
    else
    {
        g_protector.countSpeedProA1 = 0;
        g_protector.countSpeedProA2 = 0;
        g_protector.countSpeedProA3 = 0;
        /* 0 Hz */
        if (spdfbk < Math_IQ(0))
        {
            g_protector.countSpeedProB1++;
            /* 50 ms */
            if (g_protector.countSpeedProB1 > 50)
            {
                g_protector.countSpeedProB1 = 0;
                g_protector.loseSpeedProFlag = 1;
                //Fault_Report(LOSE_SPEED_FAILURE);
            }
        }
        else
        {
            g_protector.countSpeedProB1 = 0;
        }

        /* 0.02 * BASE_FREQ Hz */
        if (spdfbk < Math_IQ(0.02))
        {
            g_protector.countSpeedProB2++;
            /* 2 s */
            if (g_protector.countSpeedProB2 > 2000)
            {
                g_protector.countSpeedProB2 = 0;
                g_protector.loseSpeedProFlag = 1;
                //Fault_Report(ZERO_SPEED_FAILURE);
            }
        }
        else
        {
            g_protector.countSpeedProB2 = 0;
        }

        /* 0.95 * BASE_FREQ Hz */
        if (spdfbk > Math_IQ(0.95))
        {
            g_protector.countSpeedProB3++;
            /* 200 ms */
            if (g_protector.countSpeedProB3 > 200)
            {
                g_protector.countSpeedProB3 = 0;
                g_protector.loseSpeedProFlag = 1;
                //Fault_Report(LOSE_SPEED_FAILURE);
            }
        }
        else
        {
            g_protector.countSpeedProB3 = 0;
        }
    }

    return;
}

/*!
* @brief Motor Startup protection detection.
*
* @param[in] none
* @return none
*/
void StartUp_Check(void)
{
    /* The fault phenomenon is kept for at least 30 seconds */
    if (g_protector.startUpErrDelay)
    {
        g_protector.startUpErrDelay--;
    }
    else
    {
        //if (g_protector.ErrFlag.all | (1 << STARTUP_FAILURE))
        if (g_protector.startUpErrFlag)
        {
            g_protector.startUpErrFlag = 0;
            Fault_Clear(STARTUP_FAILURE);
            g_protector.startUpErrDelay = SetMotorErrCode(STARTUP_FAILURE);
        }
        else
        {
            //Fault_Clear(STARTUP_FAILURE);
        }
    }

    return;
}

/*!
* @brief Motor Zero Speed protection.
*
* @param[in] speed: Speed feedback pu value
* @return none
*/
void ZeroSpeed_Check(int16_t speed)
{
    static uint16_t s_zeroSpeedCnt = 0;

    if (g_startUpCtrl.startUpFlag >= 6)
    {
        if (Math_Qabs(speed) < ZERO_SPEED_THRESHOLD)
        {
            if (s_zeroSpeedCnt < ZERO_SPEED_DBC)
            {
                s_zeroSpeedCnt++;
            }
            else
            {
                s_zeroSpeedCnt = 0;
                g_protector.zeroSpeedProFlag = 1;
                #if (defined STARTUP_CHECK_RESTART_ENABLE)
                #else
                Fault_Report(ZERO_SPEED_FAILURE);
                #endif
            }
        }
        else
        {
            s_zeroSpeedCnt = 0;
//            Fault_Clear(ZERO_SPEED_FAILURE);
        }
    }

}

/*!
* @brief Motor Lose Speed protection.
*
* @param[in] speed: speed feedback pu value
* @return none
*/
void LoseSpeed_Check(int16_t speed)
{
    static int16_t oldSpeed = 0;
    static uint16_t cntLoseSpeed = 0;

    if (g_startUpCtrl.startUpFlag > 3)
    {
        if ((Math_Qabs(speed) > LOSE_SPEED_THRESHOLD) || ((int16_t)Math_Mpy(pFocVarsCfg->motorDir, speed) < 0) || (Math_Qabs(speed - oldSpeed) > LOSE_SPEED_ERR_THRESHOLD))
        {
            if (cntLoseSpeed < LOSE_SPEED_DBC)
            {
                cntLoseSpeed++;
            }
            else
            {
                cntLoseSpeed = 0;
                oldSpeed = 0;
                g_protector.loseSpeedProFlag = 1;
                #if (defined STARTUP_CHECK_RESTART_ENABLE)
                #else
                Fault_Report(LOSE_SPEED_FAILURE);
                #endif
            }
        }
        else
        {
            cntLoseSpeed = 0;
//            Fault_Clear(LOSE_SPEED_FAILURE);
        }
        oldSpeed = speed;
    }
}

/*!
* @brief Startup error restart parameters initialization.
*
* @param[in] restart: pointer to FOC_RESTARTUP structure
* @return none
*/
void Foc_RestartInit(FOC_RESTARTUP *restart)
{
    memset(restart, 0U, sizeof(*restart));
}

/*!
* @brief Startup error restart judge process.
*
* @param[in] restart: pointer to FOC_RESTARTUP structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void Foc_RestartJudgeProcess(FOC_RESTARTUP *restart, SPEED_RAMP_HANDLE *speed)
{
    restart->startUpCnt++;
    if (restart->startUpCnt < RESTART_JUDGE_TIME)
    {
        if ((g_startUpCtrl.startUpFlag == 6) && (restart->restartDoneFlag == 1))
        {
            speed->speedTarget = restart->restartSpeedBuf;
            restart->restartDoneFlag = 0;
        }
        else
        {
        }
        if ((g_protector.zeroSpeedProFlag == 1) || (g_protector.loseSpeedProFlag == 1) || (g_protector.stallProFlag == 1))
        {
            restart->runToRestartFlag = 1;
            restart->restartSpeedBuf = speed->speedTarget;
        }
        else
        {
            restart->runToRestartFlag = 0;
        }
        if (restart->runToRestartFlag == 1)
        {
            (restart->restartDlyCnt)++;
            if (restart->restartDlyCnt >= RESTART_DELAY)
            {
                restart->restartDlyCnt = 0;
                restart->restartFlag = 1;
            }
            else
            {
                PWM_Disable();
            }
            if (restart->restartFlag == 1)
            {
                PushIn_McBufStatus(RESTART);
                g_startUpCtrl.startUpFlag = 0;
                restart->restartTimes += 1;
            }
        }
        else
        {
            restart->restartFlag = 0;
        }
        if (restart->restartTimes > 0)
        {
            if (restart->restartTimes > RESTART_NUMBER)
            {
                restart->overLoadFault = 1;
                Fault_Report(STARTUP_FAILURE);
            }
            else
            {
                restart->overLoadFault = 0;
            }
        }
    }
    else
    {
        Foc_RestartInit(&g_restart);
        restart->startUpCnt = RESTART_JUDGE_TIME;
        restart->runToRestartFlag = 0;
        if (g_protector.zeroSpeedProFlag == 1)
        {
            Fault_Report(ZERO_SPEED_FAILURE);
        }
        if (g_protector.loseSpeedProFlag == 1)
        {
            Fault_Report(LOSE_SPEED_FAILURE);
        }
    }
}

/*!
* @brief Startup error restart state process.
*
* @param[in] restart: pointer to FOC_RESTARTUP structure
* @return none
*/
void Foc_RestartProcess(FOC_RESTARTUP *restart)
{
    FOC_Parameters_Initialize();
    PWM_Update(PWM_PERIOD_VALUE - 1, PWM_PERIOD_VALUE - 1, PWM_PERIOD_VALUE - 1);
    if (restart->restartFlag == 1)
    {
        restart->restartFlag = 0;
        restart->runToRestartFlag = 0;
    }
    else
    {
    }
    if (restart->restartFlag == 0)
    {
        restart->restartDoneFlag = 1;
    }
}

/*!
* @brief Motor lose phase protection in Stator resistanse identification.
*
* @param[in] paramId: pointer to PARAM_IDENTIFY_STRUCT structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void LosePhase_RsIdentify_Check(PARAM_IDENTIFY_STRUCT *paramId, FOC_VARS_CTRL *focVars)
{
    PARAM_RS_IDENTIFY_DATA *paramRsId = paramId->pParamRsIdentify;
    int32_t temp_CurErr1, temp_CurErr2, temp_CurErr3;
    int32_t temp_CurErrLimit = Math_IQ(0.05);

    /* subStep 0~5 have current, subStep 6~7 have no current injection and PWM is disable, so it is not necessary to check lose phase */
    /* in the phase A progress */
    if ((paramRsId->phaseOrder == 0) && (paramRsId->subStep < 5))
    {
        temp_CurErr1 = Math_Qabs(focVars->iaPu) - Math_Qabs(focVars->ibPu) - Math_Qabs(focVars->ibPu);
        temp_CurErr2 = Math_Qabs(focVars->iaPu) - Math_Qabs(focVars->icPu) - Math_Qabs(focVars->icPu);
        temp_CurErr3 = Math_Qabs(focVars->ibPu) - Math_Qabs(focVars->icPu);
        if (Math_Qabs(focVars->iaPu) < LOSE_PHASE_THRESHOLD)
        {
            if (g_protector.losAphTime < LOSE_PHASE_DBC)
            {
                g_protector.losAphTime++;
            }
            else
            {
                Fault_Report(LOSE_PHASEA_FAILURE);
                return;
            }
        }
        else
        {
            g_protector.losAphTime = 0;
            if ((temp_CurErr1 > temp_CurErrLimit) && (temp_CurErr2 < -temp_CurErrLimit) && (temp_CurErr3 < -temp_CurErrLimit))
            {
                if (g_protector.losBphTime < LOSE_PHASE_DBC)
                {
                    g_protector.losBphTime++;
                }
                else
                {
                    Fault_Report(LOSE_PHASEB_FAILURE);
                    return;
                }
            }
            else if ((temp_CurErr1 < -temp_CurErrLimit) && (temp_CurErr2 > temp_CurErrLimit) && (temp_CurErr3 > temp_CurErrLimit))
            {
                if (g_protector.losCphTime < LOSE_PHASE_DBC)
                {
                    g_protector.losCphTime++;
                }
                else
                {
                    Fault_Report(LOSE_PHASEC_FAILURE);
                    return;
                }
            }
            else
            {
                g_protector.losBphTime = 0;
                g_protector.losCphTime = 0;
            }
        }
    }
    /* in the phase B progress */
    else if ((paramRsId->phaseOrder == 1) && (paramRsId->subStep < 5))
    {
        temp_CurErr1 = Math_Qabs(focVars->ibPu) - Math_Qabs(focVars->iaPu) - Math_Qabs(focVars->iaPu);
        temp_CurErr2 = Math_Qabs(focVars->ibPu) - Math_Qabs(focVars->icPu) - Math_Qabs(focVars->icPu);
        temp_CurErr3 = Math_Qabs(focVars->iaPu) - Math_Qabs(focVars->icPu);
        if (Math_Qabs(focVars->ibPu) < LOSE_PHASE_THRESHOLD)
        {
            if (g_protector.losBphTime < LOSE_PHASE_DBC)
            {
                g_protector.losBphTime++;
            }
            else
            {
                Fault_Report(LOSE_PHASEB_FAILURE);
                return;
            }
        }
        else
        {
            g_protector.losBphTime = 0;
            if ((temp_CurErr1 > temp_CurErrLimit) && (temp_CurErr2 < -temp_CurErrLimit) && (temp_CurErr3 < -temp_CurErrLimit))
            {
                if (g_protector.losAphTime < LOSE_PHASE_DBC)
                {
                    g_protector.losAphTime++;
                }
                else
                {
                    Fault_Report(LOSE_PHASEA_FAILURE);
                    return;
                }
            }
            else if ((temp_CurErr1 < -temp_CurErrLimit) && (temp_CurErr2 > temp_CurErrLimit) && (temp_CurErr3 > temp_CurErrLimit))
            {
                if (g_protector.losCphTime < LOSE_PHASE_DBC)
                {
                    g_protector.losCphTime++;
                }
                else
                {
                    Fault_Report(LOSE_PHASEC_FAILURE);
                    return;
                }
            }
            else
            {
                g_protector.losAphTime = 0;
                g_protector.losCphTime = 0;
            }
        }
    }
    /* in the phase C progress */
    else if ((paramRsId->phaseOrder == 2) && (paramRsId->subStep < 5))
    {
        temp_CurErr1 = Math_Qabs(focVars->icPu) - Math_Qabs(focVars->iaPu) - Math_Qabs(focVars->iaPu);
        temp_CurErr2 = Math_Qabs(focVars->icPu) - Math_Qabs(focVars->ibPu) - Math_Qabs(focVars->ibPu);
        temp_CurErr3 = Math_Qabs(focVars->ibPu) - Math_Qabs(focVars->iaPu);
        if (Math_Qabs(focVars->icPu) < LOSE_PHASE_THRESHOLD)
        {
            if (g_protector.losCphTime < LOSE_PHASE_DBC)
            {
                g_protector.losCphTime++;
            }
            else
            {
                Fault_Report(LOSE_PHASEC_FAILURE);
                return;
            }
        }
        else
        {
            g_protector.losCphTime = 0;
            if ((temp_CurErr1 > temp_CurErrLimit) && (temp_CurErr2 < -temp_CurErrLimit) && (temp_CurErr3 < -temp_CurErrLimit))
            {
                if (g_protector.losAphTime < LOSE_PHASE_DBC)
                {
                    g_protector.losAphTime++;
                }
                else
                {
                    Fault_Report(LOSE_PHASEA_FAILURE);
                    return;
                }
            }
            else if ((temp_CurErr1 < -temp_CurErrLimit) && (temp_CurErr2 > temp_CurErrLimit) && (temp_CurErr3 > temp_CurErrLimit))
            {
                if (g_protector.losBphTime < LOSE_PHASE_DBC)
                {
                    g_protector.losBphTime++;
                }
                else
                {
                    Fault_Report(LOSE_PHASEB_FAILURE);
                    return;
                }
            }
            else
            {
                g_protector.losAphTime = 0;
                g_protector.losBphTime = 0;
            }
        }
    }
    else
    {
        temp_CurErr1 = 0;
        temp_CurErr2 = 0;
        temp_CurErr3 = 0;
        g_protector.losAphTime = 0;
        g_protector.losBphTime = 0;
        g_protector.losCphTime = 0;
    }
}

/*!
* @brief Motor Lose phase protection.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void LosePhase_Check(FOC_VARS_CTRL *focVars)
{
    static uint16_t outputPhaseCnt = 0, ITotalCnt = 0;
    static uint32_t IuTotal = 0, IvTotal = 0, IwTotal = 0;

    if (focVars->pwmOutMask == SVPWM_OUTPUT_ENABLE)
    {
        g_protector.IuAvg = (uint16_t)Math_Qabs(focVars->iaPu);
        g_protector.IvAvg = (uint16_t)Math_Qabs(focVars->ibPu);
        g_protector.IwAvg = (uint16_t)Math_Qabs(focVars->icPu);
        ITotalCnt++;
        if (ITotalCnt <= (1 << LOSE_PHASE_ADD_CNT))
        {
            IuTotal += g_protector.IuAvg;
            IvTotal += g_protector.IvAvg;
            IwTotal += g_protector.IwAvg;
        }
        else
        {
            g_protector.IuAvg = (uint16_t)(IuTotal >> LOSE_PHASE_ADD_CNT);
            g_protector.IvAvg = (uint16_t)(IvTotal >> LOSE_PHASE_ADD_CNT);
            g_protector.IwAvg = (uint16_t)(IwTotal >> LOSE_PHASE_ADD_CNT);
            ITotalCnt = 0;

            if (g_protector.IuAvg > g_protector.IvAvg)
            {
                g_protector.Imax = g_protector.IuAvg;
                g_protector.Imin = g_protector.IvAvg;
                if (g_protector.IvAvg > g_protector.IwAvg)
                {
                    g_protector.Imin = g_protector.IwAvg;
                    g_protector.losePhase = 3;
                }
                else if (g_protector.IwAvg > g_protector.IuAvg)
                {
                    g_protector.Imax = g_protector.IwAvg;
                    g_protector.losePhase = 2;
                }
                else
                {
                    g_protector.losePhase = 2;
                }
            }
            else
            {
                g_protector.Imax = g_protector.IvAvg;
                g_protector.Imin = g_protector.IuAvg;
                if (g_protector.IuAvg > g_protector.IwAvg)
                {
                    g_protector.Imin = g_protector.IwAvg;
                    g_protector.losePhase = 3;
                }
                else if (g_protector.IwAvg > g_protector.IvAvg)
                {
                    g_protector.Imax = g_protector.IwAvg;
                    g_protector.losePhase = 1;
                }
                else
                {
                    g_protector.losePhase = 1;
                }
            }

            if ((g_protector.Imin != 0) && (g_protector.Imax > LOSE_PHASE_THRESHOLD))
            {
                g_protector.losePhaseCof = (((uint32_t)g_protector.Imax) << 15) / g_protector.Imin;
            }
            else
            {
                g_protector.losePhaseCof = Math_IQ(1.0);
            }

            if (g_protector.losePhaseCof > LOSE_PHASE_JUDGE_THRESHOLD)
            {
                outputPhaseCnt ++;
                if (outputPhaseCnt > LOSE_PHASE_DBC)
                {
                    if (g_protector.losePhase == 1)
                    {
                        Fault_Report(LOSE_PHASEA_FAILURE);
                    }
                    else if (g_protector.losePhase == 2)
                    {
                        Fault_Report(LOSE_PHASEB_FAILURE);
                    }
                    else
                    {
                        Fault_Report(LOSE_PHASEC_FAILURE);
                    }
                    outputPhaseCnt = 0;
                }
            }
            else
            {
                outputPhaseCnt = 0;
            }

            IuTotal = 0;
            IvTotal = 0;
            IwTotal = 0;
        }
    }
    else
    {
        IuTotal = 0;
        IvTotal = 0;
        IwTotal = 0;

        ITotalCnt = 0;
        outputPhaseCnt = 0;
    }
}

/*!
* @brief Hall sensor sector status protection.
*
* @param[in] hallstatus: Current hall sector state
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
uint16_t Hall_Sensor_Status_Check(uint16_t hallstatus)
{
    uint16_t status = DIAGNOSTIC_OK;
//    static uint16_t cntErr1 = 0;

    if (hallstatus == 0x00 || hallstatus == 0x07)
    {
//    if (cntErr1 < 2)
//    {
//        cntErr1++;
//    }
//    else
//    {
        status = DIAGNOSTIC_FAIL;
        Fault_Report(HALL_STATUS_FAILURE);
//    }
    }
//    else
//    {
//        cntErr1 = 0;
//    }

    return status;

}

/*!
* @brief Hall Sensor protection.
*
* @param[in] hallnew: Current hall sector state
* @param[in] hallold: Previous hall sector state
* @param[in] protect: pointer to PROTECTOR structure
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
uint16_t Hall_Sensor_Change_Check(uint16_t hallnew, uint16_t hallold, PROTECTOR *protect)
{
    uint16_t status = DIAGNOSTIC_OK;
    uint16_t bitsChange = 0;
    uint16_t temp = 0;
    static uint16_t cntErr2 = 0;
    static uint16_t cntClr = 0;

    bitsChange = (hallnew ^ hallold) & 0x07;
    temp = (uint16_t)((bitsChange & 0x01) + ((bitsChange >> 1) & 0x01) + ((bitsChange >> 2) &0x01));

    if (temp > 1)
    {
        cntClr = 0;
        if (cntErr2 < 1)
        {
            cntErr2++;
        }
        else
        {
            status = DIAGNOSTIC_FAIL;
            Fault_Report(HALL_STATUS_CHANGE_FAILURE);
        }
    }
    else
    {
        if (cntClr < 10)
        {
            cntClr++;
        }
        else
        {
            cntErr2 = 0;
        }
    }

    if (hallnew != hallold)
    {
        if (hallold == 5)
        {
            if ((hallnew != 1) && (hallnew != 4))
            {
                status = DIAGNOSTIC_FAIL;
                Fault_Report(HALL_STATUS_CHANGE_FAILURE);

            }
            else
            {
                /* NULL */
            }
        }
        else if (hallold == 1)
        {
            if ((hallnew != 5) && (hallnew != 3))
            {
                status = DIAGNOSTIC_FAIL;
                Fault_Report(HALL_STATUS_CHANGE_FAILURE);

            }
            else
            {
                /* NULL */
            }

        }
        else if (hallold == 3)
        {
            if ((hallnew != 1) && (hallnew != 2))
            {
                status = DIAGNOSTIC_FAIL;
                Fault_Report(HALL_STATUS_CHANGE_FAILURE);

            }
            else
            {
                /* NULL */
            }

        }
        else if (hallold == 2)
        {
            if ((hallnew != 3) && (hallnew != 6))
            {
                status = DIAGNOSTIC_FAIL;
                Fault_Report(HALL_STATUS_CHANGE_FAILURE);

            }
            else
            {
                /* NULL */
            }

        }

        else if (hallold == 6U)
        {
            if ((hallnew != 2) && (hallnew != 4))
            {
                status = DIAGNOSTIC_FAIL;
                Fault_Report(HALL_STATUS_CHANGE_FAILURE);

            }
            else
            {
                /* NULL */
            }

        }
        else if (hallold == 4)
        {
            if ((hallnew != 6) && (hallnew != 5))
            {
                status = DIAGNOSTIC_FAIL;
                Fault_Report(HALL_STATUS_CHANGE_FAILURE);

            }
            else
            {
                /* NULL */
            }

        }
        else
        {
            /* NULL */
        }

    }

    return status;

}

/*!
* @brief Phase current short protection.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] speed: speed feedback pu value
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
int16_t Phase_Short_Dynamic_Check(FOC_VARS_CTRL *focVars, int16_t speed)
{
    static int16_t s_cntA = 0, s_cntB = 0, s_cntC = 0;
    int16_t status = DIAGNOSTIC_OK;
    uint16_t statusTemp = 0;
    int16_t ia = 0, ib = 0, ic = 0;

    ia = focVars->iaPu;
    ib = focVars->ibPu;
    ic = focVars->icPu;

    if (speed > SPEED_THRESHOLD_SHORT)
    {
        if ((ia > ib - JUDGE_THRESHOLD_SHORT) && (ia < ib + JUDGE_THRESHOLD_SHORT) )
        {
            statusTemp |= 0x1;
            if (s_cntA < IPHASE_SHORT_DBC)
            {
                s_cntA++;
            }
            else
            {
                status = DIAGNOSTIC_FAIL;
                Fault_Report(PHASE_CURRENT_SHORT_FAILURE);
            }
        }
        else
        {
            s_cntA = 0;
        }

        if ((ia > ic - JUDGE_THRESHOLD_SHORT) && (ia < ic + JUDGE_THRESHOLD_SHORT) )
        {
            statusTemp |= 0x02;
            if (s_cntB < IPHASE_SHORT_DBC)
            {
                s_cntB++;
            }
            else
            {
                status = DIAGNOSTIC_FAIL;
                Fault_Report(PHASE_CURRENT_SHORT_FAILURE);
            }
        }
        else
        {
            s_cntB = 0;
        }

        if ((ib > ic - JUDGE_THRESHOLD_SHORT) && (ib < ic + JUDGE_THRESHOLD_SHORT) )
        {
            statusTemp |= 0x04;
            if (s_cntC < IPHASE_SHORT_DBC)
            {
                s_cntC++;
            }
            else
            {
                status = DIAGNOSTIC_FAIL;
                Fault_Report(PHASE_CURRENT_SHORT_FAILURE);
            }
        }
        else
        {
            s_cntC = 0;
        }
    }
    else
    {

    }
    if (statusTemp != 0)
    {
        status = DIAGNOSTIC_FAIL;
    }

    return status;
}

/*!
* @brief Lower bridge MOSFET short circuit fault protection.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] dutycycle: pointer to PWM_GEN_OUT structure
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
int16_t Mosfet_LowerBridge_Short_check(FOC_VARS_CTRL *focVars, PWM_GEN_OUT *dutycycle)
{
    static int16_t shortCheckDuty = 0;
    int16_t status = DIAGNOSTIC_OK;

    for (shortCheckDuty = 0; shortCheckDuty < SHORT_THRESHOLD_DUTY; shortCheckDuty++)
    {
        dutycycle->pwmTa = shortCheckDuty;
        dutycycle->pwmTb = shortCheckDuty;
        dutycycle->pwmTc = shortCheckDuty;
        if (Math_Qabs(focVars->iaPu) > SHORT_THRESHOLD_CUR)
        {
            PWM_Output(PWM_OUTPUT_DISABLE);
            Fault_Report(MOSFET_LOWERBRIDGEA_SHORT_FAILURE);
            status = DIAGNOSTIC_FAIL;
        }
        else if (Math_Qabs(focVars->ibPu) > SHORT_THRESHOLD_CUR)
        {
            PWM_Output(PWM_OUTPUT_DISABLE);
            Fault_Report(MOSFET_LOWERBRIDGEB_SHORT_FAILURE);
            status = DIAGNOSTIC_FAIL;
        }
        else if (Math_Qabs(focVars->icPu) > SHORT_THRESHOLD_CUR)
        {
            PWM_Output(PWM_OUTPUT_DISABLE);
            Fault_Report(MOSFET_LOWERBRIDGEC_SHORT_FAILURE);
            status = DIAGNOSTIC_FAIL;
        }
        else
        {
            status = DIAGNOSTIC_OK;
        }

        if (g_protector.ErrFlag.all)
        {
            PWM_Output(PWM_OUTPUT_DISABLE);
        }
        else
        {
            PWM_Enable();
            PWM_Update(dutycycle->pwmTa, dutycycle->pwmTb, dutycycle->pwmTc);
        }
    }

    return status;
}

/*!
* @brief Upper bridge MOSFET short circuit fault protection.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] dutycycle: pointer to PWM_GEN_OUT structure
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
int16_t Mosfet_UpperBridge_Short_check(FOC_VARS_CTRL *focVars, PWM_GEN_OUT *dutycycle)
{
    static int16_t shortCheckDuty = 0;
    int16_t status = DIAGNOSTIC_OK;

    for (shortCheckDuty = PWM_PERIOD_VALUE; shortCheckDuty > (int16_t)(PWM_PERIOD_VALUE - SHORT_THRESHOLD_DUTY); shortCheckDuty--)
    {
        dutycycle->pwmTa = shortCheckDuty;
        dutycycle->pwmTb = shortCheckDuty;
        dutycycle->pwmTc = shortCheckDuty;
        if (Math_Qabs(focVars->iaPu) > SHORT_THRESHOLD_CUR)
        {
            PWM_Output(PWM_OUTPUT_DISABLE);
            Fault_Report(MOSFET_UPPERBRIDGEA_SHORT_FAILURE);
            status = DIAGNOSTIC_FAIL;
        }
        else if (Math_Qabs(focVars->ibPu) > SHORT_THRESHOLD_CUR)
        {
            PWM_Output(PWM_OUTPUT_DISABLE);
            Fault_Report(MOSFET_UPPERBRIDGEB_SHORT_FAILURE);
            status = DIAGNOSTIC_FAIL;
        }
        else if (Math_Qabs(focVars->icPu) > SHORT_THRESHOLD_CUR)
        {
            PWM_Output(PWM_OUTPUT_DISABLE);
            Fault_Report(MOSFET_UPPERBRIDGEC_SHORT_FAILURE);
            status = DIAGNOSTIC_FAIL;
        }
        else
        {
            status = DIAGNOSTIC_OK;
        }

        if (g_protector.ErrFlag.all)
        {
            PWM_Output(PWM_OUTPUT_DISABLE);
        }
        else
        {
            PWM_Output(PWM_OUTPUT_ENABLE);
            PWM_Update(dutycycle->pwmTa, dutycycle->pwmTb, dutycycle->pwmTc);
        }
    }

    return status;
}

/*!
* @brief Lower bridge MOSFET open circuit fault protection.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] dutycycle: pointer to PWM_GEN_OUT structure
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
int16_t Mosfet_LowerBridge_Open_check(FOC_VARS_CTRL *focVars, PWM_GEN_OUT *dutycycle)
{
    static int16_t openCheckDuty = 0;
    static int16_t phase = 1;
    static uint16_t curAOld = 0, curBOld = 0, curCOld = 0;
    int16_t status = DIAGNOSTIC_OK;

    for (phase = 1; phase < 4; phase++)
    {
        for (openCheckDuty = 0; openCheckDuty < OPEN_THRESHOLD_DUTY; openCheckDuty += DUTY_STEP)
        {
            switch (phase)
            {
            case 1:
            dutycycle->pwmTa= 0;
            dutycycle->pwmTb = openCheckDuty;
            dutycycle->pwmTc = openCheckDuty;
            if (Math_Qabs(focVars->iaPu) > curAOld)
            {
                curAOld = Math_Qabs(focVars->iaPu);
            }
            if (Math_Qabs(focVars->iaPu) > OPEN_THRESHOLD_CUR)
            {
                openCheckDuty = OPEN_THRESHOLD_DUTY;
                status = DIAGNOSTIC_OK;
            }
            else if (openCheckDuty > (OPEN_THRESHOLD_DUTY >> 1))
            {
                if (Math_Qabs(focVars->iaPu) < LOSE_PHASE_THRESHOLD)
                {
                    PWM_Output(PWM_OUTPUT_DISABLE);
                    Fault_Report(LOSE_PHASEA_FAILURE);
                }
            }
            else if (openCheckDuty >= (OPEN_THRESHOLD_DUTY - DUTY_STEP))
            {
                if (curAOld < OPEN_THRESHOLD_CUR)
                {
                    status = DIAGNOSTIC_FAIL;
                    openCheckDuty = OPEN_THRESHOLD_DUTY;
                    phase = 4;
                    Fault_Report(MOSFET_LOWERBRIDGEA_OPEN_FAILURE);
                }
            }
            else
            {
            }
            break;

            case 2:
            dutycycle->pwmTa = openCheckDuty;
            dutycycle->pwmTb = 0;
            dutycycle->pwmTc = openCheckDuty;
            if (Math_Qabs(focVars->ibPu) > curBOld)
            {
                curBOld = Math_Qabs(focVars->ibPu);
            }
            if (Math_Qabs(focVars->ibPu) > OPEN_THRESHOLD_CUR)
            {
                openCheckDuty = OPEN_THRESHOLD_DUTY;
                status = DIAGNOSTIC_OK;
            }
            else if (openCheckDuty > (OPEN_THRESHOLD_DUTY >> 1))
            {
                if (Math_Qabs(focVars->ibPu) < LOSE_PHASE_THRESHOLD)
                {
                    PWM_Output(PWM_OUTPUT_DISABLE);
                    Fault_Report(LOSE_PHASEB_FAILURE);
                }
            }
            else if (openCheckDuty >= (OPEN_THRESHOLD_DUTY - DUTY_STEP))
            {
                if (curBOld < OPEN_THRESHOLD_CUR)
                {
                    status = DIAGNOSTIC_FAIL;
                    openCheckDuty = OPEN_THRESHOLD_DUTY;
                    phase = 4;
                    Fault_Report(MOSFET_LOWERBRIDGEB_OPEN_FAILURE);
                }
            }
            else
            {
            }
            break;

            case 3:
            dutycycle->pwmTa = openCheckDuty;
            dutycycle->pwmTb = openCheckDuty;
            dutycycle->pwmTc = 0;
            if (Math_Qabs(focVars->ibPu) > curBOld)
            {
                curBOld = Math_Qabs(focVars->ibPu);
            }
            if (Math_Qabs(focVars->icPu) > OPEN_THRESHOLD_CUR)
            {
                openCheckDuty = OPEN_THRESHOLD_DUTY;
                status = DIAGNOSTIC_OK;
            }
            else if (openCheckDuty > (OPEN_THRESHOLD_DUTY >> 1))
            {
                if (Math_Qabs(focVars->icPu) < LOSE_PHASE_THRESHOLD)
                {
                    PWM_Output(PWM_OUTPUT_DISABLE);
                    Fault_Report(LOSE_PHASEC_FAILURE);
                }
            }
            else if (openCheckDuty >= (OPEN_THRESHOLD_DUTY - DUTY_STEP))
            {
                if (curCOld < OPEN_THRESHOLD_CUR)
                {
                    status = DIAGNOSTIC_FAIL;
                    openCheckDuty = OPEN_THRESHOLD_DUTY;
                    phase = 4;
                    Fault_Report(MOSFET_LOWERBRIDGEC_OPEN_FAILURE);
                }
            }
            else
            {
            }
            break;

            default:
            break;
            }

            if (g_protector.ErrFlag.all)
            {
                PWM_Output(PWM_OUTPUT_DISABLE);
            }
            else
            {
                PWM_Output(PWM_OUTPUT_ENABLE);
                PWM_Update(dutycycle->pwmTa, dutycycle->pwmTb, dutycycle->pwmTc);
            }
        }
    }

    if (status == DIAGNOSTIC_FAIL)
    {
        PWM_Output(PWM_OUTPUT_DISABLE);
        //Fault_Report(MOSFET_LOWERBRIDGE_OPEN_FAILURE);
        phase = 1;
        openCheckDuty = 0;
    }

    return status;
}

/*!
* @brief Upper bridge MOSFET open circuit fault protection.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] dutycycle: pointer to PWM_GEN_OUT structure
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
int16_t Mosfet_UpperBridge_Open_check(FOC_VARS_CTRL *focVars, PWM_GEN_OUT *dutycycle)
{
    static int16_t openCheckDuty = 0;
    static int16_t phase = 1;
    int16_t status = DIAGNOSTIC_OK;

    for (phase = 1; phase < 4; phase++)
    {
        for (openCheckDuty = 0; openCheckDuty < OPEN_THRESHOLD_DUTY; openCheckDuty += DUTY_STEP)
        {
            switch (phase)
            {
            case 1:
            dutycycle->pwmTa = 0;
            dutycycle->pwmTb = 0;
            dutycycle->pwmTc = openCheckDuty;
            if ((Math_Qabs(focVars->iaPu) > OPEN_THRESHOLD_CUR) || (Math_Qabs(focVars->ibPu) > OPEN_THRESHOLD_CUR))
            {
                openCheckDuty = OPEN_THRESHOLD_DUTY;
                status = DIAGNOSTIC_OK;
            }
            else if (openCheckDuty >= (OPEN_THRESHOLD_DUTY - DUTY_STEP))
            {
                status = DIAGNOSTIC_FAIL;
                Fault_Report(MOSFET_UPPERBRIDGEA_OPEN_FAILURE);
            }
            else
            {
                /* do nothing */
            }
            break;

            case 2:
            dutycycle->pwmTa = openCheckDuty;
            dutycycle->pwmTb = 0;
            dutycycle->pwmTc = 0;
            if ((Math_Qabs(focVars->ibPu) > OPEN_THRESHOLD_CUR) || (Math_Qabs(focVars->icPu) > OPEN_THRESHOLD_CUR))
            {
                openCheckDuty = OPEN_THRESHOLD_DUTY;
                status = DIAGNOSTIC_OK;
            }
            else if (openCheckDuty >= (OPEN_THRESHOLD_DUTY - DUTY_STEP))
            {
                status = DIAGNOSTIC_FAIL;
                Fault_Report(MOSFET_UPPERBRIDGEB_OPEN_FAILURE);
            }
            else
            {
                //do nothing
            }
            break;

            case 3:
            dutycycle->pwmTa = 0;
            dutycycle->pwmTb = openCheckDuty;
            dutycycle->pwmTc = 0;
            if ((Math_Qabs(focVars->iaPu) > OPEN_THRESHOLD_CUR) || (Math_Qabs(focVars->icPu) > OPEN_THRESHOLD_CUR))
            {
                openCheckDuty = OPEN_THRESHOLD_DUTY;
                status = DIAGNOSTIC_OK;
            }
            else if (openCheckDuty >= (OPEN_THRESHOLD_DUTY - DUTY_STEP))
            {
                status = DIAGNOSTIC_FAIL;
                Fault_Report(MOSFET_UPPERBRIDGEC_OPEN_FAILURE);
            }
            else
            {
                /* do nothing */
            }
            break;

            default:
            break;

            }

            if (g_protector.ErrFlag.all)
            {
                PWM_Output(PWM_OUTPUT_DISABLE);
            }
            else
            {
                PWM_Output(PWM_OUTPUT_ENABLE);
                PWM_Update(dutycycle->pwmTa, dutycycle->pwmTb, dutycycle->pwmTc);
            }
        }
    }

    if (status == DIAGNOSTIC_FAIL)
    {
        PWM_Output(PWM_OUTPUT_DISABLE);
        //Fault_Report(MOSFET_LOWERBRIDGE_OPEN_FAILURE);
        phase = 1;
        openCheckDuty = 0;
    }

    return status;
}

/*!
* @brief Inverter MOSFET bridge short circuit fault protection.
*
* @param[in] none
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
int16_t Mosfet_Bridge_Short_check(void)
{
    int16_t status = DIAGNOSTIC_OK;

#if SEPARATION_TYPE == SEPARATION_CHIP
    int16_t temp = 0;

    temp = Mosfet_LowerBridge_Short_check(&g_focVarsCtrl , &g_pwmValue);
    status |= temp;
    temp = Mosfet_UpperBridge_Short_check(&g_focVarsCtrl , &g_pwmValue);
    status |= temp;
#endif

#if SEPARATION_TYPE == PREDRIVE_SEPARATION
    int16_t statusTemp = 0;
    int16_t va = 0, vb = 0, vc = 0;

    va = ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    vb = ADC_GetInjectData(ADC0, ADC_ISEQ_1);
    vc = ADC_GetInjectData(ADC0, ADC_ISEQ_2);

    /* 4.5 / 5 * 4096 */
    if (va > 3686)
    {
        statusTemp |= 0x1;
        Fault_Report(MOSFET_UPPERBRIDGEA_SHORT_FAILURE);
    }
    /* 3 / 5 * 4096 */
    else if (va < 2457)
    {
        statusTemp |= 0x2;
        Fault_Report(MOSFET_LOWERBRIDGEA_SHORT_FAILURE);
    }
    else
    {
    }

    /* 4.5 / 5 * 4096 */
    if (vb > 3686)
    {
        statusTemp |= 0x4;
        Fault_Report(MOSFET_UPPERBRIDGEA_SHORT_FAILURE);
    }
    /* 3 / 5 * 4096 */
    else if (vb < 2457)
    {
        statusTemp |= 0x8;
        Fault_Report(MOSFET_LOWERBRIDGEA_SHORT_FAILURE);
    }
    else
    {
    }

    /* 4.5 / 5 * 4096 */
    if (vc > 3686)
    {
        statusTemp |= 0x10;
        Fault_Report(MOSFET_UPPERBRIDGEC_SHORT_FAILURE);
    }
    /* 3 / 5 * 4096 */
    else if (vc < 2457)
    {
        statusTemp |= 0x20;
        Fault_Report(MOSFET_LOWERBRIDGEC_SHORT_FAILURE);
    }
    else
    {
    }

    if (statusTemp != 0)
    {
        status = DIAGNOSTIC_FAIL;
    }

#endif

    return status;
}

/*!
* @brief Inverter MOSFET bridge open circuit fault protection.
*
* @param[in] none
* @return status: fault diagnosis status, 0 OK; 1 fault; 2 unknown
*/
int16_t Mosfet_Bridge_Open_check(void)
{
    int16_t status = DIAGNOSTIC_OK;

#if SEPARATION_TYPE == SEPARATION_CHIP
    int16_t temp = 0;

    temp = Mosfet_LowerBridge_Open_check(&g_focVarsCtrl, &g_pwmValue);
    status |= temp;
    temp = Mosfet_UpperBridge_Open_check(&g_focVarsCtrl, &g_pwmValue);
    status |= temp;
#endif

#if SEPARATION_TYPE == PREDRIVE_SEPARATION
    int16_t statusTemp = 0;
    int16_t va = 0, vb = 0, vc = 0;

    PWM_Update(PWM_PERIOD_VALUE, PWM_PERIOD_VALUE, PWM_PERIOD_VALUE);
    PWM_Output(PWM_OUTPUT_ENABLE);
    mdelay(10);

    va = ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    vb = ADC_GetInjectData(ADC0, ADC_ISEQ_1);
    vc = ADC_GetInjectData(ADC0, ADC_ISEQ_2);

    /* 4.5 / 5 * 4096 */
    if (va < 3686)
    {
        statusTemp |= 0x1;
        Fault_Report(MOSFET_UPPERBRIDGEA_OPEN_FAILURE);
    }
    else
    {
    }
    /* 4.5 / 5 * 4096 */
    if (vb < 3686)
    {
        statusTemp |= 0x2;
        Fault_Report(MOSFET_UPPERBRIDGEB_OPEN_FAILURE);
    }
    else
    {
    }

    /* 4.5 / 5 * 4096 */
    if (vc < 3686)
    {
        statusTemp |= 0x4;
        Fault_Report(MOSFET_UPPERBRIDGEC_OPEN_FAILURE);
    }
    else
    {
    }

    PWM_Output(PWM_OUTPUT_DISABLE);

    PWM_Update(0, 0, 0);
    PWM_Output(PWM_OUTPUT_ENABLE);
    mdelay(10);

    va = ADC_GetInjectData(ADC0, ADC_ISEQ_0);
    vb = ADC_GetInjectData(ADC0, ADC_ISEQ_1);
    vc = ADC_GetInjectData(ADC0, ADC_ISEQ_2);

    /* 3 / 5 * 4096 */
    if (va > 2457)
    {
        statusTemp |= 0x8;
        Fault_Report(MOSFET_LOWERBRIDGEA_OPEN_FAILURE);
    }
    else
    {
    }
    /* 3 / 5 * 4096 */
    if (vb > 2457)
    {
        statusTemp |= 0x10;
        Fault_Report(MOSFET_LOWERBRIDGEB_OPEN_FAILURE);
    }
    else
    {
    }

    /* 3 / 5 * 4096 */
    if (vc > 2457)
    {
        statusTemp |= 0x20;
        Fault_Report(MOSFET_LOWERBRIDGEC_OPEN_FAILURE);
    }
    else
    {
    }

    PWM_Output(PWM_OUTPUT_DISABLE);

    if (statusTemp != 0)
    {
        status = DIAGNOSTIC_FAIL;
    }

#endif

    return status;
}

/*!
* @brief Print output Motor protect Fault Idendification.
*
* @param[in] none
* @return none
*/
void Printf_Protect_Failure(void)
{
    uint16_t s_failureNumber = 0;

    if (g_protector.ErrFlag.all)
    {
        for (s_failureNumber = 0; s_failureNumber<=30; s_failureNumber++)
        {
            if (((g_protector.ErrFlag.all >> s_failureNumber) & 0x1 ) == 1)
            {
                switch (s_failureNumber)
                {
                case 0:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("OVER_CURRENT_PHASE_FAILURE\r\n"));
                break;

                case 1:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("OVER_CURRENT_IBUS_FAILURE \r\n"));
                break;

                case 2:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("OVER_CURRENT_IS_FAILURE \r\n"));
                break;

                case 3 :
//                DEBUGMSG(DEBUG_ZONE_INFO, ("VBUS_OVER_VOLTAGE_FAILURE \r\n"));
                break;

                case 4:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("VBUS_UNDER_VOLTAGE_FAILURE \r\n"));
                break;

                case 5:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("LOSE_SPEED_FAILURE \r\n"));
                break;

                case 6:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("ZERO_SPEED_FAILURE \r\n"));
                break;

                case 7:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("STARTUP_FAILURE \r\n"));
                break;

                case 8:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOTOR_STALL_FAILURE \r\n"));
                break;

                case 9 :
//                DEBUGMSG(DEBUG_ZONE_INFO, ("PHASEA_CURRENT_MIDPOINT_FAILURE \r\n"));
                break;

                case 10:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("PHASEB_CURRENT_MIDPOINT_FAILURE \r\n"));
                break;

                case 11:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("PHASEC_CURRENT_MIDPOINT_FAILURE \r\n"));
                break;

                case 12:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("LOSE_PHASEA_FAILURE \r\n"));
                break;

                case 13:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("LOSE_PHASEB_FAILURE \r\n"));
                break;

                case 14:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("LOSE_PHASEC_FAILURE \r\n"));
                break;

                case 15:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("HALL_STATUS_FAILURE \r\n"));
                break;

                case 16:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("HALL_STATUS_CHANGE_FAILURE \r\n"));
                break;

                case 17:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("PHASE_CURRENT_SHORT_FAILURE \r\n"));
                break;

                case 18:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOSFET_LOWERBRIDGEA_SHORT_FAILURE \r\n"));
                break;

                case 19 :
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOSFET_LOWERBRIDGEB_SHORT_FAILURE \r\n"));
                break;

                case 20:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOSFET_LOWERBRIDGEC_SHORT_FAILURE \r\n"));
                break;

                case 21:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOSFET_UPPERBRIDGEA_SHORT_FAILURE \r\n"));
                break;

                case 22:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOSFET_UPPERBRIDGEB_SHORT_FAILURE \r\n"));
                break;

                case 23:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOSFET_UPPERBRIDGEC_SHORT_FAILURE \r\n"));
                break;

                case 24:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOSFET_LOWERBRIDGEA_OPEN_FAILURE \r\n"));
                break;

                case 25:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOSFET_LOWERBRIDGEB_OPEN_FAILURE \r\n"));
                break;

                case 26:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOSFET_LOWERBRIDGEC_OPEN_FAILURE \r\n"));
                break;

                case 27:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOSFET_UPPERBRIDGEA_OPEN_FAILURE \r\n"));
                break;

                case 28:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOSFET_UPPERBRIDGEB_OPEN_FAILURE \r\n"));
                break;

                case 29 :
//                DEBUGMSG(DEBUG_ZONE_INFO, ("MOSFET_UPPERBRIDGEC_OPEN_FAILURE \r\n"));
                break;

                case 30:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("HALL_SELF_LEARN_FAILURE \r\n"));
                break;

                case 31:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("FAILURE \r\n"));
                break;

                case 32:
//                DEBUGMSG(DEBUG_ZONE_INFO, ("FAILURE \r\n"));
                break;

                default:
                break;
                }
            }
        }
    }
}

/*!
* @brief Hall sensor galloping protection. Prevent motor speed uncontrolled increasing.
*
* @param[in] none
* @return none
*/
void HallFlyingCarProtect(void)
{
    static uint8_t g_hallSectorBuf = 0;
    static uint8_t g_hallErrCnt = 0;

    if (g_hall.pHallCfg->hallSelfLearnEnable)
    {
        return ;
    }
    if (pFocVarsCfg->motorDir == 1)
    {
        if (g_hallSensor.hallEstThetaSector == 1)
        {
            if ((g_hallSectorBuf == 6) && (g_hallErrCnt != 0))
            {
                g_hallErrCnt--;
            }
            else
            {
                g_hallErrCnt++;
            }
        }
        else
        {
            if (((g_hallSensor.hallEstThetaSector - g_hallSectorBuf) == 1) && (g_hallErrCnt != 0))
            {
                g_hallErrCnt--;
            }
            else
            {
                g_hallErrCnt++;
            }
        }
    }
    else if (pFocVarsCfg->motorDir == -1)
    {
        if (g_hallSensor.hallEstThetaSector == 6)
        {
            if ((g_hallSectorBuf == 1) && (g_hallErrCnt != 0))
            {
                g_hallErrCnt--;
            }
            else
            {
                g_hallErrCnt++;
            }
        }
        else
        {
            if (((g_hallSectorBuf - g_hallSensor.hallEstThetaSector) == 1) && (g_hallErrCnt != 0))
            {
                g_hallErrCnt--;
            }
            else
            {
                g_hallErrCnt++;
            }
        }
    }
    else
    {
    }
    g_hallSectorBuf = g_hallSensor.hallEstThetaSector;
    if (g_hallErrCnt > 12)
    {
        g_hallErrCnt = 0;
        PushIn_McBufStatus(FAULT);
        PWM_Disable();
        Fault_Report(HALL_STATUS_CHANGE_FAILURE);
    }
}

/*!
* @brief Hall sensor self learn check.
*
* @param[in] none
* @return none
*/
void HallSelfLearnFailCheck(void)
{
    if (g_hallSelfLearn.selfLearnIndex == 22)
    {
        Fault_Report(HALL_SELF_LEARN_FAILURE);
    }
}

/*!
* @brief Power reduced protection parameters initialization.
*
* @param[in] powerDecrease: pointer to POWER_DECREASE structure
* @return none
*/
void PowerDecrease_init(POWER_DECREASE *powerDecrease)
{
    powerDecrease->spdIn = 0;
    powerDecrease->spdStart = SPD_START;
    powerDecrease->spdStop = SPD_STOP;
    powerDecrease->spdGain = Math_Div(32768, SPD_STOP - SPD_START);
    powerDecrease->spdCoeff = 32768;
    powerDecrease->spdFlag = (uint8_t)FALSE;

    powerDecrease->mosTempIn = 0;
    powerDecrease->mosTempStart = MOS_TEMP_START;
    powerDecrease->mosTempStop = MOS_TEMP_STOP;
    powerDecrease->mosTempGain = Math_Div(32768, MOS_TEMP_STOP - MOS_TEMP_START);
    powerDecrease->mosTempCoeff = 32768;
    powerDecrease->mosTempFlag = (uint8_t)FALSE;

    powerDecrease->ibusIn = 0;
    powerDecrease->ibus = 0;
    powerDecrease->ibusStart = IBUS_START;
    powerDecrease->ibusStop = IBUS_STOP;
    powerDecrease->ibusGain = Math_Div(32768, IBUS_STOP - IBUS_START);
    powerDecrease->ibusCoeff = 32768;
    powerDecrease->ibusFlag = (uint8_t)FALSE;

    powerDecrease->vbusHighIn = 0;
    powerDecrease->vbusHighStart = VBUS_HIGH_START;
    powerDecrease->vbusHighStop = VBUS_HIGH_STOP;
    powerDecrease->vbusHighGain = Math_Div(32768, VBUS_HIGH_STOP - VBUS_HIGH_START);
    powerDecrease->vbusHighCoeff = 32768;
    powerDecrease->vbusHighFlag = (uint8_t)FALSE;

    powerDecrease->vbusLowIn = 0;
    powerDecrease->vbusLowStart = VBUS_LOW_START;
    powerDecrease->vbusLowStop = VBUS_LOW_STOP;
    powerDecrease->vbusLowGain = Math_Div(32768, VBUS_LOW_START - VBUS_LOW_STOP);
    powerDecrease->vbusLowCoeff = 32768;
    powerDecrease->vbusLowFlag = (uint8_t)FALSE;

    powerDecrease->depFlag = (uint8_t)FALSE;
    powerDecrease->depDoneFlag = 0;
    powerDecrease->depCoeff = 32768;
    powerDecrease->speedBuf = 0;
}

/*!
* @brief Power reduced protection coefficient calculation.
*
* @param[in] input: power reduced input parameter
* @param[in] start: power reduced start value
* @param[in] stop: power reduced stop value
* @param[in] gain: power reduced gain
* @param[in] dep_type: DEP_TYPE enumeration, 0:over limit type power decrease, 1:low limit type power decrease.
* @return dep_Coeff, power reduced coefficient
*/
uint16_t PowerDecrease_Coeff_Calc(uint16_t input, uint16_t start, uint16_t stop, uint32_t gain, DEP_TYPE dep_type)
{
    uint16_t dep_Coeff = 32768;

    if(dep_type == DEPOWER_H)
    {
        if (input < start)
        {
            dep_Coeff = 32768;
        }
        else if (input < stop)
        {
            dep_Coeff = Math_Mpy(gain, stop - input);
        }
        else
        {
            dep_Coeff = 0;
        }
    }
    else
    {
        if (input > start)
        {
            dep_Coeff = 32768;
        }
        else if (input > stop)
        {
            dep_Coeff = Math_Mpy(gain, input - stop);
        }
        else
        {
            dep_Coeff = 0;
        }
    }
    dep_Coeff = Math_Sat(dep_Coeff, 32768, 0);

    return dep_Coeff;
}

/*!
* @brief Power reduced protection coefficient compare function. According to
*        the motor rotation speed and the temperature of MOSFET, the min value
*        is compared for protection.
*
* @param[in] powerDecrease: pointer to POWER_DECREASE structure
* @return none
*/
void PowerDecrease_MinCoeff_Cmp(POWER_DECREASE *powerDecrease)
{
    uint16_t i = 0;
    int32_t  delt_Coeff = 32768;
    uint16_t array_Coeff[DEPOWER_NUM] = {32768, 32768, 32768, 32768, 32768};
    uint16_t min_Coeff = 32768;

    array_Coeff[0] = powerDecrease->spdCoeff;
    array_Coeff[1] = powerDecrease->mosTempCoeff;
    array_Coeff[2] = powerDecrease->ibusCoeff;
    array_Coeff[3] = powerDecrease->vbusHighCoeff;
    array_Coeff[4] = powerDecrease->vbusLowCoeff;

    for (i = 0; i < DEPOWER_NUM; i++)
    {
        if( array_Coeff[i] < min_Coeff)
        {
            min_Coeff = array_Coeff[i];
        }
    }
    if (min_Coeff < 32768)
    {
        powerDecrease->depFlag = (uint8_t)TRUE;
    }
    else
    {
        powerDecrease->depFlag = (uint8_t)FALSE;
    }

    delt_Coeff = min_Coeff - powerDecrease->depCoeff;
    if (delt_Coeff > 0)
    {
        powerDecrease->depCoeff = Math_Mpy(Math_IQ(1) - POWER_DECREASE_FILTER_COEFF, min_Coeff) + Math_Mpy(POWER_DECREASE_FILTER_COEFF, powerDecrease->depCoeff);
    }
    else
    {
        powerDecrease->depCoeff = min_Coeff;
    }
    powerDecrease->depCoeff = Math_Sat(powerDecrease->depCoeff, 32768, 0);
}

/*!
* @brief Power reduced protection calculation function.
*
* @param[in] powerDecrease: pointer to POWER_DECREASE structure
* @return none
*/
void PowerDecrease_Calc(POWER_DECREASE *powerDecrease)
{
    powerDecrease->spdIn = Math_Qabs(g_asrSpeedPIIn.fbkPu);
    powerDecrease->spdCoeff = PowerDecrease_Coeff_Calc(powerDecrease->spdIn, powerDecrease->spdStart, powerDecrease->spdStop, powerDecrease->spdGain, DEPOWER_H);
    if (powerDecrease->spdCoeff < 32768)
    {
        powerDecrease->spdFlag = (uint8_t)TRUE;
    }
    else
    {
        powerDecrease->spdFlag = (uint8_t)FALSE;
    }

    powerDecrease->mosTempIn = Math_IQ(0);
    powerDecrease->mosTempCoeff = PowerDecrease_Coeff_Calc(powerDecrease->mosTempIn, powerDecrease->mosTempStart, powerDecrease->mosTempStop, powerDecrease->mosTempGain, DEPOWER_H);
    if (powerDecrease->mosTempCoeff < 32768)
    {
        powerDecrease->mosTempFlag = (uint8_t)TRUE;
    }
    else
    {
        powerDecrease->mosTempFlag = (uint8_t)FALSE;
    }

    powerDecrease->ibus = g_dcCurEst.dcCur;
    powerDecrease->ibusIn = (((uint32_t)powerDecrease->ibusIn) * 63 + powerDecrease->ibus) >> 6;
    powerDecrease->ibusCoeff = PowerDecrease_Coeff_Calc(powerDecrease->ibusIn, powerDecrease->ibusStart, powerDecrease->ibusStop, powerDecrease->ibusGain, DEPOWER_H);
    if (powerDecrease->ibusCoeff < 32768)
    {
        powerDecrease->ibusFlag = (uint8_t)TRUE;
    }
    else
    {
        powerDecrease->ibusFlag = (uint8_t)FALSE;
    }

    powerDecrease->vbusHighIn = g_adcTransform.vdcPu;
    powerDecrease->vbusHighCoeff = PowerDecrease_Coeff_Calc(powerDecrease->vbusHighIn, powerDecrease->vbusHighStart, powerDecrease->vbusHighStop, powerDecrease->vbusHighGain, DEPOWER_H);
    if (powerDecrease->vbusHighCoeff < 32768)
    {
        powerDecrease->vbusHighFlag = (uint8_t)TRUE;
    }
    else
    {
        powerDecrease->vbusHighFlag = (uint8_t)FALSE;
    }

    powerDecrease->vbusLowIn = g_adcTransform.vdcPu;
    powerDecrease->vbusLowCoeff = PowerDecrease_Coeff_Calc(powerDecrease->vbusLowIn, powerDecrease->vbusLowStart, powerDecrease->vbusLowStop, powerDecrease->vbusLowGain, DEPOWER_L);
    if (powerDecrease->vbusLowCoeff < 32768)
    {
        powerDecrease->vbusLowFlag = (uint8_t)TRUE;
    }
    else
    {
        powerDecrease->vbusLowFlag = (uint8_t)FALSE;
    }

    PowerDecrease_MinCoeff_Cmp(powerDecrease);
}

/*!
* @brief DC Bus current limit protection parameters initialization.
*
* @param[in] dcCurLimit: pointer to DCCUR_LIMIT structure
* @return none
*/
void DcCurLimit_Init(DCCUR_LIMIT *dcCurLimit)
{
    dcCurLimit->dcCur = 0;
    dcCurLimit->fallCnt = 0;
    dcCurLimit->upCnt = 0;
    dcCurLimit->curLimitCoeff = Math_IQ(0.999);
}

/*!
* @brief DC Bus current limit protection parameters calculation.
*
* @param[in] dcCurLimit: pointer to DCCUR_LIMIT structure
* @return none
*/
void DcCurLimit_Calc(DCCUR_LIMIT *dcCurLimit)
{
    /* set dcCur true value,increase 100 times */
    dcCurLimit->dcCur = g_dcCurEst.dcCurTrue;
    if (dcCurLimit->dcCur > DCCUR_H)
    {
        dcCurLimit->upCnt = 0;
        dcCurLimit->fallCnt ++;
        if (dcCurLimit->fallCnt >= DCCUR_CHECK_TIME)
        {
            dcCurLimit->curLimitCoeff = Math_Mpy(Math_IQ(0.99), dcCurLimit->curLimitCoeff);
            if (dcCurLimit->curLimitCoeff < Math_IQ(0.01))
            {
                dcCurLimit->curLimitCoeff = Math_IQ(0.01);
            }
            dcCurLimit->fallCnt = DCCUR_CHECK_TIME;
        }
    }
    else if (dcCurLimit->dcCur < DCCUR_L)
    {
        dcCurLimit->fallCnt = 0;
        dcCurLimit->upCnt ++;
        if (dcCurLimit->upCnt >= DCCUR_CHECK_TIME)
        {
            dcCurLimit->curLimitCoeff = Math_Mpy(Math_IQ(1.01), dcCurLimit->curLimitCoeff);
            dcCurLimit->curLimitCoeff = Math_Sat(dcCurLimit->curLimitCoeff, Math_IQ(0.999), 0);
            dcCurLimit->upCnt = DCCUR_CHECK_TIME;
        }
    }
    else
    {
    }
}

/*!
* @brief Input power limit protection parameters initialization.
*
* @param[in] powerLimit: pointer to POWER_LIMIT structure
* @return none
*/
void PowerLimit_Init(POWER_LIMIT *powerLimit)
{
    powerLimit->power = 0;
    powerLimit->fallCnt = 0;
    powerLimit->upCnt = 0;
    powerLimit->powerLimitCoeff = Math_IQ(0.999);
}

/*!
* @brief Input power limit protection parameters calculation.
*
* @param[in] powerLimit: pointer to POWER_LIMIT structure
* @return none
*/
void PowerLimit_Calc(POWER_LIMIT *powerLimit)
{
    /* set input power true value,increase 10 times */
    powerLimit->power = (uint16_t)((uint32_t)g_adcTransform.vdcVt * g_dcCurEst.dcCurTrue * 0.01);
    if (powerLimit->power > POWER_H)
    {
        powerLimit->upCnt = 0;
        powerLimit->fallCnt ++;
        if (powerLimit->fallCnt >= POWER_CHECK_TIME)
        {
            powerLimit->powerLimitCoeff = Math_Mpy(Math_IQ(0.99), powerLimit->powerLimitCoeff);
            if (powerLimit->powerLimitCoeff < Math_IQ(0.01))
            {
                powerLimit->powerLimitCoeff = Math_IQ(0.01);
            }
            powerLimit->fallCnt = POWER_CHECK_TIME;
        }
    }
    else if (powerLimit->power < POWER_L)
    {
        powerLimit->fallCnt = 0;
        powerLimit->upCnt ++;
        if (powerLimit->upCnt >= POWER_CHECK_TIME)
        {
            powerLimit->powerLimitCoeff = Math_Mpy(Math_IQ(1.01), powerLimit->powerLimitCoeff);
            powerLimit->powerLimitCoeff = Math_Sat(powerLimit->powerLimitCoeff, Math_IQ(0.999), 0);
            powerLimit->upCnt = POWER_CHECK_TIME;
        }
    }
    else
    {
    }
}

/*!
* @brief Limit speed loop speed target command, accord to power decrease output coefficient.
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @param[in] fallSpdCof: fall speed coefficient
* @return none
*/
void ASR_SpeedCmd_Limit(SPEED_RAMP_HANDLE *command, uint16_t fallSpdCof)
{
//    command->speedTargetLimit = Math_Mpy(MAX_SPEED_VALUE, fallSpdCof);
//    command->speedTargetLimit = Math_Sat(command->speedTargetLimit, MAX_SPEED_VALUE, MIN_SPEED_VALUE);
    command->speedTargetLimit = Math_Mpy(command->speedMax, fallSpdCof);
    command->speedTargetLimit = Math_Sat(command->speedTargetLimit, command->speedMax, command->speedMin);
    command->speedTarget = Math_Sat(command->speedTarget, command->speedTargetLimit, -command->speedTargetLimit);
}

/*!
* @brief Limit ASR speed target command, accord to power decrease output coefficient or DC current / Power limit coefficient.
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_SpeedTargetLimitProcess(SPEED_RAMP_HANDLE *command)
{
#if (defined DCCUR_LIMIT_ENABLE)
    ASR_SpeedCmd_Limit(command, g_dcCurLimit.curLimitCoeff);
//    Asr_SpeedPID_Limit(g_dcCurLimit.curLimitCoeff, -g_dcCurLimit.curLimitCoeff, &g_asrSpeedPIIn);
#endif
#if (defined POWER_LIMIT_ENABLE)
    ASR_SpeedCmd_Limit(command, g_powerLimit.powerLimitCoeff);
//    Asr_SpeedPID_Limit(g_powerLimit.powerLimitCoeff, -g_powerLimit.powerLimitCoeff, &g_asrSpeedPIIn);
#endif

#if (defined POWER_DECREASE_PROTECT_ENABLE)
    ASR_SpeedCmd_Limit(command, g_powerDecrease.depCoeff);
#endif
}

/*!
* @brief Motor protection main function. The corresponding protection functions
*        are implemented according to different time bases.
*
* @param[in] none
* @return none
*/
void Motor_Protection(void)
{
    static uint16_t protect10msCnt = 0;

#if (defined PHASE_CURRENT_PROTECTION_ENABLE)
    /* Phase over current protection, 1ms */
    Phase_Over_Current_Check(&g_focVarsCtrl);
#endif

#if (defined BUS_OVERCURRENT_PROTECTION_ENABLE)
    /* DC Bus over current protection , 1ms*/
    Bus_Over_Current_Check(g_dcCurEst.dcCurTrue);
#endif

#if (defined SENSORLESS)
    #if (defined STALL_PROTECTION_SENSORLESS_ENABLE)
    /* Sensorless stall protection, 1ms */
    Stall_Check_Sensorless(&g_speedCommand);
    #endif
#elif (defined HALL_SENSORS)
    #if (defined STALL_PROTECTION_SENSOR_ENABLE)
    /* Hall sensor stall protection, 1ms */
    Stall_Check_Sensor(g_rotationalSpeed.estSpeedRpm, &g_focVarsCtrl);
    #endif
#endif

#if (defined LOSE_SPEED_PROTECTION_ENABLE)
    /* Lose speed protection, 1ms */
    LoseSpeed_Check(g_speedCommand.speedFbk);
#endif

#if (defined ZERO_SPEED_CHECK_ENABLE)
    /* Zero speed protection, 1ms */
    ZeroSpeed_Check(g_speedCommand.speedFbk);
#endif

#if (defined LOSE_PHASE_PROTECTION_ENABLE)
    /* Lose phase check protection, 1ms */
    LosePhase_Check(&g_focVarsCtrl);
#endif

#if (defined VBUS_UNDERVOLTAGE_PROTECTION_ENABLE)
    /* DC Bus under voltage protection, 1ms */
    Vbus_UnderVoltage_Check(g_adcTransform.vdcVt);
#endif

#if (defined VBUS_OVERVOLTAGE_PROTECTION_ENABLE)
    /* DC Bus over voltage protection, 1ms */
    Vbus_OverVoltage_Check(g_adcTransform.vdcVt);
#endif

#if (defined DCCUR_LIMIT_ENABLE)
    /* DC bus current limit protection, 1ms */
    if (g_startUp.pStCtrl->startUpFlag == 6)
    {
        DcCurLimit_Calc(&g_dcCurLimit);
    }
#endif
#if (defined POWER_LIMIT_ENABLE)
    /* Input power limit protection, 1ms */
    if (g_startUp.pStCtrl->startUpFlag == 6)
    {
        PowerLimit_Calc(&g_powerLimit);
    }
#endif

#if (defined POWER_DECREASE_PROTECT_ENABLE)
    /* Power reduced protection, 1ms */
    if (g_startUp.pStCtrl->startUpFlag == 6)
    {
        PowerDecrease_Calc(&g_powerDecrease);
    }
#endif

    HallSelfLearnFailCheck();

    protect10msCnt++;
    if (protect10msCnt >= 10)
    {
        protect10msCnt = 0;
#if (defined HALL_SENSORS)
        /* Hall sensor sector status check protection, 10ms */
        Hall_Sensor_Status_Check(g_hallSensor.hallStatus);
#endif

        if ((Get_McStatus() == IDENTIFY) && (g_paramIdentify.pParamIdentifyCtrl->mainStep == IDENTIFY_RS_DCTEST))
        {
            LosePhase_Check(&g_focVarsCtrl);
        }

#if (defined PHASE_SHORT_PROTECTION_ENABLE)
        /* Phase short circuit protection, 10ms */
        Phase_Short_Dynamic_Check(&g_focVarsCtrl, g_rotationalSpeed.estSpeed);
#endif

        g_protector.protectPC++;
        if (g_protector.protectPC >= 10)
        {
            g_protector.protectPC = 0;
        }
        switch (g_protector.protectPC)
        {
        case 0:
        break;

        case 1:
        break;

        case 2:
        break;

        case 3 :
        break;

        case 4:
        break;

        case 5:
        break;

        case 6:
        break;

        default:
        break;
        }
    }
    if (g_protector.ErrFlag.all)
    {
        Stop_Motor();
        PushIn_McBufStatus(FAULT);
    }

    return;
}

/* =============================================  EOF  ============================================== */
