/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file startup.c
*
* @brief This file provides sensorless foc open loop startup functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "globalvar.h"
#include "foc_app.h"
#include "foc_ui_uart.h"
#include "motorctrl_api.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
/*!
* @brief Open loop startup curve time parameters.
*/
CURVE_TIME_DURATION  g_startUpCurveTime;

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief StartUp parameter initialization.
*
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void InitStartUpParam(START_UP *startUp)
{
    startUp->pStCtrl->curStartOffset = startUp->pStCfg->curStartOffset + Math_Mpy(Math_IQ(0.01), Math_IQ(startUp->pStCtrl->startUpTimes));

    return;
}

/*!
* @brief Clear startUp control counting parameter.
*
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void ClearStartUpParam(START_UP *startUp)
{
    startUp->pStCtrl->countStartUpST1 = 0;
    startUp->pStCtrl->countStartUpST1a = 0;
    startUp->pStCtrl->countStartUpST2 = 0;
    startUp->pStCtrl->countStartUpST3 = 0;
    startUp->pStCtrl->countStartUpST4 = 0;
    startUp->pStCtrl->countStartUpST5 = 0;

    return;
}

/*!
* @brief Open loop curve time variables deal, processing when time value is 0.
*
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUpCurveTimeInit(CURVE_TIME_DURATION *time, START_UP *startUp)
{
    uint16_t temp_timeValueUnit;

    temp_timeValueUnit = Math_DivShift(1000, 1000, 0);
    if (startUp->pStCfg->curveTime1 >= temp_timeValueUnit)
    {
        time->timePoint1 = startUp->pStCfg->curveTime1;
    }
    else
    {
        time->timePoint1 = temp_timeValueUnit;
    }
    if (startUp->pStCfg->curveTime2 >= temp_timeValueUnit)
    {
        time->timePoint2 = time->timePoint1 + startUp->pStCfg->curveTime2;
    }
    else
    {
        time->timePoint2 = time->timePoint1 + temp_timeValueUnit;
    }
    if (startUp->pStCfg->curveTime3 >= temp_timeValueUnit)
    {
        time->timePoint3 = time->timePoint2 + startUp->pStCfg->curveTime3;
    }
    else
    {
        time->timePoint3 = time->timePoint2 + temp_timeValueUnit;
    }
    if (startUp->pStCfg->curveTime4 >= temp_timeValueUnit)
    {
        time->timePoint4 = time->timePoint3 + startUp->pStCfg->curveTime4;
    }
    else
    {
        time->timePoint4 = time->timePoint3 + temp_timeValueUnit;
    }
    if (startUp->pStCfg->curveTime5 >= temp_timeValueUnit)
    {
        time->timePoint5 = time->timePoint4 + startUp->pStCfg->curveTime5;
    }
    else
    {
        time->timePoint5 = time->timePoint4 + temp_timeValueUnit;
    }
}

/*!
* @brief Set startup curve reference value, contain start and end value, time duration of each curve segment.
*
* @param[in] stCurveIn: pointer to CURVE_IN structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUpCurveRefInit(CURVE_IN *stCurveIn, CURVE_TIME_DURATION *time, START_UP *startUp)
{
    CURVE_REFVALUE *stCurveSeg1 = stCurveIn->curveSeg1;
    CURVE_REFVALUE *stCurveSeg2 = stCurveIn->curveSeg2;
    CURVE_REFVALUE *stCurveSeg3 = stCurveIn->curveSeg3;
    CURVE_REFVALUE *stCurveSeg4 = stCurveIn->curveSeg4;
    CURVE_REFVALUE *stCurveSeg5 = stCurveIn->curveSeg5;

    stCurveSeg1->curRefStart = startUp->pStCfg->curveCurStart;
    stCurveSeg1->freqRefStart = startUp->pStCfg->curveFreqStart;
    if (CURVE_TYPE == CURVE_TYPE_STRAIGHT)
    {
        stCurveSeg1->freqRefEnd = ((uint32_t)startUp->pStCfg->synSpd * g_STC.pMotor->polePairs << 15) / g_STC.pFOCVars->pCfg->baseFreq;
        stCurveSeg1->curRefEnd = startUp->pStCfg->curStartOffset;
    }
    else if (CURVE_TYPE == CURVE_TYPE_CUSTOMER)
    {
        if (startUp->pStCfg->curveSegNum > 1)
        {
            stCurveSeg1->freqRefEnd = startUp->pStCfg->curveFreqSet1;
            stCurveSeg1->curRefEnd = startUp->pStCfg->curveCurSet1;
        }
        else
        {
            stCurveSeg1->freqRefEnd = ((uint32_t)startUp->pStCfg->synSpd * g_STC.pMotor->polePairs << 15) / g_STC.pFOCVars->pCfg->baseFreq;
            stCurveSeg1->curRefEnd = startUp->pStCfg->curStartOffset;
        }
    }
    else
    {
    }
    stCurveSeg1->voltRefStart = startUp->pStCfg->curveVoltStart;
    stCurveSeg1->voltRefEnd = startUp->pStCfg->curveVoltSet1;
    stCurveSeg1->timeStart = 0;
    stCurveSeg1->timeEnd = time->timePoint1;

    stCurveSeg2->curRefStart = startUp->pStCfg->curveCurSet1;
    stCurveSeg2->curRefEnd = startUp->pStCfg->curveCurSet2;
    stCurveSeg2->freqRefStart = startUp->pStCfg->curveFreqSet1;
    stCurveSeg2->freqRefEnd = startUp->pStCfg->curveFreqSet2;
    stCurveSeg2->voltRefStart = startUp->pStCfg->curveVoltSet1;
    stCurveSeg2->voltRefEnd = startUp->pStCfg->curveVoltSet2;
    stCurveSeg2->timeStart = time->timePoint1;
    stCurveSeg2->timeEnd = time->timePoint2;

    stCurveSeg3->curRefStart = startUp->pStCfg->curveCurSet2;
    stCurveSeg3->curRefEnd = startUp->pStCfg->curveCurSet3;
    stCurveSeg3->freqRefStart = startUp->pStCfg->curveFreqSet2;
    stCurveSeg3->freqRefEnd = startUp->pStCfg->curveFreqSet3;
    stCurveSeg3->voltRefStart = startUp->pStCfg->curveVoltSet2;
    stCurveSeg3->voltRefEnd = startUp->pStCfg->curveVoltSet3;
    stCurveSeg3->timeStart = time->timePoint2;
    stCurveSeg3->timeEnd = time->timePoint3;

    stCurveSeg4->curRefStart = startUp->pStCfg->curveCurSet3;
    stCurveSeg4->curRefEnd = startUp->pStCfg->curveCurSet4;
    stCurveSeg4->freqRefStart = startUp->pStCfg->curveFreqSet3;
    stCurveSeg4->freqRefEnd = startUp->pStCfg->curveFreqSet4;
    stCurveSeg4->voltRefStart = startUp->pStCfg->curveVoltSet3;
    stCurveSeg4->voltRefEnd = startUp->pStCfg->curveVoltSet4;
    stCurveSeg4->timeStart = time->timePoint3;
    stCurveSeg4->timeEnd = time->timePoint4;

    stCurveSeg5->curRefStart = startUp->pStCfg->curveCurSet4;
    stCurveSeg5->curRefEnd = startUp->pStCfg->curveCurSet5;
    stCurveSeg5->freqRefStart = startUp->pStCfg->curveFreqSet4;
    stCurveSeg5->freqRefEnd = startUp->pStCfg->curveFreqSet5;
    stCurveSeg5->voltRefStart = startUp->pStCfg->curveVoltSet4;
    stCurveSeg5->voltRefEnd = startUp->pStCfg->curveVoltSet5;
    stCurveSeg5->timeStart = time->timePoint4;
    stCurveSeg5->timeEnd = time->timePoint5;
}

/*!
* @brief Startup curve parameters initilization, contain start and end value, time duration of each curve segment.
*
* @param[in] stCurveIn: pointer to CURVE_IN structure
* @param[in] stCurveOut: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUpCurveParamInit(CURVE_IN *stCurveIn, CURVE_OUT *stCurveOut, CURVE_TIME_DURATION *time, START_UP *startUp)
{
    stCurveIn->groupNum = 0;
    stCurveIn->refValue1 = 0;
    stCurveIn->refValue2 = 0;
    stCurveIn->timeDuration = 0;
    stCurveIn->timeEndValue = 0;
    stCurveIn->timeStartValue = 0;
    stCurveIn->timeValue = 0;

    memset(stCurveOut, 0U, sizeof(CURVE_OUT));
    StartUpCurveTimeInit(time, startUp);
    StartUpCurveRefInit(stCurveIn, time, startUp);
}

/*!
* @brief Motor startup control initialization.
*
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Ctrl_Init(START_UP *startUp)
{
    STARTUP_CTRL *stCtrl = startUp->pStCtrl;
    memset(stCtrl, 0U, sizeof(*stCtrl));
    startUp->pStCtrl->startUpElecTheta = 0;
    startUp->pStCtrl->elecThetaEst = 0;

    startUp->pStCtrl->startUpTimes = 0;

    startUp->pStCtrl->synSpd = startUp->pStCfg->synSpd;
    startUp->pStCtrl->startUpFlag = 0;
    startUp->pStCtrl->synTime = startUp->pStCfg->synTime;
    startUp->pStCtrl->synFreReach = 0;
    startUp->pStCtrl->synCount = 0;
    startUp->pStCtrl->curStartSub = startUp->pStCfg->curStartSub;
    startUp->pStCtrl->curAddSlope1 = Math_Div(startUp->pStCfg->alignCur - startUp->pStCfg->curRamp1Init, startUp->pStCfg->slopeDuration1);
    startUp->pStCtrl->curAddSlope2 = Math_Div(startUp->pStCfg->alignCurTwice - startUp->pStCfg->curRamp2Init, startUp->pStCfg->slopeDuration2);
#if (defined SENSORLESS)
    startUp->pStCtrl->alignState = 0;
#elif (defined ENCODER_SENSORS)
    if(startUp->pStCtrl->alignState)
    {
        startUp->pStCtrl->alignState = 1;
    }
    else
    {
        startUp->pStCtrl->alignState = 0;
    }
#else
#endif

    ClearStartUpParam(startUp);
    InitStartUpParam(startUp);

    StartUpCurveParamInit(&g_startUpCurveIn, &g_startUpCurveOut, &g_startUpCurveTime, startUp);
}

/*!
* @brief Bootstrap capacitor charging process.
*
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void Bootstrap_Charging_Process(STARTUP_CTRL *startUp)
{
    startUp->countStartUpST1 ++;

    if (startUp->countStartUpST1 <= 1)
    {
        PWM_Disable();
    }
    else if (startUp->countStartUpST1 <= 2)
    {
#if PHASE_UVW_POLARITY == ACTIVE_HIGH
        PWM_Update(0, 0, 0);
#elif PHASE_UVW_POLARITY == ACTIVE_LOW
        PWM_Update(PWM_PERIOD_VALUE, PWM_PERIOD_VALUE, PWM_PERIOD_VALUE);
#endif
        PWM_Enable();
    }
    else if (startUp->countStartUpST1 <= 4)
    {

    }
    else
    {
        startUp->countStartUpST1 = 1;
        startUp->countStartUpST1a ++;
        if (startUp->countStartUpST1 > 20 * MS_UINT_INT)
        {
            startUp->countStartUpST1 = 0;
            startUp->countStartUpST1a = 0;
            PWM_Enable();
            startUp->startUpFlag = 2;
        }
    }
}

/*!
* @brief Pre_positioning motor to an expected angle.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS_CTRL structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Angle_Prepositioning_Process(FOC_VARS_CTRL *focVarsCtrl, START_UP *startUp)
{
    ++(startUp->pStCtrl->countStartUpST2);
    if (startUp->pStCtrl->countStartUpST2 <= startUp->pStCfg->alignTime)
    {
        startUp->pStCtrl->startUpElecTheta = startUp->pStCfg->alignAngle;
        focVarsCtrl->elecAngle = startUp->pStCtrl->startUpElecTheta;
        focVarsCtrl->idRef = 0;
        if (startUp->pStCtrl->countStartUpST2 <= startUp->pStCfg->slopeDuration1)
        {
            focVarsCtrl->iqRef = (startUp->pStCfg->curRamp1Init + Math_Mpy(startUp->pStCtrl->countStartUpST2, startUp->pStCtrl->curAddSlope1)) * pFocVarsCfg->motorDir;
        }
        else
        {
            focVarsCtrl->iqRef = startUp->pStCfg->alignCur * pFocVarsCfg->motorDir;
        }

    }
    else
    {
#if (defined MOTOR_POLES_OBTAIN)
        startUp->pStCtrl->startUpFlag = 2;
#else
        startUp->pStCtrl->countStartUpST2 = 0;
        startUp->pStCtrl->alignState = 1;
        startUp->pStCtrl->startUpFlag = 3;
#endif
    }
}

/*!
* @brief Angle alignment process to prepare for motor open loop operation.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS_CTRL structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Angle_Alignment_Process(FOC_VARS_CTRL *focVarsCtrl, START_UP *startUp)
{
    ++ startUp->pStCtrl->countStartUpST2;
    if (startUp->pStCtrl->countStartUpST2 < startUp->pStCfg->alignTime)
    {
        startUp->pStCtrl->startUpElecTheta = Q16_180_DEGREE;
        focVarsCtrl->elecAngle = startUp->pStCtrl->startUpElecTheta;
        focVarsCtrl->idRef = Math_IQ(0);
        if (startUp->pStCtrl->countStartUpST2 <= startUp->pStCfg->slopeDuration1)
        {
            focVarsCtrl->iqRef = (startUp->pStCfg->curRamp1Init + Math_Mpy(startUp->pStCtrl->countStartUpST2, startUp->pStCtrl->curAddSlope1)) * pFocVarsCfg->motorDir;
        }
        else
        {
            focVarsCtrl->iqRef = startUp->pStCfg->alignCur * pFocVarsCfg->motorDir;
        }
    }
    else if (startUp->pStCtrl->countStartUpST2 < (startUp->pStCfg->alignTime + startUp->pStCfg->alignTimeTwice))
    {
        startUp->pStCtrl->startUpElecTheta = Q16_270_DEGREE;
        focVarsCtrl->elecAngle = startUp->pStCtrl->startUpElecTheta;
        if (startUp->pStCtrl->countStartUpST2 <= (startUp->pStCfg->alignTime + startUp->pStCfg->slopeDuration2))
        {
            focVarsCtrl->iqRef = (startUp->pStCfg->curRamp2Init + Math_Mpy(startUp->pStCtrl->countStartUpST2 - startUp->pStCfg->alignTime, startUp->pStCtrl->curAddSlope2)) * pFocVarsCfg->motorDir;
        }
        else
        {
            focVarsCtrl->iqRef = startUp->pStCfg->alignCurTwice * pFocVarsCfg->motorDir;
        }
        focVarsCtrl->idRef = Math_IQ(0);
    }
    else
    {
    #if (defined MOTOR_POLES_OBTAIN)
        startUp->pStCtrl->startUpFlag = 2;
    #else
        startUp->pStCtrl->countStartUpST2 = 0;
        startUp->pStCtrl->alignState = 1;
        startUp->pStCtrl->startUpFlag = 3;
    #endif
    }
}

/*!
* @brief Make the open angle of the operation close to the observer angle.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void Angle_Switching_Process(FOC_VARS *focVars, STARTUP_CTRL *startUp)
{
    int16_t elecThetaErr = 0;
    static uint16_t s_thetaCompTimes = 0;

    if (pFocVarsCfg->motorDir == 1)
    {
        elecThetaErr = startUp->elecThetaEst - startUp->startUpElecTheta;
    }
    else if (pFocVarsCfg->motorDir == -1)
    {
        elecThetaErr = startUp->startUpElecTheta - startUp->elecThetaEst;
    }
    else
    {
    }

    if (1 == startUp->synFreReach)
    {
        if (elecThetaErr >= Q16_1_DEGREE)
        {
            if (focVars->pCfg->motorDir == 1)
            {
                if (startUp->startUpElecTheta < startUp->elecThetaEst)
                {
                    if (++s_thetaCompTimes <= 1024)
                    {
                        startUp->startUpElecTheta = (uint16_t)(((uint32_t)(startUp->elecThetaEst) * (s_thetaCompTimes)
                                                  + (uint32_t)(startUp->startUpElecTheta) * (1024 - (s_thetaCompTimes))) >> 10);
                    }
                    else
                    {
                        startUp->countStartUpST3 = 0;
                    #if (defined OPEN_LOOP)
                        startUp->startUpFlag = 3;
                    #elif (defined CLOSE_LOOP)
                        startUp->startUpFlag = 4;
                    #endif
                        startUp->startUpElecTheta = startUp->elecThetaEst;
                        s_thetaCompTimes = 0;
                    }
                }
                else
                {
                }
            }
            else if (focVars->pCfg->motorDir == -1)
            {
                if (startUp->startUpElecTheta > startUp->elecThetaEst)
                {
                    if (++s_thetaCompTimes <= 1024)
                    {
                        startUp->startUpElecTheta = (uint16_t)(((uint32_t)(startUp->elecThetaEst) * (s_thetaCompTimes)
                                                  + (uint32_t)(startUp->startUpElecTheta) * (1024 - (s_thetaCompTimes))) >> 10);
                    }
                    else
                    {
                        startUp->countStartUpST3 = 0;
                    #if (defined OPEN_LOOP)
                        startUp->startUpFlag = 3;
                    #elif (defined CLOSE_LOOP)
                        startUp->startUpFlag = 4;
                    #endif

                        startUp->startUpElecTheta = startUp->elecThetaEst;
                        s_thetaCompTimes = 0;
                    }
                }
            }
            else
            {
            }
        }
        else if ((elecThetaErr < Q16_1_DEGREE) && (elecThetaErr > 0))
        {
            startUp->countStartUpST3 = 0;
#if (defined OPEN_LOOP)
            startUp->startUpFlag = 3;           /*!< If runs in OPEN_LOOP, do not run the follow-up process */
#elif (defined CLOSE_LOOP)
            startUp->startUpFlag = 4;
#endif

            startUp->startUpElecTheta = startUp->elecThetaEst;
            s_thetaCompTimes = 0;
        }
        else
        {
        }
    }
}

/*!
* @brief Open loop acceleration process to make the motor run to a certain speed.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void Openloop_Acceleration_Process(FOC_VARS *focVars, STARTUP_CTRL *startUp)
{
#if (defined FOC_PMSM)
    /* Normal StartUp Mode */
#if (defined DIRECT_STARTUP)
    #if (defined SMC_OBSERVE)
    startUp->elecThetaEst = Smc_PositionEstimator(&g_STC, &g_smcEstimator, &g_rotationalSpeed, &g_pidPll);
    #elif (defined FLUX_OBSERVE)
    startUp->elecThetaEst = FluxObserver_PosEst(&g_STC, &g_fluxObserver, &g_rotationalSpeed, &g_pidPll);
    #elif (defined MRAS_OBSERVE)
    startUp->elecThetaEst = Mras_PosEst(&g_STC, &g_mrasObserver, &g_pidPll, startUp->startUpFlag);
    #else
    /* Do nothing */
    #endif
    startUp->startUpElecTheta = startUp->elecThetaEst;
    startUp->countStartUpST3 ++;

    if (startUp->countStartUpST3 < 100 * MS_UINT_INT)
    {
        focVars->pCtrl->idRef = 0;
        focVars->pCtrl->iqRef = pStartUpCfg->curStartOffset;
    }
    else
    {
        startUp->countStartUpST3 = 0;
        #if (defined OPEN_LOOP)
        startUp->startUpFlag = 3;
        #elif (defined CLOSE_LOOP)
        startUp->startUpFlag = 4;
        #endif
    }

    /* If StartUp Mode */
#elif (defined IF_STARTUP)
    #if (defined SMC_OBSERVE)
    startUp->elecThetaEst = Smc_PositionEstimator(&g_STC, &g_smcEstimator, &g_rotationalSpeed, &g_pidPll);
    #elif (defined FLUX_OBSERVE)
    startUp->elecThetaEst = FluxObserver_PosEst(&g_STC, &g_fluxObserver, &g_rotationalSpeed, &g_pidPll);
    #elif (defined MRAS_OBSERVE)
    startUp->elecThetaEst = Mras_PosEst(&g_STC, &g_mrasObserver, &g_pidPll, startUp->startUpFlag);
    #else
    /* Do nothing */
    #endif

    #if (defined CLOSE_LOOP)
    Angle_Switching_Process(focVars, startUp);
    #endif

    /* Vf StartUp Mode */
#elif (defined VF_STARTUP)
    #if (defined SMC_OBSERVE)
    startUp->elecThetaEst = Smc_PositionEstimator(&g_STC, &g_smcEstimator, &g_rotationalSpeed, &g_pidPll);
    #elif (defined FLUX_OBSERVE)
    startUp->elecThetaEst = FluxObserver_PosEst(&g_STC, &g_fluxObserver, &g_rotationalSpeed, &g_pidPll);
    #elif (defined MRAS_OBSERVE)
    startUp->elecThetaEst = Mras_PosEst(&g_STC, &g_mrasObserver, &g_pidPll, startUp->startUpFlag);
    #endif
    focVars->pCtrl->idRef = 0;

    focVars->pCtrl->vdRef = 0;
    focVars->pCtrl->vqRef = (int32_t)g_startUpCurveOut.voltRefValue * focVars->pCfg->motorDir;

    #if (defined CLOSE_LOOP)
    Angle_Switching_Process(focVars, startUp);
    #endif

#endif
#elif (defined FOC_ACIM)
    Acim_FluxObserverCalc(&g_acimFocCtrl, focVars, &g_pidPll, ACIM_FO_METHOD_2);
    Acim_FluxObserverAngleCalc(&g_acimFocCtrl, &g_STC);

    startUp->elecThetaEst = g_acimFocCtrl.flux->rotorFluxAngle;

    #if (defined CLOSE_LOOP)
    Angle_Switching_Process(focVars, startUp);
    #endif
#endif
}

/*!
* @brief The closed-loop deceleration process allows the motor to smoothly cut from the open loop to the closed loop state.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Closedloop_Deceleration_Process(FOC_VARS *focVars, START_UP *startUp)
{
#if (defined FOC_PMSM)
    #if (defined SMC_OBSERVE)
    startUp->pStCtrl->elecThetaEst = Smc_PositionEstimator(&g_STC, &g_smcEstimator, &g_rotationalSpeed, &g_pidPll);
    #elif (defined FLUX_OBSERVE)
    startUp->pStCtrl->elecThetaEst = FluxObserver_PosEst(&g_STC, &g_fluxObserver, &g_rotationalSpeed, &g_pidPll);
    #elif (defined MRAS_OBSERVE)
    startUp->pStCtrl->elecThetaEst = Mras_PosEst(&g_STC, &g_mrasObserver, &g_pidPll, startUp->pStCtrl->startUpFlag);
    #else
    /* Do nothing */
    #endif

    startUp->pStCtrl->startUpElecTheta = startUp->pStCtrl->elecThetaEst;
    startUp->pStCtrl->countStartUpST4++;
    startUp->pStCtrl->countStartUpST4 = 10 * MS_UINT_INT;
    if (startUp->pStCtrl->countStartUpST4 < 10 * MS_UINT_INT)       /*!< 10ms */
    {
        if (focVars->pCtrl->iqRef > 0)
        {
            focVars->pCtrl->iqRef -= startUp->pStCtrl->curStartSub;
        }
        else
        {
            focVars->pCtrl->iqRef = 0;
        }
    }
    else
    {
        startUp->pStCtrl->startUpFlag = 5;
        startUp->pStCtrl->countStartUpST4 = 0;

        /* smoothly cut from the open loop to the closed loop state */
        Asr_SetSpeedPID_Integral(&g_asrSpeedPIOut, (pFocVarsCfg->motorDir * startUp->pStCfg->spdSmoothCur));
    }
#elif (defined FOC_ACIM)
    Acim_FluxObserverCalc(&g_acimFocCtrl, focVars, &g_pidPll, ACIM_FO_METHOD_2);
    Acim_FluxObserverAngleCalc(&g_acimFocCtrl, &g_STC);

    startUp->pStCtrl->elecThetaEst = g_acimFocCtrl.flux->rotorFluxAngle;

    startUp->pStCtrl->startUpElecTheta = startUp->pStCtrl->elecThetaEst;
    startUp->pStCtrl->countStartUpST4++;
    if (startUp->pStCtrl->countStartUpST4 < 10 * MS_UINT_INT)       /*!< 10ms */
    {
        if (focVars->pCfg->motorDir == 1)
        {
            if (focVars->pCtrl->iqRef > g_pidAsr.pPidOutput->outPu)
            {
                focVars->pCtrl->iqRef -= startUp->pStCtrl->curStartSub;
            }
            else
            {
                focVars->pCtrl->iqRef = g_pidAsr.pPidOutput->outPu;
            }
        }
        else
        {
            if (focVars->pCtrl->iqRef < g_pidAsr.pPidOutput->outPu)
            {
                focVars->pCtrl->iqRef += startUp->pStCtrl->curStartSub;
            }
            else
            {
                focVars->pCtrl->iqRef = g_pidAsr.pPidOutput->outPu;
            }
        }
    }
    else
    {
        startUp->pStCtrl->startUpFlag = 5;
        startUp->pStCtrl->countStartUpST4 = 0;

        /* smoothly cut from the open loop to the closed loop state */
//        Asr_SetSpeedPID_Integral(&g_asrSpeedPIOut, (pFocVarsCfg->motorDir * startUp->pStCfg->spdSmoothCur));
    }
#endif
}

/*!
* @brief The closed loop process runs the motor using the observer angle.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void Closedloop_Process(FOC_VARS *focVars, STARTUP_CTRL *startUp)
{
#if (defined FOC_PMSM)
    #if (defined SMC_OBSERVE)
    startUp->elecThetaEst = Smc_PositionEstimator(&g_STC, &g_smcEstimator, &g_rotationalSpeed, &g_pidPll);
    #elif (defined FLUX_OBSERVE)
    startUp->elecThetaEst = FluxObserver_PosEst(&g_STC, &g_fluxObserver, &g_rotationalSpeed, &g_pidPll);
    #elif (defined MRAS_OBSERVE)
    startUp->elecThetaEst = Mras_PosEst(&g_STC, &g_mrasObserver, &g_pidPll, startUp->startUpFlag);
    #else
    /* Do nothing */
    #endif
#elif (defined FOC_ACIM)
    Acim_FluxObserverCalc(&g_acimFocCtrl, focVars, &g_pidPll, ACIM_FO_METHOD_2);
    Acim_FluxObserverAngleCalc(&g_acimFocCtrl, &g_STC);

    startUp->elecThetaEst = g_acimFocCtrl.flux->rotorFluxAngle;
#endif

    startUp->startUpElecTheta = startUp->elecThetaEst;
}

/*!
* @brief The closed loop process switch to open loop.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void CloseToOpenloop_Process(FOC_VARS *focVars, STARTUP_CTRL *startUp)
{
    #if (defined SMC_OBSERVE)
    startUp->elecThetaEst = Smc_PositionEstimator(&g_STC, &g_smcEstimator, &g_rotationalSpeed, &g_pidPll);
    #elif (defined FLUX_OBSERVE)
    startUp->elecThetaEst = FluxObserver_PosEst(&g_STC, &g_fluxObserver, &g_rotationalSpeed, &g_pidPll);
    #elif (defined MRAS_OBSERVE)
    startUp->elecThetaEst = Mras_PosEst(&g_STC, &g_mrasObserver, &g_pidPll, startUp->startUpFlag);
    #else
    /* Do nothing */
    #endif
}

/*!
* @brief Open loop process switch to close loop process.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void OpenToCloseloop_Process(FOC_VARS *focVars, STARTUP_CTRL *startUp)
{
    /* If StartUp Mode */
#if (defined IF_STARTUP)
    #if (defined SMC_OBSERVE)
    startUp->elecThetaEst = Smc_PositionEstimator(&g_STC, &g_smcEstimator, &g_rotationalSpeed, &g_pidPll);
    #elif (defined FLUX_OBSERVE)
    startUp->elecThetaEst = FluxObserver_PosEst(&g_STC, &g_fluxObserver, &g_rotationalSpeed, &g_pidPll);
    #elif (defined MRAS_OBSERVE)
    startUp->elecThetaEst = Mras_PosEst(&g_STC, &g_mrasObserver, &g_pidPll, startUp->startUpFlag);
    #else
    /* Do nothing */
    #endif

    #if (defined CLOSE_LOOP)
    Angle_Switching_Process(focVars, startUp);
    #endif
#endif
}

/*!
* @brief Calculate curve output with unsigned type parameters.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @return none
*/
void GetCurveRefValue(CURVE_IN *in, CURVE_OUT *out)
{
    in->timeDuration = in->timeEndValue - in->timeStartValue;
    if (0 != in->timeDuration)
    {
        if (in->refValue2 > in->refValue1)
        {
            out->refCalcValue = ((uint32_t)(in->refValue2 - in->refValue1) * in->timeValue \
                                + ((uint32_t)in->refValue1 * in->timeEndValue) \
                                - ((uint32_t)in->refValue2 * in->timeStartValue)) / in->timeDuration;
        }
        else
        {
            out->refCalcValue = (((uint32_t)in->refValue1 * in->timeEndValue) \
                                - ((uint32_t)(in->refValue1 - in->refValue2) * in->timeValue) \
                                - ((uint32_t)in->refValue2 * in->timeStartValue)) / in->timeDuration;
        }
    }
    else
    {
        out->refCalcValue = in->refValue2;
    }
}

/*!
* @brief Open loop startup curve straight(default) mode.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Straight(CURVE_IN *in, CURVE_OUT *out, CURVE_TIME_DURATION *time, START_UP *startUp)
{
    if (0 == out->calcEndFlag)
    {
        if (++(in->timeValue) <= time->timePoint1)
        {
            in->timeStartValue = 0;
            in->timeEndValue = time->timePoint1;

            in->refValue1 = startUp->pStCfg->curveFreqStart;
            in->refValue2 = in->curveSeg1->freqRefEnd;
            GetCurveRefValue(in, out);
            out->frqRefValue = out->refCalcValue;

            #if (defined IF_STARTUP)
            in->refValue1 = startUp->pStCfg->curveCurStart;
            in->refValue2 = startUp->pStCfg->curveCurSet1;
            GetCurveRefValue(in, out);
            out->curRefValue = out->refCalcValue;
            #elif (defined VF_STARTUP)
            in->refValue1 = startUp->pStCfg->curveVoltStart;
            in->refValue2 = startUp->pStCfg->curveVoltSet1;
            GetCurveRefValue(in, out);
            out->voltRefValue = out->refCalcValue;
            #endif

            out->calcEndFlag = 0;
        }
        else
        {
            in->timeValue = 0;
            out->calcEndFlag = 1;
        }
    }
    else
    {
    }
}

/*!
* @brief Open loop startup curve straight(default) mode, used in close loop to open loop switch.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Straight1(CURVE_IN *in, CURVE_OUT *out, CURVE_TIME_DURATION *time, START_UP *startUp)
{
    if (0 == out->calcEndFlag1)
    {
        if (++(in->timeValue) <= (time->timePoint1))
        {
            in->timeStartValue = 0;
            in->timeEndValue = time->timePoint1;

            in->refValue1 = startUp->pStCfg->curveFreqStart1;
            in->refValue2 = in->curveSeg1->freqRefEnd;
            GetCurveRefValue(in, out);
            out->frqRefValue = out->refCalcValue;

            #if (defined IF_STARTUP)
            out->curRefValue = startUp->pStCfg->curStartOffset;
            #elif (defined VF_STARTUP)
            in->refValue1 = startUp->pStCfg->curveVoltStart;
            in->refValue2 = startUp->pStCfg->curveVoltSet1;
            GetCurveRefValue(in, out);
            out->voltRefValue = out->refCalcValue;
            #endif

            out->calcEndFlag1 = 0;
        }
        else
        {
            in->timeValue = 0;
            out->calcEndFlag1 = 1;
        }
    }
    else
    {
    }
}

/*!
* @brief Open loop startup curve straight(default) mode, used in open loop to close loop switch.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Straight2(CURVE_IN *in, CURVE_OUT *out, CURVE_TIME_DURATION *time, START_UP *startUp)
{
    if (0 == out->calcEndFlag2)
    {
        if (++(in->timeValue) <= time->timePoint1)
        {
            in->timeStartValue = 0;
            in->timeEndValue = time->timePoint1;

            in->refValue1 = defFocVars.speedMinLimit;
            in->refValue2 = in->curveSeg1->freqRefEnd;
            GetCurveRefValue(in, out);
            out->frqRefValue = out->refCalcValue;
            out->curRefValue = startUp->pStCfg->curveCurSet1;
            out->calcEndFlag2 = 0;
        }
        else
        {
            in->timeValue = 0;
            out->calcEndFlag2 = 1;
        }
    }
    else
    {
    }
}

/*!
* @brief Segment curve calculation unit block.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] ref: pointer to CURVE_REFVALUE structure
* @return none
*/
void StartUp_SegmentCurve_Calc(CURVE_IN *in, CURVE_OUT *out, CURVE_REFVALUE *ref)
{
    in->timeStartValue = ref->timeStart;
    in->timeEndValue = ref->timeEnd;

    in->refValue1 = ref->freqRefStart;
    in->refValue2 = ref->freqRefEnd;
    GetCurveRefValue(in, out);
    out->frqRefValue = out->refCalcValue;

    #if (defined IF_STARTUP)
    in->refValue1 = ref->curRefStart;
    in->refValue2 = ref->curRefEnd;
    GetCurveRefValue(in, out);
    out->curRefValue = out->refCalcValue;
    #elif (defined VF_STARTUP)
    in->refValue1 = ref->voltRefStart;
    in->refValue2 = ref->voltRefEnd;
    GetCurveRefValue(in, out);
    out->voltRefValue = out->refCalcValue;
    #endif

    out->calcEndFlag = 0;
}

/*!
* @brief When segment curve calculation finished, call this function.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] ref: pointer to CURVE_REFVALUE structure
* @return none
*/
void StartUp_SegmentCurve_CalcEnd(CURVE_IN *in, CURVE_OUT *out, CURVE_REFVALUE *ref)
{
    out->frqRefValue = ref->freqRefEnd;
    #if (defined IF_STARTUP)
    out->curRefValue = ref->curRefEnd;
    #elif (defined VF_STARTUP)
    out->voltRefValue = ref->voltRefEnd;
    #endif
    in->timeValue = 0;
    out->calcEndFlag = 1;
}

/*!
* @brief The default output of curve calculation.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_SegmentCurve_Default(CURVE_IN *in, CURVE_OUT *out, START_UP *startUp)
{
    out->frqRefValue = startUp->pStCfg->curveFreqStart;
    #if (defined IF_STARTUP)
    out->curRefValue = startUp->pStCfg->curveCurStart;
    #elif (defined VF_STARTUP)
    out->voltRefValue = startUp->pStCfg->curveVoltStart;
    #endif
    in->timeValue = 0;
    out->calcEndFlag = 1;
}

/*!
* @brief When segment curve has 1 point, call this function.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @return none
*/
void StartUp_SegmentCurve_Part1(CURVE_IN *in, CURVE_OUT *out, CURVE_TIME_DURATION *time)
{
    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg1);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd(in, out, in->curveSeg1);
    }
}

/*!
* @brief When segment curve has 2 point, call this function.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @return none
*/
void StartUp_SegmentCurve_Part2(CURVE_IN *in, CURVE_OUT *out, CURVE_TIME_DURATION *time)
{
    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg1);
    }
    else if ((in->timeValue > time->timePoint1) && (in->timeValue <= time->timePoint2))
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg2);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd(in, out, in->curveSeg2);
    }
}

/*!
* @brief When segment curve has 3 point, call this function.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @return none
*/
void StartUp_SegmentCurve_Part3(CURVE_IN *in, CURVE_OUT *out, CURVE_TIME_DURATION *time)
{
    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg1);
    }
    else if ((in->timeValue > time->timePoint1) && (in->timeValue <= time->timePoint2))
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg2);
    }
    else if ((in->timeValue > time->timePoint2) && (in->timeValue <= time->timePoint3))
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg3);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd(in, out, in->curveSeg3);
    }
}

/*!
* @brief When segment curve has 4 point, call this function.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @return none
*/
void StartUp_SegmentCurve_Part4(CURVE_IN *in, CURVE_OUT *out, CURVE_TIME_DURATION *time)
{
    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg1);
    }
    else if ((in->timeValue > time->timePoint1) && (in->timeValue <= time->timePoint2))
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg2);
    }
    else if ((in->timeValue > time->timePoint2) && (in->timeValue <= time->timePoint3))
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg3);
    }
    else if ((in->timeValue > time->timePoint3) && (in->timeValue <= time->timePoint4))
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg4);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd(in, out, in->curveSeg4);
    }
}

/*!
* @brief When segment curve has 5 point, call this function.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @return none
*/
void StartUp_SegmentCurve_Part5(CURVE_IN *in, CURVE_OUT *out, CURVE_TIME_DURATION *time)
{
    in->timeValue++;
    if (in->timeValue <= time->timePoint1)
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg1);
    }
    else if ((in->timeValue > time->timePoint1) && (in->timeValue <= time->timePoint2))
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg2);
    }
    else if ((in->timeValue > time->timePoint2) && (in->timeValue <= time->timePoint3))
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg3);
    }
    else if ((in->timeValue > time->timePoint3) && (in->timeValue <= time->timePoint4))
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg4);
    }
    else if ((in->timeValue > time->timePoint4) && (in->timeValue <= time->timePoint5))
    {
        StartUp_SegmentCurve_Calc(in, out, in->curveSeg5);
    }
    else
    {
        StartUp_SegmentCurve_CalcEnd(in, out, in->curveSeg5);
    }
}

/*!
* @brief Open loop startup curve customer mode.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Customer(CURVE_IN *in, CURVE_OUT *out, CURVE_TIME_DURATION *time, START_UP *startUp)
{
    /* curve segment number, configed by host tool */
    in->groupNum = startUp->pStCfg->curveSegNum;

    if (0 == out->calcEndFlag)
    {
        switch (in->groupNum)
        {
        case 0:
        StartUp_SegmentCurve_Default(in, out, startUp);
        break;

        case 1:
        StartUp_SegmentCurve_Part1(in, out, time);
        break;

        case 2:
        StartUp_SegmentCurve_Part2(in, out, time);
        break;

        case 3:
        StartUp_SegmentCurve_Part3(in, out, time);
        break;

        case 4:
        StartUp_SegmentCurve_Part4(in, out, time);
        break;

        case 5:
        StartUp_SegmentCurve_Part5(in, out, time);
        break;

        default:
        StartUp_SegmentCurve_Default(in, out, startUp);
        break;
        }
    }
    else
    {
    }
}

/*!
* @brief Calculation of electric angle in open loop by integral.
*
* @param[in] freqPu, target frequency
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void OpenLoopElecThetaCalc(int16_t freqPu, FOC_VARS *focVars, STARTUP_CTRL *startUp)
{
    uint16_t deltaTheta = 0;
    uint16_t freqAbs = 0;
    uint16_t operatingFreq = 0;
    uint16_t freqBase = 0;

    freqAbs = (uint16_t)(Math_Qabs(freqPu));
    operatingFreq = focVars->pCfg->speedFreq;

    freqBase = focVars->pCfg->baseFreq;

    deltaTheta = (uint16_t)((((uint32_t)freqAbs * freqBase) << 1) / operatingFreq);
    if (1 == focVars->pCfg->motorDir)
    {
        startUp->startUpElecTheta += deltaTheta;
    }
    else if (-1 == focVars->pCfg->motorDir)
    {
        startUp->startUpElecTheta -= deltaTheta;
    }
    else
    {
        startUp->startUpElecTheta += deltaTheta;
    }
}

/*!
* @brief Open loop startup curve reference value calculation main process.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUpCurve_Process(CURVE_IN *in, CURVE_OUT *out, CURVE_TIME_DURATION *time, START_UP *startUp)
{
    #if (CURVE_TYPE == CURVE_TYPE_STRAIGHT)
    StartUp_Straight(in, out, time, startUp);
    #elif (CURVE_TYPE == CURVE_TYPE_CUSTOMER)
    StartUp_Customer(in, out, time, startUp);
    #endif
}

/*!
* @brief Open loop startup curve reference value calculation main process, used in close loop to open loop switch..
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUpCurve_Process1(CURVE_IN *in, CURVE_OUT *out, CURVE_TIME_DURATION *time, START_UP *startUp)
{
    StartUp_Straight1(in, out, time, startUp);
}

/*!
* @brief Open loop startup curve reference value calculation main process, used in open loop to close loop switch.
*
* @param[in] in: pointer to CURVE_IN structure
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] time: pointer to CURVE_TIME_DURATION structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUpCurve_Process2(CURVE_IN *in, CURVE_OUT *out, CURVE_TIME_DURATION *time, START_UP *startUp)
{
    StartUp_Straight2(in, out, time, startUp);
}

/*!
* @brief Open loop startup curve angle calculation process.
*
* @param[in] out: pointer to CURVE_OUT structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return none
*/
void StartUpCurve_Angle_Process(CURVE_OUT *out, FOC_VARS *focVars, STARTUP_CTRL *startUp)
{
    int16_t freqSyn;

    freqSyn = (int32_t)(out->frqRefValue) * focVars->pCfg->motorDir;     /*!< need run direction to set negative value */

    OpenLoopElecThetaCalc(freqSyn, focVars, startUp);
}

/*!
* @brief Open loop estimate frequency judgement.
*
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] pidParam: pointer to PID_CFG structure
* @return none
*/
void StartUpFreqJudge(STARTUP_CTRL *startUp, SPEED_RAMP_HANDLE *speed, MOTOR_PARAM_PU *paramPu, PID_CFG *pidParam)
{
    int16_t estFreqUpper = 0;
    int16_t estFreqLower = 0;
    int32_t tempFreq = 0;
    int16_t estFreqAbs = 0;

    if (speed->speedFbk < 0)
    {
        estFreqAbs = 0 - (speed->speedFbk);
    }
    else
    {
        estFreqAbs = speed->speedFbk;
    }
    tempFreq = (int32_t)(startUp->synSpd) * paramPu->pairsb;
    estFreqUpper = (int16_t)(tempFreq * Math_Mpy(Math_IQ(ONE_BASE_FREQ), Math_IQ(MAX_SYNC_SPEED_THRES)));
    estFreqLower = (int16_t)(tempFreq * Math_Mpy(Math_IQ(ONE_BASE_FREQ), Math_IQ(MIN_SYNC_SPEED_THRES)));
    if ((estFreqAbs > estFreqLower) && (estFreqAbs < estFreqUpper))
    {
        startUp->synCount++;
        if (startUp->synCount > startUp->synTime)
        {
            startUp->synFreReach = 1;
            if (pidParam->curPidSelfLearn == (uint16_t)DISABLE)
            {
                Acr_SetCurD_PICof(pidParam->fluxKp, pidParam->fluxKi, 0, 0, &g_acrCurIdCof);
                Acr_SetCurQ_PICof(pidParam->torqueKp, pidParam->torqueKi, 0, 0, &g_acrCurIqCof);
            }
            else
            {
            }
        }
        else
        {
            startUp->synFreReach = 0;
        }
    }
    else
    {
    }
}

/*!
* @brief StartUp control task in foc process.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Ctrl_Foc(FOC_VARS *focVars, START_UP *startUp)
{
//#if (defined PLL_ANGLE_ESTIMATION)
//    g_rotationalSpeed.estSpeed = g_asrSpeedEstPIOut.outPuFliter;
//#else
//#endif

    switch (startUp->pStCtrl->startUpFlag)
    {
    case 1:
    //Bootstrap_Charging_Process();
//    PWM_Enable();
    startUp->pStCtrl->startUpFlag = 2;
    break;

    case 2:
#if (defined FOC_PMSM)
    #if (defined PULSE_INJECTION)
    startUp->pStCtrl->startUpFlag = 3;
    #elif (defined INIT_POS_DETEC)
    startUp->pStCtrl->startUpFlag = 3;
    #endif
#elif (defined FOC_ACIM)
    startUp->pStCtrl->startUpFlag = 3;
#endif
    break;

    case 3:
    Openloop_Acceleration_Process(focVars, startUp->pStCtrl);
    break;

    case 4:
    Closedloop_Deceleration_Process(focVars, startUp);
    break;

    case 5:
    case 6:
    Closedloop_Process(focVars, startUp->pStCtrl);
    break;

#if (defined CLOSE_TO_OPEN_SWITCH)
    case 7:
    CloseToOpenloop_Process(focVars, startUp->pStCtrl);
    break;

    case 8:
    OpenToCloseloop_Process(focVars, startUp->pStCtrl);
    break;
#endif

    default:
    break;
    }
#if (defined FOC_PMSM)
    #if (defined ELEC_FREQUENCE_HIGH)
    Asr_EstSpeedCalc(startUp->pStCtrl->elecThetaEst, &g_rotationalSpeed);
    #endif
#else
#endif
}

/*!
* @brief StartUp control task in 1ms period.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Ctrl_1ms(FOC_VARS *focVars, START_UP *startUp)
{
    if (startUp->pStCtrl->startUpFlag == 3)
    {
        #if (defined IF_STARTUP) || (defined VF_STARTUP)
        StartUpCurve_Process(&g_startUpCurveIn, &g_startUpCurveOut, &g_startUpCurveTime, startUp);
        StartUpCurve_Angle_Process(&g_startUpCurveOut, focVars, startUp->pStCtrl);

        focVars->pCtrl->idRef = 0;
        focVars->pCtrl->iqRef = (int32_t)g_startUpCurveOut.curRefValue * pFocVarsCfg->motorDir;

        if (1 == g_startUpCurveOut.calcEndFlag)
        {
            StartUpFreqJudge(startUp->pStCtrl, &g_speedCommand, &g_motorParamPu, pPidCfg);
        }
        else
        {
        }

        Set_PID_Ref(&g_pidAsr, (int32_t)g_startUpCurveOut.frqRefValue * pFocVarsCfg->motorDir);
        #endif
    }

    else if (startUp->pStCtrl->startUpFlag == 4)
    {
    }
    else if (startUp->pStCtrl->startUpFlag == 5)
    {
        #if (defined TAIL_HEAD_WIND)
        startUp->pStCtrl->countStartUpST5++;
        if (startUp->pStCtrl->countStartUpST5 < 2)
        {
            focVars->pCtrl->idRef = 0;
            focVars->pCtrl->iqRef = 0;
        }
        else
        {
            ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, focVars->pCtrl);
            startUp->pStCtrl->countStartUpST5 = 0;
            startUp->pStCtrl->startUpFlag = 6;
        }
        #else
        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, focVars->pCtrl);
        startUp->pStCtrl->startUpFlag = 6;
        #endif
        /* update smc parameter */
        #if (defined SMC_OBSERVE)
        Smc_SetParam(pSmoObserveCfg->maxErr1, pSmoObserveCfg->kslide1, pSmoObserveCfg->kslf1, &g_smcEstimator);
        #endif
    }
    else if (startUp->pStCtrl->startUpFlag == 6)
    {
        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, focVars->pCtrl);

    #if (defined CLOSE_TO_OPEN_SWITCH)
        startUp->pStCtrl->synFreReach = 0;
        startUp->pStCtrl->synCount = 0;
        g_startUpCurveOut.calcEndFlag1 = 0;
        g_startUpCurveOut.calcEndFlag2 = 0;
        if (Math_Qabs(g_speedCommand.speedFbk) <= (defFocVars.speedMinLimit - SPEED_DELAY_ZONE))
        {
            startUp->pStCtrl->startUpFlag = 7;
            startUp->pStCfg->curveCurStart1 = Math_Qabs(g_pidAcrQ.pPidInput->refPu);
            startUp->pStCfg->curveFreqStart1 = Math_Qabs(g_pidAsr.pPidInput->fbkPu);
            startUp->pStCtrl->startUpElecTheta = startUp->pStCtrl->elecThetaEst;
            Asr_SpeedPIInit(&g_pidAsr, pPidCfg);
            g_speedCommand.speedTargetRamp = defFocVars.speedMinLimit * pFocVarsCfg->motorDir;
        }
    #endif

        #if (defined QUICK_DESATURATION)
        ASR_Quick_Desaturation(&g_pidAsr, &g_acrCurIqPIOut);
        #endif
    }
#if (defined CLOSE_TO_OPEN_SWITCH)
    else if (startUp->pStCtrl->startUpFlag == 7)
    {
        #if (defined IF_STARTUP) || (defined VF_STARTUP)
        StartUpCurve_Process1(&g_startUpCurveIn, &g_startUpCurveOut, &g_startUpCurveTime, startUp);
        StartUpCurve_Angle_Process(&g_startUpCurveOut, focVars, startUp->pStCtrl);

        focVars->pCtrl->idRef = 0;
        focVars->pCtrl->iqRef = (int32_t)g_startUpCurveOut.curRefValue * pFocVarsCfg->motorDir;

        if (Math_Qabs(g_speedCommand.speedTarget) >= (defFocVars.speedMinLimit + SPEED_DELAY_ZONE))
        {
            startUp->pStCtrl->startUpFlag = 8;
        }

        #endif
    }
    else if (startUp->pStCtrl->startUpFlag == 8)
    {
        #if (defined IF_STARTUP) || (defined VF_STARTUP)
        StartUpCurve_Process2(&g_startUpCurveIn, &g_startUpCurveOut, &g_startUpCurveTime, startUp);
        StartUpCurve_Angle_Process(&g_startUpCurveOut, focVars, startUp->pStCtrl);

        focVars->pCtrl->idRef = 0;
        focVars->pCtrl->iqRef = (int32_t)g_startUpCurveOut.curRefValue * pFocVarsCfg->motorDir;

        if (1 == g_startUpCurveOut.calcEndFlag2)
        {
            StartUpFreqJudge(startUp->pStCtrl, &g_speedCommand, &g_motorParamPu, pPidCfg);
        }
        else
        {
        }
        #endif
    }
#endif
    else
    {
    }

    return;
}

/*!
* @brief Transfer the electrical angle of the motor during the startup phase.
*
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return startUpElecTheta: electric angle of startup process
*/
uint16_t Get_Sensorless_ElecAngle(STARTUP_CTRL *startUp)
{
    return startUp->startUpElecTheta;
}

/*!
* @brief Asr coefficients adaptive calculation.
*
* @param[in] none
* @return none
*/
void Acim_AsrPiCoefUpdate(void)
{
#if (defined FOC_ACIM)
    int32_t tempAsrKp, tempAsrKi;
    int16_t temp_AsrKpSetFrq1 = Math_IQ(0.1);
    int16_t temp_AsrKpSetFrq2 = Math_IQ(0.8);

#if (defined SENSORLESS_ACIM)
    int32_t temp_AsrKp1 = Math_IQ(0.6);
    int32_t temp_AsrKi1 = Math_IQ(0.005);
    int32_t temp_AsrKp2 = Math_IQ(0.9);
    int32_t temp_AsrKi2 = Math_IQ(0.005);
#elif (defined SENSOR_ACIM)
    int32_t temp_AsrKp1 = Math_IQ(0.2);
    int32_t temp_AsrKi1 = Math_IQ(0.002);
    int32_t temp_AsrKp2 = Math_IQ(0.6);
    int32_t temp_AsrKi2 = Math_IQ(0.005);
#else
    int32_t temp_AsrKp1 = Math_IQ(0.6);
    int32_t temp_AsrKi1 = Math_IQ(0.005);
    int32_t temp_AsrKp2 = Math_IQ(0.9);
    int32_t temp_AsrKi2 = Math_IQ(0.005);
#endif

    if (g_speedCommand.speedTargetRamp < temp_AsrKpSetFrq1)
    {
        tempAsrKp = temp_AsrKp1;
        tempAsrKi = temp_AsrKi1;
    }
    else if (g_speedCommand.speedTargetRamp > temp_AsrKpSetFrq1)
    {
        tempAsrKp = temp_AsrKp2;
        tempAsrKi = temp_AsrKi2;
    }
    else
    {
        tempAsrKp = (g_speedCommand.speedTargetRamp - temp_AsrKpSetFrq2) * (temp_AsrKp2 - temp_AsrKp1) \
                    / (temp_AsrKpSetFrq2 - temp_AsrKpSetFrq1) + temp_AsrKp2;
        tempAsrKi = (g_speedCommand.speedTargetRamp - temp_AsrKpSetFrq2) * (temp_AsrKi2 - temp_AsrKi1) \
                    / (temp_AsrKpSetFrq2 - temp_AsrKpSetFrq1) + temp_AsrKi2;
    }
    Asr_SetSpeed_PICof(tempAsrKp, tempAsrKi, 0, 0, g_pidAsr.pPidCoef);
#endif
}

/*!
* @brief StartUp control task in 1ms period of ACIM SLVC.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Acim_Startup_1ms(FOC_VARS *focVars, START_UP *startUp)
{
#if (defined FOC_ACIM)
    if (startUp->pStCtrl->startUpFlag == 3)
    {
        StartUpCurve_Process(&g_startUpCurveIn, &g_startUpCurveOut, &g_startUpCurveTime, startUp);
        StartUpCurve_Angle_Process(&g_startUpCurveOut, focVars, startUp->pStCtrl);

        focVars->pCtrl->idRef = g_acimFocCtrl.mot->excitCur;

        if (1 == g_startUpCurveOut.calcEndFlag)
        {
            StartUpFreqJudge(startUp->pStCtrl, &g_speedCommand, &g_motorParamPu, pPidCfg);
        }
        else
        {
        }

        focVars->pCtrl->iqRef = 1000 * pFocVarsCfg->motorDir;

        Set_PID_Ref(&g_pidAsr, (int32_t)g_startUpCurveOut.frqRefValue * pFocVarsCfg->motorDir);

    }

    else if (startUp->pStCtrl->startUpFlag == 4)
    {
//        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, focVars->pCtrl);
    }
    else if (startUp->pStCtrl->startUpFlag == 5)
    {
        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, focVars->pCtrl);
        startUp->pStCtrl->startUpFlag = 6;
    }
    else if (startUp->pStCtrl->startUpFlag >= 6)
    {
        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, focVars->pCtrl);
    }
    else
    {
    }

    return;
#endif
}

/* =============================================  EOF  ============================================== */
