/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file vf_control.c
*
* @brief This file provides Vf control functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "adc_hw.h"
#include "foc_keys.h"
#include "foc_app.h"
#include "foc_isr.h"
#include "globalvar.h"
#include "protector.h"
#include "motorctrl_api.h"
#include "vf_control.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
VF_CURVE_INPUT_PARAM        g_vfCurveInputParam;    /*!< vf curve parameters */
VF_DC_STARTUP               g_vfDcStartUp;          /*!< vf control parameters to DC start-up, PMSM */
VF_COMP_PARAM               g_vfCompParam;          /*!< vf control swing suppression parameters */
VF_COMP_SET                 g_vfCompSet;            /*!< vf control compensate setting value */
VF_VECTOR_PARAM             g_vfVect;               /*!< vf voltage and current component */
VF_INPUT                    g_vfInput;              /*!< vf control input: frequency set value */
VF_OUTPUT                   g_vfOutput;             /*!< vf control output: voltage and elec-theta, synchronous frequency; */
VF_THETA_CAL                g_vfThetaCal;           /*!< vf control parameters to calculate elec-theta */

VF_FREQ_CALC_BUF            g_vfFreqClac;           /*!< vf control frequency calculation buf struct */
VF_FREQ_HOP_PARAM           g_vfFreqHopParam;       /*!< vf control frequency hopping parameters struct */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Vf control beeline characteristic curve.
*
* @param[in] targetFreqPu: cueernt frequency command value
* @param[in] maxFreqPu: current max frequency
* @param[in] input: pointer to VFCURVE_INPUT_PARAM structure
* @return outputVoltPu
*/
uint16_t Vf_BeelineCurve(int16_t targetFreqPu, uint16_t maxFreqPu, VF_CURVE_INPUT_PARAM *input)
{
    uint16_t freqCommand = 0;
    uint16_t freqUpperLimitPu = 0;
    uint16_t freqCutoffPu = 0;
    uint16_t voltCutoffPu = 0;
    uint16_t outputVoltLowerLimit = 0;
    uint16_t outputVoltAddVoluePu = 0;
    uint16_t outputVoltPu = 0;

    freqCommand = Math_Qabs(targetFreqPu);
    freqUpperLimitPu = Math_DivShift((uint32_t)(input->motorFreqRatedPu) * maxFreqPu, input->motorMaxFreqPu, 0);
    freqCutoffPu = Math_DivShift((uint32_t)(input->torqBoostCutoffFreqPu) * maxFreqPu, input->motorMaxFreqPu, 0);
    outputVoltLowerLimit = (uint16_t)(Math_DivShift((uint32_t)(input->torqBoostVoltPct) * input->motorVoltRatedPu, 100, 0));
    voltCutoffPu = Math_DivShift(freqCutoffPu * input->motorVoltRatedPu, freqUpperLimitPu, 0);
    if (voltCutoffPu <= outputVoltLowerLimit)
    {
        voltCutoffPu = outputVoltLowerLimit;
    }
    if (freqCommand <= freqCutoffPu)
    {
        outputVoltAddVoluePu = (uint32_t)outputVoltLowerLimit - Math_DivShift(freqCommand * outputVoltLowerLimit, freqCutoffPu, 0);
        outputVoltPu = Math_DivShift((uint32_t)freqCommand * voltCutoffPu, freqCutoffPu, 0) + outputVoltAddVoluePu;
    }
    else if ((freqCommand > freqCutoffPu) && (freqCommand <= freqUpperLimitPu))
    {
        outputVoltPu = Math_DivShift((uint32_t)freqCommand * input->motorVoltRatedPu, freqUpperLimitPu, 0);
    }
    else
    {
        outputVoltPu = input->motorVoltRatedPu;
    }

    return outputVoltPu;
}

/*!
* @brief Vf control squre characteristic curve.
*
* @param[in] targetFreqPu: cueernt frequency command value
* @param[in] maxFreqPu: current max frequency
* @param[in] input: pointer to VFCURVE_INPUT_PARAM structure
* @return outputVoltPu
*/
uint16_t Vf_SqureCurve(int16_t targetFreqPu, uint16_t maxFreqPu, VF_CURVE_INPUT_PARAM *input)
{
    uint16_t freqCommand = 0;
    uint16_t freqUpperLimitPu = 0;
    uint16_t freqCutoffPu = 0;
    uint16_t voltCutoffPu = 0;
    uint16_t outputVoltLowerLimit = 0;
    uint16_t outputVoltAddVoluePu = 0;
    uint16_t outputVoltPu = 0;
    uint16_t tempSqureVar1 = 0;
    uint16_t tempSqureVar2 = 0;
    uint16_t tempSqureVar3 = 0;
    uint16_t tempSqureVar4 = 0;

    freqCommand = Math_Qabs(targetFreqPu);
    freqUpperLimitPu = Math_DivShift((uint32_t)(input->motorFreqRatedPu) * maxFreqPu, input->motorMaxFreqPu, 0);
    freqCutoffPu = Math_DivShift((uint32_t)(input->torqBoostCutoffFreqPu) * maxFreqPu, input->motorMaxFreqPu, 0);
    outputVoltLowerLimit = (uint16_t)(Math_DivShift((uint32_t)(input->torqBoostVoltPct) * input->motorVoltRatedPu, 100, 0));
    voltCutoffPu = Math_DivShift(freqCutoffPu * input->motorVoltRatedPu, freqUpperLimitPu, 0);
    if (voltCutoffPu <= outputVoltLowerLimit)
    {
        voltCutoffPu = outputVoltLowerLimit;
    }
    tempSqureVar1 = Math_DivShift(((uint32_t)freqCommand << 15), freqCutoffPu, 0);
    tempSqureVar2 = Math_DivShift(((uint32_t)freqCommand << 15), freqUpperLimitPu, 0);
    tempSqureVar3 = (uint32_t)tempSqureVar1 * outputVoltLowerLimit >> 15;
    tempSqureVar4 = (uint32_t)tempSqureVar2 * input->motorVoltRatedPu >> 15;

    if (freqCommand <= freqCutoffPu)
    {
        outputVoltAddVoluePu = ((uint32_t)outputVoltLowerLimit << 15) - (uint32_t)tempSqureVar1 * tempSqureVar3;
        outputVoltPu = ((uint32_t)tempSqureVar2 * tempSqureVar4 + outputVoltAddVoluePu) >> 15;
    }
    else if ((freqCommand > freqCutoffPu) && (freqCommand <= freqUpperLimitPu))
    {
        outputVoltPu = (uint32_t)tempSqureVar2 * tempSqureVar4 >> 15;
    }
    else
    {
        outputVoltPu = input->motorVoltRatedPu;
    }

    return outputVoltPu;
}

/*!
* @brief Vf control customer characteristic curve.
*
* @param[in] targetFreqPu: cueernt frequency command value
* @param[in] maxFreqPu: current max frequency
* @param[in] input: pointer to VFCURVE_INPUT_PARAM structure
* @return outputVoltPu
*/
uint16_t Vf_CustomerCurve(int16_t targetFreqPu, uint16_t maxFreqPu, VF_CURVE_INPUT_PARAM *input)
{
    uint16_t freqCommand = 0;
    uint16_t freqUpperLimitPu = 0;
    uint16_t outputVoltLowerLimit = 0;
    uint16_t outputVoltPu = 0;
    uint16_t tempFreq1Pu = 0;
    uint16_t tempVolt1Pu = 0;
    uint16_t tempFreq2Pu = 0;
    uint16_t tempVolt2Pu = 0;
    uint16_t tempFreq3Pu = 0;
    uint16_t tempVolt3Pu = 0;

    freqCommand = Math_Qabs(targetFreqPu);
    freqUpperLimitPu = input->motorMaxFreqPu;

    outputVoltLowerLimit = (uint16_t)(Math_DivShift((uint32_t)(input->torqBoostVoltPct) * input->motorVoltRatedPu, 100, 0));
    tempFreq1Pu = Math_DivShift((uint32_t)input->vfManualPoint1FreqPu * maxFreqPu, input->motorMaxFreqPu, 0);
    tempFreq2Pu = Math_DivShift((uint32_t)input->vfManualPoint2FreqPu * maxFreqPu, input->motorMaxFreqPu, 0);
    tempFreq3Pu = Math_DivShift((uint32_t)input->vfManualPoint3FreqPu * maxFreqPu, input->motorMaxFreqPu, 0);
    tempVolt1Pu = (uint16_t)(Math_DivShift((uint32_t)input->vfManualPoint1VoltPct * input->motorVoltRatedPu, 100, 0));
    tempVolt2Pu = (uint16_t)(Math_DivShift((uint32_t)input->vfManualPoint2VoltPct * input->motorVoltRatedPu, 100, 0));
    tempVolt3Pu = (uint16_t)(Math_DivShift((uint32_t)input->vfManualPoint3VoltPct * input->motorVoltRatedPu, 100, 0));

    if (freqCommand >= tempFreq1Pu)
    {
        if (freqCommand < tempFreq2Pu)
        {
            outputVoltPu = Math_DivShift((uint32_t)(freqCommand - tempFreq1Pu) * (tempVolt2Pu - tempVolt1Pu), (tempFreq2Pu - tempFreq1Pu), 0) + tempVolt1Pu;
        }
        else if (freqCommand < tempFreq3Pu)
        {
            outputVoltPu = Math_DivShift((uint32_t)(freqCommand - tempFreq2Pu) * (tempVolt3Pu - tempVolt2Pu), (tempFreq3Pu - tempFreq2Pu), 0) + tempVolt2Pu;
        }
        else
        {
            outputVoltPu = Math_DivShift((uint32_t)(freqCommand - tempFreq3Pu) * (input->motorVoltRatedPu - tempVolt3Pu), (freqUpperLimitPu - tempFreq3Pu), 0) + tempVolt3Pu;
        }
    }
    else
    {
        outputVoltPu = Math_DivShift(((uint32_t)(tempFreq1Pu - freqCommand) * outputVoltLowerLimit + (uint32_t)freqCommand * tempVolt1Pu), tempFreq1Pu, 0);
    }

    return outputVoltPu;
}

/*!
* @brief Vf curve output.
*
* @param[in] vfCurveSel: vf curve type
* @param[in] currentMaxFreqPu: current max frequency
* @param[in] in: pointer to VF_INPUT structure
* @param[in] out: pointer to VF_OUTPUT structure
* @param[in] curve: pointer to VF_CURVE_INPUT_PARAM structure
* @return none
*/
void Vf_CharacteristicCurve(int16_t vfCurveSel, uint16_t currentMaxFreqPu, VF_INPUT *in, VF_OUTPUT *out, VF_CURVE_INPUT_PARAM *curve)
{
    switch (vfCurveSel)
    {
        case VF_BEELINE:
        out->vfOutputVoltPu = Vf_BeelineCurve(in->vfCtrlFrqCurValPu, currentMaxFreqPu, curve);
        break;

        case VF_SQURELINE:
        out->vfOutputVoltPu = Vf_SqureCurve(in->vfCtrlFrqCurValPu, currentMaxFreqPu, curve);
        break;

        case VF_USERMANUAL:
        out->vfOutputVoltPu = Vf_CustomerCurve(in->vfCtrlFrqCurValPu, currentMaxFreqPu, curve);
        break;

        default:
        break;
    }
}

/*!
* @brief Vf control calculation of electric angle.
*
* @param[in] freqPu: cueernt frequency command value
* @param[in] freqBase: frequency base value
* @param[in] operatingFreq: execution frequency of the function
* @param[in] ang: pointer to VF_THETA_CAL structure
* @return none
*/
void Vf_ElecThetaIntegCalc(int16_t freqPu, uint16_t freqBase, uint16_t operatingFreq, VF_THETA_CAL *ang)
{
    uint16_t deltaTheta = 0;
    uint16_t freqAbs = 0;
    int16_t motorDir = 0;

    freqAbs = (uint16_t)(Math_Qabs(freqPu));
    if (freqPu >= 0)
    {
        motorDir = 1;
    }
    else
    {
        motorDir = -1;
    }

    deltaTheta = (uint16_t)(Math_DivShift((((uint32_t)freqAbs * freqBase) << 1), operatingFreq, 0));
    if (1 == motorDir)
    {
        ang->elecTheta += deltaTheta;
    }
    else if (-1 == motorDir)
    {
        ang->elecTheta -= deltaTheta;
    }
    else
    {
    }
}

/*!
* @brief Vf control calculate SVPWM voltage component.
*
* @param[in] in: pointer to VF_OUTPUT structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Vf_CalVoltVect(VF_OUTPUT *in, FOC_VARS_CTRL *focVars)
{
    int32_t pwmAlfaPu = 0;
    int32_t pwmBetaPu = 0;

    pwmAlfaPu = Math_Mpy(in->vfOutputVoltPu, Math_Cos(in->phaseAngleUa));
    pwmBetaPu = Math_Mpy(in->vfOutputVoltPu, Math_Sin(in->phaseAngleUa));

    pwmAlfaPu = Math_DivShift((int32_t)pwmAlfaPu << 15, g_adcTransform.vdcPu, 0);
    pwmBetaPu = Math_DivShift((int32_t)pwmBetaPu << 15, g_adcTransform.vdcPu, 0);

    focVars->valpha = Math_Sat(pwmAlfaPu, 32767, -32768);
    focVars->vbeta = Math_Sat(pwmBetaPu, 32767, -32768);
}

/*!
* @brief Vf control dc startup.
*
* @param[in] dc: pointer to VF_DC_STARTUP structure
* @param[in] out: pointer to VF_OUTPUT structure
* @return none
*/
void Vf_DCStartUp(VF_DC_STARTUP *dc, VF_OUTPUT *out)
{
    if (START == Get_McStatus())
    {
        dc->vfStartUpStatus = INIT_UNKNOWN;
        dc->vfStartUpCnt++;
        out->vfOutputVoltPu = Math_Mpy(VF_DC_VOLTAGE, g_vfCurveInputParam.motorVoltRatedPu);
        out->phaseAngleUa = 0;
        g_vfThetaCal.elecTheta = 0;
        if (VF_DC_TIME == dc->vfStartUpCnt)
        {
            dc->vfStartUpCnt = 0;
            dc->vfStartUpStatus = EXE_SUCCESS;
        }
        else
        {
        }
    }
    else
    {

    }
}

/*!
* @brief Vf control curve parameters initialization.
*
* @param[in] curve: pointer to VF_CURVE_INPUT_PARAM structure
* @return none
*/
void Vf_CurveInputParamInit(VF_CURVE_INPUT_PARAM *curve)
{
    curve->motorFreqRatedPu = VF_MOTOR_RATED_FREQUENCY;
    /* Udc */
    curve->motorVoltRatedPu = Math_IQ(1.0);
    /* max frequency */
    curve->motorMaxFreqPu = VF_MOTOR_RATED_FREQUENCY;

    curve->torqBoostCutoffFreqPu = VF_TORQUEBOOST_CUTOFF_FREQ;
    curve->torqBoostVoltPct = VF_TORQUEBOOST_VOLTAGE_PERCENTAGE;

    curve->vfManualPoint1FreqPu = Math_Mpy(VF_CURVE_POINT1_FREQ, curve->motorFreqRatedPu);
    curve->vfManualPoint1VoltPct = VF_CURVE_POINT1_VOLT_PERCENTAGE;
    curve->vfManualPoint2FreqPu = Math_Mpy(VF_CURVE_POINT2_FREQ, curve->motorFreqRatedPu);
    curve->vfManualPoint2VoltPct = VF_CURVE_POINT2_VOLT_PERCENTAGE;
    curve->vfManualPoint3FreqPu = Math_Mpy(VF_CURVE_POINT3_FREQ, curve->motorFreqRatedPu);
    curve->vfManualPoint3VoltPct = VF_CURVE_POINT3_VOLT_PERCENTAGE;
    if (curve->vfManualPoint2FreqPu < curve->vfManualPoint1FreqPu)
    {
        curve->vfManualPoint2FreqPu = curve->vfManualPoint1FreqPu;
    }
    if (curve->vfManualPoint3FreqPu < curve->vfManualPoint2FreqPu)
    {
        curve->vfManualPoint3FreqPu = curve->vfManualPoint2FreqPu;
    }
}

/*!
* @brief Vf control filter coefficient initialization.
*
* @param[in] currentLpfCutoffFreq: current low pass filter cutoff frequency
* @param[in] activePowerLpfCutoffFreq: power low pass filter cutoff frequency
* @param[in] reactPowerLpfCutoffFreq: reactive power low pass filter cutoff frequency
* @param[in] fSample: current sampling frequency
* @param[in] param: pointer to VF_COMP_PARAM structure
* @return none
*/
void Vf_FunctionParamInit(int16_t currentLpfCutoffFreq, int16_t activePowerLpfCutoffFreq, int16_t reactPowerLpfCutoffFreq, int16_t fSample, VF_COMP_PARAM *param)
{
    float temp_coeff1;
    float temp_coeff2;
    int16_t cutoffFreq;

    cutoffFreq = currentLpfCutoffFreq;
    temp_coeff1 = PI2 * cutoffFreq;
    temp_coeff2 = temp_coeff1 + fSample;
    param->vfSuppLPFCoeff = (int32_t)(temp_coeff1 * 32768 / temp_coeff2);

    cutoffFreq = activePowerLpfCutoffFreq;
    temp_coeff1 = PI2 * cutoffFreq;
    temp_coeff2 = temp_coeff1 + fSample;
    param->powerSuppLPFCoeff = (int32_t)(temp_coeff1 * 32768 / temp_coeff2);

    cutoffFreq = reactPowerLpfCutoffFreq;
    temp_coeff1 = PI2 * cutoffFreq;
    temp_coeff2 = temp_coeff1 + fSample;
    param->vfPowerLPFCoeff = (int32_t)(temp_coeff1 * 32768 / temp_coeff2);
}

/*!
* @brief Vf control frequency compensation parameters initialization.
*
* @param[in] set: pointer to VF_COMP_SET structure
* @return none
*/
void Vf_CompParamInit(VF_COMP_SET *set)
{
    set->curResonaFreqLimit = Math_IQ(0.1 / BASE_FREQ);
    set->curResonaFreqPoint1 = Math_IQ(6.0 / BASE_FREQ);
    set->curResonaFreqPoint2 = Math_IQ(10.0 / BASE_FREQ);
    set->curResonaFreqPoint3 = Math_IQ(45.0 / BASE_FREQ);
    set->curResonaFreqPoint4 = Math_IQ(48.0 / BASE_FREQ);
    set->curResonaGain = 1;

    set->powerSuppFreqLimit = Math_IQ(0.1 / BASE_FREQ);
    set->powerSuppFreqPoint1 = Math_IQ(6.0 / BASE_FREQ);
    set->powerSuppFreqPoint2 = Math_IQ(10.0 / BASE_FREQ);
    set->powerSuppFreqPoint3 = Math_IQ(45.0 / BASE_FREQ);
    set->powerSuppFreqPoint4 = Math_IQ(48.0 / BASE_FREQ);
    set->powerSuppGain = 1;
}

/*!
* @brief Vf control frequency hopping parameters initialization.
*
* @param[in] param: pointer to VF_FREQ_HOP_PARAM structure
* @return none
*/
void Vf_FreqHopParamInit(VF_FREQ_HOP_PARAM *param)
{
    param->freqHopP1Bottom = HOP_FREQ_POINT1 - (HOP_WIDTH_1 >> 1);
    param->freqHopP1Top = HOP_FREQ_POINT1 + (HOP_WIDTH_1 >> 1);
    param->freqHopP2Bottom = HOP_FREQ_POINT2 - (HOP_WIDTH_2 >> 1);
    param->freqHopP2Top = HOP_FREQ_POINT2 + (HOP_WIDTH_2 >> 1);
    param->freqHopP3Bottom = HOP_FREQ_POINT3 - (HOP_WIDTH_3 >> 1);
    param->freqHopP3Top = HOP_FREQ_POINT3 + (HOP_WIDTH_3 >> 1);
    param->freqHopFlag = 0;
    param->speedStatus = STABLE;
    param->freqHopOutput = 0;
}

/*!
* @brief Vf control Parameters initialization.
*
* @param[in] none
* @return none
*/
void Vf_ParamInit(void)
{
    int16_t currentLpfCutoffFreq = 0;
    int16_t activePowerLpfCutoffFreq = 0;
    int16_t reactPowerLpfCutoffFreq = 0;

    STC_ParamInit(&g_STC);
    FOC_CtrlVarsInit(&g_focVarsCtrl);
    Clear_CurrentSample(&g_adcTransform);

    memset(&g_pwmValue, 0U, sizeof(g_pwmValue));
    memset(&g_curRestruct, 0U, sizeof(g_curRestruct));

    memset(&g_vfDcStartUp, 0, sizeof(g_vfDcStartUp));
    memset(&g_vfInput, 0, sizeof(g_vfInput));
    memset(&g_vfOutput, 0, sizeof(g_vfOutput));
    memset(&g_vfCompParam, 0, sizeof(g_vfCompParam));
    memset(&g_vfVect, 0, sizeof(g_vfVect));
    memset(&g_vfThetaCal, 0, sizeof(g_vfThetaCal));

    memset(&g_vfFreqClac, 0, sizeof(g_vfFreqClac));
    /* Motor protection */
    Init_Protection();

    g_speedCommand.speedTarget = VF_DEFAULT_TARGET_FREQ;

    Vf_CurveInputParamInit(&g_vfCurveInputParam);
    currentLpfCutoffFreq = Math_IQ(10.0 / BASE_FREQ);
    activePowerLpfCutoffFreq = Math_IQ(10.0 / BASE_FREQ);
    reactPowerLpfCutoffFreq = Math_IQ(10.0 / BASE_FREQ);
    Vf_FunctionParamInit(currentLpfCutoffFreq, activePowerLpfCutoffFreq, reactPowerLpfCutoffFreq, FOC_FREQ, &g_vfCompParam);
    Vf_CompParamInit(&g_vfCompSet);
    Vf_FreqHopParamInit(&g_vfFreqHopParam);
}

/*!
* @brief Vf control synchronous frequency calculation.
*
* @param[in] in: pointer to VF_FREQ_CALC_BUF structure
* @return none
*/
void Vf_SynFreqCalc(VF_FREQ_CALC_BUF *in)
{
    static uint16_t s_freqCmdCnt = 0;

    if (in->vfCtrlSpeedSetPu > VF_CURVE_MAX_FREQUENCY)
    {
        in->vfCtrlSpeedSetPu = VF_CURVE_MAX_FREQUENCY;
    }
    else if (in->vfCtrlSpeedSetPu < 0 - VF_CURVE_MAX_FREQUENCY)
    {
        in->vfCtrlSpeedSetPu = 0 - VF_CURVE_MAX_FREQUENCY;
    }
    else
    {

    }
    s_freqCmdCnt++;
    if (s_freqCmdCnt > 100)
    {
        s_freqCmdCnt = 0;
        if (1 == pFocVarsCfg->motorDir)
        {
            if ((in->vfCtrlSpeedSetPu - in->vfCtrlFrqCurValPu) > defFocVars.motorAcceleration)
            {
                in->vfCtrlFrqCurValPu += defFocVars.motorAcceleration;
            }
            else if ((in->vfCtrlFrqCurValPu - in->vfCtrlSpeedSetPu) > defFocVars.motorDeceleration)
            {
                in->vfCtrlFrqCurValPu -= defFocVars.motorDeceleration;
            }
            else
            {
                in->vfCtrlFrqCurValPu = in->vfCtrlSpeedSetPu;
            }
        }
        else if (-1 == pFocVarsCfg->motorDir)
        {
            if ((in->vfCtrlFrqCurValPu - in->vfCtrlSpeedSetPu) > defFocVars.motorAcceleration)
            {
                in->vfCtrlFrqCurValPu -= defFocVars.motorAcceleration;
            }
            else if ((in->vfCtrlSpeedSetPu - in->vfCtrlFrqCurValPu) > defFocVars.motorDeceleration)
            {
                in->vfCtrlFrqCurValPu += defFocVars.motorDeceleration;
            }
            else
            {
                in->vfCtrlFrqCurValPu = in->vfCtrlSpeedSetPu;
            }
        }
        else
        {
        }
    }
}

/*!
* @brief Vf control main functions.
*
* @param[in] none
* @return none
*/
void Vf_MainProcess(void)
{
    /* get current frequency command */
    g_vfInput.vfCtrlSpeedSetPu = g_speedCommand.speedTarget;
    g_vfFreqClac.vfCtrlSpeedSetPu = g_vfInput.vfCtrlSpeedSetPu;
    /* calculate real-time frequency value */
    Vf_SynFreqCalc(&g_vfFreqClac);
    /* hopFreq */
    #if (defined FREQ_HOPPING_ENABLE)
    /* calculate motion direction */
    RampDirJudgeProcess(g_vfInput.vfCtrlSpeedSetPu, g_vfFreqClac.vfCtrlFrqCurValPu, defFocVars.motorAcceleration, defFocVars.motorDeceleration, &g_vfFreqHopParam);
    /* frequency hopping */
    FrequencyHoppingProcess(&g_vfFreqClac, &g_vfFreqHopParam);
    g_vfInput.vfCtrlFrqCurValPu = g_vfFreqHopParam.freqHopOutput;
    #else
    g_vfInput.vfCtrlFrqCurValPu = g_vfFreqClac.vfCtrlFrqCurValPu;
    #endif

    g_vfInput.vfCtrlFrqBeforeComp = g_vfInput.vfCtrlFrqCurValPu;

#if (defined VF_COMPENSATION)
    #if (defined VF_CURRENT_DAMP_ENABLE)
    /* frequency compensation with current */
    Vf_CurrentResonSuppProcess(g_vfVect.curQAxisPu, &g_vfInput, &g_vfCompParam, &g_vfCompSet);
    #else
    g_vfCompParam.vfSuppFrqComp = 0;
    #endif
    g_vfInput.vfCtrlFrqCompBuf = g_vfInput.vfCtrlFrqBeforeComp - g_vfCompParam.vfSuppFrqComp;

    #if (defined VF_ACTIVEPOWER_COMP_ENABLE)
    /* frequency compensation with power */
    Vf_ActivePowerCompProcess(&g_vfInput, &g_vfVect, &g_vfCompParam, &g_vfCompSet, g_focVars.pCfg->motorDir);
    #else
    g_vfCompParam.vfPowerSuppFreqComp = 0;
    #endif
    g_vfInput.vfCtrlFrqCompBuf = g_vfInput.vfCtrlFrqCompBuf + g_vfCompParam.vfPowerSuppFreqComp;
#else
    g_vfInput.vfCtrlFrqCompBuf = g_vfInput.vfCtrlFrqBeforeComp;
#endif
    g_vfInput.vfCtrlFrqAfterComp = g_vfInput.vfCtrlFrqCompBuf;
}

/*!
* @brief Vf control SateMachine main function.
*
* @param[in] none
* @return none
*/
void Vf_MainStateMachine(void)
{
    switch (Get_McStatus())
    {
    case POWER_ON:
    Stop_Motor();
    Vf_ParamInit();
    /* must behind adc init, check current sampling offset*/
    Current_Sensor_Check(&g_adcSampleReal);
    Set_McStatus(IDLE);
    break;

    case IDLE:
    Vf_ParamInit();
    break;

    case STOP:
    Stop_Motor();
    Vf_ParamInit();
    Set_McStatus(IDLE);
    break;

    case START_BRAKE:
    ShortBrake_RunCaseBrake(&g_shortBrake, pShortBrakeCfg, pFocVarsCfg);
    if (CONFIG_RESET == g_shortBrake.subStatus)
    {
        PWM_Disable();
        ShortBrake_Init(&g_shortBrake);
        Set_McStatus(START);
    }
    break;

    case START:
    PWM_Enable();
    g_focVarsCtrl.pwmOutMask = SVPWM_OUTPUT_ENABLE;
    Vf_DCStartUp(&g_vfDcStartUp, &g_vfOutput);
    if (EXE_SUCCESS == g_vfDcStartUp.vfStartUpStatus)
    {
        Set_McStatus(RUN);
    }
    break;

    case RUN:
    Vf_MainProcess();
    break;

    case SLOW_DOWN:
    g_speedCommand.speedTarget = 0;
    Vf_MainProcess();
    if (Math_Qabs(g_vfOutput.synFrq) < VF_STOP_FREQ)
    {
        Set_McStatus(STOP);
        PWM_Disable();
    }
    break;

    /* Stop brake, Short circuit brake */
    case STOP_BRAKE:
    ShortBrake_RunCaseBrake(&g_shortBrake, pShortBrakeCfg, pFocVarsCfg);
    if (CONFIG_RESET == g_shortBrake.subStatus)
    {
        PWM_Disable();
        ShortBrake_Init(&g_shortBrake);
        Set_McStatus(STOP);
    }
    break;

    case FAULT:
    Stop_Motor();
    break;

    default:
    break;
    }
}

/*!
* @brief Vf control ISR process.
*
* @param[in] none
* @return none
*/
void Vf_ISR(void)
{
    if (SVPWM_OUTPUT_ENABLE == g_focVarsCtrl.pwmOutMask)
    {
        /* get synchronous frequency */
        g_vfOutput.synFrq = g_vfInput.vfCtrlFrqAfterComp;

        /* get voltage vector angle */
        if ((RUN == Get_McStatus()) || (SLOW_DOWN == Get_McStatus()))
        {
            Vf_ElecThetaIntegCalc(g_vfOutput.synFrq, BASE_FREQ, FOC_FREQ, &g_vfThetaCal);
            g_vfOutput.phaseAngleUa = g_vfThetaCal.elecTheta;
        }

        /* get voltage output */
        Vf_CharacteristicCurve(VF_CURVE, VF_CURVE_MAX_FREQUENCY, &g_vfInput, &g_vfOutput, &g_vfCurveInputParam);

        /* electric angle compensation */
        #if (defined VF_COMPENSATION)
        Vf_VectorTransformProcess(&g_focVarsCtrl, &g_vfOutput, &g_vfVect);
        #else

        #endif

        /* calculate ualf and ubeta */
        Vf_CalVoltVect(&g_vfOutput, &g_focVarsCtrl);

        /* svpwm */
        Svpwm_Process();
        LoadToPwmModule();
    }
}

/*!
* @brief Vf control protector.
*
* @param[in] none
* @return none
*/
void VF_MotorProtector(void)
{
#if (defined PHASE_CURRENT_PROTECTION_ENABLE)
    /* Phase over current protection, 1ms */
    Phase_Over_Current_Check(&g_focVarsCtrl);
#endif

#if (defined VBUS_UNDERVOLTAGE_PROTECTION_ENABLE)
    /* DC Bus under voltage protection, 1ms */
    Vbus_UnderVoltage_Check(g_adcTransform.vdcVt);
#endif

#if (defined VBUS_OVERVOLTAGE_PROTECTION_ENABLE)
    /* DC Bus over voltage protection, 1ms */
    Vbus_OverVoltage_Check(g_adcTransform.vdcVt);
#endif
}

/*!
* @brief Vf control 1ms time base tasks.
*
* @param[in] none
* @return none
*/
void VF_1ms_Task(void)
{
    Vf_MainStateMachine();

    Get_VoltageSample(&g_adcSampleCof, &g_motorParamPu, &g_adcTransform);
    /* Start adc convert */
    ADC_SoftwareStartRegularConvert(ADC0);
    VF_MotorProtector();
}

/*!
* @brief Vf control 2ms time base tasks.
*
* @param[in] none
* @return none
*/
void VF_2ms_Task(void)
{
    #if (defined TOOL_UART_FUNCTION)
    /* Motor Debug Tool command Process */
    UI_UART_ToolCmd_Proc(&g_uiRxBuf);
    UI_UART_TimeOut_Cnt();
    UI_MasterCmd_ProcCnt();
    #endif
}

/* =============================================  EOF  ============================================== */
