/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __CUR_RESTRUCT_H__
#define __CUR_RESTRUCT_H__
/*!
* @file cur_restruct.h
*
* @brief  This file provides pwm shift pwm compare value and restruct three phase current base on bus current interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "svgen.h"
#include "param_init.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Current restruct function data structure to save phase current and pwm shift compare value
*/
typedef struct
{
    uint16_t sector;         /*!< Pwm sector */
    uint16_t sectorBuf;      /*!< Pwm sector buf*/
    int16_t AdIaVal;         /*!< Ia adc value*/
    int16_t AdIbVal;         /*!< Ib adc value*/
    int16_t AdIcVal;         /*!< Ic adc value*/

    uint16_t maxMidTrig;     /*!< Betwen max phase and mid phase trigger sample point*/
    uint16_t midMinTrig;     /*!< Betwen mid phase and min phase trigger sample point*/

    uint16_t pwmTaUp;        /*!< Pwm phase a upper Ts compare value*/
    uint16_t pwmTaDown;      /*!< Pwm phase a downer Ts compare value*/
    uint16_t pwmTbUp;        /*!< Pwm phase b upper Ts compare value*/
    uint16_t pwmTbDown;      /*!< Pwm phase b downer Ts compare value*/
    uint16_t pwmTcUp;        /*!< Pwm phase c upper Ts compare value*/
    uint16_t pwmTcDown;      /*!< Pwm phase c downer Ts compare value*/
    uint16_t nonSampArea;    /*!< Non sample area flag*/
} CUR_RESTRUCT_DATA;

/*!
* @brief Pwm shift sub-function data structure.
*/
typedef struct
{
    uint16_t pwmMaxUp;       /*!< Pwm phase max upper Ts compare value*/
    uint16_t pwmMaxDown;     /*!< Pwm phase max downer Ts compare value*/
    uint16_t pwmMidUp;       /*!< Pwm phase mid upper Ts compare value*/
    uint16_t pwmMidDown;     /*!< Pwm phase mid downer Ts compare value*/
    uint16_t pwmMinUp;       /*!< Pwm phase min upper Ts compare value*/
    uint16_t pwmMinDown;     /*!< Pwm phase min downer Ts compare value*/
} PWM_SHIFT_DATA;

/*!
* @brief Current restruct function configure structure, contain the whole least sample window length and ADC sample time.
*/
typedef struct
{
    uint16_t sampWinLength;  /*!< Sample window length */
    uint16_t adSampTim;      /*!< Ad sample cost time */
    uint16_t sampTrigEdge;   /*!< Sample trig edge, 0 means Up edge sample, 1 means Down edge sample*/
} CUR_RESTRUCT_CFG;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Pwm shift function, non-sample zone can't restruct phase current.
*        So shift pwm to make active voltage vector longer.
*        Base on different sector, call CurRestruct_PwmShiftSub function to calculate pwm shift time.
*        Especially, if the active voltage vector length more than min sample window, the shift time will be zero.
*
* @param[in] svpwmVal: SVPWM output value, pointer to PWM_GEN_OUT structure
* @param[in] curRestruct: curRestruct variable, pointer to CUR_RESTRUCT_DATA structure
* @param[in] pwmShift: pwmShift variable, pointer to PWM_SHIFT_DATA structure
* @param[in] curRestructCfg: current restruct config variable, pointer to CUR_RESTRUCT_CFG structure
* @param[in] focVarsCfg: variable which use to configure FOC control, pointer to FOC_VARS_CFG structure
* @return none
*/
void CurRestruct_PwmShift(PWM_GEN_OUT *svpwmVal, CUR_RESTRUCT_DATA *curRestruct, PWM_SHIFT_DATA *pwmShift, CUR_RESTRUCT_CFG *curRestructCfg, FOC_VARS_CFG *focVarsCfg);

/*!
* @brief This function base on bus current sample, restruct three phase current.
*        At different sector, the bus current sample value corresponding different phase current.
*        Every pwm period can get two phase current, and base on ia + ib + ic = 0, calculate the third phase current.
*
* @param[in] iBusMidMin: At second active voltage vector, bus current value
* @param[in] iBusMaxMid: At first active voltage vector, bus current value
* @param[in] curRestruct: curRestruct variable, pointer to CUR_RESTRUCT_DATA structure
* @param[in] curRestructCfg: current restruct config variable, pointer to CUR_RESTRUCT_CFG structure
* @return none
*/
void CurRestruct_IaIbIc(int16_t iBusMidMin, int16_t iBusMaxMid, CUR_RESTRUCT_DATA *curRestruct, CUR_RESTRUCT_CFG *curRestructCfg);

/* ======================================  Functions define  ======================================== */

#endif /* __CUR_RESTRUCT_H__ */

/* =============================================  EOF  ============================================== */

