/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __FLUX_WEAKENING_H__
#define __FLUX_WEAKENING_H__

/*!
* @file flux_weakening.h
*
* @brief This file provides the d-q axis reference current by volatge loop regular or look-up table functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pid_regulator.h"

/* ============================================  Define  ============================================ */
#define TABLE_SIZE_MAX  20          /*!< The maximum size of table in flux weakening look up method */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Flux weakening look up table parameters structure.
*/
typedef struct
{
    uint16_t size;                      /*!< Look up table size */
    uint16_t tableX[TABLE_SIZE_MAX];    /*!< Look up table x axis */
    uint16_t tableY[TABLE_SIZE_MAX];    /*!< Look up table y axis */
} FLUXWEAK_TABLE;

/*!
* @brief Flux weakening voltage colsed loop control parameters structure.
*/
typedef struct
{
    int32_t vdq;                        /*!< Average stator voltage amplitude */
    uint16_t voltageRef;                /*!< Voltage reference */
    uint16_t maxModule;                 /*!< Circle limitation maximum allowed module */
    uint16_t idRefOffset;               /*!< Id reference offset */
    int16_t demagCur;                   /*!< Demagnetization current */
    int16_t nominalCur;                 /*!< motor nominal current */
} FLUXWEAK_PARAM;

/*!
* @brief Flux weakening voltage colsed loop configuration parameters structure.
*/
typedef struct
{
    uint16_t voltageRef;                /*!< Voltage reference */
    int16_t kp;                         /*!< Flux weakening pid kp */
    int16_t ki;                         /*!< Flux weakening pid ki */
    int16_t min;                        /*!< Flux weakening pid input min limit */
    int16_t maxModule;                  /*!< Circle limitation maximum allowed module */
    int16_t demagCur;                   /*!< Demagnetization current */
    int16_t nominalCur;                 /*!< motor nominal current */
} FLUXWEAK_CFG;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Flux weakening control parameters initialize, including pid parameters, maximum modulation, demage current, nominal
*        current and reference voltage initialize, usually it is initialized only once before running.
*
* @param[in] pidFw: pointer to PID_REGULATOR structure
* @param[in] fwParam: pointer to FLUXWEAK_PARAM structure
* @param[in] fwCfg: pointer to FLUXWEAK_CFG structure
* @return none
*/
void Fluxweak_Init(PID_REGULATOR *pidFw, FLUXWEAK_PARAM *fwParam, FLUXWEAK_CFG *fwCfg);

/*!
* @brief Flux weakening control reference id iq current calculate, through voltage closed loop to get the d axis given current,
*        then calculate the limit value of the given current of the q axis by the maximum current.
*
* @param[in] pHandle: pointer to FLUXWEAK_PARAM structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] pidFw: pointer to PID_REGULATOR structure
* @param[in] asrOut: pointer to PID_REGULATOR_OUT structure
* @return none
*/
void Fluxweak_CurRefCalc(FLUXWEAK_PARAM *pHandle, FOC_VARS_CTRL *focVars, PID_REGULATOR *pidFw, PID_REGULATOR_OUT *asrOut);

/*!
* @brief Flux weakening control reference id iq current calculate by look-up table.
*
* @param[in] speed: motor speed pu value in Q15
* @param[in] torque: motor given torque pu value in Q15
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] fwParam: pointer to FLUXWEAK_PARAM structure
* @return none
*/
void Fluxweak_CurRef_Lookup(int16_t speed, int32_t torque, FOC_VARS_CTRL *focVars, FLUXWEAK_PARAM *fwParam);

/* ======================================  Functions define  ======================================== */

#endif /* __FLUX_WEAKENING_H__ */

/* =============================================  EOF  ============================================== */
