/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __HALL_SENSOR_H__
#define __HALL_SENSOR_H__
/*!
* @file hall_sensor.h
*
* @brief  This file provides hall eletric theta  and hall self-learn functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pwm_input.h"
#include "param_init.h"
#include "pid_regulator.h"

/* ============================================  Define  ============================================ */
#define ZERO_ORDER_FORCE_COMP           (0)                             /*!<  define force degree compensation mode */
#define ZERO_ORDER_DISPERSE_COMP        (1)                             /*!< define dispere degree compensation mode */

#define PHASE_SHIFT_05                  (PHASE_SHIFT_1 >> 1)            /*!< define 0.5 degree in Q16 format */
#define PHASE_SHIFT_1                   (uint16_t)(65536 * 1 / 360)     /*!< define 1 degree in Q16 format */
#define PHASE_SHIFT_2                   (uint16_t)(65536 * 2 / 360)     /*!< define 2 degree in Q16 format */
#define PHASE_SHIFT_5                   (uint16_t)(65536 * 5 / 360)     /*!< define 5 degree in Q16 format */
#define PHASE_SHIFT_10                  (uint16_t)(65536 * 10 / 360)    /*!< define 10 degree in Q16 format */
#define PHASE_SHIFT_30                  (uint16_t)(65536 * 30 / 360)    /*!< define 30 degree in Q16 format */
#define PHASE_SHIFT_60                  (uint16_t)(65536 * 60 / 360)    /*!< define 60 degree in Q16 format */
#define PHASE_SHIFT_90                  (uint16_t)(65536 * 90 / 360)    /*!< define 90 degree in Q16 format */

#define CLOCKWISE                       (1)                             /*!< define motor run at clockwise direction */
#define ANTI_CLOCKWISE                  (-1)                            /*!< define motor run at anti-clockwise direction */

#define SELF_LEARN_START                (0)                             /*!< define hall self-learn zero stage*/
#define SELF_LEARN_10DEGREE_FINISHED    (1)                             /*!< define hall self-learn first stage*/
#define SELF_LEARN_2DEGREE_FINISHED     (2)                             /*!< define hall self-learn second stage*/
#define SELF_LEARN_HALL_WIDTH           (3)                             /*!< define hall self-learn third stage*/

#define IF_RUN_FREQ                     (20ul)                          /*!<  define hall sector width self-learn IF run frequency*/

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Hall sensor self learn data
*/
typedef struct
{
    uint8_t zeroPosHallStatus;      /*!< Hall Zero position */
    uint8_t selfLearnStage;         /*!< Hall learning stage */
    uint8_t selfLearnOrder;         /*!< Hall learning order */
    uint8_t selfLearnSubStage;      /*!< Hall learning sub-stage */
    uint8_t tenIntervalHallStatus;  /*!< Ten degree interval hall status */
    uint8_t twoIntervalHallStatus;  /*!< Two degree interval hall status */

    uint16_t selfLearnIndex;        /*!< Hall self learn data array index */
    uint16_t selfLearnTheta;        /*!< Hall learning theta */
    uint16_t selfLearnAngle;        /*!< Hall self learn angle */
    uint16_t tsCnt;                 /*!< Hall width learn counter */
    uint16_t freTarget;             /*!< Hall width learn target frequency */
    uint16_t sectorWidthTsMax;      /*!< Hall width learn ts max */
    uint16_t sectorWidthTsMin;      /*!< Hall width learn ts min */
    uint16_t freSet;                /*!< Hall width learn actual set frequency */
    uint16_t thetaDelta;            /*!< Hall width learn delta theta per ts */
    uint16_t t1msCnt;               /*!< Hall width learn 1ms counter */
    uint16_t widthLearnCnt;         /*!< Hall width learn counter */
    uint16_t setThetaAngle;         /*!< Hall sensor self-learning output angle */
    uint16_t sectorWidthLearnArray[6];  /*!< Hall sensor width learning value buffer */
    uint16_t sectorWidthMax;        /*!< Hall sensor sector width max value */
    uint16_t sectorWidthMin;        /*!< Hall sensor sector width min value */
} HALL_LEARN_DATA;

/*!
* @brief Hall sensor configuration
*/
typedef struct
{
    uint8_t hallSelfLearnMode;      /*!< Hall self learn mode, 1: learn order and theta; 2: learn order and theta and sector width */
    uint8_t hallCalcMode;           /*!< Hall calculate rotor position mode */
    uint8_t hallSelfLearnEnable;    /*!< Hall self learn enable */
    uint8_t learnOrder;             /*!< Hall self learn finished get hall order */
    uint16_t hallSwapTrqCoff;       /*!< Hall torque boost cofficient at low speed */
    uint16_t learnIntervalTime;     /*!< Hall self learn interval time */
    uint16_t learnDegree;           /*!< Hall self learn finished get hall install error */
    uint32_t learnCur;              /*!< Hall self learn current */
} HALL_SENSOR_CFG;

/*!
* @brief Hall sensor data
*/
typedef struct
{
    uint8_t sectorWidthIndex;       /*!< Sector width array index */
    uint8_t pwdtDatValid;           /*!< pwdt isr data valid flag */
    uint8_t hallEstThetaSectorNext; /*!< Hall est status next */
    uint8_t hallEstThetaSector;     /*!< Hall est status */
    uint8_t hallEstThetaSectorBuf;  /*!< Hall est status Buf */
    uint8_t swapstate;              /*!< 0: swap mode 1:unswap mode */

    uint8_t hallStatus;             /*!< Hall status */
    int8_t rotorDirection;          /*!< Motor rotor direction 1:clockwise -1:anti-clockwise */
    int16_t estThetaStep;           /*!< Estimation rotor step */
    int16_t estThetaErr;            /*!< Estimation rotor angle error */
    int16_t estFre;                 /*!< Estimation rotor frequency */
    uint16_t estTheta;              /*!< Estimation rotor angle,Q16 format */
    uint16_t estThetaBuf;           /*!< Estimation rotor angle buff,Q16 format */
    uint16_t estThetaCompCnt;       /*!< Decentralized theta compensation Ts counter */
    uint16_t estThetaCompCntBuf;    /*!< Decentralized theta compensation Ts counter buff */
    uint16_t maxEstTheta;           /*!< Max estimation theta */
    uint16_t minEstTheta;           /*!< Min estimation theta */

    uint32_t freCalCof;             /*!< Frequency calculate coff */
} HALL_SENSOR_DATA;

/*!
* @brief Hall sensor data assemble
*/
typedef struct
{
    HALL_SENSOR_CFG         *pHallCfg;                 /*!< Hall sensor configuration parameters */
    HALL_SENSOR_DATA        *pHallSensor;              /*!< Hall sensor calculate rotor position */
    HALL_LEARN_DATA         *pHallSelfLearn;           /*!< Hall sensor self learn data */
} HALL_SENSOR;

/* ==========================================  Variables  =========================================== */
extern uint16_t g_hallSectorDegree[8];                  /*!< Six sector boundary degree */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Hall sensor initialize function, this function initilize rotor position calculate data struct and hall install self-learn struct.
*        This function will be called at main status at STOP, and Hall_module overflow interrupt.
*        If the motor have not self-learn, then this function will clear hall install imformation.
*        If the motor have been self-learn, the this function will use configuration imformation to calculate rotor position.
*
* @param[in] hallStatus: Hall module output hall status
* @param[in] hall: variable which use in hall sensor control, pointer to HALL_SENSOR structure
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void HallSensor_Init(uint8_t hallStatus, HALL_SENSOR *hall, FOC_VARS *focVars);

/*!
* @brief Hall sensor Hall_module interrupt service function, which is mainly calculate motor frequency.
*        When the three hall signals change, what ever rising edge or falling edge, will trriger Hall_module generate interrupt.
*        Base on interrupts interval time, calculate motor frequency.
*
* @param[in] hallStatus: Hall module output hall status
* @param[in] negSectorWidth: Hall module output, negtive pulse width
* @param[in] posSectorWidth: Hall module output, postive pulse width
* @param[in] hall: variable which use in hall sensor control, pointer to HALL_SENSOR structure
* @param[in] focVarsCfg: variable which use to configure FOC control, pointer to FOC_VARS_CFG structure
* @return none
*/
void HallSensor_HallSubCallBack(uint8_t hallStatus, uint32_t negSectorWidth, uint32_t posSectorWidth, HALL_SENSOR *hall, FOC_VARS_CFG *focVarsCfg);

/*!
* @brief Hall sensor ADC interrupt service function, which is mainly calculate motor rotor position.
*        This function has two methods calculate rotor position, when motor run at low speed (swap status), the rotor position is each sector middle value.
*        When motor run at middle or high speed (unswap status), the rotor positioon is the frequency integal.
*
* @param[in] hallStatus: Hall module output hall status
* @param[in] hall: variable which use in hall sensor control, pointer to HALL_SENSOR structure
* @return none
*/
void HallSensor_AdcCallBack(uint8_t hallStatus, HALL_SENSOR *hall);

/*!
* @brief Hall_module interrput service function for hall width self-learn, this function is called by Hall_module Isr function.
*        Save the interval value between two pwdt isr, then calculate the sector width.
*
* @param[in] hallStatus: Hall_module output hall status
* @param[in] negSectorWidth: Hall_module output, negtive pulse width
* @param[in] posSectorWidth: Hall_module output, postive pulse width
* @param[in] hall: variable which use in hall sensor control, pointer to HALL_SENSOR structure
* @return none
*/
void HallSensor_SelfLearn_CaptureIsr(uint8_t hallStatus, uint32_t negSectorWidth, uint32_t posSectorWidth, HALL_SENSOR *hall);

/*!
* @brief Hall sensor self-learn main function, mainly contain two parts, hall installation order and err.
*        Firstly, force rotor to different position, interval is 10 degree, and get the hall change position.
*        Secondly, force rotor to different position, interval is 2 degree, narrow the range, and get the hall change position.
*
* @param[in] hallStatus: Hall module output hall status
* @param[in] hall: variable which use in hall sensor control, pointer to HALL_SENSOR structure
* @param[in] focVars: pointer to FOC_VARS structure
* @return Given rotor position
*/
uint16_t HallSensor_SelfLearn(uint8_t hallStatus, HALL_SENSOR *hall, FOC_VARS *focVars);

/*!
* @brief When motor run at low speed (swap status), the ASR output iqRef will be very samll, and the torque will be small.
*        To make sure motor start up smoothly, this function boost trque at low speed properly.
*
* @param[in] hall: variable which use in hall sensor control, pointer to HALL_SENSOR structure
* @param[in] pidParam: PID config variable, pointer to PID_CFG structure
* @param[in] focVarsCtrl: FOC control variable, pointer to FOC_VARS_CTRL structure
* @return none
*/
void HallSensor_LowSpdBoostTrq(HALL_SENSOR *hall, PID_CFG *pidParam, FOC_VARS_CTRL *focVarsCtrl);

/*!
* @brief Provide electric angle interface. This function will be called in FOC isr function.
*
* @param[in] hall: variable which use in hall sensor control, pointer to HALL_SENSOR structure
* @return electric angle of Hall sensor
*/
uint16_t Get_HALL_ElecAngle(HALL_SENSOR *hall);

/* ======================================  Functions define  ======================================== */

#endif /* __HALL_SENSOR_H__ */

/* =============================================  EOF  ============================================== */
