/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __HIGHFREQINJ_H__
#define __HIGHFREQINJ_H__

/*!
* @file highfreqinj.h
*
* @brief This file provides the rotar angle and speed by the high frequency injection for sensorless vector control functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pid_regulator.h"
#include "param_init.h"

/* ============================================  Define  ============================================ */
#define INITPOS_ARR_MAX         400

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Initial position detect parameters structure.
*/
typedef struct
{
    int16_t arr[INITPOS_ARR_MAX];   /*!< High frequency current storage array */
    uint16_t index;                 /*!< High frequency current storage index */
    int16_t max;                    /*!< High frequency d axis current max value */
    int16_t min;                    /*!< High frequency d axis current min value */
} HFI_INIT_POS_DETECT;

/*!
* @brief High frequency injection configuration parameters structure.
*/
typedef struct
{
    uint16_t hfiDebugEnable;    /*!< High frequency injection debug mode enable switch */
    uint16_t hfiToStoEnable;    /*!< High frequency injection switch to high speed strategy enable switch */

    int16_t vPeak;              /*!< High frequency injection voltage amplitude */
    int16_t pulseAmp;           /*!< Pulse voltage amplitude */
    uint16_t pulseTime;         /*!< Pulse voltage inject time */
    uint16_t clearTime;         /*!< Current clear time */

    int16_t hfiFreq;            /*!< High frequency injection frequency */
    uint16_t msUnit;            /*!< High frequency injection ms unit */
    uint16_t hfiInitTime;       /*!< High frequency injection initial inject time */
    uint16_t hfiInitTheta;      /*!< High frequency injection initial theta */
    int16_t hfiPllKi;           /*!< High frequency injection pll ki */

    int16_t hfiVdFreqHigh;      /*!< High frequency injection vd calculate frequency high */
    int16_t hfiVdFreqLow;       /*!< High frequency injection vd calculate frequency low */

/* Filter transfer function: y(n) = A0 * x(n) + A1 * x(n-1) + A2 * x(n-2) + B1 * y(n-1) + B2 * y(n-2).
*  Different filters can be obtained by  matching different coefficients A and B.
*/
    int16_t bpfA0;              /*!< Band pass filter coefficient A0 */
    int16_t bpfA2;              /*!< Band pass filter coefficient A2 */
    int16_t bpfB2;              /*!< Band pass filter coefficient B2 */

    int16_t lpfA0;              /*!< Low pass filter coefficient A0 */
    int16_t lpfA1;              /*!< Low pass filter coefficient A1 */
    int16_t lpfA2;              /*!< Low pass filter coefficient A2 */
    int16_t lpfB2;              /*!< Low pass filter coefficient B2 */

    int16_t thetaLpfCoeff;      /*!< 1 order Low pass filter coefficient */

    int32_t bpfB1;              /*!< Band pass filter coefficient B1 */
    int32_t lpfB1;              /*!< Low pass filter coefficient B1 */
    int32_t hfiPllKp;           /*!< High frequency injection pll kp */
} HFI_CFG;

/*!
* @brief High frequency injection band pass filter control parameters structure.
*/
typedef struct
{
    int16_t a0;                 /*!< Band pass filter a0 */
    int16_t a2;                 /*!< Band pass filter a2 */
    int16_t b2;                 /*!< Band pass filter b2 */
    int32_t b1;                 /*!< Band pass filter b1 */
} HFI_BPF_COEFF;

/*!
* @brief High frequency injection low pass filter control parameters structure.
*/
typedef struct
{
    int16_t a0;                 /*!< Low pass filter a0 */
    int16_t a1;                 /*!< Low pass filter a1 */
    int16_t a2;                 /*!< Low pass filter a2 */
    int16_t b2;                 /*!< Low pass filter b2 */
    int32_t b1;                 /*!< Low pass filter b1 */
} HFI_TWO_ORDER_LPF;

/*!
* @brief High frequency injection state parameters enumerate.
*/
typedef enum
{
    HF_STATE0 = 0,              /*!< High frequency injection state0 */
    HF_STATE1 = 1,              /*!< High frequency injection state1 */
    HF_STATE2 = 2,              /*!< High frequency injection state2 */
    HF_STATE3 = 3,              /*!< High frequency injection state3 */
    HF_STATE4 = 4,              /*!< High frequency injection state4 */
    HF_STATE5 = 5,              /*!< High frequency injection state5 */
    HF_ERROR = 6                /*!< High frequency injection state6 */
} HFI_STATE;

/*!
* @brief High frequency injection control parameters structure.
*/
typedef struct
{
    HFI_STATE state;            /*!< High frequency injection state */
    uint16_t subState;          /*!< High frequency injection initial polar detect substate */

    int16_t detectVpk;          /*!< High frequency injection detect voltage amplitude */
    uint16_t hfiFreq;           /*!< High frequency injection frequency */

    int16_t vsdRef;             /*!< High frequency injection d axis given voltage */
    int16_t vsqRef;             /*!< High frequency injection q axis given voltage */

    uint16_t hfiVoltTheta;      /*!< High frequency injection angular frequency */
    uint16_t voltThetaInc;      /*!< High frequency injection angular frequency increments */
    uint16_t hfiEstTheta;       /*!< High frequency injection estimate theta */
    uint16_t hfiThetaTemp;      /*!< High frequency injection before compensation theta */
    uint16_t pwmFreq;           /*!< Pwm frequency */
    uint16_t msUnit;            /*!< High frequency injection ms unit */
    uint16_t spdCtrlloopCnt;    /*!< Switch to speed loop count */
    uint16_t thetaCompCnt;      /*!< High frequency injection compensation theta recheck count */
    uint16_t dirErrCnt;         /*!< High frequency injection run direction error count */
    uint16_t hfiToStoCount;     /*!< High frequency injection switch to high speed strategy count */
    uint16_t hfiToStoFlag;      /*!< High frequency injection switch to high speed strategy flag */
    uint16_t thetaComp;         /*!< High frequency injection compensation theta */
    uint16_t hfiEstTheta1;      /*!< High frequency injection estimate theta */

    int32_t hfiEstThetaErr;     /*!< High frequency injection injection estimate theta error */

    int32_t vdSlope;            /*!< D axis voltage ampilitude slope */
    int32_t weightFreq;         /*!< Weight frequence of HFI and observer */

    int32_t iqhFilterH;         /*!< Q axis current after band pass filter */
    int32_t iqhFilterL;         /*!< Q axis current after low pass filter */
    int32_t idhFilterH;         /*!< D axis current after band pass filter */
    int32_t idhFilterL;         /*!< D axis current after LOW pass filter */
    uint32_t delaycnt;          /*!< High frequency injection delay count */

    int32_t iqhFilterH1;        /*!< Last period q axis current after bpf */
    int32_t iqhFilterH2;        /*!< Last second period q axis current after bpf */
    int32_t idhFilterH1;        /*!< Last period d axis current after bpf */
    int32_t idhFilterH2;        /*!< Last second period d axis current after bpf */

    int32_t idhFilterL1;        /*!< Last period d axis current after lpf */
    int32_t idhFilterL2;        /*!< Last second period d axis current after lpf */
    int32_t iqhFilterL1;        /*!< Last period q axis current after lpf */
    int32_t iqhFilterL2;        /*!< Last second period q axis current after lpf */

    uint32_t isAmp;             /*!< High frequency current ampititude */
    int32_t hfiEstFreq;         /*!< High frequency injection estimated frequency */
    int32_t idh;                /*!< Current high freq d axis current before bpf */
    int32_t idh1;               /*!< Last period high freq d axis current before bpf */
    int32_t idh2;               /*!< Last two period high freq d axis current before bpf */
    int32_t iqh;                /*!< Current high freq q axis current before bpf */
    int32_t iqh1;               /*!< Last period high freq q axis current before bpf */
    int32_t iqh2;               /*!< Last two period high freq q axis current before bpf */
    int32_t idhdet;             /*!< Current high freq d axis det current before lpf */
    int32_t idhdet1;            /*!< Last period high freq d axis det current before lpf */
    int32_t idhdet2;            /*!< Last two period high freq d axis det current before lpf */
    int32_t iqhdet;             /*!< Current high freq q axis det current before lpf */
    int32_t iqhdet1;            /*!< Last period high freq q axis det current before lpf */
    int32_t iqhdet2;            /*!< Last two period high freq q axis det current before lpf */
    int32_t idPos;              /*!< Positive id value */
    int32_t idNeg;              /*!< Negative id value */

} HFI_TYPE;

/*!
* @brief High frequency injection total control parameters structure.
*/
typedef struct
{
    HFI_INIT_POS_DETECT *initPos;   /*!< Initial postion detect structure */
    HFI_BPF_COEFF *bpf;             /*!< Band pass filter structure */
    HFI_TWO_ORDER_LPF *lpf;         /*!< Two order low pass filter structure */
    HFI_TYPE *hfi;                  /*!< High frequency injection structure */
} HFI_CTRL;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief High frequency injection all the control and configuration parameters initialize,
*        usually it is initialized only once before running.
*
* @param[in] hfiCtrl: pointer to HFI_CTRL structure
* @param[in] pidHfi: pointer to PID_REGULATOR structure
* @param[in] hfiCfg: pointer to HFI_CFG structure
* @param[in] focVarsCfg: variable which use to configure FOC control, pointer to FOC_VARS_CFG structure
* @return none
*/
void Hfi_Init(HFI_CTRL *hfiCtrl, PID_REGULATOR *pidHfi, HFI_CFG *hfiCfg, FOC_VARS_CFG *focVarsCfg);

/*!
* @brief High frequency injection initial magnetic pole identification function, determine
*        the initial magnetic pole according to the inductance saturation effect.
*
* @param[in] initPos: pointer to HFI_INIT_POS_DETECT structure
* @param[in] hfi: pointer to HFI_TYPE structure
* @return none
*/
void Hfi_InitPolarDetect(HFI_INIT_POS_DETECT *initPos, HFI_TYPE *hfi);

/*!
* @brief High frequency injection initial magnetic pole identification function, determine
*        the initial magnetic pole according to the inductance saturation effect.
*
* @param[in] hfiCtrl: pointer to HFI_CTRL structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] hfiCfg: pointer to HFI_CFG structure
* @return none
*/
void Hfi_InitPolarCheck(HFI_CTRL *hfiCtrl, FOC_VARS *focVars, HFI_CFG *hfiCfg);

/*!
* @brief High frequency injection initial magnetic pole identification recheck function, to make
         sure the compensation angle is right.
*
* @param[in] hfiCtrl: pointer to HFI_CTRL structure
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void Hfi_InitPolarRecheck(HFI_CTRL *hfiCtrl, FOC_VARS *focVars);

/*!
* @brief High frequency injection detect voltage amplitude calculate function.
*
* @param[in] hfiCtrl: pointer to HFI_CTRL structure
* @param[in] hfiCfg: pointer to HFI_CFG structure
* @return none
*/
void Hfi_VdAmp_Calc(HFI_CTRL *hfiCtrl, HFI_CFG *hfiCfg);

/*!
* @brief High frequency injection angle obtain function, through injection a high frequency sinusoidal
*        voltage on the d axis, extracting the rotor position from the high frequency current response.
*
* @param[in] hfiCtrl: pointer to HFI_CTRL structure
* @param[in] pidHfi: pointer to PID_REGULATOR structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] hfiCfg: pointer to HFI_CFG structure
* @return high frequency injection estimated theta in Q16
*/
uint16_t Hfi_AngleObtain(HFI_CTRL *hfiCtrl, PID_REGULATOR *pidHfi, FOC_VARS *focVars, HFI_CFG *hfiCfg);

/* ======================================  Functions define  ======================================== */

#endif /* __HIGHFREQINJ_H__ */

/* =============================================  EOF  ============================================== */
