/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __IQ_MATH_H__
#define __IQ_MATH_H__
/*!
* @file iqmath.h
*
* @brief This file provides mathematical calculation functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include <stdint.h>
#include <Tgmath.h>

/* ============================================  Define  ============================================ */
/* Define floating point or integer math library */
#define   FLOAT_MATH    1
#define   IQ_MATH       0

#ifndef   MATH_TYPE
/* The IQmath library supports the math calculation of integer data */
#define   MATH_TYPE     IQ_MATH
#endif

/*!
* Select global Q value and scaling. The Q value is limited to the
* following range for all functions:
*
*        GLOBAL_Q = 15 or  GLOBAL_Q = 24
*/
#ifndef   GLOBAL_Q
/* Define Q format */
#define   GLOBAL_Q       15
#endif

/* GLOBAL_Q */
#if MATH_TYPE == IQ_MATH

#define   Math_IQx(factor, shift)   (int32_t) ((factor) * (1 << shift))
/* Precompiling according to Q format */
#if GLOBAL_Q == 24
#define   Math_IQ(factor)           Math_IQx(factor, 24)
#endif
#if GLOBAL_Q == 15
#define   Math_IQ(factor)           Math_IQx(factor, 15)
#endif

#endif

/* Max and Min */
#define Q15_MAX   32767
#define Q15_MIN   -32768
#define Q16_MAX   65535
#define Q16_MIN   -65536
#define Q24_MAX   16777215
#define Q24_MIN   -16777216

/* Angle degree */
#define Q16_01_DEGREE   Math_IQx(0.1 / 360 , 16)
#define Q16_1_DEGREE    Math_IQx(1.0 / 360 , 16)
#define Q16_2_DEGREE    Math_IQx(2.0 / 360 , 16)
#define Q16_5_DEGREE    Math_IQx(5.0 / 360 , 16)
#define Q16_10_DEGREE   Math_IQx(10.0 / 360 , 16)
#define Q16_20_DEGREE   Math_IQx(20.0 / 360 , 16)
#define Q16_30_DEGREE   Math_IQx(30.0 / 360 , 16)
#define Q16_45_DEGREE   Math_IQx(45.0 / 360 , 16)
#define Q16_60_DEGREE   Math_IQx(60.0 / 360 , 16)
#define Q16_70_DEGREE   Math_IQx(70.0 / 360 , 16)
#define Q16_80_DEGREE   Math_IQx(80.0 / 360 , 16)
#define Q16_90_DEGREE   Math_IQx(90.0 / 360 , 16)
#define Q16_120_DEGREE  Math_IQx(120.0 / 360 , 16)
#define Q16_150_DEGREE  Math_IQx(150.0 / 360 , 16)
#define Q16_180_DEGREE  Math_IQx(180.0 / 360 , 16)
#define Q16_210_DEGREE  Math_IQx(210.0 / 360 , 16)
#define Q16_240_DEGREE  Math_IQx(240.0 / 360 , 16)
#define Q16_270_DEGREE  Math_IQx(270.0 / 360 , 16)
#define Q16_300_DEGREE  Math_IQx(300.0 / 360 , 16)
#define Q16_330_DEGREE  Math_IQx(330.0 / 360 , 16)
#define Q16_350_DEGREE  Math_IQx(350.0 / 360 , 16)
#define Q16_355_DEGREE  Math_IQx(355.0 / 360 , 16)
#define Q16_359_DEGREE  Math_IQx(359.0 / 360 , 16)
#define Q16_360_DEGREE  Math_IQx(360.0 / 360 , 16)

/* Trgonometric */
#define PI              3.14159265358979f        /*!< Ral value of PI */
#define PI2             6.28318530717958f        /*!< Real value of PI * 2 */
#define PI_HALF         1.57079632679489f        /*!< Real value of PI / 2 */
#define INV_PI2         0.15915494309190f        /*!< Real value of 1 / (PI * 2) */
#define TAN60           1.73205080756888f        /*!< Real value of tan(60) */
#define COS30           0.86602540378444f        /*!< Real value of cos(30) */
#define COS60           0.5f                     /*!< Real value of cos(60) */

/* Common numbers real vaule */
#define INV_SQRT3       0.57735026918963f        /*!< 1/sqrt(3) = 0.57735026918963 */
#define SQRT2           1.414213562373f          /*!< sqrt(2) = 1.4142135623730950488016887242097 */
#define ONE_THIRD       0.33333333333333f        /*!< 1/3 */
#define TWO_THIRD       0.66666666666667f        /*!< 2/3 */
#define SQRT3DIV2       0.866025f                /*!< sqrt(3)/2 */

/* Common numbers pu value */
#define INV_SQRT3_Q     Math_IQ(INV_SQRT3)      /*!< Pu value of 1/sqrt(3) */
#define SQRT3DIV2_Q     Math_IQ(SQRT3DIV2)      /*!< Pu value of sqrt(3)/2 */
#define ONE_PI_IQ       Math_IQ(1.0f / PI)       /*!< Pu value of 1/PI */

/* Get the absolute value of input x */
#define Math_Qabs(x)    (int32_t)(((x) > 0) ? (x) : (-(x)))

/* Get the pu value of angle x (degree) */
#define DEGREE(x)       (uint16_t)((float)x / 360.0 * 65535.0)

/* Get the pu value of multiplication result with input x and y */
#define Math_Mpy(x, y)  ((((int32_t)(x)) * ((int32_t)(y))) >> 15)

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Division of two Q-format numbers (original/divisor), returning the product of the same Q format.
*
* @param[in] original: Numerator
* @param[in] divisor: Denominator
* @return result: Output parameters
*/
int32_t Math_Div(int32_t x, int32_t y);

/*!
* @brief Division of two Q-format numbers (original/divisor), returning the product with right shifting 'dsft' bits.
*
* @param[in] dend: Numerator
* @param[in] dsor: Denominator
* @param[in] dsft: Bit shift value
* @return result: Output parameters
*/
int32_t Math_DivShift(int32_t dend, int32_t dsor, uint8_t dsft);

/*!
* @brief Calculated sqrt root of input, returning the real root value.
*
* @param[in] input: Input parameters
* @return result: Sqrt result, real root value
*/
int32_t Math_Sqrt(int32_t input);

/*!
* @brief Calculated arcsin of input.
*
* @param[in] x: Input parameters
* @return result: Result of arcsin(input)
*/
int32_t Math_Asin(int32_t input);

/*!
* @brief Arctangent process, theta = arctan(y/x).
*
* @param[in] x: Coordinate (x, y), Q16-format.
* @param[in] y: Coordinate (x, y), Q16-format.
* @return result: Arctan value, value range is 0~65535.
*/
uint16_t Math_Q16Atan(int32_t x, int32_t y);

/*!
* @brief Calculates the sine value of the input theta.
*
* @param[in] theta: Input Q16 theta.
* @return Q15 Sinusoidal value.
*/
int16_t Math_Sin(uint16_t theta);

/*!
* @brief Calculates the cosine value of the input theta.
*
* @param[in] theta: Input Q16 theta.
* @return Q15 Cosine value.
*/
int16_t Math_Cos(uint16_t theta);

/*!
* @brief Upper and lower limits, limit input to [-32768, 32767].
*
* @param[in] input: Input parameters
* @return result: Return value after the Q value limit.
*/
int32_t Math_Qsat(int32_t input);

/*!
* @brief Upper and lower limits, limit input to [Neg, Pos].
*
* @param[in] input: Input parameters
* @param[in] Pos: Upper limit
* @param[in] Neg: lower limit
* @return result: Return value after the upper and lower limit.
*/
int32_t Math_Sat(int32_t input, int32_t Pos , int32_t Neg);

/*!
* @brief Root mean square calculation in Q15 format.
*
* @param[in] x: Input parameters 1.
* @param[in] y: Input parameters 2.
* @return Q15 root mean square value.
*/
int32_t Math_Q15Mag(int32_t x, int32_t y);

/* ======================================  Functions define  ======================================== */

#endif /* __IQ_MATH_H__ */

/* =============================================  EOF  ============================================== */
