/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __PARAM_INIT_H__
#define __PARAM_INIT_H__

/*!
* @file param_init.h
*
* @brief This file provides per unit value of motor parameters and calculation of ADC per unit coefficient functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include <string.h>
#include "iqmath.h"

/* ============================================  Define  ============================================ */
#define DELAY_100MS             (100UL)             /*!< Delay time value 100ms */
#define DELAY_500MS             (500UL)             /*!< Delay time value 500ms */
#define DELAY_1000MS            (1000ul)            /*!< Delay time value 1000ms */
#define DELAY_2000MS            (2000UL)            /*!< Delay time value 2000ms */
#define DELAY_5000MS            (5000UL)            /*!< Delay time value 5000ms */

#define USE_SVPWM_VOLT          (0)                 /*!< Use the SVPWM input voltage */
#define USE_VOLT_RESTRUCT       (1)                 /*!< Use the reconstruction voltage */

#define ATAN_WAY                (0)                 /*!< Use the arctan method */
#define PLL_WAY                 (1)                 /*!< Use the PLL method */

#define FORWARD_RUN             ((int16_t)1)        /*!< Motor run direction is forward */
#define REVERSE_RUN             (-(int16_t)1)       /*!< Motor run direction is reverse */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Motor parameter structure.
*/
typedef struct
{
    uint16_t motorType;             /*!< Motor type */
    uint32_t statorRs;              /*!< Stator resistance (unit:1mohm) */
    uint32_t statorLs;              /*!< Stator inductance (unit:1uH) */
    uint32_t ld;                    /*!< Motor D-axis inductance (unit:1uH) */
    uint32_t lq;                    /*!< Motor q-axis inductance (unit:1uH) */
    uint32_t rotorRs;               /*!< Induction motor Rotor resistance (unit:1mohm) */

    uint32_t lr;                    /*!< Induction motor rotor inductance (unit:1uH) */
    uint32_t lm;                    /*!< Induction motor mutual inductance (unit:1uH) */
    uint32_t lleak;                 /*!< Induction motor leakage inductance (unit:1uH) */
    uint32_t noLoadExcite;          /*!< Induction motor no load exciting current (unit:1A) */

    uint32_t jm;                    /*!< Rotational intertia (unit:ukg*m2) */
    uint32_t polePairs;             /*!< Number of pole pairs (constant value) */
    uint32_t encoderLines;          /*!< Encoder pulse number per Motor (constant value) */
    float bEmf;                     /*!< Back emf coefficient (unit:Vrms/Krpm) */
    uint32_t maxCur;                /*!< Maximum motor current of the motor (unit:1A,pk) */
    uint32_t ratedVoltage;          /*!< rated voltage of the HW board (unit:1V) */
    uint32_t maxSpeed;              /*!< Maximum speed of the motor (unit:1rpm) */
    uint32_t maxFreq;               /*!< Maximum speed of the motor (unit:HZ) */

    uint32_t maxTorque;             /*!< Maximum torque of the motor (unit:1mNm) */
    uint32_t maxPower;              /*!< Maximum power of the motor (unit:W) */
} MOTOR_PARAM;

/*!
* @brief Base value and Pu(per unit) value structure of inverter.
*/
typedef struct
{
    uint32_t ibAp;                  /*!< 0.1A, Current base */
    uint32_t ubVt;                  /*!< 0.1V, Voltage base */
    uint32_t fbHz;                  /*!< Hz,Frequency base */
    uint32_t pairsb;                /*!< motor pairs base */
    uint32_t vbRpm;                 /*!< rpm,Velocity base */
    uint32_t tbUs;                  /*!< 0.1us,Time base */
    uint32_t pbW;                   /*!< 0.01Kw, Power base */
    uint32_t rbOm;                  /*!< 0.01Ohm, Resistance base */
    uint32_t web;                   /*!< rad/s, base */
    uint32_t lbHm;                  /*!< uH, Inductance base */
    uint32_t tbNm;                  /*!< 2e-19Nm, Torque base */
    uint32_t jb;                    /*!< Unit:10-10*kg*m2 rotational inertia base */
    uint32_t fluxbWb;               /*!< Unit:0.001Wb */

    uint32_t tCurCtrPu;             /*!< Q15, Pu value of Current control period */
    uint32_t tSpeedCtrPu;           /*!< Q12, Pu value of Speed control period */
    uint32_t tPWMPu;                /*!< Q15, Pu value of PWM control period */
    uint32_t maxThetaPerCurTcPu;    /*!< Q15, Pu value of maximum operating angle in one current loop cycle */
    uint32_t maxThetaPerSpdTcPu;    /*!< Q15, Pu value of maximum operating angle in one speed loop cycle */

    uint32_t mTrqMaxPu;             /*!< Pu value of Maximum torque of the motor */
    uint32_t mSpeedMaxPu;           /*!< Pu value of Maximum speed of the motor */
    uint32_t mSpeedMinPu;           /*!< Pu value of Minimum speed of the motor */
    uint32_t mFreqMaxPu;            /*!< Pu value of Maximum electrical frequency of the motor */
    uint32_t mFreqMinPu;            /*!< Pu value of Minimum electrical frequency of the motor */
    uint32_t mImMaxPu;              /*!< Pu value of Maximum motor current */
    uint32_t mImMinPu;              /*!< Pu value of Minimum motor current */
    uint32_t mImRatePu;             /*!< Pu value of Motor rated current */
    uint32_t ratePowerpu;           /*!< Pu value of Motor rated power */

    uint32_t rsPu;                  /*!< Pu value of Motor resistance */
    uint32_t ldPu;                  /*!< Pu value of Motor D-axis inductance */
    uint32_t lqPu;                  /*!< Pu value of Motor q-axis inductance */
    uint32_t rs4Pu;                 /*!< Pu value of Induction motor RS resistance */
    uint32_t rr4Pu;                 /*!< Pu value of Induction motor RR resistance */
    uint32_t ls4Pu;                 /*!< Pu value of Induction motor LS inductance */
    uint32_t lr4Pu;                 /*!< Pu value of Induction motor LR inductance */
    uint32_t lm4Pu;                 /*!< Pu value of Induction motor inductance */
    uint32_t lleak4Pu;              /*!< Pu value of Induction motor leakage inductance */

    uint32_t jmPu;                  /*!< Pu value of motor rotational inertia */
    uint32_t pairsPu;               /*!< Pu value of motor Number of pole pairs */
    uint32_t fluxPu;                /*!< Pu value of motor magnetic flux */
    uint32_t mTrqRatePu;            /*!< Pu value of motor rated torque of the motor */

    uint32_t iratePu;               /*!< Pu value of motor Motor rated current */
    uint32_t mIMaxPu;               /*!< Pu value of motor Maximum  current */
    uint32_t vspCmdMaxSpeedPu;      /*!< Pu value of motor Maximum speed command */
    uint32_t vspCmdMinSpeedPu;      /*!< Pu value of motor Minimum speed command */

    uint32_t curBandWidthPu;        /*!< Pu value of current bandwidth */
    uint32_t speedBandWidthPu;      /*!< Pu value of speed bandwidth */
} MOTOR_PARAM_PU;

/*!
* @brief Hardware board parameter structure.
*/
typedef struct
{
    uint16_t pwmPolarity;           /*!< Pwm active polarity */

    uint16_t hwDeadTime;            /*!< Hardware dead time size,usually refers to the deadband size of the gate driver */
    uint16_t swDeadTime;            /*!< Software dead time size,usually refers to the deadband size set by mcu */

    uint16_t maxVoltage;            /*!< Maximum operating voltage of the controller */
    uint16_t normalVoltage;         /*!< Normal operating voltage of the controller */
    uint16_t minVoltage;            /*!< Minimum operating voltage of the controller */
    float vbusAttenuateVal;         /*!< Voltage attenuation ratio */

    uint16_t maxCurrent;            /*!< Maximum operating current of the controller */

    float rShunt;                   /*!< Resistance value for current samping */
    float opGainVal;                /*!< Amplification factor of operational amplifier when sampling current */
    float icsGainVal;               /*!< Amplification factor of ICS when sampling current */
//    uint16_t opGainPolarity;      /*!< Polarity of operational amplifier when sampling current */
    uint16_t adcRange;              /*!< ADC sampling accuracy of MCU */
    float adcRefV;                  /*!< ADC sampling voltage range of MCU  */
    uint16_t overCurrentPolarity;   /*!< Polarity setting of over current protection */

} HW_BOARD;

/*!
* @brief Common configuration parameter structure of FOC.
*/
typedef struct
{
    int16_t motorDir;               /*!< Motor direction */
    uint16_t controlMode;           /*!< Motor Control Mode */
    uint16_t voltObtainWay;         /*!< Sensorless voltage obtain way */
    int16_t angleObtainWay;         /*!< Sensorless angle obtain way */
    int16_t smoWay;                 /*!< Sensorless sliding mode control way, including two order smc and full order smc, 0:two order smo; 1:full order smo */
    int16_t fullOrderSmoFilt;       /*!< Full order smc filter enable switch, 1: means enable filt, 0: means disable filt */

    int16_t speedMaxLimit;          /*!< Max limit of speed command */
    int16_t speedMinLimit;          /*!< Min limit of speed command */
    int16_t stopSpeed;              /*!< Stop speed in slow down stop mode */
    int16_t motorAcceleration;      /*!< Motor speed loop acceleration */
    int16_t motorDeceleration;      /*!< Motor speed loop deacceleration */

    uint16_t pwmFreq;               /*!< Motor pwm frequency */
    uint16_t curFreq;               /*!< Motor current loop frequency */
    uint16_t speedFreq;             /*!< Motor speed loop frequency */
    uint16_t msUnitInt;             /*!< Ms count value */
    uint16_t baseFreq;              /*!< Base frequency */
    uint16_t halfPwmPeriod;         /*!< Half of pwm period */
    uint16_t pwmPeriod;             /*!< PWM period */
    uint16_t pwmDeadTime;           /*!< PWM dead time */
    uint32_t sysBusClk;             /*!< System bus clock */
    uint16_t pwdtPrescal;           /*!< PWDT prescaler val */
    uint16_t curSampleMode;         /*!< Current sample mode */
    uint32_t atanModeSpeedFilterH;  /*!< Low pass filter coefficient used for ELEC_FREQUENCE_HIGH */
    uint32_t atanModeSpeedFilterL;  /*!< Low pass filter coefficient used for ELEC_FREQUENCE_LOW */
} FOC_VARS_CFG;

/*!
* @brief Current loop structure, Q15 means that the value has been converted to Pu value.
*/
typedef struct
{
    int16_t iaPu;                   /*!< Q15, phase A current value in stator coordinate system */
    int16_t ibPu;                   /*!< Q15, phase B current value in stator coordinate system */
    int16_t icPu;                   /*!< Q15, phase C current value in stator coordinate system */
    int16_t ialpha;                 /*!< Q15, Alfa-axis current in stator coordinate system */
    int16_t ibeta;                  /*!< Q15, Beta-axis current in stator coordinate system */
    int16_t iqFdb;                  /*!< Q15, Q-axis feedback current in rotating coordinate system */
    int16_t idFdb;                  /*!< Q15, D-axis feedback current in rotating coordinate system */
    int16_t iqBeforeLpf;            /*!< Q15, Q-axis feedback current before lpf in rotating coordinate system */
    int16_t idBeforeLpf;            /*!< Q15, D-axis feedback current before lpf in rotating coordinate system */
    int16_t iqRef;                  /*!< Q15, Reference Q-axis feedback current in rotating coordinate system */
    int16_t idRef;                  /*!< Q15, Reference D-axis feedback current in rotating coordinate system */

    int16_t vq;                     /*!< Q15, Q-axis voltage */
    int16_t vd;                     /*!< Q15, D-axis voltage*/
    int16_t vqRef;                  /*!< Q15, Q-axis target voltage */
    int16_t vdRef;                  /*!< Q15, D-axis target voltage */
    int16_t valpha;                 /*!< Q15, Alfa axis voltage */
    int16_t vbeta;                  /*!< Q15, Beta axis voltage */

    uint16_t elecAngle;             /*!< Q15, Electrical angle (pu) */
    uint16_t ialphabeta;            /*!< Q15, Alfa and Beta axis synthesis current in stator coordinate system */
    uint16_t idq;                   /*!< Q15, D and Q axis synthesis current in rotating coordinate system */
    uint16_t vdq;                   /*!< Q15, D Q-axis synthesis voltage */
    uint16_t vdqLimit;              /*!< Q15, D Q-axis synthesis voltage after limit */

    uint16_t uOutputPu;             /*!< Q15, output voltage pu value */
    uint16_t modulationRatio;       /*!< SVPWM Bus voltage modulation ratio/depth, 10000 = 100%, means Udc utilization is 100% */

    uint16_t vdcPu;                 /*!< Q15, Pu value of Vbus voltage */

    uint16_t focExecutionCycleCnt;  /*!< Counter of FOC multi-cycle execution */
    uint16_t pwmOutMask;            /*!< Current loop execution flag */

    uint32_t vdqFilter;             /*!< Q15, D Q-axis synthesis voltage after filter */

    int32_t voltagePhaseA;          /*!< Q15, A phase voltage */
    int32_t voltagePhaseB;          /*!< Q15, B phase voltage */
    int32_t voltagePhaseC;          /*!< Q15, C phase voltage */

    int16_t voltAlphaRestruct;      /*!< Q15, Restruct Alpha phase voltage */
    int16_t voltBetaRestruct;       /*!< Q15, Restruct Beta phase voltage */
} FOC_VARS_CTRL;

/*!
* @brief Foc parameter structure.
*/
typedef struct
{
    FOC_VARS_CFG *pCfg;             /*!< Common configuration parameter structure of FOC */
    FOC_VARS_CTRL *pCtrl;           /*!< Current loop structure */
} FOC_VARS;

/*!
* @brief The current and voltage per unit coefficient structure, used to convert the sample current to Pu value.
*/
typedef struct {
    uint16_t kVdcSamplePu;          /*!< The coefficient of Sample==>Q15 Pu */
    int16_t kCurSamplePu;           /*!< The coefficient of current sample value ==>Q15 Pu */
    int16_t kCurSensorPu;           /*!< The coefficient of current sensor value ==>Q15 Pu */
} ADC_SAMPLE_COF;

/*!
* @brief Speed and toque control parameter structure.
*/
typedef struct
{
    MOTOR_PARAM *pMotor;            /*!< Motor parameters real vaule */
    MOTOR_PARAM_PU *pMotorPu;       /*!< Motor parameters unit pu */
    FOC_VARS *pFOCVars;             /*!< parameters of FOC control */
} STC_PARAM;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Calculation of the Pu parameters used in speed and torque control.
*
* @param[in] motor:MOTOR_PARAM type pointer
* @param[in] focVarsCfg: variable which use to configure FOC control, pointer to FOC_VARS_CFG structure
* @param[out] param: MOTOR_PARAM_PU type pointer
* @return none
*/
void STC_motorParamPuCalc(MOTOR_PARAM *motor, FOC_VARS_CFG *focVarsCfg, MOTOR_PARAM_PU *paramPu);

/*!
* @brief Current and voltage ADC PU factor coefficient calculation,the calculated coefficient multiplied by the sampling current
*        can be converted current into a Pu value.
*
* @param[in] paramPu: MOTOR_PARAM_PU type pointer
* @param[in] hwBoard: HW_BOARD type pointer
* @param[out] adcPuCof: ADC_SAMPLE_PARAM type pointer
* @return none
*/
void ADC_SampleCofCalc(MOTOR_PARAM_PU *paramPu, HW_BOARD *hwBoard, ADC_SAMPLE_COF *adcPuCof);

/* ======================================  Functions define  ======================================== */

#endif /* __PARAM_INIT_H__ */

/* =============================================  EOF  ============================================== */
