/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/


#ifndef __PID_REGULATOR_H__
#define __PID_REGULATOR_H__
/*!
* @file pid_regulator.h
*
* @brief This file provides different types of PID functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "iqmath.h"
#include "param_init.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief PID regulator input parameters structure.
*/
typedef struct
{
    int32_t refPu;              /*!< PID input reference value */
    int32_t fbkPu;              /*!< PID input feedback value */
    int32_t maxPu;              /*!< Parameter: upper saturation limit */
    int32_t minPu;              /*!< Parameter: lower saturation limit */
    int32_t deadZonePu;         /*!< Error deadband setting */
    int32_t feedForwardPu;      /*!< Feedforward compensation */
} PID_REGULATOR_IN;

/*!
* @brief PID regulator output parameters structure.
*/
typedef struct
{
    int32_t outPu;              /*!< PID output value */
    int32_t errPu;              /*!< Data: Err, errPu = refPu - fbkPu */
    int32_t err1Pu;             /*!< Data: Err of last calculation period*/
    int32_t err2Pu;             /*!< Data: Err of the penultimate calculation period */
    int32_t upPu;               /*!< Data: proportional term */
    int32_t uiPu;               /*!< Data: integral term */
    int32_t v1Pu;               /*!< Data: pre-saturated controller output */
    int32_t i1Pu;               /*!< Data: integrator storage: ui(k-1) */
    int32_t w1Pu;               /*!< Data: saturation record: [u(k-1) - v(k-1)], v.ui = v.ui + v.ki * v.up + v.kc * v.w1 */
    int32_t outPuFliter;        /*!< PID output after smooth filter */
    int32_t outErr;             /*!< Err of PID output */
    int32_t outPu1;             /*!< PID output before limiting amplitude */
    int32_t deCoup;             /*!< PID output decouple value */
} PID_REGULATOR_OUT;

/*!
* @brief PID regulator calculation coefficient structure.
*/
typedef struct
{
    int32_t kpPu;               /*!< Proportional gain of PID regulator */
    int32_t kiPu;               /*!< Integral gain of PID regulator */
    int32_t kcPu;               /*!< Reserve */
    int32_t kdPu;               /*!< Reserve */
} PID_REGULATOR_COF;

/*!
* @brief Integral separation PID parameter structure.
*/
typedef struct
{
    uint8_t integralEnable;     /*!< switch to enable integral calculation */

    int16_t kp;                 /*!< Proportional gain of PID regulator */
    int16_t ki;                 /*!< Integral gain of PID regulator */
    int16_t kd;                 /*!< Reserve */
    int16_t errPu;              /*!< Data: Err, errPu = refPu - fbkPu */
    int16_t integralTerm;       /*!< Data: Integral term */
    int16_t maxOutputPu;        /*!< Parameter: upper saturation limit */
    int16_t minOutputPu;        /*!< Parameter: upper saturation limit */
    int16_t upOutPu;            /*!< Proportional part of output */
    int16_t uiOutPu;            /*!< Integral part of output */
    int32_t pidOut;             /*!< Pid output */
} PID_SEPARATE_TYPE;

/*!
* @brief Configuration parameter structure. Used to set PI parameters in different PID regulators.
*/
typedef struct
{
    int32_t torqueKp;               /*!< Torque loop PID kp */
    int32_t torqueKi;               /*!< Torque loop PID ki */
    int32_t torqueKd;               /*!< Torque loop PID kd */
    int32_t fluxKp;                 /*!< Flux loop PID kp */
    int32_t fluxKi;                 /*!< Flux loop PID ki */
    int32_t fluxKd;                 /*!< Flux loop PID kd */
    int32_t maxtorque;              /*!< Torque loop PID max */
    int32_t mintorque;              /*!< Torque loop PID min */
    int32_t maxflux;                /*!< Flux loop PID max */
    int32_t minflux;                /*!< Flux loop PID min */
    int32_t speedKp;                /*!< Speed loop PID kp */
    int32_t speedKi;                /*!< Speed loop PID ki */
    int32_t speedKd;                /*!< Speed loop PID kd */
    int32_t maxSpeed;               /*!< Speed loop PID max */
    int32_t minSpeed;               /*!< Speed loop PID min */
    int32_t positionKp;             /*!< Position loop PID kp */
    int32_t positionKi;             /*!< Position loop PID ki */
    int32_t positionKd;             /*!< Position loop PID kd */
    int32_t maxPosition;            /*!< Position loop PID max */
    int32_t minPosition;            /*!< Position loop PID min */
    uint32_t curWicPu;              /*!< Current loop operating frequency pu value */
    uint32_t speedWic;              /*!< Speed loop operating frequency pu value */
    uint16_t curPidSelfLearn;       /*!< Current loop PID self learn flag */
    uint16_t speedPidSelfLearn;     /*!< Speed loop PID self learn flag */
    uint16_t damp;                  /*!< Speed loop damp */
    int16_t feedbackGain;           /*!< Feedback gain */
} PID_CFG;

/*!
* @brief PID structure. Integrates the input, coefficient and output of PID ragulator.
*/
typedef struct
{
    PID_REGULATOR_IN *pPidInput;    /*!< Input parameters structure of PID */
    PID_REGULATOR_COF *pPidCoef;    /*!< Coefficient structure of PID */
    PID_REGULATOR_OUT *pPidOutput;  /*!< Output parameters structure of PID */
} PID_REGULATOR;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Series type PID regulator. Series PID consists of proportional,
*        integral and differential terms in series which transfer function is:
*                   ki
*        kp * (1 + ---) * (1 + s * kd)
*                   s
*        where kp is the proportional gain, ki is the integration coefficient,
*        and td is the differential coefficient.
*
* @param[in] pid: pointer to PID_REGULATOR structure
* @return outPu: Output value of PID ragulator
*/
int16_t PID_SeriesRegulator(PID_REGULATOR *pid);

/*!
* @brief Parallel type PID regulator. Parallel PID consists of proportional,
*        integral and differential terms in parallel which transfer function is:
*              ki
*        kp + ---- + kd * s
*              s
*        where kp is the proportional gain, ki is the integration coefficient,
*        and kd is the differential coefficient.
*
* @param[in] pid: pointer to PID_REGULATOR structure
* @return outPu: Output value of PID ragulator
*/
int16_t PID_ParallelRegulator(PID_REGULATOR *pid);

/*!
* @brief Position type PID regulator. Positional PID control algorithm directly gives
*        the actual value of the control quantity, which can be directly input into the
*        actuator; the output of the current sampling time is related to each state in
*        the past, and the error value should be accumulated in calculation.
*
* @param[in] pid: pointer to PID_REGULATOR structure
* @return outPu: Output value of PID ragulator
*/
int16_t PID_PositionalRegulator(PID_REGULATOR *pid);

/*!
* @brief Incremental type PID regulator. The output of incremental PID is only
*        the increment value delta_u(k) of control quantity. the cumulative
*        function of the actuator is realized by the formula: u(k) = u(k-1) + delta_u(k).
*        Once kp, ki, kd are determinde, the control increment can be obtained by
*        using the deviation value (error) of the last three control variables.
*
* @param[in] pid: pointer to PID_REGULATOR structure
* @return outPu: Output value of PID ragulator
*/
int16_t PID_IncrementalRegulator(PID_REGULATOR *pid);

/*!
* @brief Feedback suppression PI regulator. The error of PID regulator
*        output is fed back to restrain the saturation of the integrator.
*
* @param[in] pid: pointer to PID_REGULATOR structure
* @param[in] pidParam: pointer to PID_CFG structure
* @return outPu: Output value of PID ragulator
*/
int16_t PID_FeedbackRegulator(PID_REGULATOR *pid, PID_CFG *pidParam);

/*!
* @brief Current loop complex vector decoupling PID regulator, dq axis current can be decoupled by using this type of PID.
*
* @param[in] dPid: pointer to d axis PID_REGULATOR structure
* @param[in] qPid: pointer to q axis PID_REGULATOR structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] freq: motor electric frequence
*/
void Current_Decoup_Regulator(PID_REGULATOR *dPid, PID_REGULATOR *qPid, MOTOR_PARAM_PU *paramPu, int32_t freq);

/* ======================================  Functions define  ======================================== */

#endif /* __PID_REGULATOR_H__ */

/* =============================================  EOF  ============================================== */
