/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __PULSE_INJECT_H__
#define __PULSE_INJECT_H__
/*!
* @file pulse_inject.h
*
* @brief This file provides motor rotor initial position checking functions interfaces.The principle is saturation characteristic of inductance.
*
*/

/* ===========================================  Includes  =========================================== */
#include <stdint.h>
#include "param_identify_api.h"

/* ============================================  Define  ============================================ */
#define BUS_CUR_DATA_THRES      1       /*!< ad data threshold */
#define MAX_AD_SAMPLE_VAL       4050    /*!< if ad sample value bigger than this value, then the ad value is incorrect */
#define SECTOR_60_DEGREE        10923   /*!< 60 degree in Q16 format */

#define INITPOS_CALC_TIMES_ORDER        2       /*!< Init-position check calculation times set value, 2 means times = 1<<2 = 4 */
#define INITPOS_CYCLE_POINTS_ORDER      4       /*!< Init-position check sampling and save points per cycle set value, 4 means points = 1<<4 = 16 */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Pulse inject to get initial position function status
*/
typedef enum {                          /*!< Define check initial position step */
    CHIP_CONFIG = 0,                    /*!< Status 1: config */
    SELF_LEARNING,                      /*!< Status 2: self-learning */
    PULSE_INJECTING,                    /*!< Status 3: pulse injecting */
    CHIP_CONFIG_RESET,                  /*!< Status 4: recover config */
    PULSE_INJECTION_ERR,                /*!< Status 5: error */
} INIT_POS_STATUS;

/*!
* @brief Pulse inject function configuration
*/
typedef struct
{
    uint16_t selfLearnThres;            /*!< Pulse inject self learn current threshold */
    uint16_t selfLearnDutyStep;         /*!< Pulse inject self learn pwm duty increase step */
} PULSE_INJ_CFG;

/*!
* @brief Pulse inject data
*/
typedef struct
{
    uint8_t pulseInjDoneFlag;           /*!< 0:pulse injection unfinished.1:pulse injection finished */
    uint8_t pulseInjFlag;               /*!< PULSE_INJECTING status finished flag */
    uint8_t flowState;                  /*!< Check initial position,initial sector transfer to openkick */
    uint8_t selfLearnFlag;              /*!< 0:Self-learn unfinished 1:Self-learn finished */
    uint8_t subStatus;                  /*!< Sub-program status */
    uint8_t periodCnt;                  /*!< Check initial position ad interrupt counter */
    uint16_t busCur[6];                 /*!< AD data array */
    uint16_t busCurVal;                 /*!< Bus current. */
    uint16_t wait;                      /*!< Delay counter */
    uint16_t pwmTs;                     /*!< PWM period value */
    uint16_t section;                   /*!< Pulse section */
    uint16_t curFirst;                  /*!< Self-learn bus current first val */
    uint16_t initEletricTheta;          /*!< Initial rotor position  */
    uint16_t pulseInjErrCnt;            /*!< Pulse injection error counter */
} PULSE_INJECT_DATA;


/*!
* @brief Polarity check data
*/
typedef struct
{
    int8_t subStatus;                   /*!< Polarity check process status */
    int16_t idPolarPos;                 /*!< Polarity check process positive d-axis current */
    int16_t idPolarNeg;                 /*!< Polarity check process negative d-axis current */
    uint16_t delayCnt;                  /*!< Polarity check process delay counter */
    uint16_t thetaComp;                 /*!< Polarity check process angle compensation value */
} INIT_POSITION_POLARITY_CHECK;

/*!
* @brief Initial position detection control data
*/
typedef struct
{
    uint8_t step;                       /*!< Initial position detection process step */
    uint8_t sampleFlag;                 /*!< Initial position detection process current sample flag */
    uint8_t saveEndFlag;                /*!< Initial position detection process current sample data save finish flag */
    uint8_t extractFlag;                /*!< Initial position detection process current information extraction flag */
    uint16_t calcCnt;                   /*!< Initial position detection process calculation counter */
    uint32_t delayCnt;                  /*!< Initial position detection process delay counter */

    uint32_t injVoltFreq;               /*!< Initial position detection process injection frequency */
    uint16_t injVoltElecTheta;          /*!< Initial position detection process injection signal electric angle */
    int16_t injVoltAmp;                 /*!< Initial position detection process injection signal amplitude */
    int16_t injVoltRef;                 /*!< Initial position detection process injection voltage reference reference variable */
    int16_t injVoltAlfa;                /*!< Initial position detection process injection voltage alpha-axis component */
    int16_t injVoltBeta;                /*!< Initial position detection process injection voltage beta-axis component */

    int16_t iaLpf;                      /*!< Initial position detection process phase-A current through LPF */
    int16_t ibLpf;                      /*!< Initial position detection process phase-B current through LPF */
    int16_t icLpf;                      /*!< Initial position detection process phase-C current through LPF */

    int16_t iaAmp;                      /*!< Initial position detection process phase-A current amplitude */
    int16_t ibAmp;                      /*!< Initial position detection process phase-B current amplitude */
    int16_t icAmp;                      /*!< Initial position detection process phase-C current amplitude */

    uint16_t estTheta0;                 /*!< Initial position detection process electric angle before polarity check */
    uint16_t estTheta;                  /*!< Initial position detection process electric angle with polarity check */
    uint16_t estThetaBuf[1 << INITPOS_CALC_TIMES_ORDER];    /*!< Initial position detection process electric angle buffer */

} INITPOS_DETEC_PARAM;

/*!
* @brief Initial position detection structure
*/
typedef struct
{
    INITPOS_DETEC_PARAM *initPosDetec;
    FFT_PARAM_Q15 *fftParam;
} INITPOS_DETEC_STRUCT;

/*!
* @brief Initial position detection configuration data
*/
typedef struct
{
    uint32_t injVoltFreq;               /*!< Initial position detection voltage signal frequency */
    uint16_t injVoltAmp;                /*!< Initial position detection voltage signal amplitude */
    uint16_t pulseAmp;                  /*!< Initial position detection voltage pulse amplitude */
    uint16_t pulseTime;                 /*!< Initial position detection voltage pulse duration time */
    uint16_t pulseClearTime;            /*!< Initial position detection voltage pulse clear time */
    uint16_t pwmFreqDef;                /*!< Initial position detection process PWM frequency default value */
    uint16_t pwmPrdDef;                 /*!< Initial position detection process PWM period default value */
} INITPOS_DETEC_CFG;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Initial position pulse inject contain two parts, pulse width self-learn and pulse injection.
*        This function can get proper pulse injection width, which is different with each motor.
*        Cann't call PulseInj_SetChlVal, otherwiese enter ad isr will call this function immediately again,
*        and will result to first fulse output err.
*
* @param[in] pulseInj: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_SelfLearn(PULSE_INJECT_DATA *pulseInj);

/*!
* @brief After pulse injection, get six direction bus current, base on saturation characteristic of inductance, calculate rotor initial position
*        Initial position pulse inject data handle, output initial position sector.
*        This function handle data only need bus current, so initial position check function is also fit to bldc control.
*
* @param[in] pulseInj: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_DataHandle(PULSE_INJECT_DATA *pulseInj);

/*!
* @brief Initial position pulse inject ADC interrupt function, it mainly contain two parts, pulse width self-learn and pulse injection.
*        This function is only for initial position check function. The interrupt address will recover after get initial position.
*
* @param[in] pulseInj: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @param[in] pulseInjCfg: pulseInject config variable, pointer to PULSE_INJ_CFG structure
* @return none
*/
void InitPos_AdcSubCallBack(PULSE_INJECT_DATA *pulseInj, PULSE_INJ_CFG *pulseInjCfg);

/*!
* @brief Output initial position check function rotor position
*
* @param[in] pulseInj: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return elecTheta: angle get from pulse injection
*/
uint16_t InitPos_GetPosAngle(PULSE_INJECT_DATA *pulseInj);

/*!
* @brief Initialization of init-position check function.
*
* @param[in] pInitPos: pointer to INITPOS_DETEC_STRUCT structure
* @param[in] polar: pointer to INIT_POSITION_POLARITY_CHECK structure
* @return none
*/
void InitPosDetec_Init(INITPOS_DETEC_STRUCT *pInitPos, INIT_POSITION_POLARITY_CHECK *polar);

/*!
* @brief Voltage Command generation of init-position check.
*
* @param[in] pInitPos: pointer to INITPOS_DETEC_STRUCT structure
* @param[in] stc: pointer to STC_PARAM structure
* @return none
*/
void InitPosDetec_CommandGen(INITPOS_DETEC_STRUCT *initpos, STC_PARAM *stc);

/*!
* @brief Main process in 1ms period of init-position check.
*
* @param[in] pInitPos: pointer to INITPOS_DETEC_STRUCT structure
* @param[in] initposCfg: pointer to INITPOS_DETEC_CFG structure
* @param[in] stc: pointer to STC_PARAM structure
* @return returnVal: init-position process finish flag
*/
uint8_t InitPosDetec_1msTask(INITPOS_DETEC_STRUCT *initpos, INITPOS_DETEC_CFG *initposCfg, STC_PARAM *stc);

/*!
* @brief Long period process of init-position check.
*
* @param[in] pInitPos: pointer to INITPOS_DETEC_STRUCT structure
* @param[in] stc: pointer to STC_PARAM structure
* @return none
*/
void InitPosDetec_LongPeriodTask(INITPOS_DETEC_STRUCT *pInitPos, STC_PARAM *stc);

/*!
* @brief ADC interrupt execution process of init-position check.
*
* @param[in] pInitPos: pointer to INITPOS_DETEC_STRUCT structure
* @param[in] polar: pointer to INIT_POSITION_POLARITY_CHECK structure
* @param[in] cfg: pointer to INITPOS_DETEC_CFG structure
* @param[in] stc: pointer to STC_PARAM structure
* @return none
*/
void InitPosDetec_IsrTask(INITPOS_DETEC_STRUCT *pInitPos, INIT_POSITION_POLARITY_CHECK *polar, INITPOS_DETEC_CFG *cfg, STC_PARAM *stc);

/* ======================================  Functions define  ======================================== */

#endif /* __INIT_POS_H__ */

/* =============================================  EOF  ============================================== */
