/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __SHORT_BRAKE_H__
#define __SHORT_BRAKE_H__

/*!
* @file short_brake.h
*
* @brief This file provides three phase short brake and electric brake functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "param_init.h"
#include "pid_regulator.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Short brake function status.
*/
typedef enum {
    INIT_CONFIG = 0,                   /*!< Step 0: PWM ADC CTU configuration */
    SHORT_BRAKING,                     /*!< Step 1: short brake */
    CONFIG_RESET,                      /*!< Step 2: short brake finished and reset PWM ADC CTU configuration */
    BRAKE_ERR,                         /*!< ERR status when error during short brake occurs */
} SHORT_BRAKE_STATUS;

/*!
* @brief Short brake configuration structure.
*/
typedef struct
{
    uint8_t shortBrakeCurRatio;        /*!< Short brake current */
    uint8_t shortBrakeEnable;          /*!< Short brake function enable */
    uint16_t shortBrakeUdcMax;         /*!< Short brake max Udc voltage */
    uint16_t shortBrakeDoneFlagTs;     /*!< Short brake finished waite time */
    int16_t shortQuitCur;              /*!< Short brake quit current */
    uint32_t shortTsCnt;               /*!< Short brake time for singleR sample */
} SHORT_BRAKE_CFG;

/*!
* @brief Short brake control data structure.
*/
typedef struct
{
    uint8_t subStatus;                 /*!< Short brake run status */
    uint16_t cnt1ms;                   /*!< Short brake 1ms counter */
    uint16_t maxCur;                   /*!< Short brake max current */
    uint32_t doneFlagTsCnt;            /*!< Short brake finished waite time */
    uint32_t shortTime;                /*!< Short brake time */
} SHORT_BRAKE_DATA;

/*!
* @brief Electric brake parameters structure.
*/
typedef struct
{
    uint8_t enterElecBrakeFlag;        /*!< Enter electric brake flag */
    uint8_t pwmOnFlag;                 /*!< Electric brake pwm on flag */
    uint16_t pwmOnCnt;                 /*!< Electric brake pwm on count */
    uint16_t pwmOffCnt;                /*!< Electric brake pwm off count */
    uint16_t maxCur;                   /*!< Electric brake max current */
    int32_t depthGain;                 /*!< Electric brake depth gain */
    int32_t iqCof;                     /*!< Q axis current coefficient */
} ELEC_BRAKE_DATA;

/*!
* @brief Electric brake configuration structure.
*/
typedef struct
{
    uint8_t isqAddStep;                /*!< Isq add step */
    uint8_t isqDecStep;                /*!< Isq decrease step */
    uint8_t elecBrakeCurRatio;         /*!< Electric brake current ratio */
    uint8_t elecBrakeUdcMax;           /*!< Electric brake udcmax */
    uint16_t brakeDepth1;              /*!< Enter electric brake depth */
    uint16_t brakeDepth2;              /*!< Exit electric brake depth */
    int16_t elecBrakeFre1;             /*!< Enter electric brake frequency */
    int16_t elecBrakeFre2;             /*!< Exit electric brake frequency */
    uint16_t elecBrakeDepth;           /*!< Electric brake depth */
    uint16_t elecBrakeMaxDepth;        /*!< Electric brake max depth */
} ELECBRAKE_CFG;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Clear short brake data struct. This function will be called in FOC_Parameters_Initialize.
*
* @param[in] shortBrake: short brake data variable, pointer to SHORT_BRAKE_DATA struct
* @return none
*/
void ShortBrake_Init(SHORT_BRAKE_DATA *shortBrake);

/*!
* @brief Short brake function main status machine, include four status.
*        INIT_CONFIG: pwm and adc configuration initilize.
*        SHORT_BRAKING: short braking control process.
*        CONFIG_RESET: short brakeing function finished, configuration recover.
*        BRAKE_ERR: in short braking process, maybe occur error.
*
* @param[in] shortBrake: short brake data variable, pointer to SHORT_BRAKE_DATA struct
* @param[in] shortBrakeCfg: short brake config variable, pointer to SHORT_BRAKE_CFG struct
* @param[in] focVarsCfg: variable which use to configure FOC control, pointer to FOC_VARS_CFG structure
* @return none
*/
void ShortBrake_RunCaseBrake(SHORT_BRAKE_DATA *shortBrake, SHORT_BRAKE_CFG *shortBrakeCfg, FOC_VARS_CFG *focVarsCfg);

/*!
* @brief Short brake ADC interrupt service function to prevent over current and over voltage.
*        Modefiy short brake settings can change short brake power.
*        In particular, short brake is not fit for signal shunt current restruct mode.
*
* @param[in] shortBrake: short brake data variable, pointer to SHORT_BRAKE_DATA struct
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] shortBrakeCfg: short brake config variable, pointer to SHORT_BRAKE_CFG struct
* @param[in] motorParam: pointer to MOTOR_PARAM_PU structure
* @return none
*/
void ShortBrake_AdcSubCallBack(SHORT_BRAKE_DATA *shortBrake, FOC_VARS *focVars, SHORT_BRAKE_CFG *shortBrakeCfg, MOTOR_PARAM_PU *motorParam);

/*!
* @brief Electric brake variables initialize, usually it is initialized only once before running.
*
* @param[in] elecBrake: pointer to ELEC_BRAKE_DATA structure
* @param[in] eleBrakeCfg: pointer to ELECBRAKE_CFG structure
* @return none
*/
void ElecBrake_Init(ELEC_BRAKE_DATA *elecBrake, ELECBRAKE_CFG *eleBrakeCfg);

/*!
* @brief Electric brake feature calculate, according to the external brake depth to change
*        the q axis current, similar to energy recovery function.
*
* @param[in] freq: motor electric frequence pu value
* @param[in] elecBrake: pointer to ELEC_BRAKE_DATA structure
* @param[in] eleBrakeCfg: pointer to ELECBRAKE_CFG structure
* @param[in] STC: pointer to STC_PARAM structure
* @param[in] dPidOut: pointer to d axis PID_REGULATOR_OUT structure
* @param[in] qPidOut: pointer to q axis PID_REGULATOR_OUT structure
* @return none
*/
void ElecBrake_Calc(int16_t freq, ELEC_BRAKE_DATA *elecBrake, ELECBRAKE_CFG *eleBrakeCfg, STC_PARAM *STC, PID_REGULATOR_OUT *dPidOut, PID_REGULATOR_OUT *qPidOut);

/* ======================================  Functions define  ======================================== */

#endif /* __SHORT_BRAKE_H__ */

/* =============================================  EOF  ============================================== */
