/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __BLDC_UI_UART_H__
#define __BLDC_UI_UART_H__
/*!
* @file bldc_ui_uart.h
*
* @brief This file provides UART communication with user interface of host computer functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac7840x.h"
#include "uart_drv.h"

/* ============================================  Define  ============================================ */
//#define TOOL_UART_FUNCTION                      /*!< If this macro definition is defined, the Host PC tool UART function will be enabled, otherwise it will be closed */
#define TOOL_UART_BAUDRATE      230400          /*!< UART communication baud rate */

#define RX_BUF_SIZE             256             /*!< UART RX buffer size, unit: byte */
#define TX_BUF_SIZE             256             /*!< UART TX buffer size, unit: byte */
#define UI_REG_BUF_SIZE         256             /*!< UART register buffer size, unit: byte */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Enumeration of motor control modes.
*/
typedef enum {
    REV0 = 0,                                   /*!< Reserve */
    REV1,                                       /*!< Reserve */
    SQUARE_HALL,                                /*!< BLDC controlled with Hall sensor */
    SQUARE_SENSORLESS,                          /*!< BLDC controlled with sensorless */
    SINE_HALL,                                  /*!< FOC with Hall sensor */
    SINE_HALL_ENCODER,                          /*!< FOC with Hall and Encoder sensors */
    SINE_SENSORLESS,                            /*!< FOC with sensorless */
    SINE_ENCODER                                /*!< FOC with Encoder sensor */
} MOTOR_MAIN_MODE;

/*!
* @brief Enumeration of sensorless FOC subitems.
*/
typedef enum {
    DEFAULT = 0,                                /*!< Default */
    SMO_ATAN,                                   /*!< Sliding mode observer algorithm with obtaining angle by arctan method */
    SMO_PLL,                                    /*!< Sliding mode observer algorithm with obtaining angle by PLL method */
    FLUX_ATAN,                                  /*!< Flux observer algorithm with obtaining angle by arctan method */
    FLUX_PLL                                    /*!< Flux observer algorithm with obtaining angle by PLL method */
} SINE_SENSORLESS_SUB_MODE;

/*!
* @brief UART buffer parameters structure.
*/
typedef struct
{
    uint16_t uartBufSize;                       /*!< UART buffer fifo size */
    uint16_t uartBufWrIndex;                    /*!< UART buffer fifo write ID */
    uint16_t uartBufRdIndex;                    /*!< UART buffer fifo read ID */
    uint16_t uartBufCnt;                        /*!< UART buffer data count */
    uint8_t  uartBufOvf;                        /*!< UART buffer fifo overflow flag */
    uint8_t  *uartBufAddr;                      /*!< UART buffer base address */
} UART_BUF;

/*!
* @brief System operation states structure.
*/
typedef struct
{
    uint8_t communicateStatus;                  /*!< Communication success or not */
    uint8_t getVersionStatus;                   /*!< Get version success or not */
    uint8_t startMotorStatus;                   /*!< Start motor success or not */
    uint8_t stopMotorStatus;                    /*!< Stop motor success or not */
    uint8_t stopRampStatus;                     /*!< Stop ramp success or not */
    uint8_t identifyStatus;                     /*!< Parameter identification success or not */
    uint8_t cmdProcStatus;                      /*!< Command process status 0x00(0%) -- 0x64(100%) */
    uint8_t identifyOffStatus;                  /*!< Turn off parameter identification success or not */
    uint8_t clearFaultStatus;                   /*!< Clear fault success or not */
    uint8_t readRegStatus;                      /*!< Read register success or not */
    uint8_t writeRegStatus;                     /*!< Write register success or not */
    uint8_t checkeErrStatus;                    /*!< Command check error status */
} CMD_PROC_STATUS;

/*!
* @brief UART read and write parameters structure.
*/
typedef struct
{
    uint8_t uiRdBuf[11];                        /*!< Save data length and data ID from read register command */
    uint8_t uiWrBuf[11];                        /*!< Save data length and data ID from write register command */
    uint8_t WriteSpeedValid;                    /*!< Write speed valid flag */
    uint8_t uiIdentifyBuf[12];                  /*!< Save motor parameter from identification command */
    uint8_t uiCheckCmd;                         /*!< Save UI command while checksum error */

    uint8_t uartTimeOutCntEn;                   /*!< Enable time count while UART process a package data */
    uint16_t uartTimeOutCnt;                    /*!< Count time in UART process a package data, unit: ms */
    uint8_t cmdProcTimeOutCntEn;                /*!< Enable time count while execute master command */
    uint16_t cmdProcTimeOutCnt;                 /*!< Count time in execute master command, unit: ms */
    uint8_t cmdReadCnt;                         /*!< Count receive read command times */
} CMD_PROC_VAR;

/*!
* @brief UART parameters transmition structure.
*/
typedef struct
{
    uint16_t appMode;                           /*!< Motor algorithm mode */
    uint16_t fault1;                            /*!< Motor protector error high word */
    uint16_t fault2;                            /*!< Motor protector error low word */
    uint16_t targetMotor;                       /*!< Control motor number */
    uint16_t status;                            /*!< Motor run states */
    uint16_t controlMode;                       /*!< Motor control mode */
    int16_t motorDir;                           /*!< Motor run direction */
    uint16_t vbus;                              /*!< Motor Vbus */
    uint16_t ibus;                              /*!< Motor Ibus */
    uint16_t motorPower;                        /*!< Motor power */
    int16_t speedRef;                           /*!< Ref in speed loop */
    int16_t speedFbk;                           /*!< Fdk in speed loop */
    int16_t speedRampEnd;                       /*!< Speed ramp speed at last */
    uint16_t accelerateTime;                    /*!< Acc time while speed ramp,unit: ms */
    uint16_t decelerateTime;                    /*!< Decelerate time while speed ramp,unit: ms */
    int16_t itorqueRef;                         /*!< Ref in q axis current loop */
    int16_t itorqueFdk;                         /*!< Fdk in q axis current loop */
    int16_t ifluxRef;                           /*!< Ref in d axis current loop */
    int16_t ifluxFdk;                           /*!< Fdk in d axis current loop */
    int16_t speedKp;                            /*!< Kp in speed loop */
    int16_t speedKi;                            /*!< Ki in speed loop */
    int16_t speedKd;                            /*!< Kd in speed loop */
    int16_t torqueKp;                           /*!< Kp in q axis current loop */
    int16_t torqueKi;                           /*!< Ki in q axis current loop */
    int16_t torqueKd;                           /*!< Kd in q axis current loop */
    int16_t fluxKp;                             /*!< Kp in d axis current loop */
    int16_t fluxKi;                             /*!< Ki in d axis current loop */
    int16_t fluxKd;                             /*!< Kd in d axis current loop */
    uint16_t hallStatus;                        /*!< Measure hall status in hall FOC mode */
    uint16_t hallEstSector;                     /*!< Est hall status in hall FOC mode */
    uint16_t swapState;                         /*!< Startup status in hall FOC mode */
    int16_t estFre;                             /*!< Est frequency in hall FOC mode */
    uint16_t brakeStatus;                       /*!< Status in short barke mode */
    uint16_t brakeWaitTime;                     /*!< Wait time after short barke mode */
    uint16_t pulseInjDoneFlag;                  /*!< InitPos function finish flag */
    uint16_t flowState;                         /*!< InitPos function identify state */
    uint16_t initPosStatus;                     /*!< Status in initPos function */
    uint16_t initPosCur1;                       /*!< Pulse injection bus current1 in initPos function */
    uint16_t initPosCur2;                       /*!< Pulse injection bus current2 in initPos function */
    uint16_t initPosCur3;                       /*!< Pulse injection bus current3 in initPos function */
    uint16_t initPosCur4;                       /*!< Pulse injection bus current4 in initPos function */
    uint16_t initPosCur5;                       /*!< Pulse injection bus current5 in initPos function */
    uint16_t initPosCur6;                       /*!< Pulse injection bus current6 in initPos function */
    uint16_t startUpFlag;                       /*!< StartUp flag in sensorless FOC mode */
    uint16_t reachFrqFlag;                      /*!< Reach sync freq flag in sensorless FOC mode */
    int16_t spdSmoothCur;                       /*!< Smooth current while switch close loop in sensorless FOC mode */
    int16_t synSpd;                             /*!< Sync mechanics freq in sensorless FOC mode */
    uint16_t openCurrent;                       /*!< IF startUp current in sensorless FOC mode */
    uint16_t alignTime;                         /*!< Alignment time in sensorless FOC mode */
    uint16_t alignCurrent;                      /*!< Alignment current in sensorless FOC mode */
    int16_t dcpUdComp;                          /*!< Compensation Ud value in FOC feedforward decoup */
    int16_t dcpUqComp;                          /*!< Compensation Uq value in FOC feedforward decoup */
    uint16_t spdAcceleration;                   /*!< Elec speed Acceleration in FOC mode */
    uint16_t spdDeceleration;                   /*!< Elec speed Deceleration in FOC mode */
    int16_t smoKslid;                           /*!< Smo observer gain in startup stage */
    int16_t smoKslf;                            /*!< Smo observer filter factor in startup stage */
    int16_t smoErrMax;                          /*!< Smo observer maxerr in startup stage */
    int16_t fluxWeakKp;                         /*!< Pid kp in fluxweaken */
    int16_t fluxWeakKi;                         /*!< Pid ki in fluxweaken */
    int16_t fluxWeakBusRef;                     /*!< Vbus ref in fluxweaken */
    int16_t fluxWeakBusFdk;                     /*!< Vbus fdk in fluxweaken */
    uint16_t heatTemperature;                   /*!< Motor temperature */
    uint16_t dacOut1;                           /*!< DAC output id in channel 1 */
    uint16_t dacOut2;                           /*!< DAC output id in channel 2 */
    int16_t ia;                                 /*!< Currrent in phase A */
    int16_t ib;                                 /*!< Currrent in phase B */
    int16_t ic;                                 /*!< Currrent in phase C */
    int16_t ialpha;                             /*!< Currrent fdk in alpha axis */
    int16_t ibeta;                              /*!< Currrent fdk in beta axis */
    int16_t id;                                 /*!< Currrent fdk in d axis */
    int16_t iq;                                 /*!< Currrent fdk in q axis */
    uint16_t is;                                /*!< Currrent fdk Is */
    int16_t idRef;                              /*!< Currrent ref in d axis */
    int16_t iqRef;                              /*!< Currrent ref in q axis */
    int16_t ud;                                 /*!< Ud */
    int16_t uq;                                 /*!< Uq */
    uint16_t udq;                               /*!< Udq */
    int16_t valpha;                             /*!< Ualpha */
    int16_t vbeta;                              /*!< Ubeta */
    uint16_t elecAngle;                         /*!< Angle in elec cycle */
    int16_t measureSpeed;                       /*!< Measure speed ,speed fdk */
    uint16_t observeEstAngle;                   /*!< Observer est angle */
    int16_t pllEstSpeed;                        /*!< Pll est speed */
    int16_t observeIalpha;                      /*!< Ialpha in observer */
    int16_t observeIbeta;                       /*!< Ibeta in observer */
    int16_t observeEalpha;                      /*!< Ealpha in observer */
    int16_t observeEbeta;                       /*!< Ebeta in observer */
    int16_t fluxFiltCoeff;                      /*!< Filter coeff in flux observer */
    uint16_t fluxCompLimt1;                     /*!< Compensation limit1 in flux observer */
    uint16_t fluxCompLimt2;                     /*!< Compensation limit2 in flux observer */
    int16_t pllkp;                              /*!< Pll P coeff */
    int16_t pllki;                              /*!< Pll I coeff */
    uint16_t hfiEstAngle;                       /*!< Hfi est angle */
    int16_t hfiEstSpeed;                        /*!< Hfi est speed */
    int16_t hfiKp;                              /*!< Hfi pid kp */
    int16_t hfiKi;                              /*!< Hfi pid ki */
    int16_t currentLoopKp;                      /*!< Kp in bus current loop */
    int16_t currentLoopKi;                      /*!< Ki in bus current loop */
    int16_t currentLoopKd;                      /*!< Kd in bus current loop */
    uint16_t currentOffset;                     /*!< Bus current offset */
    int16_t currentRef;                         /*!< Bus current loop reference */
    int16_t currentFdk;                         /*!< Bus current loop feedback */
    uint16_t bldcDuty;                          /*!< Pwm output duty in bldc mode,units: 0.01% */
    uint16_t bldcSector;                        /*!< Hall sector in bldc mode */
    uint16_t appVerYear;                        /*!< App version year */
    uint16_t appVerDate;                        /*!< App version month and date */
    uint16_t appVerMain;                        /*!< App main version */
    uint16_t appVerRelease;                     /*!< App release version */
    uint16_t libVerYear;                        /*!< Foc lib version year */
    uint16_t libVerDate;                        /*!< Foc lib version month and date */
    uint16_t libVerMain;                        /*!< Foc lib main version */
    uint16_t libVerRelease;                     /*!< Foc lib release version */
} UI_COMMUNICATION_PARAM;

/* ==========================================  Variables  =========================================== */

extern UART_BUF g_uiRxBuf;                      /*!< UART receive buffer struct */
extern UART_BUF g_uiTxBuf;                      /*!< Define UI uart interactive tx buffer struct */

extern UI_COMMUNICATION_PARAM g_uiCommParam;    /*!< UI Register param table struct */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Tool Uart peripheral initialization.
*
* @param[in] none
* @return none
*/
void UI_ToolUart_Init(void);

/*!
* @brief Uart communication register table initialization.
*
* @param[in] none
* @return none
*/
void UI_RegBuf_Init(void);

/*!
* @brief Uart process command recieved from master according to communication protocol.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_UART_ToolCmd_Proc(UART_BUF *uiRxBuf);

/*!
* @brief Uart communication consume time statistics,units: 2ms.
*
* @param[in] none
* @return none
*/
void UI_UART_TimeOut_Cnt(void);

/*!
* @brief master command process time statistics,units: 2ms.
*
* @param[in] none
* @return none
*/
void UI_MasterCmd_ProcCnt(void);

/*!
* @brief UART receive interrrupt isr,occur when receice a char from master.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_UART_RxISR_Task(UART_BUF *uartRx);

/*!
* @brief UART transmit interrrupt isr,occur when transmit a char to master.
*
* @param[in] uartTx: pointer to UART_BUF structure, store the data send out to master
* @return none
*/
void UI_UART_TxISR_Task(UART_BUF *uartTx);

/* ======================================  Functions define  ======================================== */

#endif /* __BLDC_UI_UART_H__ */

/* =============================================  EOF  ============================================== */
