/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_ui_uart.c
*
* @brief This file provides UART communication with user interface of host computer functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "string.h"
#include "gpio_drv.h"
#include "bldc_ui_uart.h"
#include "bldc_app.h"
#include "bldc_keys.h"

/* ============================================  Define  ============================================ */
#define CMD_HEAD_BYTE                   0xAA                        /*!< Head byte in a package command */

#define CMD_CHECK_COMMUNICATION         0x00                        /*!< Command used to confirm communication */
#define CMD_GET_CODE_VERSION            0x01                        /*!< Command used to get code version */
#define CMD_START_MOTOR                 0x02                        /*!< Command used to start motor */
#define CMD_STOP_MOTOR                  0x03                        /*!< Command used to stop motor */
#define CMD_STOP_RAMP                   0x04                        /*!< Command used to stop ramp */
#define CMD_START_PARAM_IDENTIFY        0x05                        /*!< Command used to start motor param identify */
#define CMD_STOP_PARAM_IDENTIFY         0x06                        /*!< Command used to stop motor param identify */
#define CMD_MOTOR_FAULT_CLEAR           0x07                        /*!< Command used to clear fault */

#define CMD_FLASH_READ                  0x08                        /*!< Command used to read flash page to recovery data to variables */
#define CMD_FLASH_SAVE                  0x09                        /*!< Command used to save data to flash page */

#define CMD_START_HALLSELFLEARN         0x0A                        /*!< Command used to start motor hall sensor self learning */
#define CMD_STOP_HALLSELFLEARN          0x0B                        /*!< Command used to stop motor hall sensor self learning */

#define CMD_READ_REGISTER               0x10                        /*!< Command used to read param in register table from Mcu */
#define CMD_WRITE_REGISTER              0x11                        /*!< Command used to write param in register table to Mcu */
#define CMD_QUERY_PROCESS               0x12                        /*!< Command used to get command execute schedule from Mcu */

#define RECEIVE_TIM_MAXCNT              15000                       /*!< Recieved master data time count, execution time base is 2ms */
#define START_MOTOR_MAXCNT              1000                        /*!< Start motor time count, execution time base is 2ms */
#define START_IDENTIFY_MAXCNT           15000                       /*!< Identify process time count, execution time base is 2ms */


/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
/* interactive buffer */
uint8_t g_uartRxBuf[RX_BUF_SIZE];
UART_BUF g_uiRxBuf = {RX_BUF_SIZE, 0, 0, 0, 0, g_uartRxBuf};        /*!< Define UI uart interactive rx buffer struct */

uint8_t g_uartTxBuf[TX_BUF_SIZE];
UART_BUF g_uiTxBuf = {TX_BUF_SIZE, 0, 0, 0, 0, g_uartTxBuf};        /*!< Define UI uart interactive tx buffer struct */

/* register */
uint16_t *g_uiRegBuf[UI_REG_BUF_SIZE];                              /*!< Define UI uart interactive register pointer,point to g_uiCommParam */
UI_COMMUNICATION_PARAM g_uiCommParam;                               /*!< Define UI uart interactive param table */

/* communicate command execute */
CMD_PROC_STATUS g_cmdProcStatus ={0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};      /*!< Master command execute status */
CMD_PROC_VAR g_cmdProcVar;                                          /*!< Master command execute intermediate variable */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Mcu transmit a char to master through Uart.
*
* @param[in] uartTx: pointer to UART_BUF structure, store the data send out to master
* @param[in] data: the char to be send out to master
* @return none
*/
void UI_UART_PutChar(UART_BUF *uartTx, uint8_t data)
{
    UART_Type *localUartRegs = (UART_Type*)UART1_BASE;
    if ((*uartTx).uartBufCnt >= (*uartTx).uartBufSize)
    {
        (*uartTx).uartBufCnt = 0;
        (*uartTx).uartBufRdIndex = (*uartTx).uartBufWrIndex;
    }
    /* close interrrrupt */
    UART1->IER &= (~UART_IER_ETC_Msk);

    /* send first char */
    if ((*uartTx).uartBufCnt == 0)
    {
        if (UART1->LSR0 & UART_LSR0_TXNF_Msk)
//        if (UART_TxIsEmpty(UART1))
        {
            localUartRegs->RBR = (data & (0x00FF));
            /* open interrrrupt */
            UART1->IER |= UART_IER_ETC_Msk;
            return;
        }
    }

    *((*uartTx).uartBufAddr + (*uartTx).uartBufWrIndex) = data;
    if (++(*uartTx).uartBufWrIndex == (*uartTx).uartBufSize)
    {
        (*uartTx).uartBufWrIndex = 0;
    }
    ++(*uartTx).uartBufCnt;
    /* open interrrrupt */
    UART1->IER |= UART_IER_ETC_Msk;
}

/*!
* @brief UART read a char in current recieve buffer pointer,do not delete from buffer.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @param[in] num: recieced data ID in the recieve buffer
* @return a char read from the recieve buffer
*/
uint8_t UI_UART_ReadChar(UART_BUF *uartRx, uint16_t num)
{
    uint8_t data;
    uint16_t i;
    i = num;
    i += (*uartRx).uartBufRdIndex;
    if (i > (*uartRx).uartBufSize - 1)
    {
        i -= (*uartRx).uartBufSize;
    }
    data = *((*uartRx).uartBufAddr  + i);
    return data;
}

/*!
* @brief UART get a char in current recieve buffer pointer,and delete from buffer.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @return a char get from the recieve buffer
*/
uint8_t UI_UART_GetChar(UART_BUF *uartRx)
{
    uint8_t data;
    data = *((*uartRx).uartBufAddr + (*uartRx).uartBufRdIndex);
    if ((*uartRx).uartBufCnt != 0)
    {
        if (++(*uartRx).uartBufRdIndex == (*uartRx).uartBufSize)
        {
            (*uartRx).uartBufRdIndex = 0;
        }

        /* close interrrrupt, disable RXNE interrupt, ERXNE = 0 */
        UART1->LSR0 &= (~UART_IER_ERXNE_Msk);
//        UART_SetRXNEInterrupt(UART1, DISABLE);

        --(*uartRx).uartBufCnt;

        /* open interrrrupt, enable RXNE interrupt, ERXNE = 1 */
        UART1->IER |= UART_IER_ERXNE_Msk;
//        UART_SetRXNEInterrupt(UART1, ENABLE);
    }
    return data;
}

/*!
* @brief UART delete a char in current recieve buffer pointer.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @param[in] num: the numbers of char to be deleted
* @return none
*/
void UI_UART_DeleteChar(UART_BUF *uartRx, uint16_t num)
{
    uint16_t i;
    if ((*uartRx).uartBufCnt >= num)
    {
        /* close interrrrupt */
        UART1->LSR0 &= (~UART_IER_ERXNE_Msk);
//        UART_SetRXNEInterrupt(UART1, DISABLE);

        (*uartRx).uartBufCnt -= num;

        /* open interrrrupt */
        UART1->IER |= UART_IER_ERXNE_Msk;
//        UART_SetRXNEInterrupt(UART1, ENABLE);

        i = num;
        i += (*uartRx).uartBufRdIndex;

        if (i >= (*uartRx).uartBufSize)
        {
            i -= (*uartRx).uartBufSize;
        }
        (*uartRx).uartBufRdIndex = i;
    }
    else
    {
        /* close interrrrupt */
        UART1->LSR0 &= (~UART_IER_ERXNE_Msk);
//        UART_SetRXNEInterrupt(UART1, DISABLE);

        i = (*uartRx).uartBufCnt;
        (*uartRx).uartBufCnt = 0;

        /* open interrrrupt */
        UART1->IER |= UART_IER_ERXNE_Msk;
//        UART_SetRXNEInterrupt(UART1, ENABLE);

        i += (*uartRx).uartBufRdIndex;
        if (i >= (*uartRx).uartBufSize)
        {
            i -= (*uartRx).uartBufSize;
        }
        (*uartRx).uartBufRdIndex = i;
    }
}

/*!
* @brief UART receive interrrupt isr,occur when receice a char from master.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_UART_RxISR_Task(UART_BUF *uartRx)
{
    uint8_t data;

    /* ERXNE has been enable,and data has been recelved on rx pin */
    if (UART1->IER & UART_IER_ERXNE_Msk)
    {
        data = (uint8_t)((UART1->RBR) & UART_RBR_RBR_THR_Msk);
//        data = (uint8_t)UART_ReceiveData(UART1);
        if ((UART1->LSR0 & 0x0E) == 0)
        {
            g_uartRxBuf[g_uiRxBuf.uartBufWrIndex] = data;
            if (++(*uartRx).uartBufWrIndex == g_uiRxBuf.uartBufSize)
            {
                (*uartRx).uartBufWrIndex = 0;
            }
            if (++(*uartRx).uartBufCnt == (*uartRx).uartBufSize)
            {
                (*uartRx).uartBufCnt = 0;
                (*uartRx).uartBufOvf = 1;
                (*uartRx).uartBufRdIndex = 0;
                (*uartRx).uartBufWrIndex = 0;
            }
        }
        else
        {
//            UART_ReceiveData(UART1);
        }

        UART1->LSR0 |= (~UART_LSR0_DR_Msk);
    }
}

/*!
* @brief UART transmit interrrupt isr,occur when transmit a char to master.
*
* @param[in] uartTx: pointer to UART_BUF structure, store the data send out to master
* @return none
*/
void UI_UART_TxISR_Task(UART_BUF *uartTx)
{
    UART_Type *localUartRegs = (UART_Type*)UART1_BASE;

    /* ETC has been enable,and data has been transmitted from tx pin */
    if ((UART1->IER & UART_IER_ETC_Msk))
    {
        /* clear TC flag */
        UART1->LSR0 &= (~UART_LSR0_TC_Msk);
        if ((*uartTx).uartBufCnt)
        {
            --g_uiTxBuf.uartBufCnt;
            localUartRegs->RBR = g_uartTxBuf[g_uiTxBuf.uartBufRdIndex];
            if (++(*uartTx).uartBufRdIndex == (*uartTx).uartBufSize)
            {
                (*uartTx).uartBufRdIndex = 0;
            }
        }
        else
        {
            /* clear ETC */
            UART1->IER &= (~UART_IER_ETC_Msk);
        }
    }
}

/*!
* @brief Check the App version valid or not.
*
* @param[in] none
* @return version valid information : 0 stand for valid, -1 stand for unvalid
*/
int8_t UI_CheckCodeVersionValid()
{
    if ((MOTORAPP_VER_YEAR < 2000) || (MOTORAPP_VER_MONTH > 12) || (MOTORAPP_VER_DAY > 31))
    {
        return -1;
    }
    else
    {
        return 0;
    }
}

/*!
* @brief Uart communicate interrrupt isr,occur when put a char with master.
*
* @param[in] driverState: device state pointer
* @param[in] event: event parameter
* @param[in] userData: user data parameter
* @return none
*/
void UI_UART_txCallback(void *driverState, uint32_t event, void *userData)
{
    /*Tx interrupt*/
    UI_UART_TxISR_Task(&g_uiTxBuf);
}

/*!
* @brief Uart communicate interrrupt isr,occur when get a char with master.
*
* @param[in] driverState: device state pointer
* @param[in] event: event parameter
* @param[in] userData: user data parameter
* @return none
*/
void UI_UART_rxCallback(void *driverState, uint32_t event, void *userData)
{
    /*Rx interrupt*/
    UI_UART_RxISR_Task(&g_uiRxBuf);
}

/*!
* @brief Uart communication register table initialization.
*
* @param[in] none
* @return none
*/
void UI_RegBuf_Init()
{
    static uint16_t readBack_NULL = 0;
    memset(g_uiRegBuf, 0U , sizeof(g_uiRegBuf));
    memset(&g_uiCommParam, 0U, sizeof(g_uiCommParam));
    uint16_t* u16Temp = NULL;
    uint16_t index = 0;

#if (defined BLDC_HALL)
    g_uiCommParam.appMode = (uint16_t)SQUARE_HALL << 8;
#elif (defined BLDC_SENSORLESS)
    g_uiCommParam.appMode = (uint16_t)SQUARE_SENSORLESS << 8;
#else
#endif
    g_uiRegBuf[0] = &g_uiCommParam.appMode;

    u16Temp = (uint16_t*)(&g_bldc_protector.faultFlag.all);
    g_uiRegBuf[1] = u16Temp;
    g_uiRegBuf[2] = u16Temp + 1;

    g_uiRegBuf[3] = (uint16_t*)&g_mcStatus;
    g_uiRegBuf[4] = &g_bldc_adSample.busVoltageTrue;
    g_uiRegBuf[5] = &g_bldc_adSample.busCurTrue;
    g_uiRegBuf[6] = &g_bldc_adSample.busPowerTrue;
    g_uiRegBuf[7] = &g_uiCommParam.heatTemperature;

    g_uiRegBuf[8] = (uint16_t*)(&g_bldc_speedCmd.speedTarget);
    g_uiRegBuf[9] = (uint16_t*)(&g_bldc_speedCmd.speedTargetRamp);
    g_uiRegBuf[10] = (uint16_t*)(&g_bldc_speedCmd.speedFbk);
    g_uiRegBuf[11] = (uint16_t*)(&g_bldc_speedCmd.speedFbkRpm);
    g_uiRegBuf[12] = (uint16_t*)(&g_bldc_speedCmd.speedTargetLimit);
    g_uiRegBuf[13] = (uint16_t*)(&pBldcVarsCfg->speedMaxLimit);
    g_uiRegBuf[14] = (uint16_t*)(&pBldcVarsCfg->speedMinLimit);

    g_uiRegBuf[15] = (uint16_t*)(&pBldcVarsCfg->motorDir);
    g_uiRegBuf[16] = &pBldcVarsCfg->controlMode;
    g_uiRegBuf[17] = (uint16_t*)(&pBldcVarsCfg->motorAcceleration);
    g_uiRegBuf[18] = (uint16_t*)(&pBldcVarsCfg->motorDeceleration);

    g_uiRegBuf[19] = &readBack_NULL;
    g_uiRegBuf[20] = &readBack_NULL;
    g_uiRegBuf[21] = &readBack_NULL;
    g_uiRegBuf[22] = &readBack_NULL;
    g_uiRegBuf[23] = &readBack_NULL;
    g_uiRegBuf[24] = &readBack_NULL;

    g_uiRegBuf[25] = &defEflashAccessCfg.accessEnable;

    g_uiRegBuf[26] = &readBack_NULL;
    g_uiRegBuf[27] = &readBack_NULL;

    g_uiRegBuf[28] = (uint16_t*)(&defBldcPid.speedKp);
    g_uiRegBuf[29] = (uint16_t*)(&defBldcPid.speedKi);
    g_uiRegBuf[30] = (uint16_t*)(&defBldcPid.speedKd);
    g_uiRegBuf[31] = (uint16_t*)(&defBldcPid.maxSpeed);
    g_uiRegBuf[32] = (uint16_t*)(&defBldcPid.minSpeed);

    g_uiRegBuf[33] = &readBack_NULL;
    g_uiRegBuf[34] = &readBack_NULL;
    g_uiRegBuf[35] = &readBack_NULL;
    g_uiRegBuf[36] = &readBack_NULL;
    g_uiRegBuf[37] = &readBack_NULL;
    g_uiRegBuf[38] = &readBack_NULL;
    g_uiRegBuf[39] = &readBack_NULL;
    g_uiRegBuf[40] = &readBack_NULL;
    g_uiRegBuf[41] = &readBack_NULL;
    g_uiRegBuf[42] = &readBack_NULL;

    g_uiRegBuf[43] = (uint16_t*)(&g_bldc_asrPIIn.refPu);
    g_uiRegBuf[44] = (uint16_t*)(&g_bldc_asrPIIn.fbkPu);
    g_uiRegBuf[45] = (uint16_t*)(&g_bldc_asrPIOut.outPu);

    for (index = 46; index < 199; index++)
    {
        g_uiRegBuf[index] = &readBack_NULL;
    }

    g_uiRegBuf[199] = (uint16_t*)(&defBldcHwBoard.normalVoltage);
    g_uiRegBuf[200] = (uint16_t*)(&defBldcMotorParam.statorRs);
    g_uiRegBuf[201] = (uint16_t*)(&defBldcMotorParam.statorLs);
    g_uiRegBuf[202] = (uint16_t*)(&defBldcMotorParam.ld);
    g_uiRegBuf[203] = (uint16_t*)(&defBldcMotorParam.lq);
    u16Temp = (uint16_t*)(&defBldcMotorParam.bEmf);
    g_uiRegBuf[204] = u16Temp;
    g_uiRegBuf[205] = u16Temp + 1;
    g_uiRegBuf[206] = (uint16_t*)(&defBldcMotorParam.maxCur);
    g_uiRegBuf[207] = (uint16_t*)(&defBldcMotorParam.polePairs);
    u16Temp = (uint16_t*)(&defBldcMotorParam.maxSpeed);
    g_uiRegBuf[208] = u16Temp;
    g_uiRegBuf[209] = u16Temp + 1;

    g_uiRegBuf[210] = (uint16_t*)&(defBldcPid.currentKp);
    g_uiRegBuf[211] = (uint16_t*)&(defBldcPid.currentKi);
    g_uiRegBuf[212] = (uint16_t*)&(defBldcPid.currentKd);
    g_uiRegBuf[213] = (uint16_t*)&defBldcPid.maxCurrent;
    g_uiRegBuf[214] = (uint16_t*)(&defBldcPid.minCurrent);

    g_uiRegBuf[215] = (uint16_t*)(&defBldcVars.initDuty);
    g_uiRegBuf[216] = (uint16_t*)(&defBldcVars.maxDuty);
    g_uiRegBuf[217] = (uint16_t*)(&defBldcVars.minDuty);
    g_uiRegBuf[218] = &readBack_NULL;
    g_uiRegBuf[219] = &readBack_NULL;
    g_uiRegBuf[220] = &readBack_NULL;

    g_uiRegBuf[221] = (uint16_t*)(&defBemfVars.prepositionDutyMin);
    g_uiRegBuf[222] = (uint16_t*)(&defBemfVars.prepositionDutyMax);
    g_uiRegBuf[223] = (uint16_t*)(&defBemfVars.prepositionDutyStep);
    g_uiRegBuf[224] = (uint16_t*)(&defBemfVars.prepositionTimeMin);
    g_uiRegBuf[225] = (uint16_t*)(&defBemfVars.prepositionTimeMax);
    g_uiRegBuf[226] = (uint16_t*)(&defBemfVars.prepositionTimeStep);
    g_uiRegBuf[227] = (uint16_t*)(&defBemfVars.kickDutyMax);
    g_uiRegBuf[228] = (uint16_t*)(&defBemfVars.kickDutyStep);
    g_uiRegBuf[229] = (uint16_t*)(&defBemfVars.kickTimeMin);
    g_uiRegBuf[230] = (uint16_t*)(&defBemfVars.kickTimeMax);
    g_uiRegBuf[231] = (uint16_t*)(&defBemfVars.kickDutyStep);

    g_uiRegBuf[232] = &readBack_NULL;
    g_uiRegBuf[233] = &readBack_NULL;
    g_uiRegBuf[234] = &readBack_NULL;
    g_uiRegBuf[235] = &readBack_NULL;
    g_uiRegBuf[236] = &readBack_NULL;
    g_uiRegBuf[237] = (uint16_t*)&g_bldc_adSample.busCurOffset;
    g_uiRegBuf[238] = (uint16_t*)(&g_bldc_acrPid.pPidInput->fbkPu);
    g_uiRegBuf[239] = (uint16_t*)(&g_bldc_acrPid.pPidInput->fbkPu);
    g_uiRegBuf[240] = (uint16_t*)&g_bldcVarsCtrl.pwmOutDuty;
    g_uiRegBuf[241] = &g_hallControl.currentPhase;

    g_uiCommParam.appVerYear = MOTORAPP_VER_YEAR;
    g_uiCommParam.appVerDate = (MOTORAPP_VER_MONTH << 8) + MOTORAPP_VER_DAY;
    g_uiCommParam.appVerMain = (MOTORAPP_MAIN_VER << 8) + MOTORAPP_SUB_VER;
    g_uiCommParam.appVerRelease = (MOTORAPP_REV_VER << 8) + MOTORAPP_BRANCH;
    g_uiCommParam.libVerYear = 0;
    g_uiCommParam.libVerDate = 0;
    g_uiCommParam.libVerMain = 0;
    g_uiCommParam.libVerRelease = 0;
    g_uiRegBuf[242] = &g_uiCommParam.appVerYear;
    g_uiRegBuf[243] = &g_uiCommParam.appVerDate;
    g_uiRegBuf[244] = &g_uiCommParam.appVerMain;
    g_uiRegBuf[245] = &g_uiCommParam.appVerRelease;
    g_uiRegBuf[246] = &g_uiCommParam.libVerYear;
    g_uiRegBuf[247] = &g_uiCommParam.libVerDate;
    g_uiRegBuf[248] = &g_uiCommParam.libVerMain;
    g_uiRegBuf[249] = &g_uiCommParam.libVerRelease;
}

/*!
* @brief Uart return read register result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_RegRead_Result(void)
{
    uint8_t i = 0;
    uint8_t uiRead_lenth = 0;
    uint8_t uiRead_checkSum = 0;
    uint16_t uiRead_returnTemp = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiRead_checkSum += CMD_HEAD_BYTE;

    if (g_cmdProcStatus.readRegStatus == 0x02)
    {
        uiRead_lenth =  5;
        UI_UART_PutChar(&g_uiTxBuf, uiRead_lenth);
        uiRead_checkSum += uiRead_lenth;
        /* read command */
        UI_UART_PutChar(&g_uiTxBuf, CMD_READ_REGISTER);
        uiRead_checkSum += CMD_READ_REGISTER;
        /* read result */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.readRegStatus);
        uiRead_checkSum += g_cmdProcStatus.readRegStatus;
    }
    else if (g_cmdProcStatus.readRegStatus == 0x00)
    {
        uiRead_lenth =  3 * g_cmdProcVar.uiRdBuf[0] + 6;
        UI_UART_PutChar(&g_uiTxBuf, uiRead_lenth);
        uiRead_checkSum += uiRead_lenth;
        /* read result */
        UI_UART_PutChar(&g_uiTxBuf, CMD_READ_REGISTER);
        uiRead_checkSum += CMD_READ_REGISTER;
        /* read result */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.readRegStatus);
        uiRead_checkSum += g_cmdProcStatus.readRegStatus;
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.uiRdBuf[0]);
        uiRead_checkSum += g_cmdProcVar.uiRdBuf[0];
        for (i = 1; i <= g_cmdProcVar.uiRdBuf[0]; i++)
        {
            UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.uiRdBuf[i]);
            uiRead_checkSum += g_cmdProcVar.uiRdBuf[i];
            uiRead_returnTemp = *g_uiRegBuf[g_cmdProcVar.uiRdBuf[i]];
            UI_UART_PutChar(&g_uiTxBuf, (uiRead_returnTemp & 0xFF));
            uiRead_checkSum += (uiRead_returnTemp & 0xFF);
            UI_UART_PutChar(&g_uiTxBuf, ((uiRead_returnTemp >> 8) & 0xFF));
            uiRead_checkSum += ((uiRead_returnTemp >> 8) & 0xFF);
        }
    }
    else
    {
        /* do nothing */
    }
    UI_UART_PutChar(&g_uiTxBuf, uiRead_checkSum);
}

/*!
* @brief Uart return write register result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_RegWrite_Result(void)
{
    uint8_t uiWrite_lenth = 5;
    uint8_t uiWrite_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiWrite_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiWrite_lenth);
    uiWrite_checkSum += uiWrite_lenth;
    /* write command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_WRITE_REGISTER);
    uiWrite_checkSum += CMD_WRITE_REGISTER;
    /* write result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.writeRegStatus);
    uiWrite_checkSum += g_cmdProcStatus.writeRegStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiWrite_checkSum);

}

/*!
* @brief Uart return communicate result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_Communicate_Result(void)
{
    uint8_t uiCom_lenth = 5;
    uint8_t uiCom_checkSum = 0;

    /* refresh communicate status,prepare to answer master */
    g_cmdProcStatus.communicateStatus = 0x00;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiCom_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiCom_lenth);
    uiCom_checkSum += uiCom_lenth;
    /* communication command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_CHECK_COMMUNICATION);
    uiCom_checkSum += CMD_CHECK_COMMUNICATION;
    /* communication result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.communicateStatus);
    uiCom_checkSum += g_cmdProcStatus.communicateStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiCom_checkSum);

    /* deinit communicate status to the init status */
    g_cmdProcStatus.communicateStatus = 0xFF;
}

/*!
* @brief Uart return get code version result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_GetVersion_Result(void)
{
    uint8_t uiGetVersion_lenth = 0;
    uint8_t uiGetVersion_checkSum = 0;
    int8_t codeValidState = -1;

    /* init getversion status */
    g_cmdProcStatus.getVersionStatus = 0xFF;
    codeValidState = UI_CheckCodeVersionValid();
    /* refresh getversion status,prepare to answer master */
    if (codeValidState)
    {
        /* version is unvalid ,get version fail */
        g_cmdProcStatus.getVersionStatus = 0x01;
    }
    else
    {
        /* version is valid ,get version success */
        g_cmdProcStatus.getVersionStatus = 0x00;
    }

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiGetVersion_checkSum += CMD_HEAD_BYTE;
    if (g_cmdProcStatus.getVersionStatus == 0x01)
    {
        uiGetVersion_lenth =  5;
        UI_UART_PutChar(&g_uiTxBuf, uiGetVersion_lenth);
        uiGetVersion_checkSum += uiGetVersion_lenth;
        /* get version command */
        UI_UART_PutChar(&g_uiTxBuf, CMD_GET_CODE_VERSION);
        uiGetVersion_checkSum += CMD_GET_CODE_VERSION;
        /* get version result */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.getVersionStatus);
        uiGetVersion_checkSum += g_cmdProcStatus.getVersionStatus;
    }
    else if (g_cmdProcStatus.getVersionStatus == 0x00)
    {
        uiGetVersion_lenth =  21;
        UI_UART_PutChar(&g_uiTxBuf, uiGetVersion_lenth);
        uiGetVersion_checkSum += uiGetVersion_lenth;
        /* get version command */
        UI_UART_PutChar(&g_uiTxBuf, CMD_GET_CODE_VERSION);
        uiGetVersion_checkSum += CMD_GET_CODE_VERSION;
        /* get version result */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.getVersionStatus);
        uiGetVersion_checkSum += g_cmdProcStatus.getVersionStatus;
        /* App version year */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[242]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[242]) & 0xFF);
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[242] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[242] >> 8) & 0xFF);
        /* App version mounth */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[243] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[243] >> 8) & 0xFF);
        /* App version day */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[243]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[243]) & 0xFF);
        /* App main version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[244] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[244] >> 8) & 0xFF);
        /* App sub version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[244]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[244]) & 0xFF);
        /* App release version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[245] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[245] >> 8) & 0xFF);
        /* App branch */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[245]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[245]) & 0xFF);
        /* Lib version year */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[246]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[246]) & 0xFF);
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[246] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[246] >> 8) & 0xFF);
        /* Lib version mounth */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[247] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[247] >> 8) & 0xFF);
        /* Lib version day */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[247]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[247]) & 0xFF);
        /* Lib main version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[248] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[248] >> 8) & 0xFF);
        /* Lib sub version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[248]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[248]) & 0xFF);
        /* Lib release version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[249] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[249] >> 8) & 0xFF);
        /* Lib branch */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[249]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[249]) & 0xFF);
    }
    else
    {
        /* do nothing */
    }
    UI_UART_PutChar(&g_uiTxBuf, uiGetVersion_checkSum);
}

/*!
* @brief Uart return start motor result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StartMotor_Result(void)
{
    uint8_t uiStartMotor_lenth = 5;
    uint8_t uiStartMotor_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStartMotor_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStartMotor_lenth);
    uiStartMotor_checkSum += uiStartMotor_lenth;
    /* start motor command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_MOTOR);
    uiStartMotor_checkSum += CMD_START_MOTOR;
    /* start motor result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.startMotorStatus);
    uiStartMotor_checkSum += g_cmdProcStatus.startMotorStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStartMotor_checkSum);
}

/*!
* @brief Uart return stop motor result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StopMotor_Result(void)
{
    uint8_t uiStopMotor_lenth = 5;
    uint8_t uiStopMotor_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStopMotor_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStopMotor_lenth);
    uiStopMotor_checkSum += uiStopMotor_lenth;
    /* stop motor command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_STOP_MOTOR);
    uiStopMotor_checkSum += CMD_STOP_MOTOR;
    /* stop motor result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.stopMotorStatus);
    uiStopMotor_checkSum += g_cmdProcStatus.stopMotorStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStopMotor_checkSum);
}

/*!
* @brief Uart return stop ramp result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StopRamp_Result(void)
{
    uint8_t uiStopRamp_lenth = 5;
    uint8_t uiStopRamp_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStopRamp_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStopRamp_lenth);
    uiStopRamp_checkSum += uiStopRamp_lenth;
    /* stop motor command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_STOP_RAMP);
    uiStopRamp_checkSum += CMD_STOP_RAMP;
    /* stop motor result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.stopRampStatus);
    uiStopRamp_checkSum += g_cmdProcStatus.stopRampStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStopRamp_checkSum);
}

/*!
* @brief Uart return identify motor paramerters result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_Identify_Result(void)
{
    uint8_t uiIdentify_lenth = 5;
    uint8_t uiIdentify_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiIdentify_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiIdentify_lenth);
    uiIdentify_checkSum += uiIdentify_lenth;
    /* parameter identification command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_PARAM_IDENTIFY);
    uiIdentify_checkSum += CMD_START_PARAM_IDENTIFY;
    /* parameter identification result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.identifyStatus);
    uiIdentify_checkSum += g_cmdProcStatus.identifyStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiIdentify_checkSum);
}

/*!
* @brief Uart return start motor schedule to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StartMotor_Schedule(void)
{
    uint8_t uiStartMotorSche_lenth = 0;
    uint8_t uiStartMotorSche_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStartMotorSche_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStartMotorSche_lenth);
    uiStartMotorSche_checkSum += uiStartMotorSche_lenth;
    /* Query process command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_QUERY_PROCESS);
    uiStartMotorSche_checkSum += CMD_QUERY_PROCESS;
    /* start motor command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_MOTOR);
    uiStartMotorSche_checkSum += CMD_START_MOTOR;

    /* StartMotor Schedule */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.cmdProcStatus);
    uiStartMotorSche_checkSum += g_cmdProcStatus.cmdProcStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStartMotorSche_checkSum);
}

/*!
* @brief Uart return clear fault result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_ClearFault_Result(void)
{
    uint8_t uiClearFault_lenth = 5;
    uint8_t uiClearFault_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiClearFault_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiClearFault_lenth);
    uiClearFault_checkSum += uiClearFault_lenth;
    /* clear fault command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_MOTOR_FAULT_CLEAR);
    uiClearFault_checkSum += CMD_MOTOR_FAULT_CLEAR;
    /* Clear Fault result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.clearFaultStatus);
    uiClearFault_checkSum += g_cmdProcStatus.clearFaultStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiClearFault_checkSum);
}

/*!
* @brief Uart return check err result to master,occur when check err.
*
* @param[in] none
* @return none
*/
void UI_ReturnCheckErr_Result(void)
{
    uint8_t uiCheckErr_lenth = 5;
    uint8_t uiCheckErr_checkSum = 0;

    switch (g_cmdProcVar.uiCheckCmd)
    {
    case CMD_CHECK_COMMUNICATION:
        g_cmdProcStatus.communicateStatus = 0x04;
        g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.communicateStatus;
        break;

    case CMD_GET_CODE_VERSION:
        g_cmdProcStatus.getVersionStatus = 0x04;
        g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.getVersionStatus;
        break;

    case CMD_START_MOTOR:
        g_cmdProcStatus.startMotorStatus = 0x04;
        g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.startMotorStatus;
        break;

    case CMD_STOP_MOTOR:
        g_cmdProcStatus.stopMotorStatus = 0x04;
        g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.stopMotorStatus;
        break;

    case CMD_STOP_RAMP:
        g_cmdProcStatus.stopRampStatus = 0x04;
        g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.stopRampStatus;
        break;

    case CMD_START_PARAM_IDENTIFY:
        g_cmdProcStatus.identifyStatus = 0x04;
        g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.identifyStatus;
        break;

    case CMD_STOP_PARAM_IDENTIFY:
        g_cmdProcStatus.identifyOffStatus = 0x04;
        g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.identifyOffStatus;
        break;

    case CMD_MOTOR_FAULT_CLEAR:
        g_cmdProcStatus.clearFaultStatus = 0x04;
        g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.clearFaultStatus;
        break;

    case CMD_READ_REGISTER:
        g_cmdProcStatus.readRegStatus = 0x04;
        g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.readRegStatus;
        break;

    case CMD_WRITE_REGISTER:
        g_cmdProcStatus.writeRegStatus = 0x04;
        g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.writeRegStatus;
        break;

    case CMD_QUERY_PROCESS:
        g_cmdProcStatus.writeRegStatus = 0x04;
        g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.writeRegStatus;
        break;

    default:
        break;
    }

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiCheckErr_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiCheckErr_lenth);
    uiCheckErr_checkSum += uiCheckErr_lenth;
    /* command id while checksum err happens */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.uiCheckCmd);
    uiCheckErr_checkSum += g_cmdProcVar.uiCheckCmd;
    /* Check err result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.checkeErrStatus);
    uiCheckErr_checkSum += g_cmdProcStatus.checkeErrStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiCheckErr_checkSum);
}

/*!
* @brief Uart communication consume time statistics,units: 2ms.
*
* @param[in] none
* @return none
*/
void UI_UART_TimeOut_Cnt(void)
{
    if (g_cmdProcVar.uartTimeOutCntEn == 1)
    {
        g_cmdProcVar.uartTimeOutCnt++;
    }
    else if (g_cmdProcVar.uartTimeOutCntEn == 0)
    {
        g_cmdProcVar.uartTimeOutCnt = 0;
    }
    else
    {

    }
}

/*!
* @brief Uart judge whether start motor success or not.
*
* @param[in] none
* @return none
*/
void UI_JudgeStartMotor(void)
{
    if ((g_mcStatus == START) || (g_mcStatus == RUN))
    {
        /* start motor success */
        g_cmdProcStatus.startMotorStatus = 0x00;
        /* stop count time, prepare to answer master */
        g_cmdProcVar.cmdProcTimeOutCntEn = 0;
    }
    else
    {
        /* starting motor, fail */
        g_cmdProcStatus.startMotorStatus = 0x01;
        /* out of start motor excute limit time, still not start ready */
        if (g_cmdProcVar.cmdProcTimeOutCnt > START_MOTOR_MAXCNT)
        {
            /* stop count time, prepare to answer master */
            g_cmdProcVar.cmdProcTimeOutCntEn = 0;
        }
    }

    if (g_cmdProcVar.cmdProcTimeOutCntEn == 0)
    {
        UI_Return_StartMotor_Result();
//         DEBUGMSG(DEBUG_ZONE_INFO, ("Cmd id = %d,start motor status is = %d\r\n", CMD_START_MOTOR, g_cmdProcStatus.startMotorStatus));//sync log
    }
}

/*!
* @brief Uart judge whether identify motor paramerters execute success or not.
*
* @param[in] none
* @return none
*/
void UI_JudgeStartIdentify(void)
{
    if (g_mcStatus == IDENTIFY)
    {
        /* identify start success,prepare to report schedule */
        g_cmdProcStatus.identifyStatus = 0x00;
        /* stop count time,prepare to answer master */
        g_cmdProcVar.cmdProcTimeOutCntEn = 0;
    }

    if (g_cmdProcVar.cmdProcTimeOutCntEn == 0)
    {
        UI_Return_Identify_Result();
    }
}

/*!
* @brief master command process time statistics,units: 2ms.
*
* @param[in] none
* @return none
*/
void UI_MasterCmd_ProcCnt(void)
{
    if (g_cmdProcVar.cmdProcTimeOutCntEn != 0)
    {
        g_cmdProcVar.cmdProcTimeOutCnt++;
        if (g_cmdProcVar.cmdProcTimeOutCntEn == 1)
        {
            UI_JudgeStartMotor();
        }
        else if (g_cmdProcVar.cmdProcTimeOutCntEn == 2)
        {
            UI_JudgeStartIdentify();
        }
        else
        {

        }
    }
    else if (g_cmdProcVar.cmdProcTimeOutCntEn == 0)
    {
        g_cmdProcVar.cmdProcTimeOutCnt = 0;
    }
}

/*!
* @brief Tool Uart peripheral initialization.
*
* @param[in] none
* @return none
*/
void UI_UART_Initialize()
{
    uart_user_config_t uartConfig;
    uart_state_t uartState;

    /*!
    * GPIO Configuration for UART1
    * PC2     --> UART1_Tx
    * PC3     --> UART1_Rx
    */

    /* UART Tx pin */
    GPIO_DRV_SetMuxModeSel(PORTC, (gpio_channel_type_t)9U, PORT_MUX_ALT2);
    /* UART Rx pin */
    GPIO_DRV_SetMuxModeSel(PORTC, (gpio_channel_type_t)8U, PORT_MUX_ALT2);

    memset(&uartConfig, 0U, sizeof(uart_user_config_t));
    memset(&uartState, 0U, sizeof(uart_state_t));
    uartConfig.baudRate = TOOL_UART_BAUDRATE;
    uartConfig.parityMode = UART_PARITY_DISABLED;
    uartConfig.bitCountPerChar = UART_8_BITS_PER_CHAR;
    uartConfig.stopBitCount = UART_ONE_STOP_BIT;
    uartConfig.transferType = UART_USING_INTERRUPTS;
//    uartConfig.fifoByteEn = ENABLE;
//    uartConfig.dmaEn = UART_DMA_TXRX_NONE;
    uartState.txCallback = UI_UART_txCallback;
    uartState.rxCallback = UI_UART_rxCallback;
//    uartConfig.callBack = UI_UART_IRQHandler;
    /* set Uart1 config */
    UART_DRV_Init(((uint32_t)(((uint32_t)(UART1) - UART0_BASE) >> 9)), &uartState, &uartConfig);
//    UART_Init(UART1, &uartConfig);

    /* clear TC, TC_FLAG = 0 */
    UART1->LSR0 &= (~UART_LSR0_TC_Msk);
    /* enable TC interrupt, ETC = 1 */
    UART1->IER |= UART_IER_ETC_Msk;
    /* enable RXNE interrupt, ERXNE = 1 */
    UART1->IER |= UART_IER_ERXNE_Msk;
//    UART_SetRXNEInterrupt(UART1, ENABLE);

    NVIC_ClearPendingIRQ(UART1_IRQn);
    NVIC_EnableIRQ(UART1_IRQn);
    NVIC_SetPriority(UART1_IRQn, 1);

}

/*!
* @brief Tool Uart peripheral initialization.
*
* @param[in] none
* @return none
*/
void UI_ToolUart_Init()
{
    /* Tool uart peripheral init */
    UI_UART_Initialize();

    /* Tool uart communication register table init */
    UI_RegBuf_Init();
}

/*!
* @brief Master start motor through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StartMotor(void)
{
    /* init start motor status */
    g_cmdProcStatus.startMotorStatus = 0xFF;

    g_mcStatus = START;

    /* start start motor excute time count */
    g_cmdProcVar.cmdProcTimeOutCntEn = 1;
}

/*!
* @brief Master stop motor through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StopMotor(void)
{
    /* init stop motor status */
    g_cmdProcStatus.stopMotorStatus = 0xFF;

    g_mcStatus = STOP;

    /* stop motor success, prepare to answer master */
    g_cmdProcStatus.stopMotorStatus = 0x00;
    UI_Return_StopMotor_Result();
}

/*!
* @brief Master stop motor ramp start through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StopRamp(void)
{
    /* init stop ramp status */
    g_cmdProcStatus.stopRampStatus = 0xFF;

    if (g_mcStatus != STOP)
    {
        /* stop ramp */
        g_bldc_speedCmd.speedTarget = g_bldc_speedCmd.speedTargetRamp;
        /* stop ramp success, prepare to answer master */
        g_cmdProcStatus.stopRampStatus = 0x00;
    }
    else
    {
        /* set ramp param,but can not stop at startup,unvalid command */
        g_cmdProcStatus.stopRampStatus = 0x02;
    }

    UI_Return_StopRamp_Result();
}

/*!
* @brief Master query command process schedule through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_QuerySchedule(UART_BUF *uiRxBuf)
{
    uint8_t queryCmd = 0;

    /* init query command excute status */
    g_cmdProcStatus.cmdProcStatus = 0xFF;
    /* get query command id */
    queryCmd = UI_UART_GetChar(uiRxBuf);
    switch (queryCmd)
    {
    case CMD_START_MOTOR:
//        UI_Return_StartMotor_Schedule();
        break;

    case CMD_START_PARAM_IDENTIFY:
        break;

    default:
        break;
    }

}

/*!
* @brief Uart return flash operation result to master.
*
* @param[in] cmd: Tool command
* @return none
*/
#if (defined DATA_ACCESS_FLASH_ENABLE)
void UI_Return_FlashTask_Result(uint8_t cmd)
{
    uint8_t uiFlashResult_length = 5;
    uint8_t uiFlashResult_checkSum = 0;
    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiFlashResult_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiFlashResult_length);
    uiFlashResult_checkSum += uiFlashResult_length;
    /* flash operation command */
    UI_UART_PutChar(&g_uiTxBuf, cmd);
    uiFlashResult_checkSum += cmd;
    /* flash operation result */
    UI_UART_PutChar(&g_uiTxBuf, g_flashTaskFinish);
    uiFlashResult_checkSum += g_flashTaskFinish;
    UI_UART_PutChar(&g_uiTxBuf, uiFlashResult_checkSum);
}
#endif

/*!
* @brief Master recovery data from flash through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @param[in] cmd: Tool command
* @return none
*/
#if (defined DATA_ACCESS_FLASH_ENABLE)
void UI_FlashRecoveryData(UART_BUF *uiRxBuf, uint8_t cmd)
{
    uint16_t tempCmd = 0;
    g_dataAccessFlash.recoveryEnable = 1;
    if (g_mcStatus == IDLE)
    {
        g_mcStatus = FLASH_TASK;
        tempCmd = UI_UART_GetChar(uiRxBuf);
        if (tempCmd == 0)
        {
            g_dataAccessFlash.recoveryMode = 1;
        }
        else if (tempCmd == 1)
        {
            g_dataAccessFlash.recoveryMode = 2;
        }
        else
        {
            g_dataAccessFlash.recoveryMode = 0;
        }
    }
    UI_Return_FlashTask_Result(cmd);
}
#endif

/*!
* @brief Master save data to flash through UI interface.
*
* @param[in] cmd: Tool command
* @return none
*/
#if (defined DATA_ACCESS_FLASH_ENABLE)
void UI_FlashSaveData(uint8_t cmd)
{
    if (g_mcStatus == IDLE)
    {
        g_dataAccessFlash.saveEnable = 1;
        g_dataAccessFlash.saveMode = 2;
        g_mcStatus = FLASH_TASK;
    }
    UI_Return_FlashTask_Result(cmd);
}
#endif

/*!
* @brief Master clear motor fault status through UI interface.
*
* @param[in] none
* @return none
*/
void UI_ClearFault(void)
{
    uint16_t i = 0;

    /* init clear fault status */
    g_cmdProcStatus.clearFaultStatus = 0xFF;
    g_mcStatus = STOP;
    for (i = 0; i <= 30; i++)
    {
        /* clear all fault bit */
//        Fault_Clear(i);
    }
    /* refresh clear fault status,prepare to answer master */
    g_cmdProcStatus.clearFaultStatus = 0x00;

    g_cmdProcVar.WriteSpeedValid = 0;
    UI_Return_ClearFault_Result();
}

/*!
* @brief Master read register data through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @param[in] dataLen: the char numbers of a master command
* @return none
*/
void UI_readRegister(UART_BUF *uiRxBuf, uint8_t dataLen)
{
    uint16_t i = 0;

    /* init read register status */
    g_cmdProcStatus.readRegStatus = 0xFF;
    /* save read register param num */
    g_cmdProcVar.uiRdBuf[0] = UI_UART_GetChar(uiRxBuf);

    if (g_cmdProcVar.uiRdBuf[0] < 11)
    {
        for (i = 1; i <= g_cmdProcVar.uiRdBuf[0]; i++)
        {
            /* save all read ragister param id */
            g_cmdProcVar.uiRdBuf[i] = UI_UART_GetChar(uiRxBuf);
        }
        /* refresh read ragister status,prepare to transmit register param to master */
        g_cmdProcStatus.readRegStatus = 0x00;
    }
    else
    {
        /* read register param num out of range,unvalid command */
        g_cmdProcStatus.readRegStatus = 0x02;
        /* init read register param num */
        g_cmdProcVar.uiRdBuf[0] = 0;
        /* Delete whole command except head and length */
        UI_UART_DeleteChar(uiRxBuf, dataLen - 2);
    }
    UI_Return_RegRead_Result();

    /* init read buff,prepare next read command */
    memset(g_cmdProcVar.uiRdBuf, 0U , sizeof(g_cmdProcVar.uiRdBuf));
}

/*!
* @brief Master write register data through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @param[in] dataLen: the char numbers of a master command
* @return none
*/
void UI_WriteRegister(UART_BUF *uiRxBuf, uint8_t dataLen)
{
    uint16_t iBuf = 0, i = 0;
    uint8_t tempChar = 0;

    /* init write register status */
    g_cmdProcStatus.writeRegStatus = 0xFF;
    /* save write register param num */
    g_cmdProcVar.uiWrBuf[0] = UI_UART_GetChar(uiRxBuf);

    if (g_cmdProcVar.uiWrBuf[0] < 11)
    {
        for (i = 1; i <= g_cmdProcVar.uiWrBuf[0]; i++)
        {
            /* save write register param id */
            g_cmdProcVar.uiWrBuf[i] = UI_UART_GetChar(uiRxBuf);
            /* receive low byte */
            iBuf = UI_UART_GetChar(uiRxBuf);
            tempChar = UI_UART_GetChar(uiRxBuf);
            /* receive high byte,splicing regiater param */
            iBuf += (tempChar << 8);

            /* write param to the pointer id address */
            *g_uiRegBuf[g_cmdProcVar.uiWrBuf[i]] = iBuf;
            if (g_bldc_speedCmd.speedAcceleration != (pBldcVarsCfg->motorAcceleration / 10))
            {
                g_bldc_speedCmd.speedAcceleration = (pBldcVarsCfg->motorAcceleration / 10);
            }
            if (g_bldc_speedCmd.speedDeceleration != (pBldcVarsCfg->motorDeceleration / 10))
            {
                g_bldc_speedCmd.speedDeceleration = (pBldcVarsCfg->motorDeceleration / 10);
            }
        }

        /* refresh write ragister status, prepare to answer master */
        g_cmdProcStatus.writeRegStatus = 0x00;
    }
    else
    {
        /* write register param num out of range,unvalid command */
        g_cmdProcStatus.writeRegStatus = 0x02;
        /* init write register param num */
        g_cmdProcVar.uiWrBuf[0] = 0;
        /* Delete whole command except head and length */
        UI_UART_DeleteChar(uiRxBuf, dataLen - 2);
    }

    UI_Return_RegWrite_Result();

    /* refresh DAC print channel 0 */
//    g_dispCh0 = g_uiCommParam.dacOut1;
    /* refresh DAC print channel 1 */
//    g_dispCh1 = g_uiCommParam.dacOut2;

    /* init write buff,prepare next write command */
    memset(g_cmdProcVar.uiWrBuf, 0U , sizeof(g_cmdProcVar.uiWrBuf));
}

/*!
* @brief Uart process command recieved from master according to communication protocol.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_UART_ToolCmd_Proc(UART_BUF *uiRxBuf)
{
    uint8_t headByte, dataLength, cmdData;
    uint8_t checkSum = 0, masterCmd = 0;
    uint16_t getDataTimeOut = 0, cmdNum = 0;

    if (((*uiRxBuf).uartBufCnt) >= 2)
    {
        headByte = UI_UART_ReadChar(uiRxBuf, 0);
        if (headByte == CMD_HEAD_BYTE)
        {
            dataLength = UI_UART_ReadChar(uiRxBuf, 1);
            g_cmdProcVar.uartTimeOutCntEn = 1;
            getDataTimeOut = g_cmdProcVar.uartTimeOutCnt;

            if ((((*uiRxBuf).uartBufCnt) >= dataLength) && (getDataTimeOut <= RECEIVE_TIM_MAXCNT))
            {
                g_cmdProcVar.uartTimeOutCntEn = 0;
                for (cmdNum = 0; cmdNum < dataLength - 1; cmdNum++)
                {
                    cmdData = UI_UART_ReadChar(uiRxBuf, cmdNum);
                    checkSum += cmdData;
                    if (cmdNum == 2)
                    {
                        g_cmdProcVar.uiCheckCmd = cmdData;
                    }
                }
                if (checkSum == UI_UART_ReadChar(uiRxBuf, cmdNum))
                {
                    /* Delete head and length */
                    UI_UART_DeleteChar(uiRxBuf, 2);
                    while (dataLength > 0)
                    {
                        masterCmd = UI_UART_GetChar(uiRxBuf);

                        switch (masterCmd)
                        {
                        case CMD_CHECK_COMMUNICATION:
                            UI_Return_Communicate_Result();
                            break;

                        case CMD_GET_CODE_VERSION:
                            UI_Return_GetVersion_Result();
                            break;

                        case CMD_START_MOTOR:
                            UI_StartMotor();
                            break;

                        case CMD_STOP_MOTOR:
                            UI_StopMotor();
                            break;

                        case CMD_STOP_RAMP:
                            UI_StopRamp();
                            break;

                        case CMD_START_PARAM_IDENTIFY:
                            break;

                        case CMD_QUERY_PROCESS:
                            UI_QuerySchedule(uiRxBuf);
                            break;

                        case CMD_STOP_PARAM_IDENTIFY:

                            break;

                        case CMD_MOTOR_FAULT_CLEAR:
                            UI_ClearFault();
                            break;

                        #if (defined DATA_ACCESS_FLASH_ENABLE)
                        case CMD_FLASH_READ:
                            UI_FlashRecoveryData(uiRxBuf, CMD_FLASH_READ);
                            break;
                        #endif

                        #if (defined DATA_ACCESS_FLASH_ENABLE)
                        case CMD_FLASH_SAVE:
                            UI_FlashSaveData(CMD_FLASH_SAVE);
                            break;
                        #endif

                        case CMD_READ_REGISTER:
                            UI_readRegister(uiRxBuf, dataLength);
                            break;

                        case CMD_WRITE_REGISTER:
                            UI_WriteRegister(uiRxBuf, dataLength);
                            break;

                        default:
                            break;
                        }

                        /* Delete Check tail */
                        UI_UART_DeleteChar(uiRxBuf, 1);
                        return;
                    }
                }
                else
                {
                    /*  check err */
                    UI_ReturnCheckErr_Result();
                    /* Delete whole command */
                    UI_UART_DeleteChar(uiRxBuf, dataLength);
                }
            }
            else if ((((*uiRxBuf).uartBufCnt) < dataLength) && (getDataTimeOut > RECEIVE_TIM_MAXCNT))
            {
                g_cmdProcVar.uartTimeOutCntEn = 0;
                /* Delete Head and Length */
                UI_UART_DeleteChar(uiRxBuf, 2);
            }
        }
        /* Delete Head */
        UI_UART_DeleteChar(uiRxBuf, 1);
    }
}

/* =============================================  EOF  ============================================== */
