/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file foc_data_access_eflash.c
*
* @brief This file provides data save and recovery operation with eflash functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "globalvar.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
int32_t g_VarAccessArray[DATA_NUM] = {0};           /*!< Data access flash transmit array */

uint16_t g_dataAccessErr = 0;                       /*!< Flash operation error, 1 : recovery user date at power on status error; 2: recovery data error */

flash_config_t flashConfig;
const flash_user_config_t FLASH_InitConfig =
{
    .pFlashBase    = PFLASH_BASE_ADDRESS,
    .pFlashSize    = PFLASH_BLOCK_SIZE,
    .dFlashBase    = DFLASH_BASE_ADDRESS,
    .flexRAMBase   = FLEXRAM_BASE_ADDRESS,
    .callback      = NULL,
};

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Array value check by using ChcekSum.
*
* @param[in] array: pointer to data array
* @param[in] length: length of array to do check sum
* @return tempCheckSum: value of check sum
*/
int32_t DataCheckSum(int32_t *array, uint16_t length)
{
    int32_t tempCheckSum = 0;

    while (length--)
    {
        tempCheckSum += *array++;
    }
    return tempCheckSum;
}

/*!
* @brief Get data value to save to flash.
*
* @param[in] dataSaveMode: operation mode of data save to flash, 1: save default macro define value to flash; 2: save user debug value to flash
* @param[in] array: operation array
* @return none
*/
void DataInputArray(uint8_t dataSaveMode, int32_t array[DATA_NUM])
{
    int32_t temp1 = 0;
    int32_t temp2 = 0;

    array[0] += 1;

    /* 0~13: motor parameter identification */
    if (dataSaveMode == SAVE_DEBUG_DATA)
    {
        array[1] = (int32_t)g_paramIdentifyBuffer.rsId;
        array[2] = (int32_t)g_paramIdentifyBuffer.ld;
        array[3] = (int32_t)g_paramIdentifyBuffer.ld;
        array[4] = (int32_t)g_paramIdentifyBuffer.lq;
        temp1 = (int32_t)g_paramIdentifyBuffer.bEmf;
        temp2 = (int32_t)((g_paramIdentifyBuffer.bEmf - temp1) * 10000);
        array[5] = temp1;
        array[6] = temp2;

        temp1 = (int32_t)g_paramIdentifyBuffer.inertia;
        temp2 = (int32_t)((g_paramIdentifyBuffer.inertia - temp1) * 10000);
        array[7] = temp1;
        array[8] = temp2;
    }
    else if (dataSaveMode == SAVE_DEFAULT_DATA)
    {
        array[1] = 0;
        array[2] = 0;
        array[3] = 0;
        array[4] = 0;
        array[5] = 0;
        array[6] = 0;
        array[7] = 0;
        array[8] = 0;
    }
    else
    {
    }
    array[9] = 0;      /* reserve */
    array[10] = 0;     /* reserve */
    array[11] = 0;     /* reserve */
    array[12] = 0;     /* reserve */
    array[13] = 0;     /* reserve */

    /* 14~30: motor parameter Macro define */
    array[14] = (int32_t)defMotorParam.statorRs;
    array[15] = (int32_t)defMotorParam.statorLs;
    array[16] = (int32_t)defMotorParam.ld;
    array[17] = (int32_t)defMotorParam.lq;
    array[18] = (int32_t)defMotorParam.polePairs;
    temp1 = (int32_t)defMotorParam.bEmf;
    temp2 = (int32_t)((defMotorParam.bEmf - temp1) * 10000);
    array[19] = temp1;
    array[20] = temp2;
    array[21] = (int32_t)defMotorParam.maxCur;
    array[22] = (int32_t)defMotorParam.ratedVoltage;
    array[23] = (int32_t)defMotorParam.maxSpeed;
    array[24] = (int32_t)defMotorParam.jm;
    array[25] = (int32_t)defMotorParam.encoderLines;
    array[26] = 0;     /* reserve */
    array[27] = 0;     /* reserve */
    array[28] = 0;     /* reserve */
    array[29] = 0;     /* reserve */
    array[30] = 0;     /* reserve */

    /* 31~68: startup parameter Macro define */
    array[31] = (int32_t)defStartUp.curStartOffset;
    array[32] = (int32_t)defStartUp.synSpd;
    array[33] = (int32_t)defStartUp.spdSmoothCur;
    array[34] = (int32_t)defStartUp.alignTime;
    array[35] = (int32_t)defStartUp.alignTimeTwice;
    array[36] = (int32_t)defStartUp.alignCur;
    array[37] = (int32_t)defStartUp.alignCurTwice;
    array[38] = (int32_t)defStartUp.slopeDuration1;
    array[39] = (int32_t)defStartUp.slopeDuration2;
    array[40] = (int32_t)defStartUp.curveFreqStart;
    array[41] = (int32_t)defStartUp.curveCurStart;
    array[42] = (int32_t)defStartUp.curveVoltStart;
    array[43] = (int32_t)defStartUp.curveFreqSet1;
    array[44] = (int32_t)defStartUp.curveFreqSet2;
    array[45] = (int32_t)defStartUp.curveFreqSet3;
    array[46] = (int32_t)defStartUp.curveFreqSet4;
    array[47] = (int32_t)defStartUp.curveFreqSet5;
    array[48] = (int32_t)defStartUp.curveCurSet1;
    array[49] = (int32_t)defStartUp.curveCurSet2;
    array[50] = (int32_t)defStartUp.curveCurSet3;
    array[51] = (int32_t)defStartUp.curveCurSet4;
    array[52] = (int32_t)defStartUp.curveCurSet5;
    array[53] = (int32_t)defStartUp.curveVoltSet1;
    array[54] = (int32_t)defStartUp.curveVoltSet2;
    array[55] = (int32_t)defStartUp.curveVoltSet3;
    array[56] = (int32_t)defStartUp.curveVoltSet4;
    array[57] = (int32_t)defStartUp.curveVoltSet5;
    array[58] = (int32_t)defStartUp.curveSegNum;
    array[59] = (int32_t)defStartUp.curveTime1;
    array[60] = (int32_t)defStartUp.curveTime2;
    array[61] = (int32_t)defStartUp.curveTime3;
    array[62] = (int32_t)defStartUp.curveTime4;
    array[63] = (int32_t)defStartUp.curveTime5;
    array[64] = 0;     /* reserve */
    array[65] = 0;     /* reserve */
    array[66] = 0;     /* reserve */
    array[67] = 0;     /* reserve */
    array[68] = 0;     /* reserve */

    /* 69~132: FOC parameters */
    array[69] = (int32_t)defFocVars.motorAcceleration;
    array[70] = (int32_t)defFocVars.motorDeceleration;
    array[71] = (int32_t)defFocVars.motorDir;
    array[72] = (int32_t)defFocVars.controlMode;
    array[73] = (int32_t)defFocVars.fullOrderSmoFilt;
    array[74] = (int32_t)defFocVars.voltObtainWay;
    array[75] = (int32_t)defFocVars.angleObtainWay;
    array[76] = (int32_t)defFocVars.smoWay;
    array[77] = 0;     /* reserve */
    array[78] = 0;     /* reserve */

    array[79] = (int32_t)defSlideMode.kslide0;
    array[80] = (int32_t)defSlideMode.kslf0;
    array[81] = (int32_t)defSlideMode.maxErr0;
    array[82] = (int32_t)defSlideMode.kslide1;
    array[83] = (int32_t)defSlideMode.kslf1;
    array[84] = (int32_t)defSlideMode.maxErr1;
    array[85] = (int32_t)defSlideMode.fullOrderGain;

    array[86] = (int32_t)defFluxObserver.flitCoeff;
    array[87] = (int32_t)defFluxObserver.compCofLimit;

    array[88] = (int32_t)defMrasObserver.cofAlpha;
    array[89] = (int32_t)defMrasObserver.cofBeta;

    array[90] = (int32_t)defPll.coeffPll;

    array[91] = 0;     /* reserve */
    array[92] = 0;     /* reserve */

    array[93] = (int32_t)defHallSensor.hallSwapTrqCoff;
    array[94] = (int32_t)defHallSensor.learnIntervalTime;
    array[95] = (int32_t)defHallSensor.learnCur;
    array[96] = (int32_t)defHallSensor.learnOrder;
    array[97] = (int32_t)defHallSensor.learnDegree;

    array[98] = (int32_t)g_hallSelfLearnBuffer.selfLearnOrder;
    array[99] = (int32_t)g_hallSelfLearnBuffer.selfLearnTheta;
    array[100] = (int32_t)g_hallSelfLearnBuffer.sectorWidthMax;
    array[101] = (int32_t)g_hallSelfLearnBuffer.sectorWidthMin;
    array[102] = 0;    /* reserve */
    array[103] = 0;    /* reserve */

    array[104] = (int32_t)defPID.torqueKp;
    array[105] = (int32_t)defPID.torqueKi;
    array[106] = (int32_t)defPID.torqueKd;
    array[107] = (int32_t)defPID.maxtorque;
    array[108] = (int32_t)defPID.mintorque;
    array[109] = (int32_t)defPID.fluxKp;
    array[110] = (int32_t)defPID.fluxKi;
    array[111] = (int32_t)defPID.fluxKd;
    array[112] = (int32_t)defPID.maxflux;
    array[113] = (int32_t)defPID.minflux;
    array[114] = (int32_t)defPID.speedKp;
    array[115] = (int32_t)defPID.speedKi;
    array[116] = (int32_t)defPID.speedKd;
    array[117] = (int32_t)defPID.maxSpeed;
    array[118] = (int32_t)defPID.minSpeed;
    array[119] = 0;    /* reserve */
    array[120] = 0;    /* reserve */

    array[121] = (int32_t)defFluxweak.voltageRef;
    array[122] = (int32_t)defFluxweak.kp;
    array[123] = (int32_t)defFluxweak.ki;
    array[124] = (int32_t)defFluxweak.min;
    array[125] = (int32_t)defFluxweak.maxModule;
    array[126] = (int32_t)defFluxweak.demagCur;
    array[127] = (int32_t)defFluxweak.nominalCur;

    array[128] = 0;    /* reserve */
    array[129] = 0;    /* reserve */
    array[130] = 0;    /* reserve */
    array[131] = 0;    /* reserve */
    array[132] = 0;    /* reserve */

    /* 133: pulse injection parameter */
    #if (defined PULSE_INJECTION)
    array[133] = (int32_t)g_pulseInject.pwmTs;
    #else
    array[133] = 0;
    #endif

    /* 134~137: speed S curve parameters */
    array[134] = (int32_t)defSCurve.accStartRatio;
    array[135] = (int32_t)defSCurve.accEndRatio;
    array[136] = (int32_t)defSCurve.decStartRatio;
    array[137] = (int32_t)defSCurve.decEndRatio;

    /* 138~141: short brake parameters */
    array[138] = (int32_t)defShortBrake.shortBrakeCurRatio;
    array[139] = (int32_t)defShortBrake.shortBrakeUdcMax;
    array[140] = (int32_t)defShortBrake.shortBrakeDoneFlagTs;
    array[141] = (int32_t)defShortBrake.shortQuitCur;

    array[142] = 0;    /* reserve */
    array[143] = 0;    /* reserve */
    array[144] = 0;    /* reserve */
    array[145] = 0;    /* reserve */
    array[146] = 0;    /* reserve */

    /* 147~168: high frequency injection parameters */
    array[147] = (int32_t)defHighFreqInj.hfiFreq;
    array[148] = (int32_t)defHighFreqInj.vPeak;
    array[149] = (int32_t)defHighFreqInj.pulseAmp;
    array[150] = (int32_t)defHighFreqInj.pulseTime;
    array[151] = (int32_t)defHighFreqInj.clearTime;
    array[152] = (int32_t)defHighFreqInj.hfiInitTime;
    array[153] = (int32_t)defHighFreqInj.hfiInitTheta;
    array[154] = (int32_t)defHighFreqInj.thetaLpfCoeff;
    array[155] = (int32_t)defHighFreqInj.hfiPllKp;
    array[156] = (int32_t)defHighFreqInj.hfiPllKi;
    array[157] = (int32_t)defHighFreqInj.bpfA0;
    array[158] = (int32_t)defHighFreqInj.bpfA2;
    array[159] = (int32_t)defHighFreqInj.bpfB1;
    array[160] = (int32_t)defHighFreqInj.bpfB2;
    array[161] = (int32_t)defHighFreqInj.lpfA0;
    array[162] = (int32_t)defHighFreqInj.lpfA1;
    array[163] = (int32_t)defHighFreqInj.lpfA2;
    array[164] = (int32_t)defHighFreqInj.lpfB1;
    array[165] = (int32_t)defHighFreqInj.lpfB2;
    array[166] = (int32_t)defHighFreqInj.hfiVdFreqHigh;
    array[167] = (int32_t)defHighFreqInj.hfiVdFreqLow;
    array[168] = 0;     /* reserve */

    /* last one: sum check */
    array[DATA_NUM - 1] = DataCheckSum(array, (DATA_NUM - 1));
}

/*!
* @brief Get flash data to recovery to variables.
*
* @param[in] dataRecoveryMode: operation mode of data recovery from flash, 1: recovery default macro define value from flash; 2: recovery user debug value from flash
* @param[in] array: operation array
* @return none
*/
void DataOutputArray(uint8_t dataRecoveryMode, int32_t array[DATA_NUM])
{

    int32_t tempChechSum = 0;

    /* check sum */
    tempChechSum = DataCheckSum(array, (DATA_NUM - 1));
    if (tempChechSum != array[DATA_NUM - 1])
    {
        g_dataAccessErr = 2;
        return;
    }

    if (dataRecoveryMode == RECOVERY_DEBUG_DATA)
    {
        if ((array[1] != 0) && (array[1] != 0xFFFFFFFF))
        {
            defMotorParam.statorRs = array[1];
            defMotorParam.statorLs = array[2];
            defMotorParam.ld = array[3];
            defMotorParam.lq = array[4];
            defMotorParam.bEmf = array[5] + array[6] / 10000.0;
            defMotorParam.jm = (uint32_t)(array[7] + array[8] / 10000.0);
        }
        else
        {
            defMotorParam.statorRs = array[14];
            defMotorParam.statorLs = array[15];
            defMotorParam.ld = array[16];
            defMotorParam.lq = array[17];
            defMotorParam.bEmf = array[19] + array[20] / 10000.0;
            defMotorParam.jm = array[24];
        }
    }
    else if (dataRecoveryMode == RECOVERY_DEFAULT_DATA)
    {
        defMotorParam.statorRs = array[14];
        defMotorParam.statorLs = array[15];
        defMotorParam.ld = array[16];
        defMotorParam.lq = array[17];
        defMotorParam.bEmf = array[19] + array[20] / 10000.0;
        defMotorParam.jm = array[24];
    }
    else
    {
        defMotorParam.statorRs = array[14];
        defMotorParam.statorLs = array[15];
        defMotorParam.ld = array[16];
        defMotorParam.lq = array[17];
        defMotorParam.bEmf = array[19] + array[20] / 10000.0;
        defMotorParam.jm = array[24];
    }
    defMotorParam.polePairs = array[18];

    defMotorParam.maxCur = array[21];
    defMotorParam.ratedVoltage = array[22];
    defMotorParam.maxSpeed = array[23];
    defMotorParam.encoderLines = array[25];

    defStartUp.curStartOffset = array[31];
    defStartUp.synSpd = array[32];
    defStartUp.spdSmoothCur = array[33];
    defStartUp.alignTime = array[34];
    defStartUp.alignTimeTwice = array[35];
    defStartUp.alignCur = array[36];
    defStartUp.alignCurTwice = array[37];
    defStartUp.slopeDuration1 = array[38];
    defStartUp.slopeDuration2 = array[39];

    defStartUp.curveFreqStart = array[40];
    defStartUp.curveCurStart = array[41];
    defStartUp.curveVoltStart = array[42];

    defStartUp.curveFreqSet1 = array[43];
    defStartUp.curveFreqSet2 = array[44];
    defStartUp.curveFreqSet3 = array[45];
    defStartUp.curveFreqSet4 = array[46];
    defStartUp.curveFreqSet5 = array[47];
    defStartUp.curveCurSet1 = array[48];
    defStartUp.curveCurSet2 = array[49];
    defStartUp.curveCurSet3 = array[50];
    defStartUp.curveCurSet4 = array[51];
    defStartUp.curveCurSet5 = array[52];
    defStartUp.curveVoltSet1 = array[53];
    defStartUp.curveVoltSet2 = array[54];
    defStartUp.curveVoltSet3 = array[55];
    defStartUp.curveVoltSet4 = array[56];
    defStartUp.curveVoltSet5 = array[57];
    defStartUp.curveSegNum = array[58];
    defStartUp.curveTime1 = array[59];
    defStartUp.curveTime2 = array[60];
    defStartUp.curveTime3 = array[61];
    defStartUp.curveTime4 = array[62];
    defStartUp.curveTime5 = array[63];

    defFocVars.motorAcceleration = array[69];
    defFocVars.motorDeceleration = array[70];
    defFocVars.motorDir = array[71];
    defFocVars.controlMode = array[72];
    defFocVars.fullOrderSmoFilt = array[73];
    defFocVars.voltObtainWay = array[74];
    defFocVars.angleObtainWay = array[75];
    defFocVars.smoWay = array[76];

    defSlideMode.kslide0 = array[79];
    defSlideMode.kslf0 = array[80];
    defSlideMode.maxErr0 = array[81];
    defSlideMode.kslide1 = array[82];
    defSlideMode.kslf1 = array[83];
    defSlideMode.maxErr1 = array[84];
    defSlideMode.fullOrderGain = array[85];

    defFluxObserver.flitCoeff = array[86];
    defFluxObserver.compCofLimit = array[87];

    defMrasObserver.cofAlpha = array[88];
    defMrasObserver.cofBeta = array[89];

    defPll.coeffPll = array[90];

    defHallSensor.hallSwapTrqCoff = array[93];
    defHallSensor.learnIntervalTime = array[94];
    defHallSensor.learnCur = array[95];
    if (dataRecoveryMode == RECOVERY_DEBUG_DATA)
    {
        if ((array[98] != 0) && (array[98] != 0xFFFFFFFF))
        {
            defHallSensor.learnOrder = array[98];
            defHallSensor.learnDegree = array[99];
        }
        else
        {
            defHallSensor.learnOrder = array[96];
            defHallSensor.learnDegree = array[97];
        }
    }
    else if (dataRecoveryMode == RECOVERY_DEFAULT_DATA)
    {
        defHallSensor.learnOrder = array[96];
        defHallSensor.learnDegree = array[97];
    }
    else
    {
        defHallSensor.learnOrder = array[96];
        defHallSensor.learnDegree = array[97];
    }

    defPID.torqueKp = array[104];
    defPID.torqueKi = array[105];
    defPID.torqueKd = array[106];
    defPID.maxtorque = array[107];
    defPID.mintorque = array[108];
    defPID.fluxKp = array[109];
    defPID.fluxKi = array[110];
    defPID.fluxKd = array[111];
    defPID.maxflux = array[112];
    defPID.minflux = array[113];
    defPID.speedKp = array[114];
    defPID.speedKi = array[115];
    defPID.speedKd = array[116];
    defPID.maxSpeed = array[117];
    defPID.minSpeed = array[118];

    defFluxweak.voltageRef = array[121];
    defFluxweak.kp = array[122];
    defFluxweak.ki = array[123];
    defFluxweak.min = array[124];
    defFluxweak.maxModule = array[125];
    defFluxweak.demagCur = array[126];
    defFluxweak.nominalCur = array[127];

    #if (defined PULSE_INJECTION)
    g_pulseInject.pwmTs = array[133];
    #endif

    defSCurve.accStartRatio = array[134];
    defSCurve.accEndRatio = array[135];
    defSCurve.decStartRatio = array[136];
    defSCurve.decEndRatio = array[137];

    defShortBrake.shortBrakeCurRatio = array[138];
    defShortBrake.shortBrakeUdcMax = array[139];
    defShortBrake.shortBrakeDoneFlagTs = array[140];
    defShortBrake.shortQuitCur = array[141];

    defHighFreqInj.hfiFreq = array[147];
    defHighFreqInj.vPeak = array[148];
    defHighFreqInj.pulseAmp = array[149];
    defHighFreqInj.pulseTime = array[150];
    defHighFreqInj.clearTime = array[151];
    defHighFreqInj.hfiInitTime = array[152];
    defHighFreqInj.hfiInitTheta = array[153];
    defHighFreqInj.thetaLpfCoeff = array[154];
    defHighFreqInj.hfiPllKp = array[155];
    defHighFreqInj.hfiPllKi = array[156];
    defHighFreqInj.bpfA0 = array[157];
    defHighFreqInj.bpfA2 = array[158];
    defHighFreqInj.bpfB1 = array[159];
    defHighFreqInj.bpfB2 = array[160];
    defHighFreqInj.lpfA0 = array[161];
    defHighFreqInj.lpfA1 = array[162];
    defHighFreqInj.lpfA2 = array[163];
    defHighFreqInj.lpfB1 = array[164];
    defHighFreqInj.lpfB2 = array[165];
    defHighFreqInj.hfiVdFreqHigh = array[166];
    defHighFreqInj.hfiVdFreqLow = array[167];

}

/*!
* @brief Flash page erase.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] pageAddress: flash operation page address
* @return ret: flash operation status
*/
status_t Eflash_DataAccessPageErase(DATA_ACCESS_FLASH *pDataAccess, uint32_t pageAddress)
{
    status_t ret = STATUS_SUCCESS;

    if (pDataAccess->eraseEnable == 1)
    {
        /* AC7840x FLASH */
        ret = FLASH_DRV_UnlockCtrl();
        ret = FLASH_DRV_EraseSector(&flashConfig, pageAddress, PFLASH_PAGE_SIZE);
        if (STATUS_SUCCESS != ret)
        {
            FLASH_DRV_LockCtrl();
            return ret;
        }

        FLASH_DRV_LockCtrl();
        pDataAccess->eraseEnable = 0;
    }
    else
    {
    }
    return ret;
}

/*!
* @brief Flash saving operation.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] pageAddress: flash operation page address
* @param[in] array: pointer to data array
* @return ret: flash operation status
*/
status_t Eflash_DataAccess_SaveProcess(DATA_ACCESS_FLASH *pDataAccess, uint32_t pageAddress, int32_t *array)
{
    status_t ret = STATUS_SUCCESS;

    if (pDataAccess->saveEnable == 1)
    {
        ret = FLASH_DRV_UnlockCtrl();
        if (STATUS_SUCCESS != ret)
        {
            FLASH_DRV_LockCtrl();
            return ret;
        }

        ret = FLASH_DRV_EraseSector(&flashConfig, pageAddress, PFLASH_PAGE_SIZE);
        pDataAccess->saveFinishFlag = 0;
        if (STATUS_SUCCESS != ret)
        {
            FLASH_DRV_LockCtrl();
            return ret;
        }

        if (pDataAccess->saveMode == SAVE_DEFAULT_DATA)
        {
            DataInputArray(pDataAccess->saveMode, array);
            ret = FLASH_DRV_Program(&flashConfig, pageAddress, DATA_SIZE, (uint8_t *)array);
        }
        else if (pDataAccess->saveMode == SAVE_DEBUG_DATA)
        {
            DataInputArray(pDataAccess->saveMode, array);
            ret = FLASH_DRV_Program(&flashConfig, pageAddress, DATA_SIZE, (uint8_t *)array);
        }
        else
        {
        }

        if (STATUS_SUCCESS != ret)
        {
            FLASH_DRV_LockCtrl();
            return ret;
        }

        FLASH_DRV_LockCtrl();
        pDataAccess->saveEnable = 0;
        pDataAccess->saveFinishFlag = 1;
        ret = STATUS_SUCCESS;
    }
    else
    {
    }
    return ret;
}

/*!
* @brief Flash recovery operation.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] pageAddress: flash operation page address
* @param[in] array: pointer to data array
* @return ret: flash operation status
*/
status_t Eflash_DataAccess_RecoveryProcess(DATA_ACCESS_FLASH *pDataAccess, uint32_t pageAddress, int32_t *array)
{
    status_t ret = STATUS_SUCCESS;

    if (pDataAccess->recoveryEnable == 1)
    {
        pDataAccess->recoveryFinishFlag = 0;

        if (pDataAccess->recoveryMode == RECOVERY_DEFAULT_DATA)
        {
            ret = FLASH_DRV_Read(&flashConfig, pageAddress , (uint8_t *)array, DATA_SIZE);

            if ((array[DATA_NUM - 1] != 0xFFFFFFFF) && (array[DATA_NUM - 1] != 0))
            {
                pDataAccess->recoveryEnable = 0;
                pDataAccess->recoveryFinishFlag = 1;
                DataOutputArray(pDataAccess->recoveryMode, array);
                ret = STATUS_SUCCESS;
            }
            else
            {
                return STATUS_BUSY;
            }
        }
        else if (pDataAccess->recoveryMode == RECOVERY_DEBUG_DATA)
        {
            ret = FLASH_DRV_Read(&flashConfig, pageAddress , (uint8_t *)array, DATA_SIZE);

            if ((array[DATA_NUM - 1] != 0xFFFFFFFF) && (array[DATA_NUM - 1] != 0))
            {
                pDataAccess->recoveryEnable = 0;
                pDataAccess->recoveryFinishFlag = 1;
                DataOutputArray(pDataAccess->recoveryMode, array);
                ret = STATUS_SUCCESS;
            }
            else
            {
                return STATUS_BUSY;
            }
        }
        else
        {
        }

        pDataAccess->recoveryFinishFlag = 0;
    }
    return ret;
}

/*!
* @brief Flash operation process, do flash tasks with tool command.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] array: pointer to data array
* @return ret: flash operation status
*/
status_t Eflash_DataAccessEflashProcess(DATA_ACCESS_FLASH *pDataAccess, int32_t *array)
{
    status_t ret = STATUS_SUCCESS;

    if (pDataAccess->eraseEnable == 1)
    {
        ret = Eflash_DataAccessPageErase(pDataAccess, ADDRESS_START_DEBUG_PARAM);
        pDataAccess->eraseEnable = 0;
    }

    if (pDataAccess->saveEnable == 1)
    {
        if (pDataAccess->saveMode == SAVE_DEFAULT_DATA)
        {
            ret = Eflash_DataAccess_SaveProcess(pDataAccess, ADDRESS_START_DEFAULT_PARAM, array);
            if (pDataAccess->saveFinishFlag == 1)
            {
                pDataAccess->saveEnable = 0;
            }
        }
        else if (pDataAccess->saveMode == SAVE_DEBUG_DATA)
        {
            ret = Eflash_DataAccess_SaveProcess(pDataAccess, ADDRESS_START_DEBUG_PARAM, array);
            if (pDataAccess->saveFinishFlag == 1)
            {
                pDataAccess->saveEnable = 0;
            }
        }
        else
        {
            pDataAccess->saveEnable = 0;
        }
    }

    if (pDataAccess->recoveryEnable == 1)
    {
        if (pDataAccess->recoveryMode == RECOVERY_DEFAULT_DATA)
        {
            ret = Eflash_DataAccess_RecoveryProcess(pDataAccess, ADDRESS_START_DEFAULT_PARAM, array);
            pDataAccess->recoveryEnable = 0;
        }
        else if (pDataAccess->recoveryMode == RECOVERY_DEBUG_DATA)
        {
            ret = Eflash_DataAccess_RecoveryProcess(pDataAccess, ADDRESS_START_DEBUG_PARAM, array);
            pDataAccess->recoveryEnable = 0;
        }
        else
        {
            pDataAccess->recoveryEnable = 0;
        }
    }
    return ret;
}

/*!
* @brief Saving macro define value to flash when mcu is power on.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] array: pointer to data array
* @return ret: flash operation status
*/
status_t Eflash_DataAccess_DefaultSave(DATA_ACCESS_FLASH *pDataAccess, int32_t *array)
{
    status_t ret = STATUS_SUCCESS;

    pDataAccess->saveMode = SAVE_DEFAULT_DATA;
    pDataAccess->saveEnable = 1;

    ret = Eflash_DataAccess_SaveProcess(pDataAccess, ADDRESS_START_DEFAULT_PARAM, array);
    if (pDataAccess->saveFinishFlag == 1)
    {
        pDataAccess->saveEnable = 0;
        pDataAccess->saveMode = 0;
        ret = STATUS_SUCCESS;
    }

    return ret;
}

/*!
* @brief Recovery user debug value from flash to variables when mcu is power on.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] array: pointer to data array
* @return none
*/
void Eflash_DataAccess_PowerOnRecovery(DATA_ACCESS_FLASH *pDataAccess, int32_t *array)
{
    int32_t tempWrite;
    int32_t tempCheckSum = 0;

    pDataAccess->recoveryMode = RECOVERY_DEBUG_DATA;

    tempWrite = *(uint8_t*)(ADDRESS_START_DEBUG_PARAM);

    if ((tempWrite != 0) && (tempWrite != 0xFFFFFFFF))
    {
        FLASH_DRV_Read(&flashConfig, ADDRESS_START_DEBUG_PARAM, (uint8_t *)array, DATA_SIZE);

        tempCheckSum = DataCheckSum(array, (DATA_NUM - 1));

        if (tempCheckSum == array[DATA_NUM - 1])
        {
            DataOutputArray(pDataAccess->recoveryMode, array);
        }
        else
        {
            g_dataAccessErr = 1;
            return;
        }

    }
    else
    {
    }
}

/*!
* @brief Flash operation command initilization.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @return none
*/
void Eflash_DataAccessEflashInit(DATA_ACCESS_FLASH *pDataAccess)
{
    memset(pDataAccess, 0U, sizeof(*pDataAccess));
}

/* =============================================  EOF  ============================================== */
