/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file foc_mcu_config.c
*
* @brief This file provides MCU peripheral initialization functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "foc_mcu_config.h"
#include "ac7840x.h"
#include "gpio_drv.h"
#include "pwm_input.h"
#include "pwm_output.h"
#include "pwm_hw.h"
#include "adc_hw.h"
#include "ctu_drv.h"
#include "acmp_drv.h"
#include "dma_drv.h"
#include "spi_master_drv.h"

#include "globalvar.h"
#include "foc_isr.h"
#include "foc_app.h"
#include "foc_keys.h"
#include "foc_data_access_eflash.h"

#include "pdt_drv.h"

/* ============================================  Define  ============================================ */
#define MAX_SYSTICK_COUNT (SysTick_LOAD_RELOAD_Msk + 1)  /* systemTick max count value */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
//uint32_t g_ADCRegularBuffer[6] = {0};  /*!< Buffer to storage Regular Group ADC Value */
uint16_t g_ADCRegularBuffer[2] = {0};  /*!< Buffer to storage Regular Group ADC Value */
uint16_t g_ADC1RegularBuffer[6] = {0};  /*!< Buffer to storage Regular Group ADC Value */
/* spi Master state */
static spi_state_t s_spiMasterState;
spi_master_config_t spiMasteConfig;

//static dma_state_t dmaController_State, dmaController1_State;

static dma_state_t dmaStateArray[2] = {0};
static dma_channel_config_t dmaController1Chn0_Config;//, dmaController1Chn1_Config;
static dma_channel_config_t dmaController1Chn1_Config;

static const dma_channel_config_t * const dmaChnConfigArray[] = {&dmaController1Chn0_Config, &dmaController1Chn1_Config};
//static const dma_channel_config_t * const dmaChn1ConfigArray[] = {&dmaController1Chn1_Config};
static dma_chn_state_t dmaController1Chn0_State, dmaController1Chn1_State;
static dma_chn_state_t * const dmaChnStateArray[] = {&dmaController1Chn0_State, &dmaController1Chn1_State};
//static dma_chn_state_t * const dmaChn1StateArray[] = {&dmaController1Chn1_State};

/* systemTick us delay factor */
static uint32_t  s_facus = 0U;

/* systemTick ms delay factor */
static uint32_t  s_facms = 0U;

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Init Delay count factor, 1s = TICKS * (1/f)  s;
*
* @param[in]  none
* @return none
*/
void InitDelay(void)
{
    s_facus = SYSCLK_FREQ / 1000000;
    s_facms = s_facus * 1000;
}

/*!
* @brief delay until the syctick count tick to 0
*
* @param[in] tick : systick count value
* @return none
*/
static void SysTickDelay(uint32_t tick)
{
    __IO uint32_t tickFlag = 0;

    SysTick->LOAD = tick - 1;
    SysTick->VAL = 0x00;
    SysTick->CTRL  = SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_ENABLE_Msk;
    do
    {
        tickFlag = SysTick->CTRL;

    } while(!(tickFlag & SysTick_CTRL_COUNTFLAG_Msk));

    SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk;
    SysTick->VAL = 0X00;
}

/*!
* @brief Repeatlly delay the ticks for the given times
*
* @param[in] times: the times of delay the tick
* @param[in] tick: each tick to delay
* @return none
*/
static void SysTickRepeatDelay(uint32_t times, uint32_t tick)
{
    uint32_t i = 0;

    for (i = 0; i < times; i++)
    {
        SysTickDelay(tick);
    }
}

/*!
* @brief delay us
*
* @param[in] us: us for delay
* @return none
*/
void udelay(uint32_t us)
{
    uint32_t tick = us * s_facus;

    SysTickRepeatDelay(tick / MAX_SYSTICK_COUNT, MAX_SYSTICK_COUNT);

    SysTickDelay(tick % MAX_SYSTICK_COUNT);
}

/*!
* @brief delay ms
*
* @param[in] ms: ms for delay
* @return none
*/
void mdelay(uint32_t ms)
{
    uint32_t tick = ms * s_facms;

    SysTickRepeatDelay(tick / MAX_SYSTICK_COUNT, MAX_SYSTICK_COUNT);

    SysTickDelay(tick % MAX_SYSTICK_COUNT);
}

/*!
* @brief FOC Control Module GPIO Initialize.
*
* @param[in] none
* @return none
*/
void FOC_GPIO_Initialize(void)
{
#ifdef USE_GPIO_CALCULATE_FOC_RUNTIME
    /* Setting the multiplexing function of GPIO pin */
    /* PD16: LIN_SLP_M used as runtime measurement pin */
    GPIO_DRV_SetMuxModeSel(PORTD, 16U, PORT_MUX_AS_GPIO);
    GPIO_DRV_SetPinDirection(GPIOD, 16U, GPIO_OUTPUT);
#endif

    /* Setting the LED to display operation states */
    GPIO_DRV_SetMuxModeSel(PORTD, 15U, PORT_MUX_AS_GPIO);
    GPIO_DRV_SetPinDirection(GPIOD, 15U, GPIO_OUTPUT);
    GPIO_DRV_SetMuxModeSel(PORTE, 3U, PORT_MUX_AS_GPIO);
    GPIO_DRV_SetPinDirection(GPIOE, 3U, GPIO_OUTPUT);
}

/*!
* @brief FOC Control Module ACMP Initialize.
*
* @param[in] none
* @return none
*/
void FOC_ACMP_Initialize(void)
{
    acmp_module_t acmpConfig;

    /*
    * GPIO Configuration for ACMP
    * PA0     --> ACMP0_IN0, BEMF_U
    * PA1     --> ACMP0_IN1, BEMF_V
    * PC3     --> ACMP0_IN4, BEMF_W
    * PE8     --> ACMP0_IN3, BEMF_MID
    */
    /* PA0, ACMP0_IN7, Bemf_U */
    GPIO_DRV_SetMuxModeSel(PORTA, 0U, PORT_PIN_DISABLED);
    /* PA1, ACMP0_IN6, Bemf_V */
    GPIO_DRV_SetMuxModeSel(PORTA, 1U, PORT_PIN_DISABLED);
    /* PC3, ACMP0_IN4, Bemf_W */
    GPIO_DRV_SetMuxModeSel(PORTC, 3U, PORT_PIN_DISABLED);
    /* PE8, ACMP0_IN3, Bemf_MID */
    GPIO_DRV_SetMuxModeSel(PORTE, 8U, PORT_PIN_DISABLED);

    memset(&acmpConfig, 0U, sizeof(acmp_module_t));

    /* ACMP Polling Mode Configuration */
    acmpConfig.pollingMode.mode                 = ACMP_POSITIVE_POLLING;

    acmpConfig.pollingMode.pollingSequence      = 7U;
    acmpConfig.pollingMode.pollingClockDivide   = ACMP_CLK_DIVIDE_256;
    acmpConfig.pollingMode.hallOutputEnable     = true;
    acmpConfig.pollingMode.hallAOutputCh        = ACMP_EXTERNAL_CH0;
    acmpConfig.pollingMode.hallBOutputCh        = ACMP_EXTERNAL_CH1;
    acmpConfig.pollingMode.hallCOutputCh        = ACMP_EXTERNAL_CH2;

    /* ACMP Init Configuration */
    acmpConfig.mux.negativeInputMux             = ACMP_EXTERNAL_CH3;
    acmpConfig.comparator.outputTrigger         = ACMP_FALLING_EDGE;
    acmpConfig.comparator.interruptEn           = false;
    acmpConfig.comparator.powerEnable           = true;

    ACMP_DRV_Init(0U, &acmpConfig);
}

/*!
* @brief FOC Control Hall Peripheral Initialize.
*
* @param[in] none
* @return none
*/
void FOC_Hall_Initialize(void)
{
    /* AC7840x Hall configuration */
    pwm_input_ch_config_type_t inputChConfig;
    pwm_input_capture_config_t inputConfig;
    pwm_config_t config;

    /* Hall A: PWM0_CH0, PC0 */
    GPIO_DRV_SetMuxModeSel(PORTC, 0U, PORT_MUX_ALT2);
    /* Hall B: PWM0_CH1, PC1 */
    GPIO_DRV_SetMuxModeSel(PORTC, 1U, PORT_MUX_ALT2);
    /* Hall C: PWM0_CH2, PC2 */
    GPIO_DRV_SetMuxModeSel(PORTC, 2U, PORT_MUX_ALT2);

    memset(&inputChConfig, 0U, sizeof(pwm_input_ch_config_type_t));
    inputChConfig.channel = PWM_CH_0;
    inputChConfig.mode = PWM_INPUTCAP_DUAL_EDGE;
    inputChConfig.measureType = PWM_POSITIVE_PLUSE_WIDTH_MEASURE;
    inputChConfig.onceMode = PWM_INPUTCAP_CONTINUOUS;
    inputChConfig.filterEn = true;
    inputChConfig.filterValue = HALL_FILTVAL;
    inputChConfig.interruptEn = true;
    inputChConfig.pulseWidthMeasureEn = true;
    inputChConfig.chCountRst = true;

    inputConfig.channelNum = 1;
    inputConfig.channelConfig = &inputChConfig;
    inputConfig.hallEn = true;

    inputConfig.filterPsc = HALL_FILTER_PSC;

    memset(&config, 0U, sizeof(pwm_config_t));
    config.mode = PWM_MODE_INPUT_CAPTURE;
    config.initModeStruct = &inputConfig;
    config.clkSource = PWM_CLK_SOURCE_SYSTEM;
    config.maxValue = PWM_MAX_COUNTER_VALUE;
    config.interruptEn = true;
    config.clkPsc = HALL_CLK_PRESCALER;
    config.channelCallback = FOC_Hall_Callback;
    config.overflowEventEn = true;
    config.overflowInterrupEn = true;
    config.overflowCallback = FOC_Hall_OverCallback;
    PWM_DRV_Init(PWM_MODULER_HALL, &config);

    /* Set interrupt priority */
    NVIC_EnableIRQ(PWM0_CHANNEL_IRQn);
    NVIC_SetPriority(PWM0_CHANNEL_IRQn, 1U);
}

/*!
* @brief FOC Control Encoder Mode Module PWM Initialize.
*
* @param[in] none
* @return none
*/
void FOC_Encoder_Initialize(void)
{
    pwm_config_t pwmConfig;
    /* Encoder config struct */
    pwm_quad_decoder_config_t encoderConfig;

//    /*
//    * GPIO Configuration for ENCODER
//    * PA1     --> PWM0_CH0
//    * PA0     --> PWM0_CH1
//    * PB13    --> ENCODER_Z
//    */
    /* PB2: PWM1_CH0, EncoderA */
    GPIO_DRV_SetMuxModeSel(PORTB, 2U, PORT_MUX_ALT2);
    /* PB3: PWM1_CH1, EncoderB */
    GPIO_DRV_SetMuxModeSel(PORTB, 3U, PORT_MUX_ALT2);
    /* PC14: PWM1_CH2, EncoderZ */
    GPIO_DRV_SetMuxModeSel(PORTC, 14U, PORT_MUX_ALT2);

    memset(&encoderConfig, 0U, sizeof(encoderConfig));
    memset(&pwmConfig, 0U, sizeof(pwmConfig));
    encoderConfig.quadEn                        = true;
    /* AB phase encode,not cnt and dir encode */
    encoderConfig.mode                          = PWM_QUAD_PHASE_ENCODE;
    encoderConfig.phaseAConfig.filterEn         = true;
    encoderConfig.phaseAConfig.filterVal        = 5U;
#if (ENCODER_POLARITY > 0U)
    encoderConfig.phaseAConfig.polarity         = PWM_QUAD_PHASE_INVERT;
#else
    encoderConfig.phaseAConfig.polarity         = PWM_QUAD_PHASE_NORMAL;
#endif
    encoderConfig.phaseBConfig.filterEn         = true;
    encoderConfig.phaseBConfig.filterVal        = 5U;
    encoderConfig.phaseBConfig.polarity         = PWM_QUAD_PHASE_NORMAL;

    pwmConfig.mode                              = PWM_MODE_QUADRATURE_DECODER;
    pwmConfig.initModeStruct                    = &encoderConfig;
    pwmConfig.clkPsc                            = 0U;
    pwmConfig.initValue                         = 0U;
    pwmConfig.maxValue                          = PULSE_PER_REVOLUTION - 1U;
    pwmConfig.clkSource                         = PWM_CLK_SOURCE_SYSTEM;
    pwmConfig.overflowInterrupEn                = true;
    pwmConfig.interruptEn                       = true;
    pwmConfig.overflowCallback                  = FOC_Encoder_Callback;

    PWM_DRV_Init(PWM_MODULER_ENCODER, &pwmConfig);
#if (defined ABZ_QUAD_ENCODER)
    GPIO_DRV_SetPinIntSel(PORTC, (gpio_channel_type_t)14U, PORT_INT_RISING_EDGE);
//    GPIO_DRV_InstallCallback(GPIO_Z_callback);
//    GPIO_SetCallback(GPIO_PIN13, GPIO_Z_callback);
#endif

    /* Set interrupt priority */
    NVIC_SetPriority(PWM1_OVERFLOW_IRQn, 2);
}

/*!
* @brief PWM Initialization for debugging, PWM module can be used for debugging output.
*
* @param[in] PWMx: PWM Module
* @return none
*/
void MC_DEBUG_PWM_Initialize(PWM_Type *PWMx)
{
    pwm_simply_config_t SimplyConfig;

    /*
    * GPIO Configuration for debug PWM
    * PA13    --> PWM2_CH0
    * PA12    --> PWM2_CH1
    */
    /* PA13: Print1 */
    GPIO_DRV_SetMuxModeSel(PORTA, 13U, PORT_MUX_ALT6);
    /* PA12: Print2 */
    GPIO_DRV_SetMuxModeSel(PORTA, 12U, PORT_MUX_ALT6);

    /* Config DEBUG-PWM Interface */
    memset(&SimplyConfig, 0U, sizeof(pwm_simply_config_t));
    SimplyConfig.allChCombineMode       = PWM_INDEPENDENT_MODE;
    SimplyConfig.countMode              = PWM_UP_COUNT;
    /* PWM_HIGH_TRUE */
    SimplyConfig.levelMode              = PWM_HIGH_TRUE;
    /* PWM_CLK_SOURCE_BUS */
    SimplyConfig.clkSource              = PWM_CLK_SOURCE_SYSTEM;
    SimplyConfig.clkPsc                 = 0U;
    SimplyConfig.initValue              = 0U;
    SimplyConfig.maxValue               = 4096U;
    SimplyConfig.chValue[0]             = 0U;
    SimplyConfig.chValue[2]             = 0U;
    SimplyConfig.chValue[4]             = 0U;
    SimplyConfig.interruptEn            = false;
    SimplyConfig.overflowCallback       = NULL;
    PWM_DRV_SimplyInit(PWM_MODULER_DEBUG, &SimplyConfig);
}

/*!
* @brief SPI Initialization, SPI can be used when external DAC is uesed for debugging.
*
* @param[in] none
* @return none
*/
void MC_DEBUG_SPI_Initialize(spi_master_config_t *spiConfig)
{
    DEVICE_ASSERT(spiConfig != NULL);

    /*
    * GPIO Configuration for SPI
    * PD0     --> SCK
    * PD1     --> SIN
    * PE1     --> CS0
    * PE0     --> SOUT
    */

    /* PD0: SPI1_SCK */
    GPIO_DRV_SetMuxModeSel(PORTD, 0U, PORT_MUX_ALT3);
    /* PD1: SPI1_SIN */
    GPIO_DRV_SetMuxModeSel(PORTD, 1U, PORT_MUX_ALT3);
    /* PE1: SPI1_CS0 */
    GPIO_DRV_SetMuxModeSel(PORTE, 1U, PORT_MUX_ALT5);
    /* PE0: SPI1_SOUT */
    GPIO_DRV_SetMuxModeSel(PORTE, 0U, PORT_MUX_ALT5);

    spiConfig->bitsPerSec       = 24000000U;
    spiConfig->whichPcs         = SPI_PCS0;
    spiConfig->pcsPolarity      = SPI_ACTIVE_LOW;
    spiConfig->isPcsContinuous  = false;
    spiConfig->bitcount         = 8U;
    spiConfig->spiSrcClk        = 24000000U;
    spiConfig->clkPhase         = SPI_CLOCK_PHASE_1ST_EDGE;
    spiConfig->clkPolarity      = SPI_SCK_ACTIVE_HIGH;
    spiConfig->msbFirst         = true;
    spiConfig->transferType     = SPI_USING_INTERRUPTS;
    spiConfig->rxDMAChannel     = 0U;
    spiConfig->txDMAChannel     = 1U;
    spiConfig->callback         = NULL;
    spiConfig->callbackParam    = NULL;

    SPI_DRV_MasterInit(0, &s_spiMasterState, spiConfig);
}

/*!
* @brief Motor control debug function initialization, SPI and PWM can be used for debugging output.
*
* @param[in] none
* @return none
*/
void MC_DEBUG_Initialize(void)
{
    if (DAC_PWM == ENABLE)
    {
        MC_DEBUG_PWM_Initialize(PWM2);
    }
    if (DAC_SPI == ENABLE)
    {
        MC_DEBUG_SPI_Initialize(&spiMasteConfig);
    }
}

/*!
* @brief FOC PWM Module Initialize.
*
* @param[in] none
* @return none
*/
void FOC_PWM_Initialize(void)
{
    PWM_DRV_Deinit(PWM_MODULER_MOTOR1);
    pwm_combine_ch_config_t chConfig[4U];
    pwm_modulation_config_t pwmConfig;
    pwm_config_t config;
    pwm_fault_config_t faultConfig;
    pwm_sync_config_t syncConfig;

    /*
    * GPIO Configuration for PWM1
    * PB5     --> PWM1_IN0
    * PB4     --> PWM1_IN1
    * PB8     --> PWM1_IN2
    * PB7     --> PWM1_IN3
    * PB10    --> PWM1_IN4
    * PB9     --> PWM1_IN5
    * PB6     --> PWM1_IN6
    */
    /* PA2: PWM3_CH0, PWM_U+ */
    GPIO_DRV_SetMuxModeSel(PORTA, 2U, PORT_MUX_ALT2);
    /* PA3: PWM3_CH1, PWM_U- */
    GPIO_DRV_SetMuxModeSel(PORTA, 3U, PORT_MUX_ALT2);
    /* PC6: PWM3_CH2, PWM_V+ */
    GPIO_DRV_SetMuxModeSel(PORTC, 6U, PORT_MUX_ALT4);
    /* PC7: PWM3_CH3, PWM_V- */
    GPIO_DRV_SetMuxModeSel(PORTC, 7U, PORT_MUX_ALT4);
    /* PD2: PWM3_CH4, PWM_W+ */
    GPIO_DRV_SetMuxModeSel(PORTD, 2U, PORT_MUX_ALT2);
    /* PD3: PWM3_CH5, PWM_W- */
    GPIO_DRV_SetMuxModeSel(PORTD, 3U, PORT_MUX_ALT2);
    /* PE7: PWM3_FLT0, PWM hardware current overflow Fault */
    GPIO_DRV_SetMuxModeSel(PORTE, 7U, PORT_MUX_ALT3);

    /* Config PWM Interface */
    memset(chConfig, 0U, sizeof(chConfig));
    memset(&pwmConfig, 0U, sizeof(pwm_modulation_config_t));
    memset(&faultConfig, 0U, sizeof(pwm_fault_config_t));
    memset(&syncConfig, 0U, sizeof(pwm_sync_config_t));
    memset(&config, 0U, sizeof(pwm_config_t));
    CKGEN_DRV_Enable(CLK_PWM3, true);
    CKGEN_DRV_SoftReset(SRST_PWM3, true);

#if (defined GATE_DRIVER_HIGH_HIGH)
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_0, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_1, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_2, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_3, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_4, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_5, PWM_LOW_LEVEL);

#elif (defined GATE_DRIVER_HIGH_LOW)
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_0, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_1, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_2, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_3, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_4, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_5, PWM_HIGH_LEVEL);
#elif (defined GATE_DRIVER_LOW_HIGH)
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_0, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_1, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_2, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_3, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_4, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_5, PWM_LOW_LEVEL);
#elif (defined GATE_DRIVER_LOW_LOW)
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_0, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_1, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_2, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_3, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_4, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_5, PWM_HIGH_LEVEL);
#endif
    PWM_InitChannelOutputEn(PWM3, true);

    /* channel 0/1 */
    chConfig[0U].pairChannel                                     = PWM_CH_0;
    chConfig[0U].ch1stValue                                      = 1U;
    chConfig[0U].ch2ndValue                                      = 1U;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0U].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0U].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[0U].complementEn                                    = true;
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0U].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
    chConfig[0U].complementEn                                    = false;
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0U].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[0U].complementEn                                    = false;
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0U].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[0U].complementEn                                    = true;
#endif
    chConfig[0U].deadtimeEn                                      = true;
    chConfig[0U].deadtime                                        = PWM_DEAD_TIME;
    chConfig[0U].deadtimePsc                                     = PWM_DEADTIME_DIVID_4;

    chConfig[0U].ch1stMatchDir                                   = PWM_MATCH_DIR_UP;
    chConfig[0U].ch2ndMatchDir                                   = PWM_MATCH_DIR_DOWN;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[0U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[0U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[0U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[0U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[0U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #endif
#endif

    /* channel 2/3 */
    chConfig[1U].pairChannel                                     = PWM_CH_2;
    chConfig[1U].ch1stValue                                      = 1U;
    chConfig[1U].ch2ndValue                                      = 1U;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1U].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1U].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[1U].complementEn                                    = true;
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1U].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
    chConfig[1U].complementEn                                    = false;
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1U].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[1U].complementEn                                    = false;
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1U].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[1U].complementEn                                    = true;
#endif
    chConfig[1U].deadtimeEn                                      = true;
    chConfig[1U].deadtime                                        = PWM_DEAD_TIME;
    chConfig[1U].deadtimePsc                                     = PWM_DEADTIME_DIVID_4;

    chConfig[1U].ch1stMatchDir                                   = PWM_MATCH_DIR_UP;
    chConfig[1U].ch2ndMatchDir                                   = PWM_MATCH_DIR_DOWN;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[1U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[1U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[1U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[1U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[1U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #endif
#endif

    /* channel 4/5 */
    chConfig[2U].pairChannel                                     = PWM_CH_4;
    chConfig[2U].ch1stValue                                      = 1U;
    chConfig[2U].ch2ndValue                                      = 1U;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2U].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2U].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[2U].complementEn                                    = true;
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2U].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
    chConfig[2U].complementEn                                    = false;
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2U].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[2U].complementEn                                    = false;
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2U].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[2U].complementEn                                    = true;
#endif
    chConfig[2U].deadtimeEn                                      = true;
    chConfig[2U].deadtime                                        = PWM_DEAD_TIME;
    chConfig[2U].deadtimePsc                                     = PWM_DEADTIME_DIVID_4;

    chConfig[2U].ch1stMatchDir                                   = PWM_MATCH_DIR_UP;
    chConfig[2U].ch2ndMatchDir                                   = PWM_MATCH_DIR_DOWN;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[2U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[2U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[2U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[2U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2U].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[2U].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #endif
#endif

#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    /* channel 6/7 */
    chConfig[3U].pairChannel                                     = PWM_CH_6;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* 0 means Up edge sample */
    #if (SAMPLE_TRIG_EDGE == 0U)
    {
        chConfig[3U].ch1stValue                                  = PWM_PERIOD_VALUE >> 2U;
        chConfig[3U].ch2ndValue                                  = (PWM_PERIOD_VALUE >> 1U);
    }
    /* 1 means Down edge sample */
    #else
    {
        chConfig[3U].ch1stValue                                  = PWM_PERIOD_VALUE >> 1U;
        chConfig[3U].ch2ndValue                                  = (PWM_PERIOD_VALUE >> 2U);
    }
    #endif

    #else
    chConfig[3U].ch1stValue                                     = (PWM_PERIOD_VALUE >> 4U);
    chConfig[3U].ch2ndValue                                     = (PWM_PERIOD_VALUE >> 4U) + (PWM_PERIOD_VALUE >> 1U);
    #endif
    #else

    #endif

    chConfig[3U].levelMode                                      = PWM_HIGH_TRUE;
    chConfig[3U].deadtimeEn                                     = false;
    chConfig[3U].complementEn                                   = false;
    /* 0 means Up edge sample */
    #if (SAMPLE_TRIG_EDGE == 0U)
    {
        chConfig[3U].ch1stMatchDir                              = PWM_MATCH_DIR_UP;
        chConfig[3U].ch2ndMatchDir                              = PWM_MATCH_DIR_UP;
    }
    /* 1 means Down edge sample */
    #else
    {
        chConfig[3U].ch1stMatchDir                              = PWM_MATCH_DIR_DOWN;
        chConfig[3U].ch2ndMatchDir                              = PWM_MATCH_DIR_DOWN;
    }
    #endif

    chConfig[3U].ch1stMatchTriggerEn                            = true;
    chConfig[3U].ch2ndMatchTriggerEn                            = true;
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    #if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[3U].pairChannel                                    = PWM_CH_6;
    chConfig[3U].ch1stValue                                     = PWM_PERIOD_VALUE;
    chConfig[3U].ch2ndValue                                     = 0U;
    chConfig[3U].levelMode                                      = PWM_HIGH_TRUE;
    chConfig[3U].deadtimeEn                                     = false;
    chConfig[3U].complementEn                                   = false;
    chConfig[3U].ch1stMatchDir                                  = PWM_MATCH_DIR_UP;
    chConfig[3U].ch2ndMatchDir                                  = PWM_MATCH_DIR_UP;
    chConfig[3U].ch1stTriggerEn                                 = true;
    chConfig[3U].ch2ndTriggerEn                                 = false;
    #endif

    #elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    #endif

    #elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif

    #elif (defined GATE_DRIVER_LOW_LOW)
    #endif

#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    chConfig[3].pairChannel                                     = PWM_CH_6;
    chConfig[3].ch1stValue                                      = 1U;
    chConfig[3].levelMode                                       = PWM_HIGH_TRUE;
    chConfig[3].deadtimeEn                                      = false;
    chConfig[3].complementEn                                    = false;
    chConfig[3].ch1stMatchDir                                   = PWM_MATCH_DIR_DOWN;
    chConfig[3].ch1stMatchTriggerEn                             = true;
#endif

    /* modulation mode config */
#if ((CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE))
    pwmConfig.initTriggerEn                                     = false;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode                                         = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode                                         = PWM_UP_COUNT;
    #endif
    pwmConfig.combineChannelNum                                 = 4U;
    pwmConfig.combineChConfig                                   = chConfig;

#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    #if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    pwmConfig.initTriggerEn                                     = true;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode                                         = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode                                         = PWM_UP_COUNT;
    #endif
    pwmConfig.combineChannelNum                                 = 3U;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    pwmConfig.initTriggerEn                                     = false;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode                                         = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode                                         = PWM_UP_COUNT;
    #endif
    pwmConfig.combineChannelNum                                 = 4U;
    #endif

    #elif (defined GATE_DRIVER_HIGH_LOW)
    pwmConfig.initTriggerEn                                     = true;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode                                         = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode                                         = PWM_UP_COUNT;
    #endif
    pwmConfig.combineChannelNum                                 = 3U;

#elif (defined GATE_DRIVER_LOW_HIGH)
    pwmConfig.initTriggerEn                                     = true;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode                                         = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode                                         = PWM_UP_COUNT;
    #endif
    pwmConfig.combineChannelNum                                 = 3U;

#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    pwmConfig.initTriggerEn                                     = true;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode                                         = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode                                         = PWM_UP_COUNT;
    #endif
    pwmConfig.combineChannelNum                                 = 3U;
    #endif
    #endif

    pwmConfig.combineChConfig                                   = chConfig;
#endif

    pwmConfig.centerAlignDutyType = PWM_DUTY_MODE_1;

    /* pwm config */
    config.mode                                                 = PWM_MODE_MODULATION;
    config.initModeStruct                                       = &pwmConfig;
    config.clkSource                                            = PWM_CLK_SOURCE_NONE;
    config.clkPsc                                               = PWM_PRSC;
    config.maxValue                                             = PWM_PERIOD_VALUE;
    config.interruptEn                                          = ENABLE;
    PWM_DRV_Init(PWM_MODULER_MOTOR1, &config);

    /* pwm sync config */
    syncConfig.syncTriggerMode                                  = PWM_SYNC_TRIGGER_SOFTWARE;
    syncConfig.hardwareSync0En                                  = false;
    syncConfig.hardwareSync1En                                  = false;
    syncConfig.hardwareSync2En                                  = false;
    syncConfig.autoClearHWTriggerEn                             = false;
    syncConfig.counterInitSyncEn                                = true;
    syncConfig.outCtrlSyncEn                                    = false;
    syncConfig.inverterSyncEn                                   = false;
    syncConfig.outmaskSyncEn                                    = false;
    syncConfig.polaritySyncEn                                   = false;
    syncConfig.chValueSyncEn[0U]                                = true;
    syncConfig.chValueSyncEn[1U]                                = true;
    syncConfig.chValueSyncEn[2U]                                = true;
    syncConfig.chValueSyncEn[3U]                                = true;
    syncConfig.maxLoadingPointEn                                = false;
    syncConfig.minLoadingPointEn                                = true;
    syncConfig.pwmSyncEnable                                    = true;
    PWM_DRV_InitSyncControl(PWM_MODULER_MOTOR1, &syncConfig);

    /* fault control config */
    faultConfig.channelConfig[PWM_FAULT_CH_0].faultInputEn      = true;
    faultConfig.channelConfig[PWM_FAULT_CH_0].faultFilterEn     = false;
    faultConfig.channelConfig[PWM_FAULT_CH_0].faultPolarity     = OVERCUR_FEEDBACK_POLARITY;
    faultConfig.mode                                            = PWM_FAULT_CTRL_MANUAL_ALL;
    faultConfig.filterValue                                     = 0U;
    faultConfig.interruptEn                                     = true;
    faultConfig.faultCtrlOutputEn[0U]                           = true;
    faultConfig.faultCtrlOutputEn[1U]                           = true;
    faultConfig.faultCtrlOutputEn[2U]                           = true;
    faultConfig.faultCtrlOutputEn[3U]                           = true;
    faultConfig.faultCallback                                   = FOC_PWM_FaultCallback;
    PWM_DRV_InitFaultControl(PWM_MODULER_MOTOR1, &faultConfig);

    /* Set interrupt priority */
    NVIC_SetPriority(PWM3_FAULT_IRQn, 0U);
    /* PWM Start */
    PWM_SetClockSource(PWM3, PWM_CLK_SOURCE_SYSTEM);
}

/*!
* @brief FOC_TIM_Initialize(timer Initialize), generates timer interrupt.
*
* @param[in] none
* @return none
*/
void FOC_TIM_Initialize(void)
{
    module_clk_config_t clkConfig = {SPLLDIV2_CLK};
    timer_user_channel_config_t timerConfig;

    memset(&timerConfig, 0U, sizeof(timer_user_channel_config_t));

    CKGEN_DRV_SetModuleClock(TIMER_CLK, &clkConfig);

    /* Timer channel configuration */
    timerConfig.timerMode               = TIMER_PERIODIC_COUNTER;
    timerConfig.periodUnits             = TIMER_PERIOD_UNITS_COUNTS;
    timerConfig.period                  = TIMER_1MS;
    timerConfig.triggerSource           = TIMER_TRIGGER_SOURCE_EXTERNAL;
    timerConfig.triggerSelect           = 0U;
    timerConfig.enableReloadOnTrigger   = false;
    timerConfig.enableStopOnInterrupt   = false;
    timerConfig.enableStartOnTrigger    = false;
    timerConfig.chainChannel            = false;
    timerConfig.isInterruptEnabled      = true;
    timerConfig.callback                = Timer0_Callback;

    TIMER_DRV_Init(0U, true);
    TIMER_DRV_InitChannel(0U, TIMER_CHANNEL_0, &timerConfig);
    TIMER_DRV_StartChannels(0U, 1U << TIMER_CHANNEL_0);

    /* ENCODER MT Speed measurement Timer  */
    memset(&timerConfig, 0U, sizeof(timer_user_channel_config_t));

    /* Timer channel configuratino */
    timerConfig.timerMode               = TIMER_PERIODIC_COUNTER;
    timerConfig.periodUnits             = TIMER_PERIOD_UNITS_COUNTS;
    timerConfig.period                  = M_T_PERIOD;
    timerConfig.triggerSource           = TIMER_TRIGGER_SOURCE_EXTERNAL;
    timerConfig.triggerSelect           = 0U;
    timerConfig.enableReloadOnTrigger   = false;
    timerConfig.enableStopOnInterrupt   = false;
    timerConfig.enableStartOnTrigger    = false;
    timerConfig.chainChannel            = false;
    timerConfig.isInterruptEnabled      = true;
    timerConfig.callback                = Timer1_Callback;

    TIMER_DRV_Init(0U, true);
    TIMER_DRV_InitChannel(0U, TIMER_CHANNEL_1, &timerConfig);
    TIMER_DRV_StartChannels(0U, 1U << TIMER_CHANNEL_1);

    /* Set interrupt priority */
    NVIC_SetPriority(TIMER_CHANNEL0_IRQn, 3U);
    NVIC_SetPriority(TIMER_CHANNEL1_IRQn, 1U);
}

/*!
* @brief MCU PDT module initialize.
*
* @param[in] none
* @return none
*/
void FOC_PDT_DELAY_Initialize(void)
{
    pdt_timer_config_t pdttimerConfig;
    pdt_trigger_delay_config_t triggerdlyConfig;

  /* PDT delay[n], software trigger, continue mode */
    memset(&pdttimerConfig, 0U, sizeof(pdt_timer_config_t));
    memset(&triggerdlyConfig, 0U, sizeof(pdt_trigger_delay_config_t));
    pdttimerConfig.clkPreMultFactor = PDT_CLK_PREMULT_FACT_AS_1;
    pdttimerConfig.clkPreDiv = PDT_CLK_PREDIV_BY_16;    /* calcculated: 1count - 0.83us */
    pdttimerConfig.loadValueMode = PDT_LOAD_VAL_IMMEDIATELY;
    pdttimerConfig.triggerInput = PDT_HARDWARE_TRIGGER;
    pdttimerConfig.continuousModeEnable = false;
    pdttimerConfig.intEnable = true;
    triggerdlyConfig.triggerDelayBypassEn = false;
    triggerdlyConfig.delayEnable= 0x01;
    triggerdlyConfig.dly[0] = 0; /* 6.6ms */

    PDT_DRV_Init(0, &pdttimerConfig);
    PDT_DRV_ConfigTriggerDelay(0, &triggerdlyConfig);
    PDT_DRV_SetTimerModulusValue(0, 6);

    PDT_DRV_LoadValuesCmd(0);
    PDT_DRV_Enable(0);
}
/*!
* @brief CTU Module Initialize.
*
* @param[in] none
* @return none
*/
void FOC_CTU_Initialize(void)
{
    CTU_DRV_DeInit(0U);
    CTU_DRV_Init(0U);

//    FOC_PDT_DELAY_Initialize();
//    TRGMUX_DRV_SetTrigSourceForTargetModule(0U, CTU_ADC_TRIGGER_SOURCE, TRGMUX_TARGET_MODULE_PDT0);
//    TRGMUX_DRV_SetTrigSourceForTargetModule(0U, TRGMUX_TRIG_SOURCE_PDT0_TRIG, TRGMUX_TARGET_MODULE_ADC0_INJECTION0);

    TRGMUX_DRV_SetTrigSourceForTargetModule(0U, CTU_ADC_TRIGGER_SOURCE, TRGMUX_TARGET_MODULE_ADC0_INJECTION0); 
    CTU_DRV_SetADCInjectSimultaneous(0U, true);
}

/*!
* @brief DMA Module Initialize.
*
* @param[in] none
* @return none
*/
void FOC_DMA_Init()
{
    uint32_t tmpMemStartAddr0                   = (uint32_t)&g_ADCRegularBuffer[0];
    uint32_t tmpMemStartAddr1                   = (uint32_t)&g_ADC1RegularBuffer[0];

    dmaController1Chn0_Config.channelPriority = DMA_CHN_PRIORITY_VERY_HIGH;
    dmaController1Chn0_Config.virtChnConfig = 15U;
    dmaController1Chn0_Config.source = DMA_REQ_ADC0; /* PWM channel 7 trigger, last channel */
//    dmaController1Chn0_Config.callback = PWM_DMACallback;
    dmaController1Chn0_Config.callbackParam = NULL;
    dmaController1Chn0_Config.enableTrigger = false;

    dmaController1Chn1_Config.channelPriority = DMA_CHN_PRIORITY_VERY_HIGH;
    dmaController1Chn1_Config.virtChnConfig = 14U;
    dmaController1Chn1_Config.source = DMA_REQ_ADC1; /* PWM channel 7 trigger, last channel */
    dmaController1Chn1_Config.callbackParam = NULL;
    dmaController1Chn1_Config.enableTrigger = false;

//    DMA_DRV_Init(&dmaController_State, dmaChnStateArray, dmaChnConfigArray, 1U);
    DMA_DRV_Init(dmaStateArray, dmaChnStateArray, dmaChnConfigArray, 2U);

    DMA_DRV_ConfigTransfer(15U, DMA_TRANSFER_PERIPH2MEM, (uint32_t)(&(ADC0->RDR[0])), tmpMemStartAddr0, DMA_TRANSFER_SIZE_2B, sizeof(g_ADCRegularBuffer));

    DMA_DRV_SetSrcOffset(15U, 4U);
    DMA_DRV_SetSrcAddr(15U, (uint32_t)(&(ADC0->RDR[0])), (uint32_t)(&(ADC0->RDR[2])));
//    DMA_DRV_SetDestOffset(15U, 4U);
    DMA_DRV_SetDestOffset(15U, 2U);

//    DMA_DRV_SetDestAddr(15U, (uint32_t)g_ADCRegularBuffer, (uint32_t)&g_ADCRegularBuffer[6]);
    DMA_DRV_SetDestAddr(15U, (uint32_t)g_ADCRegularBuffer, (uint32_t)g_ADCRegularBuffer + sizeof(g_ADCRegularBuffer));

//    DMA_DRV_ConfigureInterrupt(15, DMA_CHN_ERR_INT, false);
    DMA_DRV_ConfigureInterrupt(15, DMA_CHN_ERR_INT, true);
//    DMA_DRV_ConfigureInterrupt(15, DMA_CHN_HALF_FINISH_INT, false);
    DMA_DRV_ConfigureInterrupt(15, DMA_CHN_FINISH_INT, false);

    DMA_DRV_SetCircularMode(15U, true);

    DMA_DRV_StopChannel(15U);
    DMA_DRV_StartChannel(15U);


    DMA_DRV_ConfigTransfer(14U, DMA_TRANSFER_PERIPH2MEM, (uint32_t)(&(ADC1->RDR[0])), tmpMemStartAddr1, DMA_TRANSFER_SIZE_2B, sizeof(g_ADC1RegularBuffer));

    DMA_DRV_SetSrcOffset(14U, 4U);
    DMA_DRV_SetSrcAddr(14U, (uint32_t)(&(ADC1->RDR[0])), (uint32_t)(&(ADC1->RDR[3])));
    DMA_DRV_SetDestOffset(14U, 2U);
    DMA_DRV_SetDestAddr(14U, (uint32_t)g_ADC1RegularBuffer, (uint32_t)g_ADC1RegularBuffer + sizeof(g_ADC1RegularBuffer));

    DMA_DRV_ConfigureInterrupt(14, DMA_CHN_ERR_INT, false);
//    DMA_DRV_ConfigureInterrupt(14, DMA_CHN_HALF_FINISH_INT, false);
    DMA_DRV_ConfigureInterrupt(14, DMA_CHN_FINISH_INT, false);

    DMA_DRV_SetCircularMode(14U, true);

    DMA_DRV_StopChannel(14U);
    DMA_DRV_StartChannel(14U);
}

/*!
* @brief Calbiration ADC0 to get adc channel offset value.
*
* @param[in] sequencechannel : ADC0 Inject Group sequence num, 0-3
* @return dcoffset: ADC0 Inject Group channel offset
*/
int16_t MC_ADC_Calbiration(uint8_t sequencechannel)
{
    uint16_t index = 0;
    uint32_t sumOffset =0;
    int16_t adcOffset = 0;

    ADC_SetRegularTriggerSource(ADC0, ADC_TRIGGER_INTERNAL);
    ADC_SetInjectTriggerSource(ADC0, ADC_TRIGGER_INTERNAL);

    ADC_ClearInjectEOCFlag(ADC0, (adc_sequence_t)(ADC_ISEQ_0 + sequencechannel));
    udelay(100);

    for (index = 0; index < NUM_OF_CONVERSIONS; index ++)
    {
        ADC_DRV_SoftwareStartInjectConvert(ADC0_MODULER);
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
        /* interval mode,inject length is 2, must trigger all channel, then read the data */
        udelay(10);
        ADC_SoftwareStartInjectConvert(ADC0);
#endif

        while (!ADC_GetInjectEOCFlag(ADC0, (adc_sequence_t)(ADC_ISEQ_0 + ADC_GetInjectLength(ADC0) - 1)));
        sumOffset += ADC_GetInjectData(ADC0, (adc_sequence_t)(ADC_ISEQ_0 + sequencechannel));

        ADC_ClearInjectEOCFlag(ADC0, (adc_sequence_t)(ADC_ISEQ_0 + sequencechannel));
    }
    adcOffset = sumOffset / NUM_OF_CONVERSIONS;

    return adcOffset;
}

/*!
* @brief Calbiration ADC1 to get adc channel offset value.
*
* @param[in] sequencechannel : ADC1 Inject Group sequence num, 0-3
* @return dcoffset: ADC1 Inject Group channel offset
*/
int16_t MC_ADC1_Calbiration(uint8_t sequencechannel)
{
    uint16_t index = 0;
    uint32_t sumOffset =0;
    int16_t adcOffset = 0;

    ADC_SetRegularTriggerSource(ADC1, ADC_TRIGGER_INTERNAL);
    ADC_SetInjectTriggerSource(ADC1, ADC_TRIGGER_INTERNAL);

    ADC_ClearInjectEOCFlag(ADC1, (adc_sequence_t)(ADC_ISEQ_0 + sequencechannel));

    udelay(100);

    for (index = 0; index < NUM_OF_CONVERSIONS; index ++)
    {
        ADC_SoftwareStartInjectConvert(ADC1);
//#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
//        /* interval mode,inject length is 2, must trigger all channel, then read the data */
//        udelay(10);
//        ADC_SoftwareStartInjectConvert(ADC1);
//#endif

        while (!ADC_GetInjectEOCFlag(ADC1, (adc_sequence_t)(ADC_ISEQ_0 + ADC_GetInjectLength(ADC1) - 1)));
        sumOffset += ADC_GetInjectData(ADC1, (adc_sequence_t)(ADC_ISEQ_0 + sequencechannel));

        ADC_ClearInjectEOCFlag(ADC1, (adc_sequence_t)(ADC_ISEQ_0 + sequencechannel));
    }
    adcOffset = sumOffset / NUM_OF_CONVERSIONS;

    return adcOffset;
}

/*!
* @brief Get phase current sampling offset value.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @return none
*/
void FOC_ADC_Calbiration(ADC_SAMPLE *adcSampleReal)
{
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    adcSampleReal->ibusOffset = MC_ADC_Calbiration(0);
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    adcSampleReal->iaOffset = MC_ADC_Calbiration(0);
    adcSampleReal->ibOffset = MC_ADC_Calbiration(1);
//    adcSampleReal->ibusOffset = MC_ADC_Calbiration(2);
#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    adcSampleReal->iaOffset = MC_ADC_Calbiration(0);
    adcSampleReal->ibOffset = MC_ADC_Calbiration(1);
    adcSampleReal->icOffset = MC_ADC_Calbiration(2);
//    adcSampleReal->ibusOffset = MC_ADC_Calbiration(3);
#else

#endif
}

/*!
* @brief ADC Initialize, first calibration current Offset, then config adc as external trigger mode.
*
* @param[in] none
* @return none
*/
void FOC_ADC_Initialize(void)
{
//    ADC_DRV_Deinit(ADC0_MODULER);
//    ADC_DRV_Deinit(ADC1_MODULER);

    adc_converter_config_t tempAdcConfig;
    adc_converter_config_t* adcConfig;
    adcConfig = &tempAdcConfig;
    memset(adcConfig, 0x00, sizeof(adc_converter_config_t));

    adc_chan_config_t tempAdcChConfig;
    adc_chan_config_t* adcChConfig;
    adcChConfig = &tempAdcChConfig;
    memset(adcChConfig, 0x00, sizeof(adc_chan_config_t));

    /*
    * GPIO Configuration for ADC
    * PA7     --> ADC0_IN3, IU
    * PB1     --> ADC0_IN5, IV
    * PB0     --> ADC0_IN4, IW
    * PA6     --> ADC0_IN2, IBUS
    * PB13    --> ADC1_IN8, IBUS_SUM
    * PB12    --> ADC1_IN7, VBUS
    * PD4     --> ADC1_IN6, TEMP
    * PC16    --> ADC0_IN14, KEY
    * PC15    --> ADC0_IN13, SPEED_AD
    */

    /* ADC0 channel 13, VARIOHM_SPEED_ADJUST */
    GPIO_DRV_SetMuxModeSel(PORTC, 15U, PORT_PIN_DISABLED);
    /* ADC0 channel 14, Control keys(Star_Stop/Fast/Slow/Direction) */
    GPIO_DRV_SetMuxModeSel(PORTC, 16U, PORT_PIN_DISABLED);

    /* ADC1 channel 7, V_Bus */
    GPIO_DRV_SetMuxModeSel(PORTB, 12U, PORT_PIN_DISABLED);
    /* ADC1 channel 8, I_Bus_Avg */
    GPIO_DRV_SetMuxModeSel(PORTB, 13U, PORT_PIN_DISABLED);
    /* ADC1 channel 6, TEMP */
    GPIO_DRV_SetMuxModeSel(PORTD, 4U, PORT_PIN_DISABLED);

#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    /* ADC0 channel 2, I_Bus */
    GPIO_DRV_SetMuxModeSel(PORTA, 6U, PORT_PIN_DISABLED);

#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    /* ADC0 channel 3, I_U */
    GPIO_DRV_SetMuxModeSel(PORTA, 7U, PORT_PIN_DISABLED);
    /* ADC0 channel 5, I_V */
    GPIO_DRV_SetMuxModeSel(PORTB, 1U, PORT_PIN_DISABLED);

    /* ADC0 channel 2, I_Bus */
    GPIO_DRV_SetMuxModeSel(PORTA, 6U, PORT_PIN_DISABLED);

#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    /* ADC0 channel 3, I_U */
    GPIO_DRV_SetMuxModeSel(PORTA, 7U, PORT_PIN_DISABLED);
    /* ADC0 channel 5, I_V */
    GPIO_DRV_SetMuxModeSel(PORTB, 1U, PORT_PIN_DISABLED);
    /* ADC0 channel 4, I_W */
    GPIO_DRV_SetMuxModeSel(PORTB, 0U, PORT_PIN_DISABLED);

    /* ADC0 channel 2, I_Bus */
    GPIO_DRV_SetMuxModeSel(PORTA, 6U, PORT_PIN_DISABLED);
#else

#endif

    ADC_DRV_Init(ADC0_MODULER);
    ADC_DRV_Init(ADC1_MODULER);

    module_clk_config_t clkConfig = {SPLLDIV2_CLK};
    CKGEN_DRV_SetModuleClock(ADC0_CLK, &clkConfig);
    CKGEN_DRV_SetModuleClock(ADC1_CLK, &clkConfig);

    adcConfig->clockDivide = ADC_CLK_DIVIDE_2;
    adcConfig->resolution = ADC_RESOLUTION_12BIT;

    adcConfig->regularTrigger = ADC_TRIGGER_INTERNAL;
    adcConfig->injectTrigger = ADC_TRIGGER_INTERNAL;

    adcConfig->dmaEnable = true;

    adcConfig->voltageRef = ADC_VOLTAGEREF_VREF;
    /* ADC mode 3 */
    adcConfig->scanModeEn = true;
    adcConfig->continuousModeEn = false;
    adcConfig->regularDiscontinuousModeEn = false;
    adcConfig->injectDiscontinuousModeEn = false;
    adcConfig->injectAutoModeEn = false;
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    adcConfig->intervalModeEn = true;
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    adcConfig->intervalModeEn = false;
#else
#endif

    adcConfig->regularDiscontinuousNum = 0U;
    adcConfig->powerEn = true;

    adcConfig->regularSequenceLength = 2;
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    adcConfig->injectSequenceLength = 2;
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
//    adcConfig->injectSequenceLength = 3;
    adcConfig->injectSequenceLength = 2;        /*!< Donot get bus current sample */
#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
//    adcConfig->injectSequenceLength = 4;
    adcConfig->injectSequenceLength = 3;        /*!< Donot get bus current sample */
#else
#endif

    adcChConfig->spt = ADC_SPT_CLK_15;

    adcChConfig->supply = ADC_CH_SUPPLY_VDD;
    adcChConfig->interruptEn = false;

    ADC_DRV_ConfigConverter(ADC0_MODULER, adcConfig);
    ADC_DRV_InstallCallback(ADC0_MODULER, Motor_Ctrl_CallBack, NULL);

    adcConfig->regularSequenceLength = 3;
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    adcConfig->injectSequenceLength = 0;
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    adcConfig->injectSequenceLength = 0;
#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    adcConfig->injectSequenceLength = 0;
#else
#endif
    ADC_DRV_ConfigConverter(ADC1_MODULER, adcConfig);
//    ADC_DRV_InstallCallback(ADC1_MODULER, Motor_Ctrl_CallBack, NULL);
    ADC_DRV_InstallCallback(ADC1_MODULER, NULL, NULL);

    /* Key */
    adcChConfig->channel = ADC_CH_14;
    ADC_DRV_ConfigChan(ADC0_MODULER, ADC_RSEQ_0, adcChConfig);
    /* ADC SPEED */
    adcChConfig->channel = ADC_CH_13;
    ADC_DRV_ConfigChan(ADC0_MODULER, ADC_RSEQ_1, adcChConfig);
    /* VBUS */
    adcChConfig->channel = ADC_CH_7;
    ADC_DRV_ConfigChan(ADC1_MODULER, ADC_RSEQ_0, adcChConfig);
    /* IBUS_Avg */
    adcChConfig->channel = ADC_CH_8;
    ADC_DRV_ConfigChan(ADC1_MODULER, ADC_RSEQ_1, adcChConfig);
    /* TEMP */
    adcChConfig->channel = ADC_CH_6;
    ADC_DRV_ConfigChan(ADC1_MODULER, ADC_RSEQ_2, adcChConfig);

#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    /* ADC0 IBUS */
    adcChConfig->channel = ADC_CH_2;
    ADC_DRV_ConfigChan(ADC0_MODULER, ADC_ISEQ_0, adcChConfig);
    ADC_DRV_ConfigChan(ADC0_MODULER, ADC_ISEQ_1, adcChConfig);

#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    /* ADC0 IU */
    adcChConfig->channel = ADC_CH_3;
    ADC_DRV_ConfigChan(ADC0_MODULER, ADC_ISEQ_0, adcChConfig);
    /* ADC0 IV */
    adcChConfig->channel = ADC_CH_5;
    ADC_DRV_ConfigChan(ADC0_MODULER, ADC_ISEQ_1, adcChConfig);
    /* ADC0 IBUS */
//    adcChConfig->channel = ADC_CH_2;
//    ADC_DRV_ConfigChan(ADC0_MODULER, ADC_ISEQ_2, adcChConfig);

#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    /* ADC0 IU */
    adcChConfig->channel = ADC_CH_3;
    ADC_DRV_ConfigChan(ADC0_MODULER, ADC_ISEQ_0, adcChConfig);
    /* ADC0 IV */
    adcChConfig->channel = ADC_CH_5;
    ADC_DRV_ConfigChan(ADC0_MODULER, ADC_ISEQ_1, adcChConfig);
    /* ADC0 IW */
    adcChConfig->channel = ADC_CH_4;
    ADC_DRV_ConfigChan(ADC0_MODULER, ADC_ISEQ_2, adcChConfig);
    /* ADC0 IBUS */
//    adcChConfig->channel = ADC_CH_2;
//    ADC_DRV_ConfigChan(ADC0_MODULER, ADC_ISEQ_3, adcChConfig);

#else
#endif

    /* Calbiration Ia,Ib,Ic Offset */
    FOC_ADC_Calbiration(&g_adcSampleReal);

    ADC_SetInjectTriggerSource(ADC0, ADC_TRIGGER_EXTERNAL);
    ADC_SetRegularTriggerSource(ADC0, ADC_TRIGGER_INTERNAL);

//    ADC_SetInjectTriggerSource(ADC1, ADC_TRIGGER_EXTERNAL);
    ADC_SetRegularTriggerSource(ADC1, ADC_TRIGGER_INTERNAL);

#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    ADC_ClearInjectEOCFlag(ADC0, ADC_ISEQ_0);
    ADC_ClearInjectEOCFlag(ADC0, ADC_ISEQ_1);
#endif

#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    ADC_SetInjectEOCInterruptEnableFlag(ADC0, ADC_ISEQ_1, ENABLE);
#elif (CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE)
//    ADC_SetInjectEOCInterruptEnableFlag(ADC0, ADC_ISEQ_2, ENABLE);
    ADC_SetInjectEOCInterruptEnableFlag(ADC0, ADC_ISEQ_1, ENABLE);
#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
//    ADC_SetInjectEOCInterruptEnableFlag(ADC0, ADC_ISEQ_3, ENABLE);
    ADC_SetInjectEOCInterruptEnableFlag(ADC0, ADC_ISEQ_2, ENABLE);
#else
#endif

    FOC_DMA_Init();
    /* Start adc convert */
    ADC_DRV_SoftwareStartRegularConvert(ADC0_MODULER);
    ADC_DRV_SoftwareStartRegularConvert(ADC1_MODULER);

    NVIC_SetPriority(ADC0_IRQn, 1);
//    NVIC_SetPriority(ADC1_IRQn, 1);
}

/*!
* @brief Peripheral Initialize in foc App.
*
* @param[in] none
* @return none
*/
void FOC_Peripheral_Initialize(void)
{
    FLASH_DRV_Init(&FLASH_InitConfig, &flashConfig);

    FOC_GPIO_Initialize();
    FOC_TIM_Initialize();
    FOC_CTU_Initialize();

//    FOC_ACMP_Initialize();

#if (defined SENSOR_ACIM)
    FOC_Encoder_Initialize();
#elif (defined SENSORLESS_ACIM)
#else
#endif

#if ((defined ENCODER_SENSORS) || (defined ENCODER_HALL_SENSORS))
    FOC_Encoder_Initialize();
#endif

#if (defined DAC_FUNCTIONALITY)
    MC_DEBUG_Initialize();
#endif

    FOC_PWM_Initialize();
    FOC_ADC_Initialize();
//    SpeedControlByPWM_init();

    PWM_Output(PWM_OUTPUT_DISABLE);
}

/* =============================================  EOF  ============================================== */
