/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __BLDC_SPEED_H__
#define __BLDC_SPEED_H__
/*!
* @file bldc_speed.h
*
* @brief This file provides speed command ramp and speed loop processing functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "bldc_parameters_init.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

typedef struct
{
    float speedTarget;            /*!< Speed target, command source in per-unit (pu) */
    float speedTargetRamp;        /*!< speed command after ramp, assign to speed reference in per-unit (pu) */
    float speedFbk;               /*!< Feedback speed in per-unit (pu) */
    float speedTargetLimit;       /*!< Speed target command limit value in per-unit (pu) */
    float speedFbkRpm;            /*!< Feedback mechanical speed Unit:rpm */
    float speedMax;               /*!< Max limit of speed command */
    float speedMin;               /*!< Min limit of speed command */
    uint16_t rampCount;           /*!< Count time in 1ms unit while speed ramp control */
    float speedAcceleration;      /*!< Motor speed regulation acceleration rate */
    float speedDeceleration;      /*!< Motor speed regulation deceleration rate */
    float speedFactor;           /*!< Coefficient: to convert speed from real value to pu value */
} SPEED_RAMP_TYPE;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief The per-unit value of speed ramp value is calculated according to the speed command unit pu.
*
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @param[in] speedCmdPu: Given speed command unit pu
* @return none
*/
void BLDC_ASR_SetRampPu(SPEED_RAMP_TYPE *command, float speedCmdPu);

/*!
* @brief The per-unit value of speed targe value is calculated according to the speed command unit pu.
*
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @param[in] speedCmdPu: Given speed command unit pu
* @return none
*/
void BLDC_ASR_SetTargetPu(SPEED_RAMP_TYPE *command, float speedCmdPu);

/*!
* @brief Get the feedback speed unit in Rpm.
*
* @param[in] speed: pointer to SPEED_RAMP_TYPE structure
* @return speedFbk in Rpm unit
*/
float BLDC_ASR_GetFbkRpm(SPEED_RAMP_TYPE *speed);

/*!
* @brief Get the feedback speed unit in Pu.
*
* @param[in] speed: pointer to SPEED_RAMP_TYPE structure
* @return speedFbk in float Pu unit
*/
float BLDC_ASR_GetFbkPu(SPEED_RAMP_TYPE *speed);

/*!
* @brief Set the per-unit value of feedback speed.
*
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @param[in] speedFbk: speed feedback unit pu
* @return none
*/
void BLDC_ASR_SetFbkPu(SPEED_RAMP_TYPE *command, int16_t speedFbk);

/*!
* @brief Speed ramp calculation initialization.
*
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_TYPE structure
* @return none
*/
void BLDC_ASR_CommandInit(BLDC_VARS_CFG *bldcCfg, SPEED_RAMP_TYPE *speed);

/*!
* @brief Speed ramp configuration.
*
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_TYPE structure
* @return none
*/
void BLDC_ASR_CommandSet(BLDC_VARS_CFG *bldcCfg, SPEED_RAMP_TYPE *speed);

/*!
* @brief Speed ramp calculation. calculate the ramp output according to the speed target value.
*
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] speedRamp: pointer to SPEED_RAMP_TYPE structure
* @return none
*/
void BLDC_ASR_RampCalc(BLDC_VARS_CFG *bldcCfg, SPEED_RAMP_TYPE *speedRamp);

/*!
* @brief Get the speed ramp value. Assign to speed loop reference value.
*
* @param[in] speedRamp: pointer to SPEED_RAMP_TYPE structure
* @return speedTargetRamp: speed targe ramp value
*/
float BLDC_ASR_GetRampPu(SPEED_RAMP_TYPE *speedRamp);

/*!
* @brief Update the feedback speed of motor in different modes(Hall/sensorless).
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @return none
*/
void BLDC_ASR_FbkUpdate(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg, SPEED_RAMP_TYPE *command);

/* ======================================  Functions define  ======================================== */

#endif /* __BLDC_SPEED_H__ */

/* =============================================  EOF  ============================================== */
