/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_adc.c
*
* @brief This file provides ADC sampling functions in bldc control.
*
*/

/* ===========================================  Includes  =========================================== */
#include "adc_drv.h"
#include "adc_hw.h"
#include "bldc_keys.h"
#include "bldc_pid.h"
#include "bldc_pwm.h"
#include "bldc_bemf.h"
#include "bldc_protector.h"
#include "bldc_parameters_define.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Clear bus current sample result.
*
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return none
*/
void Clear_CurrentSample(BLDC_ADC_TYPE *adc)
{
    adc->busCurAD = 0;
    adc->busCurTrue = 0;
    adc->busPowerTrue = 0;
}

/*!
* @brief Bldc control bus voltage sample, get bus voltage feedback.
*
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return none
*/
void Get_VoltageValue(BLDC_ADC_TYPE *adc)
{
#if (defined BEMF_DETECT_SOFTWARE)
    adc->phaseVoltageA = (ADC_GetInjectData(ADC0, ADC_ISEQ_1) - adc->phaseVoltageOffsetA) < 0.0f ? 0.0f : (ADC_GetInjectData(ADC0, ADC_ISEQ_1) - adc->phaseVoltageOffsetA);
    adc->phaseVoltageB = (ADC_GetInjectData(ADC0, ADC_ISEQ_2) - adc->phaseVoltageOffsetB) < 0.0f ? 0.0f : (ADC_GetInjectData(ADC0, ADC_ISEQ_2) - adc->phaseVoltageOffsetB);
    adc->phaseVoltageC = (ADC_GetInjectData(ADC0, ADC_ISEQ_3) - adc->phaseVoltageOffsetC) < 0.0f ? 0.0f : (ADC_GetInjectData(ADC0, ADC_ISEQ_3) - adc->phaseVoltageOffsetC);
#endif
    adc->busVoltageAD = (float)((g_ADCRegularBuffer1[0] + g_ADCRegularBuffer1[2]) >> 1);
    adc->busVoltageTrue = (float)((uint32_t)(adc->busVoltageAD * (VBUS_ATTENUATE_FACTOR * VSVREF * 10)) / ADC_RANGE);        /* 120=12V */
}

/*!
* @brief Bldc control phase current sample, get phase current through bus current.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return none
*/
void Get_PhaseCurrentValue(BLDC_VARS_CTRL *bldcCtrl, BLDC_ADC_TYPE *adc)
{
#if (defined BLDC_SENSORLESS)
    switch(bldcCtrl->pBemf->currentDriverVector)
#elif (defined BLDC_HALL)
    switch(bldcCtrl->pHall->currentDriverVector)
#else
    switch(bldcCtrl->pBemf->currentDriverVector)
#endif
    {
    case AH_PWM_BL_ON:
    case BH_PWM_AL_ON:
        adc->iabPu = adc->busCurAD * CURRENT_PU_COF;
        break;

    case AH_PWM_CL_ON:
    case CH_PWM_AL_ON:
        adc->iacPu = adc->busCurAD * CURRENT_PU_COF;
        break;

    case BH_PWM_CL_ON:
    case CH_PWM_BL_ON:
        adc->ibcPu = adc->busCurAD * CURRENT_PU_COF;
        break;

    default:
        break;
    }
}

/*!
* @brief Get undrive phase vlotage AD value.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return undrive phase vlotage
*/
float BLDC_Get_UndrivePhase_Voltage(BLDC_VARS_CTRL *bldcCtrl, BLDC_ADC_TYPE *adc)
{
    switch (bldcCtrl->pBemf->currentDriverVector)
    {
    case AH_PWM_BL_ON:
    case BH_PWM_AL_ON:
        bldcCtrl->pBemf->undrivePhaseVoltage = adc->phaseVoltageC;
        break;

    case AH_PWM_CL_ON:
    case CH_PWM_AL_ON:
        bldcCtrl->pBemf->undrivePhaseVoltage = adc->phaseVoltageB;
        break;

    case BH_PWM_CL_ON:
    case CH_PWM_BL_ON:
        bldcCtrl->pBemf->undrivePhaseVoltage = adc->phaseVoltageA;
        break;

    default:
        break;
    }

    return bldcCtrl->pBemf->undrivePhaseVoltage;
}

/*!
* @brief Bldc control bus current sample, get bus current feedback.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return none
*/
void Get_CurrentValue(BLDC_VARS_CTRL *bldcCtrl, BLDC_ADC_TYPE *adc)
{
    if (adc->busCurAD > adc->busCurOffset)
    {
        if (adc->busCurAD > (adc->busCurOffset + LIMIT_PEAK_CUR_AD))
        {
            adc->busOverCurCnt++;
        }
        else
        {
            if (adc->busOverCurCnt-- <= 0)
            {
                adc->busOverCurCnt = 0;
            }
        }
        adc->busCurAD = adc->busCurAD - adc->busCurOffset;
    }
    else
    {
        adc->busCurAD = 0.0f;
    }
    Get_PhaseCurrentValue(bldcCtrl, adc);
#if (defined CURRENT_LOOP_CTRL)
    StorageDataCyclic(&adc->busCur, adc->busCurAD);
    adc->busCurAD = Filter_AverageCalc(&adc->busCur, BUFFER_SIZE);
#endif
    adc->motorFeedbackCur = adc->busCurAD * bldcCtrl->pwmOutDuty; /* effective duty */

    adc->busCurTrue =  (float)((int32_t)adc->motorFeedbackCur * 100 / CURRENT_AD_COF);    /* real current in 100 times */
    adc->busPowerTrue = (float)(((uint32_t)adc->busCurTrue * adc->busVoltageTrue) / 100); /* real power in 10 times */

}

/*!
* @brief Load the duty cycle calculated by SIX_STEP_SQUARE_WAVE algorithm into the PWM module
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @return none
*/
void LoadToPwmModular(BLDC_VARS_CTRL *bldcCtrl)
{
    if (bldcCtrl->pwmOutPu > (bldcCtrl->pwmLastOutPu + 0.0003f))
    {
        bldcCtrl->pwmOutPu += 0.0003f;
    }
    else if (bldcCtrl->pwmOutPu < (bldcCtrl->pwmLastOutPu - 0.0003f))
    {
        bldcCtrl->pwmOutPu -= 0.0003f;
    }
    else
    {
    }

    UpDatePwmDuty(PWM3, DUTY_PERIOD(bldcCtrl->pwmOutPu));
    bldcCtrl->pwmLastOutPu = bldcCtrl->pwmOutPu;
}

/*!
* @brief Bldc bus current sample and current loop PI control, executed in the RUN state
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return none
*/
void BusCurrent_Control(BLDC_VARS_CTRL *bldcCtrl, BLDC_ADC_TYPE *adc)
{
    Get_CurrentValue(bldcCtrl, adc);

#if (defined CURRENT_LOOP_CTRL)
    bldcCtrl->ibusFdkPu = adc->motorFeedbackCur * CURRENT_PU_COF;
    BLDC_CurrentLoop_Calculate(&g_bldc_acrPid, bldcCtrl);
#endif
#if (defined BLDC_HALL)
    bldcCtrl->pwmOutPu = MATH_Sat(bldcCtrl->pwmOutPu, MAX_DUTY_WITH_HALL, MIN_DUTY_WITH_HALL);
#else
    bldcCtrl->pwmOutPu = MATH_Sat(bldcCtrl->pwmOutPu, MAX_DUTY_SENSORLESS, MIN_DUTY_SENSORLESS);
#endif

    BLDC_Decrease_Duty_Protect(bldcCtrl, &g_bldc_protector);

    LoadToPwmModular(bldcCtrl);
}

/* =============================================  EOF  ============================================== */
