/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_protector.c
*
* @brief This file provides motor protection functions in bldc control.
*
*/

/* ===========================================  Includes  =========================================== */
#include "bldc_keys.h"
#include "bldc_pwm.h"
#include "bldc_control_api.h"
#include "bldc_parameters_define.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Clear Fault identification.
*
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @param[in] faultid: Fault identification
* @return none
*/
void Fault_Clear(PROTECTOR_TYPE *protect, uint16_t faultid)
{
    protect->faultFlag.all &= ~(1U << faultid);
}

/*!
* @brief Report Fault identification.
*
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @param[in] faultid: Fault identification
* @return none
*/
void Fault_Report(PROTECTOR_TYPE *protect, uint16_t faultid)
{
    protect->faultFlag.all |= (1U << faultid);
}

/*!
* @brief Derease output duty while sensorless stall happened.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @return none
*/
void BLDC_Decrease_Duty_Protect(BLDC_VARS_CTRL *bldcCtrl, PROTECTOR_TYPE *protect)
{
    if (!bldcCtrl->deDutyCoeff)
    {
        if (protect->stallRecoverTime > 0)
        {
            protect->stallRecoverTime--;
            bldcCtrl->pwmOutPu = (MOTOR_STALL_DRAG_DUTY + 0.05f);
            if (0 == protect->stallRecoverTime)
            {
                BLDC_Asr_PIInit(&g_bldc_asrPid, p_bldc_PidCfg);
                BLDC_Acr_PIInit(&g_bldc_acrPid, p_bldc_PidCfg);
            }
        }
    }
    else
    {
        bldcCtrl->pwmOutPu = MOTOR_STALL_DRAG_DUTY;
    }
}

/*!
* @brief DC Bus over current protection. When the DC Bus current exceeds
*        the threshold value, DC Bus over current fault is reported.
*
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return none
*/
void BLDC_Over_Current_Check(PROTECTOR_TYPE *protect, BLDC_ADC_TYPE *adc)
{
    if (adc->busOverCurCnt > BLDC_OVER_CURRENT_DBC)
    {
        Fault_Report(protect, OVER_CURRENT_IBUS_FAULT);
    }
    else
    {

    }
}

/*!
* @brief DC Bus under voltage protection. When the bus voltage is lower than
*        threshold value, Undervoltage fault is reported, and it will return
*        to normal operation after the bus voltage rises to the recovery threshold.
*
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @param[in] volt: DC Bus voltage value, 10 times the real value
* @return none
*/
void BLDC_Over_Voltage_Check(PROTECTOR_TYPE *protect, uint16_t volt)
{
    static int16_t s_vbusStatus = BUS_VOLTAGE_NORMAL;
    static int16_t s_prevVbusStatus = BUS_VOLTAGE_NORMAL;
    static int16_t s_debounce = 0;
//    static int16_t s_status = DETECT_OK;

    if (volt > OVER_VBUS_THRESHOLD)
    {
        s_vbusStatus = BUS_VOLTAGE_OVER;
    }
    else if (volt < OVER_VBUS_RECOVERY_THRESHOLD)
    {
        if (s_vbusStatus == BUS_VOLTAGE_OVER)
        {
            s_vbusStatus = BUS_VOLTAGE_RECOVER;
        }
    }
    else
    {
    }

    if (s_vbusStatus != s_prevVbusStatus)
    {
        s_prevVbusStatus = s_vbusStatus;
        s_debounce = 0;
    }
    else
    {
        if (s_vbusStatus == BUS_VOLTAGE_RECOVER)
        {
            if (s_debounce < 10)
            {
                s_debounce++;
            }
            else
            {
//                g_mcStatus = STOP;
                s_vbusStatus = BUS_VOLTAGE_NORMAL;
                Fault_Clear(protect, VBUS_OVER_VOLTAGE_FAULT);
            }
        }
        else if (s_vbusStatus == BUS_VOLTAGE_NORMAL)
        {
            if (s_debounce < 10)
            {
                s_debounce++;
            }
            else
            {
//                s_status = DETECT_OK;
                Fault_Clear(protect, VBUS_OVER_VOLTAGE_FAULT);
            }
        }
        else
        {
            if (s_debounce < 10)
            {
                s_debounce++;
            }
            else
            {
//                s_status = DIAGNOSTIC_FAIL;
                Fault_Report(protect, VBUS_OVER_VOLTAGE_FAULT);
            }
        }
    }

//    return s_status;
}

/*!
* @brief DC Bus under voltage protection. When the bus voltage is lower than
*        threshold value, Undervoltage fault is reported, and it will return
*        to normal operation after the bus voltage rises to the recovery threshold.
*
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @param[in] volt: DC Bus voltage value, 10 times the real value
* @return none
*/
void BLDC_Under_Voltage_Check(PROTECTOR_TYPE *protect, uint16_t volt)
{
    static int16_t s_vbusStatus = BUS_VOLTAGE_NORMAL;
    static int16_t s_prevVbusStatus = BUS_VOLTAGE_NORMAL;
    static int16_t s_debounce = 0;
//    static int16_t s_status = DETECT_OK;

    if (volt < UNDER_VBUS_THRESHOLD)
    {
        s_vbusStatus = BUS_VOLTAGE_UNDER;
    }
    else if (volt > UNDER_VBUS_RECOVERY_THRESHOLD)
    {
        if (s_vbusStatus == BUS_VOLTAGE_UNDER)
        {
            s_vbusStatus = BUS_VOLTAGE_RECOVER;
        }
    }
    else
    {
    }

    if (s_vbusStatus != s_prevVbusStatus)
    {
        s_prevVbusStatus = s_vbusStatus;
        s_debounce = 0;
    }
    else
    {
        if (s_vbusStatus == BUS_VOLTAGE_RECOVER)
        {
            if (s_debounce < 10)
            {
                s_debounce++;
            }
            else
            {
//                g_mcStatus = STOP;
                s_vbusStatus = BUS_VOLTAGE_NORMAL;
                Fault_Clear(protect, VBUS_UNDER_VOLTAGE_FAULT);
            }
        }
        else if (s_vbusStatus == BUS_VOLTAGE_NORMAL)
        {
            if (s_debounce < 10)
            {
                s_debounce++;
            }
            else
            {
//                s_status = DETECT_OK;
                Fault_Clear(protect, VBUS_UNDER_VOLTAGE_FAULT);
            }
        }
        else
        {
            if (s_debounce < 10)
            {
                s_debounce++;
            }
            else
            {
//                s_status = DIAGNOSTIC_FAIL;
                Fault_Report(protect, VBUS_UNDER_VOLTAGE_FAULT);
            }
        }
    }

//    return s_status;
}

/*!
* @brief Motor Lose phase protection.
*
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return none
*/
void BLDC_LosePhase_Check(PROTECTOR_TYPE *protect, BLDC_VARS_CTRL *bldcCtrl, BLDC_ADC_TYPE *adc)
{
    static uint16_t outputPhaseCnt = 0, ITotalCnt = 0;
    static float IuTotal = 0, IvTotal = 0, IwTotal = 0;

    if ((Get_McStatus() == START) || (Get_McStatus() == RUN))
    {
        protect->IuAvg = (float)(adc->ibcPu);
        protect->IvAvg = (float)(adc->iacPu);
        protect->IwAvg = (float)(adc->iabPu);
        ITotalCnt++;
        if (ITotalCnt <= LOSE_PHASE_ADD_CNT)
        {
            IuTotal += protect->IuAvg;
            IvTotal += protect->IvAvg;
            IwTotal += protect->IwAvg;
        }
        else
        {
            protect->IuAvg = (float)(IuTotal / LOSE_PHASE_ADD_CNT);
            protect->IvAvg = (float)(IvTotal / LOSE_PHASE_ADD_CNT);
            protect->IwAvg = (float)(IwTotal / LOSE_PHASE_ADD_CNT);
            ITotalCnt = 0;

            if (protect->IuAvg > protect->IvAvg)
            {
                protect->Imax = protect->IuAvg;
                protect->Imin = protect->IvAvg;
                if (protect->IvAvg > protect->IwAvg)        /* Iu > Iv > Iw */
                {
                    protect->Imin = protect->IwAvg;
                    protect->Imid = protect->IvAvg;
                    protect->losePhase = 1;
                }
                else if (protect->IwAvg > protect->IuAvg)  /* Iw > Iu > Iv */
                {
                    protect->Imax = protect->IwAvg;
                    protect->Imid = protect->IuAvg;
                    protect->losePhase = 3;
                }
                else    /* Iu >= Iw >= Iv */
                {
                    protect->Imid = protect->IwAvg;
                    protect->losePhase = 1;
                }
            }
            else if (protect->IuAvg < protect->IvAvg)
            {
                protect->Imax = protect->IvAvg;
                protect->Imin = protect->IuAvg;
                if (protect->IuAvg > protect->IwAvg)       /* Iv > Iu > Iw */
                {
                    protect->Imin = protect->IwAvg;
                    protect->Imid = protect->IuAvg;
                    protect->losePhase = 2;
                }
                else if (protect->IwAvg > protect->IvAvg)  /* Iw > Iv > Iu */
                {
                    protect->Imax = protect->IwAvg;
                    protect->Imid = protect->IvAvg;
                    protect->losePhase = 3;
                }
                else    /* Iv >= Iw >= Iu */
                {
                    protect->Imid = protect->IwAvg;
                    protect->losePhase = 2;
                }
            }
            else    /* Iv = Iu */
            {
                if (protect->IwAvg > protect->IuAvg)       /* Iw > Iu = Iv */
                {
                    protect->Imax = protect->IwAvg;
                    protect->Imid = protect->IvAvg;
                    protect->Imin = protect->IuAvg;
                    protect->losePhase = 3;
                }
                else      /* Iu = Iv >= Iw */
                {
                    protect->Imax = protect->IuAvg;
                    protect->Imid = protect->IvAvg;
                    protect->Imin = protect->IwAvg;
                    protect->losePhase = 6;
                }
            }

            if (0 == protect->Imax)
            {
                protect->losePhaseCoeff = (LOSE_PHASE_JUDGE_THRESHOLD + 0.00003f);
            }
            else if (protect->Imax < LOSE_PHASE_THRESHOLD)
            {
                protect->losePhaseCoeff = 1.0f;
            }
            else
            {
                if (0 == protect->Imid)
                {
                    protect->losePhaseCoeff = (LOSE_PHASE_JUDGE_THRESHOLD + 0.00003f);
                }
                else
                {
                    protect->losePhaseCoeff = protect->Imax / protect->Imid;
                }
            }

            if (protect->losePhaseCoeff > LOSE_PHASE_JUDGE_THRESHOLD)
            {
                outputPhaseCnt++;
                if (outputPhaseCnt > LOSE_PHASE_DBC)
                {
                    if (protect->losePhase == 1)
                    {
                        Fault_Report(protect, MOTOR_LOSE_PHASEA_FAULT);
                    }
                    else if (protect->losePhase == 2)
                    {
                        Fault_Report(protect, MOTOR_LOSE_PHASEB_FAULT);
                    }
                    else if (protect->losePhase == 3)
                    {
                        Fault_Report(protect, MOTOR_LOSE_PHASEC_FAULT);
                    }
                    else
                    {
                        Fault_Report(protect, MOTOR_LOSE_PHASEA_FAULT);
                        Fault_Report(protect, MOTOR_LOSE_PHASEB_FAULT);
                        Fault_Report(protect, MOTOR_LOSE_PHASEC_FAULT);
                    }
                    outputPhaseCnt = 0;
                }
            }
            else
            {
                outputPhaseCnt = 0;
            }

            IuTotal = 0;
            IvTotal = 0;
            IwTotal = 0;
        }
    }
    else
    {
        IuTotal = 0;
        IvTotal = 0;
        IwTotal = 0;

        ITotalCnt = 0;
        outputPhaseCnt = 0;
    }
}

/*!
* @brief Motor startup protection in bldc control.
*
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @return none
*/
void BLDC_Startup_Check(PROTECTOR_TYPE *protect, BLDC_VARS_CTRL *bldcCtrl)
{
    if (Get_McStatus() == START)
    {
        if (BLDC_Get_Startup_Status(&g_bldcVarsCtrl) == STARTUP_FAIL)
        {
            Fault_Report(protect, STARTUP_FAILURE_FAULT);
        }
    }
}

/*!
* @brief Motor stall protection in hall sensor control.
*
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @param[in] speed: pointer to SPEED_RAMP_TYPE structure
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] adc: pointer to BLDC_ADC_TYPE structure
* @return none
*/
void BLDC_Hall_MotorStall_Check(PROTECTOR_TYPE *protect, SPEED_RAMP_TYPE *speed, BLDC_VARS_CTRL *bldcCtrl, BLDC_ADC_TYPE *adc)
{
    if (MATH_Abs(speed->speedFbk) < MOTOR_STALL_SPEED_THRESHOLD)
    {
        if (bldcCtrl->ibusFdkPu > MOTOR_STALL_DUTY_THRESHOLD)
        {
            if (++protect->stallTime > MOTOR_STALL_JUDGE_DBC)
            {
                protect->stallFlag = 1;
                g_bldc_asrPid.pPidInput->maxPu = MOTOR_STALL_CURRENT_THRESHOLD;
            }
        }
        if ((protect->stallTime) && (protect->stallFlag))
        {
            if (++protect->stallTime > (MOTOR_STALL_JUDGE_DBC + MOTOR_STALL_RECOVER_DBC))
            {
                bldcCtrl->ibusRefPu = 0.0f;
                bldcCtrl->ibusFdkPu = 0.0f;
                Clear_CurrentSample(adc);
                protect->stallTime = 0;
                protect->stallFlag = 0;
                Fault_Report(protect, MOTOR_STALL_FAULT);
            }
        }
    }
    else
    {
        if (protect->stallTime > 0)
        {
            protect->stallTime--;
        }
        else
        {
            g_bldc_asrPid.pPidInput->maxPu = p_bldc_PidCfg->maxSpeed;
            protect->stallFlag = 0;
        }
    }
}

/*!
* @brief Motor stall protection in sensorless control.
*
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @return none
*/
void BLDC_Sensorless_MotorStall_Check(PROTECTOR_TYPE *protect, SPEED_RAMP_TYPE *command, BLDC_VARS_CTRL *bldcCtrl)
{
    if (protect->stallFlag)
    {
        if (bldcCtrl->pBemf->minChangePhaseTime < 520)  /* 400rpm = 10^7 / (POLE_PAIR_NUM * minChangePhaseTime * TIMER_XUS) */
        {
            protect->stallRecoverTime++;
            if (protect->stallRecoverTime > 100)    /* 100ms */
            {
                protect->stallTime = 0;
                BLDC_Asr_PIInit(&g_bldc_asrPid, p_bldc_PidCfg);
                BLDC_Acr_PIInit(&g_bldc_acrPid, p_bldc_PidCfg);

                bldcCtrl->deDutyCoeff = 0.0f;
                protect->stallRecoverTime = (500 * MS_UINT_INT);
                protect->stallFlag = 0;
            }
        }
    }
    else
    {
        if (((bldcCtrl->pBemf->aveChangePhaseTime > (3.0f * bldcCtrl->pBemf->minChangePhaseTime)) || (bldcCtrl->pBemf->maxChangePhaseTime > (3.0f * bldcCtrl->pBemf->aveChangePhaseTime)))
             || ((MATH_Abs(command->speedFbkRpm) <= 300.0f) && (MATH_Abs(command->speedTargetRamp) >= MIN_SPEED_VALUE)))
        {
            protect->stallTime++;
            if (protect->stallTime > MOTOR_STALL_JUDGE_DBC)
            {
                bldcCtrl->deDutyCoeff = 1.0f;
                protect->stallFlag = 1;
                protect->stallTime = 0;
            }
        }
        else
        {
            if (protect->stallTime > 0)
            {
                protect->stallTime--;
            }
        }
        protect->stallRecoverTime = 0;
    }
}

/*!
* @brief Motor stall protection in hall sensor control.
*
* @param[in] protect: pointer to PROTECTOR_TYPE structure
* @param[in] speed: pointer to SPEED_RAMP_TYPE structure
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @return none
*/
void BLDC_Limit_BusCurrent_Protection(PROTECTOR_TYPE *protect, SPEED_RAMP_TYPE *speed, BLDC_VARS_CTRL *bldcCtrl)
{
    if (bldcCtrl->ibusFdkPu > LIMIT_BUS_CURRENT_HTHRESHOLD)
    {
        if (protect->curLimitCoeff > 0.01f)
        {
            protect->curLimitCoeff = 0.99f * protect->curLimitCoeff;
        }
    }
    else if (bldcCtrl->ibusFdkPu < LIMIT_BUS_CURRENT_LTHRESHOLD)
    {
        if (protect->curLimitCoeff < 0.99f)
        {
            protect->curLimitCoeff = MATH_Min(0.01f, protect->curLimitCoeff);
            protect->curLimitCoeff = 1.01f * protect->curLimitCoeff;
        }
    }
    else
    {
    }

    speed->speedTargetLimit = speed->speedMax * protect->curLimitCoeff;
    speed->speedTargetLimit = MATH_Sat(speed->speedTargetLimit, speed->speedMax, speed->speedMin);
    speed->speedTarget = MATH_Sat(speed->speedTarget, speed->speedTargetLimit, -speed->speedTargetLimit);
}

/*!
* @brief Motor protection main function. The corresponding protection functions
*        are implemented according to different time bases.
*
* @param[in] none
* @return none
*/
void BLDC_Motor_Protection(void)
{

#if (defined BLDC_OVER_CURRENT_CHECK)
    /* DC Bus over current protection , 1ms*/
    BLDC_Over_Current_Check(&g_bldc_protector, &g_bldc_adSample);
#endif

#if (defined BLDC_UNDER_VOLTAGE_CHECK)
    /* DC Bus under voltage protection, 1ms */
    BLDC_Under_Voltage_Check(&g_bldc_protector, g_bldc_adSample.busVoltageTrue);
#endif

#if (defined BLDC_OVER_VOLTAGE_CHECK)
    /* DC Bus over voltage protection, 1ms */
    BLDC_Over_Voltage_Check(&g_bldc_protector, g_bldc_adSample.busVoltageTrue);
#endif

    BLDC_Startup_Check(&g_bldc_protector, &g_bldcVarsCtrl);

#if (defined LOSE_PHASE_PROTECTION_ENABLE)
    /* Lose phase check protection, 1ms */
    BLDC_LosePhase_Check(&g_bldc_protector, &g_bldcVarsCtrl, &g_bldc_adSample);
#endif

#if (defined MOTOR_STALL_PROTECTION_ENABLE)
    #if (defined BLDC_SENSORLESS)
    if (Get_McStatus() == RUN)
    {
        BLDC_Sensorless_MotorStall_Check(&g_bldc_protector, &g_bldc_speedCmd, &g_bldcVarsCtrl);
    }
    #else
    BLDC_Hall_MotorStall_Check(&g_bldc_protector, &g_bldc_speedCmd, &g_bldcVarsCtrl, &g_bldc_adSample);
    #endif
#endif

#if (defined LIMIT_BUS_CURRENT_PROTECTION_ENABLE)
    BLDC_Limit_BusCurrent_Protection(&g_bldc_protector, &g_bldc_speedCmd, &g_bldcVarsCtrl);
#endif

    if (g_bldc_protector.faultFlag.all)
    {
        BLDC_Stop();
        PushIn_McBufStatus(FAULT);
    }

    return;
}

/* =============================================  EOF  ============================================== */
