/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_startup.c
*
* @brief This file provides bldc sensorless startup functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "acmp_hw.h"
#include "bldc_keys.h"
#include "bldc_pwm.h"
#include "bldc_startup.h"
#include "bldc_parameters_define.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Get hall status level while startup.
*
* @param[in] PWMx: PWM Module
* @return hallInitStatus
*/
uint8_t GetHallInitStatus(PWM_Type *PWMx)
{
    uint16_t tempStatus;
    uint8_t hallInitStatus;
    uint8_t tempA, tempB, tempC;

    tempStatus = PWM_DRV_GetAllChannelInputStatus(((uint8_t)(((uint32_t)(PWMx) - PWM0_BASE) >> 11U))) >> 16;
    tempA = tempStatus & 0x1U;
    tempB = (tempStatus & 0x2U) >> 1;
    tempC = (tempStatus & 0x4U) >> 2;
    hallInitStatus = (tempA << 2) + (tempB << 1) + tempC;

    return (hallInitStatus);
}

/*!
* @brief StartUp function in bldc hall control.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @param[in] command: pointer to SPEED_RAMP_TYPE structure
* @return none
*/
void BLDC_Hall_Startup(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg, SPEED_RAMP_TYPE *command)
{
    uint8_t flowState = 0;

    if (Get_McStatus() == START)
    {
        flowState = GetHallInitStatus(PWM0);
        if (bldcCtrl->setDuty <= 0.25f)
        {
            bldcCtrl->setDuty += STARTUP_DUTY_STEP;
        }
        else
        {
            bldcCtrl->setDuty = 0.25f;
            bldcCtrl->startupStatus = STARTUP_FAIL;
        }
        UpDatePwmDuty(PWM3, DUTY_PERIOD(bldcCtrl->setDuty));

        BLDC_Commutate_BaseSector(bldcCfg->motorDir, flowState);

        if ((bldcCfg->motorDir * command->speedFbkRpm) >= 60.0f)   /* Mechanicla frequeccy >1hz,startup successful */
        {
            bldcCtrl->startupStatus = STARTUP_READY;
        }

    }
    else
    {

    }
}

/*!
* @brief Pre-position function in bldc sensorless control.
*
* @param[in] PWMx: PWM Module
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @return none
*/
void BLDC_Preposition(PWM_Type *PWMx, BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg)
 {
    uint16_t delay = INIT_PREPOSITION_DELAY ;

    for (bldcCtrl->setDuty = MIN_DUTY_PREPOSITION; bldcCtrl->setDuty < MAX_DUTY_PREPOSITION; bldcCtrl->setDuty += STEP_DUTY_PREPOSITION)
    {
        delay -= INIT_PREPOSITION_DELAY_STEP;
        if (delay <= MIN_PREPOSITION_DELAY)
        {
            delay = MIN_PREPOSITION_DELAY;
        }

        UpDatePwmDuty(PWMx, DUTY_PERIOD(bldcCtrl->setDuty));

        bldcCtrl->pBemf->currentDriverVector = (uint8_t)AH_PWM_BL_ON;
        MosDriver(PWMx, bldcCfg, bldcCtrl->pBemf->currentDriverVector);
        mdelay(delay);
    }
    bldcCtrl->pBemf->kickTimeThreshold = BLDC_SENSORLESS_INIT_DELAY_TIME;
 }

/*!
* @brief bldc openkick function in bldc sensorless control, it is usually executed after obtaining the initial rotor position.
*
* @param[in] PWMx: PWM Module
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @return none
*/
void BLDC_Bemf_OpenKick(PWM_Type *PWMx, BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg)
{
    if (bldcCfg->motorDir != FORWARD_ROTATE)
    {
        bldcCtrl->pBemf->currentDriverVector--;
        if (bldcCtrl->pBemf->currentDriverVector <= (uint8_t)INVALID_VECTOR_MIN)
        {
            bldcCtrl->pBemf->currentDriverVector = (uint8_t)CH_PWM_BL_ON;
        }
        MosDriver(PWMx, bldcCfg, bldcCtrl->pBemf->currentDriverVector);
    }
    else
    {
        bldcCtrl->pBemf->currentDriverVector++;
        if (bldcCtrl->pBemf->currentDriverVector >= (uint8_t)INVALID_VECTOR_MAX)
        {
            bldcCtrl->pBemf->currentDriverVector = (uint8_t)AH_PWM_BL_ON;
        }
        MosDriver(PWMx, bldcCfg, bldcCtrl->pBemf->currentDriverVector);

    }

    if (bldcCtrl->setDuty <= MAX_DUTY_SENSORLESS_START)
    {
        bldcCtrl->setDuty += STARTUP_DUTY_STEP;
    }
    else
    {
        bldcCtrl->setDuty = MAX_DUTY_SENSORLESS_START;
    }
    UpDatePwmDuty(PWMx, DUTY_PERIOD(bldcCtrl->setDuty));
}

/*!
* @brief switch acmp compared channel according to current magnetic vector, it is usually executed in bldc sensorless control.
*
* @param[in] curDriverValue: current magnetic vector in bldc sensorless control
* @return none
*/
void SwitchAcmpChannel(uint8_t curDriverValue)
{
    switch (curDriverValue)
    {
    case AH_PWM_BL_ON:
    ACMP_SetPositiveInputPin(ACMP0, ACMP_EXTERNAL_CH4);
    break;

    case AH_PWM_CL_ON:
    ACMP_SetPositiveInputPin(ACMP0, ACMP_EXTERNAL_CH1);
    break;

    case BH_PWM_CL_ON:
    ACMP_SetPositiveInputPin(ACMP0, ACMP_EXTERNAL_CH0);
    break;

    case BH_PWM_AL_ON:
    ACMP_SetPositiveInputPin(ACMP0, ACMP_EXTERNAL_CH4);
    break;

    case CH_PWM_AL_ON:
    ACMP_SetPositiveInputPin(ACMP0, ACMP_EXTERNAL_CH1);
    break;

    case CH_PWM_BL_ON:
    ACMP_SetPositiveInputPin(ACMP0, ACMP_EXTERNAL_CH0);
    break;

    default:
    break;
    }
}

/*!
* @brief drag motor dircet to RUN state, get into close loop control fast.
*
* @param[in] PWMx: PWM Module
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @return none
*/
void MotorDirectDrag(PWM_Type *PWMx, BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg)
{
    UpDatePwmDuty(PWMx, DUTY_PERIOD(DRAG_DUTY));
    MosDriver(PWMx, bldcCfg, bldcCtrl->pBemf->currentDriverVector);
#if (defined BEMF_DETECT_HARDWARE)
    SwitchAcmpChannel(bldcCtrl->pBemf->currentDriverVector);
#endif
    bldcCtrl->pBemf->changePhaseTime = 0;
    bldcCtrl->pBemf->saveChangePhaseTime = FIRST_COMMUTATE_TIME;
    bldcCtrl->pBemf->afterFlowTime = FIRST_STEP_DRAG_TIME;
    bldcCtrl->pBemf->forceChangePhaseFlag = 0;

    bldcCtrl->startupStatus = STARTUP_READY;
}

/*!
* @brief drag motor to RUN state through open kick.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @return none
*/
void MotorKickDrag(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg)
{
    if (bldcCtrl->pBemf->kickTimeThreshold > BLDC_SENSORLESS_MIN_DELAY_TIME)
    {
        bldcCtrl->bldcStartTime++;
        if (bldcCtrl->bldcStartTime > bldcCtrl->pBemf->kickTimeThreshold)
        {
            bldcCtrl->bldcStartTime = 0;
            if (bldcCtrl->pBemf->kickTimeThreshold > BLDC_SENSORLESS_MIN_DELAY_TIME)
            {
                bldcCtrl->pBemf->kickTimeThreshold -= BLDC_SENSORLESS_STEP_DELAY_TIME;
            }
            else
            {
                bldcCtrl->pBemf->kickTimeThreshold = BLDC_SENSORLESS_MIN_DELAY_TIME;
            }
            BLDC_Bemf_OpenKick(PWM3, bldcCtrl, bldcCfg);
        }
    }
    else
    {
        if ((bldcCtrl->pBemf->currentDriverVector + bldcCfg->motorDir) >= (uint8_t)INVALID_VECTOR_MAX)
        {
            bldcCtrl->pBemf->currentDriverVector = (uint8_t)AH_PWM_BL_ON;
        }
        else if ((bldcCtrl->pBemf->currentDriverVector + bldcCfg->motorDir) <= (uint8_t)INVALID_VECTOR_MIN)
        {
            bldcCtrl->pBemf->currentDriverVector = (uint8_t)CH_PWM_BL_ON;
        }
        else
        {
            bldcCtrl->pBemf->currentDriverVector += bldcCfg->motorDir;
        }

        MosDriver(PWM3, bldcCfg, bldcCtrl->pBemf->currentDriverVector);
#if (defined BEMF_DETECT_HARDWARE)
        SwitchAcmpChannel(bldcCtrl->pBemf->currentDriverVector);
#endif
        bldcCtrl->pBemf->changePhaseTime = 0;
        bldcCtrl->pBemf->saveChangePhaseTime = FIRST_COMMUTATE_TIME;
        bldcCtrl->pBemf->afterFlowTime = FIRST_STEP_DRAG_TIME;
        bldcCtrl->pBemf->forceChangePhaseFlag = 0;

        bldcCtrl->startupStatus = STARTUP_READY;
    }
}

/*!
* @brief BLDC motor startup function, include bldc hall and bldc sensorless control.
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @param[in] bldcCfg: pointer to BLDC_VARS_CFG structure
* @return none
*/
void BLDC_Startup(BLDC_VARS_CTRL *bldcCtrl, BLDC_VARS_CFG *bldcCfg)
{
#if (defined BLDC_HALL)
    if (bldcCtrl->bldcStartTime == 0)
    {
        BLDC_Hall_Startup(bldcCtrl, bldcCfg, &g_bldc_speedCmd);
    }
    if (bldcCtrl->bldcStartTime++ > 20)
    {
        bldcCtrl->bldcStartTime = 0;
    }
#elif (defined BLDC_SENSORLESS)
    if (bldcCtrl->pBemf->prepositonIndex < 3)   /* preposition */
    {
        bldcCtrl->pBemf->prepositonIndex++;
        BLDC_Preposition(PWM3, bldcCtrl, bldcCfg);
    }
    else        /* drag startup */
    {
        #if (defined NOKICK_STARTUP)
        MotorDirectDrag(PWM3, bldcCtrl, bldcCfg);
        #else
        MotorKickDrag(bldcCtrl, bldcCfg);
        #endif
    }
#else
#endif

}

/* =============================================  EOF  ============================================== */
