/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __ADC_DRV_H__
#define __ADC_DRV_H__
/*!
* @file adcdrv.h
*
* @brief This file provides ADC sampling functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "param_init.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Adc sampling results structure.
*/
typedef struct
{
    int16_t iaOffset;               /*!< Offset of phase A current Adc sampling */
    int16_t ibOffset;               /*!< Offset of phase B current Adc sampling */
    int16_t icOffset;               /*!< Offset of phase C current Adc sampling */
    int16_t ibusOffset;             /*!< Offset of DC Bus current Adc sampling */
    int16_t ia;                     /*!< Phase A ADC sampled value */
    int16_t ib;                     /*!< Phase B ADC sampled value */
    int16_t ic;                     /*!< Phase C ADC sampled value */
    int16_t ibus1;                  /*!< First sampling trigger value for ibus */
    int16_t ibus2;                  /*!< Second sampling trigger value for ibus */
    uint16_t vbus;                  /*!< Voltage ADC sample value */
    uint16_t TempIPM;               /*!< Reserve, for Current IPM sensor */
    int16_t iaOffset1;              /*!< Offset of phase A current Adc sampling */
    int16_t ibOffset1;              /*!< Offset of phase B current Adc sampling */
    int16_t icOffset1;              /*!< Offset of phase C current Adc sampling */
    int16_t ibusOffset1;            /*!< Offset of DC Bus current Adc sampling */
} ADC_SAMPLE;

/*!
* @brief Adc sampling results transform to pu vaule structure.
*/
typedef struct
{
    float iaPu;                     /*!< phase A current value */
    float ibPu;                     /*!< phase B current value */
    float icPu;                     /*!< phase C current value */
    float vdcPu;                    /*!< voltage value */
    float vdcPuBeforeLPF;           /*!< voltage value before LPF */
    float vdcCirlePu;               /*!< voltage linmit circle value */
    float vdcVt;                    /*!< DC bus real value */
    float ibusPuAvg;                /*!< bus current average value */
    float ibusRealAvg;              /*!< bus current average value */
} ADC_TRANSFORM;

/*!
* @brief Initial speed judge function data struct, befor start motor, judge motor initial speed
*/
typedef struct
{
    int16_t emfU;                   /*!< U phase back emf */
    int16_t emfV;                   /*!< V phase back emf */
    int16_t emfW;                   /*!< W phase back emf */
    int16_t emfUV;                  /*!< UV line back emf */
    int16_t emfUW;                  /*!< UW line back emf */
} BEMF_SAMPLE;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Clear A-B-C Phase Sample Current Result.
*
* @param[in] phaseCurrent: pointer to ADC_TRANSFORM structure
* @return none
*/
void Clear_CurrentSample(ADC_TRANSFORM *phaseCurrent);

/*!
* @brief Get back emf sample AD value.
*
* @param[in] none
* @return bemf: pointer to BEMF_SAMPLE structure
*/
BEMF_SAMPLE Get_BemfSample(void);

/*!
* @brief Get A-B-C phase sample current offset.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @return none
*/
void GetCurrentOffset(ADC_SAMPLE *adcSampleReal);

/*!
* @brief Get A-B-C Phase Sample Current Result.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @param[in] adcPuCof: pointer to ADC_SAMPLE_COF structure
* @param[in] phaseCurrent: pointer to ADC_TRANSFORM structure
* @return none
*/
void Get_CurrentSample(ADC_SAMPLE *adcSampleReal, ADC_SAMPLE_COF *adcPuCof, ADC_TRANSFORM *phaseCurrent);

/*!
* @brief Get DC Bus voltage sampling Result.
*
* @param[in] adcPuCof: pointer to ADC_SAMPLE_COF structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] vbus: pointer to ADC_TRANSFORM structure
* @return none
*/
void Get_VoltageSample(ADC_SAMPLE_COF *adcPuCof, MOTOR_PARAM_PU *paramPu, ADC_TRANSFORM *vbus);

/*!
* @brief Get Three shunts sample mode data.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @return none
*/
void Get_ThreeShuntsCur(ADC_SAMPLE *adcSampleReal);

/*!
* @brief Changing the sampling channel of ADC is used to save ADC channel and
*        improve sampling speed in 3-shunt sampling mode.
*
* @param[in] none
* @return none
*/
void ChangeAdSampChannel(void);

/*!
* @brief Adc sampling result pu value transmit to foc variable.
*
* @param[in] adcResult: pointer to ADC_TRANSFORM structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void AdcResultTransmit(ADC_TRANSFORM *adcResult, FOC_VARS_CTRL *focVars);

/* ======================================  Functions define  ======================================== */

#endif /* __ADC_DRV_H__ */

/* =============================================  EOF  ============================================== */
