/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __MOTORCTRL_API_H__
#define __MOTORCTRL_API_H__
/*!
* @file motorctrl_api.h
*
* @brief This file provides user driven motor control functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "speed_angle.h"

/* ============================================  Define  ============================================ */
/*!
* @brief define SVPWM Output ON and OFF Bit.
*/
#define SVPWM_OUTPUT_ENABLE     (1)
#define SVPWM_OUTPUT_DISABLE    (0)

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Define general operation execute status
*/
typedef enum {
    INIT_UNKNOWN = 0,                   /*!< Status 0: init unknown status */
    EXE_SUCCESS,                        /*!< Status 1: operation execute success status */
    EXE_FAIL,                           /*!< Status 2: operation execute fail status */
} OPERATION_STATUS;

/*!
* @brief User command parameters structure.
*/
typedef struct
{
    int16_t targetTorque;               /*!< Target value of torque command in per-unit (pu)*/
    int16_t targetSpeed;                /*!< Target value of speed command in per-unit (pu) */
    int16_t targetAngle;                /*!< Target value of angle command in per-unit (pu) */
} USER_COMMAND;

/*!
* @brief Motor control command parameters structure.
*/
typedef struct
{
    int16_t idPu;                       /*!< D-axis target current in per-unit (pu) */
    int16_t iqPu;                       /*!< Q-axis target current in per-unit (pu) */
    int16_t vdPu;                       /*!< D-axis target voltage in per-unit (pu) */
    int16_t vqPu;                       /*!< Q-axis target voltage in per-unit (pu) */
    int16_t speedPu;                    /*!< speed in per-unit  (pu) */
} MC_COMMAND;

/* ==========================================  Variables  =========================================== */
extern USER_COMMAND     g_userCommand;

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Startup the motor and enable the PWM module to output pulse.
*
* @param[in] none
* @return none
*/
void StartUp_Motor(void);

/*!
* @brief Stop the motor and disable the PWM module to output pulse.
*
* @param[in] none
* @return none
*/
void Stop_Motor(void);

/*!
* @brief Set d-axis current reference value.
*
* @param[in] command: pointer to FOC_VARS_CTRL structure
* @param[in] idRef: Given d-axis current reference value
* @return none
*/
void Set_Idref(FOC_VARS_CTRL *command, float idRef);

/*!
* @brief Set q-axis current reference value.
*
* @param[in] command: pointer to FOC_VARS_CTRL structure
* @param[in] iqRef: Given q-axis current reference value
* @return none
*/
void Set_Iqref(FOC_VARS_CTRL *command, float iqRef);

/*!
* @brief Clear d-axis current reference value.
*
* @param[in] command: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Clear_Idref(FOC_VARS_CTRL *command);

/*!
* @brief Clear q-axis current reference value.
*
* @param[in] command: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Clear_Iqref(FOC_VARS_CTRL *command);

/*!
* @brief Set d-axis current feedback value.
*
* @param[in] command: pointer to FOC_VARS_CTRL structure
* @param[in] idFbk: Given d-axis current feedback value
* @return none
*/
void Set_IdFbk(FOC_VARS_CTRL *command, float idFbk);

/*!
* @brief Set q-axis current feedback value.
*
* @param[in] command: pointer to FOC_VARS_CTRL structure
* @param[in] iqFbk: Given q-axis current feedback value
* @return none
*/
void Set_IqFbk(FOC_VARS_CTRL *command, float iqFbk);

/*!
* @brief Set input reference value of PID regulator.
*
* @param[in] PID: pointer to PID_REGULATOR structure
* @param[in] ref: Given reference value of PID reguletor input
* @return none
*/
void Set_PID_Ref(PID_REGULATOR *PID, float ref);

/*!
* @brief Set input feedback value of PID regulator.
*
* @param[in] PID: pointer to PID_REGULATOR structure
* @param[in] fbk: Given feedback value of PID reguletor input
* @return none
*/
void Set_PID_Fbk(PID_REGULATOR *PID, float fbk);

/* ======================================  Functions define  ======================================== */

#endif /* __MOTORCTRL_API_H__ */

/* =============================================  EOF  ============================================== */
