/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __PARAM_ID_API_H__
#define __PARAM_ID_API_H__
/*!
* @file param_identify.h
*
* @brief This file contain four motor parameters identification, include Rs Ld/Lq Flux Inertia interface.
*
*/

/* ===========================================  Includes  =========================================== */
#include "param_identify.h"
#include "param_init.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Parameter identification results storage buffer structure
*/
typedef struct
{
    uint16_t rsId;              /*!< Stator resistance identification result */
    uint16_t rsIdPhaseA;        /*!< Stator resistance identification result of phase A */
    uint16_t rsIdPhaseB;        /*!< Stator resistance identification result of phase B */
    uint16_t rsIdPhaseC;        /*!< Stator resistance identification result of phase C */
    uint16_t ld;                /*!< D-axis inductance identification result */
    uint16_t lq;                /*!< Q-axis inductance identification result */
    uint32_t inertia;           /*!< Motor inertia identification result */
    float bEmf;                 /*!< Motor back-EMF constant value identification result */
} PARAM_IDENTIFY_BUFFER;

/*!
* @brief Hall sensor self learn results storage buffer structure
*/
typedef struct
{
    uint8_t selfLearnOrder;     /*!< Hall sensor order learn result */
    uint16_t selfLearnTheta;    /*!< Hall sensor angle learn result */
    uint16_t sectorWidthMax;    /*!< Hall sensor sector width result max value */
    uint16_t sectorWidthMin;    /*!< Hall sensor sector width result min value */
} HALL_SELFLEARN_BUFFER;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Ld Lq Parameter identify function initialize configuration
*
* @param[in] none
* @return none
*/
void Ldq_IdentifyConfig(void);

/*!
* @brief Ld Lq Parameter identify update pwm period value.
*
* @param[in] period: Updata pwm period value
* @return none
*/
void Ldq_IdentifyUpdataPrd(uint16_t period);

/*!
* @brief Ld Lq Parameter identify finished, and call this function to reset configuration.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void Ldq_IdentifyConfigReset(FOC_VARS *focVars);

/*!
* @brief Ld Lq Parameter identify set different PWM output. Cotain six direction pulse output, and recover PWM configuration.
*        This function is called by Ld Lq identify isr function.
*
* @param[in] section: Select different PWM output mode
* @return none
*/
void ParamIdentify_LdqSetPwm(uint8_t section);

/*!
* @brief Get parameter identification duty.
*
* @param[in] none
* @return Schedule duty of parameter identify
*/
//uint16_t ParamIdentify_DutyCheck(void);
uint16_t ParamIdentify_DutyCheck(PARAM_IDENTIFY_STRUCT *paramIdent);

/*!
* @brief Get parameter identification results to save.
*
* @param[in] paramIdentify: pointer to PARAM_IDENTIFY_STRUCT structure
* @param[in] paramBuffer: pointer to PARAM_IDENTIFY_BUFFER structure
* @return none
*/
void ParameterIdentifySaveProcess(PARAM_IDENTIFY_STRUCT *paramIdentify, PARAM_IDENTIFY_BUFFER *paramBuffer);

/*!
* @brief Get hall self-learning duty.
*
* @param[in] hallSensor: pointer to HALL_SENSOR structure
* @return Schedule duty of hall self-learning
*/
uint16_t HallSelfLearnDutyCheck(HALL_SENSOR *hallSensor);

/*!
* @brief Get hall self learn results to save.
*
* @param[in] hallBuffer: pointer to HALL_SELFLEARN_BUFFER structure
* @param[in] hallSensor: pointer to HALL_SENSOR structure
* @return none
*/
void HallSelfLearnSaveProcess(HALL_SELFLEARN_BUFFER *hallBuffer, HALL_SENSOR *hallSensor);

/*!
* @brief Hall sensor self learning results buffer initilization.
*
* @param[in] hallBuffer: pointer to HALL_SELFLEARN_BUFFER structure
* @return none
*/
void HallSelfLearnResultInit(HALL_SELFLEARN_BUFFER *hallBuffer);

/*!
* @brief Ld Lq Parameter identify update pwm period value in Method High frequency sine wave.
*
* @param[in] pwmFreq: Updata pwm frequency value
* @param[in] period: Updata pwm period value
* @param[in] STC: pointer to STC_PARAM structure
* @return none
*/
void ParamId_UpdatePwmPrd(uint16_t pwmFreq, uint16_t period, STC_PARAM *STC);

#endif /* __PARAM_ID_API_H__ */

/* =============================================  EOF  ============================================== */
