/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __SPEED_ANGLE_H__
#define __SPEED_ANGLE_H__
/*!
* @file speed_angle.h
*
* @brief This file provides speed command ramp and speed loop processing functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "autospd_regulator.h"
#include "encoder.h"
#include "flux_observer.h"

/* ============================================  Define  ============================================ */
#define MAX_HALL_CHECK_BUF      6       /*!< Max depth of back emf decection buffer */

/*!< Give command slope calculate */
#define GIVE_SLOPE(ref, ramp, step) \
               if ((ref - ramp) > step)\
               {ramp += step ;}\
               else if ((ref - ramp) < -(step))\
               {ramp -= (step) ;}\
               else\
               {ramp = ref;}

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Define the motor speed initial status enumeration.
*/
typedef enum
{
    UNKNOWN,        /*!< Unknown, motor speed initial status is unrecognized */
    HIGH_REVERSE,   /*!< High reverse, motor speed initial status is high speed headwind */
    LOW_REVERSE,    /*!< Low reverse, motor speed initial status is low speed headwind */
    STATIC,         /*!< Static, motor speed initial status is stop */
    LOW_FORWARD,    /*!< Low forward, motor speed initial status is Low speed tailwind */
    HIGH_FORWARD    /*!< High forward, motor speed initial status is high speed tailwind */
} SPEED_INIT_STATUS;

/*!
* @brief Define S-Curve speed ramp motion mode enumeration.
*/
typedef enum
{
    SCURVE_MOTION_UNIF = 0,     /*!< Uniform motion mode, motor speed is constant value */
    SCURVE_MOTION_ACC,          /*!< Accelerated motion mode, motor speed increases */
    SCURVE_MOTION_DEC,          /*!< Decelerated motion mode, motor speed decreases */
} SCURVE_MOTION_MODE;

/*!
* @brief Define S-Curve speed ramp curve type enumeration.
*/
typedef enum
{
    SCURVE_TYPE_1 = 1,          /*!< S-curve shape type 1, the absolute acceleration of S-curve first increases, then stabilizes, and finally decrease to 0 */
    SCURVE_TYPE_2,              /*!< S-curve shape type 2, the absolute acceleration of S-curve first increases, then stabilizes */
    SCURVE_TYPE_3,              /*!< S-curve shape type 3, the absolute acceleration of S-curve first stabilizes, then decrease to 0 */
    SCURVE_TYPE_4,              /*!< S-curve shape type 4, the absolute acceleration of S-curve curve remains constant */
} SCURVE_CURVE_TYPE;

/*!
* @brief Define position control direction mode enumeration.
*/
typedef enum
{
    ANGLE_HOLD = 0,           /*!< Target position hold on, maintain current position */
    ANGLE_ACC,                /*!< Target position ramp increase */
    ANGLE_DEC,                /*!< Target position ramp decrease */
} ANGLE_DIRECTION_TYPE;

/*!
* @brief Define position control speed gear enumeration.
*/
typedef enum
{
    SPEED_GEAR0 = 0,          /*!< Position control action gear0, do the positon control action in 2s */
    SPEED_GEAR1,              /*!< Position control action gear1, do the positon control action in 1s */
    SPEED_GEAR2,              /*!< Position control action gear2, do the positon control action in 0.5s */
} ANGLE_ACTION_GEAR_TYPE;

/*!
* @brief Torque command parameters structure.
*/
typedef struct
{
    uint8_t torqueFlag;             /*!< Torque control mode flag */
    uint8_t speedFlag;              /*!< Speed control mode flag */
    float iqTarget;               /*!< Q axis current target, command source in per-unit (pu) */
    float iqTargetRamp;           /*!< Q axis current after ramp, assign to speed reference in per-unit (pu) */
    float iqChangeStep;           /*!< Q axis current change step */
    float iqMax;                  /*!< Max limit of q axis current command */
    float iqMin;                  /*!< Min limit of q axis current command */
    float speedLimit;             /*!< Torque control mode speed limit value */
} TORQUE_COMMAND;

/*!
* @brief Speed command ramp and speed feedback parameters structure.
*/
typedef struct
{
    float speedTarget;            /*!< Speed target, command source in per-unit (pu) */
    float speedTargetRamp;        /*!< speed command after ramp, assign to speed reference in per-unit (pu) */
    float speedFbk;               /*!< Feedback speed in per-unit (pu) */
    float speedTargetLimit;       /*!< Speed target command limit value in per-unit (pu) */
    float speedFbkRpm;            /*!< Feedback mechanical speed Unit:rpm */
    float speedMax;               /*!< Max limit of speed command */
    float speedMin;               /*!< Min limit of speed command */

    float speedBuffer[64];        /*!< Speed feedback FIFO, it contains latest estimated speed value */
    float avgSpeedFbk;            /*!< Average feedback mechanical speed from FIFO in per-unit (pu) */
    float avgSpeedFbkRpm;         /*!< Aeedback average mechanical speed from FIFO Unit:rpm */

    float initSpeedJudgeSpeed;    /*!< Speed value in initial speed judgment stage */

    float speedAcceleration;      /*!< Motor speed regulation acceleration rate */
    float speedDeceleration;      /*!< Motor speed regulation deceleration rate */

    float speedFactor;           /*!< Coefficient: to convert speed from real value to pu value */

    float varianceSpeed;          /*!< Feedback mechanical speed variance from FIFO */
    float initSpeedJudgeBemfAmp;  /*!< Back emf amplitude in initial speed judgment stage */

    uint16_t speedBufferReady;      /*!< Estimated speed FIFO ready flag */
    uint16_t speedBufferIndex;      /*!< Position of latest speed estimation in estimated speed FIFO */
    uint16_t speedBufferSize;       /*!< Depth of FIFO used to export average estimated speed value, it must be an integer number between 1 and 64 */

    SPEED_INIT_STATUS initStatus;   /*!< Status after initial speed judgment stage */
    ESTIMATED_SPEED *speedCalc;     /*!< Estimated speed structure */
} SPEED_RAMP_HANDLE;

/*!
* @brief Position command ramp and position feedback parameters structure.
*/
typedef struct
{
    uint8_t actionDirection;        /*!< Position target varying direction, 0: hold on, 1: target increase, 2: target decrease */
    uint8_t actionGear;             /*!< Position action gear, 0: 2s, 1: 1s, 2: 0.5s */
    int16_t angleTimeCnt;           /*!< Position target curve set time,  Unit:10ms */

    float angleDelta;             /*!< Position target change once in per-unit (pu) */
    float angleTarget;            /*!< Position target, angle command source in per-unit (pu) */
    float angleTargetBuf;         /*!< Position target last cycle, angle command source in per-unit (pu) */

    float angleTargetRamp;        /*!< Angle command after ramp arrange, assign to position reference in per-unit (pu) */
    float angleFbk;               /*!< Feedback angle in per-unit (pu) */
//    float angleTargetLimit;       /*!< Angle target command limit value in per-unit (pu) */
    float angleFbkDegree;         /*!< Feedback mechanical Angle, Unit:degree */
    float angleMax;               /*!< Max limit of angle command */
    float angleMin;               /*!< Min limit of angle command */

    float angleAdjust;            /*!< Position target need to adjust in per-unit (pu) */
    float angleTargetPulse;       /*!< Position target, angle command source in encoder pulse count */
    float angleFbkPulse;          /*!< Feedback angle in encoder pulse count */
    float angleFactor;           /*!< Coefficient: to convert angle from real value to pu value */

} POSITION_RAMP_HANDLE;

/*!
* @brief Speed loop variable PI coefficient parameters structure.
*/
typedef struct
{
    uint8_t asrVarPIEnable;         /*!< Speedloop variable PI Enable */
    float frepuH;                 /*!< Speedloop variable PI frequence value at high frequency point */
    float frepuL;                 /*!< Speedloop variable PI frequence value at low frequency point */
    float kpH;                    /*!< Speedloop variable PI kp value at high frequency point */
    float kiH;                    /*!< Speedloop variable PI ki value at high frequency point */
    float kpL;                    /*!< Speedloop variable PI kp value at low frequency point */
    float kiL;                    /*!< Speedloop variable PI ki value at low frequency point */
    float kpGain;                 /*!< Speedloop variable PI kp gain */
    float kiGain;                 /*!< Speedloop variable PI ki gain */
} ASR_SPEEDVARPI;

/*!
* @brief S-curve acceleration and deceleration parameters structure
*/
typedef struct
{
    SCURVE_MOTION_MODE motionType;  /*!< Motion mode: 1, ACC; 2, DEC; 0, UNIF */
    SCURVE_CURVE_TYPE accCurveType;      /*!< Acceleration curve type: 1, m!=0,n!=0; 2, m!=0,n=0; 3, m=0,n!=0; 4, m=n=0 */
    SCURVE_CURVE_TYPE decCurveType;      /*!< Deceleration curve type: 1, m!=0,n!=0; 2, m!=0,n=0; 3, m=0,n!=0; 4, m=n=0 */
    uint8_t curveEndFlag;      /*!< Curve calculation end flag, 1: end of curve calculation; 0: in curve calcualtion */
    uint8_t curveCalcFlag;     /*!< Curve calculation flag, 1: S curve is in calculation progress; 0: S curve is not in calculation progress */
    uint32_t timeCnt;           /*!< Time count value as time variable */
    uint32_t accDuration;       /*!< The time required for acceleration is calculated according to the value of starting speed and target speed */
    uint32_t decDuration;       /*!< The time required for deceleration is calculated according to the value of starting speed and target speed */
    uint32_t accTimePoint1;     /*!< End time of the first segment of acceleration curve */
    uint32_t accTimePoint2;     /*!< End time of the second segment of acceleration curve */
    uint32_t accTimePoint3;     /*!< End time of the third segment of acceleration curve */
    uint32_t decTimePoint1;     /*!< End time of the first segment of deceleration curve */
    uint32_t decTimePoint2;     /*!< End time of the second segment of deceleration curve */
    uint32_t decTimePoint3;     /*!< End time of the third segment of deceleration curve */

    float accStartRatio;     /*!< Proportion coefficient of acceleration starting segment m value, Math_IQ(0.0)~(1-n), max value is Math_IQ(0.99) */
    float accEndRatio;       /*!< Proportion coefficient of acceleration end segment n value, Math_IQ(0.0)~(1-m), max value is Math_IQ(0.99) */
    float decStartRatio;     /*!< Proportion coefficient of deceleration starting segment m value, Math_IQ(0.0)~(1-n), max value is Math_IQ(0.99) */
    float decEndRatio;       /*!< Proportion coefficient of deceleration end segment n value, Math_IQ(0.0)~(1-m), max value is Math_IQ(0.99) */

    float freqStart;          /*!< Starting point of curve */
    float freqEnd;            /*!< End point of curve */
    float freqDelta;          /*!< Difference value of starting and end points of curve, = |freqEnd - freqStart| */
    float target;             /*!< Target */
    float targetBuf;          /*!< Target backup value */
    float freqOutput;         /*!< S curve output */

    float freqCalc;             /*!< Calculated value of curve */
    float accStartJerk;         /*!< Jerk value of the start segment of acceleration curve */
    float accEndJerk;           /*!< Jerk value of the end segment of acceleration curve */
    float decStartJerk;         /*!< Jerk value of the start segment of deceleration curve */
    float decEndJerk;           /*!< Jerk value of the end segment of deceleration curve */

    float accA1;                /*!< Quadratic coefficient of acceleration start segment */
    float accB1;                /*!< Coefficient of first order term of acceleration start segment */
    float accA2;                /*!< Coefficient of first order term of acceleration linear segment */
    float accA3;                /*!< Quadratic coefficient of acceleration end segment */
    float accB3;                /*!< Coefficient of first order term of acceleration end segment */

    float decA1;                /*!< Quadratic coefficient of deceleration start segment */
    float decB1;                /*!< Coefficient of first order term of deceleration start segment */
    float decA2;                /*!< Coefficient of first order term of deceleration linear segment */
    float decA3;                /*!< Quadratic coefficient of deceleration end segment */
    float decB3;                /*!< Coefficient of first order term of deceleration end segment */

    float delta;                /*!< Integral calculation accumulated value */
} SCURVE_DATA;

/*!
* @brief S-curve acceleration and deceleration parameters structure
*/
typedef struct
{
    float accStartRatio;     /*!< Proportion coefficient of acceleration starting segment m value, Math_IQ(0.0)~(1-n), max value is Math_IQ(0.99) */
    float accEndRatio;       /*!< Proportion coefficient of acceleration end segment n value, Math_IQ(0.0)~(1-m), max value is Math_IQ(0.99) */
    float decStartRatio;     /*!< Proportion coefficient of deceleration starting segment m value, Math_IQ(0.0)~(1-n), max value is Math_IQ(0.99) */
    float decEndRatio;       /*!< Proportion coefficient of deceleration end segment n value, Math_IQ(0.0)~(1-m), max value is Math_IQ(0.99) */
} SCURVE_CFG;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Tail wind and head wind initial speed judge initialize.
*
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @return none
*/
void InitSpeed_Judge_Init(INITSPEED_JUDGE *judgeHandle);

/*!
* @brief The function is called in SPEED_JUDGE state to calculate the initial speed data, enter the FLY_RUN state after completion.
*
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void Speed_Judge_Process(INITSPEED_JUDGE *judgeHandle, FOC_VARS *focVars);

/*!
* @brief Tail wind and head wind processes.
         When the motor is in the HIGH_REVERSE state, it will enter the STOP state;
         when the motor is in the LOW_REVERSE state or LOW_FORWARD state, it will enter the START_BRAKE state;
         when the motor is in the STATIC state, it will enter the START state;
         when the motor is in the HIGH_FORWARD state, it will enter the RUN state;
*
* @param[in] speedHandle: pointer to SPEED_RAMP_HANDLE structure
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @return none
*/
void Tail_Head_Wind_Process(SPEED_RAMP_HANDLE *speedHandle, INITSPEED_JUDGE *judgeHandle);

/*!
* @brief The per-unit value of speed ramp value is calculated according to the speed command unit pu.
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @param[in] speedCmdPu: Given speed command unit pu
* @return none
*/
void ASR_SetSpeedTargetRamp_Pu(SPEED_RAMP_HANDLE *command, float speedCmdPu);

/*!
* @brief Get the speed ramp value from the knob on the board.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_GetSpeedRampWithKnob(SPEED_RAMP_HANDLE *speed);

/*!
* @brief The per-unit value of speed targe value is calculated according to the speed command unit pu.
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @param[in] speedCmdPu: Given speed command unit pu
* @return none
*/
void ASR_SetSpeedTarget_Pu(SPEED_RAMP_HANDLE *command, float speedCmdPu);
/*!
* @brief Get the feedback speed unit in Rpm.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
float ASR_GetSpeedFbk_Rpm(SPEED_RAMP_HANDLE *speed);

/*!
* @brief Get the feedback speed unit in Pu.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
float ASR_GetSpeedFbk_Pu(SPEED_RAMP_HANDLE *speed);

/*!
* @brief Set the per-unit value of feedback speed.
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @param[in] speedFbk: speed feedback unit pu
* @return none
*/
void ASR_SetSpeedFbk_Pu(SPEED_RAMP_HANDLE *command, float speedFbk);

/*!
* @brief Speed loop variable PI parameters initialize, usually it is initialized only once before running.
*
* @param[in] asrVarPI: pointer to ASR_SPEEDVARPI structure
* @param[in] asrPidCof: pointer to PID_REGULATOR_COF structure
* @return none
*/
void ASR_SpeedVarPIInit(ASR_SPEEDVARPI *asrVarPI, PID_REGULATOR_COF *asrPidCof);

/*!
* @brief Speed loop variable PI parameters calculate, it divides the speed loop PI parameters into three sections
         according to the speed, low speed PI is used at low speed, high speed PI is used at high speed, and the
         intermediate speed is linearly transitioned.
*
* @param[in] freq: motor electrical frequency pu value in Q15
* @param[in] asrVarPI: pointer to ASR_SPEEDVARPI structure
* @param[in] asrPidCof: pointer to PID_REGULATOR_COF structure
* @return none
*/
void ASR_SpeedVarPICalc(float freq, ASR_SPEEDVARPI *asrVarPI, PID_REGULATOR_COF *asrPidCof);

/*!
* @brief Speed loop fast desaturation function, when the speed loop is saturated, the speed
         response is accelerated by weakening the speed loop integral term.
*
* @param[in] asrPid: pointer to PID_REGULATOR structure
* @param[in] acrPidQ: pointer to q axis PID_REGULATOR_OUT structure
* @return none
*/
void ASR_Quick_Desaturation(PID_REGULATOR *asrPid, PID_REGULATOR_OUT *acrPidQ);

/*!
* @brief Rotation speed Coefficient initialization.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_RotationSpeedInit(SPEED_RAMP_HANDLE *speed);

/*!
* @brief Torque control command initialization.
*
* @param[in] torque: pointer to TORQUE_COMMAND structure
* @return none
*/
void ACR_TorqueCommandInit(TORQUE_COMMAND *torque);

/*!
* @brief Speed command calculation initialization.
*
* @param[in] focVars: pointer to FOC_VARS_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_SpeedCommandInit(FOC_VARS_CFG *focVars, SPEED_RAMP_HANDLE *speed);

/*!
* @brief Position ramp calculation initialization.
*
* @param[in] focVars: pointer to FOC_VARS_CFG structure
* @param[in] positionCmd: pointer to POSITION_RAMP_HANDLE structure
* @return none
*/
void APR_PositionCommandInit(FOC_VARS_CFG *focVars, POSITION_RAMP_HANDLE *positionCmd);

/*!
* @brief Speed ramp configuration.
*
* @param[in] focVars: pointer to FOC_VARS_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_SpeedCommandSet(FOC_VARS_CFG *focVars, SPEED_RAMP_HANDLE *speed);

/*!
* @brief Speed ramp calculation. calculate the ramp output according to the speed target value.
*
* @param[in] focVars: pointer to FOC_VARS_CFG structure
* @param[in] speedRamp: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_SpeedTargetRampCalc(FOC_VARS_CFG *focVars, SPEED_RAMP_HANDLE *speedRamp);

/*!
* @brief Update action direction and adjust value according to target angle.
*
* @param[in] positionCmd: pointer to POSITION_RAMP_HANDLE structure
* @return none
*/
void PositionCommandUpdate(POSITION_RAMP_HANDLE *positionCmd);

/*!
* @brief Position ramp calculation. calculate the ramp output according to the position target value.
*
* @param[in] positionRamp: pointer to POSITION_RAMP_HANDLE structure
* @return none
*/
void APR_PositionTargetRampCalc(POSITION_RAMP_HANDLE *positionRamp);

/*!
* @brief Stores estimated rotation speed to FIFO.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_StoreRotorSpeed(SPEED_RAMP_HANDLE *speed);

/*!
* @brief Calculated average estimated rotation speed and speed variance value from FIFO.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_AvgSpeedCalc(SPEED_RAMP_HANDLE *speed);

/*!
* @brief Get the speed ramp value. Assign to speed loop reference value.
*
* @param[in] speedRamp: pointer to SPEED_RAMP_HANDLE structure
* @return speedTargetRamp: speed targe ramp value
*/
float ASR_GetSpeedRamp(SPEED_RAMP_HANDLE *speedRamp);

/*!
* @brief Speed loop processing of motor control.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @param[in] pid: pointer to PID_REGULATOR structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void ASR_SpeedLoop_Process(SPEED_RAMP_HANDLE *speed, PID_REGULATOR *pid, FOC_VARS_CTRL *focVars);

/*!
* @brief Position loop processing of motor control.
*
* @param[in] position: pointer to POSITION_RAMP_HANDLE structure
* @param[in] pid: pointer to PID_REGULATOR structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void APR_PositionLoop_Process(POSITION_RAMP_HANDLE *position, PID_REGULATOR *pid, FOC_VARS_CTRL *focVars);

/*!
* @brief Update the feedback speed of motor in different modes(Hall/Encode/sensorless).
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_SpeedFbk_Update(SPEED_RAMP_HANDLE *command);

/*!
* @brief Get electric angle according to the different sensors.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Get_ElecAngle(FOC_VARS_CTRL *focVars);

/*
* @brief Structure variables initialization.

* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_InitProcess(SCURVE_DATA *pScurve);

/*
* @brief S curve calculation main process.

* @param[in] pScurve: pointer to SCURVE_DATA structure
* @param[in] speedRamp: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void SCurve_MainProcess(SCURVE_DATA *pScurve, SPEED_RAMP_HANDLE *speedRamp);

/*
* @brief Rotor speed and synchronous angle calculation with encoder of induction motor.

* @param[in] acimFoc: pointer to ACIM_FOC_CTRL structure
* @param[in] STC: pointer to STC_PARAM structure
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @param[in] speedCalc: pointer to ESTIMATED_SPEED structure
* @return none
*/
void Acim_SensorSpeedCalc(ACIM_FOC_CTRL *acimFoc, STC_PARAM *STC, ENCODER_TYPE *encoderSensor, ESTIMATED_SPEED *speedCalc);

/* ======================================  Functions define  ======================================== */

#endif /* __SPEED_ANGLE_H__ */

/* =============================================  EOF  ============================================== */
