/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __STARTUP_H__
#define __STARTUP_H__
/*!
* @file startup.h
*
* @brief This file provides sensorless foc open loop startup functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "param_init.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Open loop startup curve frequency, current, voltage, time reference structure.
*/
typedef struct {
    float freqRefStart;          /*!< Starting point reference value of frequency curve */
    float freqRefEnd;            /*!< Ending point reference value of frequency curve */
    float curRefStart;           /*!< Starting point reference value of current curve */
    float curRefEnd;             /*!< Ending point reference value of current curve */
    float voltRefStart;          /*!< Starting point reference value of voltage curve */
    float voltRefEnd;            /*!< Ending point reference value of voltage curve */
    uint32_t timeStart;             /*!< Start time of open loop startup curve */
    uint32_t timeEnd;               /*!< End time of open loop startup curve */
} CURVE_REFVALUE;

/*!
* @brief Open loop startup curve calculation unit input parameters structure.
*/
typedef struct {
    uint8_t groupNum;              /*!< User defined points of open loop startup curve */
    uint16_t timeDuration;          /*!< Time duration value of the curve calculation unit */
    uint32_t timeStartValue;        /*!< Starting time value of the curve calculation unit */
    uint32_t timeEndValue;          /*!< Ending time value of the curve calculation unit */
    uint32_t timeValue;             /*!< Time count value of the curve calculation unit */
    float refValue1;             /*!< Starting point value of the curve calculation unit */
    float refValue2;             /*!< Ending point value of the curve calculation unit */

    CURVE_REFVALUE *curveSeg1;      /*!< segment 1 of open loop startup curve */
    CURVE_REFVALUE *curveSeg2;      /*!< segment 2 of open loop startup curve */
    CURVE_REFVALUE *curveSeg3;      /*!< segment 3 of open loop startup curve */
    CURVE_REFVALUE *curveSeg4;      /*!< segment 4 of open loop startup curve */
    CURVE_REFVALUE *curveSeg5;      /*!< segment 5 of open loop startup curve */
} CURVE_IN;

/*!
* @brief Open loop startup curve calculation unit output parameters structure.
*/
typedef struct {
    float refCalcValue;          /*!< Output variables of the curve calculation unit */
    float frqRefValue;           /*!< Frequency curve output value of the curve calculation unit */
    float curRefValue;           /*!< Current curve output value of the curve calculation unit */
    float voltRefValue;          /*!< Voltage curve output value of the curve calculation unit */
    uint8_t calcEndFlag;           /*!< Flag of completion of the curve calculation unit */
    uint8_t calcEndFlag1;          /*!< Flag of completion of the curve calculation when close loop switch to open loop */
    uint8_t calcEndFlag2;          /*!< Flag of completion of the curve calculation when open loop switch to close loop*/
} CURVE_OUT;

/*!
* @brief Open loop startup curve calculation unit time parameters structure.
*/
typedef struct {
    uint16_t timePoint1;            /*!< Time value at the end of the first segment of the open loop startup curve  */
    uint16_t timePoint2;            /*!< Time value at the end of the second segment of the open loop startup curve  */
    uint16_t timePoint3;            /*!< Time value at the end of the third segment of the open loop startup curve  */
    uint16_t timePoint4;            /*!< Time value at the end of the fourth segment of the open loop startup curve  */
    uint16_t timePoint5;            /*!< Time value at the end of the fifth segment of the open loop startup curve  */
} CURVE_TIME_DURATION;

/*!
* @brief Open loop startup curve configuration parameters structure.
*/
typedef struct
{
    uint8_t curveSegNum;           /*!< Segment number of open loop start curve */
    uint16_t curveTime1;            /*!< Time duration value of open loop start curve segment 1 */
    uint16_t curveTime2;            /*!< Time duration value of open loop start curve segment 2 */
    uint16_t curveTime3;            /*!< Time duration value of open loop start curve segment 3 */
    uint16_t curveTime4;            /*!< Time duration value of open loop start curve segment 4 */
    uint16_t curveTime5;            /*!< Time duration value of open loop start curve segment 5 */
    uint16_t synTime;               /*!< Synchronization times to reach openloop target speed */
    uint16_t alignTime;             /*!< Time of pole alignment */
    uint16_t alignTimeTwice;        /*!< Time of the second pole alignment */
    uint16_t slopeDuration1;        /*!< Alignment once time of current arrive to traget value in ms */
    uint16_t slopeDuration2;        /*!< Alignment twice time of current arrive to traget value in ms */

    float curStartOffset;         /*!< Open loop startup current setting value */
    float synSpd;                 /*!< Target Synchronization speed of open loop */
    float spdSmoothCur;           /*!< Speed loop integration initial value, used to smooth the speed switching process */
    float alignAngle;            /*!< Angle of pole alignment */
    float alignCur;              /*!< Motor current with pole alignment */
    float alignCurTwice;         /*!< Motor current at second pole alignment */
    float curRamp1Init;          /*!< Alignment once current add initial value */
    float curRamp2Init;          /*!< Alignment twice current add initial value */
    float curveFreqStart;        /*!< Frequency initial value of open loop start curve */
    float curveCurStart;         /*!< Current initial value of open loop start curve */
    float curveVoltStart;        /*!< Voltage initial value of open loop start curve */
    float curveFreqSet1;         /*!< Frequency set value of point 1 of open loop start curve */
    float curveFreqSet2;         /*!< Frequency set value of point 2 of open loop start curve */
    float curveFreqSet3;         /*!< Frequency set value of point 3 of open loop start curve */
    float curveFreqSet4;         /*!< Frequency set value of point 4 of open loop start curve */
    float curveFreqSet5;         /*!< Frequency set value of point 5 of open loop start curve */
    float curveCurSet1;          /*!< Current set value of point 1 of open loop start curve */
    float curveCurSet2;          /*!< Current set value of point 2 of open loop start curve */
    float curveCurSet3;          /*!< Current set value of point 3 of open loop start curve */
    float curveCurSet4;          /*!< Current set value of point 4 of open loop start curve */
    float curveCurSet5;          /*!< Current set value of point 5 of open loop start curve */
    float curveVoltSet1;         /*!< Voltage set value of point 1 of open loop start curve */
    float curveVoltSet2;         /*!< Voltage set value of point 2 of open loop start curve */
    float curveVoltSet3;         /*!< Voltage set value of point 3 of open loop start curve */
    float curveVoltSet4;         /*!< Voltage set value of point 4 of open loop start curve */
    float curveVoltSet5;         /*!< Voltage set value of point 5 of open loop start curve */
    float curStartSub;           /*!< Decrement calue of current calculation */
    float curveFreqStart1;       /*!< Frequency initial value of open loop start curve */
    float curveCurStart1;        /*!< Current initial value of open loop start curve */
} START_UP_CFG;

/*!
* @brief Open loop startup curve control parameters structure.
*/
typedef struct
{
    uint8_t startUpFlag;           /*!< Open loop startup operation flag */
    uint8_t synFreReach;           /*!< Flag to reach the open loop synchronization frequency */
    uint8_t alignState;            /*!< State of the rotor alignment process */
    uint16_t synTime;               /*!< Synchronization times to reach openloop target speed */
    uint16_t synCount;              /*!< The count to judging the openloop synchronization frequency is reached or not */
    uint16_t countStartUpST1;       /*!< Count value for open loop startup control */
    uint16_t countStartUpST1a;      /*!< Count value for open loop startup control */
    uint16_t countStartUpST2;       /*!< Count value for open loop startup control */
    uint16_t countStartUpST3;       /*!< Count value for open loop startup control */
    uint16_t countStartUpST4;       /*!< Count value for open loop startup control */
    uint16_t countStartUpST5;       /*!< Count value for open loop startup control */
    uint16_t startUpTimes;          /*!< Count value for open loop startup control */
    float elecThetaEst;          /*!< Estimation of electrical angle for open loop */
    float curStartSub;           /*!< Decrement calue of current calculation */
    float startUpElecTheta;      /*!< Electric angle of open loop control */
    float synSpd;                 /*!< Target Synchronization speed of open loop */
    float curStartOffset;         /*!< Open loop startup current setting value */
    float curAddSlope1;           /*!< Alignment once current add slope */
    float curAddSlope2;           /*!< Alignment twice current add slope */
} STARTUP_CTRL;

/*!
* @brief Open loop startup curve calculation unit time parameters structure.
*/
typedef struct
{
    START_UP_CFG *pStCfg;           /*!< Open loop configuration parameter structure */
    STARTUP_CTRL *pStCtrl;          /*!< Open loop control parameter structure */
} START_UP;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Clear startUp control counting parameter.
*
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void ClearStartUpParam(START_UP *startUp);

/*!
* @brief Motor startup control initialization.
*
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Ctrl_Init(START_UP *startUp);

/*!
* @brief StartUp control task in foc process.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Ctrl_Foc(FOC_VARS *focVars, START_UP *startUp);

/*!
* @brief StartUp control task in 1ms period.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void StartUp_Ctrl_1ms(FOC_VARS *focVars, START_UP *startUp);

/*!
* @brief Pre_positioning motor to an expected angle.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS_CTRL structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Angle_Prepositioning_Process(FOC_VARS_CTRL *focVarsCtrl, START_UP *startUp);

/*!
* @brief Angle alignment process to prepare for motor open loop operation.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS_CTRL structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Angle_Alignment_Process(FOC_VARS_CTRL *focVarsCtrl, START_UP *startUp);

/*!
* @brief Transfer the electrical angle of the motor during the startup phase.
*
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return startUpElecTheta: electric angle of startup process
*/
float Get_Sensorless_ElecAngle(STARTUP_CTRL *startUp);

/*!
* @brief StartUp control task in 1ms period of ACIM SLVC.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] startUp: pointer to START_UP structure
* @return none
*/
void Acim_Startup_1ms(FOC_VARS *focVars, START_UP *startUp);

/*!
* @brief Asr coefficients adaptive calculation.
*
* @param[in] none
* @return none
*/
void Acim_AsrPiCoefUpdate(void);

/* ======================================  Functions define  ======================================== */

#endif /* __STARTUP_H__ */

/* =============================================  EOF  ============================================== */
