/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __VF_CONTROL_H__
#define __VF_CONTROL_H__
/*!
* @file vf_control.h
*
* @brief This file provides Vf control functions interfase.
*
*/

/* ===========================================  Includes  =========================================== */
#include <stdint.h>
#include "iqmath.h"
#include "vf_compensate.h"

/* ============================================  Define  ============================================ */
#define VF_BEELINE                          0               /*!< 0, beeline(straight line); */
#define VF_SQURELINE                        1               /*!< 1, squre curve; */
#define VF_USERMANUAL                       2               /*!< 2, three-points and four-segment custom curve(broken line) */
#define VF_CURVE                            VF_BEELINE      /*!< Selection V/f curve */

#define VF_TORQUEBOOST_CUTOFF_FREQ          (8.0f / BASE_FREQ)    /*!< Torque boost cutoff frequency */
#define VF_TORQUEBOOST_VOLTAGE_PERCENTAGE   0.10f                          /*!< Torque boost voltage rise percentage */

#define VF_CURVE_POINT1_FREQ                (15.0f / BASE_FREQ)   /*!< Frequency of the first point of the V/f characteristic curve */
#define VF_CURVE_POINT1_VOLT_PERCENTAGE     0.12f                          /*!< Voltage percentage of the first point of the V/f characteristic curve */
#define VF_CURVE_POINT2_FREQ                (50.0f / BASE_FREQ)   /*!< Frequency of the second point of the V/f characteristic curve */
#define VF_CURVE_POINT2_VOLT_PERCENTAGE     0.30f                          /*!< Voltage percentage of the second point of the V/f characteristic curve */
#define VF_CURVE_POINT3_FREQ                (100.0f / BASE_FREQ)  /*!< Frequency of the third point of the V/f characteristic curve */
#define VF_CURVE_POINT3_VOLT_PERCENTAGE     0.55f                          /*!< Voltage percentage of the third point of the V/f characteristic curve */

#define HOP_FREQ_POINT1                     (15.0f / BASE_FREQ)   /*!< Frequency of the first center point of frequency hopping */
#define HOP_FREQ_POINT2                     (30.0f / BASE_FREQ)   /*!< Frequency of the second center point of frequency hopping */
#define HOP_FREQ_POINT3                     (48.0f / BASE_FREQ)   /*!< Frequency of the third center point of frequency hopping */
#define HOP_WIDTH_1                         (5.0f / BASE_FREQ)    /*!< Width of the first center point of frequency hopping */
#define HOP_WIDTH_2                         (5.0f / BASE_FREQ)    /*!< Width of the second center point of frequency hopping */
#define HOP_WIDTH_3                         (2.0f / BASE_FREQ)    /*!< Width of the third center point of frequency hopping */

#define VF_MOTOR_RATED_FREQUENCY            0.9f//(150.0f / BASE_FREQ)   /*!< Motor rated frequency */
#define VF_CURVE_MAX_FREQUENCY              0.9f//(150.0f / BASE_FREQ)   /*!< V/f characteristic curve maximum frequency */
#define VF_DEFAULT_TARGET_FREQ              (25.0f / BASE_FREQ)    /*!< V/f control default target value when the target frequency is not set */
#define VF_STOP_FREQ                        (1.0f / BASE_FREQ)     /*!< Motor stop frequency of V/f control */

#define VF_DC_TIME                          100                         /*!< Duration of DC startup, unit ms */
#define VF_DC_VOLTAGE                       (0.2F)                /*!< Voltage of DC startup */

/*!
* @brief V/f control compensation macro switch, open to execute function, shield to close function.
*/
//#define VF_COMPENSATION

/*!
* @brief V/f control current damp suppress macro switch, open to execute function, shield to close function.
*/
//#define VF_CURRENT_DAMP_ENABLE

/*!
* @brief V/f control active power compensation macro switch, open to execute function, shield to close function.
*/
//#define VF_ACTIVEPOWER_COMP_ENABLE

/*!
* @brief V/f control reactive power compensation macro switch, open to execute function, shield to close function.
*/
//#define VF_REACTIVEPOWER_COMP_ENABLE

/*!
* @brief V/f control frequency hopping macro switch, open to execute function, shield to close function.
*/
//#define FREQ_HOPPING_ENABLE

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Vf control ISR process.
*
* @param[in] none
* @return none
*/
void Vf_ISR(void);

/*!
* @brief Vf control Parameters initialization.
*
* @param[in] none
* @return none
*/
void Vf_ParamInit(void);

/*!
* @brief Vf control 1ms time base tasks.
*
* @param[in] none
* @return none
*/
void VF_1ms_Task(void);

/*!
* @brief Vf control 2ms time base tasks.
*
* @param[in] none
* @return none
*/
void VF_2ms_Task(void);

/*!
* @brief Vf control main functions.
*
* @param[in] none
* @return none
*/
void Vf_MainProcess(void);

/* ======================================  Functions define  ======================================== */

#endif /* __VF_CONTROL_H__ */

/* =============================================  EOF  ============================================== */
