/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file encoder.c
*
* @brief This file contain mainly two parts, read pulse count captrued by PWM module and calculate,update the elec angle in foc control.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pwm_input.h"
#include "motorctrl_api.h"
#include "globalvar.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Get pulse count captured by PWM module.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @return none
*/
void Encoder_GetPulseCount(ENCODER_TYPE *encoderSensor)
{
    /* Get encoder paulse count */
    encoderSensor->encoderCnt = PWM1->CNT & 0xFFFF;
}

/*!
* @brief Obtain pulse count offset after encoder ALIGN.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @return none
*/
void Encoder_GetAlignOffset(ENCODER_TYPE *encoderSensor)
{
    /* Get encoder paulse count */
    Encoder_GetPulseCount(encoderSensor);
    /* Get align point paulse count offset.
    actual align position can be expressed as: (ALIGNMENT_ANGLE_DEGREE + 90 * (2 - pFocVarsCfg->motorDir)).
    take default 270 degree as example: when motorDir is FORWARD_ROTATE(1), actual align position is 0 degree.
    when motorDir is REVERSE_ROTATE(-1), actual align position is 180 degree */
    encoderSensor->alignOffSet = (int16_t)(encoderSensor->encoderCnt - (pMotorParam->encoderLines / pMotorParam->polePairs)
                                  * (ALIGNMENT_ANGLE_DEGREE + 90 * (2 - pFocVarsCfg->motorDir)) / 360.0f);
    encoderSensor->encoderCntOffSet = encoderSensor->alignOffSet;
}

/*!
* @brief Statistic motor turn circles according to PWM module overflow direction.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @return none
*/
void Encoder_MotorCircleCalc(ENCODER_TYPE *encoderSensor)
{
    /* Get cross dir in overflow moment */
    encoderSensor->encoderDir = PWM_DRV_GetQuadOverflowDir(1U);
    if (encoderSensor->encoderDir != 0)
    {
        /* 144-->0, encoder circle increase */
        encoderSensor->encoderCircleCnt++;
    }
    else
    {
        /* 0-->144, encoder circle ruduce */
        encoderSensor->encoderCircleCnt--;
    }
}

/*!
* @brief Calculate encoder elec angle and update in foc control current loop.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @param[in] motor: pointer to MOTOR_PARAM structure
* @return encoder elec angle
*/
float Encoder_ElecAngleCalc(ENCODER_TYPE *encoderSensor, MOTOR_PARAM *motor)
{
    float temp_angle = 0.0f;

    /* Get encoder paulse count */
    Encoder_GetPulseCount(encoderSensor);
    #if (!defined POSITION_LOOP_CTRL)
    encoderSensor->encoderCircleCnt = 0;
    #endif

    /* Get mechanical position encoder count, 2.5degree per count */
    encoderSensor->encoderAngleCnt = encoderSensor->encoderCnt - encoderSensor->encoderCntOffSet
                                     + motor->encoderLines * encoderSensor->encoderCircleCnt;
    /* Turn mechanical position to FOC angle in encoder count unit */
    encoderSensor->encoderFocCnt = (encoderSensor->encoderAngleCnt + motor->encoderLines
                                   * (1 - encoderSensor->encoderCircleCnt)) % motor->encoderLines;
    /* Calculate elec Angle In Degree(FOC) */
    temp_angle = (float)encoderSensor->encoderFocCnt * motor->polePairs / (motor->encoderLines);
    encoderSensor->encoderElecAngle = temp_angle - (int16_t)temp_angle;

    return encoderSensor->encoderElecAngle;
}

/*!
* @brief Encoder and HallSensor data initialize. The initial rotor position get from hall sensor.
*        The coefficient is relevant to np (pole pairs number) and AB_encoder sensor revolution.
*
* @param[in] encoderHallSensor: hall and AB_encoder sensor data variable, pointer to ENCODER_HALL_SENSOR struct
* @param[in] hallSensor: hallSensor data, pointer to HALL_SENSOR_DATA structure
* @param[in] motor: pointer to MOTOR_PARAM structure
* @return none
*/
void EncoderHall_Init(ENCODER_HALL_SENSOR *encoderHallSensor, HALL_SENSOR_DATA *hallSensor, MOTOR_PARAM *motor)
{
    encoderHallSensor->adIsrCnt = 0;
    encoderHallSensor->firstHallStatus = 0;
    encoderHallSensor->estThetaOffset = hallSensor->estTheta;
    encoderHallSensor->calCoff = motor->polePairs / motor->encoderLines;
}

/*!
* @brief Calculate rotor position. This function is called in erery ADC isr function.
*        Base on PWM modular, get counter per Ts, then calculate rotor position.
*
* @param[in] encoderHallSensor: hall and AB_encoder sensor data variable, pointer to ENCODER_HALL_SENSOR struct
* @return none
*/
void EncoderHall_AngleCal(ENCODER_HALL_SENSOR *encoderHallSensor)
{
    encoderHallSensor->encoderCntValBuf = encoderHallSensor->encoderCntVal;
    encoderHallSensor->encoderCntVal = (PWM1->CNT & 0xFFFF);
    if (PWM_DRV_GetOverflowFlag(1U))
    {
        PWM_DRV_ClearOverflowFlag(1U);
    }
    else
    {
        encoderHallSensor->estTheta = encoderHallSensor->estThetaOffset + (encoderHallSensor->encoderCntVal     \
                                    - encoderHallSensor->encoderCntOffset) * encoderHallSensor->calCoff;
    }
}

/*!
* @brief If the AB_encoder sensor PULSE_PER_REVOLUTION is not accurate, maybe need compensate rotor position ervery Ts.
*        This function can make rotor position angle accumulate more precise.
*
* @param[in] encoderHallSensor: hall and AB_encoder sensor data variable, pointer to ENCODER_HALL_SENSOR struct
* @return none
*/
void EncoderHall_AngleStep(ENCODER_HALL_SENSOR *encoderHallSensor)
{
    if (encoderHallSensor->encoderCntVal == encoderHallSensor->encoderCntValBuf)
    {
        encoderHallSensor->adIsrCnt++;
        if (encoderHallSensor->adIsrCntBuf != 0)
        {
            encoderHallSensor->estTheta += Math_DivShift(encoderHallSensor->adIsrCnt * encoderHallSensor->calCoff, encoderHallSensor->adIsrCntBuf, 0);
        }
    }
    else
    {
        encoderHallSensor->adIsrCntBuf = encoderHallSensor->adIsrCnt;
        encoderHallSensor->adIsrCnt = 0;
    }
}

/*!
* @brief If only use AB_encoder sensor calculate rotor position, will generate cumulative error. In order to ekiminate cumulative error
*        need use hall adjust rotor position every electrical cycle. This function is called in PWDT interrupt.
*
* @param[in] encoderHallSensor: hall and AB_encoder sensor data variable, pointer to ENCODER_HALL_SENSOR struct
* @param[in] hallSensor: hallSensor data, pointer to HALL_SENSOR_DATA structure
* @return none
*/
void EncoderHall_AngleAdjust(ENCODER_HALL_SENSOR *encoderHallSensor, HALL_SENSOR_DATA *hallSensor)
{
    if (encoderHallSensor->firstHallStatus == 0)
    {
        encoderHallSensor->firstHallStatus = hallSensor->hallEstThetaSector;
    }
    if (encoderHallSensor->firstHallStatus == hallSensor->hallEstThetaSector)
    {
        encoderHallSensor->estThetaOffset = g_hallSectorDegree[hallSensor->hallEstThetaSector];
        encoderHallSensor->encoderCntOffset = (PWM1->CNT & 0xFFFF);
    }
}

/*!
* @brief Used to obtain electrical angle in encoder sensor FOC.
*
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @return elec angle
*/
float Encoder_GetElecAngle(ENCODER_TYPE *encoderSensor)
{
    return encoderSensor->encoderElecAngle;
}

/*!
* @brief Output encoder_hall mode rotor position estimated.
*
* @param[in] encoderHallSensor: hall and AB_encoder sensor data variable, pointer to ENCODER_HALL_SENSOR struct
* @return rotor postion angle estimation
*/
float EncoderHall_GetElecAngle(ENCODER_HALL_SENSOR *encoderHallSensor)
{
    return encoderHallSensor->estTheta;
}

/*!
* @brief Encoder M_T measure method parameters initialization.
*
* @param[in] encoderMeasure: encoder speed measure variable, pointer to ENCODER_SPEED_MEASURE struct
* @return none
*/
void Encoder_MT_Init(ENCODER_SPEED_MEASURE *encoderMeasure)
{
    memset(encoderMeasure, 0, sizeof(*(encoderMeasure)));
    encoderMeasure->mtSpeedCoeff = ((float)T_METHOD_HZ * POLE_PAIR_NUM) / PULSE_PER_REVOLUTION;
}

/*!
* @brief Measure encoder pulse and time.
*
* @param[in] encoderMeasure: encoder speed measure variable, pointer to ENCODER_SPEED_MEASURE struct
* @return none
*/
void Encoder_MT_Measure(ENCODER_SPEED_MEASURE *encoderMeasure)
{
    uint32_t temp_Mcvr;
    uint32_t temp_Cnt;
    int16_t temp_Pulse = (int16_t)(PULSE_PER_REVOLUTION / POLE_PAIR_NUM);

    /* build high frequency pulse base count */
    if (++encoderMeasure->pulseTime > 60000)
    {
        encoderMeasure->pulseTime = 0;
    }

    /* encoder pulse edge alignment */
    temp_Cnt = PWM1->CNT;
    temp_Mcvr = PWM1->MCVR;
//    temp_Cnt = PWM_DRV_GetCounter(PWM_MODULER_ENCODER);

    if (((temp_Cnt % 4) == 1) && (encoderMeasure->speedUpdateFlag))
    {
        /* get and save M and T count */
        encoderMeasure->measurePulse1 = encoderMeasure->measurePulse2;
        encoderMeasure->measureTime1 = encoderMeasure->measureTime2;
        encoderMeasure->measurePulse2 = temp_Cnt & 0xFFFF;
        encoderMeasure->measureTime2 = encoderMeasure->pulseTime;
        /* get M and T delta count */
        if (encoderMeasure->measureTime2 < encoderMeasure->measureTime1)
        {
            encoderMeasure->deltaTime = encoderMeasure->measureTime2 + (60000 - encoderMeasure->measureTime1);
        }
        else
        {
            encoderMeasure->deltaTime = encoderMeasure->measureTime2 - encoderMeasure->measureTime1;
        }

        if ((encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1) > (temp_Pulse))
        {
            encoderMeasure->deltaPulse = encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1 - temp_Mcvr;
        }
        else if ((encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1) < -(temp_Pulse))
        {
            encoderMeasure->deltaPulse = encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1 + temp_Mcvr;
        }
        else
        {
            encoderMeasure->deltaPulse = encoderMeasure->measurePulse2 - encoderMeasure->measurePulse1;
        }
        encoderMeasure->speedUpdateFlag = 0;
    }
}

/*!
* @brief Calculate encoder pulse elec frequency by Measured encoder pulse and time.
*
* @param[in] encoderMeasure: encoder speed measure variable, pointer to ENCODER_SPEED_MEASURE struct
* @return none
*/
void Encoder_MT_SpeedCalc(ENCODER_SPEED_MEASURE *encoderMeasure)
{
    if (encoderMeasure->speedUpdateFlag == 0)
    {
        /* use M and T calculate M_T speed */
        if (encoderMeasure->deltaTime != 0)
        {
            encoderMeasure->measureFreq = ((float)encoderMeasure->deltaPulse * encoderMeasure->mtSpeedCoeff) / encoderMeasure->deltaTime;
        }
        encoderMeasure->mtSpeed = encoderMeasure->measureFreq * ONE_BASE_FREQ;
        encoderMeasure->mtSpeedFilter = (encoderMeasure->mtSpeedFilter * 7 + encoderMeasure->mtSpeed * 1) / 8;
        encoderMeasure->speedUpdateFlag = 1;
    }
}

/* =============================================  EOF  ============================================== */
