/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file vf_control.c
*
* @brief This file provides Vf control functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "adc_hw.h"
#include "foc_keys.h"
#include "foc_app.h"
#include "foc_isr.h"
#include "globalvar.h"
#include "protector.h"
#include "motorctrl_api.h"
#include "vf_control.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
VF_CURVE_INPUT_PARAM        g_vfCurveInputParam;    /*!< vf curve parameters */
VF_DC_STARTUP               g_vfDcStartUp;          /*!< vf control parameters to DC start-up, PMSM */
VF_COMP_PARAM               g_vfCompParam;          /*!< vf control swing suppression parameters */
VF_COMP_SET                 g_vfCompSet;            /*!< vf control compensate setting value */
VF_VECTOR_PARAM             g_vfVect;               /*!< vf voltage and current component */
VF_INPUT                    g_vfInput;              /*!< vf control input: frequency set value */
VF_OUTPUT                   g_vfOutput;             /*!< vf control output: voltage and elec-theta, synchronous frequency; */
VF_THETA_CAL                g_vfThetaCal;           /*!< vf control parameters to calculate elec-theta */

VF_FREQ_CALC_BUF            g_vfFreqClac;           /*!< vf control frequency calculation buf struct */
VF_FREQ_HOP_PARAM           g_vfFreqHopParam;       /*!< vf control frequency hopping parameters struct */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Vf control beeline characteristic curve.
*
* @param[in] targetFreqPu: cueernt frequency command value
* @param[in] maxFreqPu: current max frequency
* @param[in] input: pointer to VFCURVE_INPUT_PARAM structure
* @return outputVoltPu
*/
float Vf_BeelineCurve(float targetFreqPu, float maxFreqPu, VF_CURVE_INPUT_PARAM *input)
{
    float freqCommand = 0.0f;
    float freqUpperLimitPu = 0.0f;
    float freqCutoffPu = 0.0f;
    float voltCutoffPu = 0.0f;
    float outputVoltLowerLimit = 0.0f;
    float outputVoltAddVoluePu = 0.0f;
    float outputVoltPu = 0.0f;

    freqCommand = MathFloat_Abs(targetFreqPu);
    freqUpperLimitPu = MathFloat_Div(input->motorFreqRatedPu * maxFreqPu, input->motorMaxFreqPu);
    freqCutoffPu = MathFloat_Div(input->torqBoostCutoffFreqPu * maxFreqPu, input->motorMaxFreqPu);
    outputVoltLowerLimit = input->torqBoostVoltPct * input->motorVoltRatedPu;
    voltCutoffPu = MathFloat_Div(freqCutoffPu * input->motorVoltRatedPu, freqUpperLimitPu);
    if (voltCutoffPu <= outputVoltLowerLimit)
    {
        voltCutoffPu = outputVoltLowerLimit;
    }
    if (freqCommand <= freqCutoffPu)
    {
        outputVoltAddVoluePu = outputVoltLowerLimit - MathFloat_Div(freqCommand * outputVoltLowerLimit, freqCutoffPu);
        outputVoltPu = MathFloat_Div(freqCommand * voltCutoffPu, freqCutoffPu) + outputVoltAddVoluePu;
    }
    else if ((freqCommand > freqCutoffPu) && (freqCommand <= freqUpperLimitPu))
    {
        outputVoltPu = MathFloat_Div(freqCommand * input->motorVoltRatedPu, freqUpperLimitPu);
    }
    else
    {
        outputVoltPu = input->motorVoltRatedPu;
    }

    return outputVoltPu;
}

/*!
* @brief Vf control squre characteristic curve.
*
* @param[in] targetFreqPu: cueernt frequency command value
* @param[in] maxFreqPu: current max frequency
* @param[in] input: pointer to VFCURVE_INPUT_PARAM structure
* @return outputVoltPu
*/
float Vf_SqureCurve(float targetFreqPu, float maxFreqPu, VF_CURVE_INPUT_PARAM *input)
{
    float freqCommand = 0;
    float freqUpperLimitPu = 0;
    float freqCutoffPu = 0;
    float voltCutoffPu = 0;
    float outputVoltLowerLimit = 0;
    float outputVoltAddVoluePu = 0;
    float outputVoltPu = 0;
    float tempSqureVar1 = 0;
    float tempSqureVar2 = 0;
    float tempSqureVar3 = 0;
    float tempSqureVar4 = 0;

    freqCommand = MathFloat_Abs(targetFreqPu);
    freqUpperLimitPu = MathFloat_Div(input->motorFreqRatedPu * maxFreqPu, input->motorMaxFreqPu);
    freqCutoffPu = MathFloat_Div(input->torqBoostCutoffFreqPu * maxFreqPu, input->motorMaxFreqPu);
    outputVoltLowerLimit = input->torqBoostVoltPct * input->motorVoltRatedPu;
    voltCutoffPu = MathFloat_Div(freqCutoffPu * input->motorVoltRatedPu, freqUpperLimitPu);
    if (voltCutoffPu <= outputVoltLowerLimit)
    {
        voltCutoffPu = outputVoltLowerLimit;
    }
    tempSqureVar1 = MathFloat_Div(freqCommand, freqCutoffPu);
    tempSqureVar2 = MathFloat_Div(freqCommand, freqUpperLimitPu);
    tempSqureVar3 = tempSqureVar1 * outputVoltLowerLimit;
    tempSqureVar4 = tempSqureVar2 * input->motorVoltRatedPu;
    if (freqCommand <= freqCutoffPu)
    {
        outputVoltAddVoluePu = outputVoltLowerLimit - tempSqureVar1 * tempSqureVar3;
        outputVoltPu = tempSqureVar2 * tempSqureVar4 + outputVoltAddVoluePu;
    }
    else if ((freqCommand > freqCutoffPu) && (freqCommand <= freqUpperLimitPu))
    {
        outputVoltPu = tempSqureVar2 * tempSqureVar4;
    }
    else
    {
        outputVoltPu = input->motorVoltRatedPu;
    }

    return outputVoltPu;
}

/*!
* @brief Vf control customer characteristic curve.
*
* @param[in] targetFreqPu: cueernt frequency command value
* @param[in] maxFreqPu: current max frequency
* @param[in] input: pointer to VFCURVE_INPUT_PARAM structure
* @return outputVoltPu
*/
float Vf_CustomerCurve(float targetFreqPu, float maxFreqPu, VF_CURVE_INPUT_PARAM *input)
{
    float freqCommand = 0.0f;
    float freqUpperLimitPu = 0.0f;
    float outputVoltLowerLimit = 0.0f;
    float outputVoltPu = 0.0f;
    float tempFreq1Pu = 0.0f;
    float tempVolt1Pu = 0.0f;
    float tempFreq2Pu = 0.0f;
    float tempVolt2Pu = 0.0f;
    float tempFreq3Pu = 0.0f;
    float tempVolt3Pu = 0.0f;

    freqCommand = MathFloat_Abs(targetFreqPu);
    freqUpperLimitPu = input->motorMaxFreqPu;
    outputVoltLowerLimit = input->torqBoostVoltPct * input->motorVoltRatedPu;
    tempFreq1Pu = MathFloat_Div(input->vfManualPoint1FreqPu * maxFreqPu, input->motorMaxFreqPu);
    tempFreq2Pu = MathFloat_Div(input->vfManualPoint2FreqPu * maxFreqPu, input->motorMaxFreqPu);
    tempFreq3Pu = MathFloat_Div(input->vfManualPoint3FreqPu * maxFreqPu, input->motorMaxFreqPu);
    tempVolt1Pu = input->vfManualPoint1VoltPct * input->motorVoltRatedPu;
    tempVolt2Pu = input->vfManualPoint2VoltPct * input->motorVoltRatedPu;
    tempVolt3Pu = input->vfManualPoint3VoltPct * input->motorVoltRatedPu;
    if (freqCommand >= tempFreq1Pu)
    {
        if (freqCommand < tempFreq2Pu)
        {
            outputVoltPu = MathFloat_Div((freqCommand - tempFreq1Pu) * (tempVolt2Pu - tempVolt1Pu), (tempFreq2Pu - tempFreq1Pu)) + tempVolt1Pu;
        }
        else if (freqCommand < tempFreq3Pu)
        {
            outputVoltPu = MathFloat_Div((freqCommand - tempFreq2Pu) * (tempVolt3Pu - tempVolt2Pu), (tempFreq3Pu - tempFreq2Pu)) + tempVolt2Pu;
        }
        else
        {
            outputVoltPu = MathFloat_Div((freqCommand - tempFreq3Pu) * (input->motorVoltRatedPu - tempVolt3Pu), (freqUpperLimitPu - tempFreq3Pu)) + tempVolt3Pu;
        }
    }
    else
    {
        outputVoltPu = MathFloat_Div(((tempFreq1Pu - freqCommand) * outputVoltLowerLimit + (uint32_t)freqCommand * tempVolt1Pu), tempFreq1Pu);
    }

    return outputVoltPu;
}

/*!
* @brief Vf curve output.
*
* @param[in] vfCurveSel: vf curve type
* @param[in] currentMaxFreqPu: current max frequency
* @param[in] in: pointer to VF_INPUT structure
* @param[in] out: pointer to VF_OUTPUT structure
* @param[in] curve: pointer to VF_CURVE_INPUT_PARAM structure
* @return none
*/
void Vf_CharacteristicCurve(int16_t vfCurveSel, float currentMaxFreqPu, VF_INPUT *in, VF_OUTPUT *out, VF_CURVE_INPUT_PARAM *curve)
{
    switch (vfCurveSel)
    {
        case VF_BEELINE:
        out->vfOutputVoltPu = Vf_BeelineCurve(in->vfCtrlFrqCurValPu, currentMaxFreqPu, curve);
        break;

        case VF_SQURELINE:
        out->vfOutputVoltPu = Vf_SqureCurve(in->vfCtrlFrqCurValPu, currentMaxFreqPu, curve);
        break;

        case VF_USERMANUAL:
        out->vfOutputVoltPu = Vf_CustomerCurve(in->vfCtrlFrqCurValPu, currentMaxFreqPu, curve);
        break;

        default:
        break;
    }
}

/*!
* @brief Vf control calculation of electric angle.
*
* @param[in] freqPu: cueernt frequency command value
* @param[in] freqBase: frequency base value
* @param[in] operatingFreq: execution frequency of the function
* @param[in] ang: pointer to VF_THETA_CAL structure
* @return none
*/
void Vf_ElecThetaIntegCalc(float freqPu, float freqBase, uint16_t operatingFreq, VF_THETA_CAL *ang)
{
    int16_t motorDir = 0U;

    float deltaTheta = 0.0f;
    float freqAbs = 0.0f;

    freqAbs = MathFloat_Abs(freqPu);
    if (freqPu >= 0.0f)
    {
        motorDir = 1;
    }
    else
    {
        motorDir = -1;
    }

    deltaTheta = MathFloat_Div((freqAbs * freqBase), (operatingFreq * PI2));

    if (1 == motorDir)
    {
        ang->elecTheta += deltaTheta;
    }
    else if (-1 == motorDir)
    {
        ang->elecTheta -= deltaTheta;
    }
    else
    {
    }
}

/*!
* @brief Vf control calculate SVPWM voltage component.
*
* @param[in] in: pointer to VF_OUTPUT structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Vf_CalVoltVect(VF_OUTPUT *in, FOC_VARS_CTRL *focVars)
{
    float pwmAlfaPu = 0;
    float pwmBetaPu = 0;

    pwmAlfaPu = MathFloat_Mpy(in->vfOutputVoltPu, MathFloat_Cos(in->phaseAngleUa));
    pwmBetaPu = MathFloat_Mpy(in->vfOutputVoltPu, MathFloat_Sin(in->phaseAngleUa));

    pwmAlfaPu = MathFloat_Div(pwmAlfaPu, g_adcTransform.vdcPu);
    pwmBetaPu = MathFloat_Div(pwmBetaPu, g_adcTransform.vdcPu);

    focVars->valpha = MathFloat_AutoSat(pwmAlfaPu);
    focVars->vbeta = MathFloat_AutoSat(pwmBetaPu);
}

/*!
* @brief Vf control dc startup.
*
* @param[in] dc: pointer to VF_DC_STARTUP structure
* @param[in] out: pointer to VF_OUTPUT structure
* @return none
*/
void Vf_DCStartUp(VF_DC_STARTUP *dc, VF_OUTPUT *out)
{
    if (START == Get_McStatus())
    {
        dc->vfStartUpStatus = INIT_UNKNOWN;
        dc->vfStartUpCnt++;
        out->vfOutputVoltPu = MathFloat_Mpy(VF_DC_VOLTAGE, g_vfCurveInputParam.motorVoltRatedPu);
        out->phaseAngleUa = 0;
        g_vfThetaCal.elecTheta = 0;
        if (VF_DC_TIME == dc->vfStartUpCnt)
        {
            dc->vfStartUpCnt = 0;
            dc->vfStartUpStatus = EXE_SUCCESS;
        }
        else
        {
        }
    }
    else
    {

    }
}

/*!
* @brief Vf control curve parameters initialization.
*
* @param[in] curve: pointer to VF_CURVE_INPUT_PARAM structure
* @return none
*/
void Vf_CurveInputParamInit(VF_CURVE_INPUT_PARAM *curve)
{
    curve->motorFreqRatedPu = VF_MOTOR_RATED_FREQUENCY;
    /* Udc */
    curve->motorVoltRatedPu = 1.0f;
    /* max frequency */
    curve->motorMaxFreqPu = VF_MOTOR_RATED_FREQUENCY;

    curve->torqBoostCutoffFreqPu = VF_TORQUEBOOST_CUTOFF_FREQ;
    curve->torqBoostVoltPct = VF_TORQUEBOOST_VOLTAGE_PERCENTAGE;

    curve->vfManualPoint1FreqPu = MathFloat_Mpy(VF_CURVE_POINT1_FREQ, curve->motorFreqRatedPu);
    curve->vfManualPoint1VoltPct = VF_CURVE_POINT1_VOLT_PERCENTAGE;
    curve->vfManualPoint2FreqPu = MathFloat_Mpy(VF_CURVE_POINT2_FREQ, curve->motorFreqRatedPu);
    curve->vfManualPoint2VoltPct = VF_CURVE_POINT2_VOLT_PERCENTAGE;
    curve->vfManualPoint3FreqPu = MathFloat_Mpy(VF_CURVE_POINT3_FREQ, curve->motorFreqRatedPu);
    curve->vfManualPoint3VoltPct = VF_CURVE_POINT3_VOLT_PERCENTAGE;
    if (curve->vfManualPoint2FreqPu < curve->vfManualPoint1FreqPu)
    {
        curve->vfManualPoint2FreqPu = curve->vfManualPoint1FreqPu;
    }
    if (curve->vfManualPoint3FreqPu < curve->vfManualPoint2FreqPu)
    {
        curve->vfManualPoint3FreqPu = curve->vfManualPoint2FreqPu;
    }
}

/*!
* @brief Vf control filter coefficient initialization.
*
* @param[in] currentLpfCutoffFreq: current low pass filter cutoff frequency
* @param[in] activePowerLpfCutoffFreq: power low pass filter cutoff frequency
* @param[in] reactPowerLpfCutoffFreq: reactive power low pass filter cutoff frequency
* @param[in] fSample: current sampling frequency
* @param[in] param: pointer to VF_COMP_PARAM structure
* @return none
*/
void Vf_FunctionParamInit(float currentLpfCutoffFreq, float activePowerLpfCutoffFreq, float reactPowerLpfCutoffFreq, float fSample, VF_COMP_PARAM *param)
{
    float temp_coeff1;
    float temp_coeff2;
    float cutoffFreq;

    cutoffFreq = currentLpfCutoffFreq;
    temp_coeff1 = PI2 * cutoffFreq;
    temp_coeff2 = temp_coeff1 + fSample;
    param->vfSuppLPFCoeff = temp_coeff1 / temp_coeff2;

    cutoffFreq = activePowerLpfCutoffFreq;
    temp_coeff1 = PI2 * cutoffFreq;
    temp_coeff2 = temp_coeff1 + fSample;
    param->powerSuppLPFCoeff = temp_coeff1 / temp_coeff2;

    cutoffFreq = reactPowerLpfCutoffFreq;
    temp_coeff1 = PI2 * cutoffFreq;
    temp_coeff2 = temp_coeff1 + fSample;
    param->vfPowerLPFCoeff = temp_coeff1 / temp_coeff2;
}

/*!
* @brief Vf control frequency compensation parameters initialization.
*
* @param[in] set: pointer to VF_COMP_SET structure
* @return none
*/
void Vf_CompParamInit(VF_COMP_SET *set)
{
    set->curResonaFreqLimit = (0.1f / BASE_FREQ);
    set->curResonaFreqPoint1 = (6.0f / BASE_FREQ);
    set->curResonaFreqPoint2 = (10.0f / BASE_FREQ);
    set->curResonaFreqPoint3 = (45.0f / BASE_FREQ);
    set->curResonaFreqPoint4 = (48.0f / BASE_FREQ);
    set->curResonaGain = 1.0f;

    set->powerSuppFreqLimit = (0.1f / BASE_FREQ);
    set->powerSuppFreqPoint1 = (6.0f / BASE_FREQ);
    set->powerSuppFreqPoint2 = (10.0f / BASE_FREQ);
    set->powerSuppFreqPoint3 = (45.0f / BASE_FREQ);
    set->powerSuppFreqPoint4 = (48.0f / BASE_FREQ);
    set->powerSuppGain = 1.0f;
}

/*!
* @brief Vf control frequency hopping parameters initialization.
*
* @param[in] param: pointer to VF_FREQ_HOP_PARAM structure
* @return none
*/
void Vf_FreqHopParamInit(VF_FREQ_HOP_PARAM *param)
{
    param->freqHopP1Bottom = HOP_FREQ_POINT1 - (HOP_WIDTH_1 * 0.5f);
    param->freqHopP1Top = HOP_FREQ_POINT1 + (HOP_WIDTH_1 * 0.5f);
    param->freqHopP2Bottom = HOP_FREQ_POINT2 - (HOP_WIDTH_2 * 0.5f);
    param->freqHopP2Top = HOP_FREQ_POINT2 + (HOP_WIDTH_2 * 0.5f);
    param->freqHopP3Bottom = HOP_FREQ_POINT3 - (HOP_WIDTH_3 * 0.5f);
    param->freqHopP3Top = HOP_FREQ_POINT3 + (HOP_WIDTH_3 * 0.5f);
    param->freqHopFlag = 0;
    param->speedStatus = STABLE;
    param->freqHopOutput = 0;
}

/*!
* @brief Vf control Parameters initialization.
*
* @param[in] none
* @return none
*/
void Vf_ParamInit(void)
{
    float currentLpfCutoffFreq = 0.0f;
    float activePowerLpfCutoffFreq = 0.0f;
    float reactPowerLpfCutoffFreq = 0.0f;

    STC_ParamInit(&g_STC);
    FOC_CtrlVarsInit(&g_focVarsCtrl);
    Clear_CurrentSample(&g_adcTransform);

    memset(&g_pwmValue, 0, sizeof(g_pwmValue));
    memset(&g_curRestruct, 0, sizeof(g_curRestruct));

    memset(&g_vfDcStartUp, 0, sizeof(g_vfDcStartUp));
    memset(&g_vfInput, 0, sizeof(g_vfInput));
    memset(&g_vfOutput, 0, sizeof(g_vfOutput));
    memset(&g_vfCompParam, 0, sizeof(g_vfCompParam));
    memset(&g_vfVect, 0, sizeof(g_vfVect));
    memset(&g_vfThetaCal, 0, sizeof(g_vfThetaCal));

    memset(&g_vfFreqClac, 0, sizeof(g_vfFreqClac));
    /* Motor protection */
    Init_Protection();

    g_speedCommand.speedTarget = VF_DEFAULT_TARGET_FREQ;

    Vf_CurveInputParamInit(&g_vfCurveInputParam);
    currentLpfCutoffFreq = (10.0f / BASE_FREQ);
    activePowerLpfCutoffFreq = (10.0f / BASE_FREQ);
    reactPowerLpfCutoffFreq = (10.0f / BASE_FREQ);
    Vf_FunctionParamInit(currentLpfCutoffFreq, activePowerLpfCutoffFreq, reactPowerLpfCutoffFreq, FOC_FREQ, &g_vfCompParam);
    Vf_CompParamInit(&g_vfCompSet);
    Vf_FreqHopParamInit(&g_vfFreqHopParam);
}

/*!
* @brief Vf control synchronous frequency calculation.
*
* @param[in] in: pointer to VF_FREQ_CALC_BUF structure
* @return none
*/
void Vf_SynFreqCalc(VF_FREQ_CALC_BUF *in)
{
    static uint16_t s_freqCmdCnt = 0;

    if (in->vfCtrlSpeedSetPu > VF_CURVE_MAX_FREQUENCY)
    {
        in->vfCtrlSpeedSetPu = VF_CURVE_MAX_FREQUENCY;
    }
    else if (in->vfCtrlSpeedSetPu < 0 - VF_CURVE_MAX_FREQUENCY)
    {
        in->vfCtrlSpeedSetPu = 0 - VF_CURVE_MAX_FREQUENCY;
    }
    else
    {

    }
    s_freqCmdCnt++;
    if (s_freqCmdCnt > 100)
    {
        s_freqCmdCnt = 0;
        if (1 == pFocVarsCfg->motorDir)
        {
            if ((in->vfCtrlSpeedSetPu - in->vfCtrlFrqCurValPu) > defFocVars.motorAcceleration)
            {
                in->vfCtrlFrqCurValPu += defFocVars.motorAcceleration;
            }
            else if ((in->vfCtrlFrqCurValPu - in->vfCtrlSpeedSetPu) > defFocVars.motorDeceleration)
            {
                in->vfCtrlFrqCurValPu -= defFocVars.motorDeceleration;
            }
            else
            {
                in->vfCtrlFrqCurValPu = in->vfCtrlSpeedSetPu;
            }
        }
        else if (-1 == pFocVarsCfg->motorDir)
        {
            if ((in->vfCtrlFrqCurValPu - in->vfCtrlSpeedSetPu) > defFocVars.motorAcceleration)
            {
                in->vfCtrlFrqCurValPu -= defFocVars.motorAcceleration;
            }
            else if ((in->vfCtrlSpeedSetPu - in->vfCtrlFrqCurValPu) > defFocVars.motorDeceleration)
            {
                in->vfCtrlFrqCurValPu += defFocVars.motorDeceleration;
            }
            else
            {
                in->vfCtrlFrqCurValPu = in->vfCtrlSpeedSetPu;
            }
        }
        else
        {
        }
    }
}

/*!
* @brief Vf control main functions.
*
* @param[in] none
* @return none
*/
void Vf_MainProcess(void)
{
    /* get current frequency command */
    g_vfInput.vfCtrlSpeedSetPu = g_speedCommand.speedTarget;
    g_vfFreqClac.vfCtrlSpeedSetPu = g_vfInput.vfCtrlSpeedSetPu;
    /* calculate real-time frequency value */
    Vf_SynFreqCalc(&g_vfFreqClac);
    /* hopFreq */
    #if (defined FREQ_HOPPING_ENABLE)
    /* calculate motion direction */
    RampDirJudgeProcess(g_vfInput.vfCtrlSpeedSetPu, g_vfFreqClac.vfCtrlFrqCurValPu, defFocVars.motorAcceleration, defFocVars.motorDeceleration, &g_vfFreqHopParam);
    /* frequency hopping */
    FrequencyHoppingProcess(&g_vfFreqClac, &g_vfFreqHopParam);
    g_vfInput.vfCtrlFrqCurValPu = g_vfFreqHopParam.freqHopOutput;
    #else
    g_vfInput.vfCtrlFrqCurValPu = g_vfFreqClac.vfCtrlFrqCurValPu;
    #endif

    g_vfInput.vfCtrlFrqBeforeComp = g_vfInput.vfCtrlFrqCurValPu;

#if (defined VF_COMPENSATION)
    #if (defined VF_CURRENT_DAMP_ENABLE)
    /* frequency compensation with current */
    Vf_CurrentResonSuppProcess(g_vfVect.curQAxisPu, &g_vfInput, &g_vfCompParam, &g_vfCompSet);
    #else
    g_vfCompParam.vfSuppFrqComp = 0;
    #endif
    g_vfInput.vfCtrlFrqCompBuf = g_vfInput.vfCtrlFrqBeforeComp - g_vfCompParam.vfSuppFrqComp;

    #if (defined VF_ACTIVEPOWER_COMP_ENABLE)
    /* frequency compensation with power */
    Vf_ActivePowerCompProcess(&g_vfInput, &g_vfVect, &g_vfCompParam, &g_vfCompSet, g_focVars.pCfg->motorDir);
    #else
    g_vfCompParam.vfPowerSuppFreqComp = 0;
    #endif
    g_vfInput.vfCtrlFrqCompBuf = g_vfInput.vfCtrlFrqCompBuf + g_vfCompParam.vfPowerSuppFreqComp;
#else
    g_vfInput.vfCtrlFrqCompBuf = g_vfInput.vfCtrlFrqBeforeComp;
#endif
    g_vfInput.vfCtrlFrqAfterComp = g_vfInput.vfCtrlFrqCompBuf;
}

/*!
* @brief Vf control SateMachine main function.
*
* @param[in] none
* @return none
*/
void Vf_MainStateMachine(void)
{
    switch (Get_McStatus())
    {
    case POWER_ON:
    Stop_Motor();
    Vf_ParamInit();
    /* must behind adc init, check current sampling offset*/
    Current_Sensor_Check(&g_adcSampleReal);
    Set_McStatus(IDLE);
    break;

    case IDLE:
    Vf_ParamInit();
    break;

    case STOP:
    Stop_Motor();
    Vf_ParamInit();
    Set_McStatus(IDLE);
    break;

    case START_BRAKE:
    ShortBrake_RunCaseBrake(&g_shortBrake, pShortBrakeCfg, pFocVarsCfg);
    if (CONFIG_RESET == g_shortBrake.subStatus)
    {
        PWM_Disable();
        ShortBrake_Init(&g_shortBrake);
        Set_McStatus(START);
    }
    break;

    case START:
    PWM_Enable();
    g_focVarsCtrl.pwmOutMask = SVPWM_OUTPUT_ENABLE;
    Vf_DCStartUp(&g_vfDcStartUp, &g_vfOutput);
    if (EXE_SUCCESS == g_vfDcStartUp.vfStartUpStatus)
    {
        Set_McStatus(RUN);
    }
    break;

    case RUN:
    Vf_MainProcess();
    break;

    case SLOW_DOWN:
    g_speedCommand.speedTarget = 0;
    Vf_MainProcess();
    if (MathFloat_Abs(g_vfOutput.synFrq) < VF_STOP_FREQ)
    {
        Set_McStatus(STOP);
        PWM_Disable();
    }
    break;

    /* Stop brake, Short circuit brake */
    case STOP_BRAKE:
    ShortBrake_RunCaseBrake(&g_shortBrake, pShortBrakeCfg, pFocVarsCfg);
    if (CONFIG_RESET == g_shortBrake.subStatus)
    {
        PWM_Disable();
        ShortBrake_Init(&g_shortBrake);
        Set_McStatus(STOP);
    }
    break;

    case FAULT:
    Stop_Motor();
    break;

    default:
    break;
    }
}

/*!
* @brief Vf control ISR process.
*
* @param[in] none
* @return none
*/
void Vf_ISR(void)
{
    if (SVPWM_OUTPUT_ENABLE == g_focVarsCtrl.pwmOutMask)
    {
        /* get synchronous frequency */
        g_vfOutput.synFrq = g_vfInput.vfCtrlFrqAfterComp;

        /* get voltage vector angle */
        if ((RUN == Get_McStatus()) || (SLOW_DOWN == Get_McStatus()))
        {
            Vf_ElecThetaIntegCalc(g_vfOutput.synFrq, BASE_FREQ, FOC_FREQ, &g_vfThetaCal);
            g_vfOutput.phaseAngleUa = g_vfThetaCal.elecTheta;
        }

        /* get voltage output */
        Vf_CharacteristicCurve(VF_CURVE, VF_CURVE_MAX_FREQUENCY, &g_vfInput, &g_vfOutput, &g_vfCurveInputParam);

        /* electric angle compensation */
        #if (defined VF_COMPENSATION)
        Vf_VectorTransformProcess(&g_focVarsCtrl, &g_vfOutput, &g_vfVect);
        #else

        #endif

        /* calculate ualf and ubeta */
        Vf_CalVoltVect(&g_vfOutput, &g_focVarsCtrl);

        /* svpwm */
        Svpwm_Process();
        LoadToPwmModule();
    }
}

/*!
* @brief Vf control protector.
*
* @param[in] none
* @return none
*/
void VF_MotorProtector(void)
{
#if (defined PHASE_CURRENT_PROTECTION_ENABLE)
    /* Phase over current protection, 1ms */
    Phase_Over_Current_Check(&g_focVarsCtrl);
#endif

#if (defined VBUS_UNDERVOLTAGE_PROTECTION_ENABLE)
    /* DC Bus under voltage protection, 1ms */
    Vbus_UnderVoltage_Check(g_adcTransform.vdcVt);
#endif

#if (defined VBUS_OVERVOLTAGE_PROTECTION_ENABLE)
    /* DC Bus over voltage protection, 1ms */
    Vbus_OverVoltage_Check(g_adcTransform.vdcVt);
#endif
}

/*!
* @brief Vf control 1ms time base tasks.
*
* @param[in] none
* @return none
*/
void VF_1ms_Task(void)
{
    Vf_MainStateMachine();

    Get_VoltageSample(&g_adcSampleCof, &g_motorParamPu, &g_adcTransform);
    /* Start adc convert */
    ADC_SoftwareStartRegularConvert(ADC0);
    VF_MotorProtector();
}

/*!
* @brief Vf control 2ms time base tasks.
*
* @param[in] none
* @return none
*/
void VF_2ms_Task(void)
{
    #if (defined TOOL_UART_FUNCTION)
    /* Motor Debug Tool command Process */
    UI_UART_ToolCmd_Proc(&g_uiRxBuf);
    UI_UART_TimeOut_Cnt();
    UI_MasterCmd_ProcCnt();
    #endif
}

/* =============================================  EOF  ============================================== */
