/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __AUTO_CUR_REGULATOR_H__
#define __AUTO_CUR_REGULATOR_H__
/*!
* @file autocur_regulator.h
*
* @brief This file provides current loop regulator parameters and decoupling voltage calculate functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pid_regulator.h"
#include "param_init.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Current loop coefficient calculation input parameter structure.
*/
typedef struct
{
    float curLPu;                    /*!< Motor inductance pu value, it is Ld for d_ACR, and Lq for q_ACR */
    float curRsPu;                   /*!< Motor stator resistance pu value in current loop coefficient calculation */
    float curTCtrPu;                 /*!< Current loop control cycle */
    float curWicPu;                  /*!< Current loop frequency bandwidth */
} ACR_CURPI_COFIN;

/*!
* @brief Voltage decoupling calculation input parameter structure.
*/
typedef struct
{
    float dcpWsPu;                    /*!< Motor stator electric frequency pu value */
    float dcpIdRefPu;                 /*!< Reference value of d-axis current pu value */
    float dcpIqRefPu;                 /*!< Reference value of d-axis current pu value */
    float dcpULimPu;                  /*!< Limit the calculated value of decoupling voltage */
} ACR_UDQDCP_IN;

/*!
* @brief Voltage decoupling calculation output structure.
*/
typedef struct
{
    float dcpUdPu;                    /*!< D-axis decoupling voltage pu value */
    float dcpUqPu;                    /*!< Q-axis decoupling voltage pu value */
} ACR_UDQDCP_OUT;

/*!
* @brief Voltage decoupling calculation coefficient structure.
*/
typedef struct
{
    float dcpLdPu;                   /*!< Motor d-axis inductance */
    float dcpLqPu;                   /*!< Motor q-axis inductance */
    float dcpFluxFPu;                /*!< Motor flux */
} ACR_UDQDCP_COF;

/*!
* @brief Load torque estimate control parameters structure.
*/
typedef struct
{
    float machFreq;                   /*!< Motor machine frequence */
    float machFreqOld;                /*!< Motor last period machine frequence */
    float estMachFreq;                /*!< Estimated machine frequence pu value */
    float iqCompPu;                   /*!< Q axis current compensation pu value */
    float elecTorque;                 /*!< Motor electric torque */
    float estTorque;                  /*!< Estimated load torque pu value */
    float freqErr;                    /*!< Motor machine frequence error */
    float coeff1;                     /*!< Motor machine frequence gain coefficient1 */
    float coeff2;                     /*!< Load torque gain coefficient2 */
} TRQ_LOAD_EST;

/*!
* @brief Load torque estimate configuration parameters structure.
*/
typedef struct
{
    float iqCompLimit;                /*!< Q axis current compensation limit value */
    float loadEstCoeff;               /*!< Load torque estimate gain coefficient */
} LOAD_ESTIMATE_CFG;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Set the parameters of d-axis current loop PI regulator, includes proportion and integral coefficient.
*
* @param[in] kp: Proportional parameter
* @param[in] ki: Integral parameter
* @param[in] kc: Reserved parameters
* @param[in] kd: Differential parameter, reserved
* @param[in] dAcrCof: pointer to PID_REGULATOR_COF structure
* @return none
*/
void Acr_SetCurD_PICof(float kp, float ki, float kc, float kd, PID_REGULATOR_COF *dAcrCof);

/*!
* @brief Set the parameters of q-axis current loop PI regulator, includes proportion and integral coefficient.
*
* @param[in] kp: Proportional parameter
* @param[in] ki: Integral parameter
* @param[in] kc: Reserved parameters
* @param[in] kd: Differential parameter
* @param[in] dAcrCof: pointer to PID_REGULATOR_COF structure
* @return none
*/
void Acr_SetCurQ_PICof(float kp, float ki, float kc, float kd, PID_REGULATOR_COF *qAcrCof);

/*!
* @brief Coefficient calculation for current loop PI regulator. Kp and ki are calculated
*        by motor parameters and loop bandwidth parameters.
*
* @param[in] in: pointer to ACR_CURPI_COFIN structure
* @param[in] out: pointer to PID_REGULATOR_COF structure
* @return none
*/
void Acr_CurPICofCalc(ACR_CURPI_COFIN *in, PID_REGULATOR_COF *out);

/*!
* @brief Feedforward decoupled. Based on the motor voltage equation,
*        the voltage control of two current components is independently controlled
*        and the decoupling voltage dcpUdPu and dcpUqPu are calculated.
*
* @param[in] in: pointer to ACR_UDQDCP_IN structure
* @param[in] cof: pointer to ACR_UDQDCP_COF structure
* @param[in] out: pointer to ACR_UDQDCP_OUT structure
* @return none
*/
void Acr_UdqDcpCalc(ACR_UDQDCP_IN *in, ACR_UDQDCP_COF *cof, ACR_UDQDCP_OUT *out);

/*!
* @brief Current loop q-axis component output limit, set the maximun and minimun limit of Q_ACR.
*
* @param[in] max: current max pu value
* @param[in] min: current min pu value
* @param[in] qAcrIn: pointer to PID_REGULATOR_IN structure
* @return none
*/
void Acr_SetCurQ_Limit(float max, float min, PID_REGULATOR_IN *qAcrIn);


/*!
* @brief Current loop d-axis component output limit, calcutate the maximun and minimun limit of D_ACR.
*
* @param[in] max: current max pu value
* @param[in] min: current min pu value
* @param[in] dAcrIn: pointer to PID_REGULATOR_IN structure
* @return none
*/
void Acr_SetCurD_Limit(float max, float min, PID_REGULATOR_IN *dAcrIn);

/*!
* @brief Load torque estimate parameters initialize.
*
* @param[in] loadTorque: pointer to TRQ_LOAD_EST structure
* @param[in] loadTorqueCfg: pointer to LOAD_ESTIMATE_CFG structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @return none
*/
void TrqLoadEst_Init(TRQ_LOAD_EST *loadTorque, LOAD_ESTIMATE_CFG *loadTorqueCfg, MOTOR_PARAM_PU *paramPu);

/*!
* @brief Load torque estimate calculation function, calculate the load torque through the load torque observer,
* and feed forward compensation to the current loop to increase the anti-interference ability of the system.
*
* @param[in] loadTorque: pointer to TRQ_LOAD_EST structure
* @param[in] loadTorqueCfg: pointer to LOAD_ESTIMATE_CFG structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] elecFreq: motor electric frequence pu value
* @param[in] iqFdb: q axis current feedback pu value
* @return none
*/
void TrqLoadEst_Calc(TRQ_LOAD_EST *loadTorque, LOAD_ESTIMATE_CFG *loadTorqueCfg, MOTOR_PARAM_PU *paramPu, float elecFreq, float iqFdb);

/*!
* @brief When motor run at troque mode, torque command is set outside.
*        This function base on troque command set ASR upper and lower limitation.
*        Troque mode limits ASR output, speed mode limit ACR output.
*
* @param[in] troqueCmd: input troque command
* @param[in] asrSpeedPIIn: pointer to PID_REGULATOR_IN structure
* @return none
*/
void TroqueCmdSet(float troqueCmd, PID_REGULATOR_IN *asrSpeedPIIn);

/* ======================================  Functions define  ======================================== */

#endif /* __AUTO_CUR_REGULATOR_H__ */

/* =============================================  EOF  ============================================== */
