/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __AUTO_SPD_REGULATOR_H__
#define __AUTO_SPD_REGULATOR_H__
/*!
* @file autospd_regulator.h
*
* @brief This file provides speed loop regulator parameters and estimated speed calculate functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "param_init.h"
#include "autocur_regulator.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Speed loop coefficient calculation input parameter structure.
*/
typedef struct
{
    float jmPu;              /*!< Motor inertia value (pu) */
    float dampPu;            /*!< Motor damping pu value */
    float fluxPu;            /*!< Motor rotor flux pu value */
    float speedTCntCtrPu;    /*!< Speed loop control cycle */
    float speedWicPu;        /*!< Speed loop frequency bandwidth */
} ASR_SPEEDPI_COFIN;

/*!
* @brief Speed PLL calculation coefficient structure.
*/
typedef struct
{
    float coeffPll;           /*!< Q15, pll PI set coefficient */
} PLL_CFG;

/*!
* @brief Estimated speed calculation structure.
*/
typedef struct
{
    float estTheta;         /*!< Electrical angle estimated value (pu) */
    float oldEstTheta;      /*!< Electrical angle estimated value at previous step (pu) */
    float estSpeed;         /*!< Estimated speed in per-unit (pu) */
    float deltaTheta;       /*!< Electrical angle estimated value change at a step (pu) */
    float estSpeedRpm;      /*!< Estimated speed in unit: rpm, it is the estimated speed real value */
    float speedK1;          /*!< Coefficient: calculate speed Coefficient K1 */
    float speedK2;          /*!< Coefficient: calculate speed Coefficient K2 */
    float speedK3;          /*!< Coefficient: calculate speed Coefficient K3*/
} ESTIMATED_SPEED;

/*!
* @brief Initial speed judge function data struct, befor start motor, judge motor initial speed
*/
typedef struct
{
    uint16_t judgeReadyFlag;        /*!< speed judge finished flag */
    int16_t overFlowFlag;           /*!< Electrical angle over flow flag */
    int32_t overFlowCnt;            /*!< Electrical angle over flow count */
    int32_t firstOverFlowCnt;       /*!< Electrical angle over flow count */
    int32_t pwmBaseFreqRatio;       /*!< Pwm freq and base freq ratio */
    float iabcAmp;               /*!< Phase current amplitude in initial speed judgment stage */
    float bemfTheta;             /*!< Electrical angle estimated value by back emf (pu) */
    float bemfThetaOld;          /*!< Last peroid electrical angle estimated value by back emf (pu) */
    float initSpeed;              /*!< Speed value in initial speed judgment stage */
    float bemfAlpha;              /*!< Alpha axis back emf */
    float bemfBeta;               /*!< Beta axis back emf */
    float bemfAmp;                /*!< Back emf amplitude */
    float bemfThetaErr;           /*!< Electrical angle error value by back emf (pu) */
} INITSPEED_JUDGE;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Calculated estimate speed pu value and stores estimated rotation speed to FIFO.
*
* @param[in] electricAngle: electric Angle
* @param[in] speedCalc: pointer to ESTIMATED_SPEED structure
* @return none
*/
void Asr_EstSpeedCalc(float electricAngle, ESTIMATED_SPEED *speedCalc);

/*!
* @brief Get the estimated feedback speed unit to rpm.
*
* @param[in] speedCalc: pointer to ESTIMATED_SPEED structure
* @return estSpeedRpm: estimated speed value in rpm unit
*/
float Asr_GetSpeedRpm(ESTIMATED_SPEED *speedCalc);

/*!
* @brief Get the estimated feedback speed value unit to pu.
*
* @param[in] speedCalc: pointer to ESTIMATED_SPEED structure
* @return estSpeed: estimated speed pu value
*/
float Asr_GetSpeedPu(ESTIMATED_SPEED *speedCalc);

/*!
* @brief Convert the speed unit from RPM to PU.
*
* @param[in] speedcmd: Speed command value unit RPM
* @param[in] speedFactor: Speed pu calculation factor
* @return speedPu: speed pu value
*/
float Asr_RpmSpeedToPu(float speedcmd, float speedFactor);

/*!
* @brief Set the parameters of speed loop PI regulator, includes proportion and integral coefficient.
*
* @param[in] kp: Proportional parameter
* @param[in] ki: Integral parameter
* @param[in] kc: Reserved parameters
* @param[in] kd: Differential parameter, reserved
* @param[in] speedCof: pointer to PID_REGULATOR_COF structure
* @return none
*/
void Asr_SetSpeed_PICof(float kp, float ki, float kc, float kd, PID_REGULATOR_COF *speedCof);

/*!
* @brief Coefficient calculation for speed loop PI regulator using loop control parameters
*        and motor parameters:speedWicPu, dampPu, speedTCntCtrPu, web, fluxPu , contains the
*        proportional coefficient kpPu and integral coefficient kiPu.
*
* @param[in] in: pointer to ASR_SPEEDPI_COFIN structure
* @param[in] motorParam: pointer to MOTOR_PARAM_PU structure
* @param[in] out: pointer to PID_REGULATOR_COF structure
* @return none
*/
void Asr_SpeedPICofCalc(ASR_SPEEDPI_COFIN *in, MOTOR_PARAM_PU *motorParam, PID_REGULATOR_COF *out);

/*!
* @brief Speed loop output limit, set the maximun and minimun limit of ASR.
*
* @param[in] max: Speed PID out max pu value
* @param[in] min: Speed PID out min pu value
* @param[in] speedPIIn: pointer to PID_REGULATOR_IN structure
* @return none
*/
void Asr_SpeedPID_Limit(float max, float min, PID_REGULATOR_IN *speedPIIn);

/*!
* @brief Calculate speed PID integral term max and min limit, limit uiPu from min to max.
*
* @param[in] max: Speed PID integral term max pu value
* @param[in] min: Speed PID integral term min pu value
* @param[in] speedPIOut: pointer to PID_REGULATOR_OUT structure
* @return none
*/
void Asr_SpeedPIDIntegral_Limit(float max, float min ,PID_REGULATOR_OUT *speedPIOut);

/*!
* @brief Set speed PID integral term uiPu to setting value.
*
* @param[in] speedPIOut: pointer to PID_REGULATOR_OUT structure
* @param[in] uiPu: Speed PID integral term setting value
* @return none
*/
void Asr_SetSpeedPID_Integral(PID_REGULATOR_OUT *speedPIOut, float uiPu);

/*!
* @brief The function is called in SPEED_JUDGE state to calculate the initial speed data, enter the FLY_RUN state after completion.
*
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @param[in] focVars: pointer to FOC_VARS structure
* @return none.
*/
void Speed_Judge_Calc(INITSPEED_JUDGE *judgeHandle, FOC_VARS *focVars);

/*!
* @brief Get the initial speed judgment status, 0 means not complete, 1 means complete.
*
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @return judgeReadyFlag: Initial speed judgment status, 0 means incomplete, 1 means complete.
*/
int16_t Get_InitSpeed_Judge_State(INITSPEED_JUDGE *judgeHandle);

/*!
* @brief Clear the initial speed judgment status to incomplete.
*
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @return none.
*/
void Clear_InitSpeed_Judge_State(INITSPEED_JUDGE *judgeHandle);

/*!
* @brief Get the speed in SPEED_JUDGE state.
*
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @return initSpeed: Initial speed.
*/
float Get_InitSpeed_Judge_Speed(INITSPEED_JUDGE *judgeHandle);

/*!
* @brief Get the back emf amplitude in SPEED_JUDGE state.
*
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @return bemfAmp: Back emf amplitude.
*/
float Get_InitSpeed_Judge_BemfAmp(INITSPEED_JUDGE *judgeHandle);

/* ======================================  Functions define  ======================================== */

#endif /* __AUTO_SPD_REGULATOR_H__ */

/* =============================================  EOF  ============================================== */
