/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __COORDNT_H__
#define __COORDNT_H__

/*!
* @file coordnt.h
*
* @brief This file provides coordinate transformation functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "iqmath.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Coordinate transformation angle sine and cosine variables structure.
*/
typedef struct
{
    float sinPu;          /*!< Sine value (pu), Q15 */
    float cosPu;          /*!< Cosine value (pu), Q15 */
} SINCOS;

/*!
* @brief Three phases of Clark transform input structure.
*/
typedef struct
{
    float clarkAPu;       /*!< ABC to alfa_beta clark transform input: A */
    float clarkBPu;       /*!< ABC to alfa_beta clark transform input: B */
    float clarkCPu;       /*!< ABC to alfa_beta clark transform input: C */
} CRD_CLARK_IN;

/*!
* @brief Three phases of Clark transform output structure.
*/
typedef struct
{
    float clarkAlfaPu;    /*!< ABC to alfa_beta clark transform output: Alfa */
    float clarkBetaPu;    /*!< ABC to alfa_beta clark transform output: Beta */
} CRD_CLARK_OUT;

/*!
* @brief Three phases of Park transform input structure.
*/
typedef struct
{
    float parkAlfaPu;     /*!< Alfa_beta to DQ park transform input: Alfa */
    float parkBetaPu;     /*!< Alfa_beta to DQ park transform input: Beta */
    SINCOS  parkSINCOS;     /*!< Alfa_beta to DQ park transform input: vector angle sin and cos value */
} CRD_PARK_IN;

/*!
* @brief Three phases of Park transform output structure.
*/
typedef struct
{
    float parkdPu;        /*!< Alfa_beta to DQ park transform output: d-axis component */
    float parkqPu;        /*!< Alfa_beta to DQ park transform output: q-axis component */
} CRD_PARK_OUT;

/*!
* @brief Three phases of inverse Park transform input structure.
*/
typedef struct
{
    float iParkdPu;       /*!< DQ to alfa_beta i-park transform input: d-axis component */
    float iParkqPu;       /*!< DQ to alfa_beta i-park transform input: q-axis component */
    SINCOS  iParkSINCOS;    /*!< DQ to alfa_beta i-park transform input: vector angle sin and cos value */
} CRD_IPARK_IN;

/*!
* @brief Three phases of inverse Park transform output structure.
*/
typedef struct
{
    float iparkAlfaPu;    /*!< DQ to alfa_beta i-park transform output: Alfa */
    float iparkBetaPu;    /*!< DQ to alfa_beta i-park transform output: Beta */
} CRD_IPARK_OUT;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Clark transform: ABC to alfa_beta transformation.
*        alfa = a
*        beta = (2*b+a)/sqrt(3)
*
* @param[in] in: pointer to CRD_CLARK_IN structure
* @param[in] out: pointer to CRD_CLARK_OUT structure
* @return none
*/
void Crd_Clark(CRD_CLARK_IN *in, CRD_CLARK_OUT *out);

/*!
* @brief Park transform: alfa_beta to d-q transformation.
*        d = alfa*cos + beta*sin
*        q = -alfa*sin + beta*cos
*
* @param[in] in: pointer to CRD_PARK_IN structure
* @param[in] out: pointer to CRD_PARK_OUT structure
* @return none
*/
void Crd_Park(CRD_PARK_IN *in, CRD_PARK_OUT *out);

/*!
* @brief Inverse Park transform: d-q to alfa_beta transformation.
*        alfa = d*cos - q*sin
*        beta = d*sin + q*cos
*
* @param[in] in: pointer to CRD_IPARK_IN structure
* @param[in] out: pointer to CRD_IPARK_OUT structure
* @return none
*/
void Crd_IPark(CRD_IPARK_IN *in, CRD_IPARK_OUT *out);

/* ======================================  Functions define  ======================================== */

#endif /* __COORDNT_H__ */

/* =============================================  EOF  ============================================== */
