/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __ENERGY_SAVE_H__
#define __ENERGY_SAVE_H__
/*!
* @file energy_save.h
*
* @brief  This file provides energy save function interfaces. Energy save mainly adjust idRef to get the best working point.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pid_regulator.h"
#include "param_init.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Energy save function data struct, include main status, negtive adjust, active adjust and so on.
*/
typedef struct
{
    uint16_t enable;              /*!< Enable control (input) */
    uint16_t step;                /*!< Main step */
    uint16_t isValAddcnt;         /*!< Is value add counter */
    uint16_t validIdAddCnt;       /*!< Get valid Is value counter */
    uint16_t waitCnt;             /*!< Wait for current steady counter */
    uint16_t stateFlag;           /*!< Adjust direction */
    uint16_t changeAdjDirCnt;     /*!< Change adjust direction counter */
    float isValAver;           /*!< Is value averge */
    float isValAverOld;        /*!< Is value averge old */
    float isValAverOldReal;    /*!< Is value averge old real(for case2 ==> case3/4) */
    float isValAverOldBak;     /*!< Is value averge old back(for case3/4 ==> case2) */
    float  validIdAver;         /*!< Get valid Is value averge */
    float  idTarget;            /*!< Id given */
    float  idTargetStep;        /*!< Id given step (output) */
    float isValFbk;            /*!< Is value feedback */
    float isValTotal;          /*!< Is value total */
    float  validIdSum;          /*!< Get valid Is value sum */
} ENERGY_SAVE;

/*!
* @brief Energy save function configuration struct, according to different motor and workspace, user can adjust energy save configuration parameter.
*/
typedef struct
{
    uint16_t waitTime;         /*!< wait time */
    float maxCurChange;        /*!< max Current Change */
    float steadyCurChange;     /*!< steady Current Change */
    float curChangeStep;       /*!< current Change Step */
    float energySaveDec;       /*!< energy Save Decrease */
    float energySaveInc;       /*!< energy Save Increase */
    float recheckCurChange;    /*!< recheck Current Change */
    float maxIdTarget;          /*!< max id target */
    float minIdTarget;          /*!< min id target */
    float enterEnergySaveLoad; /*!< enter Energy Save function Load */
} ENERGY_SAVE_CFG;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Energy save function main state machine, include six status, execution every 1ms. Output idRef gradually at last.
*        0: Energy save data initialize
*        1: Get initial current vector amplitude
*        2: Check load is steady
*        3: Adjust at negtive direction
*        4: Adjust at active direction
*        5: Check whether load change
*
* @param[in] energySave: energySave variable, pointer to ENERGY_SAVE struct
* @param[in] energySaveCfg: variable which use to configure energy save, pointer to ENERGY_SAVE_CFG struct
* @return d-axis current reference
*/
float EnergySave(ENERGY_SAVE *energySave, ENERGY_SAVE_CFG *energySaveCfg);

/*!
* @brief This is energy save first status, first enter energy save and recheck will implement this part.
*        Energy save data struct initialize, include current vector amplitude,  id feedback and so on.
*
* @param[in] energySave: energySave variable, pointer to ENERGY_SAVE struct
* @return none
*/
void EnergySave_Init(ENERGY_SAVE *energySave);

/*!
* @brief  This is energy save second status, mainly get current vector amplitude average value.
*         Current vector amplitude average value can be used for staus 2, to judge load is steady or not.
*         This function will be called when enter energy save or the load is not steady.
*
* @param[in] energySave: energySave variable, pointer to ENERGY_SAVE struct
* @param[in] energySaveCfg: variable which use to configure energy save, pointer to ENERGY_SAVE_CFG struct
* @return none
*/
void EnergySave_InitIsVal(ENERGY_SAVE *energySave, ENERGY_SAVE_CFG *energySaveCfg);

/*!
* @brief  This is energy save third status, mainly  judge load is steady or not.
*         This function will be called when idRef adjusted, and then judge current vector amplitude change bigger or smaller.
*         If current vector amplitude gets bigger, then change adjust direction, else maintain the same adjust direction.
*
* @param[in] energySave: energySave variable, pointer to ENERGY_SAVE struct
* @param[in] energySaveCfg: variable which use to configure energy save, pointer to ENERGY_SAVE_CFG struct
* @return none
*/
void EnergySave_SteadyCheck(ENERGY_SAVE *energySave, ENERGY_SAVE_CFG *energySaveCfg);

/*!
* @brief  This is energy save fourth status, mainly  get the negtive direction adjust result.
*         This function will be called when idRef adjusted, and then judge current vector amplitude change bigger or smaller.
*         If current vector amplitude gets bigger, then change adjust direction, else maintain the same adjust direction.
*
* @param[in] energySave: energySave variable, pointer to ENERGY_SAVE struct
* @param[in] energySaveCfg: variable which use to configure energy save, pointer to ENERGY_SAVE_CFG struct
* @return none
*/
void EnergySave_NegtiveAdjust(ENERGY_SAVE *energySave, ENERGY_SAVE_CFG *energySaveCfg);

/*!
* @brief  This is energy save fifth status, mainly  get the active direction adjust result.
*         This function will be called when idRef adjusted, and then judge current vector amplitude change bigger or smaller.
*         If current vector amplitude gets bigger, then change adjust direction, else maintain the same adjust direction.
*
* @param[in] energySave: energySave variable, pointer to ENERGY_SAVE struct
* @param[in] energySaveCfg: variable which use to configure energy save, pointer to ENERGY_SAVE_CFG struct
* @return none
*/
void EnergySave_ActiveAdjust(ENERGY_SAVE *energySave, ENERGY_SAVE_CFG *energySaveCfg);

/*!
* @brief  This is energy save sixth status, energy save function finished.
*         Judge the load is changed or not. If the load changed, then enter energy save function again.
*         Energy save function can only fit to fixed motor load, if the load change, have to search proper idRef again.
*
* @param[in] energySave: energySave variable, pointer to ENERGY_SAVE struct
* @param[in] energySaveCfg: variable which use to configure energy save, pointer to ENERGY_SAVE_CFG struct
* @return none
*/
void EnergySave_AdjustFinished(ENERGY_SAVE *energySave, ENERGY_SAVE_CFG *energySaveCfg);

/*!
* @brief This function calculate current vector amplitude using Q15Mag function.
*        Because the load is always changing, the current vector amplitude will be filter, then output.
*
* @param[in] energySave: energySave variable, pointer to ENERGY_SAVE struct
* @param[in] focVars: FOC control variable, pointer to FOC_VARS_CTRL structure
* @return none
*/
void EnergySave_CalculateIsVal(ENERGY_SAVE *energySave, FOC_VARS_CTRL *focVars);

/* ======================================  Functions define  ======================================== */

#endif /* __ENERGY_SAVE_H__ */

/* =============================================  EOF  ============================================== */
