/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __FLUX_OBSERVER_H__
#define __FLUX_OBSERVER_H__
/*!
* @file flux_observer.h
*
* @brief This file provides the rotar angle and speed estimated by the flux observer for sensorless vector control functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "param_init.h"
#include "autospd_regulator.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Flux observer sensorless mode control parameters structure.
*/
typedef struct
{
    float estFreq;        /*!< Estimated electrical frequency by flux observer */
    float estTheta;      /*!< Estimated electrical theta by flux observer */
    float cutFreq;       /*!< Stator flux fliter cutoff frequency */
    float timeCoeff;      /*!< Flux observer time coefficient */
    float flitCoeff;      /*!< Stator flux fliter coefficient */

    float emfAlpha;       /*!< Alpha axis back emf */
    float emfBeta;        /*!< Beta axis back emf */
    float emfAmp;         /*!< Back emf amp */
    float emfAlphaComp;   /*!< Alpha axis back emf compensation */
    float emfBetaComp;    /*!< Beta axis back emf compensation */
    float emfCompCoeff;   /*!< Back emf compensation coefficient */

    float staFluxAlpha;   /*!< Alpha axis stator flux */
    float staFluxBeta;    /*!< Beta axis stator flux */
    float rotFluxAlpha;   /*!< Alpha axis rotator flux */
    float rotFluxBeta;    /*!< Beta axis rotator flux */
    float rotFluxAmp;    /*!< Rotator flux amplitide */
} FLUX_OBSERVER_CTRL;

/*!
* @brief Flux observer configuration parameter structure.
*/
typedef struct
{
    float flitCoeff;      /*!< LPF filter coefficient */
    float compCofLimit;  /*!< Back emf compensation uplimit */
} FLUX_OBSERVER_CFG;

/*!
* @brief Flux observer configuration and control parameter structure.
*/
typedef struct
{
    FLUX_OBSERVER_CTRL *pCtrl;      /*!< Flux observer control parameter structure */
    FLUX_OBSERVER_CFG *pCfg;        /*!< Flux observer configuration parameter structure */
} FLUX_OBSERVER;

/*!
* @brief Induction motor flux observer flux parameters structure.
*/
typedef struct
{
    int32_t rotorFluxAlpha;     /*!< Motor rotor flux vector alpha axis component */
    int32_t rotorFluxBeta;      /*!< Motor rotor flux vector beta axis component */
    uint32_t rotorFluxAmp;      /*!< Motor rotor flux vector amplitude */
    uint16_t rotorFluxAngle1;   /*!< Motor rotor flux vector position angle */
    uint16_t rotorFluxAngle;    /*!< Motor rotor flux vector position angle with compensation */
} ACIM_FLUX_PARAM;

/*!
* @brief Induction motor flux observer speed parameters structure.
*/
typedef struct
{
    int16_t rotorSpeed;         /*!< Motor rotor speed(frequency) */
    int16_t slipSpeed;          /*!< Motor stator-rotor slip frequency */
    int16_t syncSpeed;          /*!< Motor stator synchronous frequency */
} ACIM_SPEED_PARAM;

/*!
* @brief Induction motor flux observer current parameters structure.
*/
typedef struct
{
    int16_t curAlpha;           /*!< Motor stator current vector alpha axis component */
    int16_t curBeta;            /*!< Motor stator current vector beta axis component */
    int16_t curAlphaLast;       /*!< Motor stator current vector alpha axis component of the previous sampling period*/
    int16_t curBetaLast;        /*!< Motor stator current vector beta axis component of the previous sampling period*/
    int16_t curDRef;            /*!< D-axis current loop reference */
    int16_t curDFdbk;           /*!< D-axis current loop feedback */
    int16_t curQRef;            /*!< Q-axis current loop reference */
    int16_t curQFdbk;           /*!< Q-axis current loop feedback */

    int16_t excitCurCalc;       /*!< Excitation current calculation value */
} ACIM_CURRENT_PARAM;

/*!
* @brief Induction motor flux observer voltage parameters structure.
*/
typedef struct
{
    int32_t statorEmfAlpha;     /*!< Motor stator bEmf alpha axis component */
    int32_t statorEmfBeta;      /*!< Motor stator bEmf beta axis component */
    int32_t statorEmfD;         /*!< Motor stator bEmf D axis component */
    int32_t statorEmfQ;         /*!< Motor stator bEmf Q axis component */
    int32_t decoupleVoltD;      /*!< Motor current loop decoupling voltage of d-axis */
    int32_t decoupleVoltQ;      /*!< Motor current loop decoupling voltage of q-axis */

} ACIM_VOLTAGE_PARAM;

/*!
* @brief Induction motor flux observer motor parameters structure.
*/
typedef struct
{
    int16_t excitCur;           /*!< Motor no-load excition current */
    uint32_t rs;                /*!< Motor stator resistance value */
    uint32_t rr;                /*!< Motor rotor resistance value */
    uint32_t lm;                /*!< Motor stator-rotor mutually induction value */
    uint32_t ll;                /*!< Motor stator-rotor leakage induction value */
    uint32_t ls;                /*!< Motor stator induction value */
    uint32_t lr;                /*!< Motor rotor induction value */
    uint32_t tr;                /*!< Motor rotor time constant */

    uint32_t lrDivLm;           /*!< Rotor induction divide mutually induction */
    uint32_t lmCoef;            /*!< (Ls*Lr - Lm*Lm) / Lm */
    uint32_t oL;                /*!< Motor leakage coefficient */
    uint32_t oLs;               /*!< (Ls*Lr - Lm*Lm) / Lm */
    uint32_t fsampleCoef;       /*!< Sampling frequency coefficient */
    uint32_t rotorFluxCoef;     /*!< Rotor flux calculation coefficient */
    uint32_t timeCoef;          /*!< Integral calculation time coefficient */
    uint32_t slipFreqCoef;      /*!< Slip frequency calculation coefficient */
} ACIM_MOTOR_PARAM;

/*!
* @brief Induction motor flux observer structure.
*/
typedef struct
{
    ACIM_FLUX_PARAM     *flux;  /*!< Flux parameters structure */
    ACIM_SPEED_PARAM    *speed; /*!< Frequency parameters structure */
    ACIM_CURRENT_PARAM  *cur;   /*!< Current parameters structure */
    ACIM_VOLTAGE_PARAM  *volt;  /*!< Voltage parameters structure */
    ACIM_MOTOR_PARAM    *mot;   /*!< Motor parameters structure */
} ACIM_FOC_CTRL;

/*!
* @brief Induction motor flux observer type.
*/
typedef enum
{
    ACIM_FO_METHOD_1 = 1,       /*!< Induction motor flux observer method 1: integral and correction */
    ACIM_FO_METHOD_2 = 2,       /*!< Induction motor flux observer method 2: pos-neg correction */
    ACIM_FO_METHOD_3 = 3,       /*!< Induction motor flux observer method 3: pll */
} ACIM_FLUXOBSERVER_TYPE;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief PLL method calculate rotation speed PID Coefficient and pid input maxPu minPu limit value initialize,
*        usually it is initialized only once before running.
*
* @param[in] pidPll: pointer to PID_REGULATOR structure, which contains pid input, output and coefficients
* @param[in] focVarsCfg: variable which use to configure FOC control, pointer to FOC_VARS_CFG structure
* @param[in] pll: pointer to PLL_CFG structure
* @return none
*/
void Asr_SpeedPLLInit(PID_REGULATOR *pidPll, FOC_VARS_CFG *focVarsCfg, PLL_CFG *pll);

/*!
* @brief Calculate the rotor speed through by the flux observer PLL method, usually it is executed in interrupt.
*
* @param[in] fluxobserve: pointer to FLUX_OBSERVER_CTRL structure
* @param[in] pidPll: pointer to PID_REGULATOR structure
* @return motor electrical frequency estimated value by PLL in Q15
*/
float FluxObserver_PLLCalc(FLUX_OBSERVER_CTRL *fluxobserve, PID_REGULATOR *pidPll);

/*!
* @brief Flux observer variables initialize, the LPF filter coeffieient and cutoff frequency need to be passed externally,
*        usually it is initialized only once before running.
*
* @param[in] fluxobserve: pointer to FLUX_OBSERVER structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] focVarsCfg: variable which use to configure FOC control, pointer to FOC_VARS_CFG structure
* @return none
*/
void FluxObserver_Init(FLUX_OBSERVER *fluxobserve, MOTOR_PARAM_PU *paramPu, FOC_VARS_CFG *focVarsCfg);

/*!
* @brief Calculate the rotar angle and speed estimated by the flux observer for sensorless vector control,
*        which contains ATAN and PLL angle calculation methods, usually it is executed in interrupt.
*
* @param[in] STC: pointer to STC_PARAM structure
* @param[in] fluxobserve: pointer to FLUX_OBSERVER structure
* @param[in] speedCalc: pointer to ESTIMATED_SPEED structure
* @param[in] pidPll: pointer to PID_REGULATOR structure
* @return motor electrical angle in Q16
*/
float FluxObserver_PosEst(STC_PARAM *STC, FLUX_OBSERVER *fluxobserve, ESTIMATED_SPEED *speedCalc, PID_REGULATOR *pidPll);

/*!
* @brief Decoupling calculation of current loop of induction motor vector control.
*
* @param[in] acimFoc: pointer to ACIM_FOC_CTRL structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Acim_CurrentLoopDecouple(ACIM_FOC_CTRL *acimFoc, FOC_VARS_CTRL *focVars);

/*!
* @brief Flux estimation of induction motor vector control.
*
* @param[in] acimFoc: pointer to ACIM_FOC_CTRL structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] pidPll: pointer to PID_REGULATOR structure
* @param[in] observerType: flux observer type
* @return none
*/
void Acim_FluxObserverCalc(ACIM_FOC_CTRL *acimFoc, FOC_VARS *focVars, PID_REGULATOR *pidPll, uint8_t observerType);

/*!
* @brief Rotation electric angle calculation of induction motor vector control.
*
* @param[in] acimFoc: pointer to ACIM_FOC_CTRL structure
* @param[in] STC: pointer to STC_PARAM structure
* @return none
*/
void Acim_FluxObserverAngleCalc(ACIM_FOC_CTRL *acimFoc, STC_PARAM *STC);

/*!
* @brief Induction motor slip frequency calculation.
*
* @param[in] acimFoc: pointer to ACIM_FOC_CTRL structure
* @param[in] STC: pointer to STC_PARAM structure
* @return none
*/
void Acim_FluxObserverSlipFreqCalc(ACIM_FOC_CTRL *acimFoc, STC_PARAM *STC);

/*!
* @brief Induction motor slip frequency calculation for sensor FOC.
*
* @param[in] acimFoc: pointer to ACIM_FOC_CTRL structure
* @param[in] STC: pointer to STC_PARAM structure
* @return none
*/
void Acim_SensorSlipFreqCalc(ACIM_FOC_CTRL *acimFoc, STC_PARAM *STC);

/*!
* @brief Induction motor Flux observer parameters initialization.
*
* @param[in] acimFoc: pointer to ACIM_FOC_CTRL structure
* @param[in] STC: pointer to STC_PARAM structure
* @return none
*/
void Acim_FluxObserverInit(ACIM_FOC_CTRL *acimFoc, STC_PARAM *STC);

/* ======================================  Functions define  ======================================== */

#endif /* __FLUX_OBSERVER_H__ */

/* =============================================  EOF  ============================================== */
