/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __HIGHFREQINJ_H__
#define __HIGHFREQINJ_H__

/*!
* @file highfreqinj.h
*
* @brief This file provides the rotar angle and speed by the high frequency injection for sensorless vector control functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "pid_regulator.h"
#include "param_init.h"

/* ============================================  Define  ============================================ */
#define INITPOS_ARR_MAX         400

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Initial position detect parameters structure.
*/
typedef struct
{
    float arr[INITPOS_ARR_MAX];   /*!< High frequency current storage array */
    uint16_t index;                 /*!< High frequency current storage index */
    float max;                    /*!< High frequency d axis current max value */
    float min;                    /*!< High frequency d axis current min value */
} HFI_INIT_POS_DETECT;

/*!
* @brief High frequency injection configuration parameters structure.
*/
typedef struct
{
    uint16_t hfiDebugEnable;    /*!< High frequency injection debug mode enable switch */
    uint16_t hfiToStoEnable;    /*!< High frequency injection switch to high speed strategy enable switch */
    uint16_t pulseTime;         /*!< Pulse voltage inject time */
    uint16_t clearTime;         /*!< Current clear time */
    uint16_t msUnit;            /*!< High frequency injection ms unit */
    uint16_t hfiInitTime;       /*!< High frequency injection initial inject time */

    float vPeak;              /*!< High frequency injection voltage amplitude */
    float pulseAmp;           /*!< Pulse voltage amplitude */
    float hfiFreq;            /*!< High frequency injection frequency */

    float hfiInitTheta;      /*!< High frequency injection initial theta */

    float hfiVdFreqHigh;      /*!< High frequency injection vd calculate frequency high */
    float hfiVdFreqLow;       /*!< High frequency injection vd calculate frequency low */

/* Filter transfer function: y(n) = A0 * x(n) + A1 * x(n-1) + A2 * x(n-2) + B1 * y(n-1) + B2 * y(n-2).
*  Different filters can be obtained by  matching different coefficients A and B.
*/
    float bpfA0;              /*!< Band pass filter coefficient A0 */
    float bpfA2;              /*!< Band pass filter coefficient A2 */
    float bpfB2;              /*!< Band pass filter coefficient B2 */

    float lpfA0;              /*!< Low pass filter coefficient A0 */
    float lpfA1;              /*!< Low pass filter coefficient A1 */
    float lpfA2;              /*!< Low pass filter coefficient A2 */
    float lpfB2;              /*!< Low pass filter coefficient B2 */

    float thetaLpfCoeff;      /*!< 1 order Low pass filter coefficient */

    float bpfB1;              /*!< Band pass filter coefficient B1 */
    float lpfB1;              /*!< Low pass filter coefficient B1 */
    float hfiPllKp;           /*!< High frequency injection pll kp */
    float hfiPllKi;           /*!< High frequency injection pll ki */
} HFI_CFG;

/*!
* @brief High frequency injection band pass filter control parameters structure.
*/
typedef struct
{
    float a0;                 /*!< Band pass filter a0 */
    float a2;                 /*!< Band pass filter a2 */
    float b2;                 /*!< Band pass filter b2 */
    float b1;                 /*!< Band pass filter b1 */
} HFI_BPF_COEFF;

/*!
* @brief High frequency injection low pass filter control parameters structure.
*/
typedef struct
{
    float a0;                 /*!< Low pass filter a0 */
    float a1;                 /*!< Low pass filter a1 */
    float a2;                 /*!< Low pass filter a2 */
    float b2;                 /*!< Low pass filter b2 */
    float b1;                 /*!< Low pass filter b1 */
} HFI_TWO_ORDER_LPF;

/*!
* @brief High frequency injection state parameters enumerate.
*/
typedef enum
{
    HF_STATE0 = 0,              /*!< High frequency injection state0 */
    HF_STATE1 = 1,              /*!< High frequency injection state1 */
    HF_STATE2 = 2,              /*!< High frequency injection state2 */
    HF_STATE3 = 3,              /*!< High frequency injection state3 */
    HF_STATE4 = 4,              /*!< High frequency injection state4 */
    HF_STATE5 = 5,              /*!< High frequency injection state5 */
    HF_ERROR = 6                /*!< High frequency injection state6 */
} HFI_STATE;

/*!
* @brief High frequency injection control parameters structure.
*/
typedef struct
{
    HFI_STATE state;            /*!< High frequency injection state */
    uint16_t subState;          /*!< High frequency injection initial polar detect substate */
    uint16_t msUnit;            /*!< High frequency injection ms unit */
    uint16_t spdCtrlloopCnt;    /*!< Switch to speed loop count */
    uint16_t thetaCompCnt;      /*!< High frequency injection compensation theta recheck count */
    uint16_t dirErrCnt;         /*!< High frequency injection run direction error count */
    uint16_t hfiToStoCount;     /*!< High frequency injection switch to high speed strategy count */
    uint16_t hfiToStoFlag;      /*!< High frequency injection switch to high speed strategy flag */
    uint32_t delaycnt;          /*!< High frequency injection delay count */

    float detectVpk;            /*!< High frequency injection detect voltage amplitude */
    float hfiFreq;              /*!< High frequency injection frequency */
    float pwmFreq;              /*!< Pwm frequency */
    float vsdRef;               /*!< High frequency injection d axis given voltage */
    float vsqRef;               /*!< High frequency injection q axis given voltage */

    float hfiVoltTheta;         /*!< High frequency injection angular frequency */
    float voltThetaInc;         /*!< High frequency injection angular frequency increments */
    float hfiEstTheta;          /*!< High frequency injection estimate theta */
    float hfiThetaTemp;         /*!< High frequency injection before compensation theta */

    float thetaComp;            /*!< High frequency injection compensation theta */
    float hfiEstTheta1;         /*!< High frequency injection estimate theta */

    float hfiEstThetaErr;       /*!< High frequency injection injection estimate theta error */

    float vdSlope;              /*!< D axis voltage ampilitude slope */
    float weightFreq;           /*!< Weight frequence of HFI and observer */

    float iqhFilterH;           /*!< Q axis current after band pass filter */
    float iqhFilterL;           /*!< Q axis current after low pass filter */
    float idhFilterH;           /*!< D axis current after band pass filter */
    float idhFilterL;           /*!< D axis current after LOW pass filter */


    float iqhFilterH1;          /*!< Last period q axis current after bpf */
    float iqhFilterH2;          /*!< Last second period q axis current after bpf */
    float idhFilterH1;          /*!< Last period d axis current after bpf */
    float idhFilterH2;          /*!< Last second period d axis current after bpf */

    float idhFilterL1;          /*!< Last period d axis current after lpf */
    float idhFilterL2;          /*!< Last second period d axis current after lpf */
    float iqhFilterL1;          /*!< Last period q axis current after lpf */
    float iqhFilterL2;          /*!< Last second period q axis current after lpf */

    float isAmp;                /*!< High frequency current ampititude */
    float hfiEstFreq;           /*!< High frequency injection estimated frequency */
    float idh;                  /*!< Current high freq d axis current before bpf */
    float idh1;                 /*!< Last period high freq d axis current before bpf */
    float idh2;                 /*!< Last two period high freq d axis current before bpf */
    float iqh;                  /*!< Current high freq q axis current before bpf */
    float iqh1;                 /*!< Last period high freq q axis current before bpf */
    float iqh2;                 /*!< Last two period high freq q axis current before bpf */
    float idhdet;               /*!< Current high freq d axis det current before lpf */
    float idhdet1;              /*!< Last period high freq d axis det current before lpf */
    float idhdet2;              /*!< Last two period high freq d axis det current before lpf */
    float iqhdet;               /*!< Current high freq q axis det current before lpf */
    float iqhdet1;              /*!< Last period high freq q axis det current before lpf */
    float iqhdet2;              /*!< Last two period high freq q axis det current before lpf */
    float idPos;                /*!< Positive id value */
    float idNeg;                /*!< Negative id value */

} HFI_TYPE;

/*!
* @brief High frequency injection total control parameters structure.
*/
typedef struct
{
    HFI_INIT_POS_DETECT *initPos;   /*!< Initial postion detect structure */
    HFI_BPF_COEFF *bpf;             /*!< Band pass filter structure */
    HFI_TWO_ORDER_LPF *lpf;         /*!< Two order low pass filter structure */
    HFI_TYPE *hfi;                  /*!< High frequency injection structure */
} HFI_CTRL;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief High frequency injection all the control and configuration parameters initialize,
*        usually it is initialized only once before running.
*
* @param[in] hfiCtrl: pointer to HFI_CTRL structure
* @param[in] pidHfi: pointer to PID_REGULATOR structure
* @param[in] hfiCfg: pointer to HFI_CFG structure
* @param[in] focVarsCfg: variable which use to configure FOC control, pointer to FOC_VARS_CFG structure
* @return none
*/
void Hfi_Init(HFI_CTRL *hfiCtrl, PID_REGULATOR *pidHfi, HFI_CFG *hfiCfg, FOC_VARS_CFG *focVarsCfg);

/*!
* @brief High frequency injection initial magnetic pole identification function, determine
*        the initial magnetic pole according to the inductance saturation effect.
*
* @param[in] initPos: pointer to HFI_INIT_POS_DETECT structure
* @param[in] hfi: pointer to HFI_TYPE structure
* @return none
*/
void Hfi_InitPolarDetect(HFI_INIT_POS_DETECT *initPos, HFI_TYPE *hfi);

/*!
* @brief High frequency injection initial magnetic pole identification function, determine
*        the initial magnetic pole according to the inductance saturation effect.
*
* @param[in] hfiCtrl: pointer to HFI_CTRL structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] hfiCfg: pointer to HFI_CFG structure
* @return none
*/
void Hfi_InitPolarCheck(HFI_CTRL *hfiCtrl, FOC_VARS *focVars, HFI_CFG *hfiCfg);

/*!
* @brief High frequency injection initial magnetic pole identification recheck function, to make
         sure the compensation angle is right.
*
* @param[in] hfiCtrl: pointer to HFI_CTRL structure
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void Hfi_InitPolarRecheck(HFI_CTRL *hfiCtrl, FOC_VARS *focVars);

/*!
* @brief High frequency injection detect voltage amplitude calculate function.
*
* @param[in] hfiCtrl: pointer to HFI_CTRL structure
* @param[in] hfiCfg: pointer to HFI_CFG structure
* @return none
*/
void Hfi_VdAmp_Calc(HFI_CTRL *hfiCtrl, HFI_CFG *hfiCfg);

/*!
* @brief High frequency injection angle obtain function, through injection a high frequency sinusoidal
*        voltage on the d axis, extracting the rotor position from the high frequency current response.
*
* @param[in] hfiCtrl: pointer to HFI_CTRL structure
* @param[in] pidHfi: pointer to PID_REGULATOR structure
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] hfiCfg: pointer to HFI_CFG structure
* @return high frequency injection estimated theta in Q16
*/
float Hfi_AngleObtain(HFI_CTRL *hfiCtrl, PID_REGULATOR *pidHfi, FOC_VARS *focVars, HFI_CFG *hfiCfg);

/* ======================================  Functions define  ======================================== */

#endif /* __HIGHFREQINJ_H__ */

/* =============================================  EOF  ============================================== */
