/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __PARAM_ID_H__
#define __PARAM_ID_H__
/*!
* @file param_identify.h
*
* @brief This file contain four motor parameters identification, include Rs Ld/Lq Flux Inertia interface.
*
*/

/* ===========================================  Includes  =========================================== */
#include "param_init.h"
#include "hall_sensor.h"

/* ============================================  Define  ============================================ */
//#define CONVERSION_COEFF                (46341U)                        /*!< 46341 >> 15 = sqrt(2), conversion coefficient between real value and unit value */
#define CONVERSION_COEFF                SQRT2
#define RS_SAVENUM_ORDER                (9U)                            /*!< Power exponential value of save number */
#define RS_SAVENUM                      (1 << RS_SAVENUM_ORDER)         /*!< Save number, RS_SAVENUM = 2^RS_SAVENUM_ORDER */

/*
*   ^
* I |            |------------|
*   |            |            |
*   |------------|            |
*   |                         |
*   |                         |-----
*   |--1-|-2-|-3-|--1-|-2-|-3-|------> TIME
*   1: RS_STABLE_DELAY; 2:RS_SAMPLE_DELAY; 3: RS_SETTING_DELAY.
*/

#define IDENTIFY_WAIT_1S                (1000UL)                /*!< twice pulse inject interval time in ms */
#define INERTIA_ID_FREQ                 Math_IQ(0.5f)            /*!< Inertia identify set frequency */
#define INERTIA_ID_IQ                   Math_IQ(0.5f)            /*!< Inertia identify iq */

#define FFT_DATAORD_PMSMLDQ             (4U)
#define FFT_DATANUM_PMSMLDQ             (1U << FFT_DATAORD_PMSMLDQ)

/* ===========================================  Typedef  ============================================ */
typedef struct
{
    float real;                         /*!< Real part of FFT input signal data */
    float imag;                         /*!< Imag part of FFT input signal data */
} TYPE_FFT_SAMPLE_DATA;

typedef struct
{
    uint16_t dataOrder;                 /*!< FFT order */
    uint32_t dataNum;                   /*!< FFT input signal data number */
    float dcComp;                       /*!< DC component of FFT input signal */
    float fundAmp;                      /*!< Fundation amplitude of FFT input signal */
    float fundAng;                      /*!< Fundation phase angle of FFT input signal */
} FFT_PARAM;

/* Q15 FFT */
typedef struct
{
    int64_t real;                                   /*!< Signal saving array : real component */
    int64_t imag;                                   /*!< Signal saving array : imaginary component */
} TYPE_FFT_SAMPLE_DATA_Q15;

/*!
* @brief FFT data struct
*/
typedef struct
{
    uint16_t dataOrder;                             /*!< FFT calculation order */
    uint32_t dataNum;                               /*!< FFT data number */
    int32_t dcComp;                                 /*!< FFT calculation result dc component */
    int32_t fundAmp;                                /*!< FFT calculation result 1st component amplitude */
    uint16_t fundAng;                               /*!< FFT calculation result 1st component phase angle */
} FFT_PARAM_Q15;

/*!
* @brief Identify order
*/
typedef enum {
    IDENTIFY_RS_DCTEST = 0,                         /*!< Step 0 : rs identify, DC test */
    IDENTIFY_LDQ_STALLTEST,                         /*!< Step 1 : ld lq identify, stall test */
    IDENTIFY_FLUX_ROTATIONTEST,                     /*!< Step 2 : flux identify, rotation test */
    IDENTIFY_INERTIA,                               /*!< Step 3 : inertia identify */
    IDENTIFY_END                                    /*!< Step 4 : finished */
} PAR_EST_STEP;

/*!
* @brief Identify main control data struct
*/
typedef struct
{
    uint8_t identifyCtrl;                           /*!< Control motor identify start or stop */
    PAR_EST_STEP mainStep;                          /*!< Motor identify main step */
    uint8_t initCtrl;                               /*!< Motor identify init enble bit */
    uint16_t cnt1ms;                                /*!< 1ms counter */
    uint16_t errFlag;                               /*!< During the whole identify process, occur error*/

    float setElecAngle;                          /*!< Output rotor angle*/

    uint16_t rsId;                                  /*!< Motor rs identify output actual value */
    uint16_t ld;                                    /*!< Motor ld identify output actual value */
    uint16_t lq;                                    /*!< Motor lq identify output actual value */
    uint16_t rrId;                                  /*!< Motor rr identify output actual value */
    uint16_t lsId;                                  /*!< Motor ls identify output actual value */
    uint16_t lrId;                                  /*!< Motor lr identify output actual value */
    uint16_t oLsId;                                 /*!< Motor leakage inductance identify output actual value */
    uint16_t lmId;                                  /*!< Motor lm identify output actual value */

    float bEmf;                                  /*!< Motor back EMF constant identify output value, related to flux */
    float rsIdPu;                                /*!< Motor rs identify output pu value */
    float ldPu;                                  /*!< Motor ld identify output pu value */
    float lqPu;                                  /*!< Motor lq identify output pu value */
    float flux;                                  /*!< Motor flux identify output actual value */
    float fluxPu;                                /*!< Motor flux identify output pu value */
    float curNoLoadPu;                           /*!< Motor no-load current identify output pu value */
    float inertia;                               /*!< Motor inertia identify output actual value */
    float inertiaPu;                             /*!< Motor inertia identify output pu value */
} PARAM_IDENTIFY_MAIN_CTRL;

/*!
* @brief Identify Rs data struct
*/
typedef struct
{
    uint8_t subStep;                               /*!< Rs identify step */

    /* base on SVPWM */
    uint8_t phaseOrder;                            /*!< Rs identify phase order, used to control the Rs identification phase sequence */
    uint16_t delayCnt;                              /*!< Rs identification delay counter, used to control the delay duration */

    float ialfaSet1;                              /*!< The first segment setting value of current alpha axis component */
    float ibetaSet1;                              /*!< The first segment setting value of current beta axis component */
    float ialfaSet2;                              /*!< The second segment setting value of current alpha axis component */
    float ibetaSet2;                              /*!< The second segment setting value of current beta axis component */
    float ialfaRef;                               /*!< Current alpha axis component control loop reference value */
    float ibetaRef;                               /*!< Current beta axis component control loop reference value */

    float ualfaSave1;                             /*!< Average value of voltage alpha axis component sampling results saved in the first segment */
    float ialfaSave1;                             /*!< Average value of current alpha axis component sampling results saved in the first segment */
    float ualfaSave2;                             /*!< Average value of voltage alpha axis component sampling results saved in the secong segment */
    float ialfaSave2;                             /*!< Average value of current alpha axis component sampling results saved in the secong segment */
    float ualfaSum;                               /*!< Cumulative value of voltage alpha axis component sampling results */
    float ialfaSum;                               /*!< Cumulative value of current alpha axis component sampling results */

    float rsCalcPu;                              /*!< Calculated resistance of once identification process for each phase */
    float rsPhaseAPu;                            /*!< PU value of phase A stator resistance identification */
    float rsPhaseBPu;                            /*!< PU value of phase B stator resistance identification */
    float rsPhaseCPu;                            /*!< PU value of phase C stator resistance identification */
    float rsPhaseAId;                            /*!< Real value of phase A stator resistance identification */
    float rsPhaseBId;                            /*!< Real value of phase B stator resistance identification */
    float rsPhaseCId;                            /*!< Real value of phase C stator resistance identification */
    float rsAvgPu;                               /*!< PU value of average value of three-phase stator resistance identification */
} PARAM_RS_IDENTIFY_DATA;

/*!
* @brief Identify Ld Lq data struct
*/
typedef struct
{
    uint8_t step;                                   /*!< Ld lq identify step */
    uint8_t subStep;                                /*!< Ld lq identify substep */
    uint8_t section;                                /*!< Conduct phase */

    uint16_t wait;                                  /*!< Between two step wait time */
    uint16_t pwmTs;                                 /*!< Pwm module period value */
    uint16_t periodCnt;                             /*!< Ts counter */

    float curFirst;                               /*!< First inject current */
    float curLimit;                              /*!< Max current limit */
    float curPu[6];                              /*!< Inject current value */
    float curPuLast[6];                          /*!< Inject current value buffer*/
    float lPhase[6];                             /*!< Phase inductance value */
} PARAM_LDQ_IDENTIFY_DATA;

/*!
* @brief Identify flux data struct
*/
typedef struct
{
    uint8_t subStep;                                /*!< Flux identify substep */
    uint16_t cnt;                                   /*!< 1ms counter */

    float freTarget;                             /*!< Target frequency */
    float freSet;                                /*!< Actual set frequency */
    float freStep;                               /*!< Frequency add step */

    float fluxIdThetaDelta;                      /*!< Motor flux identify theta increase value */
    float fluxIdOutCurPu;                        /*!< Motor flux identify current pu */
    float fluxIdOutVolPu;                        /*!< Motor flux identify bus voltage pu */

    float totalId1;                              /*!< Flux identify current accumulate value1 */
    float totalId2;                              /*!< Flux identify current accumulate value2 */
    float totalVolt1;                            /*!< Flux identify output voltage accumulate value1 */
    float totalVolt2;                            /*!< Flux identify output voltage accumulate value2 */
} PARAM_FLUX_IDENTIFY_DATA;

/*!
* @brief Identify inertia data struct
*/
typedef struct
{
    uint8_t subStep;                                /*!< Inertia identify substep */
    uint8_t overSetFreCnt;                          /*!< Over set frequency counter */
    uint16_t cnt;                                   /*!< 1ms counter */
    uint32_t accTime;                               /*!< Accelerate use time */
} PARAM_INERTIA_IDENTIFY_DATA;

/*!
* @brief Parameter identify configuration
*/
typedef struct
{
    uint8_t identifyParSel;                        /*!< Identify parameter select */

    uint8_t statorResisOrderSel;                   /*!< Select motor stator phase order to identify stator resistance, 1: One phase; 2: Two phase; 3: Three phase */
    uint8_t pmsmLdqTypeSel;                        /*!< PMSM Ldq identification method select */

    uint16_t rsSetDelayTime;                        /*!< Unit: ms, wait for changing the amplitude of the injection current */
    uint16_t rsSampleDelayTime;                     /*!< Unit: ms, wait for sampling and saving current and voltage */
    uint16_t rsStableDelayTime;                     /*!< Unit: ms, wait for stable after changing the amplitude of the injection current */
    uint16_t ldqInitPrd;                            /*!< Ldq identify initial pwm period*/  
    uint16_t alignTime1;                            /*!< First alignment time in ms */
    uint16_t alignTime2;                            /*!< Second alignment time in ms */
    uint16_t alignWaitTime;                         /*!< Alignment done and wait time in ms */
    uint16_t calcDataNum;                           /*!< Motor identification data calculation number */
    uint16_t standByDelay;                          /*!< Motor parameters identification time delay in standby step */
    uint16_t startToCalcDelay;                      /*!< Motor parameters identification start to calculation delay */
    uint16_t calcDelay;                             /*!< Motor parameters identification calculation step delay time */
    uint16_t calcToEndDelay;                        /*!< Motor parameters identification calculation ending delay */
    uint16_t switchDelay;                           /*!< Motor parameters identification steps swicth delay */

    uint16_t parIdPwmFreqSet;                       /*!< PWM frequency set value */
    uint16_t parIdPwmPrdSet;                        /*!< PWM period set value */
    uint16_t parIdPwmFreqDef;                       /*!< PWM frequency default value */
    uint16_t parIdPwmPrdDef;                        /*!< PWM frequency default value */

    float rsFirstInjectCurAmp;                   /*!< Rs identify first inject current threshold*/
    float rsSecondInjectCurAmp;                  /*!< Rs identify second inject current threshold*/

    float ldqCurLimit;                           /*!< Ldq identify max current limit*/

    float alignCurVal;                           /*!< Alignment current */

    float fluxIdentifySetFreq;                   /*!< Flux identify set frequency */
    float fluxIdentifySetCur;                    /*!< Flux identify set current, must less than 16384 */
    float fluxIdentifyIfAcc;                     /*!< Flux identify IF run  acceleration*/

    float sigFilter1Coeff1;                      /*!< Motor identification signal filter-1 coefficient */
    float sigFilter1Coeff2;                      /*!< Motor identification signal filter-2 coefficient */
    float sigFilter2Coeff1;                      /*!< Motor identification signal filter-1 coefficient */
    float sigFilter2Coeff2;                      /*!< Motor identification signal filter-2 coefficient */

    float currentThreshold;                      /*!< Acim identification current threshold */
    float voltageThreshold;                      /*!< Acim identification voltage threshold */
    float rotationFreqLimit;                     /*!< Acim rotation test frequency limit */
    float rotationFreqStep;                      /*!< Acim rotation test frequency step */
    float stallFreqLimit;                        /*!< Acim stall test frequency limit */
    float stallCurrentLimit;                      /*!< Acim stall test current limit */
    float stallCurrentStep;                       /*!< Acim stall test current step */

    /* PMSM Ldq identify method: high frequency sine wave */
    float curDcSet;                              /*!< PMSM Ldq identification DC current set value */
    float curAcSetD;                             /*!< PMSM Ldq identification d-axis AC current set value */
    float curAcSetQ;                             /*!< PMSM Ldq identification q-axis AC current set value */
    float curFreqSetD;                           /*!< PMSM Ldq identification d-axis AC current frequency set value */
    float curFreqSetQ;                           /*!< PMSM Ldq identification q-axis AC current frequency set value */
    float curDcStep;                             /*!< PMSM Ldq identification DC current variable step value */

    float stableJudgeLimit;                      /*!< Signal stable judgement limit */

} PARAM_IDENTIFY_CFG;

/*!
* @brief Asynchronous motor parameters identification data structure
*/
typedef struct
{
    uint8_t rrStep;                                 /*!< Identification steps of rotor parameters */
    uint8_t lmStep;                                 /*!< Identification steps of mutual inductance */
    uint8_t calcStartFlag;                          /*!< Flag to start calculate */
    uint8_t calcEndFlag;                            /*!< Flag of calculation finish */
    uint32_t delayCnt;                              /*!< Delay counter */

    float curAlfaStep;                            /*!< Current increase step */
    float curAlfaRef;                             /*!< Current amplitude reference value */
    float curAlfaSet;                             /*!< Current command */
    float curAlfaFdbk;                            /*!< Current feedback */
    float voltAlfaFdbk;                           /*!< Voltage feedback */
    float curFreq;                               /*!< Current frequency */
    float curFreqStep;                           /*!< Current frequency step */
    float deltaTheta;                            /*!< Electric angle calculation step */
    float setElecTheta;                          /*!< Electric angle of parameters identification */
    float dataNum;                               /*!< Data numbers */
    float curAmp;                                /*!< Current amplitude calculation value */
    float voltAmp;                               /*!< Voltage amplitude calculation value */
    float curAng;                                /*!< Current phase angle calculation value */
    float voltAng;                               /*!< Voltage amplitude calculation value */
    float phaseAng;                              /*!< Phase angle between current and voltage */
    float curAmpAvg;                             /*!< Current amplitude calculation average value */
    float voltAmpAvg;                            /*!< Current phase angle calculation average value */
    float curAngAvg;                             /*!< Voltage amplitude calculation average value */
    float voltAngAvg;                            /*!< Voltage phase angle calculation average value */
    float curAmpSum;                             /*!< Current amplitude calculation sum value */
    float curAngSum;                             /*!< Current phase angle calculation sum value */
    float voltAmpSum;                            /*!< Voltage amplitude calculation sum value */
    float voltAngSum;                            /*!< Voltage phase angle calculation sum value */
    float rrCalcPuFirst;                         /*!< Rotor resistance first calculation value */
    float rrCalcPuSecond;                        /*!< Rotor second first calculation value */
    float oLsCalcPuFirst;                        /*!< Stator-rotor leakage inductance first calculation value */
    float oLsCalcPuSecond;                       /*!< Stator-rotor leakage inductance second calculation value */
    float lmCalcPuFirst;                         /*!< Stator-rotor mutual inductance first calculation value */
    float lmCalcPuSecond;                        /*!< Stator-rotor mutual inductance second calculation value */
    float rrCalcPu;                              /*!< Rotor resistance identification value with pu unit */
    float rrReal;                                /*!< Real value of rotor resistance identification */
    float oLsCalcPu;                             /*!< Stator-rotor leakage inductance identification value with pu unit */
    float oLsReal;                               /*!< Real value of stator-rotor leakage inductance identification */
    float lmClacPu;                              /*!< Stator-rotor mutual inductance identification value with pu unit */
    float lmReal;                                /*!< Real value of stator-rotor mutual inductance identification */

} ACIM_PARAM_IDENTIFY;

/*!
* @brief PMSM dq axis inductance identification using high freqneuct sine wave data structure
*/
typedef struct
{
    uint8_t idStep;                             /*!< Identify step */
    uint8_t sampleFlag;                         /*!< Sampling flag */
    uint8_t saveEndFlag;                        /*!< Sample finish flag */
    uint8_t extractFlag;                        /*!< Signal amplitude and angle extract finish flag */
    uint16_t calcCnt;                           /*!< Identify number, to get average identification result */
    uint32_t delayCnt;                          /*!< Counter value to set time delay */
    uint32_t ldRealValue;                       /*!< Ld identification real value in unit uH */
    uint32_t lqRealValue;                       /*!< Lq identification real value in unit uH */
    float curDRefDc;                            /*!< Ldq identification d-axis current DC reference */
    float curDRefAc;                            /*!< Ldq identification d-axis current AC reference */
    float curDRef;                              /*!< Ldq identification d-axis current reference */
    float curDRefFreq;                          /*!< Ldq identification frequency of d-axis current reference */
    float curQRefDc;                            /*!< Ldq identification q-axis current DC reference */
    float curQRefAc;                            /*!< Ldq identification q-axis current AC reference */
    float curQRef;                              /*!< Ldq identification q-axis current reference */
    float curQRefFreq;                          /*!< Ldq identification frequency of q-axis current reference */

    float freqRef;                              /*!< Ldq identification frequency of current reference */
    float setElecTheta;                         /*!< Ldq identification current command angle */
    float acRefAmp;                             /*!< Ldq identification current command amplitude */
    float acRef;                                /*!< Ldq identification current command */

    float curAmp;                               /*!< Ldq identification current feedback amplitude */
    float curAng;                               /*!< Ldq identification current feedback angle */
    float voltAmp;                              /*!< Ldq identification voltage amplitude */
    float voltAng;                              /*!< Ldq identification voltage angle */
    float ldCalcVal;                            /*!< Ldq identification d-axis inductance calculation value in uint p.u. */
    float lqCalcVal;                            /*!< Ldq identification q-axis inductance calculation value in uint p.u. */
} PMSM_LDQID_SWDATA;

/*!
* @brief PMSM dq axis inductance identification data structure
*/
typedef struct
{
    PMSM_LDQID_SWDATA *ldqIdParam;
    FFT_PARAM *fftParam;
} PMSM_LDQID_PARAM;

/*!
* @brief Parameter identify data struct
*/
typedef struct
{
    PARAM_IDENTIFY_CFG          *pParamIdentifyCfg;         /*!< Motor parameters identification configuration parameters */
    PARAM_IDENTIFY_MAIN_CTRL    *pParamIdentifyCtrl;        /*!< Parameter identif data stauct */
    PARAM_RS_IDENTIFY_DATA      *pParamRsIdentify;          /*!< Identify Rs data struct */
    PARAM_LDQ_IDENTIFY_DATA     *pParamLdqIdentify;         /*!< Identify Ld Lq data struct */
    PARAM_FLUX_IDENTIFY_DATA    *pParamFluxIdentify;        /*!< Identify flux data struct*/
    PARAM_INERTIA_IDENTIFY_DATA *pParamInertiaIdentify;     /*!< Identify inertia data struct */

    ACIM_PARAM_IDENTIFY         *pAcimParamIdentify;        /*!< Acim stall and rotation test identification */
//    PMSM_LDQID_HFS_PARAM        *pPmsmLdqIdHfs;             /*<! Identify Ld Lq using HFS data struct */

    PMSM_LDQID_PARAM            *pmsmLdqIdentify;           /*!< Identify Ld Lq with high frequency rotation injection data struct */

} PARAM_IDENTIFY_STRUCT;

/*!
* @brief Signal phase shift data struct
*/
typedef struct
{
    float inputSig[2];                            /*!< Input signal */
    float inputSigBuf[2];                         /*!< Input signal buffer, is the last value of input signal */
    float outputSig[2];                           /*!< Output signal of phase shifter */
    float sigCompD[2];                            /*!< D-axis component */
    float sigCompQ[2];                            /*!< Q-axis component */
    float sigCompDBuf[2];                         /*!< D-axis component buffer */
    float sigCompQBuf[2];                         /*!< Q-axis component buffer */
    float sigFreq[2];                            /*!< Signal frequency */
    float sigRefPhaseAng[2];                     /*!< Signal phase angle reference value */
    float sigRefPhaseAngStep[2];                 /*!< Signal phase angle step value */
    float inputSigAmp[2];                        /*!< Amplitude of input signal */
    float inputSigPhaseAng[2];                   /*!< Initial phase angle of input signal */
    float inputSigAmpBuf[2];                     /*!< Amplitude buffer of input signal */
    float inputSigPhaseAngBuf[2];                /*!< Initial phase angle buffer of input signal */
    float timeCoef;                              /*!< Sample time coefficient */
    float filterCoeff1[2];                       /*!< Filter coefficient */
    float filterCoeff2[2];                       /*!< Filter coefficient */

} SIGNAL_PHASE_SHIFT;

/*!
* @brief Identify inertia data struct
*/
typedef struct
{
    int32_t steadyTime;                                   /*!< Inertia error steady judge time */
    int32_t steadyTimeCnt;                                /*!< Inertia error steady judge time count */
    int32_t inertiaOldTime;                               /*!< Last period inertia refresh time */
    int32_t oldTimeCnt;                                   /*!< Last period inertia time count */
    float mechFreq;                                       /*!< Actual mechaine frequency */
    float mechFreqEst;                                    /*!< Estimated mechaine frequency */
    float mechFreqOld1;                                   /*!< Last period actual mechaine frequency */
    float mechFreqOld2;                                   /*!< Last two period actual mechaine frequency */
    float elecTorque;                                     /*!< Electrical torque */
    float elecTorqueOld1;                                 /*!< Last period electrical torque */
    float elecTorqueOld2;                                 /*!< Last two period electrical torque */
    float mechFreqErr;                                    /*!< Mechaine frequency error */
    float elecTorqueErr;                                  /*!< Electrical torque error */
    float inertiaCof;                                     /*!< Inertia coefficient */
    float inertiaCofOld;                                  /*!< Last period inertia coefficient */
    float inertiaGain;                                    /*!< Moment of inertia gain */
    float inertiaTrue;                                    /*!< Moment of inertia true value */
    float inertiaTrueFilt;                                /*!< Moment of inertia true value after filter */
    float inertiaTrueOld;                                 /*!< Last period moment of inertia true value */
    float inertiaTrueErr;                                 /*!< Moment of inertia true value error */
    float numerator;                                      /*!< Moment of inertia true value */
    float denominator;                                    /*!< Moment of inertia true value */
    float flux;                                           /*!< True value of motor magnetic flux */
    float ykCoeff;                                        /*!< The yk coefficient of FFLS */
    float lkCoeff;                                        /*!< The lk coefficient of FFLS */
    float pkCoeff;                                        /*!< The pk coefficient of FFLS */
    float pkCoeffOld;                                     /*!< Last period pk coefficient of FFLS */
    float forgetFactor;                                   /*!< The forget factor of FFLS */
    float period;                                         /*!< The operated period of inertia identify calculation function */
    float errThrold;                                      /*!< The inertia error threhold */
    float inertiaSteadyValue;                             /*!< Inertia of steady true value */
} INERTIA_IDENTIFY_STRUCT;

/*!
* @brief Identify inertia configuration parameters struct
*/
typedef struct
{
    int32_t steadyTime;                                     /*!< Inertia error steady judge time */
    int32_t inertiaOldTime;                                 /*!< Last period inertia refresh time */
    float inertiaGain;                                      /*!< Moment of inertia gain */
    float inertiaCofInit;                                   /*!< Inertia identify coefficient initial value */
    float operationPeriod;                                  /*!< Inertia identify operation period */
    float errorThrold;                                      /*!< Inertia error threshold */

    float forgetFactor;                                     /*!< Forget factor of inertia identify */
    float pkInit;                                           /*!< Pk parameter initial value of inertia identify */
} INERTIA_IDENTIFY_CFG;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Output parameter identify given rotor position.
*
* @param[in] paramIdentifyCtrl: the whole parameter identify control variable, pointer to PARAM_IDENTIFY_MAIN_CTRL struct
* @return setElecAngle rotor position
*/
float Get_ParamIdentify_ElecAngle(PARAM_IDENTIFY_MAIN_CTRL *paramIdentifyCtrl);

/*!
* @brief Parameter identify function main status machine. Contain five steps.
*        User can modify settings to chose which parameter identification.
*        IDENTIFY_RS_DCTEST: motor Rs identify.
*        IDENTIFY_LDQ_STALLTEST: lq lq identify.
*        IDENTIFY_FLUX_ROTATIONTEST: flux identify.
*        IDENTIFY_INERTIA: inertia identify.
*        IDENTIFY_END: identification finished and output the results.
*
* @param[in] paramIdentify: parameter identify data variable, pointer to PARAM_IDENTIFY_STRUCT struct
* @param[in] pSTC: pointer to STC_PARAM structure
* @param[in] hwBoard: HW_BOARD type pointer
* @return none
*/
void ParamIdentify(PARAM_IDENTIFY_STRUCT *paramIdentify, STC_PARAM *pSTC, HW_BOARD *hwBoard);

/*!
* @brief Ld Lq Parameter identify interrupt service function. The basic principle is inject DC voltage and sample phase current slope.
*        Then calcule phase inductance and Ld Lq. This parameter can be used for sensorless control.
*
* @param[in] paramIdentify: parameter identify data variable, pointer to PARAM_IDENTIFY_STRUCT struct
* @param[in] paramLdqIdentify: the Ld Lq identify control variable, pointer to PARAM_LDQ_IDENTIFY_DATA struct
* @param[in] focVarsCtrl: FOC control variable, pointer to FOC_VARS_CTRL structure
* @return none
*/
void ParamIdentify_LdqCallBack(PARAM_IDENTIFY_STRUCT *paramIdentify, PARAM_LDQ_IDENTIFY_DATA *paramLdqIdentify, FOC_VARS_CTRL *focVarsCtrl);

/*!
* @brief Inertia parameter identify function. Give iqRef a value, make motor acceleration and save time.
*        Then calculate motor inertia base on iqRef and time.
*        It should be noted that this method to identify inertia only suit for hall sensor mode.
*
* @param[in] paramIdentify: parameter identify data variable, pointer to PARAM_IDENTIFY_STRUCT struct
* @param[in] pSTC: pointer to STC_PARAM structure
* @param[in] hall: variable which use in hall sensor control, pointer to HALL_SENSOR structure
* @return none
*/
//void ParamIdentify_Inertia(PARAM_IDENTIFY_STRUCT *paramIdentify, STC_PARAM *pSTC, HALL_SENSOR *hall);

/*!
* @brief Parameter identify function initialize. Clear main control struct, Rs identify control struct, Ld Lq identify control struct, Flux identify control struct, Inertia identify control struct.
*
* @param[in] paramIdentify: parameter identify data variable, pointer to PARAM_IDENTIFY_STRUCT struct
* @return none
*/
void ParamIdentify_Init(PARAM_IDENTIFY_STRUCT *paramIdentify);

/*!
* @brief Acim rotor parameters identification inject signal generation.
*
* @param[in] paramIdentify: parameter identify data variable, pointer to PARAM_IDENTIFY_STRUCT struct
* @param[in] STC: pointer to STC_PARAM type structure
* @return none
*/
void AcimId_RotorIdSignalGeneration(PARAM_IDENTIFY_STRUCT *paramIdentify, STC_PARAM *STC);

/*!
* @brief Acim mutual inductance rotation identification vf control.
*
* @param[in] paramIdentify: parameter identify data variable, pointer to PARAM_IDENTIFY_STRUCT struct
* @param[in] STC: pointer to STC_PARAM type structure
* @return none
*/
void AcimId_RotationVfControl(PARAM_IDENTIFY_STRUCT *paramIdentify, STC_PARAM *STC);

/*!
* @brief Acim identification focIsr process.
*
* @param[in] paramIdentify: parameter identify data variable, pointer to PARAM_IDENTIFY_STRUCT struct
* @param[in] signal: pointer to SIGNAL_PHASE_SHIFT type structure
* @param[in] STC: pointer to STC_PARAM type structure
* @return none
*/
void AcimId_IdentificationIsr(PARAM_IDENTIFY_STRUCT *paramIdentify, SIGNAL_PHASE_SHIFT *signal, STC_PARAM *STC);

/*!
* @brief Acim parameters identification initialization setting.
*
* @param[in] paramIdentify: parameter identify data variable, pointer to PARAM_IDENTIFY_STRUCT struct
* @param[in] signal: pointer to SIGNAL_PHASE_SHIFT type structure
* @param[in] STC: pointer to STC_PARAM type structure
* @return none
*/
void AcimId_ParamInitValueSet(PARAM_IDENTIFY_STRUCT *paramIdentify, SIGNAL_PHASE_SHIFT *signal, STC_PARAM *STC);

/*!
* @brief Acim parameters identification initialization.
*
* @param[in] paramIdentify: parameter identify data variable, pointer to PARAM_IDENTIFY_STRUCT struct
* @param[in] signal: pointer to SIGNAL_PHASE_SHIFT type structure
* @return none
*/
void AcimId_ParamInit(PARAM_IDENTIFY_STRUCT *paramIdentify, SIGNAL_PHASE_SHIFT *signal);

/*!
* @brief PMSM Ldq identification sine command generation, set idRef and iqRef.
*
* @param[in] paramIdentify: pointer to PARAM_IDENTIFY_STRUCT type structure
* @param[in] STC: pointer to STC_PARAM type structure
* @return none
*/
//void PmsmId_LdqIdCommondGen(PARAM_IDENTIFY_STRUCT *paramIdentify, STC_PARAM *STC);

/*!
* @brief PMSM Ldq identification sine command generation, set idRef and iqRef.
*
* @param[in] paramIdentify: pointer to PARAM_IDENTIFY_STRUCT type structure
* @param[in] STC: pointer to STC_PARAM type structure
* @param[in] signal: pointer to SIGNAL_PHASE_SHIFT type structure
* @return none
*/
//void PmsmId_LdqCalcIsr(PARAM_IDENTIFY_STRUCT *paramIdentify, STC_PARAM *STC, SIGNAL_PHASE_SHIFT *signal);

/*!
* @brief PMSM Ldq identification configuration parameters initialization.
*
* @param[in] paramIdentify: pointer to PARAM_IDENTIFY_STRUCT type structure
* @param[in] STC: pointer to STC_PARAM type structure
* @param[in] signal: pointer to SIGNAL_PHASE_SHIFT type structure
* @return none
*/
//void PmsmId_LdqParamInitValueSet(PARAM_IDENTIFY_STRUCT *paramIdentify, STC_PARAM *STC, SIGNAL_PHASE_SHIFT *signal);

/*!
* @brief PMSM Ldq identification parameters initialization.
*
* @param[in] paramIdentify: pointer to PARAM_IDENTIFY_STRUCT type structure
* @param[in] signal: pointer to SIGNAL_PHASE_SHIFT type structure
* @return none
*/
//void PmsmId_LdqParamInit(PARAM_IDENTIFY_STRUCT *paramIdentify, SIGNAL_PHASE_SHIFT *signal);

/*!
* @brief Inertia identify parameters initialization.
*
* @param[in] inertiaIdentify: pointer to INERTIA_IDENTIFY_STRUCT structure
* @param[in] inertiaIdentCfg: pointer to INERTIA_IDENTIFY_CFG structure
* @param[in] motor: pointer to MOTOR_PARAM structure
* @return none
*/
void InertiaIdentify_Init(INERTIA_IDENTIFY_STRUCT *inertiaIdentify, INERTIA_IDENTIFY_CFG *inertiaIdentCfg, MOTOR_PARAM *motor);

/*!
* @brief Inertia identify calculation function of MRAS.
*
* @param[in] inertiaIdentify: pointer to INERTIA_IDENTIFY_STRUCT structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] elecFreq: motor electric frequency pu value in Q15
* @param[in] iqFdb: q axis current pu value in Q15
* @return none
*/
void InertiaIdentify_MRAS(INERTIA_IDENTIFY_STRUCT *inertiaIdentify, MOTOR_PARAM_PU *paramPu, float elecFreq, float iqFdb);

/*!
* @brief Inertia identify calculation function of FFLS.
*
* @param[in] inertiaIdentify: pointer to INERTIA_IDENTIFY_STRUCT structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] elecFreq: motor electric frequency pu value in Q15
* @param[in] iqFdb: q axis current pu value in Q15
* @return none
*/
void InertiaIdentify_FFLS(INERTIA_IDENTIFY_STRUCT *inertiaIdentify, MOTOR_PARAM_PU *paramPu, float elecFreq, float iqFdb);

/*!
* @brief PMSM Ldq identification high frequency sine method task is ADC interrupt callback, get current and voltage sampling data.
*
* @param[in] paramIdentify: parameter identify data variable, pointer to PARAM_IDENTIFY_STRUCT struct
* @param[in] stc: pointer to STC_PARAM type structure
* @return none
*/
void PMSMLdqId_IsrTask(PARAM_IDENTIFY_STRUCT *paramIdentify, STC_PARAM *stc);

/*!
* @brief PMSM Ldq identification sine command generation, set idRef and iqRef.
*
* @param[in] paramIdentify: parameter identify data variable, pointer to PARAM_IDENTIFY_STRUCT struct
* @param[in] stc: pointer to STC_PARAM type structure
* @return none
*/
void PMSMLdqId_IsrTask_CommandGen(PARAM_IDENTIFY_STRUCT *paramIdentify, STC_PARAM *stc);

/*!
* @brief Motor parameter identification task in 4ms period.
*
* @param[in] paramIdentify: parameter identify data variable, pointer to PARAM_IDENTIFY_STRUCT struct
* @param[in] stc: pointer to STC_PARAM type structure
* @return none
*/
void ParamIdentify_4msTask(PARAM_IDENTIFY_STRUCT *paramIdentify, STC_PARAM *stc);

/*!
* @brief FFT calculation in float format.
*
* @param[in] sig: TYPE_FFT_SAMPLE_DATA type pointer
* @param[in] fft: FFT_PARAM type pointer
* @return none
*/
void FFT_Calc(TYPE_FFT_SAMPLE_DATA *sig, FFT_PARAM *fft);

/*!
* @brief FFT calculation in fixed-point format.
*
* @param[in] sig: TYPE_FFT_SAMPLE_DATA_Q15 type pointer
* @param[in] fft: FFT_PARAM_Q15 type pointer
* @return none
*/
void FFT_Calc_Q15(TYPE_FFT_SAMPLE_DATA_Q15 *sig, FFT_PARAM_Q15 *fft);

/* ======================================  Functions define  ======================================== */

#endif /* __PARAM_ID_H__ */

/* =============================================  EOF  ============================================== */
