/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/


#ifndef __PID_REGULATOR_H__
#define __PID_REGULATOR_H__
/*!
* @file pid_regulator.h
*
* @brief This file provides different types of PID functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "iqmath.h"
#include "param_init.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief PID regulator input parameters structure.
*/
typedef struct
{
    float refPu;              /*!< PID input reference value */
    float fbkPu;              /*!< PID input feedback value */
    float maxPu;              /*!< Parameter: upper saturation limit */
    float minPu;              /*!< Parameter: lower saturation limit */
    float deadZonePu;         /*!< Error deadband setting */
    float feedForwardPu;      /*!< Feedforward compensation */
} PID_REGULATOR_IN;

/*!
* @brief PID regulator output parameters structure.
*/
typedef struct
{
    float outPu;              /*!< PID output value */
    float errPu;              /*!< Data: Err, errPu = refPu - fbkPu */
    float err1Pu;             /*!< Data: Err of last calculation period*/
    float err2Pu;             /*!< Data: Err of the penultimate calculation period */
    float upPu;               /*!< Data: proportional term */
    float uiPu;               /*!< Data: integral term */
    float v1Pu;               /*!< Data: pre-saturated controller output */
    float i1Pu;               /*!< Data: integrator storage: ui(k-1) */
    float w1Pu;               /*!< Data: saturation record: [u(k-1) - v(k-1)], v.ui = v.ui + v.ki * v.up + v.kc * v.w1 */
    float outPuFliter;        /*!< PID output after smooth filter */
    float outErr;             /*!< Err of PID output */
    float outPu1;             /*!< PID output before limiting amplitude */
    float deCoup;             /*!< PID output decouple value */
} PID_REGULATOR_OUT;

/*!
* @brief PID regulator calculation coefficient structure.
*/
typedef struct
{
    float kpPu;               /*!< Proportional gain of PID regulator */
    float kiPu;               /*!< Integral gain of PID regulator */
    float kcPu;               /*!< Reserve */
    float kdPu;               /*!< Reserve */
} PID_REGULATOR_COF;

/*!
* @brief Integral separation PID parameter structure.
*/
typedef struct
{
    uint8_t integralEnable;     /*!< switch to enable integral calculation */

    float kp;                 /*!< Proportional gain of PID regulator */
    float ki;                 /*!< Integral gain of PID regulator */
    float kd;                 /*!< Reserve */
    float errPu;              /*!< Data: Err, errPu = refPu - fbkPu */
    float integralTerm;       /*!< Data: Integral term */
    float maxOutputPu;        /*!< Parameter: upper saturation limit */
    float minOutputPu;        /*!< Parameter: upper saturation limit */
    float upOutPu;            /*!< Proportional part of output */
    float uiOutPu;            /*!< Integral part of output */
    float pidOut;             /*!< Pid output */
} PID_SEPARATE_TYPE;

/*!
* @brief Configuration parameter structure. Used to set PI parameters in different PID regulators.
*/
typedef struct
{
    float torqueKp;               /*!< Torque loop PID kp */
    float torqueKi;               /*!< Torque loop PID ki */
    float torqueKd;               /*!< Torque loop PID kd */
    float fluxKp;                 /*!< Flux loop PID kp */
    float fluxKi;                 /*!< Flux loop PID ki */
    float fluxKd;                 /*!< Flux loop PID kd */
    float maxtorque;              /*!< Torque loop PID max */
    float mintorque;              /*!< Torque loop PID min */
    float maxflux;                /*!< Flux loop PID max */
    float minflux;                /*!< Flux loop PID min */
    float speedKp;                /*!< Speed loop PID kp */
    float speedKi;                /*!< Speed loop PID ki */
    float speedKd;                /*!< Speed loop PID kd */
    float maxSpeed;               /*!< Speed loop PID max */
    float minSpeed;               /*!< Speed loop PID min */
    float positionKp;             /*!< Position loop PID kp */
    float positionKi;             /*!< Position loop PID ki */
    float positionKd;             /*!< Position loop PID kd */
    float maxPosition;            /*!< Position loop PID max */
    float minPosition;            /*!< Position loop PID min */
    float curWicPu;              /*!< Current loop operating frequency pu value */
    float speedWic;              /*!< Speed loop operating frequency pu value */
    uint16_t curPidSelfLearn;       /*!< Current loop PID self learn flag */
    uint16_t speedPidSelfLearn;     /*!< Speed loop PID self learn flag */
    float damp;                  /*!< Speed loop damp */
    float feedbackGain;           /*!< Feedback gain */
} PID_CFG;

/*!
* @brief PID structure. Integrates the input, coefficient and output of PID ragulator.
*/
typedef struct
{
    PID_REGULATOR_IN *pPidInput;    /*!< Input parameters structure of PID */
    PID_REGULATOR_COF *pPidCoef;    /*!< Coefficient structure of PID */
    PID_REGULATOR_OUT *pPidOutput;  /*!< Output parameters structure of PID */
} PID_REGULATOR;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Series type PID regulator. Series PID consists of proportional,
*        integral and differential terms in series which transfer function is:
*                   ki
*        kp * (1 + ---) * (1 + s * kd)
*                   s
*        where kp is the proportional gain, ki is the integration coefficient,
*        and td is the differential coefficient.
*
* @param[in] pid: pointer to PID_REGULATOR structure
* @return outPu: Output value of PID ragulator
*/
float PID_SeriesRegulator(PID_REGULATOR *pid);

/*!
* @brief Parallel type PID regulator. Parallel PID consists of proportional,
*        integral and differential terms in parallel which transfer function is:
*              ki
*        kp + ---- + kd * s
*              s
*        where kp is the proportional gain, ki is the integration coefficient,
*        and kd is the differential coefficient.
*
* @param[in] pid: pointer to PID_REGULATOR structure
* @return outPu: Output value of PID ragulator
*/
float PID_ParallelRegulator(PID_REGULATOR *pid);

/*!
* @brief Position type PID regulator. Positional PID control algorithm directly gives
*        the actual value of the control quantity, which can be directly input into the
*        actuator; the output of the current sampling time is related to each state in
*        the past, and the error value should be accumulated in calculation.
*
* @param[in] pid: pointer to PID_REGULATOR structure
* @return outPu: Output value of PID ragulator
*/
float PID_PositionalRegulator(PID_REGULATOR *pid);

/*!
* @brief Incremental type PID regulator. The output of incremental PID is only
*        the increment value delta_u(k) of control quantity. the cumulative
*        function of the actuator is realized by the formula: u(k) = u(k-1) + delta_u(k).
*        Once kp, ki, kd are determinde, the control increment can be obtained by
*        using the deviation value (error) of the last three control variables.
*
* @param[in] pid: pointer to PID_REGULATOR structure
* @return outPu: Output value of PID ragulator
*/
float PID_IncrementalRegulator(PID_REGULATOR *pid);

/*!
* @brief Feedback suppression PI regulator. The error of PID regulator
*        output is fed back to restrain the saturation of the integrator.
*
* @param[in] pid: pointer to PID_REGULATOR structure
* @param[in] pidParam: pointer to PID_CFG structure
* @return outPu: Output value of PID ragulator
*/
float PID_FeedbackRegulator(PID_REGULATOR *pid, PID_CFG *pidParam);

/*!
* @brief Current loop complex vector decoupling PID regulator, dq axis current can be decoupled by using this type of PID.
*
* @param[in] dPid: pointer to d axis PID_REGULATOR structure
* @param[in] qPid: pointer to q axis PID_REGULATOR structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] freq: motor electric frequence
*/
void Current_Decoup_Regulator(PID_REGULATOR *dPid, PID_REGULATOR *qPid, MOTOR_PARAM_PU *paramPu, float freq);

/* ======================================  Functions define  ======================================== */

#endif /* __PID_REGULATOR_H__ */

/* =============================================  EOF  ============================================== */
