/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/


#ifndef __POWER_DCCUR_EST_H__
#define __POWER_DCCUR_EST_H__

/*!
* @file power_dccur_estimate.h
*
* @brief This file provides input power, controller output power and bus current estimate functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "param_init.h"
#include "svgen.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Input and output power estimate parameters structure.
*/
typedef struct
{
    float dltaTheta;      /*!< Pwm delay theta */
    float uqcomp;         /*!< Dead zone q axis voltage compensation */
    float powerPuOld;     /*!< Last period output power pu value */
    float powerTrue;      /*!< Output power true value */
    float powerInPu;      /*!< Intput power pu value */
    float powerInTrue;    /*!< Intput power true value */
    float torqPu;         /*!< Output torque pu value */
    float torqTure;       /*!< Output torque true value */
    float udRefDelay;     /*!< D axis given voltage delay value */
    float uqRefDelay;     /*!< Q axis given voltage delay value */
    float efficient;      /*!< Output efficient */
    float powerPu;        /*!< Output power pu value */
} POWER_EST;

/*!
* @brief Bus current estimate parameters structure.
*/
typedef struct
{
    float ta;             /*!< A phase duty cycle */
    float tb;             /*!< B phase duty cycle */
    float tc;             /*!< C phase duty cycle */
    float dcCurTrue;      /*!< Bus current true value after amplification */
    float ia;             /*!< A phase current */
    float ib;             /*!< B phase current */
    float ic;             /*!< C phase current */
    float dcCur;          /*!< Bus current pu value */
    float dcCurOld;       /*!< Last period bus current pu value */
} DCCUR_EST;

/*!
* @brief Input and output power estimate configuration parameters structure.
*/
typedef struct
{
    float uqcomp;         /*!< Dead zone q axis voltage configuration compensation value */
    float pwmDelayCof;    /*!< Power estimate pwm delay configuration coefficient */
} POWEREST_CFG;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Input and output power estimate variables initialize, usually it is initialized only once before running.
*
* @param[in] powerEst: pointer to POWER_EST structure
* @return none
*/
void PowerEst_Init(POWER_EST *powerEst);

/*!
* @brief Input power and controller output power estimate calculate, which calculated bu d-q axis voltage,
*        d-q axis current, bus voltage and bus current.
*
* @param[in] stc: pointer to STC_PARAM structure
* @param[in] powerCfg: pointer to POWEREST_CFG structure
* @param[in] freq: motor electrical frequency pu value in Q15
* @param[in] dcCurTrue: after amplification bus current true value
* @param[in] powerEst: pointer to POWER_EST structure
* @return none
*/
void PowerEst_Calc(STC_PARAM *stc, POWEREST_CFG *powerCfg, float freq, float dcCurTrue, POWER_EST *powerEst);

/*!
* @brief Bus current estimate variables initialize, usually it is initialized only once before running.
*
* @param[in] dcCurEst: pointer to DCCUR_EST structure
* @return none
*/
void DcCurEst_Init(DCCUR_EST *dcCurEst);

/*!
* @brief Estimate the bus current through the three phase pwm duty cycle and three phase current.
*
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] pwm: pointer to PWM_GEN_OUT structure
* @param[in] ampFactor: bus current true value amplification factor, range:1-600
* @param[in] dcCurEst: pointer to DCCUR_EST structure
* @return none
*/
void DcCurEst_Calc(MOTOR_PARAM_PU *paramPu, FOC_VARS_CTRL *focVars, PWM_GEN_OUT *pwm, uint16_t ampFactor, DCCUR_EST *dcCurEst);

/* ======================================  Functions define  ======================================== */

#endif /* __POWER_DCCUR_EST_H__ */

/* =============================================  EOF  ============================================== */
