/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/


#ifndef __SMOPOS_H__
#define __SMOPOS_H__

/*!
* @file smopos.h
*
* @brief This file calculate the rotar angle and speed estimated by the sliding mode control observer
*        for sensorless vector control functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "svgen.h"
#include "autospd_regulator.h"
#include "param_init.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Sliding mode control configuration parameters structure.
*/
typedef struct
{
    float kslide0;            /*!< Open loop configuration sliding mode gain */
    float maxErr0;            /*!< Open loop configuration sliding mode boundary */
    float kslf0;              /*!< Open loop configuration sliding mode filter coefficient */
    float kslide1;            /*!< Close loop configuration sliding mode gain */
    float maxErr1;            /*!< Close loop configuration sliding mode boundary */
    float kslf1;              /*!< Close loop configuration sliding mode filter coefficient */
    float fullOrderGain;      /*!< Full order smc gain */
} SMC_CFG;

/*!
* @brief Sliding mode control parameters structure.
*/
typedef struct
{
    float kslide;             /*!< Sliding mode gain */
    float maxErr;             /*!< Sliding mode boundary */
    float kslf;               /*!< Sliding mode filter coefficient */
    float kslfa;              /*!< Sliding mode filter coefficient a */
    float kslfb;              /*!< Sliding mode filter coefficient b */
    float gain;               /*!< full order smc gain */

    float fsmopos;            /*!< Sliding mode control fsmopos coefficient */
    float gsmopos;            /*!< Sliding mode control gsmopos coefficient */

    float vAlpha;             /*!< Sliding mode control alpha axis voltage */
    float vBeta;              /*!< Sliding mode control beta axis voltage */

    float zAlpha;             /*!< Sliding mode control alpha axis back emf before filter */
    float zBeta;              /*!< Sliding mode control beta axis back emf before filter */

    float estIalpha;          /*!< Sliding mode control estimated alpha axis current */
    float estIbeta;           /*!< Sliding mode control estimated beta axis current */

    float eAlpha;             /*!< Sliding mode control alpha axis back emf after filter */
    float eBeta;              /*!< Sliding mode control beta axis back emf after filter */
    float bemfAmp;            /*!< Sliding mode control back emf amp */
    float estBemfSq;          /*!< Sliding mode control estimated the square of back emf */

    float estTheta;          /*!< Sliding mode control estimated electrical theta */
    float compCoeff;          /*!< Sliding mode control compensation coefficieng */
    float eAlphaComp;         /*!< Sliding mode control alpha axis back emf compensation value */
    float eBetaComp;          /*!< Sliding mode control beta axis back emf compensation value */
    float estFre;             /*!< Sliding mode control estimated electrical frequency */
    float eAlphaFullOrder;    /*!< Full order sliding mode control alpha axis back emf before filter */
    float eBetaFullOrder;     /*!< Full order sliding mode control beta axis back emf before filter */
    float timeCoeff;          /*!< Sliding mode control time coefficient */
    float cutFreq;           /*!< Low pass filter cut off frequency */
} SMC_ESTIMATOR;

/*!
* @brief Sliding mode control theta compensation structure.
*/
typedef struct
{
    float maxCompTheta;  /*!< Maximum compensation angle */
    float initCompTheta; /*!< Initial compensation angle */
    float cutFreq;       /*!< Cut off frequency */
    float omegaAvg;      /*!< Average speed */
    float theta0Gain;    /*!< Filter compensation angle gain */
    float theta0;        /*!< Filter compensation angle */
} THETA_COMPENSATOR;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Calculate the rotor speed through by the smc observer PLL method, usually it is executed in interrupt.
*
* @param[in] pidPll: pointer to PID_REGULATOR structure
* @param[in] focVarsCfg: variable which use to configure FOC control, pointer to FOC_VARS_CFG structure
* @param[in] smo: pointer to SMC_ESTIMATOR structure
* @return motor electrical frequency estimated value by PLL in Q15
*/
float Asr_SpeedPLLCalc(PID_REGULATOR *pidPll, FOC_VARS_CFG *focVarsCfg, SMC_ESTIMATOR *smo);

/*!
* @brief Sliding mode control parameter setting, including sliding mode boundary setting, sliding mode gain setting,
*        and filter coefficient setting.
*
* @param[in] e0: smo boundary value
* @param[in] kslide: smo gain
* @param[in] kslf: smo filter coefficient
* @param[in] smc: pointer to SMC_ESTIMATOR structure
* @return none
*/
void Smc_SetParam(float e0, float kslide, float kslf, SMC_ESTIMATOR *smc);

/*!
* @brief Calculate sliding mode control coefficient through motor resistance, inductance and PWM frequency.
*
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] smc: pointer to SMC_ESTIMATOR structure
* @return none
*/
void Smc_CoeffCale(MOTOR_PARAM_PU *paramPu, SMC_ESTIMATOR *smc);

/*!
* @brief Sliding mode control parameters initialize, including sliding mode boundary, sliding mode gain, filter coefficient,
*        control coefficients and theta compensation parameters initialize, usually it is initialized only once before running.
*
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] focVarsCfg: variable which use to configure FOC control, pointer to FOC_VARS_CFG structure
* @param[in] smoCfg: pointer to SMC_CFG structure
* @param[in] smc: pointer to SMC_ESTIMATOR structure
* @return none
*/
void Smc_ParamInit(MOTOR_PARAM_PU *paramPu, FOC_VARS_CFG *focVarsCfg, SMC_CFG *smoCfg, SMC_ESTIMATOR *smc);

/*!
* @brief Calculate the rotar angle and speed estimated by sliding mode control observer for sensorless vector control,
*        which contains ATAN and PLL angle calculation methods, usually it is executed in interrupt.
*
* @param[in] STC: pointer to STC_PARAM structure
* @param[in] smc: pointer to SMC_ESTIMATOR structure
* @param[in] speedCalc: pointer to ESTIMATED_SPEED structure
* @param[in] pidPll: pointer to PID_REGULATOR structure
* @return motor electrical angle in Q16
*/
float Smc_PositionEstimator(STC_PARAM *STC, SMC_ESTIMATOR *smc, ESTIMATED_SPEED *speedCalc, PID_REGULATOR *pidPll);

/*!
* @brief Get the square of smo back emf, through summing the squares of alpha axis and beta axis back emf.
*
* @param[in] smc: pointer to SMC_ESTIMATOR structure
* @return none
*/
void Smc_GetBemfSq(SMC_ESTIMATOR *smc);

/*!
* @brief Sliding mode control angle compensation initialize, including cutoff frequency and theta gain initialize,
*        usually it is initialized only once before running.
*
* @param[in] comp: pointer to THETA_COMPENSATOR structure
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure
* @param[in] focVarsCfg: pointer to FOC_VARS_CFG structure
* @param[in] smc: pointer to SMC_ESTIMATOR structure
* @return none
*/
void Theta_CompensatorInit(THETA_COMPENSATOR *comp, MOTOR_PARAM_PU *paramPu, FOC_VARS_CFG *focVarsCfg, SMC_ESTIMATOR *smc);

/*!
* @brief Sliding mode control angle compensation value calculate by the motor average speed and cutoff frequency.
*
* @param[in] comp: pointer to THETA_COMPENSATOR structure
* @param[in] speed: motor speed pu value in Q15
* @return none
*/
void Theta_CompensatorCalc(THETA_COMPENSATOR *comp, float speed);

/* ======================================  Functions define  ======================================== */

#endif /* __SMOPOS_H__ */

/* =============================================  EOF  ============================================== */
