/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __SVGEN_H__
#define __SVGEN_H__
/*!
* @file svgen.h
*
* @brief This file provides space vector pulse width modulation (SVPWM), dead time compensation and narrow pulse width limiting functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "param_init.h"

/* ============================================  Define  ============================================ */
#define SECTOR_1            1                               /*!< Sector 1 */
#define SECTOR_2            2                               /*!< Sector 2 */
#define SECTOR_3            3                               /*!< Sector 3 */
#define SECTOR_4            4                               /*!< Sector 4 */
#define SECTOR_5            5                               /*!< Sector 5 */
#define SECTOR_6            6                               /*!< Sector 6 */

#define DEGREEE_5           (5.0f / 360)     /*!< Define 5 degree in Q16 format */
#define DEGREEE_30          (30.0f / 360)    /*!< Define 30 degree in Q16 format */
#define DEGREEE_55          (55.0f / 360)    /*!< Define 55 degree in Q16 format */
#define DEGREEE_60          (60.0f / 360)    /*!< Define 60 degree in Q16 format */
#define DEGREEE_90          (90.0f / 360)    /*!< Define 90 degree in Q16 format */

#define MIN_SAMPLE_WINDOW   (48 << 2)                       /*!< 8us */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief SVPWM input parameter structure.
*/
typedef struct
{
    float pwmAlfaPu;                                  /*!< Voltage alpha in pu */
    float pwmBetaPu;                                  /*!< Voltage beta in pu */
    float udcPu;                                      /*!< Bus voltage in pu */
    float modRatio;                                  /*!< Modulation ratio */
} PWM_GEN_IN;

/*!
* @brief SVPWM output parameter structure.
*/
typedef struct
{
    uint16_t pwmTa;                                     /*!< A phase output compare value */
    uint16_t pwmTb;                                     /*!< B phase output compare value */
    uint16_t pwmTc;                                     /*!< C phase output compare value */
    uint16_t pwmLimitFlag;                              /*!< Reach voltage limie flag */
    uint16_t sector;                                    /*!< Svpwm voltage vector sector */
    float svgT1;                                      /*!< First effective vector length */
    float svgT2;                                      /*!< Second effective vector length */
    float svgTa;                                      /*!< A phase pwm duty */
    float svgTb;                                      /*!< B phase pwm duty */
    float svgTc;                                      /*!< C phase pwm duty */
} PWM_GEN_OUT;

/*!
* @brief Dead zone compensation ontrol parameter structure.
*/
typedef struct
{
    uint16_t curIsSector;                               /*!< Current Is sector */
    float filterCurIs;                               /*!< Current value after filter */
    float curIsDegree;                               /*!< Current Is degree Q15 */
    float curPhase;                                  /*!< Current Is phase */
} DEAD_BAND_COMP_STRUCT;

/*!
* @brief Over modulation configuration structure.
*/
typedef struct
{
    uint8_t overModEnable;                              /*!< Over modulation enable */
} OVER_MODULATION_CFG;

/*!
* @brief Dead zone compensation configuration structure.
*/
typedef struct
{
    float cutoffCurVal;                              /*!< Decrease compensation cutoff curent value */
    float maxCompFreq;                               /*!< Max compensation frequency */
} DEADCOMP_CFG;

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Space Vector PWM Algorithm, according to the amplitude and
*        angle of the voltage vector, calculates the turn-on sequence
*        and duration of the three-phase inverter.
*
* @param[in] focVars: pointer to FOC_VARS structure
* @param[in] out: SVPWM out data variable, pointer to PWM_GEN_OUT structure
* @param[in] OverModulationCfg: SVPWM over modulation config variable, pointer to OVER_MODULATION_CFG structure
* @return none
*/
void Svpwm_Gen(FOC_VARS *focVar, PWM_GEN_OUT *out, OVER_MODULATION_CFG *OverModulationCfg);

/*!
* @brief Dead zone compensation function. According to the current vector,
*        the  dead time of power switch in three-phase inverter is compensated.
*
* @param[in] deadCompCtrl: the dead zone compensation data variable, pointer to DEAD_BAND_COMP_STRUCT struct
* @param[in] pwmValue: SVPWM out data variable, pointer to PWM_GEN_OUT structure
* @param[in] deadCompCfg: the dead zone compensation configuration variable, pointer to DEADCOMP_CFG struct
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void DeadBandCompCal(DEAD_BAND_COMP_STRUCT *deadCompCtrl, PWM_GEN_OUT *pwmValue, DEADCOMP_CFG *deadCompCfg, FOC_VARS *focVars);

/*!
* @brief In some cases requre output five segments SVPWM.This function can transfer seven segments to five sgements SVPWM.
*        If user want to output five segments SVPWM, can modify settings to enable.
*
* @param[in] pwmValue: SVPWM out data variable, pointer to PWM_GEN_OUT structure
* @return none
*/
void PluseOutFiveSeg(PWM_GEN_OUT *pwmValue);

/*!
* @brief When output SVPWM pulse, at high speed will have same narrow pulse, which is invalid pwm.
*        This function can eliminate norrow pulse, then enlare voltage output upper limit.
*
* @param[in] pwmValue: PWM_GEN_OUT type pointer, pointer to g_pwmValue variable, which is the SPVWM out data struct
* @param[in] focVarsCfg: FOC_VARS_CFG type pointer, pointer to defFocVars, which configure whole FOC control
* @return none
*/
void NarrowPulseProcess(PWM_GEN_OUT *pwmValue, FOC_VARS_CFG *focVarcfg);

/*!
* @brief In order increase the max motor speed, this function enlarge the output composite voltage vector from inscribed circle to hexagon.
*        It should be noted that when motor run in over modulation zone, the current will be distorted, it is normal.
*
* @param[in] out: SVPWM out data variable, pointer to PWM_GEN_OUT structure
* @return none
*/
void OverModulation(PWM_GEN_OUT *out);

/*!
* @brief Control pwm output, if pwm output is enabled, the pwm mask is invalid, else if pwm output is disabled, the pwm mask is valid.
*
* @param[in] enable: Pwm output enable or disable
* @param[in] focVarsCtrl: FOC control variable, pointer to FOC_VARS_CTRL structure
* @return none
*/
void PWM_OutputMask(uint8_t enable, FOC_VARS_CTRL *focVarsCtrl);

/*!
* @brief Three phase voltage calculate through bus voltage reconstruction, usually it is executed in interrupt.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] pwm: pointer to PWM_GEN_OUT structure
* @return three phase volatge pu value in Q15
*/
void GetThreePhaseVolt(FOC_VARS_CTRL *focVars, PWM_GEN_OUT *pwm);

/* ======================================  Functions define  ======================================== */

#endif /* __SVGEN_H__ */

/* =============================================  EOF  ============================================== */
