/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __VF_COMPENSATE_H__
#define __VF_COMPENSATE_H__
/*!
* @file vf_compensate.h
*
* @brief This file provides V/f control compensation functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */

#include "svgen.h"
#include "param_init.h"


/* ============================================  Define  ============================================ */
#define FWD_ACC             1           /*!< Motor in forward acceleration state */
#define FWD_DEC             2           /*!< Motor in forward deceleration state */
#define REV_ACC             3           /*!< Motor in reverse acceleration state */
#define REV_DEC             4           /*!< Motor in reverse acceleration state */
#define STABLE              5           /*!< Motor in stable or static state */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief V/f control characteristic curve input data structure.
*/
typedef struct
{
    float motorFreqRatedPu;          /*!< Rated frequency of motor */
    float motorVoltRatedPu;          /*!< Rated voltage of motor */
    float motorMaxFreqPu;            /*!< Max frequency of motor */
    float torqBoostCutoffFreqPu;     /*!< V/f control torque boost cutoff frequency */
    float torqBoostVoltPct;          /*!< V/f control percentage of voltage boost relative to rated voltage */
    float vfManualPoint1FreqPu;      /*!< V/f custom curve point 1 frequency */
    float vfManualPoint1VoltPct;     /*!< V/f custom curve point 1 voltage percent */
    float vfManualPoint2FreqPu;      /*!< V/f custom curve point 2 frequency */
    float vfManualPoint2VoltPct;     /*!< V/f custom curve point 2 voltage percent */
    float vfManualPoint3FreqPu;      /*!< V/f custom curve point 3 frequency */
    float vfManualPoint3VoltPct;     /*!< V/f custom curve point 3 voltage percent */
} VF_CURVE_INPUT_PARAM;

/*!
* @brief V/f control input frequency data structure.
*/
typedef struct
{
    float vfCtrlFrqCurValPu;          /*!< V/f control frequency set value */
    float vfCtrlSpeedSetPu;           /*!< Current frequency target value */
    float vfCtrlFrqBeforeComp;        /*!< Frequency before compensate */
    float vfCtrlFrqCompBuf;           /*!< Frequency buf value */
    float vfCtrlFrqAfterComp;         /*!< Frequency after compensate */
} VF_INPUT;

/*!
* @brief V/f control output data structure.
*/
typedef struct
{
    float vfOutputVoltPu;            /*!< V/f control voltage output value */
    float synFrq;                     /*!< Current synchronization frequency (to calculate electric angle, V/f and VC, induction motor) */
    float phaseAngleUa;              /*!< V/f control electric angle (V/f and VC, induction motor) */
} VF_OUTPUT;

/*!
* @brief V/f control voltage electric angle data structure.
*/
typedef struct
{
    float elecTheta;                 /*!< V/f control voltage electric angle */
} VF_THETA_CAL;

/*!
* @brief V/f control DC start-up data structure.
*/
typedef struct
{
    uint16_t vfStartUpStatus;           /*!< V/f control DC startup status */
    uint16_t vfStartUpCnt;              /*!< V/f control DC startup count */
    float vfStartUpVolOutputPu;      /*!< V/f control DC startup voltage */
    float vfStartUpThetaPu;          /*!< V/f control DC startup theta */
} VF_DC_STARTUP;

/*!
* @brief V/f control current and voltage data structure.
*/
typedef struct
{
    float curAlfaPu;                  /*!< Current alfa */
    float curBetaPu;                  /*!< Current beta */
    float curDAxisPu;                 /*!< Current d, field */
    float curQAxisPu;                 /*!< Current q, torque */
    float voltAlfaPu;                 /*!< Voltage alfa */
    float voltBetaPu;                 /*!< Voltage beta */
    float curAmplitude;               /*!< Current amplitude */
} VF_VECTOR_PARAM;

/*!
* @brief V/f control compensation function data structure.
*/
typedef struct
{
    float currentLPF;                 /*!< V/f control reactive current value of LPF */
    float currentLPFOld;              /*!< Lst value of LPF output */
    float vfSuppFrqComp;              /*!< Swing suppression frequency compensation */
    float vfPowerSuppFreqComp;        /*!< fFrequency compensation of power suppression */
    float vfSuppVoltComp;             /*!< V/f control voltage compensation with reactive power */
    float vfSuppLPFCoeff;             /*!< Current swing suppression LPF coefficient */
    float powerSuppLPFCoeff;          /*!< Power swing suppression HPF coefficient */
    float vfPowerLPFCoeff;            /*!< Power swing suppression LPF coefficient */
    float activePowerPu;              /*!< V/f control active power value */
    float activePowerOldPu;           /*!< Last value of active power */
    float activePowerLPFPu;           /*!< Active power output of HPF */
    float activePowerHPFPu;           /*!< Last value of HPF output */
    float reactivePowerPu;            /*!< V/f control reactive power value */
    float reactivePowerLPF;           /*!< V/f control reactive power value of LPF */
    float reactivePowerLPFOldPu;      /*!< Last value of LPF output */
} VF_COMP_PARAM;

/*!
* @brief V/f control conpensation configuration data structure.
*/
typedef struct
{
    float curResonaFreqPoint1;        /*!< Current resonate suppression gain curve 1st point frequency value */
    float curResonaFreqPoint2;        /*!< Current resonate suppression gain curve 2nd point frequency value */
    float curResonaFreqPoint3;        /*!< Current resonate suppression gain curve 3rd point frequency value */
    float curResonaFreqPoint4;        /*!< Current resonate suppression gain curve 4th point frequency value */
    float curResonaGain;              /*!< Current resonate suppression gain */
    float curResonaFreqLimit;         /*!< Current resonate suppression frequency compensation limit */
    float powerSuppFreqPoint1;        /*!< Power suppression gain curve 1st point frequency value */
    float powerSuppFreqPoint2;        /*!< Power suppression gain curve 2nd point frequency value */
    float powerSuppFreqPoint3;        /*!< Power suppression gain curve 3rd point frequency value */
    float powerSuppFreqPoint4;        /*!< Power suppression gain curve 4th point frequency value */
    float powerSuppGain;              /*!< Power suppression gain */
    float powerSuppFreqLimit;         /*!< Power suppression frequency compensation limit */
} VF_COMP_SET;

/*!
* @brief V/f control frequency calculation data structure.
*/
typedef struct
{
    float vfCtrlFrqCurValPu;          /*!< V/f control frequency set value */
    float vfCtrlSpeedSetPu;           /*!< Current frequency target value */
} VF_FREQ_CALC_BUF;

/*!
* @brief V/f control frequency hopping data structure.
*/
typedef struct
{
    int16_t speedStatus;                /*!< Motion state */
    int16_t freqHopFlag;                /*!< Flag of hopping */
    float freqHopP1Bottom;            /*!< Bottom value of the 1st hopping region */
    float freqHopP1Top;               /*!< Top value of the 1st hopping region */
    float freqHopP2Bottom;            /*!< Bottom value of the 2nd hopping region */
    float freqHopP2Top;               /*!< Top value of the 2nd hopping region */
    float freqHopP3Bottom;            /*!< Bottom value of the 3rd hopping region */
    float freqHopP3Top;               /*!< Top value of the 3rd hopping region */
    float freqHopOutput;              /*!< Output of frequency hopping */
} VF_FREQ_HOP_PARAM;

/* ==========================================  Variables  =========================================== */
extern VF_INPUT         g_vfInput;      /*!< V/f control input: frequency set value */
extern VF_OUTPUT        g_vfOutput;     /*!< V/f control output: voltage and elec-theta, synchronous frequency */
extern VF_THETA_CAL     g_vfThetaCal;   /*!< V/f control parameters to calculate elec-theta */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief V/f control current and voltage transform.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @param[in] vf: pointer to VF_OUTPUT structure
* @param[in] vect: pointer to VF_VECTOR_PARAM structure
* @return none
*/
void Vf_VectorTransformProcess(FOC_VARS_CTRL *focVars, VF_OUTPUT *vf, VF_VECTOR_PARAM *vect);

/*!
* @brief V/f control current resonance suppression compensation.
*
* @param[in] current: motor current reactive component
* @param[in] in: pointer to VF_INPUT structure
* @param[in] param: pointer to VF_COMP_PARAM structure
* @param[in] set: pointer to VF_COMP_SET structure
* @return none
*/
void Vf_CurrentResonSuppProcess(float current, VF_INPUT *in, VF_COMP_PARAM *param, VF_COMP_SET *set);

/*!
* @brief V/f control power suppression compensation.
*
* @param[in] in: pointer to VF_INPUT structure
* @param[in] vect: pointer to VF_VECTOR_PARAM structure
* @param[in] param: pointer to VF_COMP_PARAM structure
* @param[in] set: pointer to VF_COMP_SET structure
* @param[in] rotorDir: motor rotation direction
* @return none
*/
void Vf_ActivePowerCompProcess(VF_INPUT *in, VF_VECTOR_PARAM *vect, VF_COMP_PARAM *param, VF_COMP_SET *set, int16_t rotorDir);

/*!
* @brief V/f control judgment of motor running ramp.
*
* @param[in] freqCmd: V/f control frequency target command value
* @param[in] freqCurrentCalc: current real time frequency
* @param[in] acceleration: frequency ramp acceleration
* @param[in] deceleration: frequency ramp deceleration
* @param[in] param: pointer to VF_FREQ_HOP_PARAM structure
* @return none
*/
void RampDirJudgeProcess(float freqCmd, float freqCurrentCalc, float acceleration, float deceleration, VF_FREQ_HOP_PARAM *param);

/*!
* @brief V/f control frequency hopping process, avoid motor resonance frequency.
*
* @param[in] in: pointer to VF_FREQ_CALC_BUF structure
* @param[in] param: pointer to VF_FREQ_HOP_PARAM structure
* @return none
*/
void FrequencyHoppingProcess(VF_FREQ_CALC_BUF *in, VF_FREQ_HOP_PARAM *param);

/* ======================================  Functions define  ======================================== */

#endif /* __VF_COMPENSATE_H__ */

/* =============================================  EOF  ============================================== */
