/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __BLDC_DATA_ACCESS_EFLASH_H__
#define __BLDC_DATA_ACCESS_EFLASH_H__
/*!
* @file bldc_data_access_eflash.h
*
* @brief This file provides data save and recovery operation with eflash functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include <stdint.h>
#include "string.h"
#include "flash_drv.h"
#include "bldc_parameters_define.h"

/* ============================================  Define  ============================================ */
#define ADDRESS_START_DEFAULT_PARAM     0x000FE000              /*!< Macro define value of FOC are saved in this flash page */
#define ADDRESS_START_DEBUG_PARAM       0x000FE800              /*!< Variables debug value of FOC are saved in this flash page */

#define DATA_NUM                        40                      /*!< Number of data save and recovery, should be less than 512, because flash page size is 2048 */
#define DATA_SIZE                       (DATA_NUM << 2)         /*!< Size of data save and recovery */

#define SAVE_DEFAULT_DATA               1                       /*!< Flash operation mode, save macro define value to flash page address ADDRESS_START_DEFAULT_PARAM */
#define SAVE_DEBUG_DATA                 2                       /*!< Flash operation mode, save user debug value to flash page address ADDRESS_START_DEBUG_PARAM */

#define RECOVERY_DEFAULT_DATA           1                       /*!< Flash operation mode, recovery macro define value from flash page address ADDRESS_START_DEFAULT_PARAM */
#define RECOVERY_DEBUG_DATA             2                       /*!< Flash operation mode, recovery user debug value from flash page address ADDRESS_START_DEFAULT_PARAM */

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Eflash access operation configuration parameters structure.
*/
typedef struct
{
    uint16_t accessEnable;              /*!< Flash access operation enable */
} DATA_ACCESS_FLASH_CFG;

/*!
* @brief Data access flash operation parameters structure.
*/
typedef struct
{
    uint8_t operateEnable;              /*!< Flash access enable */
    uint8_t saveEnable;                 /*!< Flash save operation enable */
    uint8_t recoveryEnable;             /*!< Flash recovery operation enable */
    uint8_t eraseEnable;                /*!< Flash eraze operation enable */
    uint8_t saveFinishFlag;             /*!< Flag of flash save operation */
    uint8_t recoveryFinishFlag;         /*!< Flag of flash recovery operation */
    uint8_t saveMode;                   /*!< Flash save operation mode, 1: save default value; 2: save debug value */
    uint8_t recoveryMode;               /*!< Flash recovery operation mode, 1: recovery default value; 2: recovery debug value */
} DATA_ACCESS_FLASH;

/* ==========================================  Variables  =========================================== */
extern status_t g_flashOperationStatus;        /*!< return value of eflash operation status */
extern flash_config_t flashConfig;
extern const flash_user_config_t FLASH_InitConfig;

extern int32_t g_VarAccessArray[DATA_NUM];

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Saving macro define value to flash when mcu is power on.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] array: pointer to data array
* @return ret: flash operation status
*/
status_t Eflash_DataAccess_DefaultSave(DATA_ACCESS_FLASH *pDataAccess, int32_t *array);

/*!
* @brief Recovery user debug value from flash to variables when mcu is power on.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] array: pointer to data array
* @return none
*/
void Eflash_DataAccess_PowerOnRecovery(DATA_ACCESS_FLASH *pDataAccess, int32_t *array);

/*!
* @brief Flash operation process, do flash tasks with tool command.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @param[in] array: pointer to data array
* @return ret: flash operation status
*/
status_t Eflash_DataAccessEflashProcess(DATA_ACCESS_FLASH *pDataAccess, int32_t *array);

/*!
* @brief Flash operation command initilization.
*
* @param[in] pDataAccess: pointer to DATA_ACCESS_FLASH structure, flash operation command
* @return none
*/
void Eflash_DataAccessEflashInit(DATA_ACCESS_FLASH *pDataAccess);

/* ======================================  Functions define  ======================================== */

#endif /* __BLDC_DATA_ACCESS_EFLASH_H__ */

/* =============================================  EOF  ============================================== */
