/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_mcu_config.c
*
* @brief This file provides MCU peripheral initialization functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "string.h"
#include "ac7840x.h"
#include "gpio_drv.h"
#include "adc_hw.h"
#include "pwm_hw.h"
#include "ctu_drv.h"
#include "acmp_drv.h"
#include "dma_drv.h"
#include "spi_master_drv.h"
#include "bldc_mcu_config.h"
#include "bldc_isr.h"
#include "bldc_app.h"
#include "bldc_keys.h"
#include "bldc_data_access_eflash.h"

/* ============================================  Define  ============================================ */
#define MAX_SYSTICK_COUNT (SysTick_LOAD_RELOAD_Msk + 1)  /* systemTick max count value */
/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
uint32_t g_ADCRegularBuffer[4] = {0};  /*!< Buffer to storage Regular0 Group ADC Value */
uint32_t g_ADCRegularBuffer1[4] = {0};  /*!< Buffer to storage Regular1 Group ADC Value */

static dma_state_t dmaStateArray[2] = {0};
static dma_channel_config_t dmaController1Chn0_Config, dmaController1Chn1_Config;
static const dma_channel_config_t * const dmaChnConfigArray[] = {&dmaController1Chn0_Config, &dmaController1Chn1_Config};

static dma_chn_state_t dmaController1Chn0_State, dmaController1Chn1_State;
static dma_chn_state_t * const dmaChnStateArray[] = {&dmaController1Chn0_State, &dmaController1Chn1_State};

/* systemTick us delay factor */
static uint32_t  s_facus = 0U;

/* systemTick ms delay factor */
static uint32_t  s_facms = 0U;

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Init Delay count factor, 1s = TICKS * (1/f)  s;
*
* @param[in]  none
* @return none
*/
void InitDelay(void)
{
    s_facus = SYSCLK_FREQ / 1000000;
    s_facms = s_facus * 1000;
}

/*!
* @brief delay until the syctick count tick to 0
*
* @param[in] tick : systick count value
* @return none
*/
static void SysTickDelay(uint32_t tick)
{
    __IO uint32_t tickFlag = 0;

    SysTick->LOAD = tick - 1;
    SysTick->VAL = 0x00;
    SysTick->CTRL  = SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_ENABLE_Msk;
    do
    {
        tickFlag = SysTick->CTRL;

    } while(!(tickFlag & SysTick_CTRL_COUNTFLAG_Msk));

    SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk;
    SysTick->VAL = 0X00;
}

/*!
* @brief Repeatlly delay the ticks for the given times
*
* @param[in] param : times, the times of delay the tick
* @param[in] param : tick, each tick to delay
* @return none
*/
static void SysTickRepeatDelay(uint32_t times, uint32_t tick)
{
    uint32_t i = 0;

    for (i = 0; i < times; i++)
    {
        SysTickDelay(tick);
    }
}

/*!
* @brief delay us
*
* @param[in] param : us, us for delay
* @return none
*/
void udelay(uint32_t us)
{
    uint32_t tick = us * s_facus;

    SysTickRepeatDelay(tick / MAX_SYSTICK_COUNT, MAX_SYSTICK_COUNT);

    SysTickDelay(tick % MAX_SYSTICK_COUNT);
}

/*!
* @brief delay ms
*
* @param[in] param : ms, us for delay
* @return none
*/
void mdelay(uint32_t ms)
{
    uint32_t tick = ms * s_facms;

    SysTickRepeatDelay(tick / MAX_SYSTICK_COUNT, MAX_SYSTICK_COUNT);

    SysTickDelay(tick % MAX_SYSTICK_COUNT);
}

/*!
* @brief PWM Initialization for debugging, PWM module can be used for debugging output.
*
* @param[in] PWMx: PWM Module
* @return none
*/
void MC_DEBUG_PWM_Initialize(PWM_Type *PWMx)
{
    pwm_simply_config_t SimplyConfig;

    /*
    * GPIO Configuration for debug PWM
    * PA12    --> PWM2_CH1
    * PA13    --> PWM2_CH0
    */
    /* PA13: Print1 */
    GPIO_DRV_SetMuxModeSel(PORTA, (gpio_channel_type_t)13U, PORT_MUX_ALT6);
    /* PA12: Print2 */
    GPIO_DRV_SetMuxModeSel(PORTA, (gpio_channel_type_t)12U, PORT_MUX_ALT6);

    /* Config DEBUG-PWM Interface */
    memset(&SimplyConfig, 0U, sizeof(pwm_simply_config_t));
    SimplyConfig.allChCombineMode       = PWM_INDEPENDENT_MODE;
    SimplyConfig.countMode              = PWM_UP_COUNT;
    /* PWM_HIGH_TRUE */
    SimplyConfig.levelMode              = PWM_HIGH_TRUE;
    /* PWM_CLK_SOURCE_BUS */
    SimplyConfig.clkSource              = PWM_CLK_SOURCE_SYSTEM;
    SimplyConfig.clkPsc                 = 0U;
    SimplyConfig.initValue              = 0U;
    SimplyConfig.maxValue               = 4096U;
    SimplyConfig.chValue[0]             = 0U;
    SimplyConfig.chValue[2]             = 0U;
    SimplyConfig.chValue[4]             = 0U;
    SimplyConfig.interruptEn            = false;
    SimplyConfig.overflowCallback       = NULL;
    PWM_DRV_SimplyInit(((uint8_t)(((uint32_t)(PWMx) - PWM0_BASE) >> 11U)), &SimplyConfig);
}

/*!
* @brief Calbiration ADC to get adc channel offset value.
*
* @param[in] sequencechannel : ADC Inject Group sequence num, 0-3
* @return dcoffset: ADC Inject Group channel offset
*/
int16_t MC_ADC_Calbiration(uint8_t sequencechannel)
{
    uint16_t index = 0;
    uint32_t sumOffset =0;
    int16_t adcOffset = 0;

    ADC_SetRegularTriggerSource(ADC0, ADC_TRIGGER_INTERNAL);
    ADC_SetInjectTriggerSource(ADC0, ADC_TRIGGER_INTERNAL);

    ADC_ClearInjectEOCFlag(ADC0, (adc_sequence_t)(ADC_ISEQ_0 + sequencechannel));//need to confirm
    udelay(1000);

    for (index = 0; index < NUM_OF_CONVERSIONS; index ++)
    {
        ADC_SoftwareStartInjectConvert(ADC0);
//#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
//        udelay(10);
//        ADC_SoftwareStartInjectConvert(ADC1);
//#endif
        udelay(10);
        while (!ADC_GetInjectEOCFlag(ADC0, (adc_sequence_t)(ADC_ISEQ_0 + sequencechannel)));

        sumOffset += ADC_GetInjectData(ADC0, (adc_sequence_t)(ADC_ISEQ_0 + sequencechannel));

        ADC_ClearInjectEOCFlag(ADC0, (adc_sequence_t)(ADC_ISEQ_0 + sequencechannel));
    }
    adcOffset = sumOffset / NUM_OF_CONVERSIONS;

    return adcOffset;
}

/*!
* @brief Gpio port Initialize.
*
* @param[in] none
* @return none
*/
void BLDC_Gpio_Initialize(void)
{
    /* Debug GPIO channel configuration */
    GPIO_DRV_SetMuxModeSel(PORTD, (gpio_channel_type_t)16U, PORT_MUX_AS_GPIO);
    GPIO_DRV_SetPinDirection(GPIOD, (gpio_channel_type_t)16U, (gpio_level_type_t)GPIO_OUTPUT_DIRECTION);

    /* Setting the LED to display operation states */
    GPIO_DRV_SetMuxModeSel(PORTD, 15U, PORT_MUX_AS_GPIO);
    GPIO_DRV_SetPinDirection(GPIOD, 15U, GPIO_OUTPUT);
    GPIO_DRV_SetMuxModeSel(PORTE, 3U, PORT_MUX_AS_GPIO);
    GPIO_DRV_SetPinDirection(GPIOE, 3U, GPIO_OUTPUT);
}

/*!
* @brief Timer1 Initialize, generates 1ms interrupt for bldc control time base.
*
* @param[in] none
* @return none
*/
void BLDC_TIM_Initialize(void)
{
    module_clk_config_t clkConfig= {SPLLDIV2_CLK};
    timer_user_channel_config_t timerConfig;

    memset(&timerConfig, 0U, sizeof(timer_user_channel_config_t));
    CKGEN_DRV_SetModuleClock(TIMER_CLK, &clkConfig);

    /* Timer channel configuration */
    timerConfig.timerMode               = TIMER_PERIODIC_COUNTER;
    timerConfig.periodUnits             = TIMER_PERIOD_UNITS_COUNTS;
    timerConfig.period                  = TIMER_1MS;
    timerConfig.triggerSource           = TIMER_TRIGGER_SOURCE_EXTERNAL;
    timerConfig.triggerSelect           = 0U;
    timerConfig.enableReloadOnTrigger   = false;
    timerConfig.enableStopOnInterrupt   = false;
    timerConfig.enableStartOnTrigger    = false;
    timerConfig.chainChannel            = false;
    timerConfig.isInterruptEnabled      = true;
    timerConfig.callback                = BLDC_Base_TimerCallback;

    TIMER_DRV_Init(0U,true);
    TIMER_DRV_InitChannel(0U, TIMER_CHANNEL_0, &timerConfig);
    TIMER_DRV_StartChannels(0U, 1U << TIMER_CHANNEL_0);

    /* Set interrupt priority */
    NVIC_SetPriority(TIMER_CHANNEL0_IRQn, 3);
}

/*!
* @brief Timer1 Initialize, generates interrupt to check Bemf over zero point task in bldc sebsorless control.
*
* @param[in] none
* @return none
*/
void BLDC_Bemf_TimerInit(void)
{
    module_clk_config_t clkConfig= {SPLLDIV2_CLK};
    timer_user_channel_config_t timerConfig;

    memset(&timerConfig, 0U, sizeof(timer_user_channel_config_t));
    CKGEN_DRV_SetModuleClock(TIMER_CLK, &clkConfig);

    /* Timer channel configuration */
    timerConfig.timerMode               = TIMER_PERIODIC_COUNTER;
    timerConfig.periodUnits             = TIMER_PERIOD_UNITS_COUNTS;
    timerConfig.period                  = COMMUTATION_TIMER_PERIOD;
    timerConfig.triggerSource           = TIMER_TRIGGER_SOURCE_EXTERNAL;
    timerConfig.triggerSelect           = 0U;
    timerConfig.enableReloadOnTrigger   = false;
    timerConfig.enableStopOnInterrupt   = false;
    timerConfig.enableStartOnTrigger    = false;
    timerConfig.chainChannel            = false;
    timerConfig.isInterruptEnabled      = true;
    timerConfig.callback                = Timer1_Callback;

    TIMER_DRV_Init(0U,true);
    TIMER_DRV_InitChannel(0U, TIMER_CHANNEL_1, &timerConfig);
    TIMER_DRV_StartChannels(0U, 1U << TIMER_CHANNEL_1);

    /* Set interrupt priority */
    NVIC_SetPriority(TIMER_CHANNEL1_IRQn, 1);
}

/*!
* @brief CTU Initialize, config ADC sampling trigger mode and trigger time in bldc control.
*
* @param[in] none
* @return none
*/
void BLDC_CTU_Init(void)
{
    /* need pdt config delay or not */
    CTU_DRV_DeInit(0U);
    CTU_DRV_Init(0U);

    TRGMUX_DRV_SetTrigSourceForTargetModule(0U, CTU_ADC_TRIGGER_SOURCE, TRGMUX_TARGET_MODULE_ADC0_INJECTION0); 
}

/*!
* @brief DMA Initialize in ADC config, config DMA transfer method and address for ADC regular group.
*
* @param[in] none
* @return none
*/
void BLDC_ADCDMA_Init(void)
{
    uint32_t tmpMemStartAddr = (uint32_t)&g_ADCRegularBuffer[0];
    uint32_t tmpMemStartAddr1 = (uint32_t)&g_ADCRegularBuffer1[0];

    dmaController1Chn0_Config.channelPriority = DMA_CHN_PRIORITY_VERY_HIGH;
    dmaController1Chn0_Config.virtChnConfig = 15U;
    dmaController1Chn0_Config.source = DMA_REQ_ADC0; /* PWM channel 7 trigger, last channel */
    dmaController1Chn0_Config.callbackParam = NULL;
    dmaController1Chn0_Config.enableTrigger = false;

    dmaController1Chn1_Config.channelPriority = DMA_CHN_PRIORITY_VERY_HIGH;
    dmaController1Chn1_Config.virtChnConfig = 14U;
    dmaController1Chn1_Config.source = DMA_REQ_ADC1; /* PWM channel 7 trigger, last channel */
    dmaController1Chn1_Config.callbackParam = NULL;
    dmaController1Chn1_Config.enableTrigger = false;

    DMA_DRV_Init(dmaStateArray, dmaChnStateArray, dmaChnConfigArray, 2U);
    DMA_DRV_ConfigTransfer(15U, DMA_TRANSFER_PERIPH2MEM, (uint32_t)(&(ADC0->RDR[0])), tmpMemStartAddr, DMA_TRANSFER_SIZE_2B, sizeof(g_ADCRegularBuffer));

    DMA_DRV_SetSrcOffset(15U, 4U);
    DMA_DRV_SetSrcAddr(15U, (uint32_t)(&(ADC0->RDR[0])), (uint32_t)(&(ADC0->RDR[2])));
    DMA_DRV_SetDestOffset(15U, 4U);
    DMA_DRV_SetDestAddr(15U, (uint32_t)g_ADCRegularBuffer, (uint32_t)g_ADCRegularBuffer + sizeof(g_ADCRegularBuffer));

    DMA_DRV_ConfigureInterrupt(15U, DMA_CHN_ERR_INT, true);
//    DMA_DRV_ConfigureInterrupt(15U, DMA_CHN_HALF_FINISH_INT, false);
    DMA_DRV_ConfigureInterrupt(15U, DMA_CHN_FINISH_INT, false);

    DMA_DRV_SetCircularMode(15U, true);

    DMA_DRV_StopChannel(15U);
    DMA_DRV_StartChannel(15U);
    
    DMA_DRV_ConfigTransfer(14U, DMA_TRANSFER_PERIPH2MEM, (uint32_t)(&(ADC1->RDR[0])), tmpMemStartAddr1, DMA_TRANSFER_SIZE_2B, sizeof(g_ADCRegularBuffer1));

    DMA_DRV_SetSrcOffset(14U, 4U);
    DMA_DRV_SetSrcAddr(14U, (uint32_t)(&(ADC1->RDR[0])), (uint32_t)(&(ADC1->RDR[2])));//need to confirm regular length
//    DMA_DRV_SetDestOffset(14U, 2U);    //Taranmmit once
    DMA_DRV_SetDestOffset(14U, 4U);    //Taranmmit twice
    DMA_DRV_SetDestAddr(14U, (uint32_t)g_ADCRegularBuffer1, (uint32_t)g_ADCRegularBuffer1 + sizeof(g_ADCRegularBuffer1));

    DMA_DRV_ConfigureInterrupt(14U, DMA_CHN_ERR_INT, false);
//    DMA_DRV_ConfigureInterrupt(14U, DMA_CHN_HALF_FINISH_INT, false);
    DMA_DRV_ConfigureInterrupt(14U, DMA_CHN_FINISH_INT, false);

    DMA_DRV_SetCircularMode(14U, true);

    DMA_DRV_StopChannel(14U);
    DMA_DRV_StartChannel(14U);
}

/*!
* @brief Get ADC current sampling channel offset value.
*
* @param[in] none
* @return none
*/
void BLDC_ADC_Calbiration(void)
{
    g_bldc_adSample.busCurOffset = MC_ADC_Calbiration(0U);
#if (defined BEMF_DETECT_SOFTWARE)
    g_bldc_adSample.phaseVoltageOffsetA = MC_ADC_Calbiration(1U);
    g_bldc_adSample.phaseVoltageOffsetB = MC_ADC_Calbiration(2U);
    g_bldc_adSample.phaseVoltageOffsetC = MC_ADC_Calbiration(3U);
#endif
}

/*!
* @brief ADC Initialize, config ADC mode, sampling channel, trigger source, and DMA transfer.
*
* @param[in] none
* @return none
*/
void BLDC_ADC_Init(void)
{
    ADC_DRV_Deinit(0U);
    ADC_DRV_Deinit(1U);
    adc_converter_config_t tempAdcConfig;
    adc_converter_config_t* adcConfig;
    adcConfig = &tempAdcConfig;
    memset(adcConfig, 0U, sizeof(adc_converter_config_t));

    adc_chan_config_t tempAdcChConfig;
    adc_chan_config_t* adcChConfig;
    adcChConfig = &tempAdcChConfig;
    memset(adcChConfig, 0U, sizeof(adc_chan_config_t));

    /**
    GPIO Configuration for ADC
    PC16     --> ADC0_IN14
    PB12     --> ADC1_IN7
    PC15     --> ADC0_IN13
    PB13     --> ADC1_IN8
    PA6      --> ADC0_IN2
    PA0      --> ADC0_IN0
    PA1      --> ADC0_IN1
    PC3      --> ADC0_IN11
    */

    /* ADC0 channel 14, Control keys(Star_Stop/Fast/Slow/Direction) */
    GPIO_DRV_SetMuxModeSel(PORTC, (gpio_channel_type_t)16U, PORT_PIN_DISABLED);
    /* ADC1 channel 7, VBus */
    GPIO_DRV_SetMuxModeSel(PORTB, (gpio_channel_type_t)12U, PORT_PIN_DISABLED);
    /* ADC0 channel 13, variohm adjust AD */
    GPIO_DRV_SetMuxModeSel(PORTC, (gpio_channel_type_t)15U, PORT_PIN_DISABLED);
    /* ADC1 channel 8, I_Bus_Avg */
    GPIO_DRV_SetMuxModeSel(PORTB, (gpio_channel_type_t)13U, PORT_PIN_DISABLED);
    /* ADC0 channel 2, I_Bus */
    GPIO_DRV_SetMuxModeSel(PORTA, (gpio_channel_type_t)6U, PORT_PIN_DISABLED);
    
#if (defined BEMF_DETECT_SOFTWARE)
    /* ADC0 channel 0,1,11, BEMF A,B,C */
    /* ADC0 channel 0, Bemf_A */
    GPIO_DRV_SetMuxModeSel(PORTA, (gpio_channel_type_t)0U, PORT_PIN_DISABLED);
    /* ADC0 channel 1, Bemf_B */
    GPIO_DRV_SetMuxModeSel(PORTA, (gpio_channel_type_t)1U, PORT_PIN_DISABLED);
    /* ADC0 channel 11, Bemf_C */
    GPIO_DRV_SetMuxModeSel(PORTC, (gpio_channel_type_t)3U, PORT_PIN_DISABLED);
#endif

    CKGEN_DRV_Enable(CLK_ADC0, true);
    CKGEN_DRV_Enable(CLK_ADC1, true);
    ADC_DRV_Init(0U);
    ADC_DRV_Init(1U);

    module_clk_config_t clkConfig= {SPLLDIV2_CLK};
    CKGEN_DRV_SetModuleClock(ADC0_CLK, &clkConfig);
    CKGEN_DRV_SetModuleClock(ADC1_CLK, &clkConfig);
    adcConfig->clockDivide = ADC_CLK_DIVIDE_2;
    adcConfig->resolution = ADC_RESOLUTION_12BIT;
    adcConfig->regularTrigger = ADC_TRIGGER_INTERNAL;
    adcConfig->injectTrigger = ADC_TRIGGER_INTERNAL;
    adcConfig->dmaEnable = true;
    adcConfig->voltageRef = ADC_VOLTAGEREF_VREF;

    /* ADC mode 5 */
    adcConfig->scanModeEn = true;
    adcConfig->continuousModeEn = false;
    adcConfig->regularDiscontinuousModeEn = false;
    adcConfig->injectDiscontinuousModeEn = false;
    adcConfig->injectAutoModeEn = false;
    adcConfig->intervalModeEn = false;

    adcConfig->regularDiscontinuousNum = 0;
    adcConfig->powerEn = true;
    adcConfig->regularSequenceLength = 2;
#if (defined BEMF_DETECT_SOFTWARE)
    adcConfig->injectSequenceLength = 4;
#else
    adcConfig->injectSequenceLength = 1;
#endif

    adcChConfig->spt = ADC_SPT_CLK_15;
    adcChConfig->supply = ADC_CH_SUPPLY_VDD;
    adcChConfig->interruptEn = false;

    ADC_DRV_ConfigConverter(0U, adcConfig);
    ADC_DRV_InstallCallback(0U, BLDC_ADC_Callback, NULL);

    adcConfig->regularSequenceLength = 2;
    adcConfig->injectSequenceLength = 0;

    ADC_DRV_ConfigConverter(1U, adcConfig);
    ADC_DRV_InstallCallback(1U, NULL, NULL);

    adcChConfig->channel = ADC_CH_13;
    ADC_DRV_ConfigChan(0U, ADC_RSEQ_0, adcChConfig);
    adcChConfig->channel = ADC_CH_14;
    ADC_DRV_ConfigChan(0U, ADC_RSEQ_1, adcChConfig);
    adcChConfig->channel = ADC_CH_2;
    ADC_DRV_ConfigChan(0U, ADC_ISEQ_0, adcChConfig);
#if (defined BEMF_DETECT_SOFTWARE)
    adcChConfig->channel = ADC_CH_0;
    ADC_DRV_ConfigChan(0U, ADC_ISEQ_1, adcChConfig);
    adcChConfig->channel = ADC_CH_1;
    ADC_DRV_ConfigChan(0U, ADC_ISEQ_2, adcChConfig);
    adcChConfig->channel = ADC_CH_11;
    ADC_DRV_ConfigChan(0U, ADC_ISEQ_3, adcChConfig);
#endif

    adcChConfig->channel = ADC_CH_7;
    ADC_DRV_ConfigChan(1U, ADC_RSEQ_0, adcChConfig);
    adcChConfig->channel = ADC_CH_8;
    ADC_DRV_ConfigChan(1U, ADC_RSEQ_1, adcChConfig);

    /* Calbiration Ibus Offset */
    BLDC_ADC_Calbiration();

    ADC_SetRegularTriggerSource(ADC0, ADC_TRIGGER_INTERNAL);
    ADC_SetInjectTriggerSource(ADC0, ADC_TRIGGER_EXTERNAL);

    ADC_SetRegularTriggerSource(ADC1, ADC_TRIGGER_INTERNAL);
    ADC_SetInjectTriggerSource(ADC1, ADC_TRIGGER_INTERNAL);

#if (defined BEMF_DETECT_SOFTWARE)
    ADC_SetInjectEOCInterruptEnableFlag(ADC0, ADC_ISEQ_3, true);
#else
    ADC_SetInjectEOCInterruptEnableFlag(ADC0, ADC_ISEQ_0, true);
#endif

    BLDC_ADCDMA_Init();
    ADC_DRV_SoftwareStartRegularConvert(0U);
    ADC_DRV_SoftwareStartRegularConvert(1U);
    NVIC_SetPriority(ADC0_IRQn, 1);
}

/*!
* @brief PWM sync mode initialize, config PWM sync mode.
*
* @param[in] none
* @return none
*/
void PWM_SyncModeSet(void)
{
    pwm_sync_config_t syncConfig;

    syncConfig.syncTriggerMode = PWM_SYNC_TRIGGER_SOFTWARE;
    syncConfig.hardwareSync0En = false;
    syncConfig.hardwareSync1En = false;
    syncConfig.hardwareSync2En = false;
    syncConfig.autoClearHWTriggerEn = false;
    syncConfig.counterInitSyncEn = true;
    syncConfig.outCtrlSyncEn = false;
    syncConfig.inverterSyncEn = false;
    syncConfig.outmaskSyncEn = true;
    syncConfig.polaritySyncEn = false;
    syncConfig.chValueSyncEn[0] = true;
    syncConfig.chValueSyncEn[1] = true;
    syncConfig.chValueSyncEn[2] = true;
    syncConfig.chValueSyncEn[3] = true;
    syncConfig.maxLoadingPointEn = false;
    syncConfig.minLoadingPointEn = true;
    syncConfig.pwmSyncEnable = true;

    PWM_DRV_InitSyncControl(((uint8_t)(((uint32_t)(PWM3) - PWM0_BASE) >> 11U)), &syncConfig);
}

/*!
* @brief PWM Initialize, config PWM mode, polarity, and FAULT interrupt.
*
* @param[in] none
* @return none
*/
void BLDC_PWM_Initialize(void)
{
    PWM_DRV_Deinit(((uint8_t)(((uint32_t)(PWM3) - PWM0_BASE) >> 11)));
    static pwm_combine_ch_config_t chConfig[4];
    static pwm_modulation_config_t pwmConfig;
    static pwm_config_t config;
    static pwm_fault_config_t faultConfig;
    static pwm_sync_config_t syncConfig;

    /**
    GPIO Configuration for PWM3
    PA2     --> PWM3_IN0
    PA3     --> PWM3_IN1
    PC6     --> PWM3_IN2
    PC7     --> PWM3_IN3
    PD2     --> PWM3_IN4
    PD3     --> PWM3_IN5
    PE7     --> PWM3_FLT0
    */

    /* PA2: PWM3_CH0, PWM_U+ */
    GPIO_DRV_SetMuxModeSel(PORTA, (gpio_channel_type_t)2U, PORT_MUX_ALT2);
    /* PA3: PWM3_CH1, PWM_U- */
    GPIO_DRV_SetMuxModeSel(PORTA, (gpio_channel_type_t)3U, PORT_MUX_ALT2);
    /* PC6: PWM3_CH2, PWM_V+ */
    GPIO_DRV_SetMuxModeSel(PORTC, (gpio_channel_type_t)6U, PORT_MUX_ALT4);
    /* PC7: PWM3_CH3, PWM_V- */
    GPIO_DRV_SetMuxModeSel(PORTC, (gpio_channel_type_t)7U, PORT_MUX_ALT4);
    /* PD2: PWM3_CH4, PWM_W+ */
    GPIO_DRV_SetMuxModeSel(PORTD, (gpio_channel_type_t)2U, PORT_MUX_ALT2);
    /* PD3: PWM3_CH5, PWM_W- */
    GPIO_DRV_SetMuxModeSel(PORTD, (gpio_channel_type_t)3U, PORT_MUX_ALT2);
    /* PE7: PWM3_FLT0, PWM hardware current overflow Fault */
    GPIO_DRV_SetMuxModeSel(PORTE, (gpio_channel_type_t)7U, PORT_MUX_ALT3);

    /* Config PWM Interface */
    memset(chConfig, 0U, sizeof(chConfig));
    memset(&pwmConfig, 0U, sizeof(pwm_modulation_config_t));
    memset(&faultConfig, 0U, sizeof(pwm_fault_config_t));
    memset(&syncConfig, 0U, sizeof(pwm_sync_config_t));
    memset(&config, 0U, sizeof(pwm_config_t));
    CKGEN_DRV_Enable(CLK_PWM3, true);
    CKGEN_DRV_SoftReset(SRST_PWM3,true);

#if (defined GATE_DRIVER_HIGH_HIGH)
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_0, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_1, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_2, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_3, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_4, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_5, PWM_LOW_LEVEL);
#elif (defined GATE_DRIVER_HIGH_LOW)
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_0, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_1, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_2, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_3, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_4, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_5, PWM_HIGH_LEVEL);
#elif (defined GATE_DRIVER_LOW_HIGH)
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_0, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_1, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_2, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_3, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_4, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_5, PWM_LOW_LEVEL);
#elif (defined GATE_DRIVER_LOW_LOW)
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_0, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_1, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_2, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_3, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_4, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM3, PWM_CH_5, PWM_HIGH_LEVEL);
#endif
    PWM_InitChannelOutputEn(PWM3, true);

    /* channel 0/1 */
    chConfig[0].pairChannel = PWM_CH_0;
    chConfig[0].ch1stValue = 0U;
    chConfig[0].ch2ndValue = 0U;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[0].complementEn                                    = true;
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
    chConfig[0].complementEn                                    = false;
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[0].complementEn                                    = false;
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[0].complementEn                                    = true;
#else

#endif
    chConfig[0].deadtimeEn = true;
    chConfig[0].deadtime                                        = PWM_DEAD_TIME;
    chConfig[0].deadtimePsc                                     = PWM_DEADTIME_DIVID_4;

    chConfig[0].ch1stMatchDir = PWM_MATCH_DIR_UP;
    chConfig[0].ch2ndMatchDir = PWM_MATCH_DIR_DOWN;

#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[0].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[0].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[0].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* NULL */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[0].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[0].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #endif
#else

#endif

    /* channel 2/3 */
    chConfig[1].pairChannel = PWM_CH_2;
    chConfig[1].ch1stValue = 0U;
    chConfig[1].ch2ndValue = 0U;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[1].complementEn                                    = true;
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
    chConfig[1].complementEn                                    = false;
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[1].complementEn                                    = false;
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[1].complementEn                                    = true;
#else

#endif
    chConfig[1].deadtimeEn = true;
    chConfig[1].deadtime                                        = PWM_DEAD_TIME;
    chConfig[1].deadtimePsc                                     = PWM_DEADTIME_DIVID_4;

    chConfig[1].ch1stMatchDir = PWM_MATCH_DIR_UP;
    chConfig[1].ch2ndMatchDir = PWM_MATCH_DIR_DOWN;

#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[1].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[1].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[1].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* NULL */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[1].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[1].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #endif
#else

#endif

    /* channel 4/5 */
    chConfig[2].pairChannel = PWM_CH_4;
    chConfig[2].ch1stValue = 0U;
    chConfig[2].ch2ndValue = 0U;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[2].complementEn                                    = true;
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
    chConfig[2].complementEn                                    = false;
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[2].complementEn                                    = false;
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[2].complementEn                                    = true;
#else

#endif
    chConfig[2].deadtimeEn = true;
    chConfig[2].deadtime                                        = PWM_DEAD_TIME;
    chConfig[2].deadtimePsc                                     = PWM_DEADTIME_DIVID_4;

    chConfig[2].ch1stMatchDir = PWM_MATCH_DIR_UP;
    chConfig[2].ch2ndMatchDir = PWM_MATCH_DIR_DOWN;

#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[2].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[2].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[2].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* NULL */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[2].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* NULL */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].ch1stPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[2].ch2ndPolarity = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #endif
#else

#endif

    chConfig[3].pairChannel                                     = PWM_CH_6;
    chConfig[3].ch1stValue                                      = (PWM_PERIOD_VALUE >> 1U);
    chConfig[3].levelMode                                       = PWM_HIGH_TRUE;
    chConfig[3].deadtimeEn                                      = false;
    chConfig[3].complementEn                                    = false;
    chConfig[3].ch1stMatchDir                                   = PWM_MATCH_DIR_UP;
    chConfig[3].ch1stMatchTriggerEn                             = true;

    /* modulation mode config */
    pwmConfig.initTriggerEn = false;
    pwmConfig.combineChannelNum = 4U;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode = PWM_UP_COUNT;
#endif
    pwmConfig.combineChConfig = chConfig;

    /* pwm config */
    config.mode = PWM_MODE_MODULATION;
    config.initModeStruct = &pwmConfig;
    config.clkSource = PWM_CLK_SOURCE_NONE;
    config.clkPsc = PWM_PRSC;
    config.maxValue = PWM_PERIOD_VALUE;
    config.interruptEn = false;
    config.overflowInterrupEn = false;
    config.underflowEventEn = false;
    config.overflowEventEn = false;
    PWM_DRV_Init(((uint8_t)(((uint32_t)(PWM3) - PWM0_BASE) >> 11U)), &config);

    /* pwm sync config */
    PWM_SyncModeSet();

    /* fault control config */
    faultConfig.channelConfig[PWM_FAULT_CH_0].faultInputEn = true;
    faultConfig.channelConfig[PWM_FAULT_CH_0].faultFilterEn = false;
    faultConfig.channelConfig[PWM_FAULT_CH_0].faultPolarity = OVERCUR_FEEDBACK_POLARITY;
    faultConfig.mode = PWM_FAULT_CTRL_MANUAL_ALL;
    faultConfig.filterValue = 0U;
    faultConfig.faultCtrlOutputEn[0] = true;
    faultConfig.faultCtrlOutputEn[1] = true;
    faultConfig.faultCtrlOutputEn[2] = true;
    faultConfig.faultCtrlOutputEn[3] = true;
    faultConfig.interruptEn = true;
    faultConfig.faultCallback = BLDC_PWM_FaultCallback;
    PWM_DRV_InitFaultControl(((uint8_t)(((uint32_t)(PWM3) - PWM0_BASE) >> 11U)), &faultConfig);

    NVIC_SetPriority(PWM3_FAULT_IRQn, 0);
    PWM_SetClockSource(PWM3, PWM_CLK_SOURCE_SYSTEM);
}

/*!
* @brief ACMP Initialize, config ACMP mode and positive,negative compare channel.
*
* @param[in] none
* @return none
*/
void BLDC_Bemf_ACMP_Init(void)
{
    acmp_module_t acmpConfig;

    /*
    * GPIO Configuration for ACMP
    * PA0     --> ACMP_IN0
    * PA1     --> ACMP_IN1
    * PC3     --> ACMP_IN4
    * PE8     --> ACMP_IN3
    */
    /* PA0, ACMP0_IN0, Bemf_U */
    GPIO_DRV_SetMuxModeSel(PORTA, (gpio_channel_type_t)0U, PORT_PIN_DISABLED);
    /* PA1, ACMP0_IN1, Bemf_V */
    GPIO_DRV_SetMuxModeSel(PORTA, (gpio_channel_type_t)1U, PORT_PIN_DISABLED);
    /* PC3, ACMP0_IN6, Bemf_W */
    GPIO_DRV_SetMuxModeSel(PORTC, (gpio_channel_type_t)3U, PORT_PIN_DISABLED);
    /* PE8, ACMP0_IN7, Bemf_MID */
    GPIO_DRV_SetMuxModeSel(PORTE, (gpio_channel_type_t)8U, PORT_PIN_DISABLED);

    memset(&acmpConfig, 0U, sizeof(acmp_module_t));

    /* ACMP Polling Mode Configuration */
    acmpConfig.pollingMode.mode                 = ACMP_NONE_POLLING;

    acmpConfig.pollingMode.pollingSequence      = 23U;
    acmpConfig.pollingMode.pollingClockDivide   = ACMP_CLK_DIVIDE_256;
    acmpConfig.pollingMode.hallOutputEnable     = true;
    acmpConfig.pollingMode.hallAOutputCh        = ACMP_EXTERNAL_CH0;
    acmpConfig.pollingMode.hallBOutputCh        = ACMP_EXTERNAL_CH1;
    acmpConfig.pollingMode.hallCOutputCh        = ACMP_EXTERNAL_CH4;


    /* ACMP Init Configuration */
    acmpConfig.mux.negativeInputMux             = ACMP_EXTERNAL_CH3;
    acmpConfig.comparator.outputTrigger         = ACMP_BOTH_EDGES;
    acmpConfig.comparator.interruptEn           = false;
    acmpConfig.comparator.powerEnable           = true;

    ACMP_DRV_Init(0U, &acmpConfig);
}

///*!
//* @brief Bldc sensorless Control Hall Peripheral Initialize.
//*
//* @param[in] none
//* @return none
//*/
//void BLDC_Bemf_PWDT_Initialize(void)
//{
//    PWDT_ConfigType  pwdtConfig;
//    memset(&pwdtConfig, 0, sizeof(PWDT_ConfigType));

//    /**
//    GPIO Configuration for PWDT1
//    PC8     --> PWDT1_IN0
//    PC7     --> PWDT1_IN1
//    PC6     --> PWDT1_IN2
//    */

//    /* PWDT channel 0 */
//    GPIO_SetFunc(GPIOC, GPIO_PIN8, GPIO_FUN3);
//    /* PWDT channel 1 */
//    GPIO_SetFunc(GPIOC, GPIO_PIN7, GPIO_FUN3);
//    /* PWDT channel 2 */
//    GPIO_SetFunc(GPIOC, GPIO_PIN6, GPIO_FUN3);

//    pwdtConfig.overflowInterruptEn  = ENABLE;
//    pwdtConfig.readyInterruptEn     = ENABLE;                                   /* enable data reday interrupt */
//    pwdtConfig.interruptEn          = DISABLE;                                  /* Reserve for forward|reverse wind detection */
//    pwdtConfig.mode                 = PWDT_MEASURE_MODE;                        /* enable PWDT module */
//    pwdtConfig.edgeType             = PWDT_RISING_START_CAPTURE_ALL;            /* start to measure pulse from first rising-edge and all dege */
//    pwdtConfig.clkPsc               = (PWDT_ClkPrescalerType)PWDT_CLK_PSC;      /* PWDT clock source presclaer is 8 */
//    pwdtConfig.cmpModeEn            = ENABLE;                                   /* PWDT clock source is bus clock */
//    pwdtConfig.hallModeEn           = ENABLE;
//    pwdtConfig.filterEn             = ENABLE;                                   /* Enable PWDT Hall Interface */
//    pwdtConfig.filterPsc            = (PWDT_FilterPrescalerType)PWDT_FILT_PSC;
//    pwdtConfig.filterValue          = PWDT_FILT_VAL;
//    pwdtConfig.callBack             = BLDC_Bemf_PWDTCallback;

//    PWDT_Init(PWDT1, &pwdtConfig);
//    NVIC_SetPriority(PWDT1_IRQn, 1);
//}

/*!
* @brief Bldc Control Hall Peripheral Initialize.
*
* @param[in] none
* @return none
*/
void BLDC_Hall_Initialize(void)
{
    /* AC7840x Hall configuration */
    pwm_input_ch_config_type_t inputChConfig;
    pwm_input_capture_config_t inputConfig;
    pwm_config_t config;

    /* Hall A */
    GPIO_DRV_SetMuxModeSel(PORTC, 0U, PORT_MUX_ALT2);
    /* Hall B */
    GPIO_DRV_SetMuxModeSel(PORTC, 1U, PORT_MUX_ALT2);
    /* Hall C */
    GPIO_DRV_SetMuxModeSel(PORTC, 2U, PORT_MUX_ALT2);

    memset(&inputChConfig, 0U, sizeof(pwm_input_ch_config_type_t));
    inputChConfig.channel = PWM_CH_0;
    inputChConfig.mode = PWM_INPUTCAP_DUAL_EDGE;
    inputChConfig.measureType = PWM_POSITIVE_PLUSE_WIDTH_MEASURE;
    inputChConfig.onceMode = PWM_INPUTCAP_CONTINUOUS;
    inputChConfig.filterEn = true;
    inputChConfig.filterValue = PWDT_FILT_VAL;
    inputChConfig.interruptEn = true;
    inputChConfig.pulseWidthMeasureEn = true;
    inputChConfig.chCountRst = true;

    inputConfig.channelNum = 1U;
    inputConfig.channelConfig = &inputChConfig;
    inputConfig.hallEn = true;

    inputConfig.filterPsc = PWDT_FILT_PSC;

    memset(&config, 0U, sizeof(pwm_config_t));
    config.mode = PWM_MODE_INPUT_CAPTURE;
    config.initModeStruct = &inputConfig;
    config.clkSource = PWM_CLK_SOURCE_SYSTEM;
    config.maxValue = PWM_MAX_COUNTER_VALUE;
    config.interruptEn = true;
    config.clkPsc = PWDT_CLK_PSC;
    config.channelCallback = BLDC_Hall_ChannelCallback;
    config.overflowEventEn = true;
    config.overflowInterrupEn = true;
    config.overflowCallback = BLDC_Hall_OFCallback;
    PWM_DRV_Init(0U, &config);

    /* Set interrupt priority */
    NVIC_EnableIRQ(PWM0_CHANNEL_IRQn);
    NVIC_SetPriority(PWM0_CHANNEL_IRQn, 1);
}

/*!
* @brief Peripheral Initialize in bldc App.
*
* @param[in] none
* @return none
*/
void BLDC_Peripheral_Initialize(void)
{
    FLASH_DRV_Init(&FLASH_InitConfig, &flashConfig);
    BLDC_TIM_Initialize();
    BLDC_Gpio_Initialize();
    MC_DEBUG_PWM_Initialize(PWM2);
    BLDC_CTU_Init();

    BLDC_ADC_Init();
    BLDC_PWM_Initialize();
#if (defined BLDC_HALL)
    BLDC_Hall_Initialize();
#elif (defined BLDC_SENSORLESS)
    BLDC_Bemf_TimerInit();
    #if (defined BEMF_DETECT_HARDWARE)
    BLDC_Bemf_ACMP_Init();
//    BLDC_Bemf_PWDT_Initialize();
    #endif
#else

#endif
    NVIC_EnableIRQ(ADC0_IRQn);
    NVIC_DisableIRQ(ADC1_IRQn);
}

/* =============================================  EOF  ============================================== */
