/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file foc_ui_uart.c
*
* @brief This file provides UART communication with user interface of host computer functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "gpio_drv.h"
#include "foc_ui_uart.h"
#include "foc_app.h"
#include "foc_keys.h"
#include "motor_debug.h"
#include "param_identify_api.h"
#include "version.h"
#include "foc_data_access_eflash.h"

/* ============================================  Define  ============================================ */
#define CMD_HEAD_BYTE                   0xAA                        /*!< Head byte in a package command */

#define CMD_CHECK_COMMUNICATION         0x00                        /*!< Command used to confirm communication */
#define CMD_GET_CODE_VERSION            0x01                        /*!< Command used to get code version */
#define CMD_START_MOTOR                 0x02                        /*!< Command used to start motor */
#define CMD_STOP_MOTOR                  0x03                        /*!< Command used to stop motor */
#define CMD_STOP_RAMP                   0x04                        /*!< Command used to stop ramp */
#define CMD_START_PARAM_IDENTIFY        0x05                        /*!< Command used to start motor param identify */
#define CMD_STOP_PARAM_IDENTIFY         0x06                        /*!< Command used to stop motor param identify */
#define CMD_MOTOR_FAULT_CLEAR           0x07                        /*!< Command used to clear fault */

#define CMD_FLASH_READ                  0x08                        /*!< Command used to read flash page to recovery data to variables */
#define CMD_FLASH_SAVE                  0x09                        /*!< Command used to save data to flash page */

#define CMD_START_HALLSELFLEARN         0x0A                        /*!< Command used to start motor hall sensor self learning */
#define CMD_STOP_HALLSELFLEARN          0x0B                        /*!< Command used to stop motor hall sensor self learning */

#define CMD_READ_REGISTER               0x10                        /*!< Command used to read param in register table from Mcu */
#define CMD_WRITE_REGISTER              0x11                        /*!< Command used to write param in register table to Mcu */
#define CMD_QUERY_PROCESS               0x12                        /*!< Command used to get command execute schedule from Mcu */

#define RECEIVE_TIM_MAXCNT              15000                       /*!< Recieved master data time count, execution time base is 2ms */
#define START_MOTOR_MAXCNT              1000                        /*!< Start motor time count, execution time base is 2ms */
#define START_IDENTIFY_MAXCNT           15000                       /*!< Identify process time count, execution time base is 2ms */


/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
/* interactive buffer */
uint8_t g_uartRxBuf[RX_BUF_SIZE];
UART_BUF g_uiRxBuf = {RX_BUF_SIZE, 0, 0, 0, 0, g_uartRxBuf};        /*!< Define UI uart interactive rx buffer struct */

uint8_t g_uartTxBuf[TX_BUF_SIZE];
UART_BUF g_uiTxBuf = {TX_BUF_SIZE, 0, 0, 0, 0, g_uartTxBuf};        /*!< Define UI uart interactive tx buffer struct */

/* register */
uint16_t *g_uiRegBuf[UI_REG_BUF_SIZE];                              /*!< Define UI uart interactive register pointer,point to g_uiCommParam */
UI_COMMUNICATION_PARAM g_uiCommParam;                               /*!< Define UI uart interactive param table */

/* communicate command execute */
CMD_PROC_STATUS g_cmdProcStatus ={0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};      /*!< Master command execute status */
CMD_PROC_VAR g_cmdProcVar;                                          /*!< Master command execute intermediate variable */

FOCLIB_VERSION              g_focLibVersion = {0};                  /*!< Struct to store lib version */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Mcu transmit a char to master through Uart.
*
* @param[in] uartTx: pointer to UART_BUF structure, store the data send out to master
* @param[in] data: the char to be send out to master
* @return none
*/
void UI_UART_PutChar(UART_BUF *uartTx, uint8_t data)
{
    UART_Type *localUartRegs = (UART_Type*)UART1_BASE;
    if ((*uartTx).uartBufCnt >= (*uartTx).uartBufSize)
    {
        (*uartTx).uartBufCnt = 0;
        (*uartTx).uartBufRdIndex = (*uartTx).uartBufWrIndex;
    }
    /* close interrrrupt */
    UART1->IER &= (~UART_IER_ETC_Msk);

    /* send first char */
    if ((*uartTx).uartBufCnt == 0)
    {
        if (UART1->LSR0 & UART_LSR0_TXNF_Msk)
//        if (UART_TxIsEmpty(UART1))
        {
            localUartRegs->RBR = (data & (0x00FF));
            /* open interrrrupt */
            UART1->IER |= UART_IER_ETC_Msk;
            return;
        }
    }

    *((*uartTx).uartBufAddr + (*uartTx).uartBufWrIndex) = data;
    if (++(*uartTx).uartBufWrIndex == (*uartTx).uartBufSize)
    {
        (*uartTx).uartBufWrIndex = 0;
    }
    ++(*uartTx).uartBufCnt;
    /* open interrrrupt */
    UART1->IER |= UART_IER_ETC_Msk;
}

/*!
* @brief UART read a char in current recieve buffer pointer,do not delete from buffer.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @param[in] num: recieced data ID in the recieve buffer
* @return a char read from the recieve buffer
*/
uint8_t UI_UART_ReadChar(UART_BUF *uartRx, uint16_t num)
{
    uint8_t data;
    uint16_t i;
    i = num;
    i += (*uartRx).uartBufRdIndex;
    if (i > (*uartRx).uartBufSize - 1)
    {
        i -= (*uartRx).uartBufSize;
    }
    data = *((*uartRx).uartBufAddr  + i);
    return data;
}

/*!
* @brief UART get a char in current recieve buffer pointer,and delete from buffer.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @return a char get from the recieve buffer
*/
uint8_t UI_UART_GetChar(UART_BUF *uartRx)
{
    uint8_t data;
    data = *((*uartRx).uartBufAddr + (*uartRx).uartBufRdIndex);
    if ((*uartRx).uartBufCnt != 0)
    {
        if (++(*uartRx).uartBufRdIndex == (*uartRx).uartBufSize)
        {
            (*uartRx).uartBufRdIndex = 0;
        }

        /* close interrrrupt, disable RXNE interrupt, ERXNE = 0 */
        UART1->LSR0 &= (~UART_IER_ERXNE_Msk);
//        UART_SetRXNEInterrupt(UART1, DISABLE);

        --(*uartRx).uartBufCnt;

        /* open interrrrupt, enable RXNE interrupt, ERXNE = 1 */
        UART1->IER |= UART_IER_ERXNE_Msk;
//        UART_SetRXNEInterrupt(UART1, ENABLE);
    }
    return data;
}

/*!
* @brief UART delete a char in current recieve buffer pointer.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @param[in] num: the numbers of char to be deleted
* @return none
*/
void UI_UART_DeleteChar(UART_BUF *uartRx, uint16_t num)
{
    uint16_t i;
    if ((*uartRx).uartBufCnt >= num)
    {
        /* close interrrrupt */
        UART1->LSR0 &= (~UART_IER_ERXNE_Msk);
//        UART_SetRXNEInterrupt(UART1, DISABLE);

        (*uartRx).uartBufCnt -= num;

        /* open interrrrupt */
        UART1->IER |= UART_IER_ERXNE_Msk;
//        UART_SetRXNEInterrupt(UART1, ENABLE);

        i = num;
        i += (*uartRx).uartBufRdIndex;

        if (i >= (*uartRx).uartBufSize)
        {
            i -= (*uartRx).uartBufSize;
        }
        (*uartRx).uartBufRdIndex = i;
    }
    else
    {
        /* close interrrrupt */
        UART1->LSR0 &= (~UART_IER_ERXNE_Msk);
//        UART_SetRXNEInterrupt(UART1, DISABLE);

        i = (*uartRx).uartBufCnt;
        (*uartRx).uartBufCnt = 0;

        /* open interrrrupt */
        UART1->IER |= UART_IER_ERXNE_Msk;
//        UART_SetRXNEInterrupt(UART1, ENABLE);

        i += (*uartRx).uartBufRdIndex;
        if (i >= (*uartRx).uartBufSize)
        {
            i -= (*uartRx).uartBufSize;
        }
        (*uartRx).uartBufRdIndex = i;
    }
}

/*!
* @brief UART receive interrrupt isr,occur when receice a char from master.
*
* @param[in] uartRx: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_UART_RxISR_Task(UART_BUF *uartRx)
{
    uint8_t data;

    /* ERXNE has been enable,and data has been recelved on rx pin */
    if (UART1->IER & UART_IER_ERXNE_Msk)
    {
        data = (uint8_t)((UART1->RBR) & UART_RBR_RBR_THR_Msk);
//        data = (uint8_t)UART_ReceiveData(UART1);
        if ((UART1->LSR0 & 0x0E) == 0)
        {
            g_uartRxBuf[g_uiRxBuf.uartBufWrIndex] = data;
            if (++(*uartRx).uartBufWrIndex == g_uiRxBuf.uartBufSize)
            {
                (*uartRx).uartBufWrIndex = 0;
            }
            if (++(*uartRx).uartBufCnt == (*uartRx).uartBufSize)
            {
                (*uartRx).uartBufCnt = 0;
                (*uartRx).uartBufOvf = 1;
                (*uartRx).uartBufRdIndex = 0;
                (*uartRx).uartBufWrIndex = 0;
            }
        }
        else
        {
//            UART_ReceiveData(UART1);
        }

        UART1->LSR0 |= (~UART_LSR0_DR_Msk);
    }
}

/*!
* @brief UART transmit interrrupt isr,occur when transmit a char to master.
*
* @param[in] uartTx: pointer to UART_BUF structure, store the data send out to master
* @return none
*/
void UI_UART_TxISR_Task(UART_BUF *uartTx)
{
    UART_Type *localUartRegs = (UART_Type*)UART1_BASE;

    /* ETC has been enable,and data has been transmitted from tx pin */
    if ((UART1->IER & UART_IER_ETC_Msk))
    {
        /* clear TC flag */
        UART1->LSR0 &= (~UART_LSR0_TC_Msk);
        if ((*uartTx).uartBufCnt)
        {
            --g_uiTxBuf.uartBufCnt;
            localUartRegs->RBR = g_uartTxBuf[g_uiTxBuf.uartBufRdIndex];
            if (++(*uartTx).uartBufRdIndex == (*uartTx).uartBufSize)
            {
                (*uartTx).uartBufRdIndex = 0;
            }
        }
        else
        {
            /* clear ETC */
            UART1->IER &= (~UART_IER_ETC_Msk);
        }
    }
}

/*!
* @brief Check the foc Lib version valid or not.
*
* @param[in] focLib: pointer to FOCLIB_VERSION structure, store the information to identify foc lib valid or not
* @return version valid information : 0 stand for valid, -1 stand for unvalid
*/
int8_t UI_CheckCodeVersionValid(FOCLIB_VERSION *focLib)
{
    if ((focLib->year < 2000) || (focLib->month > 12) || (focLib->day > 31) ||
    (MOTORAPP_VER_YEAR < 2000) || (MOTORAPP_VER_MONTH > 12) || (MOTORAPP_VER_DAY > 31))
    {
        return -1;
    }
    else
    {
        return 0;
    }
}


/*!
* @brief Uart communicate interrrupt isr,occur when put a char with master.
*
* @param[in] driverState: device state pointer
* @param[in] event: event parameter
* @param[in] userData: user data parameter
* @return none
*/
void UI_UART_txCallback(void *driverState, uint32_t event, void *userData)
{
    /*Tx interrupt*/
    UI_UART_TxISR_Task(&g_uiTxBuf);
}
/*!
* @brief Uart communicate interrrupt isr,occur when get a char with master.
*
* @param[in] driverState: device state pointer
* @param[in] event: event parameter
* @param[in] userData: user data parameter
* @return none
*/
void UI_UART_rxCallback(void *driverState, uint32_t event, void *userData)
{
    /*Rx interrupt*/
    UI_UART_RxISR_Task(&g_uiRxBuf);
}

/*!
* @brief Uart communication register table initialization.
*
* @param[in] none
* @return none
*/
void UI_RegBuf_Init(void)
{
#if 0
    static uint16_t readBack_NULL = 0;
    memset(g_uiRegBuf, 0U , sizeof(g_uiRegBuf));
    memset(&g_uiCommParam, 0U, sizeof(g_uiCommParam));
    uint16_t* u16Temp = NULL;

    g_focLibVersion = FocLibVersionGet();

#if (defined HALL_SENSORS)
    g_uiCommParam.appMode = (uint16_t)SINE_HALL << 8;
#elif (defined ENCODER_HALL_SENSORS)
    g_uiCommParam.appMode = (uint16_t)SINE_HALL_ENCODER << 8;
#elif (defined ENCODER_SENSORS)
    g_uiCommParam.appMode = (uint16_t)SINE_ENCODER << 8;
#elif (defined SENSORLESS)
    g_uiCommParam.appMode = (uint16_t)SINE_SENSORLESS << 8;
    #if ((defined SMC_OBSERVE) && (defined ATAN_ANGLE_ESTIMATION))
    g_uiCommParam.appMode += (uint16_t)SMO_ATAN;
    #elif ((defined SMC_OBSERVE) && (defined PLL_ANGLE_ESTIMATION))
    g_uiCommParam.appMode += (uint16_t)SMO_PLL;
    #elif ((defined FLUX_OBSERVE) && (defined ATAN_ANGLE_ESTIMATION))
    g_uiCommParam.appMode += (uint16_t)FLUX_ATAN;
    #elif ((defined FLUX_OBSERVE) && (defined PLL_ANGLE_ESTIMATION))
    g_uiCommParam.appMode += (uint16_t)FLUX_PLL;
    #else
    #endif
#else
#endif
    g_uiRegBuf[0] = &g_uiCommParam.appMode;

    u16Temp = (uint16_t*)(&g_protector.ErrFlag.all);
    g_uiRegBuf[1] = u16Temp;
    g_uiRegBuf[2] = u16Temp + 1;

    g_uiRegBuf[3] = (uint16_t*)&g_mcStatus;

    g_uiRegBuf[4] = &g_adcTransform.vdcVt;
    g_uiRegBuf[5] = (uint16_t*)(&g_dcCurEst.dcCurTrue);
    g_uiRegBuf[6] = (uint16_t*)(&g_powerEst.powerInTrue);
    g_uiRegBuf[7] = &g_uiCommParam.heatTemperature;

    g_uiRegBuf[8] = (uint16_t*)(&g_speedCommand.speedTarget);
    g_uiRegBuf[9] = (uint16_t*)(&g_speedCommand.speedTargetRamp);
    g_uiRegBuf[10] = (uint16_t*)(&g_speedCommand.speedFbk);
    g_uiRegBuf[11] = (uint16_t*)(&g_speedCommand.avgSpeedFbkRpm);
    g_uiRegBuf[12] = (uint16_t*)(&g_speedCommand.speedTargetLimit);
    g_uiRegBuf[13] = (uint16_t*)(&pFocVarsCfg->speedMaxLimit);
    g_uiRegBuf[14] = (uint16_t*)(&pFocVarsCfg->speedMinLimit);

    g_uiRegBuf[15] = (uint16_t*)(&pFocVarsCfg->motorDir);
    g_uiRegBuf[16] = &pFocVarsCfg->controlMode;
    g_uiRegBuf[17] = (uint16_t*)(&pFocVarsCfg->motorAcceleration);
    g_uiRegBuf[18] = (uint16_t*)(&pFocVarsCfg->motorDeceleration);
    g_uiRegBuf[19] = (uint16_t*)(&pFocVarsCfg->stopSpeed);
    g_uiRegBuf[20] = &pFocVarsCfg->voltObtainWay;
    g_uiRegBuf[21] = (uint16_t*)(&pFocVarsCfg->angleObtainWay);
    g_uiRegBuf[22] = (uint16_t*)(&pFocVarsCfg->smoWay);
    g_uiRegBuf[23] = (uint16_t*)(&pFocVarsCfg->fullOrderSmoFilt);
    g_uiRegBuf[24] = &defFocVars.curSampleMode;
    g_uiRegBuf[25] = &defEflashAccessCfg.accessEnable;

    g_uiRegBuf[26] = &readBack_NULL;
    g_uiRegBuf[27] = &readBack_NULL;

    g_uiRegBuf[28] = (uint16_t*)(&defPID.speedKp);
    g_uiRegBuf[29] = (uint16_t*)(&defPID.speedKi);
    g_uiRegBuf[30] = (uint16_t*)(&defPID.speedKd);
    g_uiRegBuf[31] = (uint16_t*)(&defPID.maxSpeed);
    g_uiRegBuf[32] = (uint16_t*)(&defPID.minSpeed);
    g_uiRegBuf[33] = (uint16_t*)(&defPID.torqueKp);
    g_uiRegBuf[34] = (uint16_t*)(&defPID.torqueKi);
    g_uiRegBuf[35] = (uint16_t*)(&defPID.torqueKd);
    g_uiRegBuf[36] = (uint16_t*)(&defPID.maxtorque);
    g_uiRegBuf[37] = (uint16_t*)(&defPID.mintorque);
    g_uiRegBuf[38] = (uint16_t*)(&defPID.fluxKp);
    g_uiRegBuf[39] = (uint16_t*)(&defPID.fluxKi);
    g_uiRegBuf[40] = (uint16_t*)(&defPID.fluxKd);
    g_uiRegBuf[41] = (uint16_t*)(&defPID.maxflux);
    g_uiRegBuf[42] = (uint16_t*)(&defPID.minflux);

    g_uiRegBuf[43] = (uint16_t*)(&g_asrSpeedPIIn.refPu);
    g_uiRegBuf[44] = (uint16_t*)(&g_asrSpeedPIIn.fbkPu);
    g_uiRegBuf[45] = (uint16_t*)(&g_asrSpeedPIOut.outPu);

    g_uiRegBuf[46] = (uint16_t*)(&defHallSensor.hallSwapTrqCoff);
    g_uiRegBuf[47] = (uint16_t*)(&defHallSensor.learnIntervalTime);
    g_uiRegBuf[48] = (uint16_t*)(&defHallSensor.learnCur);
    g_uiRegBuf[49] = (uint16_t*)(&defHallSensor.learnOrder);
    g_uiRegBuf[50] = (uint16_t*)(&defHallSensor.learnDegree);
    g_uiRegBuf[51] = (uint16_t*)(&g_hallSelfLearnBuffer.selfLearnOrder);
    g_uiRegBuf[52] = (uint16_t*)(&g_hallSelfLearnBuffer.selfLearnTheta);
    g_uiRegBuf[53] = (uint16_t*)(&g_hallSelfLearnBuffer.sectorWidthMax);
    g_uiRegBuf[54] = (uint16_t*)(&g_hallSelfLearnBuffer.sectorWidthMin);
    g_uiRegBuf[55] = (uint16_t*)(&defHallSensor.hallSelfLearnMode);
    g_uiRegBuf[56] = (uint16_t*)(&g_hallSensor.hallStatus);

    g_uiRegBuf[57] = (uint16_t*)(&defSlideMode.kslf0);
    g_uiRegBuf[58] = (uint16_t*)(&defSlideMode.kslide0);
    g_uiRegBuf[59] = (uint16_t*)(&defSlideMode.maxErr0);
    g_uiRegBuf[60] = (uint16_t*)(&defSlideMode.kslf1);
    g_uiRegBuf[61] = (uint16_t*)(&defSlideMode.kslide1);
    g_uiRegBuf[62] = (uint16_t*)(&defSlideMode.maxErr1);
    g_uiRegBuf[63] = (uint16_t*)(&defSlideMode.fullOrderGain);

    u16Temp = (uint16_t*)(&defMrasObserver.cofAlpha);
    g_uiRegBuf[64] = u16Temp;
    g_uiRegBuf[65] = u16Temp + 1;
    u16Temp = (uint16_t*)(&defMrasObserver.cofBeta);
    g_uiRegBuf[66] = u16Temp;
    g_uiRegBuf[67] = u16Temp + 1;

    g_uiRegBuf[68] = (uint16_t*)(&defFluxObserver.flitCoeff);
    u16Temp = (uint16_t*)(&defFluxObserver.compCofLimit);
    g_uiRegBuf[69] = u16Temp;
    g_uiRegBuf[70] = u16Temp + 1;

    g_uiRegBuf[71] = (uint16_t*)(&g_asrSpeedEstCof.kpPu);
    g_uiRegBuf[72] = (uint16_t*)(&g_asrSpeedEstCof.kiPu);
    g_uiRegBuf[73] = (uint16_t*)(&defPll.coeffPll);

    g_uiRegBuf[74] = &defFluxweak.voltageRef;
    g_uiRegBuf[75] = (uint16_t*)(&defFluxweak.kp);
    g_uiRegBuf[76] = (uint16_t*)(&defFluxweak.ki);
    g_uiRegBuf[77] = (uint16_t*)(&defFluxweak.min);
    g_uiRegBuf[78] = (uint16_t*)(&defFluxweak.maxModule);
    g_uiRegBuf[79] = (uint16_t*)(&defFluxweak.demagCur);
    g_uiRegBuf[80] = (uint16_t*)(&defFluxweak.nominalCur);

    g_uiRegBuf[81] = &readBack_NULL;
    g_uiRegBuf[82] = &readBack_NULL;
    g_uiRegBuf[83] = &readBack_NULL;

    g_uiRegBuf[84] = (uint16_t*)(&defShortBrake.shortBrakeCurRatio);
    g_uiRegBuf[85] = &defShortBrake.shortBrakeUdcMax;
    g_uiRegBuf[86] = &defShortBrake.shortBrakeDoneFlagTs;
    g_uiRegBuf[87] = (uint16_t*)(&defShortBrake.shortQuitCur);
    g_uiRegBuf[88] = (uint16_t*)(&g_shortBrake.subStatus);

    #if (defined PULSE_INJECTION)
    g_uiRegBuf[89] = &g_pulseInject.pwmTs;
    g_uiRegBuf[90] = (uint16_t*)(&g_pulseInject.pulseInjDoneFlag);
    g_uiRegBuf[91] = (uint16_t*)(&g_pulseInject.flowState);
    g_uiRegBuf[92] = (uint16_t*)(&g_pulseInject.subStatus);
    g_uiRegBuf[93] = &g_pulseInject.busCur[0];
    g_uiRegBuf[94] = &g_pulseInject.busCur[1];
    g_uiRegBuf[95] = &g_pulseInject.busCur[2];
    g_uiRegBuf[96] = &g_pulseInject.busCur[3];
    g_uiRegBuf[97] = &g_pulseInject.busCur[4];
    g_uiRegBuf[98] = &g_pulseInject.busCur[5];
    #else
    g_uiRegBuf[89] = &readBack_NULL;
    g_uiRegBuf[90] = &readBack_NULL;
    g_uiRegBuf[91] = &readBack_NULL;
    g_uiRegBuf[92] = &readBack_NULL;
    g_uiRegBuf[93] = &readBack_NULL;
    g_uiRegBuf[94] = &readBack_NULL;
    g_uiRegBuf[95] = &readBack_NULL;
    g_uiRegBuf[96] = &readBack_NULL;
    g_uiRegBuf[97] = &readBack_NULL;
    g_uiRegBuf[98] = &readBack_NULL;
    #endif

    g_uiRegBuf[99] = (uint16_t*)(&g_startUp.pStCtrl->startUpFlag);
    g_uiRegBuf[100] = (uint16_t*)(&g_startUp.pStCtrl->synFreReach);
    g_uiRegBuf[101] = (uint16_t*)(&defStartUp.spdSmoothCur);
    g_uiRegBuf[102] = (uint16_t*)(&defStartUp.synSpd);
    g_uiRegBuf[103] = (uint16_t*)(&defStartUp.curStartOffset);
    g_uiRegBuf[104] = &defStartUp.alignTime;
    g_uiRegBuf[105] = &defStartUp.alignTimeTwice;
    g_uiRegBuf[106] = &defStartUp.alignCur;
    g_uiRegBuf[107] = &defStartUp.alignCurTwice;
    g_uiRegBuf[108] = &defStartUp.slopeDuration1;
    g_uiRegBuf[109] = &defStartUp.slopeDuration2;
    g_uiRegBuf[110] = &defStartUp.curveFreqStart;
    g_uiRegBuf[111] = &defStartUp.curveCurStart;
    g_uiRegBuf[112] = &defStartUp.curveVoltStart;
    g_uiRegBuf[113] = &defStartUp.curveFreqSet1;
    g_uiRegBuf[114] = &defStartUp.curveFreqSet2;
    g_uiRegBuf[115] = &defStartUp.curveFreqSet3;
    g_uiRegBuf[116] = &defStartUp.curveFreqSet4;
    g_uiRegBuf[117] = &defStartUp.curveFreqSet5;
    g_uiRegBuf[118] = &defStartUp.curveCurSet1;
    g_uiRegBuf[119] = &defStartUp.curveCurSet2;
    g_uiRegBuf[120] = &defStartUp.curveCurSet3;
    g_uiRegBuf[121] = &defStartUp.curveCurSet4;
    g_uiRegBuf[122] = &defStartUp.curveCurSet5;
    g_uiRegBuf[123] = &defStartUp.curveVoltSet1;
    g_uiRegBuf[124] = &defStartUp.curveVoltSet2;
    g_uiRegBuf[125] = &defStartUp.curveVoltSet3;
    g_uiRegBuf[126] = &defStartUp.curveVoltSet4;
    g_uiRegBuf[127] = &defStartUp.curveVoltSet5;
    g_uiRegBuf[128] = (uint16_t*)(&defStartUp.curveSegNum);
    g_uiRegBuf[129] = &defStartUp.curveTime1;
    g_uiRegBuf[130] = &defStartUp.curveTime2;
    g_uiRegBuf[131] = &defStartUp.curveTime3;
    g_uiRegBuf[132] = &defStartUp.curveTime4;
    g_uiRegBuf[133] = &defStartUp.curveTime5;

    g_uiRegBuf[134] = &defSCurve.accStartRatio;
    g_uiRegBuf[135] = &defSCurve.accEndRatio;
    g_uiRegBuf[136] = &defSCurve.decStartRatio;
    g_uiRegBuf[137] = &defSCurve.decEndRatio;

    g_uiRegBuf[138] = (uint16_t*)(&g_udqDcpOut.dcpUdPu);
    g_uiRegBuf[139] = (uint16_t*)(&g_udqDcpOut.dcpUqPu);

    g_uiCommParam.dacOut1 = DUBUG_DAC_CH0_PARAMID;
    g_uiCommParam.dacOut2 = DUBUG_DAC_CH1_PARAMID;
    g_uiRegBuf[140] = &g_uiCommParam.dacOut1;
    g_uiRegBuf[141] = &g_uiCommParam.dacOut2;

    g_uiRegBuf[142] = (uint16_t*)(&g_focVars.pCtrl->iaPu);
    g_uiRegBuf[143] = (uint16_t*)(&g_focVars.pCtrl->ibPu);
    g_uiRegBuf[144] = (uint16_t*)(&g_focVars.pCtrl->ialpha);
    g_uiRegBuf[145] = (uint16_t*)(&g_focVars.pCtrl->ibeta);
    g_uiRegBuf[146] = (uint16_t*)(&g_focVars.pCtrl->iqFdb);
    g_uiRegBuf[147] = (uint16_t*)(&g_focVars.pCtrl->idFdb);
    g_uiRegBuf[148] = (uint16_t*)(&g_focVars.pCtrl->idq);
    g_uiRegBuf[149] = (uint16_t*)(&g_focVars.pCtrl->iqRef);
    g_uiRegBuf[150] = (uint16_t*)(&g_focVars.pCtrl->idRef);
    g_uiRegBuf[151] = (uint16_t*)(&g_focVars.pCtrl->vq);
    g_uiRegBuf[152] = (uint16_t*)(&g_focVars.pCtrl->vd);
    g_uiRegBuf[153] = (uint16_t*)(&g_focVars.pCtrl->vdqLimit);
    g_uiRegBuf[154] = (uint16_t*)(&g_focVars.pCtrl->valpha);
    g_uiRegBuf[155] = (uint16_t*)(&g_focVars.pCtrl->vbeta);
    #if (defined ENCODER_SENSORS)
    g_uiRegBuf[156] = &g_encoderSensor.encoderCnt;
    # elif (defined ENCODER_HALL_SENSORS)
    g_uiRegBuf[156] = &g_encoderHallSensor.encoderCntVal;
    #else
    g_uiRegBuf[156] = &readBack_NULL;
    #endif
    g_uiRegBuf[157] = &g_focVars.pCtrl->elecAngle;
    g_uiRegBuf[158] = &g_startUpCtrl.elecThetaEst;

    g_uiRegBuf[159] = &readBack_NULL;
    g_uiRegBuf[160] = &readBack_NULL;
    g_uiRegBuf[161] = &readBack_NULL;

    g_uiRegBuf[162] = &g_hfinj.hfiEstTheta;
    g_uiRegBuf[163] = (uint16_t*)(&g_hfinj.hfiEstFreq);
    g_uiRegBuf[164] = (uint16_t*)(&defHighFreqInj.hfiFreq);
    g_uiRegBuf[165] = (uint16_t*)(&defHighFreqInj.vPeak);
    g_uiRegBuf[166] = &readBack_NULL;
    g_uiRegBuf[167] = &readBack_NULL;
    g_uiRegBuf[168] = &readBack_NULL;
    g_uiRegBuf[169] = &readBack_NULL;
    g_uiRegBuf[170] = &readBack_NULL;
    g_uiRegBuf[171] = &readBack_NULL;
    g_uiRegBuf[172] = (uint16_t*)(&defHighFreqInj.hfiPllKp);
    g_uiRegBuf[173] = (uint16_t*)(&defHighFreqInj.hfiPllKi);
    g_uiRegBuf[174] = (uint16_t*)(&defHighFreqInj.bpfA0);
    g_uiRegBuf[175] = (uint16_t*)(&defHighFreqInj.bpfA2);
    u16Temp = (uint16_t*)(&defHighFreqInj.bpfB1);
    g_uiRegBuf[176] = u16Temp;
    g_uiRegBuf[177] = u16Temp + 1;
    g_uiRegBuf[178] = (uint16_t*)(&defHighFreqInj.bpfB2);
    g_uiRegBuf[179] = (uint16_t*)(&defHighFreqInj.lpfA0);
    g_uiRegBuf[180] = (uint16_t*)(&defHighFreqInj.lpfA1);
    g_uiRegBuf[181] = (uint16_t*)(&defHighFreqInj.lpfA2);
    u16Temp = (uint16_t*)(&defHighFreqInj.lpfB1);
    g_uiRegBuf[182] = u16Temp;
    g_uiRegBuf[183] = u16Temp + 1;
    g_uiRegBuf[184] = (uint16_t*)(&defHighFreqInj.lpfB2);

    g_uiRegBuf[185] = (uint16_t*)&g_powerDecrease.depCoeff;
    g_uiRegBuf[186] = (uint16_t*)&g_powerDecrease.depFlag;
    g_uiRegBuf[187] = (uint16_t*)&g_dcCurLimit.curLimitCoeff;
    g_uiRegBuf[188] = (uint16_t*)&g_powerLimit.powerLimitCoeff;

    u16Temp = (uint16_t*)(&g_estBemfDebug);
    g_uiRegBuf[189] = u16Temp;
    g_uiRegBuf[190] = u16Temp + 1;
    u16Temp = (uint16_t*)(&g_bemfReliableThreshold);
    g_uiRegBuf[191] = u16Temp;
    g_uiRegBuf[192] = u16Temp + 1;

    g_uiRegBuf[193] = (uint16_t*)(&g_adcSampleReal.iaOffset);
    g_uiRegBuf[194] = (uint16_t*)(&g_adcSampleReal.ibOffset);
    g_uiRegBuf[195] = (uint16_t*)(&g_adcSampleReal.icOffset);
    g_uiRegBuf[196] = (uint16_t*)(&g_adcSampleReal.ibusOffset);
    g_uiRegBuf[197] = &defCurRestruct.sampWinLength;
    g_uiRegBuf[198] = &defCurRestruct.adSampTim;

    g_uiRegBuf[199] = &defHwBoard.normalVoltage;
    g_uiRegBuf[200] = (uint16_t*)(&defMotorParam.statorRs);
    g_uiRegBuf[201] = (uint16_t*)(&defMotorParam.statorLs);
    g_uiRegBuf[202] = (uint16_t*)(&defMotorParam.ld);
    g_uiRegBuf[203] = (uint16_t*)(&defMotorParam.lq);
    u16Temp = (uint16_t*)(&defMotorParam.bEmf);
    g_uiRegBuf[204] = u16Temp;
    g_uiRegBuf[205] = u16Temp + 1;
    g_uiRegBuf[206] = (uint16_t*)(&defMotorParam.maxCur);
    g_uiRegBuf[207] = (uint16_t*)(&defMotorParam.polePairs);
    u16Temp = (uint16_t*)(&defMotorParam.maxSpeed);
    g_uiRegBuf[208] = u16Temp;
    g_uiRegBuf[209] = u16Temp + 1;

    g_uiRegBuf[210] = &readBack_NULL;
    g_uiRegBuf[211] = &readBack_NULL;
    g_uiRegBuf[212] = &readBack_NULL;
    g_uiRegBuf[213] = &readBack_NULL;
    g_uiRegBuf[214] = &readBack_NULL;
    g_uiRegBuf[215] = &readBack_NULL;
    g_uiRegBuf[216] = &readBack_NULL;
    g_uiRegBuf[217] = &readBack_NULL;
    g_uiRegBuf[218] = &readBack_NULL;
    g_uiRegBuf[219] = &readBack_NULL;
    g_uiRegBuf[220] = &readBack_NULL;
    g_uiRegBuf[221] = &readBack_NULL;
    g_uiRegBuf[222] = &readBack_NULL;
    g_uiRegBuf[223] = &readBack_NULL;
    g_uiRegBuf[224] = &readBack_NULL;
    g_uiRegBuf[225] = &readBack_NULL;
    g_uiRegBuf[226] = &readBack_NULL;
    g_uiRegBuf[227] = &readBack_NULL;
    g_uiRegBuf[228] = &readBack_NULL;
    g_uiRegBuf[229] = &readBack_NULL;
    g_uiRegBuf[230] = &readBack_NULL;
    g_uiRegBuf[231] = &readBack_NULL;
    g_uiRegBuf[232] = &readBack_NULL;
    g_uiRegBuf[233] = &readBack_NULL;
    g_uiRegBuf[234] = &readBack_NULL;
    g_uiRegBuf[235] = &readBack_NULL;
    g_uiRegBuf[236] = &readBack_NULL;
    g_uiRegBuf[237] = &readBack_NULL;
    g_uiRegBuf[238] = &readBack_NULL;
    g_uiRegBuf[239] = &readBack_NULL;
    g_uiRegBuf[240] = &readBack_NULL;
    g_uiRegBuf[241] = &readBack_NULL;

    g_uiCommParam.appVerYear = MOTORAPP_VER_YEAR;
    g_uiCommParam.appVerDate = (MOTORAPP_VER_MONTH << 8) + MOTORAPP_VER_DAY;
    g_uiCommParam.appVerMain = (MOTORAPP_MAIN_VER << 8) + MOTORAPP_SUB_VER;
    g_uiCommParam.appVerRelease = (MOTORAPP_REV_VER << 8) + MOTORAPP_BRANCH;
    g_uiCommParam.libVerYear = (uint16_t)g_focLibVersion.year;
    g_uiCommParam.libVerDate = ((uint16_t)g_focLibVersion.month << 8) + (uint16_t)g_focLibVersion.day;
    g_uiCommParam.libVerMain = ((uint16_t)g_focLibVersion.main << 8) + (uint16_t)g_focLibVersion.sub;
    g_uiCommParam.libVerRelease = ((uint16_t)g_focLibVersion.rev << 8) + (uint16_t)g_focLibVersion.branch;
    g_uiRegBuf[242] = &g_uiCommParam.appVerYear;
    g_uiRegBuf[243] = &g_uiCommParam.appVerDate;
    g_uiRegBuf[244] = &g_uiCommParam.appVerMain;
    g_uiRegBuf[245] = &g_uiCommParam.appVerRelease;
    g_uiRegBuf[246] = &g_uiCommParam.libVerYear;
    g_uiRegBuf[247] = &g_uiCommParam.libVerDate;
    g_uiRegBuf[248] = &g_uiCommParam.libVerMain;
    g_uiRegBuf[249] = &g_uiCommParam.libVerRelease;

#endif
}

/*!
* @brief Uart return read register result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_RegRead_Result(void)
{
    uint8_t i = 0;
    uint8_t uiRead_lenth = 0;
    uint8_t uiRead_checkSum = 0;
    uint16_t uiRead_returnTemp = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiRead_checkSum += CMD_HEAD_BYTE;

    if (g_cmdProcStatus.readRegStatus == 0x02)
    {
        uiRead_lenth =  5;
        UI_UART_PutChar(&g_uiTxBuf, uiRead_lenth);
        uiRead_checkSum += uiRead_lenth;
        /* read command */
        UI_UART_PutChar(&g_uiTxBuf, CMD_READ_REGISTER);
        uiRead_checkSum += CMD_READ_REGISTER;
        /* read result */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.readRegStatus);
        uiRead_checkSum += g_cmdProcStatus.readRegStatus;
    }
    else if (g_cmdProcStatus.readRegStatus == 0x00)
    {
        uiRead_lenth =  3 * g_cmdProcVar.uiRdBuf[0] + 6;
        UI_UART_PutChar(&g_uiTxBuf, uiRead_lenth);
        uiRead_checkSum += uiRead_lenth;
        /* read result */
        UI_UART_PutChar(&g_uiTxBuf, CMD_READ_REGISTER);
        uiRead_checkSum += CMD_READ_REGISTER;
        /* read result */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.readRegStatus);
        uiRead_checkSum += g_cmdProcStatus.readRegStatus;
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.uiRdBuf[0]);
        uiRead_checkSum += g_cmdProcVar.uiRdBuf[0];
        for (i = 1; i <= g_cmdProcVar.uiRdBuf[0]; i++)
        {
            UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.uiRdBuf[i]);
            uiRead_checkSum += g_cmdProcVar.uiRdBuf[i];
            uiRead_returnTemp = *g_uiRegBuf[g_cmdProcVar.uiRdBuf[i]];
            UI_UART_PutChar(&g_uiTxBuf, (uiRead_returnTemp & 0xFF));
            uiRead_checkSum += (uiRead_returnTemp & 0xFF);
            UI_UART_PutChar(&g_uiTxBuf, ((uiRead_returnTemp >> 8) & 0xFF));
            uiRead_checkSum += ((uiRead_returnTemp >> 8) & 0xFF);
        }
    }
    else
    {
        /* do nothing */
    }
    UI_UART_PutChar(&g_uiTxBuf, uiRead_checkSum);
}

/*!
* @brief Uart return write register result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_RegWrite_Result(void)
{
    uint8_t uiWrite_lenth = 5;
    uint8_t uiWrite_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiWrite_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiWrite_lenth);
    uiWrite_checkSum += uiWrite_lenth;
    /* write command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_WRITE_REGISTER);
    uiWrite_checkSum += CMD_WRITE_REGISTER;
    /* write result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.writeRegStatus);
    uiWrite_checkSum += g_cmdProcStatus.writeRegStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiWrite_checkSum);

}

/*!
* @brief Uart return communicate result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_Communicate_Result(void)
{
    uint8_t uiCom_lenth = 5;
    uint8_t uiCom_checkSum = 0;

    /* refresh communicate status,prepare to answer master */
    g_cmdProcStatus.communicateStatus = 0x00;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiCom_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiCom_lenth);
    uiCom_checkSum += uiCom_lenth;
    /* communication command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_CHECK_COMMUNICATION);
    uiCom_checkSum += CMD_CHECK_COMMUNICATION;
    /* communication result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.communicateStatus);
    uiCom_checkSum += g_cmdProcStatus.communicateStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiCom_checkSum);

    /* deinit communicate status to the init status */
    g_cmdProcStatus.communicateStatus = 0xFF;
}

/*!
* @brief Uart return get code version result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_GetVersion_Result(void)
{
    uint8_t uiGetVersion_lenth = 0;
    uint8_t uiGetVersion_checkSum = 0;
    int8_t codeValidState = -1;

    /* init getversion status */
    g_cmdProcStatus.getVersionStatus = 0xFF;
    codeValidState = UI_CheckCodeVersionValid(&g_focLibVersion);
    /* refresh getversion status,prepare to answer master */
    if (codeValidState)
    {
        /* version is unvalid ,get version fail */
        g_cmdProcStatus.getVersionStatus = 0x01;
    }
    else
    {
        /* version is valid ,get version success */
        g_cmdProcStatus.getVersionStatus = 0x00;
    }

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiGetVersion_checkSum += CMD_HEAD_BYTE;
    if (g_cmdProcStatus.getVersionStatus == 0x01)
    {
        uiGetVersion_lenth =  5;
        UI_UART_PutChar(&g_uiTxBuf, uiGetVersion_lenth);
        uiGetVersion_checkSum += uiGetVersion_lenth;
        /* get version command */
        UI_UART_PutChar(&g_uiTxBuf, CMD_GET_CODE_VERSION);
        uiGetVersion_checkSum += CMD_GET_CODE_VERSION;
        /* get version result */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.getVersionStatus);
        uiGetVersion_checkSum += g_cmdProcStatus.getVersionStatus;
    }
    else if (g_cmdProcStatus.getVersionStatus == 0x00)
    {
        uiGetVersion_lenth =  21;
        UI_UART_PutChar(&g_uiTxBuf, uiGetVersion_lenth);
        uiGetVersion_checkSum += uiGetVersion_lenth;
        /* get version command */
        UI_UART_PutChar(&g_uiTxBuf, CMD_GET_CODE_VERSION);
        uiGetVersion_checkSum += CMD_GET_CODE_VERSION;
        /* get version result */
        UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.getVersionStatus);
        uiGetVersion_checkSum += g_cmdProcStatus.getVersionStatus;
        /* App version year */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[242]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[242]) & 0xFF);
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[242] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[242] >> 8) & 0xFF);
        /* App version mounth */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[243] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[243] >> 8) & 0xFF);
        /* App version day */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[243]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[243]) & 0xFF);
        /* App main version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[244] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[244] >> 8) & 0xFF);
        /* App sub version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[244]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[244]) & 0xFF);
        /* App release version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[245] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[245] >> 8) & 0xFF);
        /* App branch */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[245]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[245]) & 0xFF);
        /* Lib version year */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[246]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[246]) & 0xFF);
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[246] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[246] >> 8) & 0xFF);
        /* Lib version mounth */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[247] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[247] >> 8) & 0xFF);
        /* Lib version day */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[247]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[247]) & 0xFF);
        /* Lib main version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[248] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[248] >> 8) & 0xFF);
        /* Lib sub version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[248]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[248]) & 0xFF);
        /* Lib release version */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[249] >> 8) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[249] >> 8) & 0xFF);
        /* Lib branch */
        UI_UART_PutChar(&g_uiTxBuf, ((*g_uiRegBuf[249]) & 0xFF));
        uiGetVersion_checkSum += ((*g_uiRegBuf[249]) & 0xFF);
    }
    else
    {
        /* do nothing */
    }
    UI_UART_PutChar(&g_uiTxBuf, uiGetVersion_checkSum);
}

/*!
* @brief Uart return start motor result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StartMotor_Result(void)
{
    uint8_t uiStartMotor_lenth = 5;
    uint8_t uiStartMotor_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStartMotor_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStartMotor_lenth);
    uiStartMotor_checkSum += uiStartMotor_lenth;
    /* start motor command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_MOTOR);
    uiStartMotor_checkSum += CMD_START_MOTOR;
    /* start motor result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.startMotorStatus);
    uiStartMotor_checkSum += g_cmdProcStatus.startMotorStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStartMotor_checkSum);
}

/*!
* @brief Uart return stop motor result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StopMotor_Result(void)
{
    uint8_t uiStopMotor_lenth = 5;
    uint8_t uiStopMotor_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStopMotor_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStopMotor_lenth);
    uiStopMotor_checkSum += uiStopMotor_lenth;
    /* stop motor command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_STOP_MOTOR);
    uiStopMotor_checkSum += CMD_STOP_MOTOR;
    /* stop motor result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.stopMotorStatus);
    uiStopMotor_checkSum += g_cmdProcStatus.stopMotorStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStopMotor_checkSum);
}

/*!
* @brief Uart return stop ramp result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StopRamp_Result(void)
{
    uint8_t uiStopRamp_lenth = 5;
    uint8_t uiStopRamp_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStopRamp_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStopRamp_lenth);
    uiStopRamp_checkSum += uiStopRamp_lenth;
    /* stop motor command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_STOP_RAMP);
    uiStopRamp_checkSum += CMD_STOP_RAMP;
    /* stop motor result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.stopRampStatus);
    uiStopRamp_checkSum += g_cmdProcStatus.stopRampStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStopRamp_checkSum);
}

/*!
* @brief Uart return identify motor paramerters result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_Identify_Result(void)
{
    uint8_t uiIdentify_lenth = 5;
    uint8_t uiIdentify_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiIdentify_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiIdentify_lenth);
    uiIdentify_checkSum += uiIdentify_lenth;
    /* parameter identification command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_PARAM_IDENTIFY);
    uiIdentify_checkSum += CMD_START_PARAM_IDENTIFY;
    /* parameter identification result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.identifyStatus);
    uiIdentify_checkSum += g_cmdProcStatus.identifyStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiIdentify_checkSum);
}

/*!
* @brief Uart return start motor schedule to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StartMotor_Schedule(void)
{
    uint8_t uiStartMotorSche_lenth = 0;
    uint8_t uiStartMotorSche_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStartMotorSche_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStartMotorSche_lenth);
    uiStartMotorSche_checkSum += uiStartMotorSche_lenth;
    /* Query process command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_QUERY_PROCESS);
    uiStartMotorSche_checkSum += CMD_QUERY_PROCESS;
    /* start motor command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_MOTOR);
    uiStartMotorSche_checkSum += CMD_START_MOTOR;

    /* StartMotor Schedule */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.cmdProcStatus);
    uiStartMotorSche_checkSum += g_cmdProcStatus.cmdProcStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStartMotorSche_checkSum);
}

/*!
* @brief Uart return identify motor paramerters schedule to master.
*
* @param[in] none
* @return none
*/
void UI_Return_Identify_Schedule(void)
{
    uint8_t uiIdentifySche_lenth = 0;
    uint8_t uiIdentifySche_checkSum = 0;
    uint16_t uiIdentifySche_duty = 0;
    float temp_Bemf = 0.0;
    uint16_t temp_BemfIntegerPart = 0;
    uint16_t temp_BemfDecimalPart = 0;

//    uiIdentifySche_lenth = (g_paramIdentifyCtrl.mainStep == 4) ? 20 : 8;
    uiIdentifySche_lenth = 20;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiIdentifySche_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiIdentifySche_lenth);
    uiIdentifySche_checkSum += uiIdentifySche_lenth;
    /* Query process command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_QUERY_PROCESS);
    uiIdentifySche_checkSum += CMD_QUERY_PROCESS;
    /* identify Query ok */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.cmdProcStatus);
    uiIdentifySche_checkSum += g_cmdProcStatus.cmdProcStatus;
    /* identify command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_PARAM_IDENTIFY);
    uiIdentifySche_checkSum += CMD_START_PARAM_IDENTIFY;
    uiIdentifySche_duty = ParamIdentify_DutyCheck(&g_paramIdentify);
    /* paramidentify Schedule Low byte, step value */
    UI_UART_PutChar(&g_uiTxBuf, uiIdentifySche_duty);
    uiIdentifySche_checkSum += uiIdentifySche_duty;
    /* paramidentify Schedule High byte, duty value */
    UI_UART_PutChar(&g_uiTxBuf, ((uiIdentifySche_duty >> 8) & 0xFF));
    uiIdentifySche_checkSum += ((uiIdentifySche_duty >> 8) & 0xFF);
//    if (uiIdentifySche_lenth == 20)
//    {
    temp_Bemf = g_paramIdentifyCtrl.bEmf;
    temp_BemfIntegerPart = (uint16_t)temp_Bemf;
    temp_BemfDecimalPart = (uint16_t)((temp_Bemf - temp_BemfIntegerPart) * 10000);

//    /* RS Low byte */
//    UI_UART_PutChar(&g_uiTxBuf, (g_paramIdentifyCtrl.rsId & 0xFF));
//    uiIdentifySche_checkSum += (g_paramIdentifyCtrl.rsId & 0xFF);
//    /* RS High byte */
//    UI_UART_PutChar(&g_uiTxBuf, ((g_paramIdentifyCtrl.rsId >> 8) & 0xFF));
//    uiIdentifySche_checkSum += ((g_paramIdentifyCtrl.rsId >> 8) & 0xFF);
//    /* LD Low byte */
//    UI_UART_PutChar(&g_uiTxBuf, (g_paramIdentifyCtrl.ld & 0xFF));
//    uiIdentifySche_checkSum += (g_paramIdentifyCtrl.ld & 0xFF);
//    /* LD High byte */
//    UI_UART_PutChar(&g_uiTxBuf, ((g_paramIdentifyCtrl.ld >> 8) & 0xFF));
//    uiIdentifySche_checkSum += ((g_paramIdentifyCtrl.ld >> 8) & 0xFF);
//    /* LQ Low byte */
//    UI_UART_PutChar(&g_uiTxBuf, (g_paramIdentifyCtrl.lq & 0xFF));
//    uiIdentifySche_checkSum += (g_paramIdentifyCtrl.lq & 0xFF);
//    /* LQ High byte */
//    UI_UART_PutChar(&g_uiTxBuf, ((g_paramIdentifyCtrl.lq >> 8) & 0xFF));
//    uiIdentifySche_checkSum += ((g_paramIdentifyCtrl.lq >> 8) & 0xFF);

    /* Bemf integer Low byte */
    UI_UART_PutChar(&g_uiTxBuf, (temp_BemfIntegerPart & 0xFF));
    uiIdentifySche_checkSum += (temp_BemfIntegerPart & 0xFF);
    /* Bemf integer High byte */
    UI_UART_PutChar(&g_uiTxBuf, ((temp_BemfIntegerPart >> 8) & 0xFF));
    uiIdentifySche_checkSum += ((temp_BemfIntegerPart >> 8) & 0xFF);
    /* Bemf decimal Low byte */
    UI_UART_PutChar(&g_uiTxBuf, (temp_BemfDecimalPart & 0xFF));
    uiIdentifySche_checkSum += (temp_BemfDecimalPart & 0xFF);
    /* Bemf decimal High byte */
    UI_UART_PutChar(&g_uiTxBuf, ((temp_BemfDecimalPart >> 8) & 0xFF));
    uiIdentifySche_checkSum += ((temp_BemfDecimalPart >> 8) & 0xFF);

    /* Inertia Low byte */
    UI_UART_PutChar(&g_uiTxBuf, 0x00);
    uiIdentifySche_checkSum += 0x00;
    /* Inertia High byte */
    UI_UART_PutChar(&g_uiTxBuf, 0x00);
    uiIdentifySche_checkSum += 0x00;

    /* reset init motor identify param */
    if (g_paramIdentifyCtrl.mainStep == IDENTIFY_END)
    {
        g_paramIdentifyCtrl.initCtrl = 1;
    }
//    }
    UI_UART_PutChar(&g_uiTxBuf, uiIdentifySche_checkSum);
}

/*!
* @brief Uart return identify motor paramerters schedule to master.
*
* @param[in] none
* @return none
*/
void UI_Return_HallSelfLearn_Schedule(void)
{
    uint8_t uiHallSelfLearnSche_lenth = 0;
    uint8_t uiHallSelfLearnSche_checkSum = 0;
    uint16_t uiHallSelfLearnSche_duty = 0;

    uiHallSelfLearnSche_lenth = 16;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiHallSelfLearnSche_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiHallSelfLearnSche_lenth);
    uiHallSelfLearnSche_checkSum += uiHallSelfLearnSche_lenth;
    /* Query process command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_QUERY_PROCESS);
    uiHallSelfLearnSche_checkSum += CMD_QUERY_PROCESS;
    /* slef learn Query ok */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.cmdProcStatus);
    uiHallSelfLearnSche_checkSum += g_cmdProcStatus.cmdProcStatus;
    /* slef learn command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_HALLSELFLEARN);
    uiHallSelfLearnSche_checkSum += CMD_START_HALLSELFLEARN;

    uiHallSelfLearnSche_duty = HallSelfLearnDutyCheck(&g_hall);

    /* slef learn Schedule low byte */
    UI_UART_PutChar(&g_uiTxBuf, (uiHallSelfLearnSche_duty & 0xFF));
    uiHallSelfLearnSche_checkSum += (uiHallSelfLearnSche_duty & 0xFF);
    /* slef learn Schedule high byte */
    UI_UART_PutChar(&g_uiTxBuf, ((uiHallSelfLearnSche_duty >> 8) & 0xFF));
    uiHallSelfLearnSche_checkSum += ((uiHallSelfLearnSche_duty >> 8) & 0xFF);

    if (uiHallSelfLearnSche_duty != 25615)
    {
        /* selflearn order low byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;
        /* selflearn order high byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;

        /* selflearn theta low byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;
        /* selflearn theta high byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;

        /* selflearn sector width max low byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;
        /* selflearn sector width max high byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;

        /* selflearn sector width min low byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;
        /* selflearn sector width min high byte */
        UI_UART_PutChar(&g_uiTxBuf, 0x00);
        uiHallSelfLearnSche_checkSum += 0x00;
    }
    else
    {
        /* selflearn order low byte */
        UI_UART_PutChar(&g_uiTxBuf, (g_hallSelfLearnBuffer.selfLearnOrder & 0xFF));
        uiHallSelfLearnSche_checkSum += (g_hallSelfLearnBuffer.selfLearnOrder & 0xFF);
        /* selflearn order high byte */
        UI_UART_PutChar(&g_uiTxBuf, ((g_hallSelfLearnBuffer.selfLearnOrder>> 8) & 0xFF));
        uiHallSelfLearnSche_checkSum += ((g_hallSelfLearnBuffer.selfLearnOrder>> 8) & 0xFF);

        /* selflearn theta low byte */
        UI_UART_PutChar(&g_uiTxBuf, (g_hallSelfLearnBuffer.selfLearnTheta & 0xFF));
        uiHallSelfLearnSche_checkSum += (g_hallSelfLearnBuffer.selfLearnTheta & 0xFF);
        /* selflearn theta high byte */
        UI_UART_PutChar(&g_uiTxBuf, ((g_hallSelfLearnBuffer.selfLearnTheta >> 8) & 0xFF));
        uiHallSelfLearnSche_checkSum += ((g_hallSelfLearnBuffer.selfLearnTheta >> 8) & 0xFF);

        /* selflearn sector width max low byte */
        UI_UART_PutChar(&g_uiTxBuf, (g_hallSelfLearnBuffer.sectorWidthMax & 0xFF));
        uiHallSelfLearnSche_checkSum += (g_hallSelfLearnBuffer.sectorWidthMax & 0xFF);
        /* selflearn sector width max high byte */
        UI_UART_PutChar(&g_uiTxBuf, ((g_hallSelfLearnBuffer.sectorWidthMax >> 8) & 0xFF));
        uiHallSelfLearnSche_checkSum += ((g_hallSelfLearnBuffer.sectorWidthMax >> 8) & 0xFF);

        /* selflearn sector width min low byte */
        UI_UART_PutChar(&g_uiTxBuf, (g_hallSelfLearnBuffer.sectorWidthMin & 0xFF));
        uiHallSelfLearnSche_checkSum += (g_hallSelfLearnBuffer.sectorWidthMin & 0xFF);
        /* selflearn sector width min high byte */
        UI_UART_PutChar(&g_uiTxBuf, ((g_hallSelfLearnBuffer.sectorWidthMin >> 8) & 0xFF));
        uiHallSelfLearnSche_checkSum += ((g_hallSelfLearnBuffer.sectorWidthMin >> 8) & 0xFF);
    }

    /* stop hall self learn */
    if (g_hallSelfLearnFinishFlag == 1)
    {
        defHallSensor.hallSelfLearnEnable = DISABLE;
        g_hallSelfLearnFinishFlag = 0;
//        HallSelfLearnResultInit(&g_hallSelfLearnBuffer);
    }

    UI_UART_PutChar(&g_uiTxBuf, uiHallSelfLearnSche_checkSum);
}

/*!
* @brief Uart return clear fault result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_ClearFault_Result(void)
{
    uint8_t uiClearFault_lenth = 5;
    uint8_t uiClearFault_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiClearFault_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiClearFault_lenth);
    uiClearFault_checkSum += uiClearFault_lenth;
    /* clear fault command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_MOTOR_FAULT_CLEAR);
    uiClearFault_checkSum += CMD_MOTOR_FAULT_CLEAR;
    /* Clear Fault result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.clearFaultStatus);
    uiClearFault_checkSum += g_cmdProcStatus.clearFaultStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiClearFault_checkSum);
}

/*!
* @brief Uart return stop identify motor paramerters result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StopIdentify_Result(void)
{
    uint8_t uiIdentifyOff_lenth = 5;
    uint8_t uiIdentifyOff_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiIdentifyOff_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiIdentifyOff_lenth);
    uiIdentifyOff_checkSum += uiIdentifyOff_lenth;
    /* stop paramidentify command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_STOP_PARAM_IDENTIFY);
    uiIdentifyOff_checkSum += CMD_STOP_PARAM_IDENTIFY;
    /* stop paramidentify result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.identifyOffStatus);
    uiIdentifyOff_checkSum += g_cmdProcStatus.identifyOffStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiIdentifyOff_checkSum);
}

/*!
* @brief Uart return check err result to master,occur when check err.
*
* @param[in] none
* @return none
*/
void UI_ReturnCheckErr_Result(void)
{
    uint8_t uiCheckErr_lenth = 5;
    uint8_t uiCheckErr_checkSum = 0;

    switch (g_cmdProcVar.uiCheckCmd)
    {
        case CMD_CHECK_COMMUNICATION:
            g_cmdProcStatus.communicateStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.communicateStatus;
        break;

        case CMD_GET_CODE_VERSION:
            g_cmdProcStatus.getVersionStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.getVersionStatus;
        break;

        case CMD_START_MOTOR:
            g_cmdProcStatus.startMotorStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.startMotorStatus;
        break;

        case CMD_STOP_MOTOR:
            g_cmdProcStatus.stopMotorStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.stopMotorStatus;
        break;

        case CMD_STOP_RAMP:
            g_cmdProcStatus.stopRampStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.stopRampStatus;
        break;

        case CMD_START_PARAM_IDENTIFY:
            g_cmdProcStatus.identifyStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.identifyStatus;
        break;

        case CMD_STOP_PARAM_IDENTIFY:
            g_cmdProcStatus.identifyOffStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.identifyOffStatus;
        break;

        case CMD_START_HALLSELFLEARN:
            g_cmdProcStatus.startHallSelfLearnStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.startHallSelfLearnStatus;
        break;

        case CMD_STOP_HALLSELFLEARN:
            g_cmdProcStatus.stopHallSelfLearnStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.stopHallSelfLearnStatus;
        break;

        case CMD_MOTOR_FAULT_CLEAR:
            g_cmdProcStatus.clearFaultStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.clearFaultStatus;
        break;

        case CMD_READ_REGISTER:
            g_cmdProcStatus.readRegStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.readRegStatus;
        break;

        case CMD_WRITE_REGISTER:
            g_cmdProcStatus.writeRegStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.writeRegStatus;
        break;

        case CMD_QUERY_PROCESS:
            g_cmdProcStatus.writeRegStatus = 0x04;
            g_cmdProcStatus.checkeErrStatus = g_cmdProcStatus.writeRegStatus;
        break;

        default:
        break;
    }

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiCheckErr_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiCheckErr_lenth);
    uiCheckErr_checkSum += uiCheckErr_lenth;
    /* command id while checksum err happens */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcVar.uiCheckCmd);
    uiCheckErr_checkSum += g_cmdProcVar.uiCheckCmd;
    /* Check err result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.checkeErrStatus);
    uiCheckErr_checkSum += g_cmdProcStatus.checkeErrStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiCheckErr_checkSum);
}

/*!
* @brief Uart communication consume time statistics,units: 2ms.
*
* @param[in] none
* @return none
*/
void UI_UART_TimeOut_Cnt(void)
{
    if (g_cmdProcVar.uartTimeOutCntEn == 1)
    {
        g_cmdProcVar.uartTimeOutCnt++;
    }
    else if (g_cmdProcVar.uartTimeOutCntEn == 0)
    {
        g_cmdProcVar.uartTimeOutCnt = 0;
    }
    else
    {

    }
}

/*!
* @brief Uart judge whether start motor success or not.
*
* @param[in] none
* @return none
*/
void UI_JudgeStartMotor(void)
{
#if ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS))
    if (defHallSensor.hallSelfLearnEnable == DISABLE)
    {
        if (Get_McStatus() == RUN)
        {
            /* start motor success */
            g_cmdProcStatus.startMotorStatus = 0x00;
            /* stop count time,prepare to answer master */
            g_cmdProcVar.cmdProcTimeOutCntEn = 0;
        }
        else
        {
            /* starting motor ,fail */
            g_cmdProcStatus.startMotorStatus = 0x01;
            /* out of start motor excute limit time,still not start ready */
            if (g_cmdProcVar.cmdProcTimeOutCnt > START_MOTOR_MAXCNT)
            {
                /* stop count time,prepare to answer master */
                g_cmdProcVar.cmdProcTimeOutCntEn = 0;
            }
        }
    }
    else
    {
        /* in hall self learn mode,starting motor fail */
        g_cmdProcStatus.startMotorStatus = 0x01;
        /* stop count time,prepare to answer master */
        g_cmdProcVar.cmdProcTimeOutCntEn = 0;
    }
#elif (defined ENCODER_SENSORS)
    if ((Get_McStatus() == ALIGN) || (Get_McStatus() == RUN))
    {
        /* start motor success */
        g_cmdProcStatus.startMotorStatus = 0x00;
        /* stop count time,prepare to answer master */
        g_cmdProcVar.cmdProcTimeOutCntEn = 0;
    }
    else
    {
        /* starting motor ,fail */
        g_cmdProcStatus.startMotorStatus = 0x01;
        /* out of start motor excute limit time,still not start ready */
        if (g_cmdProcVar.cmdProcTimeOutCnt > START_MOTOR_MAXCNT)
        {
            /* stop count time, prepare to answer master */
            g_cmdProcVar.cmdProcTimeOutCntEn = 0;
        }
    }
#elif (defined SENSORLESS)
    if ((Get_McStatus() == ANGLE_JUDGE) || (Get_McStatus() == ALIGN) || (Get_McStatus() == RUN))
    {
        /* start motor success */
        g_cmdProcStatus.startMotorStatus = 0x00;
        /* stop count time, prepare to answer master */
        g_cmdProcVar.cmdProcTimeOutCntEn = 0;
    }
    else
    {
        /* starting motor, fail */
        g_cmdProcStatus.startMotorStatus = 0x01;
        /* out of start motor excute limit time */
        if (g_cmdProcVar.cmdProcTimeOutCnt > START_MOTOR_MAXCNT)
        {
            /* stop count time, prepare to answer master */
            g_cmdProcVar.cmdProcTimeOutCntEn = 0;
        }
    }
#endif
    if (g_cmdProcVar.cmdProcTimeOutCntEn == 0)
    {
        UI_Return_StartMotor_Result();
//         DEBUGMSG(DEBUG_ZONE_INFO, ("Cmd id = %d,start motor status is = %d\r\n", CMD_START_MOTOR, g_cmdProcStatus.startMotorStatus));//sync log
    }
}

/*!
* @brief Uart judge whether identify motor paramerters execute success or not.
*
* @param[in] none
* @return none
*/
void UI_JudgeStartIdentify(void)
{
    if (Get_McStatus() == IDENTIFY)
    {
        /* identify start success, prepare to report schedule */
        g_cmdProcStatus.identifyStatus = 0x00;
        /* stop count time, prepare to answer master */
        g_cmdProcVar.cmdProcTimeOutCntEn = 0;
    }
    else
    {
        /* identify start fail */
        g_cmdProcStatus.identifyStatus = 0x01;
        /* stop count time, prepare to answer master */
        g_cmdProcVar.cmdProcTimeOutCntEn = 0;
    }

    if (g_cmdProcVar.cmdProcTimeOutCntEn == 0)
    {
        UI_Return_Identify_Result();
    }
}

/*!
* @brief master command process time statistics,units: 2ms.
*
* @param[in] none
* @return none
*/
void UI_MasterCmd_ProcCnt(void)
{
    if (g_cmdProcVar.cmdProcTimeOutCntEn != 0)
    {
        g_cmdProcVar.cmdProcTimeOutCnt++;
        if (g_cmdProcVar.cmdProcTimeOutCntEn == 1)
        {
            UI_JudgeStartMotor();
        }
        else if (g_cmdProcVar.cmdProcTimeOutCntEn == 2)
        {
            UI_JudgeStartIdentify();
        }
        else
        {

        }
    }
    else if (g_cmdProcVar.cmdProcTimeOutCntEn == 0)
    {
        g_cmdProcVar.cmdProcTimeOutCnt = 0;
    }
}

/*!
* @brief Tool Uart peripheral initialization.
*
* @param[in] none
* @return none
*/
void UI_UART_Initialize()
{
    uart_user_config_t uartConfig;
    uart_state_t uartState;

    /*!
    * GPIO Configuration for UART1
    * PC2     --> UART1_Tx
    * PC3     --> UART1_Rx
    */

    /* UART Tx pin */
    GPIO_DRV_SetMuxModeSel(PORTC, (gpio_channel_type_t)9U, PORT_MUX_ALT2);
    /* UART Rx pin */
    GPIO_DRV_SetMuxModeSel(PORTC, (gpio_channel_type_t)8U, PORT_MUX_ALT2);

    memset(&uartConfig, 0U, sizeof(uart_user_config_t));
    memset(&uartState, 0U, sizeof(uart_state_t));
    uartConfig.baudRate = TOOL_UART_BAUDRATE;
    uartConfig.parityMode = UART_PARITY_DISABLED;
    uartConfig.bitCountPerChar = UART_8_BITS_PER_CHAR;
    uartConfig.stopBitCount = UART_ONE_STOP_BIT;
    uartConfig.transferType = UART_USING_INTERRUPTS;
//    uartConfig.fifoByteEn = ENABLE;
//    uartConfig.dmaEn = UART_DMA_TXRX_NONE;
    uartState.txCallback = UI_UART_txCallback;
    uartState.rxCallback = UI_UART_rxCallback;
//    uartConfig.callBack = UI_UART_IRQHandler;
    /* set Uart1 config */
    UART_DRV_Init(((uint32_t)(((uint32_t)(UART1) - UART0_BASE) >> 9)), &uartState, &uartConfig);
//    UART_Init(UART1, &uartConfig);

    /* clear TC, TC_FLAG = 0 */
    UART1->LSR0 &= (~UART_LSR0_TC_Msk);
    /* enable TC interrupt, ETC = 1 */
    UART1->IER |= UART_IER_ETC_Msk;
    /* enable RXNE interrupt, ERXNE = 1 */
    UART1->IER |= UART_IER_ERXNE_Msk;
//    UART_SetRXNEInterrupt(UART1, ENABLE);

    NVIC_ClearPendingIRQ(UART1_IRQn);
    NVIC_EnableIRQ(UART1_IRQn);
    NVIC_SetPriority(UART1_IRQn, 3);

}

/*!
* @brief Tool Uart peripheral initialization.
*
* @param[in] none
* @return none
*/
void UI_ToolUart_Init()
{
    /* Tool uart peripheral init */
    UI_UART_Initialize();

    /* Tool uart communication register table init */
    UI_RegBuf_Init();
}

/*!
* @brief Master start motor through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StartMotor(void)
{
    /* init start motor status */
    g_cmdProcStatus.startMotorStatus = 0xFF;
    if ((SHORT_BRAKE_CTRL & 0x01) && (g_uiCommParam.appMode & 0x0300))
    {
        Set_McStatus(START_BRAKE);
    }
    else
    {
        Set_McStatus(START);
    }
    /* disable param identify */
    g_paramIdentifyCtrl.identifyCtrl = (uint8_t)DISABLE;
    /* start start motor excute time count */
    g_cmdProcVar.cmdProcTimeOutCntEn = 1;
}

/*!
* @brief Master stop motor through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StopMotor(void)
{
    /* init stop motor status */
    g_cmdProcStatus.stopMotorStatus = 0xFF;
    if (g_uiCommParam.appMode & 0x0400)
    {
        if (defHallSensor.hallSelfLearnEnable == ENABLE)
        {
//            HallSensor_Init(PWDT_GetHallStatus(PWDT1), &g_hall, &g_focVars);
            HallSensor_Init(PWM_DRV_GetHallStatus(PWM_MODULER_HALL), &g_hall, &g_focVars);
            Set_McStatus(STOP);
        }
        else if (STOP_MODE == SLOW_DOWN_STOP)
        {
            Set_McStatus(SLOW_DOWN);
        }
        else if (SHORT_BRAKE_CTRL > 1)
        {
            Set_McStatus(STOP_BRAKE);
        }
        else
        {
            Set_McStatus(STOP);
        }
    }
    else
    {
        Set_McStatus(STOP);
    }
    g_paramIdentifyCtrl.identifyCtrl = (uint8_t)DISABLE;
    /* stop motor success, prepare to answer master */
    g_cmdProcStatus.stopMotorStatus = 0x00;
    UI_Return_StopMotor_Result();
}

/*!
* @brief Master stop motor ramp start through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StopRamp(void)
{
    /* init stop ramp status */
    g_cmdProcStatus.stopRampStatus = 0xFF;

    #if (defined SENSORLESS)
    if (g_startUpCtrl.startUpFlag == 6)
    {
        /* stop ramp */
        g_speedCommand.speedTarget = g_speedCommand.speedTargetRamp;
        /* stop ramp success, prepare to answer master */
        g_cmdProcStatus.stopRampStatus = 0x00;
    }
    #elif (defined HALL_SENSORS)
    if (g_hallSensor.swapstate  > 0)
    {
        /* stop ramp */
        g_speedCommand.speedTarget = g_speedCommand.speedTargetRamp;
        /* stop ramp success, prepare to answer master */
        g_cmdProcStatus.stopRampStatus = 0x00;
    }
    #else
    /* Encoder */
    if (Get_McStatus() == RUN)
    {
        /* stop ramp */
        g_speedCommand.speedTarget = g_speedCommand.speedTargetRamp;
        /* stop ramp success,prepare to answer master */
        g_cmdProcStatus.stopRampStatus = 0x00;
    }
    #endif

    else
    {
        /* set ramp param,but can not stop at startup,unvalid command */
        g_cmdProcStatus.stopRampStatus = 0x02;
    }

    UI_Return_StopRamp_Result();
}

/*!
* @brief Master start motor parameters identify through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @param[in] paramPu: pointer to MOTOR_PARAM_PU structure, store motor paramerters
* @return none
*/
void UI_StartIdentify(UART_BUF *uiRxBuf, MOTOR_PARAM_PU *paramPu)
{
    uint16_t i = 0;

    /* init param identify status */
    g_cmdProcStatus.identifyStatus = 0xFF;
    for (i = 0; i < 12; i++)
    {
        /* save identify param */
        g_cmdProcVar.uiIdentifyBuf[i] = UI_UART_GetChar(uiRxBuf);
    }
    /* refresh motor param set by master */
    /* pole pairs num */
    paramPu->pairsb = g_cmdProcVar.uiIdentifyBuf[0] + (g_cmdProcVar.uiIdentifyBuf[1] << 8);
    /* Vdc */
    paramPu->ubVt = (g_cmdProcVar.uiIdentifyBuf[6] + (g_cmdProcVar.uiIdentifyBuf[7] << 8)) / 100;
    /* Ibus */
    paramPu->ibAp = (g_cmdProcVar.uiIdentifyBuf[4] + (g_cmdProcVar.uiIdentifyBuf[5] << 8)) / 100;
    /* caculate elec base freq */
    g_focVars.pCfg->baseFreq = (g_cmdProcVar.uiIdentifyBuf[2] + (g_cmdProcVar.uiIdentifyBuf[3] << 8)) * paramPu->pairsb / 60;
    paramPu->fbHz = g_focVars.pCfg->baseFreq;

    g_paramIdentify.pParamIdentifyCfg->identifyParSel = g_cmdProcVar.uiIdentifyBuf[10] + (g_cmdProcVar.uiIdentifyBuf[11] << 8);

    if (Get_McStatus() == IDLE)
    {
        g_paramIdentifyCtrl.identifyCtrl = (uint8_t)ENABLE;
        /* turn to identify mode */
        Set_McStatus(IDENTIFY);
    }
    else
    {
        g_paramIdentifyCtrl.identifyCtrl = (uint8_t)DISABLE;
    }

    /* start param identify excute time count */
    g_cmdProcVar.cmdProcTimeOutCntEn = 2;
}

/*!
* @brief Master query command process schedule through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_QuerySchedule(UART_BUF *uiRxBuf)
{
    uint8_t queryCmd = 0;

    /* init query command excute status */
    g_cmdProcStatus.cmdProcStatus = 0xFF;
    /* get query command id */
    queryCmd = UI_UART_GetChar(uiRxBuf);
    switch (queryCmd)
    {
    case CMD_START_MOTOR:
//     UI_Return_StartMotor_Schedule();
    break;

    case CMD_START_PARAM_IDENTIFY:
    g_cmdProcStatus.cmdProcStatus = 0x00;
    /* query command identify,report schedule */
    UI_Return_Identify_Schedule();
    break;

    case CMD_START_HALLSELFLEARN:
    g_cmdProcStatus.cmdProcStatus = 0x00;
    /* query command hall self learn, report schedule */
    UI_Return_HallSelfLearn_Schedule();
    break;

    default:
    break;
    }

}

/*!
* @brief Master stop motor parameters identify through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StopIdentify(void)
{
    /* init stop param identify status */
    g_cmdProcStatus.identifyOffStatus = 0xFF;
    Set_McStatus(STOP);
    /* init motor identify param again */
    g_paramIdentifyCtrl.initCtrl = 1;

    Ldq_IdentifyConfigReset(&g_focVars);
    /* refresh stop param identify status,prepare to answer master */
    g_cmdProcStatus.identifyOffStatus = 0x00;
    UI_Return_StopIdentify_Result();
}

/*!
* @brief Uart return flash operation result to master.
*
* @param[in] cmd: Tool command
* @return none
*/
#if (defined DATA_ACCESS_FLASH_ENABLE)
void UI_Return_FlashTask_Result(uint8_t cmd)
{
    uint8_t uiFlashResult_length = 5;
    uint8_t uiFlashResult_checkSum = 0;
    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiFlashResult_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiFlashResult_length);
    uiFlashResult_checkSum += uiFlashResult_length;
    /* flash operation command */
    UI_UART_PutChar(&g_uiTxBuf, cmd);
    uiFlashResult_checkSum += cmd;
    /* flash operation result */
    UI_UART_PutChar(&g_uiTxBuf, g_flashTaskFinish);
    uiFlashResult_checkSum += g_flashTaskFinish;
    UI_UART_PutChar(&g_uiTxBuf, uiFlashResult_checkSum);
}
#endif

/*!
* @brief Master recovery data from flash through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @param[in] cmd: Tool command
* @return none
*/
#if (defined DATA_ACCESS_FLASH_ENABLE)
void UI_FlashRecoveryData(UART_BUF *uiRxBuf, uint8_t cmd)
{
    uint16_t tempCmd = 0;
    g_dataAccessFlash.recoveryEnable = 1;
    if (Get_McStatus() == IDLE)
    {
        Set_McStatus(FLASH_TASK);
        tempCmd = UI_UART_GetChar(uiRxBuf);
        if (tempCmd == 0)
        {
            g_dataAccessFlash.recoveryMode = 1;
        }
        else if (tempCmd == 1)
        {
            g_dataAccessFlash.recoveryMode = 2;
        }
        else
        {
            g_dataAccessFlash.recoveryMode = 0;
        }
    }
    UI_Return_FlashTask_Result(cmd);
}
#endif

/*!
* @brief Master save data to flash through UI interface.
*
* @param[in] cmd: Tool command
* @return none
*/
#if (defined DATA_ACCESS_FLASH_ENABLE)
void UI_FlashSaveData(uint8_t cmd)
{
    if (Get_McStatus() == IDLE)
    {
        g_dataAccessFlash.saveEnable = 1;
        g_dataAccessFlash.saveMode = 2;
        Set_McStatus(FLASH_TASK);
    }
    UI_Return_FlashTask_Result(cmd);
}
#endif

/*!
* @brief Uart return flash operation result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StartHallSelfLearn_Result(void)
{
    uint8_t uiStartHallSelfLearn_length = 5;
    uint8_t uiStartHallSelfLearnResult_checkSum = 0;
    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStartHallSelfLearnResult_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStartHallSelfLearn_length);
    uiStartHallSelfLearnResult_checkSum += uiStartHallSelfLearn_length;
    /* flash operation command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_START_HALLSELFLEARN);
    uiStartHallSelfLearnResult_checkSum += CMD_START_HALLSELFLEARN;
    /* flash operation result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.startHallSelfLearnStatus);
    uiStartHallSelfLearnResult_checkSum += g_cmdProcStatus.startHallSelfLearnStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStartHallSelfLearnResult_checkSum);
}

/*!
* @brief Uart return flash operation result to master.
*
* @param[in] none
* @return none
*/
void UI_Return_StopHallSelfLearn_Result(void)
{
    uint8_t uiStopHallSelfLearnResult_length = 5;
    uint8_t uiStopHallSelfLearnResult_checkSum = 0;

    UI_UART_PutChar(&g_uiTxBuf, CMD_HEAD_BYTE);
    uiStopHallSelfLearnResult_checkSum += CMD_HEAD_BYTE;
    UI_UART_PutChar(&g_uiTxBuf, uiStopHallSelfLearnResult_length);
    uiStopHallSelfLearnResult_checkSum += uiStopHallSelfLearnResult_length;
    /* flash operation command */
    UI_UART_PutChar(&g_uiTxBuf, CMD_STOP_HALLSELFLEARN);
    uiStopHallSelfLearnResult_checkSum += CMD_STOP_HALLSELFLEARN;
    /* flash operation result */
    UI_UART_PutChar(&g_uiTxBuf, g_cmdProcStatus.stopHallSelfLearnStatus);
    uiStopHallSelfLearnResult_checkSum += g_cmdProcStatus.stopHallSelfLearnStatus;
    UI_UART_PutChar(&g_uiTxBuf, uiStopHallSelfLearnResult_checkSum);
}

/*!
* @brief Master start Hall self learning through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_StartHallSensorSelfLearn(UART_BUF *uiRxBuf)
{
    uint8_t modeSelect = 0;

    g_cmdProcStatus.startHallSelfLearnStatus = 0xFF;

    modeSelect = UI_UART_GetChar(uiRxBuf);
    defHallSensor.hallSelfLearnMode = modeSelect;

    if (Get_McStatus() == IDLE)
    {
        defHallSensor.hallSelfLearnEnable = ENABLE;
        Set_McStatus(HALL_SELFLEARN);
        g_cmdProcStatus.startHallSelfLearnStatus = 0x00;
    }
    else
    {
        defHallSensor.hallSelfLearnEnable = DISABLE;
        g_cmdProcStatus.startHallSelfLearnStatus = 0x01;
    }

    UI_Return_StartHallSelfLearn_Result();
}

/*!
* @brief Master stop Hall self learning through UI interface.
*
* @param[in] none
* @return none
*/
void UI_StopHallSensorSelfLearn(void)
{
    g_cmdProcStatus.stopHallSelfLearnStatus = 0xFF;
    defHallSensor.hallSelfLearnEnable = DISABLE;
    g_cmdProcStatus.stopHallSelfLearnStatus = 0x00;
    UI_Return_StopHallSelfLearn_Result();
}

/*!
* @brief Master clear motor fault status through UI interface.
*
* @param[in] none
* @return none
*/
void UI_ClearFault(void)
{
    uint16_t i = 0;

    /* init clear fault status */
    g_cmdProcStatus.clearFaultStatus = 0xFF;
    Set_McStatus(STOP);
    for (i = 0; i <= 30; i++)
    {
        /* clear all fault bit */
        Fault_Clear(i);
    }
    /* refresh clear fault status,prepare to answer master */
    g_cmdProcStatus.clearFaultStatus = 0x00;
    /* init motor identify param again */
    g_paramIdentifyCtrl.initCtrl = 1;
    g_cmdProcVar.WriteSpeedValid = 0;
    UI_Return_ClearFault_Result();
}

/*!
* @brief Master read register data through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @param[in] dataLen: the char numbers of a master command
* @return none
*/
void UI_readRegister(UART_BUF *uiRxBuf, uint8_t dataLen)
{
    uint16_t i = 0;

    /* init read register status */
    g_cmdProcStatus.readRegStatus = 0xFF;
    /* save read register param num */
    g_cmdProcVar.uiRdBuf[0] = UI_UART_GetChar(uiRxBuf);

    if (g_cmdProcVar.uiRdBuf[0] < 11)
    {
        for (i = 1; i <= g_cmdProcVar.uiRdBuf[0]; i++)
        {
            /* save all read ragister param id */
            g_cmdProcVar.uiRdBuf[i] = UI_UART_GetChar(uiRxBuf);
        }
        /* refresh read ragister status,prepare to transmit register param to master */
        g_cmdProcStatus.readRegStatus = 0x00;
    }
    else
    {
        /* read register param num out of range,unvalid command */
        g_cmdProcStatus.readRegStatus = 0x02;
        /* init read register param num */
        g_cmdProcVar.uiRdBuf[0] = 0;
        /* Delete whole command except head and length */
        UI_UART_DeleteChar(uiRxBuf, dataLen - 2);
    }
    UI_Return_RegRead_Result();

    /* init read buff,prepare next read command */
    memset(g_cmdProcVar.uiRdBuf, 0U , sizeof(g_cmdProcVar.uiRdBuf));
}

/*!
* @brief Master write register data through UI interface.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @param[in] dataLen: the char numbers of a master command
* @return none
*/
void UI_WriteRegister(UART_BUF *uiRxBuf, uint8_t dataLen)
{
    uint16_t iBuf = 0, i = 0;
    uint16_t tempChar = 0;

    /* init write register status */
    g_cmdProcStatus.writeRegStatus = 0xFF;
    /* save write register param num */
    g_cmdProcVar.uiWrBuf[0] = UI_UART_GetChar(uiRxBuf);

    if (g_cmdProcVar.uiWrBuf[0] < 11)
    {
        for (i = 1; i <= g_cmdProcVar.uiWrBuf[0]; i++)
        {
            /* save write register param id */
            g_cmdProcVar.uiWrBuf[i] = UI_UART_GetChar(uiRxBuf);
            /* receive low byte */
            iBuf = UI_UART_GetChar(uiRxBuf);
            tempChar = UI_UART_GetChar(uiRxBuf);
            /* receive high byte,splicing regiater param */
            iBuf += (tempChar << 8);

            /* write param to the pointer id address */
            *g_uiRegBuf[g_cmdProcVar.uiWrBuf[i]] = iBuf;
            if (g_speedCommand.speedAcceleration != (pFocVarsCfg->motorAcceleration / 10))
            {
                g_speedCommand.speedAcceleration = (pFocVarsCfg->motorAcceleration / 10);
            }
            if (g_speedCommand.speedDeceleration != (pFocVarsCfg->motorDeceleration / 10))
            {
                g_speedCommand.speedDeceleration = (pFocVarsCfg->motorDeceleration / 10);
            }
        }

        /* refresh write ragister status, prepare to answer master */
        g_cmdProcStatus.writeRegStatus = 0x00;
    }
    else
    {
        /* write register param num out of range,unvalid command */
        g_cmdProcStatus.writeRegStatus = 0x02;
        /* init write register param num */
        g_cmdProcVar.uiWrBuf[0] = 0;
        /* Delete whole command except head and length */
        UI_UART_DeleteChar(uiRxBuf, dataLen - 2);
    }

    UI_Return_RegWrite_Result();

    /* refresh DAC print channel 0 */
    g_dispCh0 = g_uiCommParam.dacOut1;
    /* refresh DAC print channel 1 */
    g_dispCh1 = g_uiCommParam.dacOut2;

    /* init write buff,prepare next write command */
    memset(g_cmdProcVar.uiWrBuf, 0U , sizeof(g_cmdProcVar.uiWrBuf));
}

/*!
* @brief Uart process command recieved from master according to communication protocol.
*
* @param[in] uiRxBuf: pointer to UART_BUF structure, store the data recieced from master
* @return none
*/
void UI_UART_ToolCmd_Proc(UART_BUF *uiRxBuf)
{
    uint8_t headByte, dataLength, cmdData;
    uint8_t checkSum = 0, masterCmd = 0;
    uint16_t getDataTimeOut = 0, cmdNum = 0;

    if (((*uiRxBuf).uartBufCnt) >= 2)
    {
        headByte = UI_UART_ReadChar(uiRxBuf, 0);
        if (headByte == CMD_HEAD_BYTE)
        {
            dataLength = UI_UART_ReadChar(uiRxBuf, 1);
            g_cmdProcVar.uartTimeOutCntEn = 1;
            getDataTimeOut = g_cmdProcVar.uartTimeOutCnt;

            if ((((*uiRxBuf).uartBufCnt) >= dataLength) && (getDataTimeOut <= RECEIVE_TIM_MAXCNT))
            {
                g_cmdProcVar.uartTimeOutCntEn = 0;
                for (cmdNum = 0; cmdNum < dataLength - 1; cmdNum++)
                {
                    cmdData = UI_UART_ReadChar(uiRxBuf, cmdNum);
                    checkSum += cmdData;
                    if (cmdNum == 2)
                    {
                        g_cmdProcVar.uiCheckCmd = cmdData;
                    }
                }
                if (checkSum == UI_UART_ReadChar(uiRxBuf, cmdNum))
                {
                    /* Delete head and length */
                    UI_UART_DeleteChar(uiRxBuf, 2);
                    while (dataLength > 0)
                    {
                        masterCmd = UI_UART_GetChar(uiRxBuf);

                        switch (masterCmd)
                        {
                            case CMD_CHECK_COMMUNICATION:
                                UI_Return_Communicate_Result();
                            break;

                            case CMD_GET_CODE_VERSION:
                                UI_Return_GetVersion_Result();
                            break;

                            case CMD_START_MOTOR:
                                UI_StartMotor();
                            break;

                            case CMD_STOP_MOTOR:
                                UI_StopMotor();
                            break;

                            case CMD_STOP_RAMP:
                                UI_StopRamp();
                            break;

                            case CMD_START_PARAM_IDENTIFY:
                                UI_StartIdentify(uiRxBuf, &g_motorParamPu);
                            break;

                            case CMD_QUERY_PROCESS:
                                UI_QuerySchedule(uiRxBuf);
                            break;

                            case CMD_STOP_PARAM_IDENTIFY:
                                UI_StopIdentify();
                            break;

                            case CMD_MOTOR_FAULT_CLEAR:
                                UI_ClearFault();
                            break;

                            #if (defined DATA_ACCESS_FLASH_ENABLE)
                            case CMD_FLASH_READ:
                                UI_FlashRecoveryData(uiRxBuf, CMD_FLASH_READ);
                            break;
                            #endif

                            #if (defined DATA_ACCESS_FLASH_ENABLE)
                            case CMD_FLASH_SAVE:
                                UI_FlashSaveData(CMD_FLASH_SAVE);
                            break;
                            #endif

                            case CMD_START_HALLSELFLEARN:
                                UI_StartHallSensorSelfLearn(uiRxBuf);
                            break;

                            case CMD_STOP_HALLSELFLEARN:
                                UI_StopHallSensorSelfLearn();
                            break;

                            case CMD_READ_REGISTER:
                                UI_readRegister(uiRxBuf, dataLength);
                            break;

                            case CMD_WRITE_REGISTER:
                                UI_WriteRegister(uiRxBuf, dataLength);
                            break;

                            default:
                            break;
                        }

                        /* Delete Check tail */
                        UI_UART_DeleteChar(uiRxBuf, 1);
                        return;
                    }
                }
                else
                {
                    /*  check err */
                    UI_ReturnCheckErr_Result();
                    /* Delete whole command */
                    UI_UART_DeleteChar(uiRxBuf, dataLength);
                }
            }
            else if ((((*uiRxBuf).uartBufCnt) < dataLength) && (getDataTimeOut > RECEIVE_TIM_MAXCNT))
            {
                g_cmdProcVar.uartTimeOutCntEn = 0;
                /* Delete Head and Length */
                UI_UART_DeleteChar(uiRxBuf, 2);
            }
        }
        /* Delete Head */
        UI_UART_DeleteChar(uiRxBuf, 1);
    }
}

/* =============================================  EOF  ============================================== */
