/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file task_scheduler.c
*
* @brief This file provides task scheduling based on millisecond time base functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "task_scheduler.h"
#include "foc_app.h"
#include "foc_keys.h"
#include "globalvar.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
TASK_SCHEDULER g_SchTask[SCH_TASKS_SEQUENCE_MAX];   /*!< Task scheduling structure variable */
uint8_t g_ErrorCode = 0;                            /*!< Task scheduling fault code */
static uint32_t g_ErrorTickCnt;                     /*!< Task scheduling fault detection count */
static uint8_t g_ErrorCodeBuf;                      /*!< Task scheduling fault code previous value */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Delete the task in scheduling sequence.
*
* @param[in] task_index: task sequence index
* @return none
*/
void SCH_Delete_Task(const uint8_t task_index)
{
    if (g_SchTask[task_index].pTask == 0)
    {
        /*
        * No task at this location...
        * Set the global error variable
        */
        g_ErrorCode = ERROR_SCH_CANNOT_DELETE_TASK;
        /* Also return an error code */
    }
    else
    {
    }

    g_SchTask[task_index].pTask   = 0x0000;
    g_SchTask[task_index].delay   = 0;
    g_SchTask[task_index].period  = 0;
    g_SchTask[task_index].runOrder   = 0;
}

/*!
* @brief Report the task scheduler error.
*
* @param[in] none
* @return none
*/
void SCH_Report_Status(void)
{
    #ifdef SCH_REPORT_ERRORS
    /*
    * ONLY APPLIES IF WE ARE REPORTING ERRORS
    * Check for a new error code
    */
    if (g_ErrorCode != g_ErrorCodeBuf)
    {
        g_ErrorCodeBuf = g_ErrorCode;

        if (g_ErrorCode != 0)
        {
            g_ErrorTickCnt = 60000;
        }
        else
        {
            g_ErrorTickCnt = 0;
        }
    }
    else
    {
        if (g_ErrorTickCnt != 0)
        {
            if (--g_ErrorTickCnt == 0)
            {
                /* Reset error code */
                g_ErrorCode = 0;
            }
        }
    }
    #endif
}

/*!
* @brief Dispatch the tasks.
*
* @param[in] none
* @return none
*/
void SCH_Dispatch_Tasks(void)
{
    uint8_t index;
    /* Dispatches (runs) the next task (if one is ready) */
    if (g_SchTask[0].runOrder > 0)
    {
        for (index = 0; index < SCH_TASKS_SEQUENCE_MAX; index++)
        {
            if (g_SchTask[index].runOrder > 0)
            {
                /* Run the task */
                (*g_SchTask[index].pTask)();
                /* Reset / reduce runOrder flag */
                g_SchTask[index].runOrder -= 1;
                /* Periodic tasks will be scheduled to run again */

                /* if this is a 'one shot' task, remove it from the array */
                if (g_SchTask[index].period == 0)
                {
                    SCH_Delete_Task(index);
                }
            }
        }
    }

    /* Report system status */
    SCH_Report_Status();
}

/*!
* @brief Manage the task sequency.
*
* @param[in] void (* pFunction)(), pointer to task function
* @param[in] const uint32_t delayTime, delay time of task base on ms time
* @param[in] const uint32_t taskPeriod, time period of task
* @return none
*/
void SCH_Add_Task(void (* pFunction)(), const uint32_t delayTime, const uint32_t taskPeriod)
{
    uint8_t index = 0;

    /* First find a gap in the array (if there is one) */
    while ((index < SCH_TASKS_SEQUENCE_MAX) && (g_SchTask[index].pTask != 0))
    {
        index++;
    }

    /* Have we reached the end of the list? */
    if (index == SCH_TASKS_SEQUENCE_MAX)
    {
        /*
        * Task list is full
        * Set the global error variable
        */
        g_ErrorCode = ERROR_SCH_TOO_MANY_TASKS;
        return;
    }

    /* If we're here, there is a space in the task array */
    g_SchTask[index].pTask  = pFunction;
    g_SchTask[index].delay  = delayTime;
    g_SchTask[index].period = taskPeriod;
    g_SchTask[index].runOrder  = 0;
}

/*!
* @brief Task scheduler update base on TIMER ISR.
*
* @param[in] none
* @return none
*/
void SCH_Update(void)
{
    uint8_t index;
    /* NOTE: calculations are in *TICKS* (not milliseconds) */
    for (index = 0; index < SCH_TASKS_SEQUENCE_MAX; index++)
    {
        /* Check if there is a task at this location */
        if (g_SchTask[index].pTask)
        {
            if (g_SchTask[index].delay == 0)
            {
                /* The task is due to run */
                g_SchTask[index].runOrder += 1;
                if (g_SchTask[index].period)
                {
                    /* Schedule this regular task to run again */
                    g_SchTask[index].delay = g_SchTask[index].period - 1;
                }
            }
            else
            {
                /* Not yet ready to run: just decrement the delay */
                g_SchTask[index].delay -= 1;
            }
        }
    }
}

/*!
* @brief 1 ms time base tasks, all the 1 ms tasks executing here.
*
* @param[in] none
* @return none
*/
void Task0_1ms(void)
{
    Motor_1ms_Task();
}

/*!
* @brief 2 ms time base tasks, all the 2 ms tasks executing here.
*
* @param[in] none
* @return none
*/
void Task1_2ms(void)
{
    Motor_2ms_Task();
}

/*!
* @brief 4 ms time base tasks, all the 4 ms tasks executing here.
*
* @param[in] none
* @return none
*/
void Task2_4ms(void)
{
    Motor_4ms_Task();
}

/*!
* @brief 6 ms time base tasks, all the 6 ms tasks executing here.
*
* @param[in] none
* @return none
*/
void Task3_6ms(void)
{
    Motor_6ms_Task();
}

/*!
* @brief 10 ms time base tasks, all the 10 ms tasks executing here.
*
* @param[in] none
* @return none
*/
void Task4_10ms(void)
{
    Motor_10ms_Task();
}

/*!
* @brief 100 ms time base tasks, all the 100 ms tasks executing here.
*
* @param[in] none
* @return none
*/
void Task5_100ms(void)
{

}

/*!
* @brief 1000 ms time base tasks, all the 1000 ms tasks executing here.
*
* @param[in] none
* @return none
*/
void Task6_1000ms(void)
{
    static uint8_t dspCnt = 0;

    if (Get_McStatus() == FAULT)
    {
        dspCnt++;
        if (dspCnt == 2)
        {
            dspCnt = 0;
            LED1Display(LED_TURN_ON);
            LED2Display(LED_TURN_ON);
        }
        else
        {
            LED1Display(LED_TURN_OFF);
            LED2Display(LED_TURN_OFF);
        }
    }
    else
    {
        LED1Display(LED_TURN_ON);
        LED2Display(LED_TURN_ON);
    }
}

/*!
* @brief 2000 ms time base tasks, all the 2000 ms tasks executing here.
*
* @param[in] none
* @return none
*/
void Task7_2000ms(void)
{

}

/*!
* @brief Task scheduler main processer, called by Main function.
*
* @param[in] none
* @return none
*/
void Task_Scheduler(void)
{
    /* Notice: the delay of each task sequency cannot be the same! */
    /*
    * We can add a new time base task as the follow type:
    * SCH_Add_Task(New_task_function, task_sequence_number, task_time_base);
    * where New_task_function is the name of the task name;
    * task_sequence_number is the position of the task in all task sequences;
    * task_time_base is the time base of the task, indicating its execution cycle.
    */
    /* 1ms time base tasks */
    SCH_Add_Task(Task0_1ms, 0, 1);
    /* 2ms time base tasks */
    SCH_Add_Task(Task1_2ms, 1, 2);
    /* 4ms time base tasks */
    SCH_Add_Task(Task2_4ms, 2, 4);
    /* 6ms time base tasks, reserve */
    SCH_Add_Task(Task3_6ms, 3, 6);
    /* 10ms time base tasks */
    SCH_Add_Task(Task4_10ms, 4, 10);
    /* 100ms time base tasks, reserve */
    SCH_Add_Task(Task5_100ms, 5, 100);
    /* 1000ms time base tasks, reserve */
    SCH_Add_Task(Task6_1000ms, 6, 1000);

    while (1)
    {
        SCH_Dispatch_Tasks();
        MC_Keys_Read();
    }
}

/* =============================================  EOF  ============================================== */
